/**********************************************************************************
*
* $Header: /cvs/sakai/chef-tool/src/java/org/sakaiproject/tool/announcement/AnnouncementAction.java,v 1.13 2004/06/22 03:04:56 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

//  package
package org.sakaiproject.tool.announcement;

// imports
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Stack;
import java.util.Vector;

import org.sakaiproject.cheftool.Context;
import org.sakaiproject.cheftool.JetspeedRunData;
import org.sakaiproject.cheftool.ParameterParser;
import org.sakaiproject.cheftool.PortletConfig;
import org.sakaiproject.cheftool.RunData;
import org.sakaiproject.cheftool.VelocityPortlet;
import org.sakaiproject.cheftool.VelocityPortletPaneledAction;
import org.sakaiproject.cheftool.VelocityPortletStateAction;
import org.sakaiproject.cheftool.menu.Menu;
import org.sakaiproject.cheftool.menu.MenuEntry;
import org.sakaiproject.cheftool.menu.MenuItem;
import org.sakaiproject.exception.IdInvalidException;
import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.exception.IdUsedException;
import org.sakaiproject.exception.InUseException;
import org.sakaiproject.exception.PermissionException;
import org.sakaiproject.service.framework.portal.cover.PortalService;
import org.sakaiproject.service.framework.session.cover.UsageSessionService;
import org.sakaiproject.service.framework.state.SessionState;
import org.sakaiproject.service.legacy.announcement.AnnouncementChannel;
import org.sakaiproject.service.legacy.announcement.AnnouncementChannelEdit;
import org.sakaiproject.service.legacy.announcement.AnnouncementMessage;
import org.sakaiproject.service.legacy.announcement.AnnouncementMessageEdit;
import org.sakaiproject.service.legacy.announcement.AnnouncementMessageHeader;
import org.sakaiproject.service.legacy.announcement.AnnouncementMessageHeaderEdit;
import org.sakaiproject.service.legacy.announcement.cover.AnnouncementService;
import org.sakaiproject.service.legacy.content.cover.ContentHostingService;
import org.sakaiproject.service.legacy.content.cover.ContentTypeImageService;
import org.sakaiproject.service.legacy.message.Message;
import org.sakaiproject.service.legacy.message.MessageHeader;
import org.sakaiproject.service.legacy.notification.cover.NotificationService;
import org.sakaiproject.service.legacy.realm.cover.RealmService;
import org.sakaiproject.service.legacy.resource.Reference;
import org.sakaiproject.service.legacy.resource.ReferenceVector;
import org.sakaiproject.service.legacy.resource.ResourceProperties;
import org.sakaiproject.service.legacy.security.cover.SecurityService;
import org.sakaiproject.service.legacy.site.Site;
import org.sakaiproject.service.legacy.site.SiteEdit;
import org.sakaiproject.service.legacy.site.ToolConfigurationEdit;
import org.sakaiproject.service.legacy.site.cover.SiteService;
import org.sakaiproject.service.legacy.time.cover.TimeService;
import org.sakaiproject.tool.helper.AttachmentAction;
import org.sakaiproject.tool.helper.PermissionsAction;
import org.sakaiproject.util.Filter;
import org.sakaiproject.util.FormattedText;
import org.sakaiproject.util.MergedList;
import org.sakaiproject.util.SortedIterator;
import org.sakaiproject.util.StringUtil;
import org.sakaiproject.util.observer.EventsObservingCourier;
import org.sakaiproject.util.observer.ObservingCourier;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/*
 * AnnouncementAction is an implementation of Announcement service, which provides
 * the complete function of announcements. User could check the announcements, create
 * own new and manage all the announcement items, under certain permission check.
 *
 * @author University of Michigan, CHEF Software Development Team
 * @version $Revision: 1.13 $
 */

public class AnnouncementAction extends VelocityPortletStateAction
{
	private static final String CONTEXT_ENABLED_MENU_ITEM_EXISTS = "EnabledMenuItemExists";
	private static final String CONTEXT_ENABLE_ITEM_CHECKBOXES = "EnableItemCheckBoxes";
	private static final String ENABLED_MENU_ITEM_EXISTS = CONTEXT_ENABLED_MENU_ITEM_EXISTS;
	private static final String NOT_SELECTED_FOR_REVISE_STATUS = "noSelectedForRevise";
	private static final String FINISH_DELETING_STATUS = "FinishDeleting";
	private static final String DELETE_ANNOUNCEMENT_STATUS = "deleteAnnouncement";
	private static final String POST_STATUS = "post";
	private static final String CANCEL_STATUS = "cancel";
	private static final String MERGE_STATUS = "merge";
	private static final String OPTIONS_STATUS = "options";
	
	private static final String SSTATE_NOTI_VALUE = "noti_value";
	private static final String SSTATE_PUBLICVIEW_VALUE = "public_view_value";
	
	private static final String NOTI_REQUIRED = "High - All participants";
	private static final String NOTI_OPTIONAL = "Low - Only participants who have opted in";
	private static final String NOTI_NONE = "None - No notification";
	
	private static final String CONTEXT_VAR_DISPLAY_OPTIONS = "displayOptions";
	private static final String VELOCITY_DISPLAY_OPTIONS = CONTEXT_VAR_DISPLAY_OPTIONS;
	private static final String PERMISSIONS_BUTTON_HANDLER = "doPermissions";
	private static final String PERMISSIONS_BUTTON_CAPTION = "Permissions...";
	private static final String MERGE_BUTTON_HANDLER = "doMerge";
	private static final String MERGE_BUTTON_CAPTION = "Merge...";
	private static final String SSTATE_ATTRIBUTE_MERGED_CHANNELS = "mergedChannels";
	private static final String VELOCITY_MERGED_CHANNEL_LIST = "mergedAnnouncementsCollection";
	/** state attribute names. */
	private static final String STATE_CHANNEL_REF = "channelId";
	private static final String PORTLET_CONFIG_PARM_NON_MERGED_CHANNELS = "nonMergedAnnouncementChannels";
	
	public static final String STATE_MESSAGE = "message";
	
	/**
	 * Used to provide a  interface to the MergedList class
	 * that is shared with the calendar action.
	 */
	class EntryProvider implements MergedList.EntryProvider
	{
		/* (non-Javadoc)
		 * @see org.chefproject.actions.MergedEntryList.EntryProvider#getIterator()
		 */
		public Iterator getIterator()
		{
			return AnnouncementService.getChannels().iterator(); 
		}

		/* (non-Javadoc)
		 * @see org.chefproject.actions.MergedEntryList.EntryProvider#allowGet(java.lang.Object)
		 */
		public boolean allowGet(String ref)
		{
			return AnnouncementService.allowGetChannel(ref);
		}

		/* (non-Javadoc)
		 * @see org.chefproject.actions.MergedEntryList.EntryProvider#getContext(java.lang.Object)
		 */
		public String getContext(Object obj)
		{
			AnnouncementChannel channel = (AnnouncementChannel)obj;
			return channel.getContext();
		}

		/* (non-Javadoc)
		 * @see org.chefproject.actions.MergedEntryList.EntryProvider#getReference(java.lang.Object)
		 */
		public String getReference(Object obj)
		{
			AnnouncementChannel channel = (AnnouncementChannel)obj;
			return channel.getReference();
		}

		/* (non-Javadoc)
		 * @see org.chefproject.actions.MergedEntryList.EntryProvider#getProperties(java.lang.Object)
		 */
		public ResourceProperties getProperties(Object obj)
		{
			AnnouncementChannel channel = (AnnouncementChannel)obj;
			return channel.getProperties();
		}
	}

// TODO: is this really necessary?	
//	/**
//	 * This class overrides the alert message of Merge,
//	 * when the site is already being edited
//	 */
//	public class VelocityPortletPaneledAlert
//		extends VelocityPortletPaneledAction
//	{
//		/** 
//		 * Handle a request to set options.
//		 */
//		public void doOptions(RunData runData, Context context)
//		{
//			AnnouncementActionState actionState = (AnnouncementActionState)getState(context, runData, AnnouncementActionState.class);
//			String toolId = PortalService.getCurrentToolId();
//			String siteId = PortalService.getCurrentSiteId();
//			SessionState state =
//				((JetspeedRunData) runData).getPortletSessionState(toolId);
//
//			String msg = null;
//
//			try
//			{
//				// get a lock on the site
//				SiteEdit site = SiteService.editSite(siteId);
//
//				// get this tool's configuration
//				ToolConfigurationEdit tool = site.getToolEdit(toolId);
//
//				// put in state
//				state.setAttribute(STATE_SITE_EDIT, site);
//				state.setAttribute(STATE_TOOL_EDIT, tool);
//
//				// go into options mode
//				state.setAttribute(STATE_MODE, MODE_OPTIONS);
//
//				// disable auto-updates while editing
//				disableObservers(state);
//
//				// if we're not in the main panel for this tool, schedule an update of the main panel
//				String currentPanelId =
//					runData.getParameters().getString(REQ_PANEL);
//				if (!LAYOUT_MAIN.equals(currentPanelId))
//				{
//					String address = clientWindowId(state, toolId);
//					String mainPanelId = mainPanelUpdateId(toolId);
//					CourierService.deliver(address, mainPanelId);
//				}
//				
//				// Record the SiteEdit object for this site.
//				actionState.setEditSite(site);
//				
//				actionState.setStatus(OPTIONS_STATUS);
//				return;
//			}
//			catch (IdUnusedException ignore)
//			{
//				msg = "due to a system error.";
//			}
//			catch (PermissionException e)
//			{
//				msg =
//					"you do not have permission to set this option for this Worksite.";
//			}
//			catch (InUseException e)
//			{
//				msg =
//					"you are already configuring this Worksite from another tool, or someone else is configuring this Worksite.";
//			}
//
//			// we didn't get a lock on the site, so tell the user and don't change mode
//			state.setAttribute(
//				AnnouncementAction.STATE_MESSAGE,
//				"Merge is unavailable right now: " + msg);
//
//		} // doOptions
//	} 	
	
	/**
	 * Decorator for the "Message" class.  It adds various properties to
	 * the decorated real Announcement message.
	 */
	static public class AnnouncementWrapper implements AnnouncementMessage
	{
		private boolean enforceMaxNumberOfChars;
		private AnnouncementMessage announcementMesssage;
		private boolean editable;
		private String channelDisplayName;
		private int maxNumberOfChars;

		/**
		 * Constructor
		 * @param message The message to be wrapped.
		 * @param currentChannel The channel in which the message is contained.
		 * @param hostingChannel The channel into which the message is being merged.
		 * @param maxNumberOfChars The maximum number of characters that will be returned by getTrimmedBody().
		 */
		public AnnouncementWrapper(
			AnnouncementMessage message,
			AnnouncementChannel currentChannel,
			AnnouncementChannel hostingChannel,
			AnnouncementActionState.DisplayOptions options)
		{
			this.maxNumberOfChars = options.getNumberOfCharsPerAnnouncement();
			this.enforceMaxNumberOfChars = options.isEnforceNumberOfCharsPerAnnouncement();
			this.announcementMesssage = message;

			// This message is editable only if the site matches.
			this.editable =
				currentChannel.getReference().equals(
					hostingChannel.getReference());

			Site site = null;

			try
			{
				site = SiteService.getSite(currentChannel.getContext());
			}
			catch (IdUnusedException e)
			{
				// No site available.
			}

			if (site != null)
			{
				this.channelDisplayName = site.getTitle();
			}
			else
			{
				this.channelDisplayName = "";
			}
		}
		
		/**
		 * See if the given message was posted in the last N days, where N is the value
		 * of the maxDaysInPast parameter.
		 * @param message
		 * @param maxDaysInPast
		 * @return
		 */
		private static boolean isMessageWithinLastNDays(AnnouncementMessage message, int maxDaysInPast) 
		{
			final long MILLISECONDS_IN_DAY=(24*60*60*1000);
				
			long currentTime = TimeService.newTime().getTime();
				
			long timeDeltaMSeconds = currentTime - message.getHeader().getDate().getTime();
				
			long numDays = timeDeltaMSeconds/MILLISECONDS_IN_DAY;
			
			return (numDays <=  maxDaysInPast);
		}

		/* (non-Javadoc)
		 * @see org.chefproject.core.Message#getHeader()
		 */
		public MessageHeader getHeader()
		{
			return announcementMesssage.getHeader();
		}

		/* (non-Javadoc)
		 * @see org.chefproject.core.Message#getBody()
		 */
		public String getBody()
		{
			return announcementMesssage.getBody();
		}

		/* (non-Javadoc)
		 * @see org.chefproject.core.Message#getBody()
		 */
		public String getTrimmedBody()
		{
			if ( this.enforceMaxNumberOfChars )
			{
				// trim the body, as formatted text
				String body = announcementMesssage.getBody();
				StringBuffer buf = new StringBuffer();
				boolean didTrim = FormattedText.trimFormattedText(body, this.maxNumberOfChars, buf);
				if (didTrim) buf.append("...");
				
				return buf.toString();
			}
			else
			{
				return announcementMesssage.getBody();
			}
		}

		/* (non-Javadoc)
		 * @see org.chefproject.core.Resource#getUrl()
		 */
		public String getUrl()
		{
			return announcementMesssage.getUrl();
		}

		/* (non-Javadoc)
		 * @see org.chefproject.core.Resource#getReference()
		 */
		public String getReference()
		{
			return announcementMesssage.getReference();
		}

		/* (non-Javadoc)
		 * @see org.chefproject.core.Resource#getId()
		 */
		public String getId()
		{
			return announcementMesssage.getId();
		}

		/* (non-Javadoc)
		 * @see org.chefproject.core.Resource#getProperties()
		 */
		public ResourceProperties getProperties()
		{
			return announcementMesssage.getProperties();
		}

		/* (non-Javadoc)
		 * @see org.chefproject.core.Resource#toXml(org.w3c.dom.Document, java.util.Stack)
		 */
		public Element toXml(Document doc, Stack stack)
		{
			return announcementMesssage.toXml(doc, stack);
		}

		/* (non-Javadoc)
		 * @see java.lang.Comparable#compareTo(java.lang.Object)
		 */
		public int compareTo(Object arg0)
		{
			return announcementMesssage.compareTo(arg0);
		}
		
		/**
		 * Returns true if the message is editable.
		 * @return
		 */
		public boolean isEditable()
		{
			return editable;
		}

		/**
		 * Returns the string that is used to show the channel to the user.
		 * @return
		 */
		public String getChannelDisplayName()
		{
			return channelDisplayName;
		}

		/* (non-Javadoc)
		 * @see org.chefproject.core.AnnouncementMessage#getAnnouncementHeader()
		 */
		public AnnouncementMessageHeader getAnnouncementHeader()
		{
			return announcementMesssage.getAnnouncementHeader();
		}

		/**
		 * Constructs a list of wrapped/decorated AnnouncementMessages
		 * when given a list of unwrapped/undecorated AnnouncementMessages.
		 * @param messages The list of messages.
		 * @param currentChannel The current channel being processed.
		 * @param hostingChannel The default channel of the page into which this list is being merged.
		 * @param maxNumberOfDaysInThePast Messages over this limit will not be included in the list.
		 * @param maxCharsPerAnnouncement The maximum number of characters that will be returned when getTrimmedBody() is called.
		 * @return
		 */
		static private List wrapList(
			List messages,
			AnnouncementChannel currentChannel,
			AnnouncementChannel hostingChannel,
			AnnouncementActionState.DisplayOptions options)
		{
			int maxNumberOfDaysInThePast = options.getNumberOfDaysInThePast();
			
			List messageList = new ArrayList();

			Iterator it = messages.iterator();

			while (it.hasNext())
			{
				AnnouncementMessage message = (AnnouncementMessage) it.next();
				
				// See if the message falls within the filter window.
				if ( options.isEnforceNumberOfDaysInThePastLimit() &&
					 !isMessageWithinLastNDays(message, maxNumberOfDaysInThePast) )
				{
					continue;
				}
					
				messageList.add(
					new AnnouncementWrapper(
						message,
						currentChannel,
						hostingChannel,
						options));
			}

			return messageList;
		}
	}

	/**
	 * Enable or disable the observer
	 * @param runData
	 * @param enable if true, the observer is enabled, if false, it is disabled
	 */
	protected void enableObserver(SessionState sstate, boolean enable)
	{
		// disable auto-updates while in view mode
		ObservingCourier observer =
		(ObservingCourier) sstate.getAttribute(
		VelocityPortletPaneledAction.STATE_OBSERVER);
		if (observer != null)
		{
			if (enable)
			{
				observer.enable();
			}
			else
			{
				observer.disable();
			}
		}
	}
	
	/**
	 * See if the current tab is the workspace tab.
	 * @return true if we are currently on the "My Workspace" tab.
	 */
	private boolean isOnWorkspaceTab()
	{
		// we'll really answer the question - is the current request's site a user site?
		return SiteService.isUserSite(PortalService.getCurrentSiteId());
	}

	/**
	 * Build the context for showing merged view
	 * @param portlet The velocity portlet object.
	 * @param context The context object.
	 * @param runData The RunData object.
	 * @param state The AnnouncementActionState object.
	 * @param sstate The session state.
	 */
	public String buildMergeContext(
		VelocityPortlet portlet,
		Context context,
		RunData runData,
		AnnouncementActionState state,
		SessionState sstate)
	{

		if (Log.getLogger("chef").isDebugEnabled())
			Log.debug("chef", "buildMergedCalendarContext has been reached");

		MergedList mergedAnnouncementList = new MergedList();

		MergedList.EntryProvider entryProvider = new EntryProvider();

		mergedAnnouncementList.loadFromPortletConfig(
			portlet,
			state.getChannelId(),
			isOnWorkspaceTab(),
			PORTLET_CONFIG_PARM_NON_MERGED_CHANNELS,
			entryProvider);

		// Place this object in the context so that the velocity template
		// can get at it.
		context.put(VELOCITY_MERGED_CHANNEL_LIST, mergedAnnouncementList);

		sstate.setAttribute(
			SSTATE_ATTRIBUTE_MERGED_CHANNELS,
			mergedAnnouncementList);

		String template = (String) getContext(runData).get("template");
		return template + "-merge";
	}

	/**
	 * Default is to use when Portal starts up
	 */
	public String buildMainPanelContext( VelocityPortlet portlet,
		Context context,
		RunData rundata,
		SessionState sstate)
	{
		// retrieve the state from state object
		AnnouncementActionState state = (AnnouncementActionState)getState( portlet, rundata, AnnouncementActionState.class );
				
		// Put our display options in the context so
		// that we can modify our list display accordingly
		context.put(VELOCITY_DISPLAY_OPTIONS, state.getDisplayOptions());

		// if we are in edit permissions...
		String helperMode = (String) sstate.getAttribute(PermissionsAction.STATE_MODE);
		if (helperMode != null)
		{
			String template = PermissionsAction.buildHelperContext(portlet, context, rundata, sstate);
			if (template == null)
			{
				addAlert(sstate, "There is a problem setting Permissions at this time.");
			}
			else
			{
				return template;
			}
		}

		String template = (String) getContext(rundata).get("template");

		// if we are in edit attachments...
		String mode = (String) sstate.getAttribute(AttachmentAction.STATE_MODE);
		if (mode != null)
		{
			// if the mode is not done, defer to the AttachmentAction
			if (!mode.equals(AttachmentAction.MODE_DONE))
			{
				template = AttachmentAction.buildHelperContext(portlet, context, rundata, sstate);
				return template;
			}
		}
		// when done, take the new set of attachments, (if null, there was no change)
		ReferenceVector attachments = (ReferenceVector) sstate.getAttribute(AttachmentAction.STATE_ATTACHMENTS);
		if (attachments != null)
		{
			state.setAttachments(attachments);
		}

		// clean up
		sstate.removeAttribute(AttachmentAction.STATE_MODE);
		sstate.removeAttribute(AttachmentAction.STATE_ATTACHMENTS);

		// get the current channel ID from state object or prolet initial parameter
		String channelId = state.getChannelId();
		if (channelId == null)
		{
			// try the portlet parameter
			channelId = StringUtil.trimToNull(portlet.getPortletConfig().getInitParameter("channel"));
			if (channelId == null)
			{
				// form based on the request's site's "main" channel
				channelId = AnnouncementService.channelReference(
				PortalService.getCurrentSiteId(), SiteService.MAIN_CONTAINER);
			}

			// let the state object have the current channel id
			state.setChannelId(channelId);
			state.setIsListVM(true);
		}
		context.put("channel_id", ((channelId == null) ? "Now null" : channelId));

		// get the current collection ID from state object or prolet initial parameter
		String collectionId = state.getCollectionId();
		if (collectionId == null)
		{
			// get the current channel ID for prolet initial parameter
			collectionId = StringUtil.trimToNull(portlet.getPortletConfig().getInitParameter("collection"));
			if (collectionId == null)
				collectionId = ContentHostingService.getSiteCollection(PortalService.getCurrentSiteId());


			// let the state object have the current channel id
			state.setCollectionId(collectionId);
		}

		// set if we have notification enabled
		context.put("notification", Boolean.valueOf(notificationEnabled(state)));

		// find the channel and channel information through the service
		AnnouncementChannel channel = null;

		boolean menu_new = true;
		boolean menu_delete = true;
		boolean menu_revise = true;

		try
		{
			if (AnnouncementService.allowGetChannel(channelId)
				&& isOkayToDisplayMessageMenu(state))
			{
				// get the channel name throught announcement service API
				channel = AnnouncementService.getAnnouncementChannel(channelId);

				if (channel.allowGetMessages())
				{
					// provide "announcements_messages" as an iterator on the messages
					context.put("ann_messages", channel.getMessages(null, true));

					menu_new = channel.allowAddMessage();

					List messages = getMessages(channel, null, true, state, portlet);

					menu_delete = false;
					for (int i = 0; i < messages.size(); i++)
					{
						AnnouncementWrapper message = (AnnouncementWrapper)messages.get(i);
						
						// If any message is allowed to be removed
						// Also check to see if the AnnouncementWrapper object thinks
						// that this message is editable from the default site.
						if (channel.allowRemoveMessage(message) && message.editable)
						{
							menu_delete = true;
							break;
						}
					}

					menu_revise = false;
					for (int i = 0; i < messages.size(); i++)
					{
						// if any message is allowed to be edited
						if (channel.allowEditMessage(((Message) messages.get(i)).getId()))
						{
							menu_revise = true;
							break;
						}
					}
				}
				else // if the messages in this channel are not allow to be accessed
					{
					menu_new = channel.allowAddMessage();
					menu_revise = false;
					menu_delete = false;
				} // if-else
			}
			else // if the channel is not allowed to access
				{
				menu_new = false;
				menu_revise = false;
				menu_delete = false;
			}
		}
		catch (PermissionException error)
		{
			if (Log.getLogger("chef").isDebugEnabled())
				Log.debug("chef", this + "inside .buildNormalContext(): " + error);
		}
		catch (IdUnusedException error)
		{
			if (AnnouncementService.allowAddChannel(channelId))
			{
				try
				{
					AnnouncementChannelEdit edit = AnnouncementService.addAnnouncementChannel(channelId);
					AnnouncementService.commitChannel(edit);
					channel = edit;
				}
				catch (IdUsedException err)
				{
					if (Log.getLogger("chef").isDebugEnabled())
						Log.debug("chef", this + "inside .buildNormalContext(): " + err);
				}
				catch (IdInvalidException err)
				{}
				catch (PermissionException err)
				{}
				menu_new = channel.allowAddMessage();
				menu_revise = false;
				menu_delete = false;
			}
			else
			{
				menu_new = false;
				menu_revise = false;
				menu_delete = false;
			} // if-else
		} // try-catch

		// check the state status to decide which vm to render
		String statusName = state.getStatus();

		AnnouncementActionState.DisplayOptions displayOptions = state.getDisplayOptions();
		
		buildMenu(
			portlet,
			context,
			rundata,
			state,
			menu_new,
			menu_delete,
			menu_revise,
			this.isOkToShowMergeButton(statusName),
			this.isOkToShowPermissionsButton(statusName),
			this.isOkToShowOptionsButton(statusName),
			displayOptions);
			
		// added by zqian for toolbar
		context.put("allow_new", Boolean.valueOf(menu_new));
		context.put("allow_delete", Boolean.valueOf(menu_delete));
		context.put("allow_revise", Boolean.valueOf(menu_revise));

		if (statusName != null)
		{
			if (statusName.equals(DELETE_ANNOUNCEMENT_STATUS))
			{
				template = buildDeleteAnnouncementContext(portlet, context, rundata, state);
			}
			else if (statusName.equals("showMetadata"))
			{
				template = buildShowMetadataContext(portlet, context, rundata, state, sstate);
			}
			else if ((statusName.equals("goToReviseAnnouncement"))
				|| (statusName.equals("backToReviseAnnouncement"))
				|| (statusName.equals("new"))
				|| (statusName.equals("stayAtRevise")))
			{
				template = buildReviseAnnouncementContext(portlet, context, rundata, state, sstate);
			}
			else if (statusName.equals("revisePreviw"))
			{
				template = buildPreviewContext(portlet, context, rundata, state);
			}
			else
			if ((statusName.equals(CANCEL_STATUS))
				|| (statusName.equals(POST_STATUS))
				|| (statusName.equals("FinishDeleting")))
			{
				template = buildCancelContext(portlet, context, rundata, state);
			}
			else
			if (statusName.equals("noSelectedForDeletion")
				|| (statusName.equals(NOT_SELECTED_FOR_REVISE_STATUS)))
			{
				addAlert(sstate, "You have to select the announcement first!");
			}
			else if (statusName.equals("moreThanOneSelectedForRevise"))
			{
				addAlert(sstate, "Please choose only one announcement at a time to revise!");
			}
			else if (statusName.equals("noPermissionToRevise"))
			{
				addAlert(sstate, "You don't have permission to revise this announcement!");
			}
			else
			if (statusName
				.equals(MERGE_STATUS))
			{
			template =
					buildMergeContext(portlet, context, rundata, state, sstate);
		}
			else
			if (statusName.equals(OPTIONS_STATUS))
			{
				template =
					buildOptionsPanelContext(portlet, context, rundata, sstate);
			}
		}

		if (channel != null)
		{
			// ********* for sorting *********
			if (channel.allowGetMessages()
				&& isOkayToDisplayMessageMenu(state))
			{
				context.put("currentSortedBy", state.getCurrentSortedBy());
				if (state.getCurrentSortAsc())
					context.put("currentSortAsc", "true");
				else
					context.put("currentSortAsc", "false");

				Vector drafts = new Vector();
				Vector nonDrafts = new Vector();
				Vector showMessagesList = new Vector();

				try
				{
					List messages = getMessages(channel, null, true, state, portlet);
					for (int i = 0; i < messages.size(); i++)
					{
						AnnouncementMessage m = (AnnouncementMessage) messages.get(i);

						if (m.getAnnouncementHeader().getDraft())
						{
							drafts.addElement(m);
						}
						else
						{
							nonDrafts.add(m);
						}
					}

					SortedIterator sortedDraftIterator = new SortedIterator(drafts.iterator(), new AnnouncementComparator(state.getCurrentSortedBy(), state.getCurrentSortAsc()));
					SortedIterator sortedNonDraftIterator = new SortedIterator(nonDrafts.iterator(), new AnnouncementComparator(state.getCurrentSortedBy(), state.getCurrentSortAsc()));

					if (state.getCurrentSortAsc())
					{
						while (sortedDraftIterator.hasNext())
							showMessagesList.add((AnnouncementMessage)sortedDraftIterator.next());

						while (sortedNonDraftIterator.hasNext())
							showMessagesList.add((AnnouncementMessage)sortedNonDraftIterator.next());
					}
					else
					{
						while (sortedDraftIterator.hasNext())
							showMessagesList.add((AnnouncementMessage)sortedDraftIterator.next());

						while (sortedNonDraftIterator.hasNext())
							showMessagesList.add((AnnouncementMessage)sortedNonDraftIterator.next());
					}

					context.put("showMessagesList", showMessagesList.iterator());
					context.put("showMessagesList2", showMessagesList.iterator());
					context.put("messageListVector", showMessagesList);
				}
				catch (PermissionException error)
				{
					if (Log.getLogger("chef").isDebugEnabled())
						Log.debug("chef", this + "inside .buildNormalContext(): " + error);
				} // try-catch

			} // if allowGetMessages()
		}
		// ********* for sorting *********

		
		// ********* for site column display ********
		Site site = null;
		try
		{
			site = SiteService.getSite(channel.getContext());
		}
		catch (IdUnusedException e)
		{
		// No site available.
		}
		
		String currentChannelDisplayName = "";
		if (site != null)
		{
			currentChannelDisplayName = site.getTitle();
		}
		context.put("currentChannelDisplayName", currentChannelDisplayName);
		// ********* for site column display ********
		
		context.put("isOnWorkspaceTab", ((isOnWorkspaceTab())?"true":"false"));
		
		context.put("channel", channel);

		// inform the observing courier that we just updated the page...
		// if there are pending requests to do so they can be cleared
		((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER)).justDelivered();

		return template;

	}	//	buildMainPanelContext
	
	/** 
	 * Setup for the options panel.
	 */
	public String buildOptionsPanelContext( VelocityPortlet portlet, 
											Context context,
											RunData runData,
											SessionState state)
	{
		// retrieve the state from state object
		AnnouncementActionState actionState = (AnnouncementActionState)getState( portlet, runData, AnnouncementActionState.class );
		context.put(CONTEXT_VAR_DISPLAY_OPTIONS, actionState.getDisplayOptions());

		String channelId = actionState.getChannelId();
		Reference channelRef = new Reference(channelId);
				context.put("description", "Setting options for Announcements in worksite "
				+ SiteService.getSiteDisplay(channelRef.getContext()));

		// pick the "-customize" template based on the standard template name
		String template = (String)getContext(runData).get("template");
		return template + "-customize";
	
	}   // buildOptionsPanelContext

	/**
	 * Returns true if it is okay to display the revise/delete/etc. menu buttons.
	 * @param state
	 * @return
	 */
	private boolean isOkayToDisplayMessageMenu(AnnouncementActionState state)
	{
		String selectedMessageReference = state.getMessageReference();
		
		// If we're in the list state or if there is otherwise no message selected,
		// return true.
		if (state.getStatus() == null
			|| state.getStatus().equals(CANCEL_STATUS)
			|| selectedMessageReference == null
			|| selectedMessageReference.length() == 0)
		{
			return true;
		}
		if (state.getStatus().equals(MERGE_STATUS))
		{
			return false;
		}
		else
		{
			String channelID =
				getChannelIdFromReference(selectedMessageReference);
				
			boolean selectedMessageMatchesDefaultChannel =
				state.getChannelId().equals(channelID);

			return selectedMessageMatchesDefaultChannel;
		}
	}

	/**
	 * Returns true if it is ok to show the options button in the toolbar.
	 * @param statusName
	 * @return
	 */
	private boolean isOkToShowOptionsButton(String statusName)
	{
		if ( SiteService.allowUpdateSite(PortalService.getCurrentSiteId()) && !isOnWorkspaceTab())
		{
			return (
				statusName == null
					|| statusName.equals(CANCEL_STATUS)
					|| statusName.equals(POST_STATUS)
					|| statusName.equals(DELETE_ANNOUNCEMENT_STATUS)
					|| statusName.equals(FINISH_DELETING_STATUS)
					|| statusName.equals("noSelectedForDeletion")
					|| statusName.equals(NOT_SELECTED_FOR_REVISE_STATUS));
		}
		else
		{
			return false;
		}
	}

	/* what i've done to make this tool automaticlly updated
	 * includes some corresponding imports
	 * in buildMail, tell observer just the page is just refreshed
	 * in the do() functions related to show the list, enable the obeserver
	 * in other do() functions related to  not show the list, disable the obeserver
	 * in the do(), define the session sstate object, and protlet.
	 *
	 * add initState
	 * add updateObservationOfChannel()
	 * add state attribute STATE_CHANNEL_REF
	 */
	
	/**
	 * Returns true if it is okay to show the merge button in the menu.
	 * @param statusName
	 * @return
	 */
	private boolean isOkToShowMergeButton(String statusName)
	{
		if ( SiteService.allowUpdateSite(PortalService.getCurrentSiteId()) && !isOnWorkspaceTab())
		{
			return (
				statusName == null
					|| statusName.equals(CANCEL_STATUS)
					|| statusName.equals(POST_STATUS)
					|| statusName.equals(DELETE_ANNOUNCEMENT_STATUS)
					|| statusName.equals(FINISH_DELETING_STATUS)
					|| statusName.equals("noSelectedForDeletion")
					|| statusName.equals(NOT_SELECTED_FOR_REVISE_STATUS));
		}
		else
		{
			return false;
		}
	}

	/**
	 * Returns true if it is okay to show the permissions button in the menu.
	 * @param statusName
	 * @return
	 */
	private boolean isOkToShowPermissionsButton(String statusName)
	{
		if ( SiteService.allowUpdateSite(PortalService.getCurrentSiteId()) )
		{
			return (
				statusName == null
					|| statusName.equals(CANCEL_STATUS)
					|| statusName.equals(POST_STATUS)
					|| statusName.equals(DELETE_ANNOUNCEMENT_STATUS)
					|| statusName.equals(FINISH_DELETING_STATUS)
					|| statusName.equals("noSelectedForDeletion")
					|| statusName.equals(NOT_SELECTED_FOR_REVISE_STATUS));
		}
		else
		{
			return false;
		}
	}

	/**
	 * This should be the single point for getting lists of announcements in this action.
	 * It collects together all the announcements and wraps the real announcements in a
	 * decorator object that adds extra properties for use in the VM template.
	 * @param channel
	 * @param filter
	 * @param ascending
	 * @return
	 * @throws PermissionException
	 */
	private List getMessages(
		AnnouncementChannel defaultChannel,
		Filter filter,
		boolean ascending,
		AnnouncementActionState state,
		VelocityPortlet portlet)
		throws PermissionException
	{
		List messageList = new ArrayList();

		//
		// If we're on the "MyWorkSpace" tab and we're not an administrator,
		// then merge all the announcements that we can see.
		//
		if (isOnWorkspaceTab())
		{
			if ( SecurityService.isSuperUser() )
			{
				messageList.addAll(
					AnnouncementWrapper.wrapList(
						defaultChannel.getMessages(filter, ascending),
						defaultChannel,
						defaultChannel,
						state.getDisplayOptions()));
			}
			else
			{
				Iterator channelsIt =
					AnnouncementService.getChannels().iterator();

				while (channelsIt.hasNext())
				{
					AnnouncementChannel curChannel =
						(AnnouncementChannel) channelsIt.next();

					// skip channels from "special" sites
					if (SiteService.isSpecialSite(curChannel.getContext()))
						continue;

					// skip channels that the user cannot access
					if (!AnnouncementService.allowGetChannel(curChannel.getReference()))
						continue;

					messageList.addAll(
						AnnouncementWrapper.wrapList(
							curChannel.getMessages(filter, ascending),
							curChannel,
							defaultChannel,
							state.getDisplayOptions()));
				}
			}
		}
		else
		{
			MergedList mergedAnnouncementList = new MergedList();
		
			MergedList.EntryProvider entryProvider = new EntryProvider();
		
			mergedAnnouncementList.loadFromPortletConfig(
				portlet,
				state.getChannelId(),
				isOnWorkspaceTab(),
				PORTLET_CONFIG_PARM_NON_MERGED_CHANNELS,
				entryProvider);
				
			Iterator channelsIt = mergedAnnouncementList.iterator();

			while (channelsIt.hasNext())
			{
				MergedList.MergedEntry curEntry =
					(MergedList.MergedEntry) channelsIt.next();

				// If this entry should not be merged, skip to the next one.
				if ( !curEntry.isMerged() )
				{
					continue;
				}
				
				AnnouncementChannel curChannel = null;
				try
				{
					curChannel =
						(AnnouncementChannel) AnnouncementService.getChannel(
							curEntry.getReference());
				}
				catch (IdUnusedException e)
				{
					Log.debug("chef", this +"getMessages()" + e);
				}
				catch (PermissionException e)
				{
					Log.debug("chef", this +"getMessages()" + e);
				}

				if ( curChannel != null )
				{
					if (AnnouncementService.allowGetChannel(curChannel.getReference()))
					{
						messageList.addAll(
							AnnouncementWrapper.wrapList(
								curChannel.getMessages(filter, ascending),
								curChannel,
								defaultChannel,
								state.getDisplayOptions()));
					}
				}
			}

		}

		// Do an overall sort.  We couldn't do this earlier since each merged channel
		// would potentially cause a reshuffle.
		Collections.sort(messageList);

		// Reverse if we're not ascending.
		if (!ascending)
		{
			Collections.reverse(messageList);
		}

		// Apply any necessary list truncation.
		messageList = trimListToMaxNumberOfAnnouncements(messageList, state.getDisplayOptions());
		
		return messageList;
	}

	/**
	 * This will limit the maximum number of announcements that is shown.
	 * @param messageList
	 * @param numberOfAnnouncements
	 * @return
	 */
	private List trimListToMaxNumberOfAnnouncements(List messageList, AnnouncementActionState.DisplayOptions options)
	{
		if ( options.isEnforceNumberOfAnnouncementsLimit() )
		{
			int numberOfAnnouncements = options.getNumberOfAnnouncements();
			ArrayList destList = new ArrayList();
		
			// We need to go backwards through the list, limiting it to the number
			// of announcements that we're allowed to display.
			for (int i = messageList.size() - 1, curAnnouncementCount = 0;
				i >= 0 && curAnnouncementCount < numberOfAnnouncements;
				i--, curAnnouncementCount++)
			{
				destList.add(messageList.get(i));
			}
		
			return destList;
		}
		else
		{
			return messageList;
		}
	}

	/**
	 * Build the context for preview an attachment
	 * @param portlet The velocity portlet object
	 * @param context The context object
	 * @param runData The RunData object
	 * @param state The ResourceActionState object
	 */
	protected String buildPreviewContext(
		VelocityPortlet portlet,
		Context context,
		RunData rundata,
		AnnouncementActionState state)
	{
		context.put("conService", ContentHostingService.getInstance());

		// to get the content Type Image Service
		context.put("contentTypeImageService", ContentTypeImageService.getInstance());

		String subject = state.getTempSubject();
		String body = state.getTempBody();
		context.put("subject", subject);
		context.put("body", body);

		context.put("user", UsageSessionService.getSessionUser());

		context.put("date", TimeService.newTime());

		ReferenceVector attachments = state.getAttachments();
		context.put("attachments", attachments);

		String peid = ((JetspeedRunData) rundata).getJs_peid();
		SessionState sstate= ((JetspeedRunData)rundata).getPortletSessionState(peid);
		
		// output the public view options
		boolean pubview = Boolean.valueOf((String)sstate.getAttribute(SSTATE_PUBLICVIEW_VALUE)).booleanValue();
		if (pubview)
			context.put("IsPubView", "Yes");
		else
			context.put("IsPubView", "No");
		
		// output the notification options
		String notification = (String)sstate.getAttribute(SSTATE_NOTI_VALUE);;
		if ("r".equals(notification))
		{
			context.put("noti", AnnouncementAction.NOTI_REQUIRED);
		}
		else if ("n".equals(notification))
		{
			context.put("noti", AnnouncementAction.NOTI_NONE);
		}
		else
		{
			context.put("noti", AnnouncementAction.NOTI_OPTIONAL);
		}

		// pick the "browse" template based on the standard template name
		String template = (String) getContext(rundata).get("template");
		return template + "-preview";

	} // buildPreviewContext

	/**
	 * Build the context for revising the announcement
	 * @param portlet The velocity portlet object
	 * @param context The context object
	 * @param runData The RunData object
	 * @param state The ResourceActionState object
	 */
	protected String buildReviseAnnouncementContext(
		VelocityPortlet portlet,
		Context context,
		RunData rundata,
		AnnouncementActionState state,
		SessionState sstate)
	{
		context.put("service", ContentHostingService.getInstance());

		// to get the content Type Image Service
		context.put("contentTypeImageService", ContentTypeImageService.getInstance());

		ReferenceVector attachments = state.getAttachments();

		// if this a new annoucement, get the subject and body from temparory record
		if (state.getIsNewAnnouncement())
		{
			context.put("new", "true");
			context.put("tempSubject", state.getTempSubject());
			context.put("tempBody", state.getTempBody());

			// default pubview - %%% could find the inheritance even if we don't have an announcement yet... -ggolden
			context.put("pubviewset", Boolean.FALSE);
			//context.put("pubview", Boolean.FALSE);
			
			// output the sstate saved public view options
			boolean pubview = Boolean.valueOf((String)sstate.getAttribute(SSTATE_PUBLICVIEW_VALUE)).booleanValue();
			if (pubview)
				context.put("pubview", Boolean.TRUE);
			else
			context.put("pubview", Boolean.FALSE);
			
			// output the notification options
			String notification = (String)sstate.getAttribute(SSTATE_NOTI_VALUE);;
			// "r", "o" or "n"
			context.put("noti", notification);
		}
		// if this is an existing one
		else
			if (state.getStatus().equals("goToReviseAnnouncement"))
			{
				// get the message object through service
				context.put("new", "false");
				// get the message object through service
				//AnnouncementMessage message = channel.getAnnouncementMessage( messageId );
				//context.put("message", message);

				AnnouncementMessageEdit edit = state.getEdit();
				context.put("message", edit);

				// find out about pubview
				context.put("pubviewset", new Boolean(RealmService.getPubViewInheritance(edit.getReference())));
				context.put("pubview", new Boolean(RealmService.getPubView(edit.getReference())));
				
				// there is no chance to get the notification setting at this point
			}
			else // if state is "backToRevise"
			{
				context.put("new", "true");
				context.put("tempSubject", state.getTempSubject());
				context.put("tempBody", state.getTempBody());
				
				boolean pubview = Boolean.valueOf((String)sstate.getAttribute(SSTATE_PUBLICVIEW_VALUE)).booleanValue();
				if (pubview)
					context.put("pubview", Boolean.TRUE);
				else
					context.put("pubview", Boolean.FALSE);
				
				// output the notification options
				String notification = (String)sstate.getAttribute(SSTATE_NOTI_VALUE);;
				// "r", "o" or "n"
				context.put("noti", notification);
			}

		context.put("attachments", attachments);
		context.put("newAnn", (state.getIsNewAnnouncement()) ? "true" : "else");

		String template = (String) getContext(rundata).get("template");
		return template + "-revise";

	} // buildReviseAnnouncementContext

	/**
	 * Build the context for viewing announcement content
	 * @param portlet The velocity portlet object
	 * @param context The context object
	 * @param runData The RunData object
	 * @param state The ResourceActionState object
	 */
	protected String buildShowMetadataContext(
		VelocityPortlet portlet,
		Context context,
		RunData rundata,
		AnnouncementActionState state,
		SessionState sstate)
	{

		context.put("conService", ContentHostingService.getInstance());

		// to get the content Type Image Service
		context.put("contentTypeImageService", ContentTypeImageService.getInstance());

		// get the channel and message id information from state object
		String messageReference = state.getMessageReference();

		// get the message object through service
		try
		{
			// get the channel id throught announcement service
			AnnouncementChannel channel =
				AnnouncementService.getAnnouncementChannel(
					this.getChannelIdFromReference(messageReference));

			// get the message object through service
			AnnouncementMessage message =
				channel.getAnnouncementMessage(
					this.getMessageIDFromReference(messageReference));

			context.put("message", message);
			
			// find out about pubview
			context.put("pubviewset", new Boolean(RealmService.getPubViewInheritance(message.getReference())));
			context.put("pubview", new Boolean(RealmService.getPubView(message.getReference())));
				

			boolean menu_new = channel.allowAddMessage();
			boolean menu_delete = channel.allowRemoveMessage(message);
			boolean menu_revise = channel.allowEditMessage(message.getId());
			
			// Check to see if we can display an enabled menu.
			// We're currently not allowing modification of
			// announcements if they are not from this site.
			if ( !this.isOkayToDisplayMessageMenu(state) )
			{
				menu_new = menu_delete = menu_revise = false;
			}

			// check the state status to decide which vm to render
			String statusName = state.getStatus();

			AnnouncementActionState.DisplayOptions displayOptions = state.getDisplayOptions();
			
			buildMenu(
				portlet,
				context,
				rundata,
				state,
				menu_new,
				menu_delete,
				menu_revise,
				this.isOkToShowMergeButton(statusName),
				this.isOkToShowPermissionsButton(statusName),
				this.isOkToShowOptionsButton(statusName),
				displayOptions);

			context.put("allow_new", Boolean.valueOf(menu_new));
			context.put("allow_delete", Boolean.valueOf(menu_delete));
			context.put("allow_revise", Boolean.valueOf(menu_revise));
		}
		catch (IdUnusedException e)
		{
			if (Log.getLogger("chef").isDebugEnabled())
				Log.debug("chef", this +"buildShowMetadataContext()" + e);
		}
		catch (PermissionException e)
		{
			if (Log.getLogger("chef").isDebugEnabled())
				Log.debug("chef", this +"buildShowMetadataContext()" + e);
			addAlert(
				sstate,
				"You don't have permissions to access the message(s) - "
					+ e.toString());
		}

		String template = (String) getContext(rundata).get("template");
		return template + "-metadata";

	}

	/**
	 * Get the message id from a message reference.
	 * @param messageReference
	 * @return
	 */
	private String getMessageIDFromReference(String messageReference)
	{
		// "crack" the reference (a.k.a dereference, i.e. make a Reference)
		// and get the event id and channel reference
		Reference ref = new Reference(messageReference);
		return ref.getId();
	}

	/**
	 * Get the channel id from a message reference.
	 * @param messageReference
	 * @return
	 */
	private String getChannelIdFromReference(String messageReference)
	{
		// "crack" the reference (a.k.a dereference, i.e. make a Reference)
		// and get the event id and channel reference
		Reference ref = new Reference(messageReference);
		String channelId =
			AnnouncementService.channelReference(
				ref.getContext(),
				ref.getContainer());
		return channelId;
	} // getChannelIdFromReference
	
	/**
	 * Given a channel id, get a channel reference from it.
	 * @param channelId
	 * @return
	 */
	private String getChannelReferenceFromChannelID(String channelId)
	{
		// get the channel id throught announcement service
		AnnouncementChannel channel = null;
		try
		{
			channel = AnnouncementService.getAnnouncementChannel(channelId);
		}
		catch (IdUnusedException e)
		{
			Log.debug("chef", this +"getChannelReferenceFromChannelID()" + e);
		}
		catch (PermissionException e)
		{
			Log.debug("chef", this +"getChannelReferenceFromChannelID()" + e);
		}
			
		if ( channel != null )
		{
			return channel.getReference();
		}
		else
		{
			return null; 
		}
	}

	/**
	 * corresponding to chef_announcements doShowMetadata
	 * @param runData The RunData object
	 * @param context The context object
	 * @param itemId The string used to record the announcement id
	 **/
	public void doShowmetadata(RunData rundata, Context context)
	{
		// retrieve the state from state object
		AnnouncementActionState state = (AnnouncementActionState)getState( context, rundata, AnnouncementActionState.class );

		String peid = ((JetspeedRunData) rundata).getJs_peid();
		SessionState sstate= ((JetspeedRunData)rundata).getPortletSessionState(peid);

		String itemReference =
			rundata.getParameters().getString("itemReference");
		state.setMessageReference(itemReference);

		state.setIsListVM(false);
		state.setIsNewAnnouncement(false);
		state.setStatus("showMetadata");

		// disable auto-updates while in view mode
		((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER)).disable();

	} //	doShowMetadata

	/**
	 * Build the context for cancelling the operation and going back to list view
	 * @param portlet The velocity portlet object
	 * @param context The context object
	 * @param runData The RunData object
	 * @param state The ResourceActionState object
	 */
	protected String buildCancelContext(
		VelocityPortlet portlet,
		Context context,
		RunData rundata,
		AnnouncementActionState state)
	{
		//buildNormalContext(portlet, context, rundata);

		String template = (String) getContext(rundata).get("template");
		return template;

	} // buildCancelContext

	/**
	 * Build the context for asking for the delete confirmation
	 * @param portlet The velocity portlet object
	 * @param context The context object
	 * @param runData The RunData object
	 * @param state The ResourceActionState object
	 */
	protected String buildDeleteAnnouncementContext(
		VelocityPortlet portlet,
		Context context,
		RunData rundata,
		AnnouncementActionState state)
	{
		Vector v = state.getDelete_messages();
		if (v == null)
			v = new Vector();
		context.put("delete_messages", v.iterator());

		String template = (String) getContext(rundata).get("template");
		return template + "-delete";

	} // buildDeleteAnnouncementContext

	/**
	 * Action is to use when doNewannouncement requested,
	 * corresponding to chef_announcements menu "New..."
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doNewannouncement(RunData rundata, Context context)
	{
		// retrieve the state from state object
		AnnouncementActionState state =
			(AnnouncementActionState) getState(context,
				rundata,
				AnnouncementActionState.class);

		String peid = ((JetspeedRunData) rundata).getJs_peid();
		SessionState sstate =
			((JetspeedRunData) rundata).getPortletSessionState(peid);

		state.setIsListVM(false);
		state.setAttachments(null);
		state.setSelectedAttachments(null);
		state.setDeleteMessages(null);
		state.setIsNewAnnouncement(true);
		state.setTempBody("");
		state.setTempSubject("");
		state.setStatus("new");

		sstate.setAttribute(AnnouncementAction.SSTATE_PUBLICVIEW_VALUE, null);
		sstate.setAttribute(AnnouncementAction.SSTATE_NOTI_VALUE, null);

		// disable auto-updates while in view mode
		((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER)).disable();

	} // doNewannouncement

	/**
	 * Action is to use when doPost requested,
	 * corresponding to chef_announcements-revise or -preview "eventSubmit_doPost"
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doPost(RunData rundata, Context context)
	{
		// retrieve the state from state object
		AnnouncementActionState state =
			(AnnouncementActionState) getState(context,
				rundata,
				AnnouncementActionState.class);

		String peid = ((JetspeedRunData) rundata).getJs_peid();
		SessionState sstate =
			((JetspeedRunData) rundata).getPortletSessionState(peid);

		// get the channel and message id information from state object
		String channelId = state.getChannelId();

		// *** make sure the subject and body won't be empty
		String from = rundata.getParameters().getString("fromInterface");
		if (from != null)
			if (from.equals("revise"))
			{
				// read in the subject input from announcements-new.vm
				String subject = rundata.getParameters().getString("subject");
				// read in the body input
				String body = rundata.getParameters().getString("body");
				body = processFormattedTextFromBrowser(sstate, body);

				state.setTempSubject(subject);
				state.setTempBody(body);

				if (subject.length() == 0)
				{
					addAlert(sstate, "You need to fill in the subject!");
				}
				else
					if (body.length() == 0)
					{
						addAlert(
							sstate,
							"You need to fill in the body of the announcement!");
					}
			}

		// there is any error message caused by empty subject or body
		if (sstate.getAttribute(STATE_MESSAGE) != null)
		{
			// read the public view setting and save it in session state
			String publicView = rundata.getParameters().getString("pubview");
			sstate.setAttribute(AnnouncementAction.SSTATE_PUBLICVIEW_VALUE, publicView);
			
			// read the notification options & save it in session state
			String notification = rundata.getParameters().getString("notify");
			sstate.setAttribute(AnnouncementAction.SSTATE_NOTI_VALUE, notification);

			state.setIsListVM(false);
			state.setStatus("stayAtRevise");

			// disable auto-updates while in view mode
			((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER))
				.disable();
		}
		else
		{
			// read in the subject input from announcements-new.vm
			String subject = rundata.getParameters().getString("subject");
			// read in the body input
			String body = rundata.getParameters().getString("body");
			body = processFormattedTextFromBrowser(sstate, body);
			// if no input, which means coming from attachment editing
			if ((subject == null) || (body == null))
			{
				// get the string value from state object
				subject = state.getTempSubject();
				body = state.getTempBody();
			}

			// read the notification options
			String notification = rundata.getParameters().getString("notify");
			// if from the post button of preview page
			if (notification == null)
				notification = (String)sstate.getAttribute(AnnouncementAction.SSTATE_NOTI_VALUE);
			
			int noti = NotificationService.NOTI_OPTIONAL;
			if ("r".equals(notification))
			{
				noti = NotificationService.NOTI_REQUIRED;
			}
			else if ("n".equals(notification))
			{
				noti = NotificationService.NOTI_NONE;
			}

			try
			{
				// if a new created announcement to be posted
				if (state.getIsNewAnnouncement())
				{
					// get the channel id throught announcement service
					AnnouncementChannel channel =
						AnnouncementService.getAnnouncementChannel(channelId);

					AnnouncementMessageEdit msg =
						channel.addAnnouncementMessage();
					msg.setBody(body);
					AnnouncementMessageHeaderEdit header =
						msg.getAnnouncementHeaderEdit();
					header.setSubject(subject);
					header.setDraft(false);
					header.replaceAttachments(state.getAttachments());

					// read the pubview settings
					boolean pubviewset = RealmService.getPubViewInheritance(msg.getReference());
					boolean pubview = false;
					if (!pubviewset)
					{
						//pubview = Boolean.valueOf(rundata.getParameters().getString("pubview")).booleanValue();
						String pubviewStr = rundata.getParameters().getString("pubview");
						// if from the post in preview, get the setting from sstate object
						if (pubviewStr == null)
							pubviewStr = (String) sstate.getAttribute(AnnouncementAction.SSTATE_PUBLICVIEW_VALUE);
						pubview = Boolean.valueOf(pubviewStr).booleanValue();
					}

					channel.commitMessage(msg, noti);

					if (!pubviewset)
					{
						try
						{
							RealmService.setPubView(msg.getReference(), pubview);
						}
						catch (InUseException ee) {} // %%%
					}

				}
				// if a existing announcement to be posted after being revised or previewed
				else
				{
					if (subject.length() > 0)
					{
						// get the message object through service
						//AnnouncementMessageEdit msg = channel.editAnnouncementMessage( messageId );
						AnnouncementMessageEdit msg = state.getEdit();

						// get the channel id throught announcement service
						AnnouncementChannel channel =
							AnnouncementService.getAnnouncementChannel(
								this.getChannelIdFromReference(
									msg.getReference()));

						msg.setBody(body);
						AnnouncementMessageHeaderEdit header =
							msg.getAnnouncementHeaderEdit();
						header.setSubject(subject);
						header.setDraft(false);
						header.replaceAttachments(state.getAttachments());

						// read the pubview settings
						boolean pubviewset = RealmService.getPubViewInheritance(msg.getReference());
						boolean pubview = false;
						if (!pubviewset)
						{
							//pubview = Boolean.valueOf(rundata.getParameters().getString("pubview")).booleanValue();
							String pubviewStr = rundata.getParameters().getString("pubview");
							// if from the post in preview, get the setting from sstate object
							if (pubviewStr == null)
								pubviewStr = (String) sstate.getAttribute(AnnouncementAction.SSTATE_PUBLICVIEW_VALUE);
							pubview = Boolean.valueOf(pubviewStr).booleanValue();
						}

						channel.commitMessage(msg, noti);

						if (!pubviewset)
						{
							try
							{
								RealmService.setPubView(msg.getReference(), pubview);
							}
							catch (InUseException ee) {} // %%%
						}

						state.setEdit(null);
					}
				} // if-else
			}
			catch (IdUnusedException e)
			{
				if (Log.getLogger("chef").isDebugEnabled())
					Log.debug("chef", this +"doPost()" + e);
			}
			catch (PermissionException e)
			{
				if (Log.getLogger("chef").isDebugEnabled())
					Log.debug("chef", this +"doPost()" + e);
				addAlert(
					sstate,
					"You don't have permissions to create this announcement -"
						+ subject);
			}

			state.setIsListVM(true);
			state.setAttachments(null);
			state.setSelectedAttachments(null);
			state.setDeleteMessages(null);
			state.setStatus(POST_STATUS);
			state.setMessageReference("");

			// make sure auto-updates are enabled
			((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER))
				.enable();
		}
	} //	doPost

	/**
	 * Action is to use when doPreviewrevise requested from preview status
	 * corresponding to chef_announcements-preview "eventSubmit_doPreviewrevise"
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doPreviewrevise(RunData rundata, Context context)
	{

		// retrieve the state from state object
		AnnouncementActionState state =
			(AnnouncementActionState) getState(context,
				rundata,
				AnnouncementActionState.class);

		String peid = ((JetspeedRunData) rundata).getJs_peid();
		SessionState sstate =
			((JetspeedRunData) rundata).getPortletSessionState(peid);

		state.setStatus("backToReviseAnnouncement");

		// disable auto-updates while in view mode
		((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER)).disable();

	} // doPreviewrevise

	/**
	 * Action is to use when ddoDelete requested, to perform deletion
	 * corresponding to chef_announcements-delete "eventSubmit_doDelete"
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doDelete(RunData rundata, Context context)
	{

		// retrieve the state from state object
		AnnouncementActionState state =
			(AnnouncementActionState) getState(context,
				rundata,
				AnnouncementActionState.class);

		String peid = ((JetspeedRunData) rundata).getJs_peid();
		SessionState sstate =
			((JetspeedRunData) rundata).getPortletSessionState(peid);

		// get the messages to be deleted from state object
		Vector v = state.getDelete_messages();
		Iterator delete_messages = v.iterator();

		while (delete_messages.hasNext())
		{
			try
			{
				AnnouncementMessage message =
					(AnnouncementMessage) delete_messages.next();

				// get the channel id throught announcement service
				AnnouncementChannel channel =
					AnnouncementService.getAnnouncementChannel(
						this.getChannelIdFromReference(message.getReference()));

				if (channel.allowRemoveMessage(message))
				{
					// remove message from channel
					AnnouncementMessageEdit edit =
						channel.editAnnouncementMessage(message.getId());
					channel.removeMessage(edit);

					// make sure auto-updates are enabled
					((EventsObservingCourier) sstate
						.getAttribute(STATE_OBSERVER))
						.enable();
				}
				else
				{
					addAlert(
						sstate,
						"you don't have permission to delete the messages.");
				}
			}
			catch (IdUnusedException e)
			{
				if (Log.getLogger("chef").isDebugEnabled())
					Log.debug("chef", this +"doDeleteannouncement()" + e);
			}
			catch (PermissionException e)
			{
				if (Log.getLogger("chef").isDebugEnabled())
					Log.debug("chef", this +"doDeleteannouncement()" + e);
			}
			catch (NoSuchElementException e)
			{
				if (Log.getLogger("chef").isDebugEnabled())
					Log.debug("chef", this +"doDeleteannouncement()" + e);
			}
			catch (InUseException e)
			{
				if (Log.getLogger("chef").isDebugEnabled())
					Log.debug("chef", this +"doPost()" + e);
				//addAlert(sstate, "this announcement is being edited by someone else");

				((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER))
					.disable();
			}
		}

		state.setIsListVM(true);
		state.setStatus("FinishDeleting");

	} //	doDelete

	/**
	 * Action is to use when doDeleteannouncement requested,
	 * corresponding to chef_announcements or chef_announcements-metadata menu "Delete"
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doDeleteannouncement(RunData rundata, Context context)
	{
		// retrieve the state from state object
		AnnouncementActionState state =
			(AnnouncementActionState) getState(context,
				rundata,
				AnnouncementActionState.class);

		String peid = ((JetspeedRunData) rundata).getJs_peid();
		SessionState sstate =
			((JetspeedRunData) rundata).getPortletSessionState(peid);

		// get the channel and message id information from state object
		String messageReference = state.getMessageReference();

		// if in main screen
		if (state.getIsListVM())
		{
			// then, read in the selected announcment items
			String[] messageReferences =
				rundata.getParameters().getStrings("selectedMembers");
			if (messageReferences != null)
			{
				Vector v = new Vector();
				for (int i = 0; i < messageReferences.length; i++)
				{
					// get the message object through service
					try
					{
						// get the channel id throught announcement service
						AnnouncementChannel channel =
							AnnouncementService.getAnnouncementChannel(
								this.getChannelIdFromReference(
									messageReferences[i]));
						// get the message object through service
						AnnouncementMessage message =
							channel.getAnnouncementMessage(
								this.getMessageIDFromReference(
									messageReferences[i]));

						v.addElement(message);
					}
					catch (IdUnusedException e)
					{
						if (Log.getLogger("chef").isDebugEnabled())
							Log.debug(
								"chef",
								this +"doDeleteannouncement()" + e);
						// addAlert(sstate, e.toString());
					}
					catch (PermissionException e)
					{
						if (Log.getLogger("chef").isDebugEnabled())
							Log.debug(
								"chef",
								this +"doDeleteannouncement()" + e);
						addAlert(
							sstate,
							"you don't have permissions to delete this announcement -"
								+ messageReferences[i]);
					}
				}

				//record the items to be deleted
				state.setDeleteMessages(v);
				state.setIsListVM(false);
				state.setStatus(DELETE_ANNOUNCEMENT_STATUS);

				// disable auto-updates while in view mode
				((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER))
					.disable();
			}
			else
			{
				state.setIsListVM(true);
				state.setStatus("noSelectedForDeletion");

				// make sure auto-updates are enabled
				((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER))
					.enable();
			}

		}
		// if not in main screen
		else
		{
			state.setIsNewAnnouncement(false);
			Vector v = new Vector();

			// get the message object through service
			try
			{
				// get the channel id throught announcement service
				AnnouncementChannel channel =
					AnnouncementService.getAnnouncementChannel(
						this.getChannelIdFromReference(messageReference));
				// get the message object through service
				AnnouncementMessage message =
					channel.getAnnouncementMessage(
						this.getMessageIDFromReference(messageReference));

				v.addElement(message);
			}
			catch (IdUnusedException e)
			{
				if (Log.getLogger("chef").isDebugEnabled())
					Log.debug("chef", this +"doDeleteannouncement()" + e);
				// addAlert(sstate, e.toString());
			}
			catch (PermissionException e)
			{
				if (Log.getLogger("chef").isDebugEnabled())
					Log.debug("chef", this +"doDeleteannouncement()" + e);
				addAlert(
					sstate,
					"you don't have permissions to delete this announcement!");
			}

			//state.setDelete_messages(delete_messages);
			state.setDeleteMessages(v);

			state.setIsListVM(false);
			if (sstate.getAttribute(STATE_MESSAGE) == null)
			{
				// add folder sucessful
				state.setStatus(DELETE_ANNOUNCEMENT_STATUS);
			}
		}

		// disable auto-updates while in confirm mode
		((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER)).disable();

	} // doDeleteannouncement

	/**
	 * Action is to use when doReviseannouncement requested,
	 * corresponding to chef_announcements the link of any draft announcement item
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doReviseannouncement(RunData rundata, Context context)
	{
		// retrieve the state from state object
		AnnouncementActionState state =
			(AnnouncementActionState) getState(context,
				rundata,
				AnnouncementActionState.class);

		String peid = ((JetspeedRunData) rundata).getJs_peid();
		SessionState sstate =
			((JetspeedRunData) rundata).getPortletSessionState(peid);

		state.setStatus("goToReviseAnnouncement");

		String messageReference =
			rundata.getParameters().getString("itemReference");
		state.setMessageReference(messageReference);

		// get the message object through service
		try
		{
			// get the channel id throught announcement service
			AnnouncementChannel channel =
				AnnouncementService.getAnnouncementChannel(
					this.getChannelIdFromReference(messageReference));
			// get the message object through service
			//AnnouncementMessage message = channel.getAnnouncementMessage( messageId );

			AnnouncementMessageEdit edit =
				channel.editAnnouncementMessage(
					this.getMessageIDFromReference(messageReference));
			state.setEdit(edit);

			//ReferenceVector attachmentList = (message.getHeader()).getAttachments();
			ReferenceVector attachmentList =
				(edit.getHeader()).getAttachments();
			state.setAttachments(attachmentList);

			// disable auto-updates while in confirm mode
			((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER))
				.disable();
		}
		catch (IdUnusedException e)
		{
			if (Log.getLogger("chef").isDebugEnabled())
				Log.debug("chef", this +"announcementRevise" + e);
			// addAlert(sstate, e.toString());
		}
		catch (PermissionException e)
		{
			if (Log.getLogger("chef").isDebugEnabled())
				Log.debug("chef", this +"announcementRevise" + e);
			state.setStatus("showMetadata");
		}
		catch (InUseException err)
		{
			if (Log.getLogger("chef").isDebugEnabled())
				Log.debug(
					"chef",
					this +"inside .doReviseannouncementfrommenu" + err);
			addAlert(
				sstate,
				"This item is being edited by another user. Please try again later.");
			state.setStatus("showMetadata");
		}
		state.setIsNewAnnouncement(false);

	} //	doReviseannouncement

	/**
	 * Action is to use when doReviseannouncementfrommenu requested,
	 * corresponding to chef_announcements.vm and -metadata.vm menu "Revise"
	 * @param runData The RunData object
	 * @param context The context object
	 **/

	public void doReviseannouncementfrommenu(RunData rundata, Context context)
	{

		// retrieve the state from state object
		AnnouncementActionState state = (AnnouncementActionState)getState( context, rundata, AnnouncementActionState.class );

		String peid = ((JetspeedRunData) rundata).getJs_peid();
		SessionState sstate= ((JetspeedRunData)rundata).getPortletSessionState(peid);

		// get the channel and message id information from state object
		String messageReference = state.getMessageReference();

		// if in main screen
		if (state.getIsListVM())
		{
			// then, read in the selected announcment items
			String[] messageReferences =
				rundata.getParameters().getStrings("selectedMembers");
			if (messageReferences != null)
			{
				if (messageReferences.length > 1)
				{
					state.setIsListVM(true);
					state.setStatus("moreThanOneSelectedForRevise");

					// make sure auto-updates are enabled
					((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER)).enable();

				}
				else
					if (messageReferences.length == 1)
					{
						state.setIsListVM(false);
						state.setMessageReference(messageReferences[0]);
						state.setStatus("goToReviseAnnouncement");

						// get the message object through service
						try
						{
							// get the channel id throught announcement service
							AnnouncementChannel channel =
								AnnouncementService.getAnnouncementChannel(
									this.getChannelIdFromReference(
										messageReferences[0]));
							// get the message object through service
							AnnouncementMessage message =
								channel.getAnnouncementMessage(
									this.getMessageIDFromReference(
										messageReferences[0]));

							if (channel.allowEditMessage(message.getId()))
							{
							state.setAttachments(message.getHeader().getAttachments());
								// %%% -ggolden AnnouncementMessageEdit edit = channel.editAnnouncementMessage( messageIds[0] );
								AnnouncementMessageEdit edit =
									channel.editAnnouncementMessage(
										this.getMessageIDFromReference(
											messageReferences[0]));
								state.setEdit(edit);
							}
							else
							{
								state.setIsListVM(true);
								state.setStatus("noPermissionToRevise");
							}
						}
						catch (IdUnusedException e)
						{
							if (Log.getLogger("chef").isDebugEnabled())
							Log.debug("chef", this + "announcementReviseFromMenu" + e);
						}
						catch (PermissionException e)
						{
							if (Log.getLogger("chef").isDebugEnabled())
							Log.debug("chef", this + "announcementReviseFromMenu" + e);
						addAlert(sstate, "You don't have permissions to access the message(s) - "+ e.toString());
						}
						// %%% -ggolden catch(InUseException err) 
						catch (InUseException err)
						{
							if (Log.getLogger("chef").isDebugEnabled())
							Log.debug("chef", this + "inside .doReviseannouncementfrommenu" + err);
						addAlert(sstate, "This item is being edited by another user. Please try again later.");
							state.setIsListVM(false);
							state.setStatus("showMetadata");

							// make sure auto-updates are enabled
						((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER)).disable();
						}
					}
			}
			else
			{
				state.setIsListVM(true);
				state.setStatus(NOT_SELECTED_FOR_REVISE_STATUS);

				// make sure auto-updates are enabled
				((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER)).enable();
			}
		}
		// if the user is viewing a certain announcement already
		else
		{
			state.setIsListVM(false);
			state.setStatus("goToReviseAnnouncement");

			// get the message object through service
			try
			{
				// get the channel id throught announcement service
				AnnouncementChannel channel =
					AnnouncementService.getAnnouncementChannel(
						this.getChannelIdFromReference(messageReference));
				// get the message object through service
				AnnouncementMessage message =
					channel.getAnnouncementMessage(
						this.getMessageIDFromReference(messageReference));

				if (channel.allowEditMessage(message.getId()))
				{
					// %%% -ggolden 
					AnnouncementMessageEdit edit =
						channel.editAnnouncementMessage(
							this.getMessageIDFromReference(messageReference));
					state.setEdit(edit);
					state.setAttachments(message.getHeader().getAttachments());
				}
				else
				{
					state.setIsListVM(true);
					state.setStatus("noPermissionToRevise");
				}
			}
			catch (IdUnusedException e)
			{
				if (Log.getLogger("chef").isDebugEnabled())
					Log.debug("chef", this +"announcementReviseFromMenu" + e);
				// addAlert(sstate, e.toString());
			}
			catch (PermissionException e)
			{
				if (Log.getLogger("chef").isDebugEnabled())
					Log.debug("chef", this +"announcementReviseFromMenu" + e);
				addAlert(sstate, "You don't have permissions to access the message(s) - "+ e.toString());
			}
			catch (InUseException err)
			{
				if (Log.getLogger("chef").isDebugEnabled())
					Log.debug("chef", this + "inside .doReviseannouncementfrommenu" + err);
				addAlert(sstate, "This item is being edited by another user. Please try again later.");
				state.setIsListVM(false);
				state.setStatus("showMetadata");

				// disable auto-updates while in view mode
				((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER)).disable();
			}
		}

		state.setIsNewAnnouncement(false);
	} //	doReviseannouncementfrommenu

	/**
	 * Action is to use when doRevisePreview requested,
	 * corresponding to chef_announcements-revise "eventSubmit_doRevisePreview" from revise view to preview view
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doRevisepreview(RunData rundata, Context context)
	{
		// retrieve the state from state object
		AnnouncementActionState state =
			(AnnouncementActionState) getState(context,
				rundata,
				AnnouncementActionState.class);

		String peid = ((JetspeedRunData) rundata).getJs_peid();
		SessionState sstate =
			((JetspeedRunData) rundata).getPortletSessionState(peid);

		// *** make sure the subject and body won't be empty
		//String from = rundata.getParameters().getString("fromInterface");
		//if (from.equals("revise"))
		//{
			// read in the subject input from announcements-new.vm
			String subject = rundata.getParameters().getString("subject");
			// read in the body input
			String body = rundata.getParameters().getString("body");
			body = processFormattedTextFromBrowser(sstate, body);

			state.setTempSubject(subject);
			state.setTempBody(body);

			if (subject.length() == 0)
			{
				addAlert(sstate, "You need to fill in the subject!");
			}
			else
				if (body.length() == 0)
				{
				addAlert(sstate, "You need to fill in the body of the announcement!");
				}
		//} // if (from.equals("revise"))

		// read the public view setting and save it in session state
		String publicView = rundata.getParameters().getString("pubview");
		sstate.setAttribute(SSTATE_PUBLICVIEW_VALUE, publicView);
		
		// read the notification options & save it in session state
		String notification = rundata.getParameters().getString("notify");
		sstate.setAttribute(SSTATE_NOTI_VALUE, notification);

		// there is any error message caused by empty subject or boy
		if (sstate.getAttribute(STATE_MESSAGE) != null)
		{
			state.setIsListVM(false);
			state.setStatus("stayAtRevise");
		}
		else
		{
			// read in the subject input from announcements-new.vm
			subject = rundata.getParameters().getString("subject");
			state.setTempSubject(subject);

			// read in the body input
			body = rundata.getParameters().getString("body");
			body = processFormattedTextFromBrowser(sstate, body);
			state.setTempBody(body);

			state.setStatus("revisePreviw");
		} // if-else

		// disable auto-updates while in view mode
		((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER)).disable();

	} //	doRevisepreview

	public void doAttachments(RunData data, Context context)
	{
		SessionState state =
			((JetspeedRunData) data).getPortletSessionState(
				((JetspeedRunData) data).getJs_peid());

		// setup... we'll use the attachment action's mode
		state.setAttribute(
			AttachmentAction.STATE_MODE,
			AttachmentAction.MODE_MAIN);
		state.setAttribute(
			AttachmentAction.STATE_FROM_TEXT,
			"Announcement "
				+ "\""
				+ data.getParameters().getString("subject")
				+ "\"");

		// load the attachment editor's STATE_ATTACHMENTS with a copy of our attachment vector
		AnnouncementActionState myState =
			(AnnouncementActionState) getState(context,
				data,
				AnnouncementActionState.class);
		ReferenceVector attachments = myState.getAttachments();
		// whether there is alread an attachment //%%%zqian
		if (attachments.size() > 0)
		{
			state.setAttribute(AttachmentAction.STATE_HAS_ATTACHMENT_BEFORE, Boolean.TRUE);
		}
		else
		{
			state.setAttribute(AttachmentAction.STATE_HAS_ATTACHMENT_BEFORE, Boolean.FALSE);
		}
		state.setAttribute(
			AttachmentAction.STATE_ATTACHMENTS,
			attachments.clone());

		// read in the input of subject and body
		String subject = data.getParameters().getString("subject");
		String body = data.getParameters().getString("body");
		body = processFormattedTextFromBrowser(state, body);
		myState.setTempSubject(subject);
		myState.setTempBody(body);
		myState.setStatus("backToReviseAnnouncement");

		// read the public view setting and save it in session state
		String publicView = data.getParameters().getString("pubview");
		state.setAttribute(AnnouncementAction.SSTATE_PUBLICVIEW_VALUE, publicView);
		
		// read the notification options & save it in session state
		String notification = data.getParameters().getString("notify");
		state.setAttribute(AnnouncementAction.SSTATE_NOTI_VALUE, notification);

	} // doAttachments

	/**
	 * Action is to use when doCancel requested,
	 * corresponding to chef_announcement "eventSubmit_doCancel"
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doCancel(RunData rundata, Context context)
	{

		// retrieve the state from state object
		AnnouncementActionState state =
			(AnnouncementActionState) getState(context,
				rundata,
				AnnouncementActionState.class);
				
		// Carry out the base class options cancel if we're in the
		// middle of an options page.
		if ( state.getStatus().equals(OPTIONS_STATUS) )
		{
			cancelOptions();
		}

		String peid = ((JetspeedRunData) rundata).getJs_peid();
		SessionState sstate =
			((JetspeedRunData) rundata).getPortletSessionState(peid);

		// Release any lock on the site.
		SiteEdit site = state.getEditSite();
			
		if ( site != null )
		{
			SiteService.cancelEdit(site);
			state.setEditSite(null);
		}

		state.setIsListVM(true);
		state.setAttachments(null);
		state.setSelectedAttachments(null);
		state.setDeleteMessages(null);
		state.setStatus(CANCEL_STATUS);
		
		// we are done with customization... back to the main (list) mode
		sstate.removeAttribute(STATE_MODE);

		// re-enable auto-updates when going back to list mode
		enableObservers(sstate);


		
		try
		{
			if (state.getEdit() != null)
			{
				// get the channel id throught announcement service
				AnnouncementChannel channel =
					AnnouncementService.getAnnouncementChannel(
						this.getChannelIdFromReference(
							state.getEdit().getReference()));

				channel.cancelMessage(state.getEdit());
				state.setEdit(null);
			}
		}
		catch (IdUnusedException e)
		{
			if (Log.getLogger("chef").isDebugEnabled())
				Log.debug("chef", this +"doCancel()" + e);
		}
		catch (PermissionException e)
		{
			if (Log.getLogger("chef").isDebugEnabled())
				Log.debug("chef", this +"doCancel()" + e);
		}

		// make sure auto-updates are enabled
		((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER)).enable();


	} // doCancel

	/**
	 * Action is to use when doLinkcancel requested,
	 * corresponding to chef_announcement "eventSubmit_doLinkcancel"
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doLinkcancel(RunData rundata, Context context)
	{

		// retrieve the state from state object
		AnnouncementActionState state =
			(AnnouncementActionState) getState(context,
				rundata,
				AnnouncementActionState.class);

		String peid = ((JetspeedRunData) rundata).getJs_peid();
		SessionState sstate =
			((JetspeedRunData) rundata).getPortletSessionState(peid);

		state.setIsListVM(true);
		state.setAttachments(null);
		state.setSelectedAttachments(null);
		state.setDeleteMessages(null);
		state.setStatus(CANCEL_STATUS);

		// make sure auto-updates are enabled
		((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER)).enable();

	} // doLinkcancel

	/**
	 * Action is to use when doSavedraft requested,
	 * corresponding to chef_announcements-preview "eventSubmit_doSavedraft"
	 * @param runData The RunData object
	 * @param context The context object
	 */
	public void doSavedraft(RunData rundata, Context context)
	{
		// retrieve the state from state object
		AnnouncementActionState state = (AnnouncementActionState)getState( context, rundata, AnnouncementActionState.class );

		String peid = ((JetspeedRunData) rundata).getJs_peid();
		SessionState sstate= ((JetspeedRunData)rundata).getPortletSessionState(peid);

		// get the channel id information from state object
		String channelId = state.getChannelId();
		// get the message object through service
		String messageReference = state.getMessageReference();
		String subject = "";

		// *** make sure the subject and body won't be empty
		String from = rundata.getParameters().getString("fromInterface");
		if (from != null)
			if (from.equals("revise"))
			{
				// read in the subject input from announcements-new.vm
				subject = rundata.getParameters().getString("subject");
				// read in the body input
				String body = rundata.getParameters().getString("body");
				body = processFormattedTextFromBrowser(sstate, body);

				state.setTempSubject(subject);
				state.setTempBody(body);

				if (subject.length() == 0)
				{
					addAlert(sstate, "You need to fill in the subject!");
				}
				else if (body.length() == 0)
				{
					addAlert(sstate, "You need to fill in the body of the announcement!");
				}
			}

		// there is any error message caused by empty subject or boy
		if (sstate.getAttribute(STATE_MESSAGE) != null)
		{
			// read the public view setting and save it in session state
			String publicView = rundata.getParameters().getString("pubview");
			sstate.setAttribute(SSTATE_PUBLICVIEW_VALUE, publicView);
			
			// read the notification options & save it in session state
			String notification = rundata.getParameters().getString("notify");
			sstate.setAttribute(SSTATE_NOTI_VALUE, notification);

			state.setIsListVM(false);
			state.setStatus("stayAtRevise");

			// disable auto-updates while in view mode
			((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER)).disable();

		}
		else
		{
			try
			{
				// read in the subject input from announcements-new.vm
				subject = rundata.getParameters().getString("subject");
				// read in the body input
				String body = rundata.getParameters().getString("body");
				body = processFormattedTextFromBrowser(sstate, body);

				// if no input, which means coming from revise
				if ((subject == null) || (body == null))
				{
					// get the string value from state object
					subject = state.getTempSubject();
					body = state.getTempBody();
				}

				// read the notification options
				String notification = rundata.getParameters().getString("notify");
				
				// if from the post button of preview page
				if (notification == null)
					notification = (String)sstate.getAttribute(AnnouncementAction.SSTATE_NOTI_VALUE);

				int noti = NotificationService.NOTI_OPTIONAL;
				if ("r".equals(notification))
				{
					noti = NotificationService.NOTI_REQUIRED;
				}
				else if ("n".equals(notification))
				{
					noti = NotificationService.NOTI_NONE;
				}
			
				if (state.getIsNewAnnouncement())
				{
					// get the channel id throught announcement service
					AnnouncementChannel channel =
						AnnouncementService.getAnnouncementChannel(channelId);

					AnnouncementMessageEdit msg =
						channel.addAnnouncementMessage();
					msg.setBody(body);
					AnnouncementMessageHeaderEdit header =
						msg.getAnnouncementHeaderEdit();
					header.setSubject(subject);
					header.setDraft(true);
					header.replaceAttachments(state.getAttachments());
					
					// read the pubview settings
					boolean pubviewset = RealmService.getPubViewInheritance(msg.getReference());
					boolean pubview = false;
					if (!pubviewset)
					{
						//pubview = Boolean.valueOf(rundata.getParameters().getString("pubview")).booleanValue();
						String pubviewStr = rundata.getParameters().getString("pubview");
						// if from the post in preview, get the setting from sstate object
						if (pubviewStr == null)
							pubviewStr = (String) sstate.getAttribute(AnnouncementAction.SSTATE_PUBLICVIEW_VALUE);
						pubview = Boolean.valueOf(pubviewStr).booleanValue();
					}
					
					channel.commitMessage(msg);

					if (!pubviewset)
					{
						try
						{
							RealmService.setPubView(msg.getReference(), pubview);
						}
						catch (InUseException ee) {} // %%%
					}
					
					//set message id information in state object
					//messageReference = msg.getReference();
					//state.setMessageReference(messageReference);
				}
				else
				{
					if (subject.length() > 0)
					{
					// get the message object through service
					//AnnouncementMessageEdit msg = channel.editAnnouncementMessage( messageId );
					AnnouncementMessageEdit msg = state.getEdit();

					// get the channel id throught announcement service
					AnnouncementChannel channel =
						AnnouncementService.getAnnouncementChannel(
							this.getChannelIdFromReference(msg.getReference()));

					msg.setBody(body);
					AnnouncementMessageHeaderEdit header =
						msg.getAnnouncementHeaderEdit();
					header.setSubject(subject);
					header.setDraft(true);
					header.replaceAttachments(state.getAttachments());
						
						// read the pubview settings
						boolean pubviewset = RealmService.getPubViewInheritance(msg.getReference());
						boolean pubview = false;
						if (!pubviewset)
						{
							//pubview = Boolean.valueOf(rundata.getParameters().getString("pubview")).booleanValue();
							String pubviewStr = rundata.getParameters().getString("pubview");
							// if from the post in preview, get the setting from sstate object
							if (pubviewStr == null)
								pubviewStr = (String) sstate.getAttribute(AnnouncementAction.SSTATE_PUBLICVIEW_VALUE);
							pubview = Boolean.valueOf(pubviewStr).booleanValue();
						}
						
					channel.commitMessage(msg);
					//set message id information in state object
						//messageReference = msg.getReference();
						//state.setMessageReference(messageReference);

						if (!pubviewset)
						{
							try
							{
								RealmService.setPubView(msg.getReference(), pubview);
							}
							catch (InUseException ee)
							{} // %%%
						}

					state.setEdit(null);
				}
			}
			}
			catch (IdUnusedException e)
			{
				if (Log.getLogger("chef").isDebugEnabled())
					Log.debug("chef", this +"doSavedraft()" + e);
			}
			catch (PermissionException e)
			{
				addAlert(sstate, "you don't have permissions to create this announcement -"+ subject);
				if (Log.getLogger("chef").isDebugEnabled())
					Log.debug("chef", this +"doSavedraft()" + e);
			} // try-catch

			state.setStatus(CANCEL_STATUS);
			state.setAttachments(null);
			state.setIsListVM(true);

			// make sure auto-updates are enabled
			((EventsObservingCourier) sstate.getAttribute(STATE_OBSERVER)).enable();
		}
	} //doSavedraft

	// ********* starting for sorting *********

	/**
	 * Does  initialization of sort parameters in the state.
	 * @param rundata
	 * @param context
	 * @param field
	 */
	private void setupSort(RunData rundata, Context context, String field)
	{
		// retrieve the state from state object
		AnnouncementActionState state =
			(AnnouncementActionState) getState(context,
				rundata,
				AnnouncementActionState.class);

		if (state.getCurrentSortedBy().equals(field))
		{
			// current sorting sequence
			boolean asc = state.getCurrentSortAsc();

			//toggle between the ascending and descending sequence
			if (asc)
				asc = false;
			else
				asc = true;

			state.setCurrentSortAsc(asc);
		}
		else
		{
			// if the messages are not already sorted by subject, set the sort sequence to be ascending
			state.setCurrentSortedBy(field);
			state.setCurrentSortAsc(true);
		}
	} // setupSort

	/**
	 * Do sort by subject
	 * @param runData The RunData object for this request
	 * @param context The Context used by the template
	 */
	public void doSortbysubject(RunData rundata, Context context)
	{
		if (Log.getLogger("chef").isDebugEnabled())
			Log.debug("chef", "AnnouncementAction.doSortbysubject get Called");

		setupSort(rundata, context, "subject");
	} // doSortbysubject

	/**
	 * Do sort by from - the author
	 * @param runData The Rundata object for this request
	 * @param context The Context used by the template
	 */
	public void doSortbyfrom(RunData rundata, Context context)
	{
		if (Log.getLogger("chef").isDebugEnabled())
			Log.debug("chef", "AnnouncementAction.doSortbyfrom get Called");

		setupSort(rundata, context, "from");
	} // doSortbyfrom

	/**
	 * Do sort by the date of the announcement.
	 * @param runData The RunData object for this request
	 * @param context The Context used by the template
	 */
	public void doSortbydate(RunData rundata, Context context)
	{
		if (Log.getLogger("chef").isDebugEnabled())
			Log.debug("chef", "AnnouncementAction.doSortbydate get Called");

		setupSort(rundata, context, "date");
	} // doSortbydate

	/**
	 * Do sort by the announcement channel name.
	 * @param runData The RunData object for this request
	 * @param context The Context used by the template
	 */
	public void doSortbychannel(RunData rundata, Context context)
	{
		if (Log.getLogger("chef").isDebugEnabled())
			Log.debug("chef", "AnnouncementAction.doSortbychannel get Called");

		setupSort(rundata, context, "channel");
	} // doSortbydate

	private class AnnouncementComparator implements Comparator
	{
		// the criteria
		String m_criteria = null;

		// the criteria - asc
		boolean m_asc = true;

		/**
		 * constructor
		 * @param criteria The sort criteria string
		 * @param asc The sort order string. "true" if ascending; "false" otherwise.
		 */
		public AnnouncementComparator(String criteria, boolean asc)
		{
			m_criteria = criteria;
			m_asc = asc;

		} // constructor

		/**
		 * implementing the compare function
		 * @param o1 The first object
		 * @param o2 The second object
		 * @return The compare result. 1 is o1 < o2; -1 otherwise
		 */
		public int compare(Object o1, Object o2)
		{
			int result = -1;

			if (m_criteria.equals("subject"))
			{
				// sorted by the discussion message subject
				result =
					((AnnouncementMessage) o1)
						.getAnnouncementHeader()
						.getSubject()
						.compareToIgnoreCase(
							((AnnouncementMessage) o2)
								.getAnnouncementHeader()
								.getSubject());
			}
			else
				if (m_criteria.equals("date"))
				{
					// sorted by the discussion message date
					if (((AnnouncementMessage) o1)
						.getAnnouncementHeader()
						.getDate()
						.before(
							((AnnouncementMessage) o2)
								.getAnnouncementHeader()
								.getDate()))
					{
						result = -1;
					}
					else
					{
						result = 1;
					}
				}
				else
					if (m_criteria.equals("from"))
					{
						// sorted by the discussion message subject
						result =
							((AnnouncementMessage) o1)
								.getAnnouncementHeader()
								.getFrom()
								.getSortName()
								.compareToIgnoreCase(
									((AnnouncementMessage) o2)
										.getAnnouncementHeader()
										.getFrom()
										.getSortName());
					}
					else
						if (m_criteria.equals("channel"))
						{
							// sorted by the channel name.
							result =
								((AnnouncementWrapper) o1)
									.getChannelDisplayName()
									.compareToIgnoreCase(
									((AnnouncementWrapper) o2)
										.getChannelDisplayName());
						}

			// sort ascending or descending
			if (!m_asc)
			{
				result = -result;
			}
			return result;

		} // compare

	} // AnnouncementComparator

	// ********* ending for sorting *********

	/**
	 * Action is to parse the function calls
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	/*
	public void doParse_list_announcement(RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState(((JetspeedRunData)data).getJs_peid());
		ParameterParser params = data.getParameters();
		
		String source = params.getString("source");
		if (source.equalsIgnoreCase("new"))
		{
			// create new announcement
			doNewannouncement(data, context);
		}
		else if (source.equalsIgnoreCase("revise"))
		{
			// revise announcement
			doReviseannouncementfrommenu(data, context);
		}
		else if (source.equalsIgnoreCase("delete"))
		{
			// delete announcement
			doDeleteannouncement(data, context);
		}
		
	}	// doParse_list_announcement
	*/

	/**
	 * Build the menu.
	 */
	private void buildMenu(
		VelocityPortlet portlet,
		Context context,
		RunData rundata,
		AnnouncementActionState state,
		boolean menu_new,
		boolean menu_delete,
		boolean menu_revise,
		boolean menu_merge,
		boolean menu_permissions,
		boolean menu_options,
		AnnouncementActionState.DisplayOptions displayOptions)
	{
		Menu bar = new Menu(portlet, rundata, "AnnouncementAction");
		boolean buttonRequiringCheckboxesPresent = false;
		
		if (!displayOptions.isShowOnlyOptionsButton())
		{
			String statusName = state.getStatus();
			if (statusName != null)
			{
				if (statusName.equals("showMetadata"))
				{
					bar.add(new MenuEntry("New...",null, menu_new,	MenuItem.CHECKED_NA,"doNewannouncement"));
					bar.add(new MenuEntry("Delete",null, menu_delete,MenuItem.CHECKED_NA,"doDeleteannouncement"));
					bar.add(new MenuEntry("Revise",null,menu_revise,MenuItem.CHECKED_NA,"doReviseannouncementfrommenu"));
					buttonRequiringCheckboxesPresent=true ;
					//bar.add( new MenuEntry("Go home", null, true/* visible at all views */, MenuItem.CHECKED_NA, "doHome") );
				}
				else
				{
					bar.add(new MenuEntry("New...",null,menu_new,MenuItem.CHECKED_NA,"doNewannouncement","announcementListForm"));
					bar.add(new MenuEntry("Delete",null,menu_delete,MenuItem.CHECKED_NA,"doDeleteannouncement","announcementListForm"));
					bar.add(new MenuEntry("Revise",null,menu_revise,MenuItem.CHECKED_NA,"doReviseannouncementfrommenu","announcementListForm"));
					//bar.add( new MenuEntry("Go home", null, true/* visible at all views */, MenuItem.CHECKED_NA, "doHome") );
					buttonRequiringCheckboxesPresent=true ;
					
				} // if (statusName.equals("showMetadata"))
			}
			else
			{
				bar.add(new MenuEntry("New...",null,menu_new,MenuItem.CHECKED_NA,"doNewannouncement","announcementListForm"));
				bar.add(new MenuEntry("Delete",null,menu_delete,MenuItem.CHECKED_NA,"doDeleteannouncement","announcementListForm"));
				bar.add(new MenuEntry("Revise",null,menu_revise,MenuItem.CHECKED_NA,"doReviseannouncementfrommenu","announcementListForm"));
				buttonRequiringCheckboxesPresent=true ;
				//bar.add( new MenuEntry("Go home", null, true/* visible at all views */, MenuItem.CHECKED_NA, "doHome") );
			} // if-else (statusName != null)
	
			// add merge button, if allowed
			if (menu_merge)
			{
				bar.add(new MenuEntry(MERGE_BUTTON_CAPTION, MERGE_BUTTON_HANDLER));
			}
		} // if-else (!displayOptions.isShowOnlyOptionsButton())

		// add options if allowed
		if (menu_options)
		{
			addOptionsMenu(bar, (JetspeedRunData) rundata);
		}
		
		// let the permissions button to be the last one in the toolbar
		if (!displayOptions.isShowOnlyOptionsButton())
		{
			// add permissions, if allowed
			if (menu_permissions)
			{
				bar.add(new MenuEntry(PERMISSIONS_BUTTON_CAPTION,PERMISSIONS_BUTTON_HANDLER));
			}
		}

		// Set menu state attribute
		SessionState stateForMenus =
			((JetspeedRunData) rundata).getPortletSessionState(portlet.getID());
		stateForMenus.setAttribute(MenuItem.STATE_MENU, bar);

		Iterator it = bar.getItems().iterator();

		// See if we have any enabled menu items.
		boolean enabledItemExists = false;

		while (it.hasNext())
		{
			MenuItem menuItem = (MenuItem) it.next();
			if (menuItem.getIsEnabled())
			{
				enabledItemExists = true;
				break;
			}
		}

		// Set a flag in the context to indicate that at least one menu item is enabled.
		context.put(
			ENABLED_MENU_ITEM_EXISTS,
			Boolean.valueOf(enabledItemExists));
			
		context.put(CONTEXT_ENABLE_ITEM_CHECKBOXES, Boolean.valueOf(enabledItemExists && buttonRequiringCheckboxesPresent) );
		context.put(CONTEXT_ENABLED_MENU_ITEM_EXISTS, Boolean.valueOf(enabledItemExists) );
		
		context.put(Menu.CONTEXT_MENU, bar);
		context.put(Menu.CONTEXT_ACTION, "AnnouncementAction");

	} // buildMenu

	/* what i've done to make this tool automaticlly updated
	 * includes some corresponding imports
	 * in buildMail, tell observer just the page is just refreshed
	 * in the do() functions related to show the list, enable the obeserver
	 * in other do() functions related to  not show the list, disable the obeserver
	 * in the do(), define the session sstate object, and protlet.
	 *
	 * add initState
	 * add updateObservationOfChannel()
	 * add state attribute STATE_CHANNEL_REF
	 */
	
	
	/**
	 * Populate the state object, if needed.
	 * @param state The state object.
	 * @param portlet The portlet.
	 * @param rundata The current request's rundata.
	 */
	protected void initState(SessionState state, VelocityPortlet portlet, JetspeedRunData rundata)
	{
		super.initState(state, portlet, rundata);

		// retrieve the state from state object
		AnnouncementActionState annState = (AnnouncementActionState)getState( portlet, rundata, AnnouncementActionState.class );

		// get the current channel ID from state object or prolet initial parameter
		String channelId = annState.getChannelId();
		if (channelId == null)
		{
			// try the portlet parameter
			channelId = StringUtil.trimToNull(portlet.getPortletConfig().getInitParameter("channel"));
			if (channelId == null)
			{
				// form based on the request's site's "main" channel
				channelId = AnnouncementService.channelReference(
				PortalService.getCurrentSiteId(), SiteService.MAIN_CONTAINER);
			}

			// let the state object have the current channel id
			annState.setChannelId(channelId);
			annState.setIsListVM(true);
		}
		state.setAttribute(STATE_CHANNEL_REF, channelId);

		// get the current collection ID from state object or prolet initial parameter
		String collectionId = annState.getCollectionId();
		if (collectionId == null)
		{
			// get the current channel ID for prolet initial parameter
			collectionId = StringUtil.trimToNull(portlet.getPortletConfig().getInitParameter("collection"));
			if (collectionId == null)
				collectionId = ContentHostingService.getSiteCollection(PortalService.getCurrentSiteId());

			// let the state object have the current channel id
			annState.setCollectionId(collectionId);
		}

		//String channel = StringUtil.trimToNull(config.getInitParameter(PARAM_CHANNEL));
		// setup the observer to notify our main panel
		if (state.getAttribute(STATE_OBSERVER) == null)
		{
			// the delivery location for this tool
			String deliveryId = clientWindowId(state, portlet.getID());

			// the html element to update on delivery
			String elementId = mainPanelUpdateId(portlet.getID());

			// the event resource reference pattern to watch for
			Reference r = new Reference(channelId);
			String pattern = AnnouncementService.messageReference(r.getContext(), r.getId(), "");

			EventsObservingCourier observer = new EventsObservingCourier(deliveryId, elementId, pattern);
			
			state.setAttribute(STATE_OBSERVER, observer);

			MergedList mergedAnnouncementList = new MergedList();

			MergedList.EntryProvider entryProvider = new EntryProvider();

			// Get the list of merged announcement sources.
			mergedAnnouncementList.loadFromPortletConfig(
					portlet,
					annState.getChannelId(),
					isOnWorkspaceTab(),
					PORTLET_CONFIG_PARM_NON_MERGED_CHANNELS,
					entryProvider);
			
			// make sure the observer is in sync with state
			updateObservationOfChannel(mergedAnnouncementList, rundata, state, annState);
		}
			
		// Set up or display options if we haven't done it yet.
		if (annState.getDisplayOptions() == null)
		{
			loadDisplayOptionsFromPortletConfig(portlet, annState);
		}
		

	} // initState

	/**
	 * Adds the merged sites to the list of events that we're interested
	 * in watching.
	 * @param mergedAnnouncementList
	 * @param annState
	 * @param observer
	 */
	private void addMergedAnnouncementsToObserver(MergedList mergedAnnouncementList, AnnouncementActionState annState, EventsObservingCourier observer)
	{
		Iterator it = mergedAnnouncementList.iterator();
		
		while (it.hasNext())
		{
			MergedList.MergedEntry entry = (MergedList.MergedEntry) it.next();

			if ( entry.isMerged() )
			{
				Reference ref = new Reference(entry.getReference());

				String pattern =
					AnnouncementService.messageReference(
						ref.getContext(),
						ref.getId(),
						"");
	
				observer.addResourcePattern(pattern);				
			}
		}
	}

	/**
	 * Loads the display options object we save in the ActionState with the
	 * settings from the PortletConfig.
	 * @param portlet
	 * @param annState
	 */
	private void loadDisplayOptionsFromPortletConfig(
		VelocityPortlet portlet,
		AnnouncementActionState annState)
	{
		AnnouncementActionState.DisplayOptions displayOptions = new AnnouncementActionState.DisplayOptions();
		annState.setDisplayOptions(displayOptions);
		
		PortletConfig portletConfig = portlet.getPortletConfig();
		
		displayOptions.loadProperties(portletConfig.getInitParameters());
		
	} // initState

	/**
	 * Setup our observer to be watching for change events for our channel.
	 * @param mergedAnnouncementList
	 * @param runData
	 * @param state
	 * @param annState
	 */
	private void updateObservationOfChannel(MergedList mergedAnnouncementList, RunData runData, SessionState state, AnnouncementActionState annState)
	{
		String peid = ((JetspeedRunData) runData).getJs_peid();
		
		EventsObservingCourier observer =
			(EventsObservingCourier) state.getAttribute(STATE_OBSERVER);

		addMergedAnnouncementsToObserver(mergedAnnouncementList, annState, observer);

		// the delivery location for this tool
		String deliveryId = clientWindowId(state, peid);
		observer.setDeliveryId(deliveryId);

	} // updateObservationOfChannel

	/**
	* Fire up the permissions editor
	*/
	public void doPermissions(RunData data, Context context)
	{
		SessionState state =
			((JetspeedRunData) data).getPortletSessionState(
				((JetspeedRunData) data).getJs_peid());
		
		// retrieve the state from state object
		AnnouncementActionState stateObj =
			(AnnouncementActionState) getState(context, data, AnnouncementActionState.class);
		// change the return state to be a clean list view without alert messages
		stateObj.setStatus(CANCEL_STATUS);
		

		// retrieve the state from state object
		AnnouncementActionState astate =
			(AnnouncementActionState) getState(context,
				data,
				AnnouncementActionState.class);

		String channelRefStr = astate.getChannelId();
		Reference channelRef = new Reference(channelRefStr);
		String siteRef = SiteService.siteReference(channelRef.getContext());

		// setup for editing the permissions of the site for this tool, using the roles of this site, too
		state.setAttribute(PermissionsAction.STATE_REALM_ID, siteRef);
		state.setAttribute(PermissionsAction.STATE_REALM_ROLES_ID, siteRef);

		// ... with this description
		state.setAttribute(
			PermissionsAction.STATE_DESCRIPTION,
			"Set permissions for Announcements in worksite "
				+ SiteService.getSiteDisplay(channelRef.getContext()));

		// ... showing only locks that are prpefixed with this
		state.setAttribute(PermissionsAction.STATE_PREFIX, "annc.");

		// start the helper
		state.setAttribute(
			PermissionsAction.STATE_MODE,
			PermissionsAction.MODE_MAIN);

	} // doPermissions


	/**
	 * Handle the "Merge" button on the toolbar
	 * @param runData
	 * @param context
	 */
	public void doMerge(
		RunData runData,
		Context context)
	{
		AnnouncementActionState state = (AnnouncementActionState)getState(context, runData, AnnouncementActionState.class);
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState sstate = ((JetspeedRunData)runData).getPortletSessionState(peid);

// TODO: ???
//		// get a lock on the site and setup for options work
//		VelocityPortletPaneledAlert alert = new VelocityPortletPaneledAlert();
//		alert.doOptions(runData, context);
		doOptions(runData, context);

		// if we didn't end up in options mode, bail out
		if (!MODE_OPTIONS.equals(sstate.getAttribute(STATE_MODE)))
			return;

		// Disable the observer
		enableObserver(sstate, false);

		state.setStatus(MERGE_STATUS);
	} // doMerge

	/**
	 * Handles the user clicking on the save button on the page to specify which
	 * calendars will be merged into the present schedule.
	 * @param runData The RunData object
	 * @param context The context object
	 */
	public void doUpdate(
		RunData runData,
		Context context)
	{
		AnnouncementActionState state = (AnnouncementActionState)getState(context, runData, AnnouncementActionState.class);
		
		if ( state.getStatus().equals(MERGE_STATUS) )
		{
			doMergeUpdate(runData, context);
		}
		else
		if ( state.getStatus().equals(OPTIONS_STATUS))
		{
			doOptionsUpdate(runData, context);
		}
		else
		{
			Log.debug("chef", this +".doUpdate - Unexpected status");
		}
	}
	
	/**
	 * This handles the "doUpdate" if we're processing an update from the "merge" page.
	 * @param runData
	 * @param context
	 */
	private void doMergeUpdate(RunData runData, Context context)
	{
		AnnouncementActionState state = (AnnouncementActionState)getState(context, runData, AnnouncementActionState.class);
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState sstate = ((JetspeedRunData)runData).getPortletSessionState(peid);
		
		// Get the merged calendar list out of our session state
		MergedList mergedChannelList =
			(MergedList) sstate.getAttribute(SSTATE_ATTRIBUTE_MERGED_CHANNELS);
		
		if (mergedChannelList != null)
		{
			// Get the information from the run data and load it into
			// our calendar list that we have in the session state.
			mergedChannelList.loadFromRunData(runData);
		}
		else
		{
			if (Log.getLogger("chef").isDebugEnabled())
			{
				Log.debug("chef", this +".doUpdate mergedChannelList == null");
			}
		}
		
		// update the tool config
		ToolConfigurationEdit tool =
			(ToolConfigurationEdit) sstate.getAttribute(STATE_TOOL_EDIT);
		if (mergedChannelList != null)
		{
			tool.getPropertiesEdit().addProperty(
				PORTLET_CONFIG_PARM_NON_MERGED_CHANNELS,
				mergedChannelList.getPortletConfigString());
		}
		else
		{
			tool.getPropertiesEdit().removeProperty(
				PORTLET_CONFIG_PARM_NON_MERGED_CHANNELS);
		}
		
		// commit the change
		saveOptions();
		
		updateObservationOfChannel(mergedChannelList, runData, sstate, state);

		// Turn the observer back on.
		enableObserver(sstate, true);
		
		state.setStatus(null);
		
		sstate.removeAttribute(STATE_MODE);
	} // doUpdate} //	AnnouncementAction


	/**
	 * This handles the "doUpdate" if we're in a processing an update from the options page.
	 * @param runData
	 * @param context
	 */
	public void doOptionsUpdate(RunData runData, Context context)
	{
		AnnouncementActionState state = (AnnouncementActionState)getState(context, runData, AnnouncementActionState.class);
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState sstate = ((JetspeedRunData)runData).getPortletSessionState(peid);
		
		doUpdateDisplayOptions(runData, state, sstate);
	
		// We're omitting processing of the "showAnnouncementBody" since these
		// options are currently mutually exclusive.
	
		// commit the change
		saveOptions();
		
		// Turn the observer back on.
		enableObserver(sstate, true);
		
		state.setStatus(null);
		
		sstate.removeAttribute(STATE_MODE);
	}

	
	/* (non-Javadoc)
	 * @see org.chefproject.actions.VelocityPortletPaneledAction#doOptions(org.apache.turbine.util.RunData, org.apache.velocity.context.Context)
	 */
	public void doOptions(RunData runData, Context context)
	{
		super.doOptions(runData, context);
		
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState sstate = ((JetspeedRunData)runData).getPortletSessionState(peid);
		
		// go to option editing if no error message which means the option editing is locked successfully
		// otherwise, stay at whatever state we are at
		String msg = (String)sstate.getAttribute(STATE_MESSAGE);
		if ((msg == null)||(msg.equals("")))
		{
			// Keep track of our state
			AnnouncementActionState state = (AnnouncementActionState)getState(context, runData, AnnouncementActionState.class);
			state.setStatus(OPTIONS_STATUS);
		}
	} // doOptions

	/**
	 * @param runData
	 * @param state
	 * @param sstate
	 */
	private void doUpdateDisplayOptions(
		RunData runData,
		AnnouncementActionState state,
		SessionState sstate)
	{
		AnnouncementActionState.DisplayOptions displayOptions = state.getDisplayOptions();
		
		ParameterParser parameters = runData.getParameters();
		
		displayOptions.loadProperties(parameters);
		
		// update the tool config
		ToolConfigurationEdit tool =
			(ToolConfigurationEdit) sstate.getAttribute(STATE_TOOL_EDIT);
		
		displayOptions.saveProperties(tool.getPropertiesEdit());
	}   // doUpdateDisplayOptions
	
	/**
	 * is notification enabled?
	 */
	protected boolean notificationEnabled(AnnouncementActionState state)
	{
		return true;
	}	// notificationEnabled
	
	/**
	 * Processes formatted text that is coming back from the browser 
	 * (from the formatted text editing widget).
	 * @param state Used to pass in any user-visible alerts or errors when processing the text
	 * @param strFromBrowser The string from the browser
	 * @return The formatted text
	 */
	private String processFormattedTextFromBrowser(SessionState state, String strFromBrowser)
	{
		StringBuffer alertMsg = new StringBuffer();
		String text = FormattedText.processFormattedText(strFromBrowser, alertMsg);
		if (alertMsg.length() > 0) addAlert(state, alertMsg.toString());
		return text;
	}
}

/**********************************************************************************
*
* $Header: /cvs/sakai/chef-tool/src/java/org/sakaiproject/tool/announcement/AnnouncementAction.java,v 1.13 2004/06/22 03:04:56 ggolden Exp $
*
***********************************************************************************/
