/**********************************************************************************
*
* $Header: /cvs/sakai/chef-tool/src/java/org/sakaiproject/tool/authn/LoginServlet.java,v 1.8 2004/06/22 03:05:00 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.tool.authn;

// imports
import java.io.IOException;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.sakaiproject.cheftool.VmServlet;
import org.sakaiproject.service.framework.config.cover.ServerConfigurationService;
import org.sakaiproject.service.framework.log.cover.Log;
import org.sakaiproject.service.framework.log.cover.Logger;
import org.sakaiproject.service.framework.session.cover.UsageSessionService;
import org.sakaiproject.service.framework.state.SessionState;
import org.sakaiproject.service.framework.state.cover.StateService;
import org.sakaiproject.service.legacy.user.User;
import org.sakaiproject.service.legacy.user.cover.UserDirectoryService;
import org.sakaiproject.util.StringUtil;

/**
* <p>LoginServlet handles user login.</p>
* 
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.8 $
*/
public class LoginServlet extends VmServlet
{
	/** HTTP State attribute for the redirect. */
	public final static String REDIRECT = "AuthnRedirect";

	/** path to indicate redirect should be to the MyWorkspace. */
	public final static String MYWORKSPACE_PATH = "/myworkspace";

	/** path to the login screen. */
	protected final static String LOGIN_PATH = "/login";

	/** error template. */
	protected final static String ERROR_VM = "/vm/authn/error.vm";

	/** login template. */
	protected final static String LOGIN_VM = "login";

	/**
	 * initialize the servlet
	 *
	 * @param config the servlet config parameter
	 * @exception ServletException in case of difficulties
	 */
	public void init(ServletConfig config) throws ServletException
	{
		super.init(config);
	}

	/**
	 * respond to an HTTP GET request
	 *
	 * @param req HttpServletRequest object with the client request
	 * @param res HttpServletResponse object back to the client
	 * @exception ServletException in case of difficulties
	 * @exception IOException in case of difficulties
	 */
	public void doGet(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException
	{
		dispatch(req, res);
	}

	/**
	 * respond to an HTTP POST request
	 *
	 * @param req HttpServletRequest object with the client request
	 * @param res HttpServletResponse object back to the client
	 * @exception ServletException in case of difficulties
	 * @exception IOException in case of difficulties
	 */
	public void doPost(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException
	{
		dispatch(req, res);
	}

	/**
	 * handle get and post communication from the user
	 * @param req HttpServletRequest object with the client request
	 * @param res HttpServletResponse object back to the client
	 */
	public void dispatch(HttpServletRequest req, HttpServletResponse res)
	{
		// some state for login
		SessionState state = StateService.getCurrentSessionState(LoginServlet.class.getName());

		try
		{
			// get the path info
			// TODO: UTF-8! String path = ChefParameterParser.getPathDecoded(req);
			String path = req.getPathInfo();
			if (path == null)
				path = "";

			// if requested, clear any leftover redirects
			if (MYWORKSPACE_PATH.equals(path))
			{
				state.removeAttribute(REDIRECT);
			}

			// is this login response?
			if (LOGIN_PATH.equals(path))
			{
				// process the login parameters, and if unsuccessfull send out the login screen
				boolean outputSent = processLogin(req, res);

				// if we sent out the login screen, we are done this request
				if (outputSent)
					return;
			}

			// if not yet logged in, and we are taking container authentication, try to login
			User user = UsageSessionService.getSessionUser();
			boolean loggedIn = !(user.equals(UserDirectoryService.getAnonymousUser()));
			// TODO: container auth.
			//			boolean containerAuthentication = "true".equals(JetspeedResources.getString("container.auth"));
			//			if ((!loggedIn) && (containerAuthentication))
			//			{
			//				String userId = data.getRequest().getRemoteUser();
			//				if (userId != null)
			//				{
			//					if (Log.getLogger("chef").isDebugEnabled())
			//						Log.debug("chef", this + ": Logging in from remoteUser: " + userId);
			//	
			//					JetspeedUser juser = null;
			//					try
			//					{
			//						juser = JetspeedSecurity.login(userId, null);
			//	
			//						user = UsageSessionService.getSessionUser();
			//						loggedIn = !(user.equals(UserDirectoryService.getAnonymousUser()));
			//					}
			//					catch (LoginException e)
			//					{
			//						Log.warn("chef", this + ": login exception with remoteUser: " + e);
			//					}
			//				}
			//			}

			// if (still) not logged in yet, do a login
			if (!loggedIn)
			{
				processLogin(req, res);
				return;
			}

			// logged in, so redirect
			String redirect = (String) state.getAttribute(REDIRECT);
			state.removeAttribute(REDIRECT);

			// just in case we have no redirect, send them home
			if (redirect == null)
				redirect = ServerConfigurationService.getUserHomeUrl();

			res.sendRedirect(redirect);
		}
		catch (Exception e)
		{
			Log.warn("chef", this +".dispatch(): exception: ", e);
			respondError(req, res, e.toString());
		}
	}

	/**
	 * Make the "error" response for problems processing the request.
	 * @param req HttpServletRequest object with the client request.
	 * @param res HttpServletResponse object back to the client.
	 * @param ref The request reference string.
	 * @param msg The error message.
	 */
	protected void respondError(HttpServletRequest req, HttpServletResponse res, String msg)
	{
		try
		{
			setVmReference("msg", msg, req);
			includeVm(ERROR_VM, req, res);
		}
		catch (Throwable t)
		{
			Logger.warn(this +".respondError(): ", t);
		}

	}

	/**
	 * Handle a "/login" response
	 * @param req HttpServletRequest object with the client request.
	 * @param res HttpServletResponse object back to the client.
	 * @return true if output was sent (i.e. the login screen), false if not
	 */
	protected boolean processLogin(HttpServletRequest req, HttpServletResponse res)
	{
		boolean attempted = false;

		// if there are username and password parameters, authenticate
		String fromLogin = req.getParameter("fromLogin");
		String name = StringUtil.trimToNull(req.getParameter("username"));
		String pw = StringUtil.trimToZero(req.getParameter("password"));

		// this will be set (in the URL) if being redirected from login after an invalid attempt
		String err = req.getParameter("err");
		if (err != null)
		{
			attempted = true;
		}

		if (fromLogin != null)
		{
			try
			{
				attempted = true;

				if (UsageSessionService.loginSessionUser(name, pw, req.getRemoteAddr(), req.getHeader("user-agent")))
				{
					// successfully logged in
					return false;
				}
			}
			catch (Exception e)
			{
				Logger.warn(this +".processLogin(): exception logging in", e);
			}
		}

		// unsuccessfull login
		try
		{
			String userIdText = ServerConfigurationService.getBoolean("email-for-user-id", true) ? "email" : "user id";
			setVmReference("userid_text", userIdText, req);

			setVmReference("login_path", LOGIN_PATH, req);

			if (attempted)
			{
				setVmReference("alertMessage", "Invalid " + userIdText + " or password", req);
			}

			// pick our interFACE
			String face = req.getParameter("face");
			if (face == null)
			{
				face = LOGIN_VM;
			}
			includeVm("/vm/authn/" + face + ".vm", req, res);
		}
		catch (Exception e)
		{
			Logger.warn(this +".respondLogin(): ", e);
		}

		return true;
	}
}

/**********************************************************************************
*
* $Header: /cvs/sakai/chef-tool/src/java/org/sakaiproject/tool/authn/LoginServlet.java,v 1.8 2004/06/22 03:05:00 ggolden Exp $
*
**********************************************************************************/
