/**********************************************************************************
*
* $Header: /cvs/sakai/chef-tool/src/java/org/sakaiproject/tool/authn/WelcomeServlet.java,v 1.6 2004/06/22 03:05:00 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.tool.authn;

// imports
import java.io.IOException;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.sakaiproject.cheftool.VmServlet;
import org.sakaiproject.service.framework.log.cover.Log;
import org.sakaiproject.service.framework.log.cover.Logger;
import org.sakaiproject.service.framework.session.cover.UsageSessionService;

/**
* <p>LoginServlet is a placeholder for a myWorkspace.</p>
* 
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.6 $
*/
public class WelcomeServlet extends VmServlet
{
	/** welcome template. */
	protected final static String WELCOME_VM = "/vm/authn/welcome.vm";

	/** error template. */
	protected final static String ERROR_VM = "/vm/authn/error.vm";

	/**
	 * initialize the servlet
	 *
	 * @param config the servlet config parameter
	 * @exception ServletException in case of difficulties
	 */
	public void init(ServletConfig config) throws ServletException
	{
		super.init(config);
	}

	/**
	 * respond to an HTTP GET request
	 *
	 * @param req HttpServletRequest object with the client request
	 * @param res HttpServletResponse object back to the client
	 * @exception ServletException in case of difficulties
	 * @exception IOException in case of difficulties
	 */
	public void doGet(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException
	{
		dispatch(req, res);
	}

	/**
	 * respond to an HTTP POST request
	 *
	 * @param req HttpServletRequest object with the client request
	 * @param res HttpServletResponse object back to the client
	 * @exception ServletException in case of difficulties
	 * @exception IOException in case of difficulties
	 */
	public void doPost(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException
	{
		dispatch(req, res);
	}

	/**
	 * handle get and post communication from the user
	 * @param req HttpServletRequest object with the client request
	 * @param res HttpServletResponse object back to the client
	 */
	public void dispatch(HttpServletRequest req, HttpServletResponse res)
	{
		try
		{
			setVmReference("user", UsageSessionService.getSessionUser(), req);
			includeVm(WELCOME_VM, req, res);
		}
		catch (Exception e)
		{
			Log.warn("chef", this +".dispatch(): exception: ", e);
			respondError(req, res, e.toString());
		}
	}

	/**
	 * Make the "error" response for problems processing the request.
	 * @param req HttpServletRequest object with the client request.
	 * @param res HttpServletResponse object back to the client.
	 * @param ref The request reference string.
	 * @param msg The error message.
	 */
	protected void respondError(HttpServletRequest req, HttpServletResponse res, String msg)
	{
		try
		{
			setVmReference("msg", msg, req);
			includeVm(ERROR_VM, req, res);
		}
		catch (Throwable t)
		{
			Logger.warn(this +".respondError(): ", t);
		}

	}
}

/**********************************************************************************
*
* $Header: /cvs/sakai/chef-tool/src/java/org/sakaiproject/tool/authn/WelcomeServlet.java,v 1.6 2004/06/22 03:05:00 ggolden Exp $
*
**********************************************************************************/
