/**********************************************************************************
*
* $Header: /cvs/sakai/chef-tool/src/java/org/sakaiproject/tool/chat/ChatAction.java,v 1.11 2004/06/22 03:04:58 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.tool.chat;

// imports
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.sakaiproject.cheftool.Context;
import org.sakaiproject.cheftool.JetspeedRunData;
import org.sakaiproject.cheftool.PortletConfig;
import org.sakaiproject.cheftool.RunData;
import org.sakaiproject.cheftool.VelocityPortlet;
import org.sakaiproject.cheftool.VelocityPortletPaneledAction;
import org.sakaiproject.cheftool.menu.Menu;
import org.sakaiproject.cheftool.menu.MenuEntry;
import org.sakaiproject.cheftool.menu.MenuItem;
import org.sakaiproject.exception.IdInvalidException;
import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.exception.IdUsedException;
import org.sakaiproject.exception.PermissionException;
import org.sakaiproject.service.legacy.chat.ChatChannel;
import org.sakaiproject.service.legacy.chat.ChatChannelEdit;
import org.sakaiproject.service.legacy.chat.ChatMessage;
import org.sakaiproject.service.legacy.chat.ChatMessageEdit;
import org.sakaiproject.service.legacy.chat.cover.ChatService;
import org.sakaiproject.service.framework.config.cover.ServerConfigurationService;
import org.sakaiproject.service.framework.courier.cover.CourierService;
import org.sakaiproject.service.framework.portal.cover.PortalService;
import org.sakaiproject.service.legacy.presence.cover.PresenceService;
import org.sakaiproject.service.legacy.resource.Reference;
import org.sakaiproject.service.framework.session.cover.UsageSessionService;
import org.sakaiproject.service.legacy.site.ToolConfigurationEdit;
import org.sakaiproject.service.legacy.site.cover.SiteService;
import org.sakaiproject.service.framework.state.SessionState;
import org.sakaiproject.service.legacy.time.Time;
import org.sakaiproject.service.legacy.time.TimeBreakdown;
import org.sakaiproject.service.legacy.time.cover.TimeService;
import org.sakaiproject.util.Validator;
import org.sakaiproject.tool.helper.PermissionsAction;
import org.sakaiproject.util.Filter;
import org.sakaiproject.util.StringUtil;
import org.sakaiproject.util.delivery.FocusDelivery;
import org.sakaiproject.util.delivery.RefreshDelivery;
import org.sakaiproject.util.observer.PresenceObservingCourier;

/**
* <p>ChatAction is the CHEF chat tool.</p>
* 
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.11 $
*/
public class ChatAction
	extends VelocityPortletPaneledAction
{
	/** portlet configuration parameter names. */
	private static final String PARAM_CHANNEL = "channel";
	private static final String PARAM_DISPLAY_DATE = "display-date";
	private static final String PARAM_DISPLAY_TIME = "display-time";
	private static final String PARAM_DISPLAY_USER = "display-user";
	private static final String PARAM_SOUND_ALERT = "sound-alert";
	private static final String PARAM_MEMBER_FILTER = "member-filter";
	private static final String PARAM_FILTER_TYPE = "filter-type";
	private static final String PARAM_FILTER_PARAM = "filter-param";

	/** Configure form field names. */
	private static final String FORM_CHANNEL = "channel";
	private static final String FORM_NEW_CHANNEL = "new-channel";
	private static final String FORM_FILTER_TYPE = "filter-type";
	private static final String FORM_FILTER_PARAM_DAYS = "filter-param-days";
	private static final String FORM_FILTER_PARAM_NUMBER = "filter-param-number";
	
	/** Message filter names */
	private static final String FILTER_BY_NUMBER = "SelectMessagesByNumber";
	private static final String FILTER_BY_TIME = "SelectMessagesByTime";
	private static final String FILTER_TODAY = "SelectTodaysMessages";
	private static final String FILTER_ALL = "SelectAllMessages";
	private static final String[] ALL_FILTERS =
		{ FILTER_BY_NUMBER, FILTER_BY_TIME, FILTER_TODAY, FILTER_ALL };

	/** Default values to use in case of input errors */
	private static final int DEFAULT_PARAM = 0;
	private static final int DEFAULT_DAYS = 3;
	private static final int DEFAULT_MSGS = 12;
	
	/** Control form field names. */
	private static final String FORM_MESSAGE = "message";

	/** names and values of request parameters to select sub-panels */
	private static final String MONITOR_PANEL = "List";
	private static final String CONTROL_PANEL = "Control";
	private static final String PRESENCE_PANEL = "Presence";
	private static final String TOOLBAR_PANEL = "Toolbar";

	/** state attribute names. */
	private static final String STATE_CHANNEL_REF = "channelId";
	private static final String STATE_SITE = "siteId";
	private static final String STATE_DISPLAY_DATE = "display-date";
	private static final String STATE_DISPLAY_TIME = "display-time";
	private static final String STATE_DISPLAY_USER = "display-user";
	private static final String STATE_SOUND_ALERT = "sound-alert";
	private static final String STATE_UPDATE = "update";
	private static final String STATE_CHANNEL_PROBLEM = "channel-problem";
	private static final String STATE_FILTER_TYPE = "filter-type";
	private static final String STATE_FILTER_PARAM = "filter-param";
	private static final String STATE_MESSAGE_FILTER = "message-filter";
	private static final String STATE_MORE_SELECTED = "more-selected";
	private static final String STATE_MORE_MESSAGES_LABEL = "more-messages-label";
	private static final String STATE_MORE_MESSAGES_FILTER = "more-messages-filter";
	private static final String STATE_FEWER_MESSAGES_LABEL = "fewer-messages-label";
	private static final String STATE_FEWER_MESSAGES_FILTER = "fewer-messages-filter";
	private static final String STATE_BROWSER = "browser";
	private static final String STATE_CHAT_PRESENCE_OBSERVER = STATE_OBSERVER2;
	private static final String STATE_COLOR_MAPPER = "color-mapper";
	private static final String STATE_MAIN_MESSAGE = "message-for-chat-layout";
	
	private static final String NEW_CHAT_CHANNEL = "new-chat-channel";

	/** channel problem messages */
	private static final String CHANNEL_MISSING = "This chat is not available.";
	private static final String CHANNEL_PERMISSION = "You do not have permission to view this chat.";
	private static final String CHANNEL_CREATE_PERMISSION = "You do not have permission to create this chat.";

	/** UI messages. */
	private static final String PERMISSION_POST_MESSAGE = "You do not have permission to post to this chat.";
	private static final String POST_PROBLEM_MESSAGE = "There was a problem sending the prior message.";

	/**
	* Populate the state object, if needed.
	* @param state The state object.
	* @param portlet The portlet.
	* @param rundata The current request's rundata.
	*/
	protected void initState(SessionState state, VelocityPortlet portlet, JetspeedRunData rundata)
	{
		super.initState(state, portlet, rundata);

		// detect that we have not done this, yet
		if (state.getAttribute(STATE_CHANNEL_REF) == null)
		{
			PortletConfig config = portlet.getPortletConfig();
			
			// read the channel from configuration, or, if not specified, use the default for the page
			String channel = StringUtil.trimToNull(config.getInitParameter(PARAM_CHANNEL));
			if (channel == null)
			{
				channel = ChatService.channelReference(PortalService.getCurrentSiteId(), SiteService.MAIN_CONTAINER);
			}

			state.setAttribute(STATE_CHANNEL_REF, channel);

			if (state.getAttribute(STATE_DISPLAY_DATE) == null)
			{
				state.setAttribute(STATE_DISPLAY_DATE, new Boolean(config.getInitParameter(PARAM_DISPLAY_DATE)));
			}

			if (state.getAttribute(STATE_DISPLAY_TIME) == null)
			{
				state.setAttribute(STATE_DISPLAY_TIME, new Boolean(config.getInitParameter(PARAM_DISPLAY_TIME)));
			}

			if (state.getAttribute(STATE_DISPLAY_USER) == null)
			{
				state.setAttribute(STATE_DISPLAY_USER, new Boolean(config.getInitParameter(PARAM_DISPLAY_USER)));
			}
			
			if (state.getAttribute(STATE_SOUND_ALERT) == null)
			{
				state.setAttribute(STATE_SOUND_ALERT, new Boolean(config.getInitParameter(PARAM_SOUND_ALERT)));
			}
			
			if (state.getAttribute(STATE_COLOR_MAPPER) == null)
			{
				ColorMapper mapper = new ColorMapper();

				// always set this user's color to first color (red)
				mapper.getColor(UsageSessionService.getSessionUser().getId());
				
				state.setAttribute(STATE_COLOR_MAPPER, mapper);
			}

			if(state.getAttribute(STATE_FILTER_TYPE) == null)
			{
				String filter_type = config.getInitParameter(PARAM_FILTER_TYPE, FILTER_BY_TIME);
				String filter_param = config.getInitParameter(PARAM_FILTER_PARAM, String.valueOf(DEFAULT_DAYS));
				
				updateMessageFilters(state, filter_type, filter_param);
			}

			// the event resource reference pattern to watch for
			// setup the observer to notify our MONITOR_PANEL panel (inside the Main panel)
			if (state.getAttribute(STATE_OBSERVER) == null)
			{
				// the delivery location for this tool
				String deliveryId = clientWindowId(state, portlet.getID());
				
				// the html element to update on delivery
				String elementId = mainPanelUpdateId(portlet.getID()) + "." + MONITOR_PANEL;
				Reference r = new Reference(channel);
				String pattern = ChatService.messageReference(r.getContext(), r.getId(), "");
				boolean wantsBeeps = ((Boolean) state.getAttribute(STATE_SOUND_ALERT)).booleanValue();

				state.setAttribute(STATE_OBSERVER, new ChatObservingCourier(deliveryId, elementId, pattern, wantsBeeps));
			}

			// the event resource reference pattern to watch for
			// setup the observer to notify our PRESENCE_PANEL panel (inside the Main panel)
			if (state.getAttribute(STATE_CHAT_PRESENCE_OBSERVER) == null)
			{
				// this is where deliveries go (if tool id is part of the request, it will be part of the delivery id)
				String deliveryId = PortalService.getCurrentClientWindowId();
				
				// the html element to update on delivery
				String elementId = mainPanelUpdateId(portlet.getID()) + "." + PRESENCE_PANEL;

				// location is current portal
				String location = PortalService.getCurrentLocationId(portlet.getID());
				
				// setup an observer to notify us
				PresenceObservingCourier observer2 = new PresenceObservingCourier(deliveryId, elementId, location, true);

				// PresenceObservingCourier.FRAME_PRESENCE, instead of elementId???
				state.setAttribute(STATE_CHAT_PRESENCE_OBSERVER, observer2);

				if (Log.getLogger("chef").isDebugEnabled())
					Log.debug("chef", this + ": observer: " + observer2 + " elementId: " + elementId
								+ " deliveryAddress: " + deliveryId
								+ " location: " + location);
			}
		}

		// make sure the observer is in sync with state
		updateObservationOfChannel(state, portlet.getID());

	}   // initState

	/**
	* Setup our observer to be watching for change events for our channel.
	* @param state The state object.
	* @param peid The portlet id.
	*/
	private void updateObservationOfChannel(SessionState state, String peid)
	{
		// make sure the pattern matches the channel we are looking at
		String channel = (String) state.getAttribute(STATE_CHANNEL_REF);
		Reference r = new Reference(channel);
		String pattern = ChatService.messageReference(r.getContext(), r.getId(), "");

		// the delivery location for this tool
		String deliveryId = clientWindowId(state, peid);
		
		// update the observer looking for new messages
		ChatObservingCourier observer1 = (ChatObservingCourier) state.getAttribute(STATE_OBSERVER);
		observer1.setResourcePattern(pattern);
		observer1.setDeliveryId(deliveryId);
		
		// update the observer looking for changes in chat presence
		PresenceObservingCourier observer2 = (PresenceObservingCourier) state.getAttribute(STATE_CHAT_PRESENCE_OBSERVER);
		observer2.setDeliveryId(deliveryId);
		
	}   // updateObservationOfChannel
	
	/**
	* Get the channel from ChatService or create it.
	*/
	private ChatChannel getChannel(SessionState state, String name)
	{
		// deal with the channel not yet existing
		ChatChannel channel = null;
		try
		{
			channel = ChatService.getChatChannel(name);
		}
		catch (IdUnusedException ignore) {}
		catch (PermissionException ignore) {}

		if (	(channel == null)
			&&	(state.getAttribute(STATE_CHANNEL_PROBLEM) == null))
		{
			// create the channel
			try
			{
				ChatChannelEdit edit = ChatService.addChatChannel(name);
				ChatService.commitChannel(edit);
				channel = edit;
			}
			catch (IdUsedException e)
			{
				// strange, the channel already exists!
				try
				{
					channel = ChatService.getChatChannel(name);
				}
				catch (IdUnusedException ignore) {}
				catch (PermissionException ignore) {}
			}
			catch (IdInvalidException e)
			{
				// stranger, we cannot use this id!
				state.setAttribute(STATE_CHANNEL_PROBLEM, CHANNEL_MISSING);
				Log.warn("chef", this + ".doSend(): creating channel: ", e);
			}
			catch (PermissionException e)
			{
				// rats, this user cannot create the channel
				state.setAttribute(STATE_CHANNEL_PROBLEM, CHANNEL_CREATE_PERMISSION);
			}
		}
		return channel;

	}	// getChannel

	/**
	* Update the state message-filtering attributes to use the filtering criteria 
	* specified by the filter_type and filter_param parameters.
	* @param state The session state.
	* @param filter_type A string specifying the filter type.
	* @param filter_param A string specifying the filter param.
	*/
	private void updateMessageFilters(SessionState state, String filter_type, String filter_param)
	{
		state.setAttribute(STATE_MORE_MESSAGES_LABEL, "Show All Messages");
		state.setAttribute(STATE_MORE_MESSAGES_FILTER, new SelectAllMessages());
		state.setAttribute(STATE_FILTER_PARAM, String.valueOf(DEFAULT_PARAM));
		
		state.setAttribute(STATE_MORE_SELECTED, new Boolean(false));
		state.setAttribute(STATE_FILTER_TYPE, filter_type);
		if(filter_type.equals(FILTER_ALL))
		{
			state.setAttribute(STATE_FEWER_MESSAGES_FILTER, new SelectMessagesByTime(DEFAULT_DAYS));
			state.setAttribute(STATE_FEWER_MESSAGES_LABEL, "Show Past " + DEFAULT_DAYS + " Days");
			state.setAttribute(STATE_MESSAGE_FILTER, state.getAttribute(STATE_MORE_MESSAGES_FILTER));
			state.setAttribute(STATE_MORE_SELECTED, new Boolean(true));
		}
		else if(filter_type.equals(FILTER_TODAY))
		{
			state.setAttribute(STATE_FEWER_MESSAGES_FILTER, new SelectTodaysMessages());
			state.setAttribute(STATE_FEWER_MESSAGES_LABEL, "Show Today's Messages");
			state.setAttribute(STATE_MESSAGE_FILTER, state.getAttribute(STATE_FEWER_MESSAGES_FILTER));
		}
		else if(filter_type.equals(FILTER_BY_NUMBER))
		{
			int number = DEFAULT_MSGS;
			try
			{
				number = Integer.parseInt(filter_param);
				if(number <= 0) 
				{
					throw new Exception();
				}
				state.setAttribute(STATE_FEWER_MESSAGES_FILTER, new SelectMessagesByNumber(number, state));
				state.setAttribute(STATE_FILTER_PARAM, filter_param);
			}
			catch(Exception e)
			{
				Log.warn("chef", this + ".updateMessageFilters() invalid param: " );
			}
			state.setAttribute(STATE_FILTER_PARAM, String.valueOf(number));
			state.setAttribute(STATE_FEWER_MESSAGES_FILTER, new SelectMessagesByNumber(number, state));
			state.setAttribute(STATE_FEWER_MESSAGES_LABEL, "Show Last " + number + " Messages");
			state.setAttribute(STATE_MESSAGE_FILTER, state.getAttribute(STATE_FEWER_MESSAGES_FILTER));
		}
		else if(filter_type.equals(FILTER_BY_TIME))
		{
			int number = DEFAULT_DAYS;
			try
			{
				number = Integer.parseInt(filter_param);
				if(number <= 0) 
				{
					throw new Exception();
				}
			}
			catch(Exception e)
			{
				Log.warn("chef", this + ".updateMessageFilters() invalid param: " );
			}
			state.setAttribute(STATE_FILTER_PARAM, String.valueOf(number));
			state.setAttribute(STATE_FEWER_MESSAGES_FILTER, new SelectMessagesByTime(number));
			state.setAttribute(STATE_FEWER_MESSAGES_LABEL, "Show Past " + number + " Days");
			state.setAttribute(STATE_MESSAGE_FILTER, state.getAttribute(STATE_FEWER_MESSAGES_FILTER));
		}
		else
		{
			state.setAttribute(STATE_FILTER_PARAM, String.valueOf(DEFAULT_DAYS));
			state.setAttribute(STATE_FEWER_MESSAGES_FILTER, new SelectMessagesByTime(DEFAULT_DAYS));
			state.setAttribute(STATE_FEWER_MESSAGES_LABEL, "Show Past " + DEFAULT_DAYS + " Days");
			state.setAttribute(STATE_MESSAGE_FILTER, state.getAttribute(STATE_FEWER_MESSAGES_FILTER));
		}

	}	// updateMessageFilters

	/** 
	* build the context for the Main (Layout) panel
	* @return (optional) template name for this panel
	*/
	public String buildMainPanelContext(VelocityPortlet portlet, 
										Context context,
										RunData rundata,
										SessionState state)
	{
		// if there's an alert message specifically targetted for main fraim, display it
		String msg = (String) state.getAttribute(STATE_MAIN_MESSAGE);
		context.put("alertMessage", msg);
		state.removeAttribute(STATE_MAIN_MESSAGE);
		
		// if we are in edit permissions...
		String helperMode = (String) state.getAttribute(PermissionsAction.STATE_MODE);
		if (helperMode != null)
		{
			String template = PermissionsAction.buildHelperContext(portlet, context, rundata, state);
			if (template == null)
			{
				addAlert(state, "There is a problem setting Permissions at this time.");
			}
			else
			{
				return template;
			}
		}

		String mode = (String) state.getAttribute(STATE_MODE);
		if (MODE_OPTIONS.equals(mode))
		{
			return buildOptionsPanelContext(portlet, context, rundata, state);
		}
		
		List focus_elements = new Vector();
		focus_elements.add(CONTROL_PANEL);
		focus_elements.add(FORM_MESSAGE);
		
		context.put("focus_path", focus_elements);

		context.put("panel-control", CONTROL_PANEL);
		context.put("panel-monitor", MONITOR_PANEL);
		context.put("panel-presence", PRESENCE_PANEL);
		context.put("panel-toolbar", TOOLBAR_PANEL);

		return (String)getContext(rundata).get("template") + "-Layout";

	}   // buildLayoutPanelContext

	/** 
	* Handle a user clicking on the view-date menu.
	**/	
	public void doToggle_date_display(RunData runData, Context context)
	{
		toggleState(runData, STATE_DISPLAY_DATE);

		// schedule a refresh of the "List" panel
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState state = ((JetspeedRunData)runData).getPortletSessionState(peid);
		String address = ((ChatObservingCourier) state.getAttribute(STATE_OBSERVER)).getDeliveryId();
		String elementID = ((ChatObservingCourier) state.getAttribute(STATE_OBSERVER)).getElementId();
		CourierService.deliver(address, elementID);

		// schedule a delivery to return focus to Control panel
		String mainPanelId = mainPanelUpdateId(peid);
		String[] focusPath = {mainPanelId, CONTROL_PANEL, FORM_MESSAGE};
		CourierService.deliver(new FocusDelivery(address, focusPath));

	}	// doToggle_date_display

	/** 
	* Handle a user clicking on the view-time menu.
	**/	
	public void doToggle_time_display(RunData runData, Context context)
	{
		toggleState(runData, STATE_DISPLAY_TIME);

		// schedule a refresh of the "List" panel
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState state = ((JetspeedRunData)runData).getPortletSessionState(peid);
		String address = ((ChatObservingCourier) state.getAttribute(STATE_OBSERVER)).getDeliveryId();
		String elementID = ((ChatObservingCourier) state.getAttribute(STATE_OBSERVER)).getElementId();
		CourierService.deliver(new RefreshDelivery(address, elementID));

		// schedule a delivery to return focus to Control panel
		String mainPanelId = mainPanelUpdateId(peid);
		String[] focusPath = {mainPanelId, CONTROL_PANEL, FORM_MESSAGE};
		CourierService.deliver(new FocusDelivery(address, focusPath));

	}	// doToggle_time_display

	/** 
	* Handle a user clicking on the view-user menu.
	**/	
	public void doToggle_user_display(RunData runData, Context context)
	{
		toggleState(runData, STATE_DISPLAY_USER);

	}	// doToggle_user_display

	/** 
	* Handle a user clicking on the sound-alert button.
	**/	
	public void doToggle_sound_alert(RunData runData, Context context)
	{
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState state = ((JetspeedRunData)runData).getPortletSessionState(peid);
		
		// toggle the state setting
		boolean newValue = !((Boolean) state.getAttribute(STATE_SOUND_ALERT)).booleanValue();
		state.setAttribute(STATE_SOUND_ALERT, new Boolean(newValue));
		ChatObservingCourier observer = (ChatObservingCourier) state.getAttribute(STATE_OBSERVER);
		observer.alertEnabled(newValue);

		// schedule a delivery to return focus to Control panel
		String address = ((ChatObservingCourier) state.getAttribute(STATE_OBSERVER)).getDeliveryId();
		String mainPanelId = mainPanelUpdateId(peid);
		String[] focusPath = {mainPanelId, CONTROL_PANEL, FORM_MESSAGE};
		CourierService.deliver(new FocusDelivery(address, focusPath));

	}	// doToggle_sound_alert

	/** 
	* Toggle the state attribute
	* @param runData
	* @param stateName The name of the state attribute to toggle
	**/	
	private void toggleState(RunData runData, String stateName)
	{
		// access the portlet element id to find our state
		// %%% use CHEF api instead of Jetspeed to get state
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState state = ((JetspeedRunData)runData).getPortletSessionState(peid);
		
		// toggle the state setting
		boolean newValue = !((Boolean) state.getAttribute(stateName)).booleanValue();
		state.setAttribute(stateName, new Boolean(newValue));

	}	// toggleState

	/** 
	* Handle a user clicking on the show-all/show-today button.
	**/	
	public void doToggle_filter(RunData runData, Context context)
	{
		// access the portlet element id to find our state
		// %%% use CHEF api instead of Jetspeed to get state
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState state = ((JetspeedRunData)runData).getPortletSessionState(peid);
		
		// toggle the filter setting between show-more and show-fewer
		if(((Boolean)state.getAttribute(STATE_MORE_SELECTED)).booleanValue())
		{
			state.setAttribute(STATE_MESSAGE_FILTER, state.getAttribute(STATE_FEWER_MESSAGES_FILTER));
		}
		else
		{
			state.setAttribute(STATE_MESSAGE_FILTER, state.getAttribute(STATE_MORE_MESSAGES_FILTER));
		}
		toggleState(runData, STATE_MORE_SELECTED);

		// %%%% schedule a refresh of the "List" panel
		String address = ((ChatObservingCourier) state.getAttribute(STATE_OBSERVER)).getDeliveryId();
		String elementID = ((ChatObservingCourier) state.getAttribute(STATE_OBSERVER)).getElementId();
		CourierService.deliver(address, elementID);

		// schedule a delivery to return focus to Control panel
		String mainPanelId = mainPanelUpdateId(peid);
		String[] focusPath = {mainPanelId, CONTROL_PANEL, FORM_MESSAGE};
		CourierService.deliver(new FocusDelivery(address, focusPath));

	}	// doToggle_filter

	/** 
	* build the context for the Toolbar panel
	* @return (optional) template name for this panel
	*/
	public String buildToolbarPanelContext(VelocityPortlet portlet, 
										Context context,
										RunData rundata,
										SessionState state)
	{
		// build the menu
		Menu bar = new Menu(portlet, rundata, (String) state.getAttribute(STATE_ACTION));

		boolean displayDate = ((Boolean)state.getAttribute(STATE_DISPLAY_DATE)).booleanValue();
		bar.add( new MenuEntry((displayDate ? "Hide Date" : "Show Date"), null, true,
				(displayDate ? MenuItem.CHECKED_TRUE : MenuItem.CHECKED_FALSE),
				"doToggle_date_display") );
		boolean displayTime = ((Boolean)state.getAttribute(STATE_DISPLAY_TIME)).booleanValue();
		bar.add( new MenuEntry((displayTime ? "Hide Time" : "Show Time"), null, true,
				(displayTime ? MenuItem.CHECKED_TRUE : MenuItem.CHECKED_FALSE),
				"doToggle_time_display") );

		// if the java beep is disabled, don't offer the alert
		if (ServerConfigurationService.getBoolean("java.beep", false))
		{
			boolean soundAlert = ((Boolean)state.getAttribute(STATE_SOUND_ALERT)).booleanValue();
			bar.add( new MenuEntry((soundAlert ? "Turn OFF New Message Alert" : "Turn ON New Message Alert"), null, true,
					(soundAlert ? MenuItem.CHECKED_TRUE : MenuItem.CHECKED_FALSE),
					"doToggle_sound_alert") );
		}

		boolean moreSelected = ((Boolean)state.getAttribute(STATE_MORE_SELECTED)).booleanValue();
		bar.add( new MenuEntry((moreSelected ? (String)state.getAttribute(STATE_FEWER_MESSAGES_LABEL) : (String)state.getAttribute(STATE_MORE_MESSAGES_LABEL)), null, true,
				(moreSelected ? MenuItem.CHECKED_TRUE : MenuItem.CHECKED_FALSE),
				"doToggle_filter") );

		// add options if allowed
		addOptionsMenu(bar, (JetspeedRunData) rundata);

		// add permissions, if allowed
		if (SiteService.allowUpdateSite(PortalService.getCurrentSiteId()))
		{
			bar.add( new MenuEntry("Permissions...", "doPermissions") );
		}

		context.put(Menu.CONTEXT_MENU, bar);
		context.put(Menu.CONTEXT_ACTION, state.getAttribute(STATE_ACTION));
		
		return null; // (String)getContext(rundata).get("template") + "-Toolbar";

	}   // buildToolbarPanelContext
	
	/** 
	* build the context for the List panel
	* @return (optional) template name for this panel
	*/
	public String buildListPanelContext(VelocityPortlet portlet, 
										Context context,
										RunData rundata,
										SessionState state)
	{
		// display info
		context.put("display_date", state.getAttribute(STATE_DISPLAY_DATE));
		context.put("display_time", state.getAttribute(STATE_DISPLAY_TIME));
		context.put("display_user", state.getAttribute(STATE_DISPLAY_USER));
		context.put("sound_alert", state.getAttribute(STATE_SOUND_ALERT));
		
		// provide a color mapper to keep track of user color coding for the session
		context.put("color_mapper", (ColorMapper)state.getAttribute(STATE_COLOR_MAPPER));

		// find the channel and get the messages
		try
		{
			ChatChannel channel = ChatService.getChatChannel((String) state.getAttribute(STATE_CHANNEL_REF));

			// provide "chat_messages" as an iterator on the messages
			Filter filter = (Filter)state.getAttribute(STATE_MESSAGE_FILTER);
			context.put("chat_messages", channel.getMessages(filter, true));
			// true means we always display messages in ascending order.  

		}
		catch (PermissionException e)
		{
			context.put("alertMessage", CHANNEL_PERMISSION);
		}
		catch (IdUnusedException e) {}
		catch (Exception e)
		{
			Log.warn("chef", this + ".buildListPanelContext()", e);
		}

		// inform the observing courier that we just updated the page...
		// if there are pending requests to do so they can be cleared
		((ChatObservingCourier) state.getAttribute(STATE_OBSERVER)).justDelivered();

		return null;

	}   // buildListPanelContext
	
	/** 
	* build the context for the Control panel
	* (has a send field)
	* @return (optional) template name for this panel
	*/
	public String buildControlPanelContext(VelocityPortlet portlet, 
										Context context,
										RunData rundata,
										SessionState state)
	{
		// put this pannel's name for the return url
		context.put("panel-control", CONTROL_PANEL);

		// set the action for form processing
		context.put(Menu.CONTEXT_ACTION, state.getAttribute(STATE_ACTION));
		
		// set the form field name for the send button
		context.put("form-submit", BUTTON + "doSend");

		// set the form field name for the send button
		context.put("form-message", FORM_MESSAGE);

		// is this user going to be able to post?
		boolean allowed = false;

		// if channel exists, we can check the user's allow addChatMessage(channel)
		ChatChannel channel = null;
		try
		{
			channel = ChatService.getChatChannel(
					(String)state.getAttribute(STATE_CHANNEL_REF));
		}
		catch (IdUnusedException ignore) {}
		catch (PermissionException ignore) {}

		if (channel != null)
		{
			allowed = channel.allowAddMessage();
		}
		// if no channel, we can check the user's allow addChatChannel(channelId)
		else
		{
			allowed = ChatService.allowAddChannel((String)state.getAttribute(STATE_CHANNEL_REF));
		}
		Boolean allow = new Boolean(allowed);
		if (!allowed)
		{
			context.put("alertMessage", PERMISSION_POST_MESSAGE);	// %%% or no message?
		}
		context.put("allow-send", allow);

		return null;

	}   // buildControlPanelContext

	/** 
	* build the context for the Presence panel
	* (has a send field)
	* @return (optional) template name for this panel
	*/
	public String buildPresencePanelContext(VelocityPortlet portlet, 
										Context context,
										RunData rundata,
										SessionState state)
	{
		String template = null;
		
		// get the observer
		PresenceObservingCourier observer = (PresenceObservingCourier) state.getAttribute(STATE_CHAT_PRESENCE_OBSERVER);

		// put into context a list of sessions with chat presence
		String location = PortalService.getCurrentLocationId(portlet.getID());

		// get the current presence list (User objects) for this page
		List sessions = PresenceService.getPresence(location);
		context.put("sessions", sessions);

		// inform the observing courier that we just updated the page...
		// if there are pending requests to do so they can be cleared
		observer.justDelivered();

		return null;
		
	}	// buildPresencePanelContext
	
	/** 
	* Handle a user posting a new chat message.
	**/	
	public void doSend(RunData runData, Context context)
	{
		// access the portlet element id to find our state
		// %%% use CHEF api instead of Jetspeed to get state
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState state = ((JetspeedRunData)runData).getPortletSessionState(peid);

		// read in the message input
		String message = runData.getParameters().getCleanString(FORM_MESSAGE);

		// ignore empty messages
		if ((message == null) || (message.length() == 0)) return;

		// deal with the channel not yet existing
		ChatChannel channel = getChannel(state, (String) state.getAttribute(STATE_CHANNEL_REF));

		// post the message
		if (channel != null)
		{
			try
			{
				ChatMessageEdit edit = channel.addChatMessage();
				edit.setBody(message);
				channel.commitMessage(edit);
			}
			catch (PermissionException e)
			{
				addAlert(state, PERMISSION_POST_MESSAGE);
			}
			catch (Exception e)	// %%% why?
			{
				addAlert(state, POST_PROBLEM_MESSAGE);
				Log.warn("chef", this + ".doSend()", e);
			}
		}
		else
		{
			addAlert(state, (String) state.getAttribute(STATE_CHANNEL_PROBLEM));
		}

		// schedule a delivery to return focus to Control panel
		String address = ((ChatObservingCourier) state.getAttribute(STATE_OBSERVER)).getDeliveryId();
		String mainPanelId = mainPanelUpdateId(peid);
		String[] focusPath = {mainPanelId, CONTROL_PANEL, FORM_MESSAGE};
		CourierService.deliver(new FocusDelivery(address, focusPath));

	}	// doSend

	/** 
	* Setup for the options panel.
	*/
	public String buildOptionsPanelContext( VelocityPortlet portlet, 
											Context context,
											RunData rundata,
											SessionState state)
	{
		// provide "filter_type" with the current default value for filtering messages
		context.put("filter_type", (String) state.getAttribute(STATE_FILTER_TYPE));
		
		// provide "filter_type_form" with form field name for selecting a message filter
		context.put("filter_type_form", FORM_FILTER_TYPE);
		
		// provide "filter_days_param" as current value or default value for number of days
		context.put("filter_days_param", (String) state.getAttribute(STATE_FILTER_PARAM));
		
		// provide "filter_days_param_form" with form field name for filter parameter (number of days/messages)
		context.put("filter_days_param_form", FORM_FILTER_PARAM_DAYS);

		// provide "filter_param" as current value or default value for number of days/messages
		context.put("filter_number_param", (String) state.getAttribute(STATE_FILTER_PARAM));
		
		// provide "filter_param_form" with form field name for filter parameter (number of days/messages)
		context.put("filter_number_param_form", FORM_FILTER_PARAM_NUMBER);
		
		// provide "default_chat_channel" with the dafault channel-id for the user/group
		context.put("default_chat_channel", SiteService.MAIN_CONTAINER);
		
		// provide "chat_channel" with the current channel's id
		String siteID = PortalService.getCurrentSiteId();
		String defaultChannel = ChatService.channelReference(siteID, SiteService.MAIN_CONTAINER);
		String sitePrefix = defaultChannel.substring(0, defaultChannel.lastIndexOf(SiteService.MAIN_CONTAINER));
		String currentChannel = ((String) state.getAttribute(STATE_CHANNEL_REF)).substring(sitePrefix.length());
		context.put("chat_channel", currentChannel);

		// provide "chat_channels" as  a list of channels belonging to this site
		Iterator aChannel = ChatService.getChannels().iterator();
		List channel_list = new Vector();
		while(aChannel.hasNext())
		{
			ChatChannel theChannel = (ChatChannel) aChannel.next();
			if(theChannel.getContext().equals(siteID) &&
				!theChannel.getId().equals(SiteService.MAIN_CONTAINER) &&
				!theChannel.getId().equals(currentChannel))
			{
				channel_list.add(theChannel.getId());
			}
		}
		context.put("chat_channels", channel_list);
		
		// provide "new_chat_channel" as flag to create a new channel
		context.put("new_chat_channel", NEW_CHAT_CHANNEL);

		// provide "form_new_channel" with form field name for specifying a new channel name
		context.put("form_new_channel", FORM_NEW_CHANNEL);

		// provide "chat_channel_form" with the form name for the new channel selection field
		context.put("chat_channel_form", FORM_CHANNEL);

		// set the action for form processing
		context.put(Menu.CONTEXT_ACTION, state.getAttribute(STATE_ACTION));
		context.put("form-submit", BUTTON + "doUpdate");
		context.put("form-cancel", BUTTON + "doCancel");

		// pick the "-customize" template based on the standard template name
		String template = (String)getContext(rundata).get("template");
		return template + "-customize";

	}   // buildOptionsPanelContext

	/**
	* Handle a user clicking the "Done" button in the Options panel
	*/
	public void doUpdate(RunData data, Context context)
	{
		// access the portlet element id to find our state
		// %%% use CHEF api instead of Jetspeed to get state
		String peid = ((JetspeedRunData)data).getJs_peid();
		SessionState state = ((JetspeedRunData)data).getPortletSessionState(peid);
		
		String siteID = PortalService.getCurrentSiteId();
		String newChannel = data.getParameters().getString(FORM_CHANNEL);
		String currentChannel = ((String) state.getAttribute(STATE_CHANNEL_REF)).substring(siteID.length() + 1);

		if (newChannel != null && newChannel.equals(NEW_CHAT_CHANNEL))
		{
			newChannel = data.getParameters().getString(FORM_NEW_CHANNEL);
			try
			{
				// make sure channel name is valid Resource ID (for items entered by user)
				Validator.checkResourceId(newChannel);
			}
			catch(IdInvalidException e)
			{
				// if name is not valid, save error message and return to Options panel
				addAlert(state, "You entered \"" + newChannel + "\" for Chat room name. Chat room name" + e.getMessage());
				return;
			}
		}
 		if (newChannel != null && !newChannel.equals(currentChannel))
		{
			state.setAttribute(STATE_CHANNEL_REF, ChatService.channelReference(siteID, newChannel));
			if (Log.getLogger("chef").isDebugEnabled())
				Log.debug("chef", this + ".doUpdate(): newChannel: " + newChannel);
			// ChatChannel chan = getChannel(state, newChannel);
			updateObservationOfChannel(state, peid);

			// update the tool config
			ToolConfigurationEdit tool = (ToolConfigurationEdit) state.getAttribute(STATE_TOOL_EDIT);
			tool.getPropertiesEdit().addProperty(PARAM_CHANNEL, (String)state.getAttribute(STATE_CHANNEL_REF));
			tool.setTitle("Chat Room: \"" + newChannel + "\"");

			// deliver an update to the title panel (to show the new channel name)
			String deliveryId = PortalService.getCurrentClientWindowId(null);
			String titleId = titlePanelUpdateId(peid);
			CourierService.deliver(deliveryId, titleId);
		}

		// filter
		String filter_type = data.getParameters().getString(FORM_FILTER_TYPE);
		if ( filter_type != null )
		{
			if( filter_type.equals(FILTER_ALL))
			{
				if(!filter_type.equals((String) state.getAttribute(STATE_FILTER_TYPE)))
				{
					updateMessageFilters(state, filter_type, null);

					// update the tool config
					ToolConfigurationEdit tool = (ToolConfigurationEdit) state.getAttribute(STATE_TOOL_EDIT);
					tool.getPropertiesEdit().addProperty(PARAM_FILTER_TYPE, (String)state.getAttribute(STATE_FILTER_TYPE));
					tool.getPropertiesEdit().addProperty(PARAM_FILTER_PARAM, (String)state.getAttribute(STATE_FILTER_PARAM));
				}
			}
			else if( filter_type.equals(FILTER_BY_TIME))
			{
				String filter_days_param = data.getParameters().getString(FORM_FILTER_PARAM_DAYS);
				if(filter_days_param != null)
				{
					if(!filter_type.equals((String) state.getAttribute(STATE_FILTER_TYPE)) 
						|| !filter_days_param.equals((String) state.getAttribute(STATE_FILTER_PARAM)))
					{
						updateMessageFilters(state, filter_type, filter_days_param);

						// update the tool config
						ToolConfigurationEdit tool = (ToolConfigurationEdit) state.getAttribute(STATE_TOOL_EDIT);
						tool.getPropertiesEdit().addProperty(PARAM_FILTER_TYPE, (String)state.getAttribute(STATE_FILTER_TYPE));
						tool.getPropertiesEdit().addProperty(PARAM_FILTER_PARAM, (String)state.getAttribute(STATE_FILTER_PARAM));
					}
				}
			}
			else if( filter_type.equals(FILTER_BY_NUMBER))
			{
				String filter_number_param = data.getParameters().getString(FORM_FILTER_PARAM_NUMBER);
				if(filter_number_param != null)
				{
					if(!filter_type.equals((String) state.getAttribute(STATE_FILTER_TYPE)) 
						|| !filter_number_param.equals((String) state.getAttribute(STATE_FILTER_PARAM)))
					{
						updateMessageFilters(state, filter_type, filter_number_param);

						// update the tool config
						ToolConfigurationEdit tool = (ToolConfigurationEdit) state.getAttribute(STATE_TOOL_EDIT);
						tool.getPropertiesEdit().addProperty(PARAM_FILTER_TYPE, (String)state.getAttribute(STATE_FILTER_TYPE));
						tool.getPropertiesEdit().addProperty(PARAM_FILTER_PARAM, (String)state.getAttribute(STATE_FILTER_PARAM));
					}
				}
			}
		}
		
		// we are done with customization... back to the main mode
		state.removeAttribute(STATE_MODE);

		// re-enable auto-updates when leaving options
		enableObservers(state);

		// commit the change
		saveOptions();

	}   // doUpdate

	/**
	* Handle a user clicking the "Done" button in the Options panel
	*/
	public void doCancel(RunData data, Context context)
	{
		// access the portlet element id to find our state
		// %%% use CHEF api instead of Jetspeed to get state
		String peid = ((JetspeedRunData)data).getJs_peid();
		SessionState state = ((JetspeedRunData)data).getPortletSessionState(peid);
		
		// we are done with customization... back to the main mode
		state.removeAttribute(STATE_MODE);

		// re-enable auto-updates when leaving options
		enableObservers(state);

		// cancel the options
		cancelOptions();

	}   // doCancel
	
	/**
	* Say if the tool will require the "reset tool" button on the title bar.
	* @return true to have the "reset tool" button, false if not.
	* The default is to have it.
	*/
	protected boolean requireResetToolLink()
	{
		return false;

	}	// requireResetToolLink

	/**  A filter */
	private class SelectMessagesByTime
		 implements Filter
	{
		/**  The number of days back to accept messages. */
		private int m_days = 0;

		/**  The cutoff time - messages before this are rejected. */
		private Time m_cutoff = null;

		/**
		 *  Constructor
		 *
		 *@param	days	The number of days back to accept messages.
		 */
		public SelectMessagesByTime(int days)
		{
			// Log.info("chef", this + ".SelectMessagesByTime(" + days + ")");
			m_days = days;

			// compute the cutoff - Note: use the filter fast - the clock is ticking.
			m_cutoff = TimeService.newTime(System.currentTimeMillis() - ((long)days * 24l * 60l * 60l * 1000l));
			
		}	// SelectMessagesByTime


		/**
		 *  Does this object satisfy the criteria of the filter?
		 *
		 *@param	o	The object to test.
		 *@return		true if the object is accepted by the filter, false if not.
		 */
		public boolean accept(Object o)
		{
			// we want to test only messages
			if(!(o instanceof ChatMessage))
			{
				return false;
			}

			// if the item is older than m_days days ago, reject it.
			if(((ChatMessage)o).getHeader().getDate().before(m_cutoff))
			{
				return false;
			}

			// accept this one
			return true;
			
		}		// accept


		public String toString()
		{
			return this.getClass().getName() + " " + Integer.toString(m_days);
		}

	}		// SelectMessagesByTime


	/**  A filter */
	private class SelectMessagesByNumber
		 implements Filter
	{
		/**  The cutoff value - messages before this date/time are rejected. */
		private Time m_first;
		
		/** the number of messages to select */
		private int m_number;

		/**
		 *  Constructor
		 *
		 *@param	number	The number of the messages to be returned
		 *@param	state	The state of the current session.
		 */
		public SelectMessagesByNumber(int number, SessionState state)
		{
			// Log.info("chef", this + ".SelectMessagesByNumber(" + number + ")");
			
			m_number = number;

			try
			{
				// compute the cutoff
				ChatChannel ch = ChatService.getChatChannel((String)state.getAttribute(STATE_CHANNEL_REF));
				List list = (List)ch.getMessages(null, false);
				if(list.size() < number) 
				{
					// use all messages
					m_first = ((ChatMessage) list.get(list.size() - 1)).getHeader().getDate();
				}
				else
				{
					m_first = ((ChatMessage) list.get(number - 1)).getHeader().getDate();
				}
			}
			catch(Exception e)
			{
				// Channel is not initialized, so use current time as cut-off
				m_first = TimeService.newTime(System.currentTimeMillis());
			}
			
		}	// SelectMessagesByNumber

		/**
		 *  Does this object satisfy the criteria of the filter?
		 *
		 *@param	o	The object to test.
		 *@return		true if the object is accepted by the filter, false if not.
		 */
		public boolean accept(Object o)
		{
			// we want to test only messages
			if(!(o instanceof ChatMessage))
			{
				return false;
			}

			// if the item has a lower ID than m_cutoff, reject it.
			if(((ChatMessage)o).getHeader().getDate().compareTo(m_first) < 0)
			{
				return false;
			}

			// accept this one
			return true;
			
		}		// accept


		public String toString()
		{
			return this.getClass().getName() + " " + m_number;
			
		}	// toString

	}	// SelectMessagesByNumber

	/**  A filter */
	private class SelectAllMessages
		 implements Filter
	{
		/**  Constructor for the SelectAllMessages object */
		public SelectAllMessages()
		{
			super();
			// Log.info("chef", this + ".SelectAllMessages()");
			
		}	// SelectAllMessages

		/**
		 *  Does this object satisfy the criteria of the filter?
		 *
		 *@param	o	The object to test.
		 *@return		true if the object is accepted by the filter, false if not.
		 */
		public boolean accept(Object o)
		{
			// we want to test only messages
			if(!(o instanceof ChatMessage))
			{
				return false;
			}

			// accept this one
			return true;
			
		}	// accept


		public String toString()
		{
			return this.getClass().getName();
			
		}	// toString

	}	// SelectAllMessages

	/**  A filter that gets all messages since midnight today, local time */
	private class SelectTodaysMessages
		 implements Filter
	{
		/**  The cutoff time - messages before this are rejected. */
		private Time m_cutoff = null;

		/**  Constructor for the SelectTodaysMessages object */
		public SelectTodaysMessages()
		{
			super();
			// Log.info("chef", this + ".SelectTodaysMessages()");
			
			TimeBreakdown now = (TimeService.newTime(System.currentTimeMillis())).breakdownLocal();
			// compute the cutoff for midnight today.
			m_cutoff = TimeService.newTimeLocal(now.getYear(), now.getMonth(),now.getDay(), 0, 0, 0, 0);
			
		}	// SelectTodaysMessages

		/**
		 *  Does this object satisfy the criteria of the filter?
		 *
		 *@param	o	The object to test.
		 *@return		true if the object is accepted by the filter, false if not.
		 */
		public boolean accept(Object o)
		{
			// we want to test only messages
			if(!(o instanceof ChatMessage))
			{
				return false;
			}

			// if the item is older today, reject it.
			if(((ChatMessage)o).getHeader().getDate().before(m_cutoff))
			{
				return false;
			}

			// accept this one
			return true;
			
		}	// accept


		public String toString()
		{
			return this.getClass().getName();
			
		}	// toString

	}	// SelectTodaysMessages
	
	
	/** 
	* Handle a request to set options.
	*/	
	public void doOptions(RunData runData, Context context)
	{
		super.doOptions(runData, context);
		
		// access the portlet element id to find our state
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState state = ((JetspeedRunData)runData).getPortletSessionState(peid);
		
		// if there's an alert message, divert it to the main frame
		String msg = (String)state.getAttribute(STATE_MESSAGE);
		state.setAttribute(STATE_MAIN_MESSAGE, msg);
		state.removeAttribute(VelocityPortletPaneledAction.STATE_MESSAGE);
		
		// schedule a main refresh
		String toolId = PortalService.getCurrentToolId();
		String address = clientWindowId(state, toolId);
		String mainPanelId = mainPanelUpdateId(toolId);
		CourierService.deliver(address, mainPanelId);
		
	}	// doOptions



	/**
	* Fire up the permissions editor
	*/
	public void doPermissions(RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState(((JetspeedRunData)data).getJs_peid());

		String channelRefStr = (String) state.getAttribute(STATE_CHANNEL_REF);
		Reference channelRef = new Reference(channelRefStr);
		String siteRef = SiteService.siteReference(channelRef.getContext());

		// setup for editing the permissions of the site for this tool, using the roles of this site, too
		state.setAttribute(PermissionsAction.STATE_REALM_ID, siteRef);
		state.setAttribute(PermissionsAction.STATE_REALM_ROLES_ID, siteRef);

		// ... with this description
		state.setAttribute(PermissionsAction.STATE_DESCRIPTION, "Set permissions for Chat room in worksite "
				+ SiteService.getSiteDisplay(channelRef.getContext()));

		// ... showing only locks that are prpefixed with this
		state.setAttribute(PermissionsAction.STATE_PREFIX, "chat.");

		// start the helper
		state.setAttribute(PermissionsAction.STATE_MODE, PermissionsAction.MODE_MAIN);

		// schedule a main refresh
		String toolId = PortalService.getCurrentToolId();
		String address = clientWindowId(state, toolId);
		String mainPanelId = mainPanelUpdateId(toolId);
		CourierService.deliver(address, mainPanelId);

	}	// doPermissions

}   // class ChatAction

/**********************************************************************************
*
* $Header: /cvs/sakai/chef-tool/src/java/org/sakaiproject/tool/chat/ChatAction.java,v 1.11 2004/06/22 03:04:58 ggolden Exp $
*
**********************************************************************************/
