/**********************************************************************************
*
* $Header: /cvs/sakai/chef-tool/src/java/org/sakaiproject/tool/support/ContactSupportAction.java,v 1.5 2004/06/22 03:04:58 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.tool.support;

// imports
import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;

import org.sakaiproject.cheftool.Context;
import org.sakaiproject.cheftool.JetspeedRunData;
import org.sakaiproject.cheftool.RunData;
import org.sakaiproject.cheftool.VelocityPortlet;
import org.sakaiproject.cheftool.VelocityPortletPaneledAction;
import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.service.framework.config.cover.ServerConfigurationService;
import org.sakaiproject.service.framework.portal.cover.PortalService;
import org.sakaiproject.service.legacy.resource.ResourceProperties;
import org.sakaiproject.service.framework.session.cover.UsageSessionService;
import org.sakaiproject.service.legacy.site.Site;
import org.sakaiproject.service.legacy.site.cover.SiteService;
import org.sakaiproject.service.framework.state.SessionState;
import org.sakaiproject.service.legacy.user.User;
import org.sakaiproject.service.legacy.user.cover.UserDirectoryService;

/**
 * The ContactSupport tool.
 *
 * @author University of Michigan, CHEF Software Development Team
 * @version $Revision: 1.5 $
 */
public class ContactSupportAction extends VelocityPortletPaneledAction
{
	private final static String STATE_FORM = "form";
	private final static String STATE_CONFIRM = "confirm";
	private final static String STATE_EXCEPTION = "exception";
	private final static String STATE_DOC = "doc";
	private final static String ENC_CHARSETS = "UTF-8";
	private final static String GATEWAY_TITLE = "Gateway";
	
	/**
	 * Build the context for normal display
	 * @param portlet The velocity portlet object
	 * @param context The context object
	 * @param data The RunData object
	 */
	public String buildMainPanelContext(	VelocityPortlet portlet,
	Context context,
	RunData rundata,
	SessionState state)
	{
		String mode = (String) state.getAttribute(STATE_MODE);
		
		if ((mode == null)||("doc".equals(mode)))
		{
			return buildDocModeContext(portlet, context, rundata, state);
		}
		else if ("form".equals(mode))
		{
			return buildFormModeContext(portlet, context, rundata, state);
		}
		else if ("confirm".equals(mode))
		{
			return buildConfirmModeContext(portlet, context, rundata, state);
		}
		else if ("exception".equals(mode))
		{
			return buildExceptionModeContext(portlet, context, rundata, state);
		}
		else
		{
			Log.warn("chef", this + ".buildMainPanelContext: invalid mode: " + mode);
			return null;
		}
		
	} // buildMainPanelContext
	
	/**
	 * build the context for the form mode (in the Main panel).
	 */
	private String buildFormModeContext(VelocityPortlet portlet,
	Context context,
	RunData rundata,
	SessionState state)
	{
		context.put("user", UsageSessionService.getSessionUser());
		
		return (String)getContext(rundata).get("template") + "_form";
		
	}   // buildFormModeContext
	
	/**
	 * build the context for the confirm mode (in the Main panel).
	 */
	private String buildConfirmModeContext(VelocityPortlet portlet,
	Context context,
	RunData rundata,
	SessionState state)
	{
		return (String)getContext(rundata).get("template") + "_confirm";
		
	}   // buildConfirmModeContext
	
	/**
	 * build the context for the doc mode (in the Main panel).
	 */
	private String buildDocModeContext(VelocityPortlet portlet,
	Context context,
	RunData rundata,
	SessionState state)
	{
		return (String)getContext(rundata).get("template") + "_doc";
		
	}   // buildDocModeContext
	
	/**
	 * build the context for the exception mode (in the Main panel).
	 */
	private String buildExceptionModeContext(VelocityPortlet portlet,
	Context context,
	RunData rundata,
	SessionState state)
	{
		return (String)getContext(rundata).get("template") + "_exception";
		
	}   // buildExceptionModeContext
	
	/**
	 * Handle a send comments/questions request.
	 **/
	public void doSend(RunData runData, Context context)
	{
		// access the portlet element id to find our state
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState state = ((JetspeedRunData)runData).getPortletSessionState(peid);
		
		String userName = "";
		userName = runData.getParameters().getString("username");
		String displayName = "";
		displayName = runData.getParameters().getString("displayName");
		String comments = "";
		comments = runData.getParameters().getString("comments");
		String email = "";
		email = runData.getParameters().getString("email");
		
		try
		{
			// Sending information through HTTP: POST
			// get the base URL from configuration
			String postURL = ServerConfigurationService.getString("support.url",null);
			URL url = new URL(postURL);
			HttpURLConnection conn = (HttpURLConnection)url.openConnection();
			
			String content = "";
			
			String UNAVAILABLE = "~unavailable~";

			String CTNG_IP = UNAVAILABLE;
			String CTNG_agent = UNAVAILABLE;
			String CTNG_sessionId = UNAVAILABLE;
			String CTNG_worksiteURL = UNAVAILABLE;
			String CTNG_worksiteTitle = UNAVAILABLE;
			String CTNG_worksiteCreatorUsername = UNAVAILABLE;
			String CTNG_worksiteCreatorEmail = UNAVAILABLE;
			
			// collect all the data
			try
			{
				if (UsageSessionService.getSession() != null)
				{
					CTNG_IP = UsageSessionService.getSession().getIpAddress();
					CTNG_agent = UsageSessionService.getSession().getUserAgent();
					CTNG_sessionId = UsageSessionService.getSession().getId();
				}
				else if (runData.getRequest() != null)
				{
					CTNG_IP = runData.getRequest().getRemoteAddr();
					CTNG_agent = runData.getRequest().getHeader("user-agent");
					CTNG_sessionId = UNAVAILABLE;
				}// if-else if
				
				Site worksite = SiteService.getSite(PortalService.getCurrentSiteId());
				if (worksite != null)
				{					
					CTNG_worksiteURL = worksite.getUrl();
					CTNG_worksiteTitle = worksite.getTitle();
					
					try
					{						
						String creatorId = worksite.getProperties().getProperty(ResourceProperties.PROP_CREATOR);
						if (creatorId != null)
						{
							User creator = UserDirectoryService.getUser(creatorId);
							
							CTNG_worksiteCreatorUsername = creatorId;
							CTNG_worksiteCreatorEmail = creator.getEmail();
						}
					}
					catch (IdUnusedException error)
					{
						Log.debug("chef", "id ununused exception" + error);
					} // try-catch
				} // if (worksite != null)
			}
			catch (IdUnusedException e)
			{
				Log.debug("chef", "id ununused exception" + e);
			} // try-catch : collect data
			
			try
			{
				content =
				"CTNG_username=" + URLEncoder.encode(userName, ENC_CHARSETS) +
				"&CTNG_displayName=" + URLEncoder.encode(displayName, ENC_CHARSETS) +
				"&CTNG_comments=" + URLEncoder.encode(comments, ENC_CHARSETS) +
				"&CTNG_email=" + URLEncoder.encode(email, ENC_CHARSETS) +
				"&CTNG_IP=" + URLEncoder.encode(CTNG_IP, ENC_CHARSETS) +
				"&CTNG_agent=" + URLEncoder.encode(CTNG_agent, ENC_CHARSETS) +
				"&CTNG_sessionId=" + URLEncoder.encode(CTNG_sessionId, ENC_CHARSETS) +
				"&CTNG_worksiteURL=" + URLEncoder.encode(CTNG_worksiteURL, ENC_CHARSETS) +
				"&CTNG_worksiteTitle=" + URLEncoder.encode(CTNG_worksiteTitle, ENC_CHARSETS) +
				"&CTNG_worksiteCreatorUsername=" + URLEncoder.encode(CTNG_worksiteCreatorUsername, ENC_CHARSETS) +
				"&CTNG_worksiteCreatorEmail=" + URLEncoder.encode(CTNG_worksiteCreatorEmail, ENC_CHARSETS);
			}
			catch(UnsupportedEncodingException err)
			{
				Log.warn("chef","UnsupportedEncodingException" + err);
			}
			
			if (Log.getLogger("chef").isDebugEnabled())
				Log.debug("chef", "content to be sent is: " + content);
			
			conn.setDoOutput(true);
			conn.setDoInput(true);
			conn.setUseCaches(false);
			conn.setRequestMethod("POST");
			conn.setRequestProperty("Content-type", "application/x-www-form-urlencoded");
			conn.setRequestProperty("Content-length", "" + content.length());
			DataOutputStream ostream = new DataOutputStream( conn.getOutputStream() );
			
			ostream.writeBytes(content);
			ostream.close();
			
			// read the answer
			BufferedReader br = new BufferedReader(new InputStreamReader(conn.getInputStream()));
			if (br == null)
			{
				state.setAttribute(STATE_MODE, STATE_EXCEPTION);
			}
			else
			{
				String line;
				while ((line = br.readLine()) != null)
					if (Log.getLogger("chef").isDebugEnabled())
						Log.debug("chef","response from server" +line);
			}
			br.close();
			conn.disconnect();
			
			// set state to be "confirm"
			state.setAttribute(STATE_MODE, STATE_CONFIRM);
		}
		catch (MalformedURLException e)
		{
			Log.warn("chef","net exception" + e);
			state.setAttribute(STATE_MODE, STATE_EXCEPTION);
			
		}
		catch (IOException e)
		{
			Log.warn("chef","io exception" + e);
			state.setAttribute(STATE_MODE, STATE_EXCEPTION);
		}
	}	// doSend
	
	/**
	 * Handle a new comments/questions request.
	 **/
	public void doNew(RunData runData, Context context)
	{
		// access the portlet element id to find our state
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState state = ((JetspeedRunData)runData).getPortletSessionState(peid);
		
		// set state to be "confirm"
		state.setAttribute(STATE_MODE, STATE_FORM);
		
	}	// doNew
	
	/**
	 * Handle a "going back to help documents" request.
	 **/
	public void doDoc(RunData runData, Context context)
	{
		// access the portlet element id to find our state
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState state = ((JetspeedRunData)runData).getPortletSessionState(peid);
		
		// set state to be "confirm"
		state.setAttribute(STATE_MODE, STATE_DOC);
		
	}	// doDoc
	
	/**
	 * Populate the state object, if needed.
	 * @param state The state object.
	 * @param portlet The portlet.
	 * @param rundata The current request's rundata.
	 */
	protected void initState(SessionState state, VelocityPortlet portlet, JetspeedRunData rundata)
	{
		super.initState(state, portlet, rundata);
		
	}   // initState
	
	
}   // ContactSupportAction

/**********************************************************************************
*
* $Header: /cvs/sakai/chef-tool/src/java/org/sakaiproject/tool/support/ContactSupportAction.java,v 1.5 2004/06/22 03:04:58 ggolden Exp $
*
**********************************************************************************/
