/**********************************************************************************
*
* $Header: /cvs/sakai/component/src/java/org/sakaiproject/component/framework/current/BasicCurrentService.java,v 1.5 2004/06/22 03:10:08 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

package org.sakaiproject.component.framework.current;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.sakaiproject.service.framework.current.CurrentService;
import org.sakaiproject.service.framework.log.Logger;

/**
 * <p>BasicCurrentService implements the CurrentService.</p>
 * <p>See the {@link org.sakaiproject.service.framework.current.CurrentService} interface for details.</p>
 * 
 * @author University of Michigan, CHEF Software Development Team
 * @version $Revision: 1.5 $
 */
public class BasicCurrentService implements CurrentService
{
	/** map keyed by thread of binding maps. */
	protected Map m_currentThreads = null;

	/*******************************************************************************
	* Dependencies and their setter methods
	*******************************************************************************/

	/** Dependency: a Logger */
	protected Logger logger = null;

	/**
	 * Dependency: a Logger.
	 * @param service the logger.
	 */
	public void setLogger(Logger service)
	{
		this.logger = service;
	}

	/*******************************************************************************
	* Init and Destroy
	*******************************************************************************/

	/**
	 * Final initialization, once all dependencies are set.
	 */
	public void init()
	{
		// allocate a thread-safe map to store the bindings for each thread
		m_currentThreads = Collections.synchronizedMap(new HashMap());

		logger.info(this +".init()");
	}

	/**
	 * Final cleanup.
	 */
	public void destroy()
	{
		m_currentThreads.clear();
		m_currentThreads = null;

		logger.info(this +".destroy()");
	}

	/*******************************************************************************
	* Work interface methods: org.sakaiproject.service.framework.current.CurrentService
	*******************************************************************************/

	/**
	 * {@inheritDoc}
	 */
	public void setInThread(String name, Object value)
	{
		// find the map that might already exist
		Map bindings = (Map) m_currentThreads.get(Thread.currentThread());
		if (bindings == null)
		{
			// make a new map
			bindings = new HashMap();

			// store the map associated with this thread
			m_currentThreads.put(Thread.currentThread(), bindings);
		}

		// remove if nulling
		if (value == null)
		{
			bindings.remove(name);
		}

		// otherwise bind the object
		else
		{
			bindings.put(name, value);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public void clearInThread()
	{
		// clear the bindings map associated with this thread
		m_currentThreads.remove(Thread.currentThread());
	}

	/**
	 * {@inheritDoc}
	 */
	public Object getInThread(String name)
	{
		// find the map that might already exist
		Map bindings = (Map) m_currentThreads.get(Thread.currentThread());
		if (bindings == null)
		{
			return null;
		}
		
		return bindings.get(name);
	}
}

/**********************************************************************************
*
* $Header: /cvs/sakai/component/src/java/org/sakaiproject/component/framework/current/BasicCurrentService.java,v 1.5 2004/06/22 03:10:08 ggolden Exp $
*
**********************************************************************************/
