/**********************************************************************************
*
* $Header: /cvs/sakai/embedded/src/java/org/sakaiproject/portal/sedna/SednaServlet.java,v 1.8 2004/06/22 03:10:50 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

package org.sakaiproject.portal.sedna;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.service.framework.config.cover.ServerConfigurationService;
import org.sakaiproject.service.framework.session.cover.UsageSessionService;
import org.sakaiproject.service.legacy.site.Site;
import org.sakaiproject.service.legacy.site.SitePage;
import org.sakaiproject.service.legacy.site.cover.SiteService;
import org.sakaiproject.service.legacy.user.User;
import org.sakaiproject.service.legacy.user.cover.UserDirectoryService;
import org.sakaiproject.util.Validator;
import org.sakaiproject.vm.VmServlet;

/**
 * <p>SednaServlet is ...</p>
 * 
 * @author University of Michigan, CHEF Software Development Team
 * @version $Revision: 1.8 $
 */
public class SednaServlet extends VmServlet
{
	/** The "face" (vm file) to use. */
	protected String m_face = "portal";

	/**
	 * Initialize the servlet.
	 * @param config The servlet config.
	 * @throws ServletException
	 */
	public void init(ServletConfig config) throws ServletException
	{
		super.init(config);
		String face = config.getInitParameter("face");
		if ((face != null) && (face.trim().length() > 0))
		{
			m_face = face.trim();
		}
	}

	/**
	 * Handle a post - just like a get.
	 */
	protected void doPost(HttpServletRequest req, HttpServletResponse res) throws ServletException
	{
		doGet(req, res);
	}

	/**
	 * Handle a get - respond with the requested portal display
	 */
	protected void doGet(HttpServletRequest req, HttpServletResponse res) throws ServletException
	{
		// parse the request
		try
		{
			// who's the user?
			User user = UsageSessionService.getSessionUser();
			boolean loggedIn = !(UserDirectoryService.getAnonymousUser().equals(user));
			setVmReference("loggedIn", Boolean.valueOf(loggedIn), req);

			// which site - use the myWorkspace or gateway if missing
			String id = req.getParameter("site");
			if (id == null)
			{
				// if logged in, send them to their worksite
				if (loggedIn)
				{
					id = SiteService.getUserSiteId(user.getId());
				}
				// otherwise send them to the gateway
				else
				{
					// TODO: paramaterize, or !gateway
					id = "~anon";
				}
			}

			// get the site - if a user site, and missing, use the default user site
			Site site = null;
			try
			{
				site = SiteService.getSite(id);
			}
			catch (IdUnusedException e)
			{
				if (SiteService.isUserSite(id))
				{
					site = SiteService.getSite(SiteService.getUserSiteId(""));
				}
			}

			if (site == null)
			{
				// TODO: which site for errors, gateway?
				site = SiteService.getSite("~anon");
			}

			setVmReference("site", site, req);

			// get the page - optional
			id = req.getParameter("page");
			SitePage page = null;
			if (id != null)
			{
				page = site.getPage(id);
			}

			// if no page in request, use the first page of the site
			else
			{
				page = (SitePage) site.getPages().get(0);
			}
			setVmReference("page", page, req);

			setVmReference("config", ServerConfigurationService.getInstance(), req);
			setVmReference("validator", new Validator(), req);
		}
		catch (Exception e)
		{
		}

		// get the "face" - vm file to use
		String face = req.getParameter("face");
		if (face == null)
		{
			face = m_face;
		}

		// set standard no-cache headers
		setNoCacheHeaders(res);

		includeVm("/vm/" + face + ".vm", req, res);
	}
}

/**********************************************************************************
*
* $Header: /cvs/sakai/embedded/src/java/org/sakaiproject/portal/sedna/SednaServlet.java,v 1.8 2004/06/22 03:10:50 ggolden Exp $
*
**********************************************************************************/
