/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/session/BasicUsageSessionService.java,v 1.6 2004/06/22 03:13:32 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.component.framework.session;

// imports
import java.util.Hashtable;

import org.sakaiproject.service.framework.session.UsageSession;

/**
* <p>BasicUsageSessionService is a CHEF usage session service that is stand-alone and an authority of ALL
* possible sessions.</p>
* 
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.6 $
*/
public class BasicUsageSessionService
	extends BaseUsageSessionService
{	
	/*******************************************************************************
	* Abstractions, etc.
	*******************************************************************************/

	/**
	* Construct storage for this service.
	*/
	protected Storage newStorage() { return new BasicStorage(); }

	/*******************************************************************************
	* Storage
	*******************************************************************************/

	protected class BasicStorage
		implements Storage
	{
		/** The sessions, keyed by id. */
		protected Hashtable m_sessions = new Hashtable();

		/**
		* Open and be ready to read / write.
		*/
		public void open()
		{
		}	// open

		/**
		* Close.
		*/
		public void close()
		{
			m_sessions.clear();
		}	// close

		/**
		* Take this session into storage.
		* @param session The usage session.
		*/
		public void addSession(UsageSession session)
		{
			m_sessions.put(session.getId(), session);

		}	// addSession

		/**
		* Access a session by id
		* @param id The session id.
		* @return The session object.
		*/
		public UsageSession getSession(String id)
		{
			return (UsageSession) m_sessions.get(id);

		}	// getSession

		/**
		* Access a local (i.e. maintained in this app server) session by id
		* @param id The session id.
		* @return The session object.
		*/
		public UsageSession getLocalSession(String id)
		{
			// all are local!
			return getSession(id);

		}	// getLocalSession

		/**
		* This session is now closed.
		* @param session The session which is closed.
		*/
		public void closeSession(UsageSession session) {}

	}   // Storage

}   // BaseUsageSessionService

/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/session/BasicUsageSessionService.java,v 1.6 2004/06/22 03:13:32 ggolden Exp $
*
**********************************************************************************/
