/**********************************************************************************
*
* $Header: /cvs/sakai/james/src/java/org/sakaiproject/james/SakaiMailet.java,v 1.4 2004/06/22 03:13:43 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.james;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.Date;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.mail.Address;
import javax.mail.BodyPart;
import javax.mail.Header;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.internet.ContentType;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeUtility;

import org.apache.mailet.GenericMailet;
import org.apache.mailet.Mail;
import org.apache.mailet.MailAddress;
import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.exception.PermissionException;
import org.sakaiproject.service.legacy.alias.cover.AliasService;
import org.sakaiproject.service.legacy.content.ContentResource;
import org.sakaiproject.service.legacy.content.cover.ContentHostingService;
import org.sakaiproject.service.legacy.email.MailArchiveChannel;
import org.sakaiproject.service.legacy.email.cover.MailArchiveService;
import org.sakaiproject.service.framework.log.cover.Logger;
import org.sakaiproject.service.legacy.resource.Reference;
import org.sakaiproject.service.legacy.resource.ReferenceVector;
import org.sakaiproject.service.legacy.resource.ResourceProperties;
import org.sakaiproject.service.legacy.resource.ResourcePropertiesEdit;
import org.sakaiproject.service.framework.session.cover.UsageSessionService;
import org.sakaiproject.service.legacy.site.cover.SiteService;
import org.sakaiproject.service.legacy.time.cover.TimeService;
import org.sakaiproject.service.legacy.user.User;
import org.sakaiproject.service.legacy.user.cover.UserDirectoryService;
import org.sakaiproject.util.StringUtil;
import org.sakaiproject.util.Validator;

/**
 * <p>SakaiMailet ...</p>
 *
 * @author University of Michigan, CHEF Software Development Team
 * @version $Revision: 1.4 $
 */
public class SakaiMailet extends GenericMailet
{
	/** The user name of the postmaster user - the one who posts incoming mail. */
	public static final String POSTMASTER = "postmaster";

	/**
	 * Called when created.
	 */
	public void init() throws MessagingException
	{
		Logger.info(this +".init()");
	}

	/**
	* Called when leaving.
	*/
	public void destroy()
	{
		Logger.info(this +".destroy()");

		super.destroy();

	} // destroy

	/**
	 * Process incoming mail.
	 * @param mail ...
	 */
	public void service(Mail mail) throws MessagingException
	{
		UsageSessionService.loginThreadUser(POSTMASTER);
		try
		{
			MimeMessage msg = mail.getMessage();
			String id = msg.getMessageID();

			Address[] fromAddresses = msg.getFrom();
			String from = null;
			String fromAddr = null;
			if ((fromAddresses != null) && (fromAddresses.length == 1))
			{
				from = fromAddresses[0].toString();
				if (fromAddresses[0] instanceof InternetAddress)
				{
					fromAddr = ((InternetAddress) (fromAddresses[0])).getAddress();
				}
			}
			else
			{
				from = mail.getSender().toString();
				fromAddr = mail.getSender().toInternetAddress().getAddress();
				// mail.getSender().getUser() + "@" + mail.getSender().getHost();
			}

			Collection to = mail.getRecipients();

			Date sent = msg.getSentDate();

			String subject = StringUtil.trimToNull(msg.getSubject());
			if (subject == null)
				subject = "<no subject>";

			Enumeration headers = msg.getAllHeaderLines();
			List mailHeaders = new Vector();
			while (headers.hasMoreElements())
			{
				String line = (String) headers.nextElement();
				mailHeaders.add(line);
			}

			// we will parse the body and attachments later, when we know we need to.
			String body = null;
			ReferenceVector attachments = null;

			if (Logger.isInfoEnabled())
				Logger.info(
					this
						+ ": "
						+ id
						+ " : mail: from:"
						+ from
						+ " sent: "
						+ TimeService.newTime(sent.getTime()).toStringLocalFull()
						+ " subject: "
						+ subject);

			// process for each recipient
			Iterator it = to.iterator();
			while (it.hasNext())
			{
				String mailId = null;
				try
				{
					MailAddress recipient = (MailAddress) it.next();
					if (Logger.isInfoEnabled())
						Logger.info(this +": " + id + " : checking to: " + recipient);

					// is the host ok? %%%
					//String host = recipient.getHost();

					// the recipient's mail id
					mailId = recipient.getUser();

					// find the channel (mailbox) that this is adressed to
					// for now, check only for it being a site or alias to a site.
					// %%% - add user and other later -ggolden
					MailArchiveChannel channel = null;

					// first, assume the mailId is a site id
					String channelRef = MailArchiveService.channelReference(mailId, SiteService.MAIN_CONTAINER);
					try
					{
						channel = MailArchiveService.getMailArchiveChannel(channelRef);
					}
					catch (IdUnusedException goOn)
					{
					}

					// next, if not a site, see if it's an alias to a site or channel
					if (channel == null)
					{
						// if not an alias, it will throw the IdUnusedException caught below
						Reference ref = new Reference(AliasService.getTarget(mailId));

						// if ref is a site
						if (ref.getType().equals(SiteService.SERVICE_NAME))
						{
							// now we have a site reference, try for it's channel
							channelRef = MailArchiveService.channelReference(ref.getId(), SiteService.MAIN_CONTAINER);
						}

						// if ref is a channel
						else if (ref.getType().equals(MailArchiveService.SERVICE_NAME))
						{
							// ref is a channel
							channelRef = ref.getReference();
						}

						// ref is something else
						else
						{
							if (Logger.isInfoEnabled())
								Logger.info(this +": " + id + " : mail rejected: unknown address: " + mailId);

							throw new IdUnusedException(mailId);
						}

						// if there's no channel for this site, it will throw the IdUnusedException caught below
						channel = MailArchiveService.getMailArchiveChannel(channelRef);
					}

					// skip disabled channels
					if (!channel.getEnabled())
					{
						if (from.startsWith(POSTMASTER))
						{
							mail.setState(Mail.GHOST);
						}
						else
						{
							mail.setErrorMessage("This mail channel is not accepting emails.");
						}

						if (Logger.isInfoEnabled())
							Logger.info(this +": " + id + " : mail rejected: channel not enabled: " + mailId);

						continue;
					}

					// for non-open channels, make sure the from is a member
					if (!channel.getOpen())
					{
						// find the user with this email address
						try
						{
							User user = UserDirectoryService.findUserByEmail(fromAddr);

							// set this use for the permission test
							UsageSessionService.loginThreadUser(user.getId());
							boolean allowed = channel.allowAddMessage();
							UsageSessionService.loginThreadUser(POSTMASTER);

							// does this use have the ability to send messages to the channel?
							if (!allowed)
							{
								if (Logger.isInfoEnabled())
									Logger.info(
										this
											+ ": "
											+ id
											+ " : mail rejected: from: "
											+ fromAddr
											+ " not authorized: "
											+ mailId);

								mail.setErrorMessage("Email from this address is not accepted in this mail channel.");
								continue;
							}
						}
						catch (IdUnusedException e)
						{
							// we don't know this user, skip it
							if (Logger.isInfoEnabled())
								Logger.info(this +": " + id + " : mail rejected: unknown from: " + fromAddr);

							mail.setErrorMessage("Email from this address is not accepted in this mail channel.");
							continue;
						}
					}

					// prepare the message if it has not yet been prepared - we need it.%%%
					if (body == null)
					{
						body = "";
						attachments = new ReferenceVector();
						try
						{
							StringBuffer bodyBuf = new StringBuffer();
							Integer embedCount = parseParts(msg, id, bodyBuf, attachments, new Integer(-1));
							body = bodyBuf.toString();
						}
						catch (MessagingException e)
						{
							Logger.warn(this +".service(): msg.getContent() threw: " + e);
						}
						catch (IOException e)
						{
							Logger.warn(this +".service(): msg.getContent() threw: " + e);
						}
					}

					// post the message to the group's channel
					channel.addMailArchiveMessage(
						subject,
						from.toString(),
						TimeService.newTime(sent.getTime()),
						mailHeaders,
						attachments,
						body);

					if (Logger.isInfoEnabled())
						Logger.info(this +": " + id + " : delivered to:" + mailId);

					// all is happy - ghost the message to stop further processing
					mail.setState(Mail.GHOST);
				}
				catch (IdUnusedException goOn)
				{
					if (Logger.isInfoEnabled())
						Logger.info(this +": " + id + " : mail rejected: " + goOn.toString());

					if (from.startsWith(POSTMASTER + "@"))
					{
						mail.setState(Mail.GHOST);
					}
					else
					{
						mail.setErrorMessage("Recipient is not known on this system.");
					}
				}
				catch (PermissionException e)
				{
					if (Logger.isInfoEnabled())
						Logger.info(this +": " + id + " : " + e);
				}
			}
		}
		finally
		{
			UsageSessionService.logoutThreadUser();
		}
	}

	/**
	 * Create an attachment, adding it to the list of attachments.
	 */
	protected Reference createAttachment(ReferenceVector attachments, String type, String fileName, byte[] body, String id)
	{
		// we just want the file name part - strip off any drive and path stuff
		String name = Validator.getFileName(fileName);

		// make a set of properties to add for the new resource
		ResourcePropertiesEdit props = ContentHostingService.newResourceProperties();
		props.addProperty(ResourceProperties.PROP_DISPLAY_NAME, name);
		props.addProperty(ResourceProperties.PROP_DESCRIPTION, fileName);

		// make an attachment resource for this URL
		try
		{
			ContentResource attachment = ContentHostingService.addAttachmentResource(name, type, body, props);

			// add a dereferencer for this to the attachments
			Reference ref = new Reference(attachment.getReference());
			attachments.add(ref);

			if (Logger.isInfoEnabled())
				Logger.info(this +": " + id + " : attachment: " + ref.getReference() + " size: " + body.length);

			return ref;
		}
		catch (Exception any)
		{
			Logger.warn(this +": " + id + " : exception adding attachment resource: " + name + " : " + any.toString());
			return null;
		}
	}

	/**
	 * Read in a stream from the mime body into a byte array
	 */
	protected byte[] readBody(int size, InputStream stream)
	{
		if (size <= 0)
			return null;

		byte[] byteContent = new byte[size];

		try
		{
			int lenRead = 0;
			while (lenRead < size)
			{
				int count = stream.read(byteContent, lenRead, size - lenRead);
				if (count <= 0)
					break;
				lenRead += count;
			}
		}
		catch (IOException e)
		{
			Logger.warn(this +"readBody(): " + e);
		}

		return byteContent;
	}

	/**
	 * Breaks email messages into parts which can be saved as files (saves as attachments)
	 * or viewed as plain text (added to body of message).
	 * @param p	The message-part embedded in a message..
	 * @param id	The string containing the message's id.
	 * @param bodyBuf	The string-buffer in which the message body is being built.
	 * @param attachments	The ReferenceVector in which references to attachments are collected.
	 * @param embedCount	An Integer that counts embedded messages (outer message is zero).
	 * @return Value of embedCount (updated if this call processed any embedded messages).
	 */
	protected Integer parseParts(Part p, String id, StringBuffer bodyBuf, ReferenceVector attachments, Integer embedCount)
		throws MessagingException, IOException
	{
		String closing = "";
		// process embedded messages
		if (p instanceof Message)
		{
			// increment embedded message counter
			int thisCount = embedCount.intValue() + 1;
			embedCount = new Integer(thisCount);

			// process inner messages, inserting start and end labels except for outer message.
			if (thisCount > 0)
			{
				// make sure previous message parts ended with newline
				if (bodyBuf.length() > 0 && bodyBuf.charAt(bodyBuf.length() - 1) != '\n')
				{
					bodyBuf.append("\n");
				}
				bodyBuf.append("\n========= Begin embedded email message " + thisCount + " =========\n\n");
				parseEnvelope((Message) p, id, bodyBuf, attachments, embedCount);
				closing = "\n========== End embedded email message " + thisCount + " ==========\n\n";
			}
		}

		String type = p.getContentType();

		// discard if content-type is unknown
		if (type == null || type.equals(""))
		{
			// do nothing
		}
		// add plain text to body
		else if (p.isMimeType("text/plain") && p.getFileName() == null)
		{
			Object o = p.getContent();
			String txt = null;
			if (o instanceof String)
			{
				txt = (String) p.getContent();
			}

			else if (o instanceof InputStream)
			{
				InputStream in = (InputStream) o;
				ByteArrayOutputStream out = new ByteArrayOutputStream();
				byte[] buf = new byte[in.available()];
				for (int len = in.read(buf); len != -1; len = in.read(buf))
					out.write(buf, 0, len);
				ContentType contentType = new ContentType(p.getContentType());
				String charset = contentType.getParameter("charset");
				txt = out.toString(MimeUtility.javaCharset(charset));
			}

			// make sure previous message parts ended with newline
			if (bodyBuf.length() > 0 && bodyBuf.charAt(bodyBuf.length() - 1) != '\n')
			{
				bodyBuf.append("\n");
			}
			bodyBuf.append(txt);
		}
		// process subparts of multiparts
		else if (p.isMimeType("multipart/*"))
		{
			Multipart mp = (Multipart) p.getContent();
			int count = mp.getCount();
			for (int i = 0; i < count; i++)
			{
				embedCount = parseParts(mp.getBodyPart(i), id, bodyBuf, attachments, embedCount);
			}
		}
		// process embedded messages
		else if (p.isMimeType("message/rfc822"))
		{
			embedCount = parseParts((Part) p.getContent(), id, bodyBuf, attachments, embedCount);
		}
		// everything else gets treated as an attachment
		else
		{
			ContentType cType = new ContentType(type);
			String name = p.getFileName();
			String disposition = p.getDisposition();
			int size = p.getSize();

			if (name == null)
			{
				name = "unknown";
				// if file's parent is multipart/alternative, 
				// provide a better name for the file
				if (p instanceof BodyPart)
				{
					Multipart parent = ((BodyPart) p).getParent();
					if (parent != null)
					{
						String pType = parent.getContentType();
						ContentType pcType = new ContentType(pType);
						if (pcType.getBaseType().equalsIgnoreCase("multipart/alternative"))
						{
							name = "message" + embedCount;
						}
					}
				}
				if (p.isMimeType("text/html"))
				{
					name += ".html";
				}
				else if (p.isMimeType("text/richtext"))
				{
					name += ".rtx";
				}
				else if (p.isMimeType("text/rtf"))
				{
					name += ".rtf";
				}
				else if (p.isMimeType("text/enriched"))
				{
					name += ".etf";
				}
				else if (p.isMimeType("text/plain"))
				{
					name += ".txt";
				}
				else if (p.isMimeType("text/xml"))
				{
					name += ".xml";
				}
			}

			// read the attachments bytes, and create it as an attachment in content hosting
			byte[] bodyBytes = readBody(size, p.getInputStream());
			if ((bodyBytes != null) && (bodyBytes.length > 0))
			{
				// can we ignore the attachment it it's just whitespace chars??
				Reference attachment = createAttachment(attachments, cType.getBaseType(), name, bodyBytes, id);

				// attachment reference URL goes here
				if (attachment != null)
				{
					// make sure previous message parts ended with newline
					if (bodyBuf.length() > 0 && bodyBuf.charAt(bodyBuf.length() - 1) != '\n')
					{
						bodyBuf.append("\n");
					}
					bodyBuf.append("[see attachment: \"" + name + "\", size: " + size + " bytes]\n\n");
				}
			}
		}
		// make sure previous message parts ended with newline
		if (bodyBuf.length() > 0 && bodyBuf.charAt(bodyBuf.length() - 1) != '\n')
		{
			bodyBuf.append("\n");
		}
		bodyBuf.append(closing);

		return embedCount;
	}

	/**
	 * Saves header info about embedded email messages.
	 * @param innerMsg	The message-part believed to be an embedded message..
	 * @param id	The string containing the outer message's id.
	 * @param bodyBuf	The string-buffer in which the message body is being built.
	 * @param attachments	The ReferenceVector in which references to attachments are collected.
	 * @param embedCount	An Integer that counts embedded messages (Outer message is zero).
	 */
	protected void parseEnvelope(
		Message innerMsg,
		String id,
		StringBuffer bodyBuf,
		ReferenceVector attachments,
		Integer embedCount)
		throws MessagingException, IOException
	{
		Address[] innerFroms = innerMsg.getFrom();
		// make sure previous message parts ended with newline
		if (bodyBuf.length() > 0 && bodyBuf.charAt(bodyBuf.length() - 1) != '\n')
		{
			bodyBuf.append("\n");
		}
		if (innerFroms != null)
		{
			String innerFrom = "";
			for (int n = 0; n < innerFroms.length; n++)
			{
				innerFrom += innerFroms[n].toString();
			}
			if (innerFrom.length() > 0)
			{
				bodyBuf.append("From: " + innerFrom + "\n");
			}
		}
		Address[] innerRecipients = innerMsg.getRecipients(Message.RecipientType.TO);
		if (innerRecipients != null)
		{
			String innerRecipient = "";
			for (int n = 0; n < innerRecipients.length; n++)
			{
				innerRecipient += innerRecipients[n].toString();
			}
			if (innerRecipient.length() > 0)
			{
				bodyBuf.append("To: " + innerRecipient + "\n");
			}
		}
		String innerSubject = innerMsg.getSubject().trim();
		if (innerSubject.equals(""))
		{
			innerSubject = "<no subject>";
		}
		Date innerDate = innerMsg.getSentDate();

		bodyBuf.append("Subject: " + innerSubject + "\nDate: " + innerDate.toString() + "\n\n");
		Enumeration innerHdrs = innerMsg.getAllHeaders();
		String hdrStr = new String();
		while (innerHdrs.hasMoreElements())
		{
			Header iHdr = (Header) innerHdrs.nextElement();
			hdrStr += iHdr.getName() + ": " + iHdr.getValue() + "\n";
		}
		if (hdrStr.length() > 0)
		{
			String name = "headers" + embedCount.toString() + ".txt";
			Reference attachment = createAttachment(attachments, "text/plain", name, hdrStr.getBytes(), id);
			bodyBuf.append("[see attachment: \"" + name + "\" size: " + hdrStr.length() + " bytes]\n\n");
		}
	}
}

/**********************************************************************************
*
* $Header: /cvs/sakai/james/src/java/org/sakaiproject/james/SakaiMailet.java,v 1.4 2004/06/22 03:13:43 ggolden Exp $
*
**********************************************************************************/
