/**********************************************************************************
*
* $Header: /cvs/sakai/legacy-component/src/java/org/sakaiproject/component/legacy/archive/BasicArchiveService.java,v 1.15 2004/06/22 03:14:02 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.component.legacy.archive;

// imports
import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Stack;

import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.service.legacy.archive.ArchiveService;
import org.sakaiproject.service.framework.component.cover.ComponentManager;
import org.sakaiproject.service.framework.config.cover.ServerConfigurationService;
import org.sakaiproject.service.legacy.content.ContentHostingService;
import org.sakaiproject.service.framework.log.Logger;
import org.sakaiproject.service.legacy.resource.ReferenceVector;
import org.sakaiproject.service.legacy.resource.ResourceService;
import org.sakaiproject.service.legacy.site.Site;
import org.sakaiproject.service.legacy.site.cover.SiteService;
import org.sakaiproject.service.legacy.time.Time;
import org.sakaiproject.service.legacy.time.cover.TimeService;
import org.sakaiproject.util.Xml;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
* <p>...</p>
* 
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.15 $
*/
public class BasicArchiveService
	implements ArchiveService
{
	/** A full path and file name to the storage file. */
	protected String m_storagePath = "/";

	/*******************************************************************************
	* Dependencies and their setter methods
	*******************************************************************************/

	/** Dependency: logging service */
	protected Logger m_logger = null;

	/**
	 * Dependency: logging service.
	 * @param service The logging service.
	 */
	public void setLogger(Logger service)
	{
		m_logger = service;
	}

	/**
	 * Configuration: Set the Storage Path.
	 * @param path The storage path.
	 */
	public void setStoragePath(String path)
	{
		m_storagePath = path;
	}

	/*******************************************************************************
	* Init and Destroy
	*******************************************************************************/

	/**
	 * Final initialization, once all dependencies are set.
	 */
	public void init()
	{
		m_logger.info(this + ".init(): storage path: " + m_storagePath);
	}

	/**
	 * Final cleanup.
	 */
	public void destroy()
	{
		m_logger.info(this + ".destroy()");
	}

	/*******************************************************************************
	* ArchiveService implementation
	*******************************************************************************/

	/**
	* Create an archive for the resources of a site.
	* @param siteId The id of the site to archive.
	* @return A log of messages from the archive.
	*/
	public String archive(String siteId)
	{
		StringBuffer results = new StringBuffer();

		if (m_logger.isDebugEnabled())
			m_logger.debug(this + ".archive(): site: " + siteId);

		Site theSite = null;
		try
		{
			theSite = SiteService.getSite(siteId);
		}
		catch (IdUnusedException e)
		{
			results.append("Site: " + siteId + " not found.\n");
			m_logger.warn(this + ".archive(): site not found: " + siteId);
			return results.toString();
		}

		// collect all the attachments we need
		ReferenceVector attachments = new ReferenceVector();

		Time now = TimeService.newTime();

		// this is the folder we are writing files to
		String storagePath = m_storagePath + siteId + "-archive/";

		// create the directory for the archive
		File dir = new File(m_storagePath + siteId + "-archive/");
		dir.mkdirs();

		// for each registered ResourceService, give it a chance to archve
		List services = ServerConfigurationService.getResourceServices();
		for (Iterator iServices = services.iterator(); iServices.hasNext();)
		{
			ResourceService service = (ResourceService) iServices.next();
			if (service == null) continue;

			Document doc = Xml.createDocument();
			Stack stack = new Stack();
			Element root = doc.createElement("archive");
			doc.appendChild(root);
			root.setAttribute("source", siteId);
			root.setAttribute("server", ServerConfigurationService.getServerId());
			root.setAttribute("date", now.toString());
			stack.push(root);

			String msg = service.archive(siteId, doc, stack, storagePath, attachments);
			results.append(msg);
	
			stack.pop();
			
			String fileName = storagePath + service.getLabel() + ".xml";
			Xml.writeDocument(doc, fileName);
		}

		// archive the collected attachments
		if (attachments.size() > 0)
		{
			Document doc = Xml.createDocument();
			Stack stack = new Stack();
			Element root = doc.createElement("archive");
			doc.appendChild(root);
			root.setAttribute("source", siteId);
			root.setAttribute("server", ServerConfigurationService.getServerId());
			root.setAttribute("date", now.toString());
			stack.push(root);

			String msg = org.sakaiproject.service.legacy.content.cover.ContentHostingService.archiveResources(attachments, doc, stack, storagePath);
			results.append(msg);

			stack.pop();

			String fileName = storagePath + "attachment.xml";
			Xml.writeDocument(doc, fileName);
		}

		// *** Site
/*
		doc = Xml.createDocument();
		stack = new Stack();
		root = doc.createElement("archive");
		doc.appendChild(root);
		root.setAttribute("site", siteId);
		root.setAttribute("date", now.toString());
		stack.push(root);

		archiveSite(theSite, doc, stack);

		stack.pop();
		Xml.writeDocument(doc, m_storagePath + siteId + "-archive/" + siteId + "-site.xml");
*/
		// *** Users
/*
		doc = Xml.createDocument();
		stack = new Stack();
		root = doc.createElement("archive");
		doc.appendChild(root);
		root.setAttribute("site", siteId);
		root.setAttribute("date", now.toString());
		stack.push(root);

		// %%% needs to become realm -ggolden
		//archiveUsers(theSite, doc, stack);

		stack.pop();
		Xml.writeDocument(doc, m_storagePath + siteId + "-archive/" + siteId + "-user.xml");
*/

		return results.toString();

	}	// archive

	/**
	* Archive the site definition.
	* @param site the site.
	* @param doc The document to contain the xml.
	* @param stack The stack of elements, the top of which will be the containing
	* element of the "site" element.
	*/
/*
	protected void archiveSite(Site site, Document doc, Stack stack)
	{
		Element element = doc.createElement(SiteService.SERVICE_NAME);
		((Element)stack.peek()).appendChild(element);
		stack.push(element);

		try
		{
			site.toXml(doc, stack);
		}
		catch (Exception any)
		{
			m_logger.warn(this + ".archve: exception archiving site: "
						+ site.getId() + ": ", any);
		}

		stack.pop();

	}	// archiveSite
*/

	/**
	* Archive the users defined in this site (internal users only).
	* @param site the site.
	* @param doc The document to contain the xml.
	* @param stack The stack of elements, the top of which will be the containing
	* element of the "site" element.
	*/
/*
	protected void archiveUsers(Site site, Document doc, Stack stack)
	{
		Element element = doc.createElement(UserDirectoryService.SERVICE_NAME);
		((Element)stack.peek()).appendChild(element);
		stack.push(element);

		try
		{
			// get the site's user list
			User[] users = site.getMembers();
			if ((users != null) && (users.length > 0))
			{
				for (int i = 0; i < users.length; i++)
				{
					// if we can get an edit, then it's internaly kept, and we need to archive it, else it's
					// from outside, and we won't archive it
					try
					{
						UserEdit edit = UserDirectoryService.editUser(users[i].getId());
						edit.toXml(doc, stack);
						UserDirectoryService.cancelEdit(edit);
					}
					catch (Exception any) {}
				}
			}
		}
		catch (Exception any)
		{
			m_logger.warn(this + ".archve: exception archiving users: "
						+ site.getId() + ": ", any);
		}

		stack.pop();

	}	// archiveUsers
*/

	/**
	* Process a merge for the file, or if it's a directory, for all contained files (one level deep).
	* @param fileName The site name (for the archive file) to read from.
	* @param mergeId The id string to use to make ids in the merge consistent and unique.
	* If null or blank, the date/time string of the merge is used.
	*/
	public String merge(String fileName, String siteId)
	{
		StringBuffer results = new StringBuffer();

		File[] files = null;

		// see if the name is a directory
		File file = new File(m_storagePath + fileName);
		if ((file == null) || (!file.exists()))
		{
			results.append("file: " + file.getPath() + " not found.\n");
			m_logger.warn(this + ".merge(): file not found: " + file.getPath());
			return results.toString();
		}

		if (file.isDirectory())
		{
			files = file.listFiles();
		}
		else
		{
			files = new File[1];
			files[0] = file;
		}

		// track old to new attachment names
		Map attachmentNames = new HashMap();

		// see if there's a site definition
		for (int i = 0; i < files.length; i++)
		{
			if (files[i].getPath().indexOf("site.xml") != -1)
			{
				processMerge(files[i].getPath(), siteId, results, attachmentNames);
				files[i] = null;
				break;
			}
		}
				
		// see if there's a user definition
		for (int i = 0; i < files.length; i++)
		{
			if (files[i].getPath().indexOf("user.xml") != -1)
			{
				processMerge(files[i].getPath(), siteId, results, attachmentNames);
				files[i] = null;
				break;
			}
		}

		// see if there's an attachments definition
		for (int i = 0; i < files.length; i++)
		{
			if (files[i].getPath().indexOf("attachment.xml") != -1)
			{
				processMerge(files[i].getPath(), siteId, results, attachmentNames);
				files[i] = null;
				break;
			}
		}

		// process each remaining file that is an .xml file
		for (int i = 0; i < files.length; i++)
		{
			if (files[i] != null)
				if (files[i].getPath().endsWith(".xml"))
					processMerge(files[i].getPath(), siteId, results, attachmentNames);
		}

		return results.toString();

	}	// merge

	/**
	* Read in an archive file and merge the entries into the specified site.
	* @param fileName The site name (for the archive file) to read from.
	* @param siteId The id of the site to merge the content into.
	* @param results A buffer to accumulate result messages.
	* @param attachmentNames A map of old to new attachment names.
	*/
	protected void processMerge(String fileName, String siteId, StringBuffer results, Map attachmentNames)
	{
		// correct for windows backslashes
		fileName = fileName.replace('\\', '/');

		if (m_logger.isDebugEnabled())
			m_logger.debug(this + ".merge(): processing file: " + fileName);

		Site theSite = null;
		try
		{
			theSite = SiteService.getSite(siteId);
		}
		catch (IdUnusedException ignore) {}

		// read the whole file into a DOM
		Document doc = Xml.readDocument(fileName);
		if (doc == null)
		{
			results.append("Error reading xml from: " + fileName + "\n");
			m_logger.warn(this + ".processMerge(): problem reading doc: " + fileName);
			return;
		}

		// verify the root element
		Element root = doc.getDocumentElement();
		if (!root.getTagName().equals("archive"))
		{
			results.append("File: " + fileName + " does not contain archive xml.  Found this root tag: " + root.getTagName() + "\n");
			m_logger.warn(this + ".processMerge(): not an archive file: " + root.getTagName());
			return;
		}

		// get the from site id
		String fromSite = root.getAttribute("source");

		// the children
		NodeList children = root.getChildNodes();
		final int length = children.getLength();
		for(int i = 0; i < length; i++)
		{
			Node child = children.item(i);
			if (child.getNodeType() != Node.ELEMENT_NODE) continue;
			Element element = (Element)child;

			// look for site stuff
			if (element.getTagName().equals(SiteService.SERVICE_NAME))
			{
				// mergeSite(siteId, fromSite, element);
				
				// get the site, possible created
				try
				{
					theSite = SiteService.getSite(siteId);
				}
				catch (IdUnusedException ignore)
				{
					m_logger.warn(this + ".processMerge(): site not found after mergeSite: " + siteId);
				}
			}

			else
			{
				// we need a site now
				if (theSite == null)
				{
					m_logger.warn(this + ".processMerge(): site not found: " + siteId);
					results.append("Site: " + siteId + " not found.\n");
					return;
				}
				
				// get the service name
				String serviceName = translateServiceName(element.getTagName());

				// get the service
				ResourceService service = null;
				try
				{
					service = (ResourceService) ComponentManager.get(serviceName);
				}
				catch (Throwable t)
				{
					m_logger.warn(this + ".processMerge(): file: " + fileName + " exception while finding service node: " + serviceName + ": " + t);
					results.append("Did not recognize the resource service: " + serviceName + " in file: " + fileName + "\n");
				}			
				
				if (service != null)
				{
					try
					{
						String msg = service.merge(siteId, element, fileName, fromSite, attachmentNames);
						results.append(msg);
					}
					catch (Throwable t)
					{
						m_logger.warn(this + ".processMerge(): file: " + fileName + " exception while attempting service.merge(): " + serviceName + ": " + t);
						results.append("Exception occured processing file: " + fileName + "\n");					
					}
				}
			}
		}

	}	// processMerge

	/**
	* Merge the content attachment resources from the attachments part of the
	* archive file into the content hosting service.
	* Map the attachment folder to something new based on the mergeId.
	* @param mergeId The value pre-pended to the numeric attachment folder id from the archive.
	* to make it unique here.
	* @param root The XML DOM tree of content to merge.
	*/
/*
	protected void mergeAttachments(String mergeId, Element root)
	{
		try
		{
			ContentHostingService service = (ContentHostingService)TurbineServices.getInstance()
									.getService(ContentHostingService.SERVICE_NAME);

			NodeList children = root.getChildNodes();
			final int length = children.getLength();
			for(int i = 0; i < length; i++)
			{
				Node child = children.item(i);
				if (child.getNodeType() == Node.ELEMENT_NODE)
				{
					Element element = (Element)child;

					// for "resource" kids
					if (element.getTagName().equals("resource"))
					{
						// map the attachment area folder name
						String oldId = element.getAttribute("id");
						if (oldId.startsWith("/attachment/"))
						{
							String newId = "/attachment/"
								+ mergeId + "-"
								+ oldId.substring(12);
							element.setAttribute("id", newId);
						}

						// 	resource: add if missing
						service.mergeResource(element);
					}
				}
			}
		}
		catch (Exception any)
		{
			m_logger.warn(this + ".mergeAttachments(): exception: ", any);
		}
	}	// mergeAttachments
*/

	/**
	* Merge the site definition from the site part of the archive file into the site service.
	* Translate the id to the siteId.
	* @param siteId The id of the site getting imported into.
	* @param fromSiteId The id of the site the archive was made from.
	* @param element The XML DOM tree of messages to merge.
	*/
/*
	protected void mergeSite(String siteId, String fromSiteId, Element element)
	{
		try
		{
			Site site = null;
			try
			{
				site = SiteService.getSite(siteId);
			}
			catch (IdUnusedException e) {}

			NodeList children = element.getChildNodes();
			final int length = children.getLength();
			for(int i = 0; i < length; i++)
			{
				Node child = children.item(i);
				if (child.getNodeType() != Node.ELEMENT_NODE) continue;
				Element element2 = (Element)child;
				if (!element2.getTagName().equals("site")) continue;
	
				// adjust the site element's id
				String oldId = element2.getAttribute("id");
				element2.setAttribute("id", siteId);

				// if we have a site, find the node that has the roles and merge them in
				if (site != null)
				{
					NodeList children2 = element2.getChildNodes();
					final int length2 = children2.getLength();
					for(int i2 = 0; i2 < length2; i2++)
					{
						Node child2 = children2.item(i2);
						if (child2.getNodeType() != Node.ELEMENT_NODE) continue;
						Element element3 = (Element)child2;
						if (!element3.getTagName().equals("roles")) continue;
						
						// site.mergeRoles(element3); %%%
					}
				}

				// if we do not have a site, add it
				else
				{
					// %%%site = service.mergeSite(element2);
				}
			}
			
		}
		catch (Exception any)
		{
			m_logger.warn(this + ".mergeSite(): exception: ", any);
		}
	}	// mergeSite
*/

	/**
	* Merge the user definitions from the users part of the archive file into the user service.
	* @param element The XML DOM tree of messages to merge.
	*/
/*
	protected void mergeUsers(Element element)
	{
		try
		{
			NodeList children = element.getChildNodes();
			final int length = children.getLength();
			for(int i = 0; i < length; i++)
			{
				Node child = children.item(i);
				if (child.getNodeType() != Node.ELEMENT_NODE) continue;
				Element element2 = (Element)child;
				if (!element2.getTagName().equals("user")) continue;
	
				// merge in this user definition
				try
				{
					UserDirectoryService.mergeUser(element2);
				}
				catch (IdUsedException e) {}
				catch (IdInvalidException e) { m_logger.warn(this + ".mergeUsers(): exception: " + e); }
				catch (PermissionException e) { m_logger.warn(this + ".mergeUsers(): exception: " + e); }
			}
			
		}
		catch (Exception any)
		{
			m_logger.warn(this + ".mergeUsers(): exception: ", any);
		}
	}	// mergeUsers
*/

	/**
	 * Old archives have the old CHEF 1.2 service names...
	 */
	protected String translateServiceName(String name)
	{
		if ("org.chefproject.service.GenericContentHostingService".equals(name))
		{
			return ContentHostingService.class.getName();
		}
		
		return name;
	}

}	// BasicArchiveService

/**********************************************************************************
*
* $Header: /cvs/sakai/legacy-component/src/java/org/sakaiproject/component/legacy/archive/BasicArchiveService.java,v 1.15 2004/06/22 03:14:02 ggolden Exp $
*
**********************************************************************************/
