/**********************************************************************************
*
* $Header: /cvs/sakai/legacy-component/src/java/org/sakaiproject/component/legacy/chat/DbChatService.java,v 1.7 2004/06/22 03:13:57 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.component.legacy.chat;

// import
import java.util.List;

import org.sakaiproject.service.legacy.message.Message;
import org.sakaiproject.service.legacy.message.MessageChannel;
import org.sakaiproject.service.legacy.message.MessageChannelEdit;
import org.sakaiproject.service.legacy.message.MessageEdit;
import org.sakaiproject.service.framework.sql.SqlService;
import org.sakaiproject.util.storage.BaseDbDoubleStorage;
import org.sakaiproject.util.storage.StorageUser;

/**
* <p>DbCachedChatService fills out the BaseChatService with a database, jdbc, turbine connection
* pool, Oracle implementation.</p>
* <p>The sql scripts in src/sql/chef_chat.sql must be run on the default database turbine
* points to.</p>
* 
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.7 $
* @see org.chefproject.service.BaseChatService
*/
public class DbChatService
	extends BaseChatService
{
	/** The name of the db table holding chat channels. */
	protected String m_cTableName = "CHAT_CHANNEL";

	/** The name of the db table holding chat messages. */
	protected String m_rTableName = "CHAT_MESSAGE";

	/** If true, we do our locks in the remote database, otherwise we do them here. */
	protected boolean m_locksInDb = true;

	protected static final String[] FIELDS = { "MESSAGE_DATE" };

	/*******************************************************************************
	* Constructors, Dependencies and their setter methods
	*******************************************************************************/

	/** Dependency: SqlService */
	protected SqlService m_sqlService = null;

	/**
	 * Dependency: SqlService.
	 * @param service The SqlService.
	 */
	public void setSqlService(SqlService service)
	{
		m_sqlService = service;
	}

	/**
	 * Configuration: set the table name for the container.
	 * @param path The table name for the container.
	 */
	public void setContainerTableName(String name)
	{
		m_cTableName = name;
	}

	/**
	 * Configuration: set the table name for the resource.
	 * @param path The table name for the resource.
	 */
	public void setResourceTableName(String name)
	{
		m_rTableName = name;
	}

	/**
	 * Configuration: set the locks-in-db
	 * @param path The storage path.
	 */
	public void setLocksInDb(String value)
	{
		m_locksInDb = new Boolean(value).booleanValue();
	}

	/*******************************************************************************
	* Init and Destroy
	*******************************************************************************/

	/**
	 * Final initialization, once all dependencies are set.
	 */
	public void init()
	{
		try
		{
			super.init();

			m_logger.info(this +".init(): tables: " + m_cTableName + " " + m_rTableName + " locks-in-db: " + m_locksInDb);
		}
		catch (Throwable t)
		{
			m_logger.warn(this +".init(): ", t);
		}
	}

	/*******************************************************************************
	* BaseMessageService extensions
	*******************************************************************************/

	/**
	* Construct a Storage object.
	* @return The new storage object.
	*/
	protected Storage newStorage()
	{
		return new DbStorage(this);

	}	// newStorage

	/*******************************************************************************
	* Storage implementation
	*******************************************************************************/

	protected class DbStorage
		extends BaseDbDoubleStorage
		implements Storage
	{
		/**
		* Construct.
		* @param user The StorageUser class to call back for creation of Resource and Edit objects.
		*/
		public DbStorage(StorageUser user)
		{
			super(m_cTableName, "CHANNEL_ID", m_rTableName, "MESSAGE_ID", "CHANNEL_ID",
					"MESSAGE_DATE", FIELDS, m_locksInDb, "channel", "message", user, m_sqlService);

		}	// DbStorage

		/** Channels **/
		
		public boolean checkChannel(String ref) { return super.getContainer(ref) != null; }

		public MessageChannel getChannel(String ref) { return (MessageChannel) super.getContainer(ref); }

		public List getChannels() { return super.getAllContainers(); }

		public MessageChannelEdit putChannel(String ref)
			{ return (MessageChannelEdit) super.putContainer(ref); }

		public MessageChannelEdit editChannel(String ref)
			{ return (MessageChannelEdit) super.editContainer(ref); }

		public void commitChannel(MessageChannelEdit edit)
			{ super.commitContainer(edit); }

		public void cancelChannel(MessageChannelEdit edit)
			{ super.cancelContainer(edit); }

		public void removeChannel(MessageChannelEdit edit)
			{ super.removeContainer(edit); }

		/** messages **/
		
		public boolean checkMessage(MessageChannel channel, String id)
			{ return super.checkResource(channel, id); }

		public Message getMessage(MessageChannel channel, String id)
			{ return (Message) super.getResource(channel, id); }

		public List getMessages(MessageChannel channel) { return super.getAllResources(channel); }

		public MessageEdit putMessage(MessageChannel channel,String id)
			{ return (MessageEdit) super.putResource(channel, id, null); }

		public MessageEdit editMessage(MessageChannel channel, String id)
			{ return (MessageEdit) super.editResource(channel, id); }

		public void commitMessage(MessageChannel channel, MessageEdit edit)
			{ super.commitResource(channel, edit); }

		public void cancelMessage(MessageChannel channel, MessageEdit edit)
			{ super.cancelResource(channel, edit); }

		public void removeMessage(MessageChannel channel, MessageEdit edit)
			{ super.removeResource(channel, edit); }

	}   // DbStorage

}	// DbCachedChatService

/**********************************************************************************
*
* $Header: /cvs/sakai/legacy-component/src/java/org/sakaiproject/component/legacy/chat/DbChatService.java,v 1.7 2004/06/22 03:13:57 ggolden Exp $
*
**********************************************************************************/
