/**********************************************************************************
*
* $Header: /cvs/sakai/legacy-component/src/java/org/sakaiproject/component/legacy/notification/SiteEmailNotificationContent.java,v 1.5 2004/06/22 03:14:00 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.component.legacy.notification;

// imports
import org.sakaiproject.service.framework.config.cover.ServerConfigurationService;
import org.sakaiproject.service.legacy.content.cover.ContentHostingService;
import org.sakaiproject.service.legacy.event.Event;
import org.sakaiproject.service.legacy.notification.NotificationAction;
import org.sakaiproject.service.legacy.resource.Reference;
import org.sakaiproject.service.legacy.resource.Resource;
import org.sakaiproject.service.legacy.resource.ResourceProperties;
import org.sakaiproject.service.legacy.site.Site;
import org.sakaiproject.service.legacy.site.cover.SiteService;

/**
* <p>SiteEmailNotificationContent fills the notification message with details from the content change that triggered
* the notification event.</p>
* 
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.5 $
*/
public class SiteEmailNotificationContent
	extends SiteEmailNotification
{
	/**
	* Construct.
	*/
	public SiteEmailNotificationContent() {}

	/**
	* Construct.
	*/
	public SiteEmailNotificationContent(String siteId)
	{
		super(siteId);

	}	// SiteEmailNotificationContent

	/**
	* Make a new one like me.
	* @return A new action just like me.
	*/
	public NotificationAction getClone()
	{
		SiteEmailNotificationContent clone = new SiteEmailNotificationContent();
		clone.set(this);

		return clone;

	}	// getClone

	/**
	* Get the from address for the email.
	* @param event The event that matched criteria to cause the notification.
	* @return the from address for the email.
	*/
	protected String getFrom(Event event)
	{
		// get the content & properties
		Reference ref = new Reference(event.getResource());
		Resource r = ref.getResource();
		ResourceProperties props = ref.getProperties();

		// make it from the message's creator		
		try
		{
			return props.getUserProperty(ResourceProperties.PROP_MODIFIED_BY).getEmail();
		}
		catch (Throwable e)
		{
			return "postmaster@" + ServerConfigurationService.getServerName();
		}

	}	// getFrom

	/**
	* Get the subject for the email.
	* @param event The event that matched criteria to cause the notification.
	* @return the subject for the email.
	*/
	public String getSubject(Event event)
	{
		Reference ref = new Reference(event.getResource());
		Resource r = ref.getResource();
		ResourceProperties props = ref.getProperties();

		// get the function
		String function = event.getEvent();

		// use either the configured site, or if not configured, the site (context) of the resource
		String siteId = (getSite() != null) ? getSite() : ref.getContext();

		// get a site title
		String title = siteId;
		try
		{
			Site site = SiteService.getSite(siteId);
			title = site.getTitle();
		}
		catch (Exception ignore) {}

		// use the message's subject
		return "[ " + title + " - Resources ]   "
			+ (ContentHostingService.EVENT_RESOURCE_ADD.equals(function) ? "New: " : "Changed: ")
			+ ref.getReference();

	}	// getSubject

	/**
	* Get the message for the email.
	* @param event The event that matched criteria to cause the notification.
	* @return the message for the email.
	*/
	protected String getMessage(Event event)
	{
		StringBuffer buf = new StringBuffer();

		// get the content & properties
		Reference ref = new Reference(event.getResource());
		Resource r = ref.getResource();
		ResourceProperties props = ref.getProperties();

		// get the function
		String function = event.getEvent();

		// use either the configured site, or if not configured, the site (context) of the resource
		String siteId = (getSite() != null) ? getSite() : ref.getContext();

		// get a site title
		String title = siteId;
		try
		{
			Site site = SiteService.getSite(siteId);
			title = site.getTitle();
		}
		catch (Exception ignore) {}

		buf.append("The following resource has been "
			+ (ContentHostingService.EVENT_RESOURCE_ADD.equals(function) ? "added " : "updated ")
			+ "in the " + title + " site at " + ServerConfigurationService.getServerUrl() + "\n");

		// resource title
		buf.append("\nTitle: " + props.getPropertyFormatted(ResourceProperties.PROP_DISPLAY_NAME) + "\n");

		// resource description
		buf.append("\nDescription:\n\n" + props.getPropertyFormatted(ResourceProperties.PROP_DESCRIPTION) + "\n");

		// add a reference to the resource
		buf.append("\nAccess this resource at the following URL:\n\n" + ref.getUrl() + "\n");

		return buf.toString();

	}	// getMessage

}   // SiteEmailNotificationContent

/**********************************************************************************
*
* $Header: /cvs/sakai/legacy-component/src/java/org/sakaiproject/component/legacy/notification/SiteEmailNotificationContent.java,v 1.5 2004/06/22 03:14:00 ggolden Exp $
*
**********************************************************************************/
