/**********************************************************************************
*
* $Header: /cvs/sakai/sakai-component/src/java/org/sakaiproject/component/common/agent/TestAgentManager.java,v 1.3 2004/06/22 03:14:25 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

package org.sakaiproject.component.common.agent;

import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.exception.PermissionException;
import org.sakaiproject.exception.VersionException;
import org.sakaiproject.service.common.agent.Agent;
import org.sakaiproject.service.common.agent.AgentIterator;
import org.sakaiproject.service.common.agent.AgentManager;
import org.sakaiproject.service.common.agent.User;
import org.sakaiproject.service.common.id.Id;
import org.sakaiproject.service.common.id.IdManager;
import org.sakaiproject.service.common.shared.Properties;
import org.sakaiproject.service.common.shared.PropertiesIterator;
import org.sakaiproject.service.common.shared.Type;
import org.sakaiproject.service.common.shared.TypeIterator;
import org.sakaiproject.service.common.shared.Version;
import org.sakaiproject.service.framework.log.Logger;

/**
 * <p>TestAgentManager is a test implementation of the AgentManager.</p>
 * 
 * @author University of Michigan, CHEF Software Development Team
 * @version $Revision: 1.3 $
 */
public class TestAgentManager implements AgentManager
{
	/*******************************************************************************
	* Dependencies and their setter methods
	*******************************************************************************/

	/** Dependency: IdManager */
	protected IdManager m_idManager = null;

	/**
	 * Dependency: IdManager.
	 * @param service The IdManager
	 */
	public void setIdManager(IdManager service)
	{
		m_idManager = service;
	}

	/** Dependency: logging service */
	protected Logger m_logger = null;

	/**
	 * Dependency: logging service.
	 * @param service The logging service.
	 */
	public void setLogger(Logger service)
	{
		m_logger = service;
	}

	/*******************************************************************************
	* Init and Destroy
	*******************************************************************************/

	/**
	 * Final initialization, once all dependencies are set.
	 */
	public void init()
	{
		m_logger.info(this +".init()");
	}

	/**
	 * Final cleanup.
	 */
	public void destroy()
	{
		m_logger.info(this +".destroy()");
	}

	/*******************************************************************************
	* Work interface methods: org.sakaiproject.service.common.agent.AgentManager
	*******************************************************************************/

	/**
	 * {@inheritDoc}
	 */
	public boolean allowCreateGroup()
	{
		return false;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean allowCreateUser()
	{
		return false;
	}

	/**
	 * {@inheritDoc}
	 */
	public User createUser() throws PermissionException
	{
		throw new PermissionException("","","");
	}

	/**
	 * {@inheritDoc}
	 */
	public User getAnonymousUser()
	{
		return new MyUser("", "", "", "");
	}

	/**
	 * {@inheritDoc}
	 */
	public AgentIterator getIteratorOnArray(Agent[] agents)
	{
		return new MyAgentIterator(Arrays.asList(agents).iterator());
	}

	/**
	 * {@inheritDoc}
	 */
	public AgentIterator getIteratorOnCollection(Collection agents)
	{
		return new MyAgentIterator(agents.iterator());
	}

	/**
	 * {@inheritDoc}
	 */
	public User getUser(Id id) throws IdUnusedException
	{
		Id testId = m_idManager.getId("admin");
		if (id.equals(testId))
			return new MyUser("admin", "admin@sakaiproject.org", "Sakai", "Administrator");

		testId = m_idManager.getId("007");
		if (id.equals(testId))
			return new MyUser("007", "bond@mi6.org", "James", "Bond");

		testId = m_idManager.getId("1492");
		if (id.equals(testId))
			return new MyUser("1492", "sailer@isabella.org", "Christopher", "Colombus");

		throw new IdUnusedException(id.getIdString());
	}

	/**
	 * {@inheritDoc}
	 */
	public User getUserByEmail(String email) throws IdUnusedException
	{
		if (email.equals("admin@sakaiproject.org"))
			return new MyUser("admin", "admin@sakaiproject.org", "Sakai", "Administrator");

		if (email.equals("bond@mi6.org"))
			return new MyUser("007", "bond@mi6.org", "James", "Bond");

		if (email.equals("sailer@isabella.org"))
			return new MyUser("1492", "sailer@isabella.org", "Christopher", "Colombus");

		throw new IdUnusedException(email);
	}

	/**
	 * {@inheritDoc}
	 */
	public AgentIterator getUsers()
	{
		Vector rv = new Vector();
		rv.add(new MyUser("admin", "admin@sakaiproject.org", "Sakai", "Administrator"));
		rv.add(new MyUser("007", "bond@mi6.org", "James", "Bond"));
		rv.add(new MyUser("1492", "sailer@isabella.org", "Christopher", "Colombus"));

		return new MyAgentIterator(rv.iterator());
	}

	/**
	 * {@inheritDoc}
	 */
	public AgentIterator getUsersBySearch(Object searchCriteria, Type agentSearchType)
	{
		return getUsers();
	}

	/**
	 * {@inheritDoc}
	 */
	public AgentIterator getUsersByType(Type agentType)
	{
		return getUsers();
	}

	/**
	 * {@inheritDoc}
	 */
	public TypeIterator getUserSearchTypes()
	{
		Vector rv = new Vector();
		return new MyTypeIterator(rv);
	}

	/**
	 * {@inheritDoc}
	 */
	public TypeIterator getUserTypes()
	{
		Vector rv = new Vector();
		return new MyTypeIterator(rv);
	}

	/**
	 * <p>MyAgentIterator implements AgentIterator.</p>
	 */
	public class MyAgentIterator implements AgentIterator
	{
		/** The wrapped iterator. */
		protected Iterator m_iterator = null;

		public MyAgentIterator(Iterator it)
		{
			m_iterator = it;
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean hasNext()
		{
			return m_iterator.hasNext();
		}

		/**
		 * {@inheritDoc}
		 */
		public Object next()
		{
			return m_iterator.next();
		}

		/**
		 * {@inheritDoc}
		 */
		public Agent nextAgent()
		{
			return (Agent) m_iterator.next();
		}

		/**
		 * {@inheritDoc}
		 */
		public void remove()
		{
			m_iterator.remove();
		}
	}

	/**
	 * <p>MyTypeIterator implements TypeIterator.</p>
	 */
	public class MyTypeIterator implements TypeIterator
	{
		/** The wrapped iterator. */
		protected Iterator m_iterator = null;

		public MyTypeIterator(List list)
		{
			m_iterator = list.iterator();
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean hasNext()
		{
			return m_iterator.hasNext();
		}

		/**
		 * {@inheritDoc}
		 */
		public Object next()
		{
			return m_iterator.next();
		}

		/**
		 * {@inheritDoc}
		 */
		public Type nextType()
		{
			return (Type) m_iterator.next();
		}

		/**
		 * {@inheritDoc}
		 */
		public void remove()
		{
			m_iterator.remove();
		}
	}

	public class MyUser implements User
	{
		protected String m_email = null;
		protected String m_first = null;

		protected Id m_id = null;
		protected String m_last = null;

		public MyUser(Id id)
		{
			m_id = id;
		}

		public MyUser(String id, String email, String first, String last)
		{
			m_id = m_idManager.getId(id);
			m_email = email;
			m_first = first;
			m_last = last;
		}

		/**
		 * {@inheritDoc}
		 */
		public Properties addPropertiesType(Type propertiesType)
		{
			// TODO Auto-generated method stub
			return null;
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean allowDelete()
		{
			return false;
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean allowUpdate()
		{
			return false;
		}

		/**
		 * {@inheritDoc}
		 */
		public int compareTo(Object o)
		{
			// TODO Auto-generated method stub
			return 0;
		}

		/**
		 * {@inheritDoc}
		 */
		public void delete() throws PermissionException
		{
			throw new PermissionException("", "", "");
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean equals(Object obj)
		{
			if (!(obj instanceof MyUser))
				return false;

			return m_id.equals(((MyUser) obj).m_id);
		}

		/**
		 * {@inheritDoc}
		 */
		public String getDescription()
		{
			return getDisplayName();
		}

		/**
		 * {@inheritDoc}
		 */
		public String getDisplayName()
		{
			return m_first + " " + m_last;
		}

		/**
		 * {@inheritDoc}
		 */
		public String getEmail()
		{
			return m_email;
		}

		/**
		 * {@inheritDoc}
		 */
		public String getFirstName()
		{
			return m_first;
		}

		/**
		 * {@inheritDoc}
		 */
		public Id getId()
		{
			return m_id;
		}

		/**
		 * {@inheritDoc}
		 */
		public String getLastName()
		{
			return m_last;
		}

		/**
		 * {@inheritDoc}
		 */
		public PropertiesIterator getProperties()
		{
			// TODO Auto-generated method stub
			return null;
		}

		/**
		 * {@inheritDoc}
		 */
		public Properties getPropertiesByType(Type propertiesType)
		{
			// TODO Auto-generated method stub
			return null;
		}

		/**
		 * {@inheritDoc}
		 */
		public TypeIterator getPropertiesTypes()
		{
			// TODO Auto-generated method stub
			return null;
		}

		/**
		 * {@inheritDoc}
		 */
		public TypeIterator getPropertyTypes()
		{
			// TODO Auto-generated method stub
			return null;
		}

		/**
		 * {@inheritDoc}
		 */
		public String getReference()
		{
			return "";
		}

		/**
		 * {@inheritDoc}
		 */
		public Type getType()
		{
			// TODO Auto-generated method stub
			return null;
		}

		/**
		 * {@inheritDoc}
		 */
		public String getUrl()
		{
			return "";
		}

		/**
		 * {@inheritDoc}
		 */
		public Version getVersion()
		{
			// TODO Auto-generated method stub
			return null;
		}

		/**
		 * {@inheritDoc}
		 */
		public int hashCode()
		{
			return m_id.hashCode();
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean isCurrentVersion()
		{
			// TODO Auto-generated method stub
			return false;
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean isCurrentVersion(Version version)
		{
			// TODO Auto-generated method stub
			return false;
		}

		/**
		 * {@inheritDoc}
		 */
		public void removePropertiesType(Type propertiesType)
		{
			// TODO Auto-generated method stub

		}

		/**
		 * {@inheritDoc}
		 */
		public void setDescription(String description)
		{
		}

		/**
		 * {@inheritDoc}
		 */
		public void setDisplayName(String displayName)
		{
		}

		/**
		 * {@inheritDoc}
		 */
		public void setEmail(String email)
		{
			m_email = email;
		}

		/**
		 * {@inheritDoc}
		 */
		public void setFirstName(String name)
		{
			m_first = name;
		}

		/**
		 * {@inheritDoc}
		 */
		public void setLastName(String name)
		{
			m_last = name;
		}

		/**
		 * {@inheritDoc}
		 */
		public void setType(Type type)
		{
		}

		/**
		 * {@inheritDoc}
		 */
		public void update() throws VersionException, PermissionException
		{
			throw new PermissionException("","","");
		}

		/**
		 * {@inheritDoc}
		 */
		public void updateIgnoreVersion() throws PermissionException
		{
			throw new PermissionException("","","");
		}
	}
}

/**********************************************************************************
*
* $Header: /cvs/sakai/sakai-component/src/java/org/sakaiproject/component/common/agent/TestAgentManager.java,v 1.3 2004/06/22 03:14:25 ggolden Exp $
*
**********************************************************************************/
