/**********************************************************************************
*
* $Header: /cvs/sakai/sakai-component/src/java/org/sakaiproject/component/common/id/SakaiIdManager.java,v 1.2 2004/06/22 03:14:25 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

package org.sakaiproject.component.common.id;

import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;

import org.sakaiproject.service.common.id.Id;
import org.sakaiproject.service.common.id.IdIterator;
import org.sakaiproject.service.common.id.IdManager;
import org.sakaiproject.service.framework.log.Logger;
import org.sakaiproject.service.legacy.id.IdService;

/**
 * <p>SakaiIdManager implements the Sakai common id service's IdManager.</p>
 * 
 * @author University of Michigan, CHEF Software Development Team
 * @version $Revision: 1.2 $
 */
public class SakaiIdManager implements IdManager
{
	/*******************************************************************************
	* Constructors, Dependencies and their setter methods
	*******************************************************************************/

	/** Dependency: logging service */
	protected Logger m_logger = null;

	/**
	 * Dependency: logging service.
	 * @param service The logging service.
	 */
	public void setLogger(Logger service)
	{
		m_logger = service;
	}

	/** Dependency: IdService. */
	protected IdService m_IdService = null;

	/**
	 * Dependency: IdService.
	 * @param manager The IdService.
	 */
	public void setIdService(IdService manager)
	{
		m_IdService = manager;
	}

	/*******************************************************************************
	* Init and Destroy
	*******************************************************************************/

	/**
	 * Final initialization, once all dependencies are set.
	 */
	public void init()
	{
		m_logger.info(this +".init()");
	}

	/**
	* Returns to uninitialized state.
	*/
	public void destroy()
	{
		m_logger.info(this +".destroy()");
	}

	/*******************************************************************************
	* IdManager implementation
	*******************************************************************************/

	/**
	 * {@inheritDoc}
	 */
	public Id createId()
	{
		return new SakaiId();
	}

	/**
	 * {@inheritDoc}
	 */
	public Id getId(String idString)
	{
		return new SakaiId(idString);
	}

	/**
	 * {@inheritDoc}
	 */
	public IdIterator getIteratorOnCollection(Collection ids)
	{
		return new SakaiIdIterator(ids.iterator());
	}

	/**
	 * {@inheritDoc}
	 */
	public IdIterator getIteratorOnArray(Id[] ids)
	{
		return new SakaiIdIterator(Arrays.asList(ids).iterator());
	}

	/**
	 * <p>SaiaiId implements the entity Id object</p>
	 */
	public class SakaiId implements Id
	{
		/** The string representation. */
		protected String m_id = null;

		/**
		 * Construct and assign a unique Id.
		 */
		public SakaiId()
		{
			m_id = m_IdService.getUniqueId();
		}

		/**
		 * Construct with an already generated "unique" id.
		 * @param id The id string.
		 */
		public SakaiId(String id)
		{
			m_id = id;
		}

		/**
		 * {@inheritDoc}
		 */
		public String getIdString()
		{
			return m_id;
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean equals(Object obj)
		{
			if (!(obj instanceof SakaiId))
				return false;

			return m_id.equals(((SakaiId) obj).m_id);
		}

		/**
		 * {@inheritDoc}
		 */
		public int hashCode()
		{
			return m_id.hashCode();
		}
	}

	public class SakaiIdIterator implements IdIterator
	{
		/** The iterator of Ids. */
		protected Iterator m_iterator = null;

		public SakaiIdIterator(Iterator iterator)
		{
			m_iterator = iterator;
		}

		/**
		 * {@inheritDoc}
		 */
		public Id nextId()
		{
			return (Id) m_iterator.next();
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean hasNext()
		{
			return m_iterator.hasNext();
		}

		/**
		 * {@inheritDoc}
		 */
		public Object next()
		{
			return m_iterator.next();
		}

		/**
		 * {@inheritDoc}
		 */
		public void remove()
		{
		}
	}
}

/**********************************************************************************
*
* $Header: /cvs/sakai/sakai-component/src/java/org/sakaiproject/component/common/id/SakaiIdManager.java,v 1.2 2004/06/22 03:14:25 ggolden Exp $
*
**********************************************************************************/
