/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/common/authentication/Authentication.java,v 1.2 2004/06/22 03:14:42 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

package org.sakaiproject.service.common.authentication;

import org.sakaiproject.service.common.id.Id;
import org.sakaiproject.service.common.shared.Resource;

/**
 * <p>Authentication is a record of a set of authentication data a user might use to authenticate.</p>
 * 
 * @author University of Michigan, CHEF Software Development Team
 * @version $Revision: 1.2 $
 */
public interface Authentication extends Resource
{
	/**
	 * Access the AgentId.
	 * @return The AgentId.
	 */
	Id getAgentId();

	/**
	 * Access the password.
	 * @return The password.
	 */
	String getPw();

	/**
	 * Access the id offered by the user to authenticate.
	 * @return The user name.
	 */
	String getUserName();

	/**
	 * Set the agent Id.
	 * @param id The agent Id
	 */
	void setAgentId(Id id);

	/**
	 * Set the password.
	 * @param pw The clear text password.
	 */
	void setPw(String pw);

	/**
	 * Set the user name.
	 * @param userId The user name.
	 */
	void setUserName(String userName);
}

/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/common/authentication/Authentication.java,v 1.2 2004/06/22 03:14:42 ggolden Exp $
*
**********************************************************************************/
