/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/common/id/IdManager.java,v 1.3 2004/06/22 03:14:47 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

package org.sakaiproject.service.common.id;

import java.util.Collection;

/**
 * <p>IdManager manages the creation of globally unique permanent identifiers.</p>
 * <p>The Id object is used to model the identifiers.  The identifiers can be assigned by the service,
 * or provided by the service user.  If provided, the user is responsible for assuring that the identifiers
 * provided and those generated by the service form a set of unique values.</p>
 * 
 * @author University of Michigan, Sakai Software Development Team
 * @version $Revision: 1.3 $
 */
public interface IdManager
{
	/**
	 * Return an Id with a newly created unique String representation.
	 * @return An Id with a newly created unique String representation.
	 */
	Id createId();

	/**
	 * Return an Id with the passed in String representation.
	 * Caller is responsible for assuring uniqueness of the String representation.
	 * @param idString The String representation for this Id.
	 * @return Id An Id with this String representation.
	 */
	Id getId(String idString);

	/**
	 * Return an IdIterator based on a complete set of Id objects in a Collection.
	 * @param ids The collection of Id objects.
	 * @return an IdIterator over the Id objects in the collection.
	 */
	IdIterator getIteratorOnCollection(Collection ids);

	/**
	 * Return an IdIterator based on a complete set of Id objects in an Array.
	 * @param ids The array of Id objects.
	 * @return an IdIterator over the Id objects in the array.
	 */
	IdIterator getIteratorOnArray(Id[] ids);
}

/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/common/id/IdManager.java,v 1.3 2004/06/22 03:14:47 ggolden Exp $
*
**********************************************************************************/
