/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/common/shared/Properties.java,v 1.3 2004/06/22 03:14:50 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

package org.sakaiproject.service.common.shared;

import java.util.Iterator;

/**
* <p>Properties are a Typed set of key=value pairs.</p>
* 
* @author University of Michigan, Sakai Software Development Team
* @version $Revision: 1.3 $
*/
public interface Properties
{
	/**
	 * Remove all properties
	 */
	void clear();

	/**
	 * Access all the keys of defined properties.
	 * @return An Iterator listing all the keys of defined properties.
	 */
	Iterator getKeys();

	/**
	 * Access the property associated with this key.
	 * @param key The property key.
	 * @return The property associated with this key, or null if not found.
	 */
	Object getProperty(Object key);

	/**
	 * Access the property type.
	 * @return The property type.
	 */
	Type getType();

	/**
	 * Remove the property associated with this key.
	 * @param key The property key.
	 */
	void removeProperty(Object key);

	/**
	 * Set the property associated with this key.
	 * @param key The property key.
	 * @param value The property value.
	 */
	void setProperty(Object key, Object value);

	/**
	 * Set the property type.
	 * @param type The new property type for the properties.
	 */
	void setType(Type type);
}

/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/common/shared/Properties.java,v 1.3 2004/06/22 03:14:50 ggolden Exp $
*
**********************************************************************************/
