/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/common/shared/Type.java,v 1.3 2004/06/22 03:14:50 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

package org.sakaiproject.service.common.shared;

/**
* <p>Type represents a resource type.</p>
* <p>A type is described with a hierarchy of three strings:
* domain
* -- authority
* -- -- keyword
* and a "displayName" sort of description</p>
* 
* @author University of Michigan, Sakai Software Development Team
* @version $Revision: 1.3 $
*/
public interface Type
{
	/**
	 * Access the authority part of the Type.
	 * @return The type authority.
	 */
	String getAuthority();

	/**
	 * Access the description part of the Type.
	 * @return The type description.
	 */
	String getDescription();

	/**
	 * Get the domain part of this Type.
	 * @return The type domain.
	 */
	String getDomain();

	/**
	 * Access the keyword part of the Type.
	 * @return The type keyword.
	 */
	String getKeyword();
}

/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/common/shared/Type.java,v 1.3 2004/06/22 03:14:50 ggolden Exp $
*
**********************************************************************************/
