/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/framework/session/UsageSessionService.java,v 1.4 2004/06/22 03:14:52 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.service.framework.session;

// imports
import org.sakaiproject.service.legacy.user.User;

/**
* <p>UsageSessionService is the Interface for CHEF services that keep track of usage sessions.</p>
* 
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.4 $
*/
public interface UsageSessionService
{
	/** This string can be used to find the service in the service manager. */
	public static final String SERVICE_NAME = UsageSessionService.class.getName();

	/** Name for the event of logging in. */
	public static final String EVENT_LOGIN = "user.login";

	/** Name for the event of logging out. */
	public static final String EVENT_LOGOUT = "user.logout";

	/**
	* Authenticate, and if successful, set as the session's authenticated user.
	* @param id The user id.
	* @param pw The password.
	* @param remoteAddress The IP address from the user is making a request.
	* @param userAgent The string describing the user's browser.
	* @return true if successfully authenticated, false if not.
	*/
	public boolean loginSessionUser(String id, String pw, String remoteAddress, String userAgent);

	/**
	* Logout the session user
	*/
	public void logoutSessionUser();

	/**
	* Accept a user id as the "user" for the current (non-session) thread.
	* @param id The user id.
	*/
	public void loginThreadUser(String id);

	/**
	* Remove the user for this thread.
	*/
	public void logoutThreadUser();

	/**
	* Access the authenticated session user for the current request's session.
	* If the current thread is not associated with a request and thus a session, return the
	* thread user, if any.
	* @return The User object for the current authenticated user (or thread user), or the anon. user if none.
	*/
	public User getSessionUser();

	/**
	* Access all the information about this session.
	* @return The UsageSession object holding the information about this session.
	*/
	public UsageSession getSession();

	/**
	* Access all the information about a session.
	* @param id the Session id.
	* @return The UsageSession object holding the information about a session.
	*/
	public UsageSession getSession(String id);

	/**
	* Is this session a local one?
	* @param id The session id.
	* @return true if the session with this id exists and is locally maintained, false if not.
	*/
	public boolean isLocalSession(String id);

}	// UsageSessionService

/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/framework/session/UsageSessionService.java,v 1.4 2004/06/22 03:14:52 ggolden Exp $
*
**********************************************************************************/
