/**********************************************************************************
*
* $Header: /cvs/sakai/util/src/java/org/sakaiproject/util/PresenceUtil.java,v 1.2 2004/06/22 03:15:26 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

package org.sakaiproject.util;

import org.sakaiproject.util.StringUtil;

/**
 * <p>PresenceUtil extracts some presence keeping logic for use in places where we don't want
 * startup dependencies on the PresenceService.  The service impls. and these impls. can share
 * this logic.</p>
 * 
 * @author University of Michigan, Sakai Software Development Team
 * @version $Revision: 1.2 $
 */
public class PresenceUtil
{
	/**
	 * Form a part of a location string - use a "*" if it's not present
	 * @param el the part.
	 * @return the part, or "*" if it's null.
	 */
	static public String locElement(String el)
	{
		if (el != null) return el;
		return "*";
	}

	/**
	 * {@inheritDoc}
	 */
	static public String locationId(String site, String page, String tool)
	{
		return locElement(site) + "/" + locElement(page) + "/" + locElement(tool);
	}

	/**
	 * {@inheritDoc}
	 */
	static public String[] parseLocation(String location)
	{
		String[] parsed = StringUtil.split(location, "/");
		String[] rv = new String[3];
		rv[0] = "*".equals(parsed[0]) ? null : parsed[0];
		rv[1] = "*".equals(parsed[1]) ? null : parsed[1];
		rv[2] = "*".equals(parsed[2]) ? null : parsed[2];

		return rv;
	}
}

/**********************************************************************************
*
* $Header: /cvs/sakai/util/src/java/org/sakaiproject/util/PresenceUtil.java,v 1.2 2004/06/22 03:15:26 ggolden Exp $
*
**********************************************************************************/
