/**********************************************************************************
*
* $Header: /cvs/sakai/velocity/src/java/org/sakaiproject/vm/VmServlet.java,v 1.3 2004/06/22 03:15:41 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

package org.sakaiproject.vm;

import java.io.IOException;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
* <p>VmServlet is a Servlet that makes use of the Velocity Template Engine.</p>
* <p>This extends our ComponentServlet, giving us also the ability to find registered service components.</p>
* 
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.3 $
*/
public abstract class VmServlet extends ComponentServlet
{
// TODO: validator?
//	/** A validator. */
//	protected final Validator m_validator = new Validator();

	/**
	 * Add a reference object to the velocity context by name.
	 * @param name The reference name.
	 * @param value The reference value object.
	 */
	public void setVmReference(String name, Object value, HttpServletRequest request)
	{
		request.setAttribute(name, value);
	}

	/**
	 * Add some standard references to the vm context.
	 * @param request The request.
	 * @param response The response.
	 */
	protected void setVmStdRef(HttpServletRequest request, HttpServletResponse response)
	{
		// include some standard references

		setVmReference("sakai_ActionURL", getActionURL(request), request);

//		TODO: validator?
//		setVmReference("sakai_Validator", m_validator, request);

		// add our fragment flag (false)
		setVmReference("sakai_fragment", Boolean.FALSE, request);
	}

	/**
	 * Include the Velocity template, expanded with the current set of references
	 * @param template The path, relative to the webapp context, of the template file
	 * @param request The render request.
	 * @param response The render response.
	 * @throws PortletException if something goes wrong.
	 */
	protected void includeVm(String template, HttpServletRequest request, HttpServletResponse response)
		throws ServletException
	{
		// include some standard references
		setVmStdRef(request, response);

		response.setContentType("text/html");

		try
		{
			RequestDispatcher dispatcher = getServletContext().getRequestDispatcher(template);
			dispatcher.include(request, response);
		}
		catch (IOException e)
		{
			throw new ServletException("includeVm: template: " + template, e);
		}
	}

	/**
	 * Get a new ActionURL.
	 * @param req The current request.
	 * @return A new ActionURL.
	 */
	protected ActionURL getActionURL(HttpServletRequest request)
	{
		// form the URL to this servlet (not including the path or query of this request)
		StringBuffer base = request.getRequestURL();
		String path = request.getPathInfo();
		if (path != null)
		{
			base.setLength(base.length() - path.length());
		}

		ActionURL a = new ActionURL(base.toString());
		
		// set the pid and panel, if present in the request
		a.setPid(request.getParameter(ActionURL.PARAM_PID));
		a.setPanel(request.getParameter(ActionURL.PARAM_PANEL));
		a.setSite(request.getParameter(ActionURL.PARAM_SITE));
		a.setPage(request.getParameter(ActionURL.PARAM_PAGE));

		return a;
	}
}

/**********************************************************************************
*
* $Header: /cvs/sakai/velocity/src/java/org/sakaiproject/vm/VmServlet.java,v 1.3 2004/06/22 03:15:41 ggolden Exp $
*
**********************************************************************************/
