/**********************************************************************************
*
* $Header: /cvs/sakai/chef-tool/src/java/org/sakaiproject/tool/assignment/AssignmentAction.java,v 1.13 2004/06/22 03:04:56 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.tool.assignment;

import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.sakaiproject.cheftool.Context;
import org.sakaiproject.cheftool.JetspeedRunData;
import org.sakaiproject.cheftool.NewPagedResourceAction;
import org.sakaiproject.cheftool.ParameterParser;
import org.sakaiproject.cheftool.RunData;
import org.sakaiproject.cheftool.VelocityPortlet;
import org.sakaiproject.cheftool.menu.Menu;
import org.sakaiproject.cheftool.menu.MenuEntry;
import org.sakaiproject.cheftool.menu.MenuItem;
import org.sakaiproject.exception.IdInvalidException;
import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.exception.IdUsedException;
import org.sakaiproject.exception.InUseException;
import org.sakaiproject.exception.PermissionException;
import org.sakaiproject.service.legacy.announcement.AnnouncementChannel;
import org.sakaiproject.service.legacy.announcement.AnnouncementMessage;
import org.sakaiproject.service.legacy.announcement.AnnouncementService;
import org.sakaiproject.service.legacy.assignment.Assignment;
import org.sakaiproject.service.legacy.assignment.AssignmentContentEdit;
import org.sakaiproject.service.legacy.assignment.AssignmentEdit;
import org.sakaiproject.service.legacy.assignment.AssignmentSubmission;
import org.sakaiproject.service.legacy.assignment.AssignmentSubmissionEdit;
import org.sakaiproject.service.legacy.assignment.cover.AssignmentService;
import org.sakaiproject.service.legacy.calendar.Calendar;
import org.sakaiproject.service.legacy.calendar.CalendarEvent;
import org.sakaiproject.service.legacy.calendar.CalendarService;
import org.sakaiproject.service.framework.config.cover.ServerConfigurationService;
import org.sakaiproject.service.legacy.content.ContentTypeImageService;
import org.sakaiproject.service.framework.portal.cover.PortalService;
import org.sakaiproject.service.legacy.resource.Reference;
import org.sakaiproject.service.legacy.resource.ReferenceVector;
import org.sakaiproject.service.legacy.resource.ResourceProperties;
import org.sakaiproject.service.legacy.resource.ResourcePropertiesEdit;
import org.sakaiproject.service.framework.session.cover.UsageSessionService;
import org.sakaiproject.service.legacy.site.cover.SiteService;
import org.sakaiproject.service.framework.state.SessionState;
import org.sakaiproject.service.legacy.time.Time;
import org.sakaiproject.service.legacy.time.TimeBreakdown;
import org.sakaiproject.service.legacy.time.cover.TimeService;
import org.sakaiproject.service.legacy.user.User;
import org.sakaiproject.service.legacy.user.UserDirectoryService;
import org.sakaiproject.tool.helper.AttachmentAction;
import org.sakaiproject.tool.helper.PermissionsAction;
import org.sakaiproject.util.SortedIterator;
import org.sakaiproject.util.StringUtil;
import org.sakaiproject.util.observer.MultipleEventsObservingCourier;
import org.sakaiproject.util.FormattedText;

/**
 * <p>AssignmentAction is the action class for the dicussion tool.</p>
 *
 * @author University of Michigan, CHEF Software Development Team
 * @version $Revision: 1.13 $
 */

public class AssignmentAction
extends NewPagedResourceAction
{
	/** The attachments	 */
	private static final String ATTACHMENTS = "Assignment.attachments";
	
	/** The content type image lookup service in the State. */
	private static final String STATE_CONTENT_TYPE_IMAGE_SERVICE = "Assignment.content_type_image_service";
	
	/** The user directory service in the State. */
	private static final String STATE_USER_DIRECTORY_SERVICE = "Assignment.user_directory_service";
	
	/** The calendar service in the State. */
	private static final String STATE_CALENDAR_SERVICE = "Assignment.calendar_service";
	
	/** The announcement service in the State. */
	private static final String STATE_ANNOUNCEMENT_SERVICE = "Assignment.announcement_service";
	
	/** The calendar object */
	private static final String CALENDAR = "calendar";
	
	/** The announcement channel */
	private static final String ANNOUNCEMENT_CHANNEL = "announcement_channel";
	
	/** The state mode	 */
	private static final String STATE_MODE = "Assignment.mode";
	
	/** The context string */
	private static final String STATE_CONTEXT_STRING = "Assignment.context_string";
	
	/** The user */
	private static final String STATE_USER = "Assignment.user";
	
	// SECTION MOD
	/** Used to keep track of the section info not currently being used. */
	private static final String STATE_SECTION_STRING = "Assignment.section_string";


	/****************************** sort assignment ***********************/
	/** state sort **/
	private static final String SORTED_BY = "Assignment.sorted_by";
	
	/** state sort ascendingly **/
	private static final String SORTED_ASC = "Assignment.sorted_asc";
	
	/** sort by assignment title */
	private static final String SORTED_BY_TITLE = "title";
	
	/** sort by assignment section */
	private static final String SORTED_BY_SECTION = "section";
	
	/** sort by assignment due date */
	private static final String SORTED_BY_DUEDATE = "duedate";
	
	/** sort by assignment open date */
	private static final String SORTED_BY_OPENDATE = "opendate";
	
	/** sort by assignment status */
	private static final String SORTED_BY_ASSIGNMENT_STATUS = "assignment_status";
	
	/** sort by assignment submission status */
	private static final String SORTED_BY_SUBMISSION_STATUS = "submission_status";
	
	/** sort by assignment number of submissions */
	private static final String SORTED_BY_NUM_SUBMISSIONS = "num_submissions";
	
	/** sort by assignment number of ungraded submissions */
	private static final String SORTED_BY_NUM_UNGRADED = "num_ungraded";
	
	/** sort by assignment submission grade */
	private static final String SORTED_BY_GRADE = "grade";
	
	/** sort by assignment maximun grade available */
	private static final String SORTED_BY_MAX_GRADE = "max_grade";
	
	/***************************** sort submission ************************/
	/** state sort submission**/
	private static final String SORTED_SUBMISSION_BY = "Assignment.submission_sorted_by";
	
	/** state sort submission ascendingly **/
	private static final String SORTED_SUBMISSION_ASC = "Assignment.submission_sorted_asc";
	
	/** state sort submission by submitters last name **/
	private static final String SORTED_SUBMISSION_BY_LASTNAME = "lastname";
	
	/** state sort submission by submit time **/
	private static final String SORTED_SUBMISSION_BY_SUBMIT_TIME = "submit_time";

	/** state sort submission by submission grade **/
	private static final String SORTED_SUBMISSION_BY_GRADE = "submission_grade";
	
	/** state sort submission by submission status **/
	private static final String SORTED_SUBMISSION_BY_STATUS = "status";

	/** state sort submission by submission released **/
	private static final String SORTED_SUBMISSION_BY_RELEASED = "released";
	
	/** state sort submission by assignment title */
	private static final String SORTED_SUBMISSION_BY_ASSIGNMENT = "assignment";

	/** state sort submission by max grade */
	private static final String SORTED_SUBMISSION_BY_MAX_GRADE = "submission_scale";
	
	/**********************************************************************/
	/** assignment id */
	private static final String ASSIGNMENT_ID = "assignment_id";
		
	/********************** student's view assignment submission *******************************/
	/** the assignment object been viewing **/
	private static final String VIEW_SUBMISSION_ASSIGNMENT_REFERENCE = "Assignment.view_submission_assignment_reference";
	
	/** the assignment object submission attachment **/
	private static final String VIEW_SUBMISSION_ATTACHMENT = "Assignment.view_submission_attachment";
	
	/** the submission text to the assignment **/
	private static final String VIEW_SUBMISSION_TEXT = "Assignment.view_submission_text";
	
	/** the submission answer to Honor Pledge **/
	private static final String VIEW_SUBMISSION_HONOR_PLEDGE_YES = "Assignment.view_submission_honor_pledge_yes";
	
	/******************* student's preview of submission ****************************/
	/** the assignment id **/
	private static final String PREVIEW_SUBMISSION_ASSIGNMENT_REFERENCE = "preview_submission_assignment_reference";
	
	/** the submission text **/
	private static final String PREVIEW_SUBMISSION_TEXT = "preview_submission_text";
	
	/** the submission honor pledge answer **/
	private static final String PREVIEW_SUBMISSION_HONOR_PLEDGE_YES = "preview_submission_honor_pledge_yes";
	
	/** the submission attachments **/
	private static final String PREVIEW_SUBMISSION_ATTACHMENTS = "preview_attachments";
	
	/** the flag indicate whether the to show the student view or not */
	private static final String PREVIEW_ASSIGNMENT_STUDENT_VIEW_HIDE_FLAG = "preview_assignment_student_view_hide_flag";
	
	/** the flag indicate whether the to show the assignment info or not */
	private static final String PREVIEW_ASSIGNMENT_ASSIGNMENT_HIDE_FLAG = "preview_assignment_assignment_hide_flag";
	
	/** the assignment id */
	private static final String PREVIEW_ASSIGNMENT_ASSIGNMENT_ID= "preview_assignment_assignment_id";

	/** the assignment content id */
	private static final String PREVIEW_ASSIGNMENT_ASSIGNMENTCONTENT_ID = "preview_assignment_assignmentcontent_id";

	/**************** view assignment ******************************************/
	/** the hide assignment flag in the view assignment page **/
	private static final String VIEW_ASSIGNMENT_HIDE_ASSIGNMENT_FLAG = "view_assignment_hide_assignment_flag";
	
	/** the hide student view flag in the view assignment page **/
	private static final String VIEW_ASSIGNMENT_HIDE_STUDENT_VIEW_FLAG = "view_assignment_hide_student_view_flag";
	
	/********************* instructor's view assignment ******************************/
	private static final String VIEW_ASSIGNMENT_ID = "view_assignment_id";
	
	/********************* instructor's edit assignment ******************************/
	private static final String EDIT_ASSIGNMENT_ID = "edit_assignment_id";
	
	/********************* instructor's delete assignment ids ******************************/
	private static final String DELETE_ASSIGNMENT_IDS = "delete_assignment_ids";
	
	/********************* instructor's duplicate assignment ids ******************************/
	private static final String DUPLICATE_ASSIGNMENT_IDS = "duplicate_assignment_ids";
	
	/********************* flags controls the grade assignment page layout ********************/
	private static final String GRADE_ASSIGNMENT_EXPAND_FLAG = "grade_assignment_expand_flag";
	private static final String GRADE_SUBMISSION_EXPAND_FLAG = "grade_submission_expand_flag";
	
	/********************* instructor's grade submission ******************************/
	private static final String GRADE_SUBMISSION_ASSIGNMENT_ID = "grade_submission_assignment_id";
	private static final String GRADE_SUBMISSION_SUBMISSION_ID = "grade_submission_submission_id";
	private static final String GRADE_SUBMISSION_FEEDBACK_COMMENT = "grade_submission_feedback_comment";
	private static final String GRADE_SUBMISSION_FEEDBACK_TEXT = "grade_submission_feedback_text";
	private static final String GRADE_SUBMISSION_FEEDBACK_ATTACHMENT = "grade_submission_feedback_attachment";
	private static final String GRADE_SUBMISSION_GRADE = "grade_submission_grade";
	private static final String GRADE_SUBMISSION_ASSIGNMENT_EXPAND_FLAG = "grade_submission_assignment_expand_flag";
	
	/********************* instructor's export assignment ******************************/
	private static final String EXPORT_ASSIGNMENT_REF = "export_assignment_ref";
	private static final String EXPORT_ASSIGNMENT_ID = "export_assignment_id";
	/******************** instructor's new assignment *******************************/
	private static final String NEW_ASSIGNMENT_TITLE = "new_assignment_title";
	// open date
	private static final String NEW_ASSIGNMENT_OPENMONTH = "new_assignment_openmonth";
	private static final String NEW_ASSIGNMENT_OPENDAY = "new_assignment_openday";
	private static final String NEW_ASSIGNMENT_OPENYEAR = "new_assignment_openyear";
	private static final String NEW_ASSIGNMENT_OPENHOUR = "new_assignment_openhour";
	private static final String NEW_ASSIGNMENT_OPENMIN = "new_assignment_openmin";
	private static final String NEW_ASSIGNMENT_OPENAMPM = "new_assignment_openampm";
	// due date
	private static final String NEW_ASSIGNMENT_DUEMONTH = "new_assignment_duemonth";
	private static final String NEW_ASSIGNMENT_DUEDAY = "new_assignment_dueday";
	private static final String NEW_ASSIGNMENT_DUEYEAR = "new_assignment_dueyear";
	private static final String NEW_ASSIGNMENT_DUEHOUR = "new_assignment_duehour";
	private static final String NEW_ASSIGNMENT_DUEMIN = "new_assignment_duemin";
	private static final String NEW_ASSIGNMENT_DUEAMPM = "new_assignment_dueampm";
	// close date
	private static final String NEW_ASSIGNMENT_ENABLECLOSEDATE = "new_assignment_enableclosedate";
	private static final String NEW_ASSIGNMENT_CLOSEMONTH = "new_assignment_closemonth";
	private static final String NEW_ASSIGNMENT_CLOSEDAY = "new_assignment_closeday";
	private static final String NEW_ASSIGNMENT_CLOSEYEAR = "new_assignment_closeyear";
	private static final String NEW_ASSIGNMENT_CLOSEHOUR = "new_assignment_closehour";
	private static final String NEW_ASSIGNMENT_CLOSEMIN = "new_assignment_closemin";
	private static final String NEW_ASSIGNMENT_CLOSEAMPM = "new_assignment_closeampm";
	
	private static final String NEW_ASSIGNMENT_ATTACHMENT = "new_assignment_attachment";
	private static final String NEW_ASSIGNMENT_SECTION = "new_assignment_section";
	private static final String NEW_ASSIGNMENT_SUBMISSION_TYPE = "new_assignment_submission_type";
	private static final String NEW_ASSIGNMENT_GRADE_TYPE = "new_assignment_grade_type";
	private static final String NEW_ASSIGNMENT_GRADE_POINTS = "new_assignment_grade_points";
	private static final String NEW_ASSIGNMENT_DESCRIPTION = "new_assignment_instructions";
	private static final String NEW_ASSIGNMENT_RELEASE_GRADE_TYPE = "new_assignment_release_grade_type";
	private static final String NEW_ASSIGNMENT_DUE_DATE_SCHEDULED = "new_assignment_due_date_scheduled";
	private static final String NEW_ASSIGNMENT_OPEN_DATE_ANNOUNCED = "new_assignment_open_date_announced";
	private static final String NEW_ASSIGNMENT_CHECK_ADD_HONOR_PLEDGE = "new_assignment_check_add_honor_pledge";
	private static final String NEW_ASSIGNMENT_HIDE_OPTION_FLAG = "new_assignment_hide_option_flag";
	private static final String NEW_ASSIGNMENT_FOCUS = "new_assignment_focus";
	private static final String NEW_ASSIGNMENT_DESCRIPTION_EMPTY = "new_assignment_description_empty";
	
	private static final String ATTACHMENTS_MODIFIED = "attachments_modified";

	/****************************** instructor's view student submission ******************/
	// the show/hide table based on member id
	private static final String STUDENT_LIST_SHOW_TABLE = "STUDENT_LIST_SHOW_TABLE";
	
	/****************************** student view grade submission id ************/
	private static final String VIEW_GRADE_SUBMISSION_ID = "view_grade_submission_id";
	
	/****************************** modes ****************************/
	/** The student list view of assignments*/
	private static final String MODE_STUDENT_LIST_ASSIGNMENTS = "Student View";
	
	/** The student view of an assignment submission	 */
	private static final String MODE_STUDENT_VIEW_SUBMISSION = "Assignment.mode_view_submission";
	
	/** The student preview of an assignment submission	 */
	private static final String MODE_STUDENT_PREVIEW_SUBMISSION = "Assignment.mode_student_preview_submission";
	
	/** The student view of graded submission	 */
	private static final String MODE_STUDENT_VIEW_GRADE = "Assignment.mode_student_view_grade";
	
	/** The instructor view of owned assignments	 */
	private static final String MODE_INSTRUCTOR_LIST_ASSIGNMENTS = "List of Assignments";
	
	/** The instructor view of create a new assignment	 */
	private static final String MODE_INSTRUCTOR_NEW_ASSIGNMENT = "Assignment.mode_instructor_new_assignment";
	
	/** The instructor view to edit an assignment	 */
	private static final String MODE_INSTRUCTOR_EDIT_ASSIGNMENT = "Assignment.mode_instructor_edit_assignment";
	
	/** The instructor view to delete an assignment	 */
	private static final String MODE_INSTRUCTOR_DELETE_ASSIGNMENT = "Assignment.mode_instructor_delete_assignment";
	
	/** The instructor view to grade an assignment	 */
	private static final String MODE_INSTRUCTOR_GRADE_ASSIGNMENT = "Assignment.mode_instructor_grade_assignment";
	
	/** The instructor view to grade a submission	 */
	private static final String MODE_INSTRUCTOR_GRADE_SUBMISSION = "Assignment.mode_instructor_grade_submission";
	
	/** The instructor view of preview grading a submission	 */
	private static final String MODE_INSTRUCTOR_PREVIEW_GRADE_SUBMISSION = "Assignment.mode_instructor_preview_grade_submission";
	
	/** The instructor view of view grading a submission	 */
	private static final String MODE_INSTRUCTOR_VIEW_GRADE_SUBMISSION = "Assignment.mode_instructor_view_grade_submission";
	
	/** The instructor preview of one assignment	 */
	private static final String MODE_INSTRUCTOR_PREVIEW_ASSIGNMENT = "Assignment.mode_instructor_preview_assignments";
	
	/** The instructor view of one assignment	 */
	private static final String MODE_INSTRUCTOR_VIEW_ASSIGNMENT = "Assignment.mode_instructor_view_assignments";
	
	/** The instructor view to list students of an assignment	 */
	private static final String MODE_INSTRUCTOR_VIEW_STUDENTS_ASSIGNMENT = "List of Assignments by student";
	
	/** The instructor view of assignment submission report */
	private static final String MODE_INSTRUCTOR_REPORT_SUBMISSIONS = "Grade Report";
	
	/*************************** vm names ***************************/
	/** The student list view of assignments */
	private static final String TEMPLATE_STUDENT_LIST_ASSIGNMENTS = "_student_list_assignments";
	
	/** The student view of showing an assignment submission */
	private static final String TEMPLATE_STUDENT_VIEW_SUBMISSION = "_student_view_submission";
	
	/** The student preview an assignment submission */
	private static final String TEMPLATE_STUDENT_PREVIEW_SUBMISSION = "_student_preview_submission";
	
	/** The student view of graded submission */
	private static final String TEMPLATE_STUDENT_VIEW_GRADE = "_student_view_grade";
	
	/** The instructor list view of assignments */
	private static final String TEMPLATE_INSTRUCTOR_LIST_ASSIGNMENTS = "_instructor_list_assignments";
	
	/** The instructor view to create a new assignment */
	private static final String TEMPLATE_INSTRUCTOR_NEW_ASSIGNMENT = "_instructor_new_assignment";
	
	/** The instructor view to edit assignment */
	private static final String TEMPLATE_INSTRUCTOR_EDIT_ASSIGNMENT = "_instructor_edit_assignment";
	
	/** The instructor view to edit assignment */
	private static final String TEMPLATE_INSTRUCTOR_DELETE_ASSIGNMENT = "_instructor_delete_assignment";
	
	/** The instructor view to edit assignment */
	private static final String TEMPLATE_INSTRUCTOR_GRADE_SUBMISSION = "_instructor_grading_submission";
	
	/** The instructor preview to edit assignment */
	private static final String TEMPLATE_INSTRUCTOR_PREVIEW_GRADE_SUBMISSION = "_instructor_preview_grading_submission";
	
	/** The instructor view to edit assignment */
	private static final String TEMPLATE_INSTRUCTOR_VIEW_GRADE_SUBMISSION = "_instructor_view_grading_submission";
	
	/** The instructor view to grade the assignment */
	private static final String TEMPLATE_INSTRUCTOR_GRADE_ASSIGNMENT = "_instructor_list_submissions";
	
	/** The instructor preview of assignment */
	private static final String TEMPLATE_INSTRUCTOR_PREVIEW_ASSIGNMENT = "_instructor_preview_assignment";
	
	/** The instructor view of assignment */
	private static final String TEMPLATE_INSTRUCTOR_VIEW_ASSIGNMENT = "_instructor_view_assignment";
	
	/** The instructor view to edit assignment */
	private static final String TEMPLATE_INSTRUCTOR_VIEW_STUDENTS_ASSIGNMENT = "_instructor_student_list_submissions";
	
	/** The instructor view to assignment submission report */
	private static final String TEMPLATE_INSTRUCTOR_REPORT_SUBMISSIONS = "_instructor_report_submissions";
	
	/** the alert message to be shown when there is a InUseException */
	private static final String INUSE_ERROR_MESSAGE = "Someone else is editing this ";
	
	/** the alert message to be shown when the close date is before open date */
	private static final String CLOSE_OPEN_DATE_ERROR_MESSAGE = "Accept submissions deadline should be set after the open date. ";
	
	/** the alert message to be shown when the close date is before due date */
	private static final String CLOSE_DUE_DATE_ERROR_MESSAGE = "Accept submissions deadline should be set after the due date. ";
	
	/** The opening mark comment */
	private static final String COMMENT_OPEN = "{{";
	
	/** The closing mark for comment */
	private static final String COMMENT_CLOSE = "}}";

	/**
	 * central place for dispatching the build routines based on the state name
	 */
	public String buildMainPanelContext (	VelocityPortlet portlet,
											Context context,
											RunData data,
											SessionState state)
	{		
		// if we are in edit permissions...
		String helperMode = (String) state.getAttribute(PermissionsAction.STATE_MODE);
		if (helperMode != null)
		{
			String template = PermissionsAction.buildHelperContext(portlet, context, data, state);
			if (template == null)
			{
				addAlert(state, "There is a problem setting Permissions at this time.");
			}
			else
			{
				return template;
			}
		}

		String template = null;
		
		// if we are in edit attachments..
		String attachment_mode = (String) state.getAttribute (AttachmentAction.STATE_MODE);
		if (attachment_mode != null)
		{
			// if the attachment mode is not done, defer to the AttachmentAction
			if (!attachment_mode.equals (AttachmentAction.MODE_DONE))
			{
				template = AttachmentAction.buildHelperContext (portlet, context, data, state);
				return template;
			}
			
			// when done, get the new attachments (if null, no change was made)
			Vector attachments = (Vector) state.getAttribute (AttachmentAction.STATE_ATTACHMENTS);
			if (attachments != null)
			{
				state.setAttribute(ATTACHMENTS_MODIFIED, new Boolean(true));
				state.setAttribute (ATTACHMENTS, attachments);
				context.put ("attachments", attachments);
			}
			
			// clean up
			state.removeAttribute (AttachmentAction.STATE_MODE);
			state.removeAttribute (AttachmentAction.STATE_ATTACHMENTS);
		}	// if
		
		String contextString = (String) state.getAttribute (STATE_CONTEXT_STRING);		
		String mode = (String)state.getAttribute (STATE_MODE);
		if (mode.equals (MODE_STUDENT_LIST_ASSIGNMENTS))
		{
			// enable the observer when comming to the student assignment list view
			// watch only for the assignment update
			MultipleEventsObservingCourier o = (MultipleEventsObservingCourier) state.getAttribute(STATE_OBSERVER);
			String pattern = AssignmentService.assignmentReference((String) state.getAttribute (STATE_CONTEXT_STRING), "");
			o.setResourcePattern(pattern);
			pattern = AssignmentService.submissionReference((String) state.getAttribute (STATE_CONTEXT_STRING), "", "");
			// remove the trailing "/" in order to match all the submission for all assignment
			// %%% not a clean solution, we can fix the service to have a null passed as the parameters we don't want to have in the reference later 
			o.addResourcePattern(pattern.substring(0, pattern.lastIndexOf("/")));
			o.enable();
			o.setDeliveryId(clientWindowId(state, portlet.getID()));

			// build the context for the student assignment view
			template = build_student_list_assignments_context (portlet, context, data, state);
		}
		else if (mode.equals (MODE_STUDENT_VIEW_SUBMISSION))
		{
			// disable auto-updates while leaving the list view
			((MultipleEventsObservingCourier) state.getAttribute(STATE_OBSERVER)).disable();
				
			// build the context for showing one assignment submission
			template = build_student_view_submission_context (portlet, context, data, state);
		}
		else if (mode.equals (MODE_STUDENT_PREVIEW_SUBMISSION))
		{
			// build the context for showing one assignment submission
			template = build_student_preview_submission_context (portlet, context, data, state);
		}
		else if (mode.equals (MODE_STUDENT_VIEW_GRADE))
		{
			// disable auto-updates while leaving the list view
			((MultipleEventsObservingCourier) state.getAttribute(STATE_OBSERVER)).disable();
		
			// build the context for showing one graded submission
			template = build_student_view_grade_context (portlet, context, data, state);
		}
		else if (mode.equals (MODE_INSTRUCTOR_LIST_ASSIGNMENTS))
		{
			// enable the observer when comming to the instructor assignment list view
			// watch for both the assignment update and the submission update for any assignment
			MultipleEventsObservingCourier o = (MultipleEventsObservingCourier) state.getAttribute(STATE_OBSERVER);
			String pattern = AssignmentService.submissionReference((String) state.getAttribute (STATE_CONTEXT_STRING), "", "");
			// remove the trailing "/" in order to match all the submission for all assignment
			// %%% not a clean solution, we can fix the service to have a null passed as the parameters we don't want to have in the reference later 
			o.addResourcePattern(pattern.substring(0, pattern.lastIndexOf("/")));
			o.enable();
			o.setDeliveryId(clientWindowId(state, portlet.getID()));
			
			// build the context for the instructor assignment list view
			template = build_instructor_list_assignments_context (portlet, context, data, state);
		}
		else if (mode.equals (MODE_INSTRUCTOR_NEW_ASSIGNMENT))
		{
			// disable auto-updates while leaving the list view
			((MultipleEventsObservingCourier) state.getAttribute(STATE_OBSERVER)).disable();
				
			// build the context for the instructor's create new assignment view
			template = build_instructor_new_assignment_context (portlet, context, data, state);
		}
		else if (mode.equals (MODE_INSTRUCTOR_EDIT_ASSIGNMENT))
		{
			// disable auto-updates while leaving the list view
			((MultipleEventsObservingCourier) state.getAttribute(STATE_OBSERVER)).disable();
						
			// build the context for the instructor's edit assignment
			template = build_instructor_edit_assignment_context (portlet, context, data, state);
		}
		else if (mode.equals (MODE_INSTRUCTOR_DELETE_ASSIGNMENT))
		{
			// disable auto-updates while leaving the list view
			((MultipleEventsObservingCourier) state.getAttribute(STATE_OBSERVER)).disable();
				
			// build the context for the instructor's delete assignment
			template = build_instructor_delete_assignment_context (portlet, context, data, state);
		}
		else if (mode.equals (MODE_INSTRUCTOR_GRADE_ASSIGNMENT))
		{
			// change the observer to watch for the submission update for this assignment
			MultipleEventsObservingCourier o = (MultipleEventsObservingCourier) state.getAttribute(STATE_OBSERVER);
			String pattern = AssignmentService.submissionReference((String) state.getAttribute (STATE_CONTEXT_STRING), "",  (String) state.getAttribute (EXPORT_ASSIGNMENT_ID));
			o.setResourcePattern(pattern);
			o.enable();
			o.setDeliveryId(clientWindowId(state, portlet.getID()));
				
			// build the context for the instructor's grade assignment
			template = build_instructor_grade_assignment_context (portlet, context, data, state);
		}
		else if (mode.equals (MODE_INSTRUCTOR_GRADE_SUBMISSION))
		{
			// disable auto-updates while leaving the list view
			((MultipleEventsObservingCourier) state.getAttribute(STATE_OBSERVER)).disable();
		
			// build the context for the instructor's grade submission
			template = build_instructor_grade_submission_context (portlet, context, data, state);
		}
		else if (mode.equals (MODE_INSTRUCTOR_PREVIEW_GRADE_SUBMISSION))
		{
			// build the context for the instructor's preview grade submission
			template = build_instructor_preview_grade_submission_context (portlet, context, data, state);
		}
		else if (mode.equals (MODE_INSTRUCTOR_PREVIEW_ASSIGNMENT))
		{
			// build the context for preview one assignment
			template = build_instructor_preview_assignment_context (portlet, context, data, state);
		}
		else if (mode.equals (MODE_INSTRUCTOR_VIEW_ASSIGNMENT))
		{
			// disable auto-updates while leaving the list view
			((MultipleEventsObservingCourier) state.getAttribute(STATE_OBSERVER)).disable();
				
			// build the context for view one assignment
			template = build_instructor_view_assignment_context (portlet, context, data, state);
		}
		else if (mode.equals (MODE_INSTRUCTOR_VIEW_STUDENTS_ASSIGNMENT))
		{
			// set the observer to watch for both assignment and submission updates for any assignment
			MultipleEventsObservingCourier o = (MultipleEventsObservingCourier) state.getAttribute(STATE_OBSERVER);
			String pattern = AssignmentService.assignmentReference((String) state.getAttribute (STATE_CONTEXT_STRING), "");
			o.setResourcePattern(pattern);
			pattern = AssignmentService.submissionReference((String) state.getAttribute (STATE_CONTEXT_STRING), "", "");
			// %%% not a clean solution, we can fix the service to have a null passed as the parameters we don't want to have in the reference later 
			o.addResourcePattern(pattern.substring(0, pattern.lastIndexOf("/"))); 
			o.enable();
			o.setDeliveryId(clientWindowId(state, portlet.getID()));
			
			// build the context for the instructor's create new assignment view
			template = build_instructor_view_students_assignment_context (portlet, context, data, state);
		}
		else if (mode.equals (MODE_INSTRUCTOR_REPORT_SUBMISSIONS))
		{
			// set the observer to watch for both assignment and submission updates for any assignment
			MultipleEventsObservingCourier o = (MultipleEventsObservingCourier) state.getAttribute(STATE_OBSERVER);
			String pattern = AssignmentService.assignmentReference((String) state.getAttribute (STATE_CONTEXT_STRING), "");
			o.setResourcePattern(pattern);
			pattern = AssignmentService.submissionReference((String) state.getAttribute (STATE_CONTEXT_STRING), "", "");
			// %%% not a clean solution, we can fix the service to have a null passed as the parameters we don't want to have in the reference later 
			o.addResourcePattern(pattern.substring(0, pattern.lastIndexOf("/"))); 
			o.enable();
			o.setDeliveryId(clientWindowId(state, portlet.getID()));
			
			// build the context for the instructor's view of report submissions
			template = build_instructor_report_submissions (portlet, context, data, state);
		}

		return template;
		
	}	// buildNormalContext
	
	/**
	 * build the student view of owned assignments list
	 */
	protected String build_student_list_assignments_context (	VelocityPortlet portlet,
																Context context,
																RunData data,
																SessionState state)
	{		
		state.setAttribute(STATE_SELECTED_VIEW, "Student View");
		
		// get the user information
		User u = (User) state.getAttribute (STATE_USER);
		// get the assignment specific to the class and person
		String sortedBy = (String) state.getAttribute (SORTED_BY);
		String sortedAsc = (String) state.getAttribute (SORTED_ASC);
		
		// get those opened and posted assignments
		Iterator assignments = prepPage(state).iterator();
		
		//SortedIterator sortedAssignments = new SortedIterator (v.iterator (), new AssignmentComparator (sortedBy, sortedAsc, u));
		context.put ("assignments", assignments);
		context.put ("sortedBy", sortedBy);
		context.put ("sortedAsc", sortedAsc);
		context.put ("currentTime", TimeService.newTime());
		context.put ("user", (User) state.getAttribute (STATE_USER));
		context.put ("service", AssignmentService.getInstance());
		
		Boolean a = new Boolean (AssignmentService.allowAddAssignment ((String) state.getAttribute (STATE_CONTEXT_STRING)));
		context.put ("allowAdd", a);
		
		// build the menu
		buildMenu (portlet, context, data, state, "studentForm", a.booleanValue (), false, false, "");
		
		context.put("allowAddNewAssignment", new Boolean(AssignmentService.allowAddAssignment ((String) state.getAttribute (STATE_CONTEXT_STRING))));

		add2ndToolbarFields(data, context);
		
		// inform the observing courier that we just updated the page...
		// if there are pending requests to do so they can be cleared
		((MultipleEventsObservingCourier) state.getAttribute(STATE_OBSERVER)).justDelivered();

		String template = (String) getContext(data).get("template");
		return template + TEMPLATE_STUDENT_LIST_ASSIGNMENTS;
		
	}	// build_student_list_assignmfents_context
	
	/**
	 * build the student view of showing an assignment submission
	 */
	protected String build_student_view_submission_context (VelocityPortlet portlet,
															Context context,
															RunData data,
															SessionState state)
	{
		// let the vm know this is instructor or not
		boolean allowAddNewAssignment = AssignmentService.allowAddAssignment ((String) state.getAttribute (STATE_CONTEXT_STRING));
		context.put ("allowAdd", new Boolean (allowAddNewAssignment));
		
		context.put ("context", (String) state.getAttribute (STATE_CONTEXT_STRING));
		
		User user = (User) state.getAttribute (STATE_USER);
		String currentAssignmentReference = (String) state.getAttribute (VIEW_SUBMISSION_ASSIGNMENT_REFERENCE);
		try
		{
			Assignment currentAssignment = AssignmentService.getAssignment(currentAssignmentReference);
			context.put ("assignment", currentAssignment);
			context.put ("submission", AssignmentService.getSubmission (currentAssignment.getReference (), user));
		}
		catch (IdUnusedException e)
		{
			addAlert(state, "Can not find the assignment.");
		}
		catch (PermissionException e)
		{
			addAlert(state, "You are not allowed to view the submission to this assignment.");
		}
		
		// name value pairs for the vm
		context.put ("name_assignment_id", ASSIGNMENT_ID);
		context.put ("name_submission_text", VIEW_SUBMISSION_TEXT);
		context.put ("value_submission_text", state.getAttribute (VIEW_SUBMISSION_TEXT));
		context.put ("name_submission_honor_pledge_yes", VIEW_SUBMISSION_HONOR_PLEDGE_YES );
		context.put ("value_submission_honor_pledge_yes", state.getAttribute (VIEW_SUBMISSION_HONOR_PLEDGE_YES));
		context.put ("attachments", state.getAttribute (ATTACHMENTS));
		context.put ("contentTypeImageService", state.getAttribute (STATE_CONTENT_TYPE_IMAGE_SERVICE));
		context.put ("currentTime", TimeService.newTime());
		
		String template = (String) getContext(data).get("template");
		return template + TEMPLATE_STUDENT_VIEW_SUBMISSION;
		
	}	// build_student_view_submission_context
	
	/**
	 * build the student preview of showing an assignment submission
	 */
	protected String build_student_preview_submission_context (	VelocityPortlet portlet,
																Context context,
																RunData data,
																SessionState state)
	{		
		User user = (User) state.getAttribute (STATE_USER);
		String aReference = (String) state.getAttribute (PREVIEW_SUBMISSION_ASSIGNMENT_REFERENCE);

		try
		{
			context.put ("assignment", AssignmentService.getAssignment (aReference));
			context.put ("submission", AssignmentService.getSubmission (aReference, user));
		}
		catch (IdUnusedException e)
		{
			addAlert(state, "Can not find the assignment. ");
		}
		catch (PermissionException e)
		{
			addAlert(state, "You are not allowed to view the submission to this assignment. ");
		}
		
		context.put ("name_assignment_id", ASSIGNMENT_ID);
		context.put ("text", state.getAttribute (PREVIEW_SUBMISSION_TEXT));
		context.put ("honor_pledge_yes", state.getAttribute (PREVIEW_SUBMISSION_HONOR_PLEDGE_YES));
		context.put ("attachments", state.getAttribute (PREVIEW_SUBMISSION_ATTACHMENTS));
		context.put ("contentTypeImageService", state.getAttribute (STATE_CONTENT_TYPE_IMAGE_SERVICE));
		
		String template = (String) getContext(data).get("template");
		return template + TEMPLATE_STUDENT_PREVIEW_SUBMISSION;
		
	}	// build_student_preview_submission_context
	
	/**
	 * build the student view of showing a graded submission
	 */
	protected String build_student_view_grade_context (	VelocityPortlet portlet,
														Context context,
														RunData data,
														SessionState state)
	{
		context.put ("contentTypeImageService", state.getAttribute (STATE_CONTENT_TYPE_IMAGE_SERVICE));
		
		try
		{
			AssignmentSubmission s = AssignmentService.getSubmission ((String) state.getAttribute (VIEW_GRADE_SUBMISSION_ID));
			context.put ("submission", s);
		}
		catch (IdUnusedException e)
		{
			addAlert(state, "Can not find the assignment submission. ");
		}
		catch (PermissionException e)
		{
			addAlert(state, "You are not allowed to get the submission to this assignment. ");
		}
		
		String template = (String) getContext(data).get("template");
		return template + TEMPLATE_STUDENT_VIEW_GRADE;
		
	}	// build_student_view_grade_context

	/**
	 * build the instructor view of owned assignments list
	 */
	protected String build_instructor_list_assignments_context (VelocityPortlet portlet,
																Context context,
																RunData data,
																SessionState state)
	{
		context.put ("contextString", (String) state.getAttribute (STATE_CONTEXT_STRING));
		context.put ("service", AssignmentService.getInstance());
		context.put ("currentTime", TimeService.newTime());
		String sortedBy = (String) state.getAttribute (SORTED_BY);
		String sortedAsc = (String) state.getAttribute (SORTED_ASC);
		
		String gId = (String) state.getAttribute (STATE_CONTEXT_STRING);
		
		// filter out draft assignments from posted assignments
		Iterator assignmentsForContext = AssignmentService.getAssignmentsForContext (gId);
		Vector postedAssignments = new Vector ();
		Vector draftAssignments = new Vector ();
		Vector totalAssignments = new Vector ();
		while (assignmentsForContext.hasNext ())
		{
			Assignment a = (Assignment) assignmentsForContext.next ();
			if (a.getDraft())
			{
				draftAssignments.add(a);
			}
			else
			{
				postedAssignments.add(a);
			}
		}

		SortedIterator dIterator = new SortedIterator (draftAssignments.iterator(), new AssignmentComparator (sortedBy, sortedAsc));
		SortedIterator pIterator = new SortedIterator (postedAssignments.iterator(), new AssignmentComparator (sortedBy, sortedAsc));

		if (sortedAsc.equals(Boolean.TRUE.toString()))
		{
			// sort draft assignments first
			totalAssignments.addAll(iterator_to_vector(dIterator));
			totalAssignments.addAll(iterator_to_vector(pIterator));
		}
		else
		{
			// sort posted assignments first
			totalAssignments.addAll(iterator_to_vector(pIterator));
			totalAssignments.addAll(iterator_to_vector(dIterator));
		}
		
		//Iterator assignments = totalAssignments.iterator();
		Iterator assignments = prepPage(state).iterator();
		context.put ("sortedBy", sortedBy);
		context.put ("sortedAsc", sortedAsc);
		context.put ("assignments", assignments);
		
		// build the menu
		buildMenu (portlet, context, data, state, "listAssignmentsForm", true, true, assignments.hasNext (), "");
		context.put("contextString", state.getAttribute (STATE_CONTEXT_STRING));
		
		add2ndToolbarFields(data, context);
		
		// inform the observing courier that we just updated the page...
		// if there are pending requests to do so they can be cleared
		((MultipleEventsObservingCourier) state.getAttribute(STATE_OBSERVER)).justDelivered();
		
		String template = (String) getContext(data).get("template");
		return template + TEMPLATE_INSTRUCTOR_LIST_ASSIGNMENTS;
		
	}	// build_instructor_list_assignments_context
	
	/**
	 * build the instructor view of create a new assignment
	 */
	protected String build_instructor_new_assignment_context (	VelocityPortlet portlet,
																Context context,
																RunData data,
																SessionState state)
	{
		// get the exist assignment
		context.put ("existAssignment", AssignmentService.getAssignmentContents (UsageSessionService.getSessionUser ()));
		
		// put the names and values into vm file
		context.put ("name_title", NEW_ASSIGNMENT_TITLE);
		
		context.put ("name_OpenMonth", NEW_ASSIGNMENT_OPENMONTH);
		context.put ("name_OpenDay", NEW_ASSIGNMENT_OPENDAY);
		context.put ("name_OpenYear", NEW_ASSIGNMENT_OPENYEAR);
		context.put ("name_OpenHour", NEW_ASSIGNMENT_OPENHOUR);
		context.put ("name_OpenMin", NEW_ASSIGNMENT_OPENMIN);
		context.put ("name_OpenAMPM", NEW_ASSIGNMENT_OPENAMPM);
		
		context.put ("name_DueMonth", NEW_ASSIGNMENT_DUEMONTH);
		context.put ("name_DueDay", NEW_ASSIGNMENT_DUEDAY);
		context.put ("name_DueYear", NEW_ASSIGNMENT_DUEYEAR);
		context.put ("name_DueHour", NEW_ASSIGNMENT_DUEHOUR);
		context.put ("name_DueMin", NEW_ASSIGNMENT_DUEMIN);
		context.put ("name_DueAMPM", NEW_ASSIGNMENT_DUEAMPM);
		
		context.put	("name_EnableCloseDate", NEW_ASSIGNMENT_ENABLECLOSEDATE);
		context.put ("name_CloseMonth", NEW_ASSIGNMENT_CLOSEMONTH);
		context.put ("name_CloseDay", NEW_ASSIGNMENT_CLOSEDAY);
		context.put ("name_CloseYear", NEW_ASSIGNMENT_CLOSEYEAR);
		context.put ("name_CloseHour", NEW_ASSIGNMENT_CLOSEHOUR);
		context.put ("name_CloseMin", NEW_ASSIGNMENT_CLOSEMIN);
		context.put ("name_CloseAMPM", NEW_ASSIGNMENT_CLOSEAMPM);
		
		context.put ("name_Section", NEW_ASSIGNMENT_SECTION);
		context.put ("name_SubmissionType", NEW_ASSIGNMENT_SUBMISSION_TYPE);
		context.put ("name_GradeType", NEW_ASSIGNMENT_GRADE_TYPE);
		context.put ("name_GradePoints", NEW_ASSIGNMENT_GRADE_POINTS);
		context.put ("name_Description", NEW_ASSIGNMENT_DESCRIPTION);
		context.put ("name_ReleaseGradeType", NEW_ASSIGNMENT_RELEASE_GRADE_TYPE);
		context.put ("name_CheckAddDueDate", ResourceProperties.NEW_ASSIGNMENT_CHECK_ADD_DUE_DATE);
		context.put ("name_CheckAutoAnnounce", ResourceProperties.NEW_ASSIGNMENT_CHECK_AUTO_ANNOUNCE);
		context.put ("name_CheckAddHonorPledge", NEW_ASSIGNMENT_CHECK_ADD_HONOR_PLEDGE);
		
		// set the values
		context.put ("value_title", state.getAttribute (NEW_ASSIGNMENT_TITLE));
		context.put ("value_OpenMonth", state.getAttribute (NEW_ASSIGNMENT_OPENMONTH));
		context.put ("value_OpenDay", state.getAttribute (NEW_ASSIGNMENT_OPENDAY));
		context.put ("value_OpenYear", state.getAttribute (NEW_ASSIGNMENT_OPENYEAR));
		context.put ("value_OpenHour", state.getAttribute (NEW_ASSIGNMENT_OPENHOUR));
		context.put ("value_OpenMin", state.getAttribute (NEW_ASSIGNMENT_OPENMIN));
		context.put ("value_OpenAMPM", state.getAttribute (NEW_ASSIGNMENT_OPENAMPM));
		
		context.put ("value_DueMonth", state.getAttribute (NEW_ASSIGNMENT_DUEMONTH));
		context.put ("value_DueDay", state.getAttribute (NEW_ASSIGNMENT_DUEDAY));
		context.put ("value_DueYear", state.getAttribute (NEW_ASSIGNMENT_DUEYEAR));
		context.put ("value_DueHour", state.getAttribute (NEW_ASSIGNMENT_DUEHOUR));
		context.put ("value_DueMin", state.getAttribute (NEW_ASSIGNMENT_DUEMIN));
		context.put ("value_DueAMPM", state.getAttribute (NEW_ASSIGNMENT_DUEAMPM));
		
		context.put	("value_EnableCloseDate", state.getAttribute (NEW_ASSIGNMENT_ENABLECLOSEDATE));
		context.put ("value_CloseMonth", state.getAttribute (NEW_ASSIGNMENT_CLOSEMONTH));
		context.put ("value_CloseDay", state.getAttribute (NEW_ASSIGNMENT_CLOSEDAY));
		context.put ("value_CloseYear", state.getAttribute (NEW_ASSIGNMENT_CLOSEYEAR));
		context.put ("value_CloseHour", state.getAttribute (NEW_ASSIGNMENT_CLOSEHOUR));
		context.put ("value_CloseMin", state.getAttribute (NEW_ASSIGNMENT_CLOSEMIN));
		context.put ("value_CloseAMPM", state.getAttribute (NEW_ASSIGNMENT_CLOSEAMPM));
		
		context.put ("value_Sections", state.getAttribute (NEW_ASSIGNMENT_SECTION));
		context.put ("value_SubmissionType", state.getAttribute (NEW_ASSIGNMENT_SUBMISSION_TYPE));
		context.put ("value_GradeType", state.getAttribute (NEW_ASSIGNMENT_GRADE_TYPE));
		// format to show one decimal place
		String maxGrade = (String) state.getAttribute (NEW_ASSIGNMENT_GRADE_POINTS);
		context.put ("value_GradePoints", displayGrade(state, maxGrade));
		context.put ("value_Description", state.getAttribute (NEW_ASSIGNMENT_DESCRIPTION));
		context.put ("value_ReleaseGradeType", state.getAttribute (NEW_ASSIGNMENT_RELEASE_GRADE_TYPE));
		context.put ("value_CheckAddDueDate", state.getAttribute (ResourceProperties.NEW_ASSIGNMENT_CHECK_ADD_DUE_DATE));
		context.put ("value_CheckAutoAnnounce", state.getAttribute (ResourceProperties.NEW_ASSIGNMENT_CHECK_AUTO_ANNOUNCE));
		String s = (String) state.getAttribute (NEW_ASSIGNMENT_CHECK_ADD_HONOR_PLEDGE);
		if (s == null)
			s = "1";
		context.put ("value_CheckAddHonorPledge", s);
		
		context.put ("monthTable", monthTable ());
		context.put ("gradeTypeTable", gradeTypeTable ());
		context.put ("submissionTypeTable", submissionTypeTable ());
		context.put ("hide_assignment_option_flag", state.getAttribute (NEW_ASSIGNMENT_HIDE_OPTION_FLAG));
		context.put ("attachments", state.getAttribute (ATTACHMENTS));
		context.put ("contentTypeImageService", state.getAttribute (STATE_CONTENT_TYPE_IMAGE_SERVICE));
		
		context.put("fField", state.getAttribute (NEW_ASSIGNMENT_FOCUS));
		
		String template = (String) getContext(data).get("template");
		return template + TEMPLATE_INSTRUCTOR_NEW_ASSIGNMENT;
		
	}	// build_instructor_new_assignment_context
	
	/**
	 * build the instructor view of create a new assignment
	 */
	protected String build_instructor_preview_assignment_context (	VelocityPortlet portlet,
																	Context context,
																	RunData data,
																	SessionState state)
	{	
		context.put ("time", TimeService.newTime());
		
		context.put ("user", UsageSessionService.getSessionUser ());
		
		context.put ("value_Title", (String) state.getAttribute (NEW_ASSIGNMENT_TITLE));
		
		// open time
		int openMonth = ((Integer) state.getAttribute (NEW_ASSIGNMENT_OPENMONTH)).intValue ();
		int openDay = ((Integer) state.getAttribute (NEW_ASSIGNMENT_OPENDAY)).intValue ();
		int openYear = ((Integer) state.getAttribute (NEW_ASSIGNMENT_OPENYEAR)).intValue ();
		int openHour = ((Integer) state.getAttribute (NEW_ASSIGNMENT_OPENHOUR)).intValue ();
		int openMin = ((Integer) state.getAttribute (NEW_ASSIGNMENT_OPENMIN)).intValue ();
		String openAMPM = (String) state.getAttribute (NEW_ASSIGNMENT_OPENAMPM);
		if ((openAMPM.equals ("PM"))&&(openHour!=12))
		{
			openHour = openHour + 12;
		}
		if ((openHour == 12) && (openAMPM.equals ("AM")))
		{
			openHour = 0;
		}
		Time openTime = TimeService.newTimeLocal(openYear, openMonth, openDay, openHour, openMin, 0, 0);
		context.put ("value_OpenDate", openTime);
		
		// due time
		int dueMonth = ((Integer) state.getAttribute (NEW_ASSIGNMENT_DUEMONTH)).intValue ();
		int dueDay = ((Integer) state.getAttribute (NEW_ASSIGNMENT_DUEDAY)).intValue ();
		int dueYear = ((Integer) state.getAttribute (NEW_ASSIGNMENT_DUEYEAR)).intValue ();
		int dueHour = ((Integer) state.getAttribute (NEW_ASSIGNMENT_DUEHOUR)).intValue ();
		int dueMin = ((Integer) state.getAttribute (NEW_ASSIGNMENT_DUEMIN)).intValue ();
		String dueAMPM = (String) state.getAttribute (NEW_ASSIGNMENT_DUEAMPM);
		if ((dueAMPM.equals ("PM")) && (dueHour!=12))
		{
			dueHour = dueHour + 12;
		}
		if ((dueHour == 12) && (dueAMPM.equals ("AM")))
		{
			dueHour = 0;
		}
		Time dueTime = TimeService.newTimeLocal(dueYear, dueMonth, dueDay, dueHour, dueMin, 0, 0);
		context.put ("value_DueDate", dueTime);
		
		// close time
		Time closeTime = TimeService.newTime();
		Boolean enableCloseDate = (Boolean)state.getAttribute (NEW_ASSIGNMENT_ENABLECLOSEDATE);
		context.put ("value_EnableCloseDate", enableCloseDate);
		if ((enableCloseDate).booleanValue ())
		{
			int closeMonth = ((Integer) state.getAttribute (NEW_ASSIGNMENT_CLOSEMONTH)).intValue ();
			int closeDay = ((Integer) state.getAttribute (NEW_ASSIGNMENT_CLOSEDAY)).intValue ();
			int closeYear = ((Integer) state.getAttribute (NEW_ASSIGNMENT_CLOSEYEAR)).intValue ();
			int closeHour = ((Integer) state.getAttribute (NEW_ASSIGNMENT_CLOSEHOUR)).intValue ();
			int closeMin = ((Integer) state.getAttribute (NEW_ASSIGNMENT_CLOSEMIN)).intValue ();
			String closeAMPM = (String) state.getAttribute (NEW_ASSIGNMENT_CLOSEAMPM);
			if ((closeAMPM.equals ("PM")) && (closeHour!=12))
			{
				closeHour = closeHour + 12;
			}
			if ((closeHour == 12) && (closeAMPM.equals ("AM")))
			{
				closeHour = 0;
			}
			closeTime = TimeService.newTimeLocal(closeYear, closeMonth, closeDay, closeHour, closeMin, 0, 0);
			context.put ("value_CloseDate", closeTime);
		}
		
		context.put ("value_Sections", state.getAttribute (NEW_ASSIGNMENT_SECTION));
		context.put ("value_SubmissionType", state.getAttribute (NEW_ASSIGNMENT_SUBMISSION_TYPE));
		context.put ("value_GradeType", state.getAttribute (NEW_ASSIGNMENT_GRADE_TYPE));
		String maxGrade = (String) state.getAttribute (NEW_ASSIGNMENT_GRADE_POINTS);			
		context.put ("value_GradePoints", displayGrade(state, maxGrade));
		context.put ("value_Description", state.getAttribute (NEW_ASSIGNMENT_DESCRIPTION));
		context.put ("value_ReleaseGradeType", state.getAttribute (NEW_ASSIGNMENT_RELEASE_GRADE_TYPE));
		context.put ("value_CheckAddDueDate", state.getAttribute (ResourceProperties.NEW_ASSIGNMENT_CHECK_ADD_DUE_DATE));
		context.put ("value_CheckAutoAnnounce", state.getAttribute (ResourceProperties.NEW_ASSIGNMENT_CHECK_AUTO_ANNOUNCE));
		context.put ("value_CheckAddHonorPledge", state.getAttribute (NEW_ASSIGNMENT_CHECK_ADD_HONOR_PLEDGE));
		
		context.put ("monthTable", monthTable ());
		context.put ("gradeTypeTable", gradeTypeTable ());
		context.put ("submissionTypeTable", submissionTypeTable ());
		context.put ("hide_assignment_option_flag", state.getAttribute (NEW_ASSIGNMENT_HIDE_OPTION_FLAG));
		context.put ("attachments", state.getAttribute (ATTACHMENTS));
		context.put ("contentTypeImageService", state.getAttribute (STATE_CONTENT_TYPE_IMAGE_SERVICE));
	
		context.put ("preview_assignment_assignment_hide_flag", state.getAttribute (PREVIEW_ASSIGNMENT_ASSIGNMENT_HIDE_FLAG));
		context.put ("preview_assignment_student_view_hide_flag", state.getAttribute (PREVIEW_ASSIGNMENT_STUDENT_VIEW_HIDE_FLAG));
		context.put ("value_assignment_id", state.getAttribute (PREVIEW_ASSIGNMENT_ASSIGNMENT_ID));
		context.put ("value_assignmentcontent_id", state.getAttribute (PREVIEW_ASSIGNMENT_ASSIGNMENTCONTENT_ID));

		context.put ("currentTime", TimeService.newTime());
		
		String template = (String) getContext(data).get("template");
		return template + TEMPLATE_INSTRUCTOR_PREVIEW_ASSIGNMENT;
		
	}	// build_instructor_preview_assignment_context
	
	/**
	 * build the instructor view to edit an new assignment
	 */
	protected String build_instructor_edit_assignment_context (	VelocityPortlet portlet,
																Context context,
																RunData data,
																SessionState state)
	{	
		try
		{
			context.put ("assignment", AssignmentService.getAssignment ((String) state.getAttribute (EDIT_ASSIGNMENT_ID)));
		}
		catch (IdUnusedException e )
		{
			addAlert(state, "Can not find the assignment. ");
		}
		catch (PermissionException e)
		{
			addAlert(state, "You are not allowed to view the assignment. ");
		}
		
		// put the names and values into vm file
		context.put ("name_title", NEW_ASSIGNMENT_TITLE);
		
		context.put ("name_OpenMonth", NEW_ASSIGNMENT_OPENMONTH);
		context.put ("name_OpenDay", NEW_ASSIGNMENT_OPENDAY);
		context.put ("name_OpenYear", NEW_ASSIGNMENT_OPENYEAR);
		context.put ("name_OpenHour", NEW_ASSIGNMENT_OPENHOUR);
		context.put ("name_OpenMin", NEW_ASSIGNMENT_OPENMIN);
		context.put ("name_OpenAMPM", NEW_ASSIGNMENT_OPENAMPM);
		
		context.put ("name_DueMonth", NEW_ASSIGNMENT_DUEMONTH);
		context.put ("name_DueDay", NEW_ASSIGNMENT_DUEDAY);
		context.put ("name_DueYear", NEW_ASSIGNMENT_DUEYEAR);
		context.put ("name_DueHour", NEW_ASSIGNMENT_DUEHOUR);
		context.put ("name_DueMin", NEW_ASSIGNMENT_DUEMIN);
		context.put ("name_DueAMPM", NEW_ASSIGNMENT_DUEAMPM);
		
		context.put	("name_EnableCloseDate", NEW_ASSIGNMENT_ENABLECLOSEDATE);
		context.put ("name_CloseMonth", NEW_ASSIGNMENT_CLOSEMONTH);
		context.put ("name_CloseDay", NEW_ASSIGNMENT_CLOSEDAY);
		context.put ("name_CloseYear", NEW_ASSIGNMENT_CLOSEYEAR);
		context.put ("name_CloseHour", NEW_ASSIGNMENT_CLOSEHOUR);
		context.put ("name_CloseMin", NEW_ASSIGNMENT_CLOSEMIN);
		context.put ("name_CloseAMPM", NEW_ASSIGNMENT_CLOSEAMPM);
		
		context.put ("name_Section", NEW_ASSIGNMENT_SECTION);
		context.put ("name_SubmissionType", NEW_ASSIGNMENT_SUBMISSION_TYPE);
		context.put ("name_GradeType", NEW_ASSIGNMENT_GRADE_TYPE);
		context.put ("name_GradePoints", NEW_ASSIGNMENT_GRADE_POINTS);
		context.put ("name_Description", NEW_ASSIGNMENT_DESCRIPTION);
		context.put ("name_ReleaseGradeType", NEW_ASSIGNMENT_RELEASE_GRADE_TYPE);
		context.put ("name_CheckAddDueDate", ResourceProperties.NEW_ASSIGNMENT_CHECK_ADD_DUE_DATE);
		context.put ("name_CheckAutoAnnounce", ResourceProperties.NEW_ASSIGNMENT_CHECK_AUTO_ANNOUNCE);
		context.put ("name_CheckAddHonorPledge", NEW_ASSIGNMENT_CHECK_ADD_HONOR_PLEDGE);
		
		// set the values
		context.put ("value_title", state.getAttribute (NEW_ASSIGNMENT_TITLE));
		context.put ("value_OpenMonth", state.getAttribute (NEW_ASSIGNMENT_OPENMONTH));
		context.put ("value_OpenDay", state.getAttribute (NEW_ASSIGNMENT_OPENDAY));
		context.put ("value_OpenYear", state.getAttribute (NEW_ASSIGNMENT_OPENYEAR));
		context.put ("value_OpenHour", state.getAttribute (NEW_ASSIGNMENT_OPENHOUR));
		context.put ("value_OpenMin", state.getAttribute (NEW_ASSIGNMENT_OPENMIN));
		context.put ("value_OpenAMPM", state.getAttribute (NEW_ASSIGNMENT_OPENAMPM));
		
		context.put ("value_DueMonth", state.getAttribute (NEW_ASSIGNMENT_DUEMONTH));
		context.put ("value_DueDay", state.getAttribute (NEW_ASSIGNMENT_DUEDAY));
		context.put ("value_DueYear", state.getAttribute (NEW_ASSIGNMENT_DUEYEAR));
		context.put ("value_DueHour", state.getAttribute (NEW_ASSIGNMENT_DUEHOUR));
		context.put ("value_DueMin", state.getAttribute (NEW_ASSIGNMENT_DUEMIN));
		context.put ("value_DueAMPM", state.getAttribute (NEW_ASSIGNMENT_DUEAMPM));
		
		context.put	("value_EnableCloseDate", state.getAttribute (NEW_ASSIGNMENT_ENABLECLOSEDATE));
		if (!((Boolean) state.getAttribute (NEW_ASSIGNMENT_ENABLECLOSEDATE)).booleanValue())
		{
			addAlert(state, "Accept submissions deadline has not been specified. Please choose a date for it and post.");
		}
		context.put ("value_CloseMonth", state.getAttribute (NEW_ASSIGNMENT_CLOSEMONTH));
		context.put ("value_CloseDay", state.getAttribute (NEW_ASSIGNMENT_CLOSEDAY));
		context.put ("value_CloseYear", state.getAttribute (NEW_ASSIGNMENT_CLOSEYEAR));
		context.put ("value_CloseHour", state.getAttribute (NEW_ASSIGNMENT_CLOSEHOUR));
		context.put ("value_CloseMin", state.getAttribute (NEW_ASSIGNMENT_CLOSEMIN));
		context.put ("value_CloseAMPM", state.getAttribute (NEW_ASSIGNMENT_CLOSEAMPM));
		
		context.put ("value_Sections", state.getAttribute (NEW_ASSIGNMENT_SECTION));
		context.put ("value_SubmissionType", state.getAttribute (NEW_ASSIGNMENT_SUBMISSION_TYPE));
		context.put ("value_GradeType", state.getAttribute (NEW_ASSIGNMENT_GRADE_TYPE));
		String maxGrade = (String) state.getAttribute (NEW_ASSIGNMENT_GRADE_POINTS);
		context.put ("value_GradePoints", displayGrade(state, maxGrade));
		context.put ("value_Description", state.getAttribute (NEW_ASSIGNMENT_DESCRIPTION));
		context.put ("value_ReleaseGradeType", state.getAttribute (NEW_ASSIGNMENT_RELEASE_GRADE_TYPE));
		context.put ("value_CheckAddDueDate", state.getAttribute (ResourceProperties.NEW_ASSIGNMENT_CHECK_ADD_DUE_DATE));
		context.put ("value_CheckAutoAnnounce", state.getAttribute (ResourceProperties.NEW_ASSIGNMENT_CHECK_AUTO_ANNOUNCE));
		String s = (String) state.getAttribute (NEW_ASSIGNMENT_CHECK_ADD_HONOR_PLEDGE);
		if (s == null)
			s = "1";
		context.put ("value_CheckAddHonorPledge", s);
		
		context.put ("monthTable", monthTable ());
		context.put ("gradeTypeTable", gradeTypeTable ());
		context.put ("submissionTypeTable", submissionTypeTable ());
		context.put ("hide_assignment_option_flag", state.getAttribute (NEW_ASSIGNMENT_HIDE_OPTION_FLAG));
		context.put ("attachments", state.getAttribute (ATTACHMENTS));
		context.put ("contentTypeImageService", state.getAttribute (STATE_CONTENT_TYPE_IMAGE_SERVICE));
		
		String template = (String) getContext(data).get("template");
		return template + TEMPLATE_INSTRUCTOR_EDIT_ASSIGNMENT;
		
	}	// build_instructor_edit_assignment_context
	
	/**
	 * build the instructor view to delete an assignment
	 */
	protected String build_instructor_delete_assignment_context (	VelocityPortlet portlet,
																	Context context,
																	RunData data,
																	SessionState state)
	{
		Vector assignments = new Vector ();
		Vector assignmentIds = (Vector) state.getAttribute (DELETE_ASSIGNMENT_IDS);
		for (int i=0; i<assignmentIds.size (); i++)
		{
			try
			{
				Assignment a = AssignmentService.getAssignment ((String) assignmentIds.get (i));
				
				Iterator submissions = AssignmentService.getSubmissions (a);
				if (submissions.hasNext ())
				{
					// if there is submission to the assignment, show the alert
					addAlert(state, "Are you sure you want to delete " + a.getTitle () + " which has submissions already? ");
				}
				assignments.add (a);
			}
			catch (IdUnusedException e )
			{
				addAlert(state, "Can not find the assignment. ");
			}
			catch (PermissionException e)
			{
				addAlert(state, "You are not allowed to view the assignment. ");
			}
		}
		context.put ("assignments", assignments);
		context.put ("service", AssignmentService.getInstance());
		context.put ("currentTime", TimeService.newTime());
		
		String template = (String) getContext(data).get("template");
		return template + TEMPLATE_INSTRUCTOR_DELETE_ASSIGNMENT;
		
	}	// build_instructor_delete_assignment_context
	
	/**
	 * build the instructor view to grade an submission
	 */
	protected String build_instructor_grade_submission_context (VelocityPortlet portlet,
																Context context,
																RunData data,
																SessionState state)
	{
		// assignment
		try
		{
			context.put ("assignment", AssignmentService.getAssignment ((String) state.getAttribute (GRADE_SUBMISSION_ASSIGNMENT_ID)));
		}
		catch (IdUnusedException e )
		{
			addAlert(state, "Can not find the assignment submission. ");
		}
		catch (PermissionException e)
		{
			addAlert(state, "You are not allowed to view the assignment submission. ");
		}
		
		// assignment submission
		try
		{
			context.put ("submission", AssignmentService.getSubmission ((String) state.getAttribute (GRADE_SUBMISSION_SUBMISSION_ID)));
		}
		catch (IdUnusedException e )
		{
			addAlert(state, "Can not find the assignment submission. ");
		}
		catch (PermissionException e)
		{
			addAlert(state, "You are not allowed to view the assignment submission. ");
		}
		
		context.put ("user", state.getAttribute (STATE_USER));
		context.put ("submissionTypeTable", submissionTypeTable ());
		context.put ("gradeTypeTable", gradeTypeTable ());
		context.put ("instructorAttachments", state.getAttribute (ATTACHMENTS));
		context.put ("contentTypeImageService", state.getAttribute (STATE_CONTENT_TYPE_IMAGE_SERVICE));
		context.put ("service", AssignmentService.getInstance());
		
		// names
		context.put ("name_grade_assignment_id", GRADE_SUBMISSION_ASSIGNMENT_ID);
		context.put ("name_feedback_comment", GRADE_SUBMISSION_FEEDBACK_COMMENT);
		context.put ("name_feedback_text", GRADE_SUBMISSION_FEEDBACK_TEXT);
		context.put ("name_feedback_attachment", GRADE_SUBMISSION_FEEDBACK_ATTACHMENT);
		context.put ("name_grade", GRADE_SUBMISSION_GRADE);
		
		// values
		context.put ("value_grade_assignment_id", state.getAttribute (GRADE_SUBMISSION_ASSIGNMENT_ID));
		context.put ("value_feedback_comment", state.getAttribute (GRADE_SUBMISSION_FEEDBACK_COMMENT));
		context.put ("value_feedback_text", state.getAttribute (GRADE_SUBMISSION_FEEDBACK_TEXT));
		context.put ("value_feedback_attachment", state.getAttribute (ATTACHMENTS));
		
		// format to show one decimal place in grade
		context.put ("value_grade", displayGrade(state, (String) state.getAttribute (GRADE_SUBMISSION_GRADE)));
		
		context.put("assignment_expand_flag", state.getAttribute(GRADE_SUBMISSION_ASSIGNMENT_EXPAND_FLAG));
		context.put ("gradingAttachments", state.getAttribute (ATTACHMENTS));
		
		String template = (String) getContext(data).get("template");
		return template + TEMPLATE_INSTRUCTOR_GRADE_SUBMISSION;
		
	}	// build_instructor_grade_submission_context
	
	/**
	 * build the instructor preview of grading submission
	 */
	protected String build_instructor_preview_grade_submission_context (VelocityPortlet portlet,
																		Context context,
																		RunData data,
																		SessionState state)
	{
		// assignment
		try
		{
			context.put ("assignment", AssignmentService.getAssignment ((String) state.getAttribute (GRADE_SUBMISSION_ASSIGNMENT_ID)));
		}
		catch (IdUnusedException e )
		{
			addAlert(state, "Can not find the assignment. ");
		}
		catch (PermissionException e)
		{
			addAlert(state, "You are not allowed to view the assignment. ");
		}
		
		// submission
		try
		{
			context.put ("submission", AssignmentService.getSubmission ((String) state.getAttribute (GRADE_SUBMISSION_SUBMISSION_ID)));
		}
		catch (IdUnusedException e )
		{
			addAlert(state, "Can not find the assignment submission. ");
		}
		catch (PermissionException e)
		{
			addAlert(state, "You are not allowed to view the assignment submission. ");
		}
		
		User user = (User) state.getAttribute (STATE_USER);
		context.put ("user", user);
		context.put ("submissionTypeTable", submissionTypeTable ());
		context.put ("gradeTypeTable", gradeTypeTable ());
		context.put ("contentTypeImageService", state.getAttribute (STATE_CONTENT_TYPE_IMAGE_SERVICE));
		context.put ("service", AssignmentService.getInstance());
		
		// filter the feedback text for the instructor comment and mark it as red
		String feedbackText = (String) state.getAttribute (GRADE_SUBMISSION_FEEDBACK_TEXT);
		context.put ("feedback_comment", state.getAttribute (GRADE_SUBMISSION_FEEDBACK_COMMENT));
		context.put ("feedback_text", feedbackText);
		context.put ("feedback_attachment", state.getAttribute (GRADE_SUBMISSION_FEEDBACK_ATTACHMENT));

		// format to show one decimal place
		String grade = (String) state.getAttribute (GRADE_SUBMISSION_GRADE);
		if (grade != null)
		{
			try 
			{
				Integer.parseInt(grade);
				if (grade.indexOf(".") == -1 && (grade.length() >= 1))
				{
					grade = grade.substring(0, grade.length()-1) + "." + grade.substring(grade.length()-1);
				}
			}
			catch (NumberFormatException e){}
		}
		else
		{
			grade = "";
		}
		context.put ("grade", grade);
		
		context.put ("comment_open", COMMENT_OPEN);
		context.put ("comment_close", COMMENT_CLOSE);
		
		String template = (String) getContext(data).get("template");
		return template + TEMPLATE_INSTRUCTOR_PREVIEW_GRADE_SUBMISSION;
		
	}	// build_instructor_preview_grade_submission_context
	
	/**
	 * build the instructor view to grade an assignment
	 */
	protected String build_instructor_grade_assignment_context (VelocityPortlet portlet,
																Context context,
																RunData data,
																SessionState state)
	{
		context.put ("user", state.getAttribute (STATE_USER));
		
		try
		{
			Assignment a = AssignmentService.getAssignment ((String) state.getAttribute (EXPORT_ASSIGNMENT_REF));
			context.put ("assignment", a);
			state.setAttribute(EXPORT_ASSIGNMENT_ID, a.getId());
			
			String sortedSubmissionBy = (String) state.getAttribute (SORTED_SUBMISSION_BY);
			String sortedSubmissionAsc = (String) state.getAttribute (SORTED_SUBMISSION_ASC);
			context.put("sortedBy", sortedSubmissionBy);
			context.put("sortedAsc", sortedSubmissionAsc);
			List submissions = prepPage(state);
			context.put("submissions", submissions);
		}
		catch (IdUnusedException e )
		{
			addAlert(state, "Can not find the assignment. ");
		}
		catch (PermissionException e)
		{
			addAlert(state, "You are not allowed to view the assignment. ");
		}
		
		context.put ("submissionTypeTable", submissionTypeTable ());
		context.put ("gradeTypeTable", gradeTypeTable ());
		context.put ("attachments", state.getAttribute (ATTACHMENTS));
		context.put ("contentTypeImageService", state.getAttribute (STATE_CONTENT_TYPE_IMAGE_SERVICE));
		context.put ("service", AssignmentService.getInstance());
		
		context.put ("assignment_expand_flag", state.getAttribute (GRADE_ASSIGNMENT_EXPAND_FLAG));
		context.put ("submission_expand_flag", state.getAttribute (GRADE_SUBMISSION_EXPAND_FLAG));
		
		// the user directory service
		UserDirectoryService uService = (UserDirectoryService) state.getAttribute (STATE_USER_DIRECTORY_SERVICE);
		context.put ("userDirectoryService", uService);
		
		// build the menu
		buildMenu (portlet, context, data, state, "listSubmissionsForm", true, true, false, "listSubmission");
		
		add2ndToolbarFields(data, context);
		
		String template = (String) getContext(data).get("template");
		return template + TEMPLATE_INSTRUCTOR_GRADE_ASSIGNMENT;
		
	}	// build_instructor_grade_assignment_context
	
	/**
	 * build the instructor view of an assignment
	 */
	protected String build_instructor_view_assignment_context(	VelocityPortlet portlet,
																Context context,
																RunData data,
																SessionState state)
	{
		try
		{
			context.put ("assignment", AssignmentService.getAssignment ((String) state.getAttribute (VIEW_ASSIGNMENT_ID)));
		}
		catch (IdUnusedException e )
		{
			addAlert(state, "Can not find the assignment. ");
		}
		catch (PermissionException e)
		{
			addAlert(state, "You are not allowed to view the assignment. ");
		}
		
		context.put ("currentTime", TimeService.newTime());
		context.put ("submissionTypeTable", submissionTypeTable ());
		context.put ("gradeTypeTable", gradeTypeTable ());
		context.put ("hideAssignmentFlag", state.getAttribute (VIEW_ASSIGNMENT_HIDE_ASSIGNMENT_FLAG));
		context.put ("hideStudentViewFlag", state.getAttribute (VIEW_ASSIGNMENT_HIDE_STUDENT_VIEW_FLAG));
		context.put ("contentTypeImageService", state.getAttribute (STATE_CONTENT_TYPE_IMAGE_SERVICE));
		
		// the user directory service
		UserDirectoryService uService = (UserDirectoryService) state.getAttribute (STATE_USER_DIRECTORY_SERVICE);
		context.put ("userDirectoryService", uService);
		
		String template = (String) getContext(data).get("template");
		return template + TEMPLATE_INSTRUCTOR_VIEW_ASSIGNMENT;
		
	}	// build_instructor_view_assignment_context
	
	/**
	 * build the instructor view to view the list of students for an assignment
	 */
	protected String build_instructor_view_students_assignment_context (VelocityPortlet portlet,
																		Context context,
																		RunData data,
																		SessionState state)
	{
		Iterator assignments = AssignmentService.getAssignmentsForContext ((String) state.getAttribute (STATE_CONTEXT_STRING));
		Vector assignmentsVector = new Vector ();
		while (assignments.hasNext ())
		{
			assignmentsVector.add (assignments.next ());
		}
		
		List studentMembers = prepPage(state);
		context.put ("studentMembers", studentMembers);
		context.put ("assignmentService", AssignmentService.getInstance());
		context.put ("assignments", assignmentsVector);
		context.put ("studentListShowTable", (Hashtable) state.getAttribute (STUDENT_LIST_SHOW_TABLE));
		
		// build the menu
		buildMenu (portlet, context, data, state, "studentAssignmentForm", true, true, false, "studentAssignment");
		
		add2ndToolbarFields(data, context);
		
		String template = (String) getContext(data).get("template");
		return template + TEMPLATE_INSTRUCTOR_VIEW_STUDENTS_ASSIGNMENT;
		
	}	// build_instructor_view_students_assignment_context
	
	/**
	 * build the instructor view to report the submissions
	 */
	protected String build_instructor_report_submissions (	VelocityPortlet portlet,
															Context context,
															RunData data,
															SessionState state)
	{		
		context.put ("submissions", prepPage(state));
		
		// build the menu
		buildMenu (portlet, context, data, state, "reportForm", true, true, false, "report");
		context.put ("sortedBy", (String) state.getAttribute (SORTED_SUBMISSION_BY));
		context.put ("sortedAsc", (String) state.getAttribute (SORTED_SUBMISSION_ASC));
		
		add2ndToolbarFields(data, context);
		
		String template = (String) getContext(data).get("template");
		return template + TEMPLATE_INSTRUCTOR_REPORT_SUBMISSIONS;
		
	}	// build_instructor_report_submissions
	
	/**
	 *	build menu
	 */
	private static void buildMenu (	VelocityPortlet portlet,
	Context context,
	RunData data,
	SessionState state,
	String formName,
	boolean allowAddAssignment,
	boolean instructorView,
	boolean hasNextAssignment,
	String viewMode)
	{
		String contextString = (String) state.getAttribute (STATE_CONTEXT_STRING);
				
		// build a test menu
		Menu bar = new Menu (portlet, data, "AssignmentAction");
		
		if (instructorView)
		{
			if (viewMode.equalsIgnoreCase ("listSubmission"))
			{
				// in instructor list assignment submission view
				bar.add ( new MenuEntry ("New...", null, true, MenuItem.CHECKED_NA, "doNew_assignment", formName) );
				bar.add ( new MenuEntry ("Release Grades", null, true, MenuItem.CHECKED_NA, "doRelease_grades", formName));
				String accessPointUrl = (ServerConfigurationService.getAccessUrl()).concat(AssignmentService.submissionsZipReference(contextString, (String) state.getAttribute (EXPORT_ASSIGNMENT_REF)));
				bar.add ( new MenuEntry ("Download All", "").setUrl(accessPointUrl));	
			}
			else if (viewMode.equalsIgnoreCase ("report"))
			{
				bar.add ( new MenuEntry ("New...", null, true, MenuItem.CHECKED_NA, "doNew_assignment", formName) );
				String accessPointUrl = ServerConfigurationService.getAccessUrl() + AssignmentService.gradesSpreadsheetReference(contextString, null);
				bar.add ( new MenuEntry ("Download Spreadsheet", "").setUrl(accessPointUrl));	
			}
			else if (viewMode.equalsIgnoreCase ("studentAssignment"))
			{
				bar.add ( new MenuEntry ("New...", null, true, MenuItem.CHECKED_NA, "doNew_assignment", formName) );
			}
			else
			{
				// in the instructor views
				bar.add ( new MenuEntry ("New...", null, true, MenuItem.CHECKED_NA, "doNew_assignment", formName) );
				bar.add ( new MenuEntry ("Revise", null, hasNextAssignment, MenuItem.CHECKED_NA, "doEdit_assignment", formName) );
				bar.add ( new MenuEntry ("Delete", null, hasNextAssignment, MenuItem.CHECKED_NA, "doDelete_confirm_assignment", formName) );
				bar.add ( new MenuEntry ("Duplicate", null,hasNextAssignment, MenuItem.CHECKED_NA, "doDuplicate_assignment", formName) );
				bar.add ( new MenuEntry ("Grade", null, hasNextAssignment, MenuItem.CHECKED_NA, "doGrade_assignment", formName) );
			}
		}
		else
		{
			// in the student views
			if (allowAddAssignment)
			{
			}
		}
		
		// add permissions, if allowed
		if (SiteService.allowUpdateSite(PortalService.getCurrentSiteId()))
		{
			bar.add( new MenuEntry("Permissions...", "doPermissions") );
		}

		// Set menu state attribute
		SessionState stateForMenus = ((JetspeedRunData)data).getPortletSessionState (portlet.getID ());
		stateForMenus.setAttribute (MenuItem.STATE_MENU, bar);
		
		List items = bar.getItems();
		if (!items.isEmpty())
		{
			context.put (Menu.CONTEXT_MENU, bar);
			context.put (Menu.CONTEXT_ACTION, "AssignmentAction");
		}
		
	}	// buildMenu
	
	/**
	 * Go to the instructor view
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doView_instructor (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_LIST_ASSIGNMENTS);
		state.setAttribute (SORTED_BY, SORTED_BY_DUEDATE);
		state.setAttribute(SORTED_ASC, Boolean.TRUE.toString());
		
	}	//doView_instructor
	
	/**
	 * Go to the student view
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doView_student (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// to the student list of assignment view
		state.setAttribute (SORTED_BY, SORTED_BY_DUEDATE);
		state.setAttribute (STATE_MODE, MODE_STUDENT_LIST_ASSIGNMENTS);
		
	}	//doView_student
	
	/**
	 * Action is to view the content of one specific assignment submission
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doView_submission (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// reset the submission context
		resetViewSubmission(state);
		
		ParameterParser params = data.getParameters ();
		String assignmentReference = params.getString ("assignmentReference");
		state.setAttribute (VIEW_SUBMISSION_ASSIGNMENT_REFERENCE, assignmentReference);
		
		User u = (User) state.getAttribute (STATE_USER);

		try
		{
			AssignmentSubmission submission= AssignmentService.getSubmission (assignmentReference, u);

			if (submission!=null)
			{
				state.setAttribute (VIEW_SUBMISSION_TEXT, submission.getSubmittedText ());
				state.setAttribute (VIEW_SUBMISSION_HONOR_PLEDGE_YES, (new Boolean (submission.getHonorPledgeFlag ())).toString ());
				ReferenceVector v = new ReferenceVector ();
				Iterator l = submission.getSubmittedAttachments ().iterator ();
				while (l.hasNext ())
				{
					v.add (l.next ());
				}
				state.setAttribute (ATTACHMENTS, v);
			}
			else
			{
				state.setAttribute (VIEW_SUBMISSION_HONOR_PLEDGE_YES, "false");
				state.setAttribute (ATTACHMENTS, new ReferenceVector ());
			}	

			state.setAttribute (STATE_MODE, MODE_STUDENT_VIEW_SUBMISSION);
		}
		catch (IdUnusedException e )
		{
			addAlert(state, "Can not find the assignment submission. ");
		}
		catch (PermissionException e)
		{
			addAlert(state, "You are not allowed to view the assignment submission. ");
		}	//	try
		
	}	//doView_submission
	
	/**
	 * Preview of the submission
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doPreview_submission (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		ParameterParser params = data.getParameters ();
		//String assignmentId = params.getString(assignmentId);
		state.setAttribute (PREVIEW_SUBMISSION_ASSIGNMENT_REFERENCE, state.getAttribute (VIEW_SUBMISSION_ASSIGNMENT_REFERENCE));
		
		// retrieve the submission text (as formatted text)
		String text = processFormattedTextFromBrowser(state, params.getCleanString(VIEW_SUBMISSION_TEXT));

		state.setAttribute (PREVIEW_SUBMISSION_TEXT, text);
		state.setAttribute (VIEW_SUBMISSION_TEXT, text);
		
		// assign the honor pledge attribute
		String honor_pledge_yes = params.getString (VIEW_SUBMISSION_HONOR_PLEDGE_YES);
		if (honor_pledge_yes==null)
		{
			honor_pledge_yes = "false";
		}
		state.setAttribute (PREVIEW_SUBMISSION_HONOR_PLEDGE_YES, honor_pledge_yes);
		state.setAttribute (VIEW_SUBMISSION_HONOR_PLEDGE_YES, honor_pledge_yes);
		
		state.setAttribute (PREVIEW_SUBMISSION_ATTACHMENTS, state.getAttribute (ATTACHMENTS));
		state.setAttribute (STATE_MODE, MODE_STUDENT_PREVIEW_SUBMISSION);
		
	}	//doPreview_submission
	
	/**
	 * Preview of the grading of submission
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doPreview_grade_submission (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		ParameterParser params = data.getParameters ();

		String feedbackText = processFormattedTextFromBrowser(state, params.getCleanString(GRADE_SUBMISSION_FEEDBACK_TEXT));		
		String grade = params.getString (GRADE_SUBMISSION_GRADE);
		state.setAttribute (GRADE_SUBMISSION_GRADE, grade);
		String feedbackComment = processFormattedTextFromBrowser(state, params.getCleanString (GRADE_SUBMISSION_FEEDBACK_COMMENT));
		state.setAttribute (GRADE_SUBMISSION_FEEDBACK_COMMENT, feedbackComment);
		state.setAttribute (GRADE_SUBMISSION_FEEDBACK_TEXT,  feedbackText);
		state.setAttribute (GRADE_SUBMISSION_FEEDBACK_ATTACHMENT,  state.getAttribute (ATTACHMENTS));
		if (grade != null && !grade.equals(""))
		{
			state.setAttribute (GRADE_SUBMISSION_GRADE, grade);
		}
		else
		{
			addAlert(state, "Please specify the grade. ");
		}
				
		if (state.getAttribute(STATE_MESSAGE) == null)
		{
			String sId = (String) state.getAttribute (GRADE_SUBMISSION_SUBMISSION_ID);
			try
			{
				AssignmentSubmission s = AssignmentService.getSubmission (sId);
				Assignment a = s.getAssignment ();
				if (a.getContent ().getTypeOfGrade ()==3)
				{
					if ((grade!=null) && (grade.length ()!=0))
					{			
						validPointGrade(state, grade);
						state.setAttribute (GRADE_SUBMISSION_GRADE, grade);
						if (state.getAttribute(STATE_MESSAGE) == null)
						{					
							grade = scalePointGrade(state, grade);
						}
						if (state.getAttribute(STATE_MESSAGE) == null)
						{
							try
							{
								int l = Integer.parseInt (grade);
								if (l > a.getContent ().getMaxGradePoint ())
								{
									addAlert(state, "Grade can not be larger than the maximum grade. ");
								}
							}
							catch (NumberFormatException e){}
						}
					}
				}
			}
			catch (IdUnusedException e)
			{
				addAlert(state, "Can not find the assignment. ");
			}
			catch (PermissionException e)
			{
				addAlert(state, "You are not allowed to view the assignment submission. ");
			}
		}
		if (state.getAttribute(STATE_MESSAGE) == null)
		{
			state.setAttribute (GRADE_SUBMISSION_GRADE, grade);
			state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_PREVIEW_GRADE_SUBMISSION);
		}

	}	//doPreview_grade_submission
	
	/**
	 * Action is to end the preview submission process
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doDone_preview_submission (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// back to the student list view of assignments
		state.setAttribute (STATE_MODE, MODE_STUDENT_VIEW_SUBMISSION);
		
	}	//doDone_preview_submission
	
	/**
	 * Action is to end the view assignment process
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doDone_view_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// back to the student list view of assignments
		state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_LIST_ASSIGNMENTS);
		
	}	//doDone_view_assignments
	
	/**
	 * Action is to end the preview new assignment process
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doDone_preview_new_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// back to the new assignment page
		state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_NEW_ASSIGNMENT);
		
	}	//doDone_preview_new_assignment
	
	/**
	 * Action is to end the preview edit assignment process
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doDone_preview_edit_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// back to the edit assignment page
		state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_EDIT_ASSIGNMENT);
		
	}	//doDone_preview_edit_assignment
	
	/**
	 * Action is to end the show submission process
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doCancel_show_submission (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// reset the view assignment
		state.setAttribute (VIEW_ASSIGNMENT_ID, "");
		
		// back to the student list view of assignments
		state.setAttribute (STATE_MODE, MODE_STUDENT_LIST_ASSIGNMENTS);
		
	}	//doCancel_show_submission
	
	/**
	 * Action is to cancel the delete assignment process
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doCancel_delete_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// reset the show assignment object
		state.setAttribute (DELETE_ASSIGNMENT_IDS, new Vector ());
		
		// back to the instructor list view of assignments
		state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_LIST_ASSIGNMENTS);
		
	}	//doCancel_delete_assignment
	
	/**
	 * Action is to end the show submission process
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void  doCancel_edit_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// back to the student list view of assignments
		state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_LIST_ASSIGNMENTS);
		
	}	//	doCancel_edit_assignment
	
	/**
	 * Action is to end the show submission process
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doCancel_new_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// reset the assignment object
		resetAssignment (state);
		
		// back to the student list view of assignments
		state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_LIST_ASSIGNMENTS);
		state.setAttribute (SORTED_BY, SORTED_BY_DUEDATE);
		state.setAttribute(SORTED_ASC, Boolean.TRUE.toString());
		
	}	//doCancel_new_assignment
	
	/**
	 * Action is to cancel the grade submission process
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doCancel_grade_submission (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// reset the assignment object
		//resetAssignment (state);
		
		// back to the student list view of assignments
		state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_GRADE_ASSIGNMENT);
		
	}	//doCancel_grade_submission
	
	/**
	 * Action is to cancel the preview grade process
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doCancel_preview_grade_submission (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// back to the instructor view of grading a submission
		state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_GRADE_SUBMISSION);
		
	}	//doCancel_preview_grade_submission
	
	/**
	 * Action is to return to the view of list assignments
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doList_assignments (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// back to the student list view of assignments
		state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_LIST_ASSIGNMENTS);
		state.setAttribute (SORTED_BY, SORTED_BY_DUEDATE);
		state.setAttribute(SORTED_ASC, Boolean.TRUE.toString());
		
		
	}	//doList_assignments

	/**
	 * Action is to cancel the student view grade process
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doCancel_view_grade (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// reset the view grade submission id
		state.setAttribute (VIEW_GRADE_SUBMISSION_ID, "");
		
		// back to the student list view of assignments
		state.setAttribute (STATE_MODE, MODE_STUDENT_LIST_ASSIGNMENTS);
		
	}	//doCancel_view_grade
	
	/**
	 * Action is to save the grade to submission
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doSave_grade_submission (RunData data, Context context)
	{
		grade_submission_option(data, "save");
		
	}	// doSave_grade_submission
	
	/**
	 * Action is to release the grade to submission
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doRelease_grade_submission (RunData data, Context context)
	{
		grade_submission_option(data, "release");
		
	}	// doRelease_grade_submission
	
	/**
	 *
	 */
	public void doReturn_grade_submission (RunData data, Context context)
	{
		grade_submission_option(data, "return");
		
	}	// doReturn_grade_submission
	
	/**
	 * Common grading routine plus specific operation to differenciate cases when saving, releasing or returning grade.
	 */
	private void grade_submission_option(RunData data, String gradeOption)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		ParameterParser params = data.getParameters ();
		
		String feedbackComment = processFormattedTextFromBrowser(state, params.getCleanString (GRADE_SUBMISSION_FEEDBACK_COMMENT));
		if (feedbackComment != null)
		{
			state.setAttribute (GRADE_SUBMISSION_FEEDBACK_COMMENT,  feedbackComment);
		}
		
		String feedbackText = processFormattedTextFromBrowser(state, params.getCleanString (GRADE_SUBMISSION_FEEDBACK_TEXT));
		if (feedbackText != null)
		{
			state.setAttribute (GRADE_SUBMISSION_FEEDBACK_TEXT,  feedbackText);
		}	

		state.setAttribute (GRADE_SUBMISSION_FEEDBACK_ATTACHMENT,  state.getAttribute (ATTACHMENTS));
		
		String g = params.getCleanString (GRADE_SUBMISSION_GRADE);
		if (g != null)
		{
			state.setAttribute (GRADE_SUBMISSION_GRADE, g);
		}
		
		String sId = (String) state.getAttribute (GRADE_SUBMISSION_SUBMISSION_ID);
		try
		{
			// for points grading, one have to enter number as the points
			String grade = (String) state.getAttribute (GRADE_SUBMISSION_GRADE);

			Assignment a = AssignmentService.getSubmission(sId).getAssignment ();
			int typeOfGrade = a.getContent ().getTypeOfGrade ();
			if (typeOfGrade==3)
			{
				if ((grade.length ()==0))
				{
					if (gradeOption.equals("release"))
					{
						// in case of releasing grade, user must specify a grade
						addAlert(state, "Please specify the grade. ");
					}
				}
				else
				{
					// the preview grade process might already scaled up the grade by 10
					if (!((String) state.getAttribute(STATE_MODE)).equals(MODE_INSTRUCTOR_PREVIEW_GRADE_SUBMISSION))
					{					
						validPointGrade(state, grade);
						state.setAttribute (GRADE_SUBMISSION_GRADE, grade);
						if (state.getAttribute(STATE_MESSAGE) == null)
						{
							grade = scalePointGrade(state, grade);
						}
					}	
					
					if (state.getAttribute(STATE_MESSAGE) == null)
					{
						int maxGrade = a.getContent ().getMaxGradePoint (); 
						if (Integer.parseInt(grade) > maxGrade)
						{
							addAlert(state, "Grade can not be larger than the maximum grade. ");
						}
					}
				}
			}

			// if ungraded and grade type is not "ungraded" type
			if (( grade == null || grade.equals ("ungraded")) && (typeOfGrade!=1) && gradeOption.equals("release"))
			{
				addAlert(state, "Please specify the grade. ");
			}

			if (state.getAttribute(STATE_MESSAGE) == null)
			{
				state.setAttribute (GRADE_SUBMISSION_GRADE, grade);
				AssignmentSubmissionEdit sEdit = AssignmentService.editSubmission (sId);

				if (grade == null)
				{
					sEdit.setGrade ("");
					sEdit.setGraded(false);
					sEdit.setGradeReleased(false);
				}
				else
				{
					if (typeOfGrade==1)
					{
						sEdit.setGrade ("no grade");
						sEdit.setGraded (true);
					}
					else
					{
						if (!grade.equals(""))
						{
							if (typeOfGrade == 3)
							{
								sEdit.setGrade(grade);
							}
							else
							{
								sEdit.setGrade (grade);
							}
							sEdit.setGraded (true);
						}
					}
				}
				
				if (gradeOption.equals("release"))
				{
					sEdit.setGradeReleased (true);
					// clear the returned flag
					sEdit.setReturned(false);
					sEdit.setTimeReturned(null);
				}
				else if (gradeOption.equals("return"))
				{
					sEdit.setReturned (true);
					sEdit.setTimeReturned(TimeService.newTime());
					sEdit.setGradeReleased(false);
					sEdit.setHonorPledgeFlag (Boolean.FALSE.booleanValue ());
				}
		
				sEdit.setFeedbackComment ((String) state.getAttribute (GRADE_SUBMISSION_FEEDBACK_COMMENT));
				sEdit.setFeedbackText ((String) state.getAttribute (GRADE_SUBMISSION_FEEDBACK_TEXT));

				ReferenceVector v = (ReferenceVector) state.getAttribute(GRADE_SUBMISSION_FEEDBACK_ATTACHMENT);
				if (v!=null)
				{
					// clear the old attachments first
					sEdit.clearFeedbackAttachments ();

					for (int i=0; i<v.size (); i++)
					{
						sEdit.addFeedbackAttachment ((Reference) v.get (i));
					}
				}

				AssignmentService.commitEdit (sEdit);
			}	// if
		}
		catch (IdUnusedException e)
		{
		}
		catch (PermissionException e)
		{
		}
		catch (InUseException e)
		{
			addAlert(state, INUSE_ERROR_MESSAGE + "submission. ");
		}	// try
		
		if ( state.getAttribute(STATE_MESSAGE) == null)
		{
			state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_GRADE_ASSIGNMENT);
			state.setAttribute (ATTACHMENTS, new ReferenceVector ());
		}
	
	}	// grade_submission_option
	
	/**
	 * Action is to save the submission as a draft
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doSave_submission (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		ParameterParser params = data.getParameters ();
		
		// retrieve the submission text (as formatted text)
		String text = processFormattedTextFromBrowser(state, params.getCleanString(VIEW_SUBMISSION_TEXT));
			
		if (text == null)
		{
			text = (String) state.getAttribute(VIEW_SUBMISSION_TEXT);
		}

		String honorPledgeYes = params.getString (VIEW_SUBMISSION_HONOR_PLEDGE_YES); 		
		if (honorPledgeYes==null)
		{
			honorPledgeYes = "false";
		}
		
		String assignmentId = params.getString (ASSIGNMENT_ID);
		
		String aReference = (String) state.getAttribute (VIEW_SUBMISSION_ASSIGNMENT_REFERENCE);
		User u = (User) state.getAttribute (STATE_USER);
		
		try
		{
			Assignment a = AssignmentService.getAssignment(aReference);
			
			AssignmentSubmission submission= AssignmentService.getSubmission (aReference, u);
			if (submission!=null)
			{
				// the submission already exists, change the text and honor pledge value, save as draft
				try
				{
					AssignmentSubmissionEdit edit = AssignmentService.editSubmission (submission.getReference ());
					edit.setSubmittedText (text);
					edit.setHonorPledgeFlag (Boolean.valueOf (honorPledgeYes).booleanValue ());
					edit.setSubmitted (false);
					//edit.addSubmitter (u);
					edit.setAssignment (a);
					
					// add attachments
					ReferenceVector attachments = (ReferenceVector) state.getAttribute (ATTACHMENTS);
					if (attachments != null)
					{
						// clear the old attachments first
						edit.clearSubmittedAttachments ();
						
						// add each new attachment
						Iterator it = attachments.iterator ();
						while (it.hasNext ())
						{
							edit.addSubmittedAttachment ((Reference) it.next ());
						}
					}
					AssignmentService.commitEdit (edit);
				}
				catch (IdUnusedException e)
				{
					addAlert(state, "Can not find the assignment " + a.getTitle ());
				}
				catch (PermissionException e)
				{
					addAlert(state, "You are not allowed to submit an answer. ");
				}
				catch (InUseException e)
				{
					addAlert(state, INUSE_ERROR_MESSAGE + "submission. ");
				}
			}
			else
			{
				// new submission, save as draft
				try
				{			
					AssignmentSubmissionEdit edit = AssignmentService.addSubmission ((String) state.getAttribute (STATE_CONTEXT_STRING), assignmentId);
					edit.setSubmittedText (text);
					edit.setHonorPledgeFlag (Boolean.valueOf (honorPledgeYes).booleanValue ());
					edit.setSubmitted (false);
					//edit.addSubmitter (u);
					edit.setAssignment (a);
					
					// add attachments
					ReferenceVector attachments = (ReferenceVector) state.getAttribute (ATTACHMENTS);
					if (attachments != null)
					{
						// add each attachment
						Iterator it = attachments.iterator ();
						while (it.hasNext ())
						{
							edit.addSubmittedAttachment ((Reference) it.next ());
						}
					}
					AssignmentService.commitEdit (edit);
				}
				catch(PermissionException e)
				{
					addAlert(state, "You are not allowed to create assignment submission. ");
				}
			}
		}
		catch (IdUnusedException e )
		{
			addAlert(state, "Can not find the assignment submission. ");
		}
		catch (PermissionException e)
		{
			addAlert(state, "You are not allowed to view the assignment submission. ");
		}
		
		if (state.getAttribute(STATE_MESSAGE) == null)
		{		
			state.setAttribute (STATE_MODE, MODE_STUDENT_LIST_ASSIGNMENTS);
			state.setAttribute (ATTACHMENTS, new ReferenceVector ());
		}

	}	//doSave_submission
	
	/**
	 * Action is to post the submission
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doPost_submission (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		ParameterParser params = data.getParameters ();
		
		// retrieve the submission text (as formatted text)
		String text = processFormattedTextFromBrowser(state, params.getCleanString(VIEW_SUBMISSION_TEXT));
		
		if (text == null)
		{
			text = (String) state.getAttribute(VIEW_SUBMISSION_TEXT);
		}
		else
		{
			state.setAttribute(VIEW_SUBMISSION_TEXT, text);
		}
		
		String honorPledgeYes = params.getString (VIEW_SUBMISSION_HONOR_PLEDGE_YES); 
		if (honorPledgeYes == null)
		{
			honorPledgeYes = (String) state.getAttribute(VIEW_SUBMISSION_HONOR_PLEDGE_YES);
		}
		
		if (honorPledgeYes==null)
		{
			honorPledgeYes = "false";
		}
		String assignmentId = params.getString (ASSIGNMENT_ID);
		
		User u = (User) state.getAttribute (STATE_USER);
		String aReference = (String) state.getAttribute (VIEW_SUBMISSION_ASSIGNMENT_REFERENCE);
		Assignment a = null;
		try
		{
			a = AssignmentService.getAssignment(aReference);

			if (a.getContent ().getHonorPledge ()!=1)
			{
				if (!Boolean.valueOf (honorPledgeYes).booleanValue ())
				{
					addAlert(state, "You have to check the honor pledge before submission. ");
				}
				state.setAttribute (VIEW_SUBMISSION_HONOR_PLEDGE_YES, honorPledgeYes);
			}

			// check the submission inputs based on the submission type
			int submissionType = a.getContent ().getTypeOfSubmission ();
			if (submissionType == 1)
			{
				// for the inline only submission
				if (text.length ()==0)
				{
					addAlert(state, "You must type in your answer in the text input area before submission. ");
				}
			}
			else if (submissionType == 2)
			{
				// for the attachment only submission
				Vector v = (Vector) state.getAttribute (ATTACHMENTS);
				if ((v == null) || (v.size ()==0))
				{
					addAlert(state, "You must attach at least one document before submission. ");
				}
			}else if (submissionType==3)
			{
				// for the inline and attachment submission
				Vector v = (Vector) state.getAttribute (ATTACHMENTS);
				if ((text.length ()==0) && ((v==null) || (v.size ()==0)))
				{
					addAlert(state, "You must either type in your answer in the text input or attach at least one document before submission. ");
				}
			}
		}
		catch (IdUnusedException e)
		{
			addAlert(state, "Can not find the assignment.  ");
		}
		catch(PermissionException e)
		{
			addAlert(state, "You do not have permission to view the assignment. ");
		}

		if ((state.getAttribute(STATE_MESSAGE) == null) && (a != null))
		{
			try
			{
				AssignmentSubmission submission= AssignmentService.getSubmission (a.getReference (), u);
				if (submission!=null)
				{
					// the submission already exists, change the text and honor pledge value, post it
					try
					{
						AssignmentSubmissionEdit sEdit = AssignmentService.editSubmission (submission.getReference ());
						sEdit.setSubmittedText (text);
						sEdit.setHonorPledgeFlag (Boolean.valueOf (honorPledgeYes).booleanValue ());
						sEdit.setTimeSubmitted (TimeService.newTime());
						sEdit.setSubmitted (true);
						
						// for resubmissions
						// when resubmit, keep the Returned flag on till the instructor grade again.
						if (sEdit.getReturned())
						{
							ResourcePropertiesEdit sPropertiesEdit = sEdit.getPropertiesEdit();
							if (sEdit.getGraded())
							{
								// add the current grade into previous grade histroy
								String previousGrades = (String) sEdit.getProperties().getProperty(ResourceProperties.PROP_SUBMISSION_SCALED_PREVIOUS_GRADES);
								if (previousGrades == null)
								{
									previousGrades = (String) sEdit.getProperties().getProperty(ResourceProperties.PROP_SUBMISSION_PREVIOUS_GRADES);
									if (previousGrades != null)
									{		
										int typeOfGrade = a.getContent ().getTypeOfGrade ();
										if (typeOfGrade==3)
										{
											// point grade assignment type
											// some old unscaled grades, need to scale the number and remove the old property
											String[] grades = StringUtil.split(previousGrades, " ");
											String newGrades = "";
											for (int jj = 0; jj<grades.length; jj++)
											{
												String grade = grades[jj]; 
												if (grade.indexOf(".") == -1)
												{
													// show the grade with decimal point
													grade = grade.concat(".0");
												}
												newGrades = newGrades.concat(grade + " ");
											}
											previousGrades = newGrades;
										}
										sPropertiesEdit.removeProperty(ResourceProperties.PROP_SUBMISSION_PREVIOUS_GRADES);
									}
									else
									{
										previousGrades = "";
									}
								}
								previousGrades = previousGrades.concat(sEdit.getGradeDisplay() + " ");
								
								sPropertiesEdit.addProperty(ResourceProperties.PROP_SUBMISSION_SCALED_PREVIOUS_GRADES, previousGrades);
								
								// clear the current grade and make the submission ungraded
								sEdit.setGraded(false);
								sEdit.setGrade("");
								sEdit.setGradeReleased(false);
							}
							sPropertiesEdit.addProperty(ResourceProperties.PROP_SUBMISSION_PREVIOUS_FEEDBACK_TEXT, sEdit.getFeedbackText ());
							sPropertiesEdit.addProperty(ResourceProperties.PROP_SUBMISSION_PREVIOUS_FEEDBACK_COMMENT, sEdit.getFeedbackComment ());
							
							// reset the previous grading context
							sEdit.setFeedbackText("");
							sEdit.setFeedbackComment("");
							sEdit.clearFeedbackAttachments();
						}
						//sEdit.addSubmitter (u);
						sEdit.setAssignment (a);
						
						// add attachments
						ReferenceVector attachments = (ReferenceVector) state.getAttribute (ATTACHMENTS);
						if (attachments != null)
						{
							// clear the old attachments first
							sEdit.clearSubmittedAttachments ();
							
							// add each new attachment
							Iterator it = attachments.iterator ();
							while (it.hasNext ())
							{
								sEdit.addSubmittedAttachment ((Reference) it.next ());
							}
						}
						
						AssignmentService.commitEdit (sEdit);
					}
					catch (IdUnusedException e)
					{
						addAlert(state, "Can not find the assignment " + a.getTitle ());
					}
					catch(PermissionException e)
					{
						addAlert(state, "You do not have permission to edit the submission. ");
					}
					catch (InUseException e)
					{
						addAlert(state, INUSE_ERROR_MESSAGE + "submission. ");
					}
				}
				else
				{
					// new submission, post it
					try
					{						
						AssignmentSubmissionEdit edit = AssignmentService.addSubmission ((String) state.getAttribute (STATE_CONTEXT_STRING), assignmentId);
						edit.setSubmittedText (text);
						edit.setHonorPledgeFlag (Boolean.valueOf (honorPledgeYes).booleanValue ());
						edit.setTimeSubmitted (TimeService.newTime());
						edit.setSubmitted (true);
						//edit.addSubmitter (u);
						edit.setAssignment (a);
						
						// add attachments
						ReferenceVector attachments = (ReferenceVector) state.getAttribute (ATTACHMENTS);
						if (attachments != null)
						{
							// add each attachment
							Iterator it = attachments.iterator ();
							while (it.hasNext ())
							{
								edit.addSubmittedAttachment ((Reference) it.next ());
							}
						}
						
						AssignmentService.commitEdit (edit);
					}
					catch(PermissionException e)
					{
						addAlert(state, "You are not allowed to submit an assignment. ");
					}
				}	// if -else
			}
			catch (IdUnusedException e )
			{
				addAlert(state, "Can not find the assignment submission. ");
			}
			catch (PermissionException e)
			{
				addAlert(state, "You are not allowed to view the assignment submission. ");
			}
			
		}   // if
		
		if (state.getAttribute(STATE_MESSAGE) == null)
		{
			state.setAttribute (STATE_MODE, MODE_STUDENT_LIST_ASSIGNMENTS);
			state.setAttribute (ATTACHMENTS, new ReferenceVector ());
		}

	}	//doPost_submission
	
	/**
	 * Action is to show the new assignment screen
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doNew_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		resetAssignment (state);
		state.setAttribute (ATTACHMENTS, new ReferenceVector ());		
		state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_NEW_ASSIGNMENT);
		
	}	//doNew_Assignment
	
	/**
	 * Action is to save the input infos for assignment fields
	 * @param data The RunData object
	 **/
	protected void setNewAssignmentParameters(RunData data)
	{
		// read the form inputs
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		ParameterParser params = data.getParameters ();
		
		// put the input value into the state attributes
		String title = params.getString (NEW_ASSIGNMENT_TITLE);
		state.setAttribute (NEW_ASSIGNMENT_TITLE, title);
		state.setAttribute (NEW_ASSIGNMENT_OPENMONTH, new Integer (params.getString (NEW_ASSIGNMENT_OPENMONTH)));
		state.setAttribute (NEW_ASSIGNMENT_OPENDAY, new Integer (params.getString (NEW_ASSIGNMENT_OPENDAY)));
		state.setAttribute (NEW_ASSIGNMENT_OPENYEAR, new Integer (params.getString (NEW_ASSIGNMENT_OPENYEAR)));
		state.setAttribute (NEW_ASSIGNMENT_OPENHOUR, new Integer (params.getString (NEW_ASSIGNMENT_OPENHOUR)));
		state.setAttribute (NEW_ASSIGNMENT_OPENMIN, new Integer (params.getString (NEW_ASSIGNMENT_OPENMIN)));
		state.setAttribute (NEW_ASSIGNMENT_OPENAMPM, params.getString (NEW_ASSIGNMENT_OPENAMPM));

		state.setAttribute (NEW_ASSIGNMENT_DUEMONTH, new Integer (params.getString (NEW_ASSIGNMENT_DUEMONTH)));
		state.setAttribute (NEW_ASSIGNMENT_DUEDAY, new Integer (params.getString (NEW_ASSIGNMENT_DUEDAY)));
		state.setAttribute (NEW_ASSIGNMENT_DUEYEAR, new Integer (params.getString (NEW_ASSIGNMENT_DUEYEAR)));
		state.setAttribute (NEW_ASSIGNMENT_DUEHOUR, new Integer (params.getString (NEW_ASSIGNMENT_DUEHOUR)));
		state.setAttribute (NEW_ASSIGNMENT_DUEMIN, new Integer (params.getString (NEW_ASSIGNMENT_DUEMIN)));
		state.setAttribute (NEW_ASSIGNMENT_DUEAMPM, params.getString (NEW_ASSIGNMENT_DUEAMPM));

		state.setAttribute (NEW_ASSIGNMENT_ENABLECLOSEDATE, new Boolean (true));
		state.setAttribute (NEW_ASSIGNMENT_CLOSEMONTH, new Integer (params.getString (NEW_ASSIGNMENT_CLOSEMONTH)));
		state.setAttribute (NEW_ASSIGNMENT_CLOSEDAY, new Integer (params.getString (NEW_ASSIGNMENT_CLOSEDAY)));
		state.setAttribute (NEW_ASSIGNMENT_CLOSEYEAR, new Integer (params.getString (NEW_ASSIGNMENT_CLOSEYEAR)));
		state.setAttribute (NEW_ASSIGNMENT_CLOSEHOUR, new Integer (params.getString (NEW_ASSIGNMENT_CLOSEHOUR)));
		state.setAttribute (NEW_ASSIGNMENT_CLOSEMIN, new Integer (params.getString (NEW_ASSIGNMENT_CLOSEMIN)));
		state.setAttribute (NEW_ASSIGNMENT_CLOSEAMPM, params.getString (NEW_ASSIGNMENT_CLOSEAMPM));

		// SECTION MOD
		String sections_string = "";
		String mode = (String)state.getAttribute(STATE_MODE);
		if(mode == null)
			mode = "";

		if(mode.equals(MODE_INSTRUCTOR_NEW_ASSIGNMENT))
		{
			sections_string = "001";
		}
		else if(mode.equals(MODE_INSTRUCTOR_EDIT_ASSIGNMENT))
		{
			sections_string = (String)state.getAttribute(STATE_SECTION_STRING);
		}

		state.setAttribute (NEW_ASSIGNMENT_SECTION, sections_string);
		state.setAttribute (NEW_ASSIGNMENT_SUBMISSION_TYPE, new Integer (params.getString (NEW_ASSIGNMENT_SUBMISSION_TYPE)));

		// grade type and grade points
		int gradeType = Integer.parseInt (params.getString (NEW_ASSIGNMENT_GRADE_TYPE));
		state.setAttribute (NEW_ASSIGNMENT_GRADE_TYPE, new Integer (gradeType));

		// the grade point
		String	gradePoints = params.getString (NEW_ASSIGNMENT_GRADE_POINTS);
		state.setAttribute (NEW_ASSIGNMENT_GRADE_POINTS, gradePoints);
		if (gradePoints != null)
		{
			if (gradeType == 3)
			{
				if ((gradePoints.length ()==0))
				{
					// in case of releasing grade, user must specify a grade
					addAlert(state, "Please specify the grade. ");
				}
				else
				{
					validPointGrade(state, gradePoints);
					// when scale is points, grade must be integer and less than maximum value						
					if (state.getAttribute(STATE_MESSAGE) == null)
					{					
						gradePoints = scalePointGrade(state, gradePoints);
					}
					state.setAttribute (NEW_ASSIGNMENT_GRADE_POINTS, gradePoints);
				}
			}
		}

		// treat the new assignment description as formatted text
		String str = processFormattedTextFromBrowser(state, params.getCleanString(NEW_ASSIGNMENT_DESCRIPTION));
		state.setAttribute (NEW_ASSIGNMENT_DESCRIPTION, str);
		
		state.setAttribute (NEW_ASSIGNMENT_RELEASE_GRADE_TYPE, params.getString (NEW_ASSIGNMENT_RELEASE_GRADE_TYPE));
		if (params.getString (ResourceProperties.NEW_ASSIGNMENT_CHECK_ADD_DUE_DATE)!=null && params.getString (ResourceProperties.NEW_ASSIGNMENT_CHECK_ADD_DUE_DATE).equalsIgnoreCase (Boolean.TRUE.toString ()))
		{
			state.setAttribute (ResourceProperties.NEW_ASSIGNMENT_CHECK_ADD_DUE_DATE, Boolean.TRUE.toString ());
		}
		else
		{
			state.setAttribute (ResourceProperties.NEW_ASSIGNMENT_CHECK_ADD_DUE_DATE, Boolean.FALSE.toString ());
		}

		if (params.getString (ResourceProperties.NEW_ASSIGNMENT_CHECK_AUTO_ANNOUNCE)!=null && params.getString (ResourceProperties.NEW_ASSIGNMENT_CHECK_AUTO_ANNOUNCE).equalsIgnoreCase (Boolean.TRUE.toString ()))
		{
			state.setAttribute (ResourceProperties.NEW_ASSIGNMENT_CHECK_AUTO_ANNOUNCE, Boolean.TRUE.toString ());
		}
		else
		{
			state.setAttribute (ResourceProperties.NEW_ASSIGNMENT_CHECK_AUTO_ANNOUNCE, Boolean.FALSE.toString ());
		}

		String s = params.getString (NEW_ASSIGNMENT_CHECK_ADD_HONOR_PLEDGE);

		// set the honor pledge to be "no honor pledge"
		if (s == null)
			s = "1";
		state.setAttribute (NEW_ASSIGNMENT_CHECK_ADD_HONOR_PLEDGE, s);

		state.setAttribute (NEW_ASSIGNMENT_ATTACHMENT, state.getAttribute (ATTACHMENTS));
	
		// all those functions need to check against inputs
		if (title.length ()==0)
		{
			// empty assignment title
			addAlert(state, "Please specify the assignment title. ");
		}
		
	}	//setNewAssignmentParameters
	
	/**
	 * Action is to hide the preview assignment student view
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doHide_submission_assignment_instruction (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		state.setAttribute (GRADE_SUBMISSION_ASSIGNMENT_EXPAND_FLAG, new Boolean (false));
		
	}	// doHide_preview_assignment_student_view
	
	/**
	 * Action is to show the preview assignment student view
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doShow_submission_assignment_instruction (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		state.setAttribute (GRADE_SUBMISSION_ASSIGNMENT_EXPAND_FLAG, new Boolean (true));
		
	}	// doShow_submission_assignment_instruction
	
	/**
	 * Action is to hide the preview assignment student view
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doHide_preview_assignment_student_view (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		state.setAttribute (PREVIEW_ASSIGNMENT_STUDENT_VIEW_HIDE_FLAG, new Boolean (true));
		
	}	// doHide_preview_assignment_student_view
	
	/**
	 * Action is to show the preview assignment student view
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doShow_preview_assignment_student_view (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		state.setAttribute (PREVIEW_ASSIGNMENT_STUDENT_VIEW_HIDE_FLAG, new Boolean (false));
		
	}	// doShow_preview_assignment_student_view
	
	/**
	 * Action is to hide the preview assignment assignment infos
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doHide_preview_assignment_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		state.setAttribute (PREVIEW_ASSIGNMENT_ASSIGNMENT_HIDE_FLAG, new Boolean (true));
		
	}	// doHide_preview_assignment_assignment
	
	/**
	 * Action is to show the preview assignment assignment info
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doShow_preview_assignment_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		state.setAttribute (PREVIEW_ASSIGNMENT_ASSIGNMENT_HIDE_FLAG, new Boolean (false));
		
	}	// doShow_preview_assignment_assignment
	
	/**
	 * Action is to hide the assignment option
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doHide_assignment_option (RunData data, Context context)
	{
		setNewAssignmentParameters(data);
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		state.setAttribute (NEW_ASSIGNMENT_HIDE_OPTION_FLAG, new Boolean (true));
		state.setAttribute (NEW_ASSIGNMENT_FOCUS, "eventSubmit_doShow_assignment_option");
		
	}	//doHide_assignment_option
	
	/**
	 * Action is to show the assignment option
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doShow_assignment_option (RunData data, Context context)
	{
		setNewAssignmentParameters(data);
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		state.setAttribute (NEW_ASSIGNMENT_HIDE_OPTION_FLAG, new Boolean (false));
		state.setAttribute (NEW_ASSIGNMENT_FOCUS, NEW_ASSIGNMENT_CHECK_ADD_HONOR_PLEDGE);
		
	}	//doShow_assignment_option
	
	/**
	 * Action is to hide the assignment content in the view assignment page
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doHide_view_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		state.setAttribute (VIEW_ASSIGNMENT_HIDE_ASSIGNMENT_FLAG, new Boolean (true));
		
	}	//doHide_view_assignment
	
	/**
	 * Action is to show the assignment content in the view assignment page
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doShow_view_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		state.setAttribute (VIEW_ASSIGNMENT_HIDE_ASSIGNMENT_FLAG, new Boolean (false));
		
	}	//doShow_view_assignment
	
	/**
	 * Action is to hide the student view in the view assignment page
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doHide_view_student_view (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		state.setAttribute (VIEW_ASSIGNMENT_HIDE_STUDENT_VIEW_FLAG, new Boolean (true));
		
	}	//doHide_view_student_view
	
	/**
	 * Action is to show the student view in the view assignment page
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doShow_view_student_view (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		state.setAttribute (VIEW_ASSIGNMENT_HIDE_STUDENT_VIEW_FLAG, new Boolean (false));
		
	}	//doShow_view_student_view
	
	/**
	 * Action is to post draft assignment
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doPost_draft_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		ParameterParser params = data.getParameters ();
		
		String assignmentId = params.getString ("assignmentId");
		
		try
		{
			AssignmentEdit aEdit = AssignmentService.editAssignment (assignmentId);
			aEdit.setDraft (false);
			
			// add the due date to schedule
			String checkAddDueTime = aEdit.getProperties ().getProperty (ResourceProperties.NEW_ASSIGNMENT_CHECK_ADD_DUE_DATE);
			if (checkAddDueTime.equalsIgnoreCase (Boolean.TRUE.toString ()) && (aEdit.getProperties ().getProperty (NEW_ASSIGNMENT_DUE_DATE_SCHEDULED)==null))
			{
				Time dueTime = aEdit.getDueTime ();
				String title = aEdit.getTitle ();
				Calendar c = (Calendar) state.getAttribute (CALENDAR);
				if (c != null)
				{				
					// already has calendar object
					try
					{
						CalendarEvent e = c.addEvent (	TimeService.newTimeRange(dueTime.getTime (), 0*60*1000),
														"Due: " + title,
														"Assignment " + title  + " is due on " + dueTime.toStringLocalFull () + ". ",
														"Deadline",
														/*location*/"",
														new ReferenceVector ());
						aEdit.getPropertiesEdit ().addProperty (NEW_ASSIGNMENT_DUE_DATE_SCHEDULED, Boolean.TRUE.toString ());
						if (e != null)
						{
							aEdit.getPropertiesEdit ().addProperty (ResourceProperties.PROP_ASSIGNMENT_DUEDATE_CALENDAR_EVENT_ID, e.getId());
						}
					}
					catch (PermissionException e)
					{
						addAlert(state, "Can not add the due date to schedule. ");
					}
				} // if
			}	// if
			
			// add the open date to annoucement
			String checkAutoAnnounce = aEdit.getProperties ().getProperty (ResourceProperties.NEW_ASSIGNMENT_CHECK_AUTO_ANNOUNCE);
			if (checkAutoAnnounce.equalsIgnoreCase (Boolean.TRUE.toString ()) && (aEdit.getProperties ().getProperty (NEW_ASSIGNMENT_OPEN_DATE_ANNOUNCED)==null))
			{
				Time openTime = aEdit.getOpenTime ();
				String title = aEdit.getTitle ();
				AnnouncementChannel channel = (AnnouncementChannel) state.getAttribute (ANNOUNCEMENT_CHANNEL);
				if (channel != null)
				{				
					// the announcement channel in place
					try
					{
						AnnouncementMessage message = channel.addAnnouncementMessage (	"Assignment: Open Date for "  + title,
																						false,
																						new ReferenceVector (),
																						"Open date for assignment " + title + " is " + openTime.toStringLocalFull () + ". ");
						aEdit.getPropertiesEdit ().addProperty (NEW_ASSIGNMENT_OPEN_DATE_ANNOUNCED, Boolean.TRUE.toString ());
						if (message != null)
						{
							aEdit.getPropertiesEdit ().addProperty (ResourceProperties.PROP_ASSIGNMENT_OPENDATE_ANNOUNCEMENT_MESSAGE_ID, message.getId());
						}
					}
					catch (PermissionException e)
					{
						addAlert(state, "Can not make an announcement for the assignment open date. ");
					}
				} // if
			}	// if
			
			if (state.getAttribute(STATE_MESSAGE) == null)
			{
				AssignmentService.commitEdit (aEdit);
			}
		}
		catch (IdUnusedException e)
		{
			addAlert(state, "Can not found such a assignment. ");
		}
		catch (PermissionException e)
		{
			addAlert(state, "You are not allowed to post the drafted assignment. ");
		}
		catch (InUseException e)
		{
			addAlert(state, INUSE_ERROR_MESSAGE + "assignment. ");
		}
		
		if (state.getAttribute(STATE_MESSAGE) == null)
		{		
			state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_LIST_ASSIGNMENTS);
			state.setAttribute (SORTED_BY, SORTED_BY_DUEDATE);
			state.setAttribute(SORTED_ASC, Boolean.TRUE.toString());
		}
		
	} // doPost_draft_assignment
	
	/**
	 * Action is to post assignment
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doPost_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());

		String mode = (String) state.getAttribute(STATE_MODE);
		if (!mode.equals(MODE_INSTRUCTOR_PREVIEW_ASSIGNMENT))
		{
			// read input data if the mode is not preview mode
			setNewAssignmentParameters(data);
		}
		
		if (state.getAttribute(STATE_MESSAGE) == null)
		{		
			ParameterParser params = data.getParameters ();

			// put the names and values into vm file
			String title = (String) state.getAttribute (NEW_ASSIGNMENT_TITLE);
			
			// open time
			int openMonth = ((Integer) state.getAttribute (NEW_ASSIGNMENT_OPENMONTH)).intValue ();
			int openDay = ((Integer) state.getAttribute (NEW_ASSIGNMENT_OPENDAY)).intValue ();
			int openYear = ((Integer) state.getAttribute (NEW_ASSIGNMENT_OPENYEAR)).intValue ();
			int openHour = ((Integer) state.getAttribute (NEW_ASSIGNMENT_OPENHOUR)).intValue ();
			int openMin = ((Integer) state.getAttribute (NEW_ASSIGNMENT_OPENMIN)).intValue ();
			String openAMPM = (String) state.getAttribute (NEW_ASSIGNMENT_OPENAMPM);
			if ((openAMPM.equals ("PM")) && (openHour!=12))
			{
				openHour = openHour + 12;
			}
			if ((openHour == 12) && (openAMPM.equals ("AM")))
			{
				openHour = 0;
			} 
			Time openTime = TimeService.newTimeLocal(openYear, openMonth, openDay, openHour, openMin, 0, 0);

			// due time
			int dueMonth = ((Integer) state.getAttribute (NEW_ASSIGNMENT_DUEMONTH)).intValue ();
			int dueDay = ((Integer) state.getAttribute (NEW_ASSIGNMENT_DUEDAY)).intValue ();
			int dueYear = ((Integer) state.getAttribute (NEW_ASSIGNMENT_DUEYEAR)).intValue ();
			int dueHour = ((Integer) state.getAttribute (NEW_ASSIGNMENT_DUEHOUR)).intValue ();
			int dueMin = ((Integer) state.getAttribute (NEW_ASSIGNMENT_DUEMIN)).intValue ();
			String dueAMPM = (String) state.getAttribute (NEW_ASSIGNMENT_DUEAMPM);
			if ((dueAMPM.equals ("PM")) && (dueHour !=12))
			{
				dueHour = dueHour + 12;
			}
			if ((dueHour == 12) && (dueAMPM.equals ("AM")))
			{
				dueHour = 0;
			}
			Time dueTime = TimeService.newTimeLocal(dueYear, dueMonth, dueDay, dueHour, dueMin, 0, 0);

			// checks on the times
			if (dueTime.before (openTime))
			{
				addAlert(state, "Assignment due date should be set after the open date. ");
			}

			// close time
			Time closeTime = TimeService.newTime();
			boolean enableCloseDate = ((Boolean)state.getAttribute (NEW_ASSIGNMENT_ENABLECLOSEDATE)).booleanValue ();
			if (enableCloseDate)
			{
				int closeMonth = ((Integer) state.getAttribute (NEW_ASSIGNMENT_CLOSEMONTH)).intValue ();
				int closeDay = ((Integer) state.getAttribute (NEW_ASSIGNMENT_CLOSEDAY)).intValue ();
				int closeYear = ((Integer) state.getAttribute (NEW_ASSIGNMENT_CLOSEYEAR)).intValue ();
				int closeHour = ((Integer) state.getAttribute (NEW_ASSIGNMENT_CLOSEHOUR)).intValue ();
				int closeMin = ((Integer) state.getAttribute (NEW_ASSIGNMENT_CLOSEMIN)).intValue ();
				String closeAMPM = (String) state.getAttribute (NEW_ASSIGNMENT_CLOSEAMPM);
				if ((closeAMPM.equals ("PM"))&& (closeHour!=12))
				{
					closeHour = closeHour + 12;
				}
				if ((closeHour == 12) && (closeAMPM.equals ("AM")))
				{
					closeHour = 0;
				}
				closeTime = TimeService.newTimeLocal(closeYear, closeMonth, closeDay, closeHour, closeMin, 0, 0);

				// if there is a close time, check it
				if (closeTime.before (openTime))
				{
					addAlert(state, CLOSE_OPEN_DATE_ERROR_MESSAGE);
				}
				if (closeTime.before (dueTime))
				{
					addAlert(state, CLOSE_DUE_DATE_ERROR_MESSAGE);
				}
			}

			// sections
			String s = (String) state.getAttribute (NEW_ASSIGNMENT_SECTION);

			int submissionType = ((Integer) state.getAttribute (NEW_ASSIGNMENT_SUBMISSION_TYPE)).intValue ();

			int gradeType = ((Integer) state.getAttribute (NEW_ASSIGNMENT_GRADE_TYPE)).intValue ();

			String	gradePoints = (String) state.getAttribute (NEW_ASSIGNMENT_GRADE_POINTS);

			String description = (String) state.getAttribute (NEW_ASSIGNMENT_DESCRIPTION);

			String releaseGradeType =(String) state.getAttribute (NEW_ASSIGNMENT_RELEASE_GRADE_TYPE);

			String checkAddDueTime = (String) state.getAttribute (ResourceProperties.NEW_ASSIGNMENT_CHECK_ADD_DUE_DATE);

			String checkAutoAnnounce = (String) state.getAttribute (ResourceProperties.NEW_ASSIGNMENT_CHECK_AUTO_ANNOUNCE);

			String checkAddHonorPledge = (String) state.getAttribute (NEW_ASSIGNMENT_CHECK_ADD_HONOR_PLEDGE);

			// the attachments
			ReferenceVector attachments = (ReferenceVector) state.getAttribute (ATTACHMENTS);
			ReferenceVector attachments1 = (ReferenceVector) attachments.clone ();

			boolean newAssignment = true;

			// correct inputs
			if (state.getAttribute(STATE_MESSAGE) == null)
			{
				if (((description == null) || (description.length() == 0)) && ((attachments == null || attachments.size() == 0)) && (state.getAttribute(NEW_ASSIGNMENT_DESCRIPTION_EMPTY) == null))
				{
					// if there is no description nor an attachment, show the following alert message. 
					// One could ignore the message and still post the assignment
					addAlert(state, "This assignment has no instructions. Are you sure you want to post it?");
					state.setAttribute(NEW_ASSIGNMENT_DESCRIPTION_EMPTY, Boolean.TRUE.toString());
				}
				else
				{
					state.removeAttribute(NEW_ASSIGNMENT_DESCRIPTION_EMPTY);
				}
			}
			
			// correct inputs
			if (state.getAttribute(STATE_MESSAGE) == null)
			{
				User u = (User) state.getAttribute (STATE_USER);

				String assignmentId = params.getString ("assignmentId");
				String assignmentContentId = params.getString ("assignmentContentId");

				// AssignmentContent object
				AssignmentContentEdit ac = null;
				if (assignmentContentId.length ()==0)
				{
					// new assignment
					// only show alert when dealing with new assignment
					// allow editing assignment after due date
					if (dueTime.before(TimeService.newTime()))
					{
						addAlert(state, "Assignment due date should not be set to be in the past. ");
					}
					else
					{
						try
						{
							ac = AssignmentService.addAssignmentContent ((String) state.getAttribute (STATE_CONTEXT_STRING));
						}
						catch (PermissionException e)
						{
							addAlert(state, "You are not allowed to create an assignment content. ");
						}
					}
				}
				else
				{
					try
					{
						// edit assignment
						ac = AssignmentService.editAssignmentContent (assignmentContentId);
					}
					catch (InUseException e )
					{
						addAlert(state, "The assignment content is being edited by another user. ");
					}
					catch (IdUnusedException e )
					{
						addAlert(state, "Can not find the assignment content. ");
					}
					catch (PermissionException e)
					{
						addAlert(state, "You are not allowed to view the assignment content. ");
					}

				}

				// Assignment
				AssignmentEdit a = null;
				if (assignmentId.length ()==0)
				{
					// create a new assignment
					try
					{
						a = AssignmentService.addAssignment ((String) state.getAttribute (STATE_CONTEXT_STRING));
					}
					catch (PermissionException e)
					{
						addAlert(state, "You are not allowed to add an assignment. ");
					}
				}
				else
				{
					// not a new assignment
					newAssignment = false;

					try
					{
						// edit assignment
						a = AssignmentService.editAssignment (assignmentId);
					}
					catch (InUseException e )
					{
						addAlert(state, "The assignment is being edited by another user. ");
					}
					catch (IdUnusedException e )
					{
						addAlert(state, "Can not find the assignment. ");
					}
					catch (PermissionException e)
					{
						addAlert(state, "You are not allowed to view the assignment. ");
					}	// try-catch
				}	// if-else

				if ((state.getAttribute(STATE_MESSAGE) == null) && (ac != null) && (a != null))
				{
					ac.setTitle (title);
					ac.setInstructions (description);
					ac.setHonorPledge (Integer.parseInt (checkAddHonorPledge));
					ac.setTypeOfSubmission (submissionType);
					ac.setTypeOfGrade (gradeType);
					if (gradeType==3)
					{
						ac.setMaxGradePoint (Integer.parseInt (gradePoints));
					}
					ac.setGroupProject (true);
					ac.setIndividuallyGraded (false);

					// release type
					ac.setReleaseGrades (Boolean.valueOf (releaseGradeType).booleanValue ());

					if (submissionType!=1)
					{
						ac.setAllowAttachments (true);
					}
					else
					{
						ac.setAllowAttachments (false);
					}

					Boolean attachMod = (Boolean)state.getAttribute(ATTACHMENTS_MODIFIED);
					if (attachMod.booleanValue())
					{
						// clear attachments
						ac.clearAttachments ();

						// add each attachment
						Iterator it = ((ReferenceVector) attachments1.clone ()).iterator ();
						while (it.hasNext ())
						{
							ac.addAttachment ((Reference) it.next ());
						}
						state.setAttribute(ATTACHMENTS_MODIFIED, new Boolean(false));
					}

					// commit the changes
					AssignmentService.commitEdit (ac);

				}

				// create assignment
				try
				{
					a.setTitle (title);
					a.setContent (ac);
					a.setContext ((String) state.getAttribute (STATE_CONTEXT_STRING));
					a.setSection (s);
					// old open time
					Time oldOpenTime = a.getOpenTime ();
					a.setOpenTime (openTime);
					// old due time
					Time oldDueTime = a.getDueTime ();
					a.setDueTime (dueTime);
					// set the drop dead date as the due date
					a.setDropDeadTime (dueTime);
					if (enableCloseDate)
					{
						a.setCloseTime (closeTime);
					}
					else
					{
						// if editing an old assignment with close date
						if (a.getCloseTime ()!=null)
						{
							a.setCloseTime (null);
						}
					}

					// post the assignment
					a.setDraft (false);

					// set the auto check/auto announce property
					ResourcePropertiesEdit aPropertiesEdit = a.getPropertiesEdit ();
					aPropertiesEdit.addProperty (ResourceProperties.NEW_ASSIGNMENT_CHECK_ADD_DUE_DATE, checkAddDueTime);
					aPropertiesEdit.addProperty (ResourceProperties.NEW_ASSIGNMENT_CHECK_AUTO_ANNOUNCE, checkAutoAnnounce);

					// add the due date to schedule
					String dueDateScheduled = a.getProperties ().getProperty (NEW_ASSIGNMENT_DUE_DATE_SCHEDULED);
					boolean dueDateModified = false;

					if (dueDateScheduled!=null && dueDateScheduled.equalsIgnoreCase (Boolean.TRUE.toString ()) && oldDueTime!=null && (!oldDueTime.toStringLocalFull ().equals (dueTime.toStringLocalFull ())))
					{
						dueDateModified = true;
					}
					if (((checkAddDueTime.equalsIgnoreCase (Boolean.TRUE.toString ())) && (dueDateScheduled==null))|| dueDateModified)
					{
						Calendar c = (Calendar) state.getAttribute (CALENDAR);
						if (c != null)
						{
							// already has calendar object
							try
							{
								CalendarEvent e = null;
								if (dueDateModified)
								{
									// get the old event
									boolean found = false;
									String oldEventId = aPropertiesEdit.getProperty (ResourceProperties.PROP_ASSIGNMENT_DUEDATE_CALENDAR_EVENT_ID);
									if (oldEventId != null)
									{
										try
										{					
											e = c.getEvent(oldEventId);
											found = true;
										}
										catch (IdUnusedException ee)
										{
											Log.warn("chef", "The old event has been deleted: event id=" + oldEventId + ". ");
										}
										catch (PermissionException ee)
										{
											Log.warn("chef", "You do not have the permission to view the schedule event id= " + oldEventId + ".");
										}
									}
									else
									{
										TimeBreakdown b = oldDueTime.breakdownLocal ();
										// TODO: check- this was new Time(year...), not local! -ggolden
										Time startTime = TimeService.newTimeLocal(b.getYear (), b.getMonth (), b.getDay (), 0, 0, 0, 0);
										Time endTime = TimeService.newTimeLocal(b.getYear (), b.getMonth (), b.getDay (), 23, 59, 59, 999);
										Iterator events = c.getEvents (TimeService.newTimeRange(startTime, endTime), null).iterator ();

										while ((!found) && (events.hasNext ()))
										{
											e = (CalendarEvent) events.next ();
											if (((String) e.getDisplayName ()).indexOf ("Assignment " + title)!=-1)
											{
												found = true;
											}
										}
									}

									// remove the founded old event
									if (found)
									{
										// found the old event delete it
										try
										{
											c.removeEvent (c.editEvent (e.getId ()));
										}
										catch (PermissionException ee)
										{
											addAlert(state, "Can not remove the old event for assignment " + title + ". ");
										}
										catch (InUseException ee)
										{
											addAlert(state, INUSE_ERROR_MESSAGE + "calendar. ");
										}
									}
								}
								e = null;
								e = c.addEvent (/*TimeRange*/ TimeService.newTimeRange(dueTime.getTime (), /*0 duration*/0*60*1000),
												/*title*/"Due: " + title,
												/*description*/ "Assignment " + title  + " is due on " + dueTime.toStringLocalFull () + ". ",
												/*type*/"Deadline",
												/*location*/"",
												/*attachments*/new ReferenceVector ());
								aPropertiesEdit.addProperty (NEW_ASSIGNMENT_DUE_DATE_SCHEDULED, Boolean.TRUE.toString ());
								if (e != null)
								{
									aPropertiesEdit.addProperty (ResourceProperties.PROP_ASSIGNMENT_DUEDATE_CALENDAR_EVENT_ID, e.getId()); 
								}
							}
							catch (PermissionException e)
							{
								addAlert(state, "Can not add the due date to schedule. ");
							}	// try-catch
						}	// if
					} //if
					
					// the open date been announced
					String openDateAnnounced = a.getProperties ().getProperty (NEW_ASSIGNMENT_OPEN_DATE_ANNOUNCED);
					boolean openDateModified = false;
					if (openDateAnnounced!=null && openDateAnnounced.equalsIgnoreCase (Boolean.TRUE.toString ()) && oldOpenTime!=null && (!oldOpenTime.toStringLocalFull ().equals (openTime.toStringLocalFull ())))
					{
						openDateModified = true;
					}

					// add the open date to annoucement
					if (((checkAutoAnnounce.equalsIgnoreCase (Boolean.TRUE.toString ())) && (openDateAnnounced==null))
					|| (openDateModified))
					{
						AnnouncementChannel channel = (AnnouncementChannel) state.getAttribute (ANNOUNCEMENT_CHANNEL);
						if (channel != null)
						{
							// announcement channel is in place
							try
							{
								AnnouncementMessage message = null;
								if (!openDateModified)
								{
									message = channel.addAnnouncementMessage (	/*subject*/"Assignment: Open Date for " + title,
																				/*draft*/false,
																				/*attachment*/new ReferenceVector (),
																				/*body*/"Open date for assignment " + title  + " is " + openTime.toStringLocalFull () + ". ");
								}
								else
								{
									message = channel.addAnnouncementMessage (	/*subject*/"Assignment: New Open Date for " + title,
																				/*draft*/false,
																				/*attachment*/new ReferenceVector (),
																				/*body*/"New open date for assignment " + title  + " is " + openTime.toStringLocalFull () + ". ");
								}
								aPropertiesEdit.addProperty (NEW_ASSIGNMENT_OPEN_DATE_ANNOUNCED, Boolean.TRUE.toString ());
								if (message != null)
								{
									aPropertiesEdit.addProperty (ResourceProperties.PROP_ASSIGNMENT_OPENDATE_ANNOUNCEMENT_MESSAGE_ID, message.getId());
								}
							}
							catch (PermissionException e)
							{
								addAlert(state, "Can not make an announcement for the assignment open date. ");
							}
						}
					}	// if

					if (state.getAttribute(STATE_MESSAGE) == null)
					{
						// add to schedule and announcement is successful
						AssignmentService.commitEdit (a);
					}
					else
					{
						// add to schedule and announcement is not successful
						// remove the assignment in case of post a new assignment
						if (newAssignment)
						{

							try
							{
								AssignmentService.removeAssignment (a);
							}
							catch (PermissionException e)
							{
								addAlert(state, "You are not allowed to remove the assignment. ");
							}

						}
					}
				}
				catch (IdUnusedException e)
				{
					addAlert(state, "Can not find the assignment. ");
				}	//	try-catch

			}	// if

		}
		
		if (state.getAttribute(STATE_MESSAGE) == null)
		{		
			state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_LIST_ASSIGNMENTS);
			state.setAttribute (SORTED_BY, SORTED_BY_DUEDATE);
			state.setAttribute(SORTED_ASC, Boolean.TRUE.toString());
			
			state.setAttribute (ATTACHMENTS, new ReferenceVector ());
			resetAssignment (state);
		}

	}	//doPost_assignment
	
	/**
	 * Action is to post new assignment
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doSave_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());

		String mode = (String) state.getAttribute(STATE_MODE);
		if (!mode.equals(MODE_INSTRUCTOR_PREVIEW_ASSIGNMENT))
		{
			// read input data if the mode is not preview mode
			setNewAssignmentParameters(data);
		}
		if (state.getAttribute(STATE_MESSAGE) != null) return;

		ParameterParser params = data.getParameters ();

		// put the names and values into vm file
		String title = (String) state.getAttribute (NEW_ASSIGNMENT_TITLE);

		// open time
		int openMonth = ((Integer) state.getAttribute (NEW_ASSIGNMENT_OPENMONTH)).intValue ();
		int openDay = ((Integer) state.getAttribute (NEW_ASSIGNMENT_OPENDAY)).intValue ();
		int openYear = ((Integer) state.getAttribute (NEW_ASSIGNMENT_OPENYEAR)).intValue ();
		int openHour = ((Integer) state.getAttribute (NEW_ASSIGNMENT_OPENHOUR)).intValue ();
		int openMin = ((Integer) state.getAttribute (NEW_ASSIGNMENT_OPENMIN)).intValue ();
		String openAMPM = (String) state.getAttribute (NEW_ASSIGNMENT_OPENAMPM);

		if ((openAMPM.equals ("PM"))&&(openHour!=12))
		{
			openHour = openHour + 12;
		}
		if ((openHour == 12) && (openAMPM.equals ("AM")))
		{
			openHour = 0;
		}
		Time openTime = TimeService.newTimeLocal(openYear, openMonth, openDay, openHour, openMin, 0, 0);

		// due time
		int dueMonth = ((Integer) state.getAttribute (NEW_ASSIGNMENT_DUEMONTH)).intValue ();
		int dueDay = ((Integer) state.getAttribute (NEW_ASSIGNMENT_DUEDAY)).intValue ();
		int dueYear = ((Integer) state.getAttribute (NEW_ASSIGNMENT_DUEYEAR)).intValue ();
		int dueHour = ((Integer) state.getAttribute (NEW_ASSIGNMENT_DUEHOUR)).intValue ();
		int dueMin = ((Integer) state.getAttribute (NEW_ASSIGNMENT_DUEMIN)).intValue ();
		String dueAMPM = (String) state.getAttribute (NEW_ASSIGNMENT_DUEAMPM);
		if ((dueAMPM.equals ("PM"))&&(dueHour!=12))
		{
			dueHour = dueHour + 12;
		}
		if ((dueHour == 12) && (dueAMPM.equals ("AM")))
		{
			dueHour = 0;
		}
		Time dueTime = TimeService.newTimeLocal(dueYear, dueMonth, dueDay, dueHour, dueMin, 0, 0);

		// close time
		Time closeTime = TimeService.newTime();
		boolean enableCloseDate = ((Boolean)state.getAttribute (NEW_ASSIGNMENT_ENABLECLOSEDATE)).booleanValue ();
		if (enableCloseDate)
		{
			int closeMonth = ((Integer) state.getAttribute (NEW_ASSIGNMENT_CLOSEMONTH)).intValue ();
			int closeDay = ((Integer) state.getAttribute (NEW_ASSIGNMENT_CLOSEDAY)).intValue ();
			int closeYear = ((Integer) state.getAttribute (NEW_ASSIGNMENT_CLOSEYEAR)).intValue ();
			int closeHour = ((Integer) state.getAttribute (NEW_ASSIGNMENT_CLOSEHOUR)).intValue ();
			int closeMin = ((Integer) state.getAttribute (NEW_ASSIGNMENT_CLOSEMIN)).intValue ();
			String closeAMPM = (String) state.getAttribute (NEW_ASSIGNMENT_CLOSEAMPM);
			if ((closeAMPM.equals ("PM"))&&(closeHour!=12))
			{
				closeHour = closeHour + 12;
			}
			if ((closeHour == 12) && (closeAMPM.equals ("AM")))
			{
				closeHour = 0;
			}
			closeTime = TimeService.newTimeLocal(closeYear, closeMonth, closeDay, closeHour, closeMin, 0, 0);
		}
		/*else
		{
			closeTime = dueTime;
		}*/

		// checks on the inconsistancy among times
		if (dueTime.before (openTime))
		{
			addAlert(state, "Assignment due date should be set after the open date. ");
		}
		if ((enableCloseDate)&&(closeTime.before (openTime)))
		{
			addAlert(state, CLOSE_OPEN_DATE_ERROR_MESSAGE);
		}
		if ((enableCloseDate)&&(closeTime.before (dueTime)))
		{
			addAlert(state, CLOSE_DUE_DATE_ERROR_MESSAGE);
		}

		// sections
		String s= (String) state.getAttribute (NEW_ASSIGNMENT_SECTION);

		int submissionType = ((Integer) state.getAttribute (NEW_ASSIGNMENT_SUBMISSION_TYPE)).intValue ();

		int gradeType = ((Integer) state.getAttribute (NEW_ASSIGNMENT_GRADE_TYPE)).intValue ();

		String	gradePoints = (String) state.getAttribute (NEW_ASSIGNMENT_GRADE_POINTS);

		String description = (String) state.getAttribute (NEW_ASSIGNMENT_DESCRIPTION);

		String releaseGradeType =(String) state.getAttribute (NEW_ASSIGNMENT_RELEASE_GRADE_TYPE);

		String checkAddDueTime = (String) state.getAttribute (ResourceProperties.NEW_ASSIGNMENT_CHECK_ADD_DUE_DATE);

		String checkAutoAnnounce = (String) state.getAttribute (ResourceProperties.NEW_ASSIGNMENT_CHECK_AUTO_ANNOUNCE);

		String checkAddHonorPledge = (String) state.getAttribute (NEW_ASSIGNMENT_CHECK_ADD_HONOR_PLEDGE);

		// the attachments
		ReferenceVector attachments = (ReferenceVector) state.getAttribute (ATTACHMENTS);
		ReferenceVector attachments1 = (ReferenceVector) attachments.clone ();

		// correct inputs
		if (state.getAttribute(STATE_MESSAGE) != null) return;

		User u = (User) state.getAttribute (STATE_USER);

		String assignmentId = params.getString ("assignmentId");
		String assignmentContentId = params.getString ("assignmentContentId");
		
		// these are the edits
		AssignmentContentEdit cedit = null;
		AssignmentEdit aedit = null;

		// AssignmentContent object
		if (assignmentContentId == null ||assignmentContentId.length ()==0)
		{
			// new assignment
			if (dueTime.before(TimeService.newTime()))
			{
				// only show the alert when dealing with the posting new assignment
				// allow editing assignment when the due date has passed.
				addAlert(state, "Assignment due date should not be set to be in the past. ");
			}
			else
			{
				try
				{
					cedit = AssignmentService.addAssignmentContent ((String) state.getAttribute (STATE_CONTEXT_STRING));
				}
				catch (PermissionException e)
				{
					addAlert(state, "You are not allowed to create an assignment content. ");
				}
			}
		}
		else
		{
			try
			{
				// edit assignment
				cedit = AssignmentService.editAssignmentContent (assignmentContentId);
			}
			catch (IdUnusedException e )
			{
				addAlert(state, "Can not find the assignment content. ");
			}
			catch (PermissionException e)
			{
				addAlert(state, "You are not allowed to view the assignment content. ");
			}
			catch (InUseException e)
			{
				addAlert(state, INUSE_ERROR_MESSAGE + "content. ");
			}
		}

		if (state.getAttribute(STATE_MESSAGE) != null) return;

		// assignment object
		if (assignmentId == null || assignmentId.length ()==0)
		{
			// new assignment
			try
			{
				aedit = AssignmentService.addAssignment ((String) state.getAttribute (STATE_CONTEXT_STRING));
			}
			catch (PermissionException e)
			{
				addAlert(state, "You are not allowed to add an assignment. ");
			}
		}
		else
		{
			try
			{
				// edit assignment
				aedit = AssignmentService.editAssignment (assignmentId);
			}
			catch (IdUnusedException e )
			{
				addAlert(state, "Can not find the assignment. ");
			}
			catch (PermissionException e)
			{
				addAlert(state, "You are not allowed to view the assignment. ");
			}
			catch (InUseException e)
			{
				addAlert(state, INUSE_ERROR_MESSAGE + "assignment. ");
			}
		}	// if-else

		if (state.getAttribute(STATE_MESSAGE) != null)
		{
			AssignmentService.cancelEdit(cedit);
			return;
		}

		// create assignment content
		cedit.setTitle (title);
		cedit.setInstructions (description);
		cedit.setHonorPledge (Integer.parseInt (checkAddHonorPledge));
		cedit.setTypeOfSubmission (submissionType);
		cedit.setTypeOfGrade (gradeType);
		if (gradeType==3)
		{
			cedit.setMaxGradePoint (Integer.parseInt (gradePoints));
		}
		cedit.setGroupProject (true);
		cedit.setIndividuallyGraded (false);

		// release type
		cedit.setReleaseGrades (Boolean.valueOf (releaseGradeType).booleanValue ());

		if (submissionType!=1)
		{
			cedit.setAllowAttachments (true);
		}
		else
		{
			cedit.setAllowAttachments (false);
		}

		Boolean attachMod = (Boolean)state.getAttribute(ATTACHMENTS_MODIFIED);
		if (attachMod.booleanValue())
		{
			// clear attachments
			cedit.clearAttachments ();

			// add each attachment
			Iterator it = ((ReferenceVector) attachments1.clone ()).iterator ();
			while (it.hasNext ())
			{
				cedit.addAttachment ((Reference) it.next ());
			}
			state.setAttribute(ATTACHMENTS_MODIFIED, new Boolean(false));
		}

		// create assignment
		aedit.setContent (cedit);
		aedit.setTitle (title);
		aedit.setContext ((String) state.getAttribute (STATE_CONTEXT_STRING));
		aedit.setSection (s);
		aedit.setOpenTime (openTime);
		aedit.setDueTime (dueTime);
		// set the drop dead date as the due date
		aedit.setDropDeadTime (dueTime);
		if (enableCloseDate)
		{
			aedit.setCloseTime (closeTime);
		}
		aedit.getPropertiesEdit ().addProperty (ResourceProperties.NEW_ASSIGNMENT_CHECK_ADD_DUE_DATE, checkAddDueTime);
		aedit.getPropertiesEdit ().addProperty (ResourceProperties.NEW_ASSIGNMENT_CHECK_AUTO_ANNOUNCE, checkAutoAnnounce);

		// the assignment is drafted
		aedit.setDraft (true);

		// commit
		AssignmentService.commitEdit(aedit);
		AssignmentService.commitEdit(cedit);

		if (state.getAttribute(STATE_MESSAGE) == null)
		{		
			state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_LIST_ASSIGNMENTS);
			state.setAttribute (SORTED_BY, SORTED_BY_DUEDATE);
			state.setAttribute(SORTED_ASC, Boolean.TRUE.toString());
		
			state.setAttribute (ATTACHMENTS, new ReferenceVector ());
			resetAssignment (state);
		}
		
	}	//doSave_assignment
	
	/**
	 * Action is to preview the selected assignment
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doPreview_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());

		setNewAssignmentParameters(data);
		
		String assignmentId = data.getParameters ().getString("assignmentId");
		state.setAttribute(PREVIEW_ASSIGNMENT_ASSIGNMENT_ID, assignmentId);
		
		String assignmentContentId = data.getParameters ().getString("assignmentContentId");
		state.setAttribute(PREVIEW_ASSIGNMENT_ASSIGNMENTCONTENT_ID, assignmentContentId);
		
		state.setAttribute(PREVIEW_ASSIGNMENT_ASSIGNMENT_HIDE_FLAG, new Boolean(false));
		state.setAttribute(PREVIEW_ASSIGNMENT_STUDENT_VIEW_HIDE_FLAG, new Boolean(true));
		if (state.getAttribute(STATE_MESSAGE) == null)
		{
			state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_PREVIEW_ASSIGNMENT);
		}
		
	}	// doPreview_assignment
	
	/**
	 * Action is to view the selected assignment
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doView_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		ParameterParser params = data.getParameters ();
		
		// show the assignment portion
		state.setAttribute (VIEW_ASSIGNMENT_HIDE_ASSIGNMENT_FLAG, new Boolean (false));
		// show the student view portion
		state.setAttribute (VIEW_ASSIGNMENT_HIDE_STUDENT_VIEW_FLAG, new Boolean (false));
		
		String assignmentId = params.getString ("assignmentId");
		state.setAttribute (VIEW_ASSIGNMENT_ID, assignmentId);
		
		try
		{
			Assignment a = AssignmentService.getAssignment (assignmentId);
		}
		catch (IdUnusedException e )
		{
			addAlert(state, "Can not find the assignment. ");
		}
		catch (PermissionException e)
		{
			addAlert(state, "You are not allowed to view the assignment. ");
		}
		
		if (state.getAttribute(STATE_MESSAGE) == null)
		{
			state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_VIEW_ASSIGNMENT);
		}
		
	}	//doView_Assignment
	
	/**
	 * Action is to show the edit assignment screen
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doEdit_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		ParameterParser params = data.getParameters ();
		
		String[] assignmentIds = params.getStrings ("selectedAssignments");
		boolean editAllowed = true;
		
		if (assignmentIds!=null)
		{
			if (assignmentIds.length >1)
			{
				addAlert(state, "Please choose only one assignment for revising.");
				editAllowed = false;
			}
			else
			{
				// whether the user can modify the assignment
				String contextString = (String) state.getAttribute (STATE_CONTEXT_STRING);
				if (!AssignmentService.allowUpdateAssignment (assignmentIds[0]))
				{
					addAlert(state, "You are not allowed to modify the selected assignment.");
					editAllowed = false;
				}
				else
				{
					state.setAttribute (EDIT_ASSIGNMENT_ID, assignmentIds[0] );
					try
					{
						Assignment a = AssignmentService.getAssignment (assignmentIds[0]);
						
						Iterator submissions = AssignmentService.getSubmissions (a);
						if (submissions.hasNext ())
						{
							// if there is submission to the assignment, show the alert
							addAlert(state, "Assignment " + a.getTitle () + " has submissions.");
						}
						
						// SECTION MOD
						state.setAttribute(STATE_SECTION_STRING, a.getSection());
						
						// put the names and values into vm file
						state.setAttribute (NEW_ASSIGNMENT_TITLE, a.getTitle ());
						
						TimeBreakdown openTime = a.getOpenTime ().breakdownLocal ();
						state.setAttribute (NEW_ASSIGNMENT_OPENMONTH, new Integer (openTime.getMonth ()));
						state.setAttribute (NEW_ASSIGNMENT_OPENDAY, new Integer (openTime.getDay ()));
						state.setAttribute (NEW_ASSIGNMENT_OPENYEAR, new Integer (openTime.getYear ()));
						int openHour = openTime.getHour ();
						if (openHour == 0)
						{
							// for midnight point, we mark it as 12AM
							openHour = 12;
						}
						state.setAttribute (NEW_ASSIGNMENT_OPENHOUR, new Integer ((openHour>12)?openHour-12:openHour));
						state.setAttribute (NEW_ASSIGNMENT_OPENMIN, new Integer (openTime.getMin ()));
						if (((String)a.getOpenTime ().toStringLocalFull ()).indexOf ("pm")!=-1)
						{
							state.setAttribute (NEW_ASSIGNMENT_OPENAMPM, "PM");
						}
						else
						{
							state.setAttribute (NEW_ASSIGNMENT_OPENAMPM, "AM");
						}
						
						TimeBreakdown dueTime = a.getDueTime ().breakdownLocal ();
						state.setAttribute (NEW_ASSIGNMENT_DUEMONTH, new Integer (dueTime.getMonth ()));
						state.setAttribute (NEW_ASSIGNMENT_DUEDAY, new Integer (dueTime.getDay ()));
						state.setAttribute (NEW_ASSIGNMENT_DUEYEAR, new Integer (dueTime.getYear ()));
						int dueHour = dueTime.getHour ();
						if (dueHour == 0)
						{
							//for midnight point, we mark it as 12AM
							dueHour = 12;
						}
						state.setAttribute (NEW_ASSIGNMENT_DUEHOUR, new Integer ((dueHour>12)?dueHour-12:dueHour));
						state.setAttribute (NEW_ASSIGNMENT_DUEMIN, new Integer (dueTime.getMin ()));
						if (((String)a.getDueTime ().toStringLocalFull ()).indexOf ("pm")!=-1)
						{
							state.setAttribute (NEW_ASSIGNMENT_DUEAMPM, "PM");
						}
						else
						{
							state.setAttribute (NEW_ASSIGNMENT_DUEAMPM, "AM");
						}
						// generate alert when editing an assignment past due date
						if (a.getDueTime().before(TimeService.newTime()))
						{
							addAlert(state, "You are revising an assignment after the due date. ");
						}
						
						if (a.getCloseTime ()!= null)
						{
							state.setAttribute (NEW_ASSIGNMENT_ENABLECLOSEDATE, new Boolean (true));
							TimeBreakdown closeTime = a.getCloseTime ().breakdownLocal ();
							state.setAttribute (NEW_ASSIGNMENT_CLOSEMONTH, new Integer (closeTime.getMonth ()));
							state.setAttribute (NEW_ASSIGNMENT_CLOSEDAY, new Integer (closeTime.getDay ()));
							state.setAttribute (NEW_ASSIGNMENT_CLOSEYEAR, new Integer (closeTime.getYear ()));
							int closeHour = closeTime.getHour ();
							if (closeHour == 0)
							{
								// for the midnight point, we mark it as 12 AM
								closeHour = 12;
							}
							state.setAttribute (NEW_ASSIGNMENT_CLOSEHOUR, new Integer ((closeHour>12)?closeHour-12:closeHour));
							state.setAttribute (NEW_ASSIGNMENT_CLOSEMIN, new Integer (closeTime.getMin ()));
							if (((String)a.getCloseTime ().toStringLocalFull ()).indexOf ("pm")!=-1)
							{
								state.setAttribute (NEW_ASSIGNMENT_CLOSEAMPM, "PM");
							}
							else
							{
								state.setAttribute (NEW_ASSIGNMENT_CLOSEAMPM, "AM");
							}
						}
						else
						{
							state.setAttribute (NEW_ASSIGNMENT_ENABLECLOSEDATE, new Boolean (false));
							state.setAttribute (NEW_ASSIGNMENT_CLOSEMONTH, state.getAttribute(NEW_ASSIGNMENT_DUEMONTH));
							state.setAttribute (NEW_ASSIGNMENT_CLOSEDAY, state.getAttribute(NEW_ASSIGNMENT_DUEDAY));
							state.setAttribute (NEW_ASSIGNMENT_CLOSEYEAR, state.getAttribute(NEW_ASSIGNMENT_DUEYEAR));
							state.setAttribute (NEW_ASSIGNMENT_CLOSEHOUR, state.getAttribute(NEW_ASSIGNMENT_DUEHOUR));
							state.setAttribute (NEW_ASSIGNMENT_CLOSEMIN, state.getAttribute(NEW_ASSIGNMENT_DUEMIN));
							state.setAttribute (NEW_ASSIGNMENT_CLOSEAMPM, state.getAttribute(NEW_ASSIGNMENT_DUEAMPM));
						}
					
						state.setAttribute (NEW_ASSIGNMENT_SECTION, a.getSection ());
						
						state.setAttribute (NEW_ASSIGNMENT_SUBMISSION_TYPE, new Integer (a.getContent ().getTypeOfSubmission ()));
						int typeOfGrade = a.getContent ().getTypeOfGrade ();
						state.setAttribute (NEW_ASSIGNMENT_GRADE_TYPE, new Integer (typeOfGrade));
						if (typeOfGrade==3)
						{
							state.setAttribute (NEW_ASSIGNMENT_GRADE_POINTS, a.getContent ().getMaxGradePointDisplay ());
						}
						state.setAttribute (NEW_ASSIGNMENT_DESCRIPTION, a.getContent ().getInstructions ());
						state.setAttribute (NEW_ASSIGNMENT_RELEASE_GRADE_TYPE, String.valueOf (a.getContent ().releaseGrades ()));
						state.setAttribute (ResourceProperties.NEW_ASSIGNMENT_CHECK_ADD_DUE_DATE, a.getProperties ().getProperty (ResourceProperties.NEW_ASSIGNMENT_CHECK_ADD_DUE_DATE));
						state.setAttribute (ResourceProperties.NEW_ASSIGNMENT_CHECK_AUTO_ANNOUNCE, a.getProperties ().getProperty (ResourceProperties.NEW_ASSIGNMENT_CHECK_AUTO_ANNOUNCE));
						state.setAttribute (NEW_ASSIGNMENT_CHECK_ADD_HONOR_PLEDGE, Integer.toString (a.getContent ().getHonorPledge ()));
						state.setAttribute (ATTACHMENTS, a.getContent ().getAttachments ());
					}
					catch (IdUnusedException e )
					{
						addAlert(state, "Can not find the assignment. ");
						editAllowed = false;
					}
					catch (PermissionException e)
					{
						addAlert(state, "You are not allowed to view the assignment. ");
						editAllowed = false;
					}
					
					if (editAllowed)
					{
						state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_EDIT_ASSIGNMENT);
					}
				}	// if-else
			}	// if-else
		}
		else
		{
			addAlert(state, "You must first choose an assignment by checking the box next to the assignment name. Then click the Revise button to revise the assignment.");
		}
		
	}	//doEdit_Assignment
	
	/**
	 * Action is to show the delete assigment confirmation screen
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doDelete_confirm_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		ParameterParser params = data.getParameters ();
		
		String[] assignmentIds = params.getStrings ("selectedAssignments");
		
		if (assignmentIds!=null)
		{
			String contextString = (String) state.getAttribute (STATE_CONTEXT_STRING);
			
			Vector ids = new Vector ();
			for (int i=0; i<assignmentIds.length; i++)
			{
				ids.add (assignmentIds[i]);
				try
				{
					Assignment a = AssignmentService.getAssignment (assignmentIds[i]);					
					
					if (!AssignmentService.allowRemoveAssignment (assignmentIds[i]))
					{
						addAlert(state, "You are not allowed to remove " + "\"" + a.getTitle () + "\". ");
					}
				}
				catch (IdUnusedException e )
				{
					addAlert(state, "Can not find the assignment. ");
				}
				catch (PermissionException e)
				{
					addAlert(state, "You are not allowed to view the assignment. ");
				}
			}
			if (state.getAttribute(STATE_MESSAGE) == null)
			{
				// can remove all the selected assignments
				state.setAttribute (DELETE_ASSIGNMENT_IDS, ids);
				state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_DELETE_ASSIGNMENT);
			}
		}
		else
		{
			addAlert(state, "You must first choose the assignment(s) by checking the box next to the assignment name. Then click the Delete button to delete the assignment(s).");
		}
		
	}	//doDelete_confirm_Assignment
	
	/**
	 * Action is to delete the confirmed assignments
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doDelete_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// get the delete assignment ids
		Vector ids = (Vector) state.getAttribute (DELETE_ASSIGNMENT_IDS);
		for (int i=0; i<ids.size (); i++)
		{
			try
			{
				Assignment a = AssignmentService.getAssignment ((String) ids.get (i));
				AssignmentEdit aEdit = AssignmentService.editAssignment(a.getReference());
				ResourcePropertiesEdit pEdit = aEdit.getPropertiesEdit();
				
				String title = a.getTitle();
				
				// remove releted event if there is one
				String isThereEvent = pEdit.getProperty (NEW_ASSIGNMENT_DUE_DATE_SCHEDULED);
				if (isThereEvent != null && isThereEvent.equals(Boolean.TRUE.toString()))
				{
					// remove the associated calender event
					Calendar c = (Calendar) state.getAttribute (CALENDAR);
					if (c != null)
					{
						// already has calendar object
						// get the old event
						CalendarEvent e = null;
						boolean found = false;
						String oldEventId = pEdit.getProperty (ResourceProperties.PROP_ASSIGNMENT_DUEDATE_CALENDAR_EVENT_ID);
						if (oldEventId != null)
						{
							try
							{
								e = c.getEvent(oldEventId);
								found = true;
							}
							catch (IdUnusedException ee)
							{
								addAlert(state, "Can not find the calender event. ");
							}
							catch (PermissionException ee)
							{
							}
						}
						else
						{
							TimeBreakdown b = a.getDueTime().breakdownLocal ();
							// TODO: check- this was new Time(year...), not local! -ggolden
							Time startTime = TimeService.newTimeLocal(b.getYear (), b.getMonth (), b.getDay (), 0, 0, 0, 0);
							Time endTime = TimeService.newTimeLocal(b.getYear (), b.getMonth (), b.getDay (), 23, 59, 59, 999);
							Iterator events = c.getEvents (TimeService.newTimeRange(startTime, endTime), null).iterator ();
							while ((!found) && (events.hasNext ()))
							{
								e = (CalendarEvent) events.next ();
								if (((String) e.getDisplayName ()).indexOf ("Assignment " + title)!=-1)
								{
									found = true;
								}
							}
						}
						// remove the founded old event
						if (found)
						{
							// found the old event delete it
							try
							{
								c.removeEvent (c.editEvent (e.getId ()));
								pEdit.removeProperty (NEW_ASSIGNMENT_DUE_DATE_SCHEDULED);
								pEdit.removeProperty (ResourceProperties.PROP_ASSIGNMENT_DUEDATE_CALENDAR_EVENT_ID);
							}
							catch (PermissionException ee)
							{
								addAlert(state, "Can not remove the old event for assignment " + title + ". ");
							}
							catch (InUseException ee)
							{
								addAlert(state, INUSE_ERROR_MESSAGE + "calendar. ");
							}
						}
					}					
				}	// if-else
				
				// remove the assignment by marking the remove status property true
				pEdit.addProperty(ResourceProperties.PROP_ASSIGNMENT_DELETED, Boolean.TRUE.toString());

				AssignmentService.commitEdit (aEdit);
			}
			catch (InUseException e)
			{
				addAlert(state, INUSE_ERROR_MESSAGE + "assignment. ");
			}
			catch (IdUnusedException e )
			{
				addAlert(state, "Can not find the assignment. ");
			}
			catch (PermissionException e)
			{
				addAlert(state, "You are not allowed to view the assignment. ");
			}
		}	// for
		
		if (state.getAttribute(STATE_MESSAGE) == null)
		{
			state.setAttribute (DELETE_ASSIGNMENT_IDS, new Vector ());
		
			state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_LIST_ASSIGNMENTS);
		}
		
	}	//doDelete_Assignment
	
	/**
	 * Action is to delete the assignment and also the related AssignmentSubmission
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doDeep_delete_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// get the delete assignment ids
		Vector ids = (Vector) state.getAttribute (DELETE_ASSIGNMENT_IDS);
		for (int i=0; i<ids.size (); i++)
		{
			String currentId = (String) ids.get (i);
			try
			{
				AssignmentEdit a = AssignmentService.editAssignment (currentId);
				try
				{
					AssignmentService.removeAssignment (a);
				}
				catch (PermissionException e)
				{
					addAlert(state, "You are not allowed to remove assignment " + a.getTitle () + ". ");
				}
			}
			catch (IdUnusedException e )
			{
				addAlert(state, "Can not find the assignment. ");
			}
			catch (PermissionException e)
			{
				addAlert(state, "You are not allowed to view the assignment. ");
			}
			catch (InUseException e)
			{
				addAlert(state, INUSE_ERROR_MESSAGE + "assignment. ");
			}
		}
		if (state.getAttribute(STATE_MESSAGE) == null)
		{		
			state.setAttribute (DELETE_ASSIGNMENT_IDS, new Vector ());
			state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_LIST_ASSIGNMENTS);
		}
		
	}	//doDeep_delete_Assignment
	
	/**
	 * Action is to show the duplicate assignment screen
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doDuplicate_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// we are changing the view, so start with first page again. 
		resetPaging(state);
					
		String contextString = (String) state.getAttribute (STATE_CONTEXT_STRING);
		ParameterParser params = data.getParameters ();
		String[] assignmentIds = params.getStrings ("selectedAssignments");
		
		if (assignmentIds!=null)
		{
			for (int i=0; i<assignmentIds.length; i++)
			{
				try
				{
					AssignmentEdit aEdit = AssignmentService.addDuplicateAssignment (contextString, assignmentIds[i]);
					
					// clean the duplicate's property
					ResourcePropertiesEdit aPropertiesEdit = aEdit.getPropertiesEdit ();
					aPropertiesEdit.removeProperty (NEW_ASSIGNMENT_DUE_DATE_SCHEDULED);
					aPropertiesEdit.removeProperty (ResourceProperties.PROP_ASSIGNMENT_DUEDATE_CALENDAR_EVENT_ID);
					aPropertiesEdit.removeProperty (NEW_ASSIGNMENT_OPEN_DATE_ANNOUNCED);
					aPropertiesEdit.removeProperty (ResourceProperties.PROP_ASSIGNMENT_OPENDATE_ANNOUNCEMENT_MESSAGE_ID);
					
					AssignmentService.commitEdit (aEdit);
				}
				catch (PermissionException e)
				{
					addAlert(state, "You are not allowed to duplicate assignment. ");
				}
				catch (IdInvalidException e)
				{
					addAlert(state, "The assignment id " + assignmentIds[i] + " is not valid. ");
				}
				catch (IdUnusedException e)
				{
					addAlert(state, "The assignment id " + assignmentIds[i] + " has not been used. ");
				}
				catch(Exception e)
				{}
				
			}
		}
		else
		{
			addAlert(state, "You must first choose an assignment by checking the box next to the assignment name. Then click the Duplicate button to duplicate the assignment. ");
		}
		
	}	//doDuplicate_Assignment
	
	/**
	 * Action is to show the grade submission screen
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doGrade_submission (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		ParameterParser params = data.getParameters ();
		
		// reset the grade assignment id
		state.setAttribute (GRADE_SUBMISSION_ASSIGNMENT_ID, params.getString ("assignmentId"));
		state.setAttribute (GRADE_SUBMISSION_SUBMISSION_ID, params.getString ("submissionId"));
		
		try
		{
			AssignmentSubmission s = AssignmentService.getSubmission ((String) state.getAttribute (GRADE_SUBMISSION_SUBMISSION_ID));
			
			if ((s.getFeedbackText ()!=null) && (s.getFeedbackText ().length ()==0))
			{
				state.setAttribute (GRADE_SUBMISSION_FEEDBACK_TEXT, s.getSubmittedText ());
			}
			else
			{
				state.setAttribute (GRADE_SUBMISSION_FEEDBACK_TEXT, s.getFeedbackText ());
			}
			state.setAttribute (GRADE_SUBMISSION_FEEDBACK_COMMENT, s.getFeedbackComment ());
			
			ReferenceVector v = new ReferenceVector ();
			Iterator attachments = s.getFeedbackAttachments ().iterator ();
			while (attachments.hasNext ())
			{
				v.add (attachments.next ());
			}
			state.setAttribute (ATTACHMENTS, v);
			
			state.setAttribute (GRADE_SUBMISSION_GRADE, s.getGrade ());
		}
		catch (IdUnusedException e )
		{
			addAlert(state, "Can not find the assignment submission. ");
		}
		catch (PermissionException e)
		{
			addAlert(state, "You are not allowed to view the assignment submission. ");
		}
		
		if (state.getAttribute(STATE_MESSAGE) == null)
		{
			state.setAttribute (GRADE_SUBMISSION_ASSIGNMENT_EXPAND_FLAG, new Boolean (false));
			state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_GRADE_SUBMISSION);
		}
		
	}	//doGrade_submission
	
	/**
	 * Action is to release all the grades of the submission
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doRelease_grades (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		ParameterParser params = data.getParameters ();
		
		try
		{
			// get the assignment
			Assignment a = AssignmentService.getAssignment (params.getString ("assignmentId"));
			Iterator submissions = AssignmentService.getSubmissions (a);
			while (submissions.hasNext ())
			{
				AssignmentSubmission s = (AssignmentSubmission) submissions.next ();
				AssignmentSubmissionEdit sEdit = AssignmentService.editSubmission (s.getReference ());
				String grade = s.getGrade();
				if (s.getGraded () && (grade!=null) && (!grade.equals("")))
				{
					sEdit.setGradeReleased (true);
				}
				
				// clear the returned flag
				sEdit.setReturned(false);
				sEdit.setTimeReturned(null);
				
				AssignmentService.commitEdit (sEdit);
			}	// while
		}
		catch (IdUnusedException e )
		{
			addAlert(state, "Can not find the assignment. ");
		}
		catch (PermissionException e)
		{
			addAlert(state, "You are not allowed to view the assignment. ");
		}
		catch (InUseException e)
		{
			addAlert(state, INUSE_ERROR_MESSAGE + "submission. ");
		}
		
	}	//doRelease_grades
	
	/**
	 * Action is to show the assignment in grading page
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doExpand_grade_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		state.setAttribute (GRADE_ASSIGNMENT_EXPAND_FLAG, new Boolean (true));
		
	}	// doExpand_grade_assignment
	
	/**
	 * Action is to hide the assignment in grading page
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doCollapse_grade_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		state.setAttribute (GRADE_ASSIGNMENT_EXPAND_FLAG, new Boolean (false));
		
	}	// doCollapse_grade_assignment
	
	/**
	 * Action is to show the submissions in grading page
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doExpand_grade_submission (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		state.setAttribute (GRADE_SUBMISSION_EXPAND_FLAG, new Boolean (true));
		
	}	// doExpand_grade_submission
	
	/**
	 * Action is to hide the submissions in grading page
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doCollapse_grade_submission (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		state.setAttribute (GRADE_SUBMISSION_EXPAND_FLAG, new Boolean (false));
		
	}	// doCollapse_grade_submission
	
	/**
	 * Action is to show the grade assignment
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doGrade_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		ParameterParser params = data.getParameters ();
		
		// reset the export assignment id
		state.setAttribute (EXPORT_ASSIGNMENT_REF, "");
		
		String[] assignmentIds = params.getStrings ("selectedAssignments");
		
		if (assignmentIds!=null)
		{
			if (assignmentIds.length >1)
			{
				// can not choose more than one assignment at one time
				addAlert(state, "Please choose only one assignment to grade.");
			}
			else
			{
				// get the assignment id
				String id = assignmentIds[0];
				state.setAttribute (EXPORT_ASSIGNMENT_REF, id );
				try
				{
					Assignment a = AssignmentService.getAssignment(id);
					state.setAttribute(EXPORT_ASSIGNMENT_ID, a.getId());
					state.setAttribute (GRADE_ASSIGNMENT_EXPAND_FLAG, new Boolean (false));
					state.setAttribute (GRADE_SUBMISSION_EXPAND_FLAG, new Boolean (true));		
					state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_GRADE_ASSIGNMENT);
				}
				catch (IdUnusedException e)
				{
					addAlert(state, "There is no such assignment. ");
				}
				catch (PermissionException e)
				{
					addAlert(state, "You do not have permission to view the assignment. ");
				}
			}
		}
		else
		{
			addAlert(state, "You must first choose an assignment by checking the box next to the assignment name. Then click the Grade button to grade the assignment.");
		}
		
	}	//doGrade_Assignment
	
	/**
	 * Action is to show the grade assignment
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doGrade_assignment_from (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		ParameterParser params = data.getParameters ();
		
		state.setAttribute (EXPORT_ASSIGNMENT_REF, params.getString ("assignmentId") );
		
		try
		{
			Assignment a = AssignmentService.getAssignment ((String) state.getAttribute (EXPORT_ASSIGNMENT_REF));
			context.put ("assignment", a);
			state.setAttribute(EXPORT_ASSIGNMENT_ID, a.getId());
			state.setAttribute (GRADE_ASSIGNMENT_EXPAND_FLAG, new Boolean (false));
			state.setAttribute (GRADE_SUBMISSION_EXPAND_FLAG, new Boolean (true));
			state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_GRADE_ASSIGNMENT);
		
		}
		catch (IdUnusedException e )
		{
			addAlert(state, "Can not find the assignment. ");
		}
		catch (PermissionException e)
		{
			addAlert(state, "You are not allowed to view the assignment. ");
		}		
	}	//doGrade_assignment_from
	
	
	/**
	 * Action is to show the View Students assignment screen
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doView_students_assignment (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		ParameterParser params = data.getParameters ();		
		state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_VIEW_STUDENTS_ASSIGNMENT);
		
	}	//doView_students_Assignment
	
	/**
	 * Action is to show the student submissions
	 * @param runData The RunData object
	 * @param context The context object
	 */
	public void doShow_student_submission (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		Hashtable t = (Hashtable) state.getAttribute (STUDENT_LIST_SHOW_TABLE);
		ParameterParser params = data.getParameters ();
		
		String id = params.getString ("studentId");
		// add the student id into the table
		t.put (id, "show");
		
		state.setAttribute (STUDENT_LIST_SHOW_TABLE, t);
		
	}	// doShow_student_submission
	
	/**
	 * Action is to hide the student submissions
	 * @param runData The RunData object
	 * @param context The context object
	 */
	public void doHide_student_submission (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		Hashtable t = (Hashtable) state.getAttribute (STUDENT_LIST_SHOW_TABLE);
		ParameterParser params = data.getParameters ();
		
		String id = params.getString ("studentId");
		// remove the student id from the table
		t.remove (id);
		
		state.setAttribute (STUDENT_LIST_SHOW_TABLE, t);
		
	}	// doHide_student_submission
	
	/**
	 * Action is to show the graded assignment submission
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doView_grade (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		ParameterParser params = data.getParameters ();
		
		state.setAttribute (VIEW_GRADE_SUBMISSION_ID, params.getString ("submissionId"));
						
		state.setAttribute (STATE_MODE, MODE_STUDENT_VIEW_GRADE);
		
	}	//	doView_grade
	
	/**
	 * Action is to show the student submissions
	 * @param runData The RunData object
	 * @param context The context object
	 */
	public void doReport_submissions (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());

		state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_REPORT_SUBMISSIONS);
		state.setAttribute(SORTED_BY, SORTED_SUBMISSION_BY_LASTNAME);
		
	}	// doReport_submissions
	
	/**
	 * Action is to use when doAattchmentsadding requested,
	 * corresponding to chef_Assignments-new "eventSubmit_doAattchmentsadding"
	 * when "add attachments" is clicked
	 * @param runData The RunData object
	 * @param context The context object
	 **/
	public void doAttachments (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		ParameterParser params = data.getParameters ();

		String mode = (String) state.getAttribute (STATE_MODE);
		if (mode.equals (MODE_STUDENT_VIEW_SUBMISSION))
		{
			// retrieve the submission text (as formatted text)
			String text = processFormattedTextFromBrowser(state, params.getCleanString(VIEW_SUBMISSION_TEXT));
			
			state.setAttribute (VIEW_SUBMISSION_TEXT, text);
			if (params.getString (VIEW_SUBMISSION_HONOR_PLEDGE_YES)!=null)
			{
				state.setAttribute (VIEW_SUBMISSION_HONOR_PLEDGE_YES,"true");
			}
			state.setAttribute (AttachmentAction.STATE_FROM_TEXT, " submission: ");
		}
		else if (mode.equals (MODE_INSTRUCTOR_NEW_ASSIGNMENT))
		{
			state.setAttribute (AttachmentAction.STATE_FROM_TEXT, " new assignment: ");
			setNewAssignmentParameters(data);
		}
		else if (mode.equals (MODE_INSTRUCTOR_EDIT_ASSIGNMENT))
		{
			state.setAttribute (AttachmentAction.STATE_FROM_TEXT, "assignment " + "\"" + state.getAttribute (NEW_ASSIGNMENT_TITLE) + "\":");
			setNewAssignmentParameters(data);
		}
		else if (mode.equals (MODE_INSTRUCTOR_GRADE_SUBMISSION))
		{
			String feedbackComment = processFormattedTextFromBrowser(state, params.getCleanString (GRADE_SUBMISSION_FEEDBACK_COMMENT));
			state.setAttribute (GRADE_SUBMISSION_FEEDBACK_COMMENT, feedbackComment);
			String feedbackText = processFormattedTextFromBrowser(state, params.getCleanString (GRADE_SUBMISSION_FEEDBACK_TEXT));
			state.setAttribute (GRADE_SUBMISSION_FEEDBACK_TEXT, feedbackText);
			state.setAttribute (GRADE_SUBMISSION_FEEDBACK_ATTACHMENT,  params.getString (GRADE_SUBMISSION_FEEDBACK_ATTACHMENT));
			String grade = params.getString (GRADE_SUBMISSION_GRADE);
			try
			{
				Assignment a = AssignmentService.getAssignment ((String) state.getAttribute (GRADE_SUBMISSION_ASSIGNMENT_ID));
			
				if (a.getContent().getTypeOfGrade() == 3)
				{
					// for point grades, scale the point grade by 10
					state.setAttribute (GRADE_SUBMISSION_GRADE,  scalePointGrade(state, grade));
				}
				else
				{
					// for other grade type, do not scale.
					state.setAttribute (GRADE_SUBMISSION_GRADE, grade );
				}
			}
			catch (IdUnusedException e )
			{
				addAlert(state, "Can not find the assignment. ");
			}
			catch (PermissionException e)
			{
				addAlert(state, "You are not allowed to view the assignment. ");
			}
			
		}
		
		if (state.getAttribute(STATE_MESSAGE) == null)
		{
			Vector attachments = (Vector) state.getAttribute (ATTACHMENTS);
			if (attachments!=null)
			{
				if (attachments.size() > 0)
				{
					state.setAttribute(AttachmentAction.STATE_HAS_ATTACHMENT_BEFORE, Boolean.TRUE);
				}
				else
				{
					state.setAttribute(AttachmentAction.STATE_HAS_ATTACHMENT_BEFORE, Boolean.FALSE);
				}
				state.setAttribute (AttachmentAction.STATE_ATTACHMENTS, attachments.clone ());
			}

			// setup... we'll use the attachment action's mode
			state.setAttribute (AttachmentAction.STATE_MODE, AttachmentAction.MODE_MAIN);
		}
		
	}	//doAttachments	
	
	/**
	* Populate the state object, if needed - override to do something!
	* @param state The state object.
	* @param portlet The portlet.
	* @param rundata The current request's rundata.
	*/
	protected void initState(SessionState state, VelocityPortlet portlet, JetspeedRunData data)
	{
		super.initState(state, portlet, data);
		
		// show the list of assignment view first
		if (state.getAttribute(STATE_SELECTED_VIEW) == null)
		{
			state.setAttribute(STATE_SELECTED_VIEW, "List of Assignments");
		}

		if (state.getAttribute (STATE_USER)==null)
		{
			state.setAttribute (STATE_USER, UsageSessionService.getSessionUser ());
			
		}
		
		/** The content type image lookup service in the State. */
		ContentTypeImageService iService = (ContentTypeImageService) state.getAttribute (STATE_CONTENT_TYPE_IMAGE_SERVICE);
		if (iService == null)
		{
			iService = org.sakaiproject.service.legacy.content.cover.ContentTypeImageService.getInstance();
			state.setAttribute (STATE_CONTENT_TYPE_IMAGE_SERVICE, iService);
		}	// if
		
		/** The user directory service in the State. */
		UserDirectoryService uService = (UserDirectoryService) state.getAttribute (STATE_USER_DIRECTORY_SERVICE);
		if (uService == null)
		{
			uService = org.sakaiproject.service.legacy.user.cover.UserDirectoryService.getInstance();
			state.setAttribute (STATE_USER_DIRECTORY_SERVICE, uService);
		}	// if
		
		/** The calendar service in the State. */
		CalendarService cService = (CalendarService) state.getAttribute (STATE_CALENDAR_SERVICE);
		if (cService == null)
		{
			cService = org.sakaiproject.service.legacy.calendar.cover.CalendarService.getInstance();
			state.setAttribute (STATE_CALENDAR_SERVICE, cService);
			
			String calendarId = ServerConfigurationService.getString ("calendar", null);
			if(calendarId == null)
			{
				calendarId = cService.
						calendarReference(PortalService.getCurrentSiteId(), SiteService.MAIN_CONTAINER);
				try
				{
					state.setAttribute (CALENDAR, cService.getCalendar (calendarId));
				}
				catch (IdUnusedException e)
				{
					Log.warn("chef","No calendar found. ");
					try
					{
						cService.addCalendar (calendarId);
					}
					catch (PermissionException e1)
					{
						Log.warn("chef","Can not create calendar. ");
					}
					catch(IdUsedException e1)
					{
						Log.warn("chef","The calendar id has already been used. ");
					}
					catch(IdInvalidException e1)
					{
						Log.warn("chef","This calendar could not be created because the Id is invalid. ");
					}
					catch(Exception ex)
					{
						Log.warn("chef", "Assignment : Action : init state : calendar exception : " + ex);
					}
				}
				catch (PermissionException e)
				{
					Log.warn("chef","No permission to get the calender. ");
				}
				catch(Exception ex)
				{
					Log.warn("chef", "Assignment : Action : init state : calendar exception : " + ex);
				}
			}
		}	// if
		
		/** The announcement service in the State. */
		AnnouncementService aService = (AnnouncementService) state.getAttribute (STATE_ANNOUNCEMENT_SERVICE);
		if (aService == null)
		{
			aService = org.sakaiproject.service.legacy.announcement.cover.AnnouncementService.getInstance();
			state.setAttribute (STATE_ANNOUNCEMENT_SERVICE, aService);
			
			String channelId = ServerConfigurationService.getString ("channel", null);
			if(channelId == null)
			{
				channelId = aService.
						channelReference(PortalService.getCurrentSiteId(), SiteService.MAIN_CONTAINER);
				try
				{
					state.setAttribute (ANNOUNCEMENT_CHANNEL, aService.getAnnouncementChannel (channelId));
				}
				catch (IdUnusedException e)
				{
					Log.warn("chef","No announcement channel found. ");
					// the announcement channel is not created yet; go create
					try
					{
						aService.addAnnouncementChannel (channelId);
					}
					catch (PermissionException ee)
					{
						Log.warn("chef","Can not create announcement channel. ");
					}
					catch(IdUsedException ee)
					{
						
					}
					catch(IdInvalidException ee)
					{
						Log.warn("chef","The announcement channel could not be created because the Id is invalid. ");
					}
					catch(Exception ex)
					{
						Log.warn("chef", "Assignment : Action : init state : announcement exception : " + ex);
					}
				}
				catch (PermissionException e)
				{
					Log.warn("chef","No permission to annoucement channel. ");
				}
				catch(Exception ex)
				{
					Log.warn("chef", "Assignment : Action : init state : calendar exception : " + ex);
				}
			}
			
		}	// if
		
		if (state.getAttribute(VIEW_SUBMISSION_ASSIGNMENT_REFERENCE) == null)
		{
			// reset the view submission attributes
			resetViewSubmission (state);
		}
		if (state.getAttribute (STATE_CONTEXT_STRING)==null)
		{
			state.setAttribute (STATE_CONTEXT_STRING, PortalService.getCurrentSiteId());
		}	// if context string is null
		
		if (state.getAttribute (SORTED_BY)==null)
		{
			state.setAttribute (SORTED_BY, SORTED_BY_DUEDATE);
		}
		
		if (state.getAttribute (SORTED_ASC)==null)
		{
			state.setAttribute (SORTED_ASC, Boolean.TRUE.toString ());
		}
		
		if (state.getAttribute (SORTED_SUBMISSION_BY)==null)
		{
			state.setAttribute (SORTED_SUBMISSION_BY, SORTED_SUBMISSION_BY_LASTNAME);
		}
		
		if (state.getAttribute (SORTED_SUBMISSION_ASC)==null)
		{
			state.setAttribute (SORTED_SUBMISSION_ASC, Boolean.TRUE.toString ());
		}
		
		if (state.getAttribute (NEW_ASSIGNMENT_HIDE_OPTION_FLAG)==null)
		{
			resetAssignment (state);
		}
		
		if (state.getAttribute (STUDENT_LIST_SHOW_TABLE)==null)
		{
			state.setAttribute (STUDENT_LIST_SHOW_TABLE, new Hashtable ());
		}
		
		if(state.getAttribute(ATTACHMENTS_MODIFIED) == null)
		{
			state.setAttribute(ATTACHMENTS_MODIFIED, new Boolean(false));
		}

		// SECTION MOD
		if(state.getAttribute(STATE_SECTION_STRING) == null)
		{
			state.setAttribute(STATE_SECTION_STRING, "001");
		}
		
		// setup the observer to notify the Main panel
		if (state.getAttribute(STATE_OBSERVER) == null)
		{			
			// the delivery location for this tool
			String deliveryId = clientWindowId(state, portlet.getID());

			// the html element to update on delivery
			String elementId = mainPanelUpdateId(portlet.getID());

			// the event resource reference pattern to watch for
			String pattern = AssignmentService.assignmentReference((String) state.getAttribute (STATE_CONTEXT_STRING), "");

			state.setAttribute(STATE_OBSERVER, new MultipleEventsObservingCourier(deliveryId, elementId, pattern));
		}
		
		if (state.getAttribute(STATE_MODE) == null)
		{
			boolean allowAddNewAssignment = AssignmentService.allowAddAssignment ((String) state.getAttribute (STATE_CONTEXT_STRING));

			if (allowAddNewAssignment)
			{
				state.setAttribute (STATE_MODE, MODE_INSTRUCTOR_LIST_ASSIGNMENTS);
			}
			else
			{
				state.setAttribute (STATE_MODE, MODE_STUDENT_LIST_ASSIGNMENTS);
			}
		}
		
		if (state.getAttribute(STATE_SEARCH) == null)
		{
			state.setAttribute(STATE_SEARCH, "");
		}
		
		// make sure the observer is in sync with state
		updateObservation(state, portlet.getID());
	}	// initState
	
   /**
	* Setup our observer to be watching for change events
	* @param state The state object.
	*/
	private void updateObservation(SessionState state, String peid)
	{
		MultipleEventsObservingCourier observer = (MultipleEventsObservingCourier) state.getAttribute(STATE_OBSERVER);

		String pattern = AssignmentService.assignmentReference((String) state.getAttribute (STATE_CONTEXT_STRING), "");
		observer.setResourcePattern(pattern);

		// the delivery location for this tool
		String deliveryId = clientWindowId(state, peid);
		observer.setDeliveryId(deliveryId);
		
	}   // updateObservation
	
	/**
	 *	reset the attributes for view submission
	 */
	private void resetViewSubmission (SessionState state)
	{
		state.removeAttribute (VIEW_SUBMISSION_ASSIGNMENT_REFERENCE);
		state.removeAttribute (VIEW_SUBMISSION_TEXT);
		state.setAttribute (VIEW_SUBMISSION_HONOR_PLEDGE_YES, "false");
		
	}	// resetViewSubmission
	
	/**
	 *	reset the attributes for view submission
	 */
	private void resetAssignment (SessionState state)
	{
		// put the input value into the state attributes
		state.setAttribute (NEW_ASSIGNMENT_TITLE, "");
		
		// get current time
		Time t = TimeService.newTime();
		TimeBreakdown tB = t.breakdownLocal ();
		int month = tB.getMonth ();
		int day = tB.getDay ();
		int year = tB.getYear ();
		int hour = tB.getHour ();
		int min = tB.getMin ();
		String ampm= new String ();
		if (hour>12)
		{
			hour = hour-12;
			ampm="PM";
		}
		else if (hour == 0)
		{
			// for midnight point, we mark it as 12AM
			hour = 12;
			ampm = "AM";
		}
		else
		{
			ampm="AM";
		}
		
		// set the open time to be 12:00 PM
		state.setAttribute (NEW_ASSIGNMENT_OPENMONTH, new Integer (month));
		state.setAttribute (NEW_ASSIGNMENT_OPENDAY, new Integer (day));
		state.setAttribute (NEW_ASSIGNMENT_OPENYEAR, new Integer (year));
		state.setAttribute (NEW_ASSIGNMENT_OPENHOUR, new Integer (12));
		state.setAttribute (NEW_ASSIGNMENT_OPENMIN, new Integer (0));
		state.setAttribute (NEW_ASSIGNMENT_OPENAMPM, "PM");
		
		// due date is shifted forward by 7 days
		t.setTime (t.getTime () + 7*24*60*60*1000);
		tB = t.breakdownLocal ();
		month = tB.getMonth ();
		day = tB.getDay ();
		year = tB.getYear ();
		hour = tB.getHour ();
		min = tB.getMin ();
		ampm= new String ();
		if (hour>12)
		{
			hour = hour-12;
			ampm="PM";
		}
		else if (hour==0)
		{
			hour = 12;
			ampm = "AM";
		}
		else
		{
			ampm="AM";
		}
		
		// set the due time to be 5:00pm
		state.setAttribute (NEW_ASSIGNMENT_DUEMONTH, new Integer (month));
		state.setAttribute (NEW_ASSIGNMENT_DUEDAY, new Integer (day));
		state.setAttribute (NEW_ASSIGNMENT_DUEYEAR, new Integer (year));
		state.setAttribute (NEW_ASSIGNMENT_DUEHOUR, new Integer (5));
		state.setAttribute (NEW_ASSIGNMENT_DUEMIN, new Integer (0));
		state.setAttribute (NEW_ASSIGNMENT_DUEAMPM, "PM");
		
		// enable the close date by default
		state.setAttribute (NEW_ASSIGNMENT_ENABLECLOSEDATE, new Boolean (true));
		// set the close time to be 5:00 pm, same as the due time by default
		state.setAttribute (NEW_ASSIGNMENT_CLOSEMONTH, new Integer (month));
		state.setAttribute (NEW_ASSIGNMENT_CLOSEDAY, new Integer (day));
		state.setAttribute (NEW_ASSIGNMENT_CLOSEYEAR, new Integer (year));
		state.setAttribute (NEW_ASSIGNMENT_CLOSEHOUR, new Integer (5));
		state.setAttribute (NEW_ASSIGNMENT_CLOSEMIN, new Integer (0));
		state.setAttribute (NEW_ASSIGNMENT_CLOSEAMPM, "PM");
		
		state.setAttribute (NEW_ASSIGNMENT_SECTION, "001");
		state.setAttribute (NEW_ASSIGNMENT_SUBMISSION_TYPE, new Integer (Assignment.TEXT_AND_ATTACHMENT_ASSIGNMENT_SUBMISSION));
		state.setAttribute (NEW_ASSIGNMENT_GRADE_TYPE, new Integer (Assignment.UNGRADED_GRADE_TYPE));
		state.setAttribute (NEW_ASSIGNMENT_GRADE_POINTS, "");
		state.setAttribute (NEW_ASSIGNMENT_DESCRIPTION, "");
		state.setAttribute (NEW_ASSIGNMENT_RELEASE_GRADE_TYPE, Boolean.FALSE.toString ());
		state.setAttribute (ResourceProperties.NEW_ASSIGNMENT_CHECK_ADD_DUE_DATE, Boolean.FALSE.toString ());
		state.setAttribute (ResourceProperties.NEW_ASSIGNMENT_CHECK_AUTO_ANNOUNCE, Boolean.FALSE.toString ());
		// make the honor pledge not include as the default
		state.setAttribute (NEW_ASSIGNMENT_CHECK_ADD_HONOR_PLEDGE, (new Integer (Assignment.HONOR_PLEDGE_NONE)).toString ());
		
		state.setAttribute (NEW_ASSIGNMENT_ATTACHMENT, new ReferenceVector ());
		
		state.setAttribute (NEW_ASSIGNMENT_HIDE_OPTION_FLAG, new Boolean (false));
		
		state.setAttribute (NEW_ASSIGNMENT_FOCUS, NEW_ASSIGNMENT_TITLE);
		
		state.removeAttribute(NEW_ASSIGNMENT_DESCRIPTION_EMPTY);
		
	}	// resetNewAssignment
	
	/**
	 * construct a Hashtable using integer as the key and three character string of the month as the value
	 */
	private Hashtable monthTable ()
	{
		Hashtable n = new Hashtable ();
		n.put (new Integer (1), "JAN");
		n.put (new Integer (2), "FEB");
		n.put (new Integer (3), "MAR");
		n.put (new Integer (4), "APR");
		n.put (new Integer (5), "MAY");
		n.put (new Integer (6), "JUN");
		n.put (new Integer (7), "JUL");
		n.put (new Integer (8), "AUG");
		n.put (new Integer (9), "SEP");
		n.put (new Integer (10), "OCT");
		n.put (new Integer (11), "NOV");
		n.put (new Integer (12), "DEC");
		return n;
		
	}	// monthTable
	
	/**
	 * construct a Hashtable using the integer as the key and grade type String as the value
	 */
	private Hashtable gradeTypeTable ()
	{
		Hashtable n = new Hashtable ();
		n.put (new Integer (2), "Letter grade");
		n.put (new Integer (3), "Points");
		n.put (new Integer (4), "Pass/Fail");
		n.put (new Integer (5), "Checkmark");
		n.put (new Integer (1), "Ungraded");
		return n;
		
	}	// gradeTypeTable
	
	/**
	 * construct a Hashtable using the integer as the key and submission type String as the value
	 */
	private Hashtable submissionTypeTable ()
	{
		Hashtable n = new Hashtable ();
		n.put (new Integer (1), "Inline only");
		n.put (new Integer (2), "Attachments only");
		n.put (new Integer (3), "Inline and Attachments");
		return n;
		
	}	// submissionTypeTable
	
	/**
	 * get the AssignmentSubmission status
	 */
	private String getAssignmentSubmissionStatus (Assignment a, User u)
	{
		try
		{
			AssignmentSubmission submission = AssignmentService.getSubmission (a.getId (), u);
			if (submission!=null)
			{
				if (submission.getSubmitted ())
				{
					if (submission.getGraded () && submission.getGradeReleased ())
						return "Graded";
					else if (submission.getReturned ())
						return "Returned " + submission.getTimeReturned ().toStringLocalFull ();
					else
						return "Submitted " +  submission.getTimeSubmitted ().toStringLocalFull ();
				}
				else
					return "Drafted";
			}
			else
				return "Not Started";
			
		}
		catch (IdUnusedException e)
		{
			return "Can not find the assignment submission. ";
		}
		catch (PermissionException e)
		{
			return "You are not allowed to view the submission to this assignment. ";
		}
	}
	
	/**
	 * Sort based on the given property
	 * @param data The RunData object for this request
	 */
	public void doSort ( RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// we are changing the sort, so start from the first page again
		resetPaging(state);
		
		//get the ParameterParser from RunData
		ParameterParser params = data.getParameters ();
		
		String criteria = params.getString ("criteria");
		
		// current sorting sequence
		String asc = "";
		if (!criteria.equals (state.getAttribute (SORTED_BY)))
		{
			state.setAttribute (SORTED_BY, criteria);
			asc = Boolean.TRUE.toString ();
			state.setAttribute (SORTED_ASC, asc);
		}
		else
		{
			// current sorting sequence
			asc = (String) state.getAttribute (SORTED_ASC);
			
			//toggle between the ascending and descending sequence
			if (asc.equals (Boolean.TRUE.toString ()))
			{
				asc = Boolean.FALSE.toString ();
			}
			else
			{
				asc = Boolean.TRUE.toString ();
			}
			state.setAttribute (SORTED_ASC, asc);
		}
		
	}	// doSort
	
	/**
	 * Sort submission based on the given property
	 * @param data The RunData object for this request
	 */
	public void doSort_submission ( RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// we are changing the sort, so start from the first page again
		resetPaging(state);
		
		//get the ParameterParser from RunData
		ParameterParser params = data.getParameters ();
		
		String criteria = params.getString ("criteria");
		
		// current sorting sequence
		String asc = "";
		
		if (!criteria.equals (state.getAttribute (SORTED_SUBMISSION_BY)))
		{
			state.setAttribute (SORTED_SUBMISSION_BY, criteria);
			asc = Boolean.TRUE.toString ();
			state.setAttribute (SORTED_SUBMISSION_ASC, asc);
		}
		else
		{
			// current sorting sequence
			state.setAttribute (SORTED_SUBMISSION_BY, criteria);
			asc = (String) state.getAttribute (SORTED_SUBMISSION_ASC);
			
			//toggle between the ascending and descending sequence
			if (asc.equals (Boolean.TRUE.toString ()))
			{
				asc = Boolean.FALSE.toString ();
			}
			else
			{
				asc = Boolean.TRUE.toString ();
			}
			state.setAttribute (SORTED_SUBMISSION_ASC, asc);
		}		
	}	// doSort_submission
	
	/**
	 * the AssignmentComparator clas
	 */
	private class AssignmentComparator
	implements Comparator
	{
		/**
		 * the criteria
		 */
		String m_criteria = null;
		
		/**
		 * the criteria
		 */
		String m_asc = null;
		
		/**
		 * the user
		 */
		User m_user = null;
		
		/**
		 * constructor
		 * @param criteria The sort criteria string
		 * @param asc The sort order string. TRUE_STRING if ascending; "false" otherwise.
		 */
		public AssignmentComparator (String criteria, String asc)
		{
			m_criteria = criteria;
			m_asc = asc;
			
		}	// constructor
		
		/**
		 * constructor
		 * @param criteria The sort criteria string
		 * @param asc The sort order string. TRUE_STRING if ascending; "false" otherwise.
		 * @param user	The user object
		 */
		public AssignmentComparator (String criteria, String asc, User user)
		{
			m_criteria = criteria;
			m_asc = asc;
			m_user = user;
		}	// constructor
		
		/**
		 * 	implementing the compare function
		 * @param o1 The first object
		 * @param o2 The second object
		 * @return The compare result. 1 is o1 < o2; -1 otherwise
		 */
		public int compare ( Object o1, Object o2)
		{
			int result = -1;
			
			/************* fo sorting assignments *******************/
			if (m_criteria.equals (SORTED_BY_TITLE))
			{
				// sorted by the assignment title
				String s1 = ((Assignment) o1).getTitle ();
				String s2 = ((Assignment)o2).getTitle ();
				result =  s1.compareToIgnoreCase (s2);
			}
			else if (m_criteria.equals (SORTED_BY_SECTION))
			{
				// sorted by the assignment section
				String s1 = ((Assignment) o1).getSection ();
				String s2 = ((Assignment) o2).getSection ();
				result = s1.compareToIgnoreCase (s2);
			}
			else if (m_criteria.equals (SORTED_BY_DUEDATE))
			{
				// sorted by the assignment due date
				Time t1 = ((Assignment) o1).getDueTime ();
				Time t2 = ((Assignment) o2).getDueTime ();
				
				if (t1==null)
				{
					result = -1;
				}
				else if (t2==null)
				{
					result = 1;
				}
				else if (t1.before (t2))
				{
					result = -1;
				}
				else
				{
					result = 1;
				}
			}
			else if (m_criteria.equals (SORTED_BY_OPENDATE))
			{
				// sorted by the assignment open
				Time t1 = ((Assignment) o1).getOpenTime ();
				Time t2 = ((Assignment) o2).getOpenTime ();
				
				if (t1==null)
				{
					result = -1;
				}
				else if (t2==null)
				{
					result = 1;
				}
				if (t1.before (t2))
				{
					result = -1;
				}
				else
				{
					result = 1;
				}
			}
			else if (m_criteria.equals (SORTED_BY_ASSIGNMENT_STATUS))
			{
				String s1 = getAssignmentStatus ((Assignment) o1);
				String s2 = getAssignmentStatus ((Assignment) o2);
				result = s1.compareToIgnoreCase (s2);
			}
			else if (m_criteria.equals (SORTED_BY_NUM_SUBMISSIONS))
			{
				// sort by numbers of submissions
				
				// initialize
				int subNum1 = 0;
				int subNum2 = 0;
				
				Iterator submissions1 = AssignmentService.getSubmissions ((Assignment) o1);
				while (submissions1.hasNext ())
				{
					AssignmentSubmission submission1 = (AssignmentSubmission) submissions1.next ();
					if (submission1.getSubmitted())
						subNum1++;
				}
				
				Iterator submissions2 = AssignmentService.getSubmissions ((Assignment) o2);
				while (submissions2.hasNext ())
				{
					AssignmentSubmission submission2 = (AssignmentSubmission) submissions2.next ();
					if (submission2.getSubmitted())
						subNum2++;
				}
				
				result=(subNum1 > subNum2)?1:-1;
				
			}
			else if (m_criteria.equals (SORTED_BY_NUM_UNGRADED))
			{
				// sort by numbers of ungraded submissions
				
				// initialize
				int ungraded1 = 0;
				int ungraded2 = 0;
				
				Iterator submissions1 = AssignmentService.getSubmissions ((Assignment) o1);
				while (submissions1.hasNext ())
				{
					AssignmentSubmission submission1 = (AssignmentSubmission) submissions1.next ();
					if (submission1.getSubmitted() && !submission1.getGraded())
						ungraded1++;
				}
				
				Iterator submissions2 = AssignmentService.getSubmissions ((Assignment) o2);
				while (submissions2.hasNext ())
				{
					AssignmentSubmission submission2 = (AssignmentSubmission) submissions2.next ();
					if (submission2.getSubmitted() && !submission2.getGraded())
						ungraded2++;
				}
				
				result=(ungraded1 > ungraded2)?1:-1;
				
			}
			else if (m_criteria.equals (SORTED_BY_SUBMISSION_STATUS))
			{
				try
				{
					AssignmentSubmission submission1 = AssignmentService.getSubmission (((Assignment) o1).getId (), m_user);
					String status1= getSubmissionStatus (submission1, (Assignment) o1);
					
					AssignmentSubmission submission2 = AssignmentService.getSubmission (((Assignment) o2).getId (), m_user);
					String status2= getSubmissionStatus (submission2, (Assignment) o2);
					
					result = status1.compareTo (status2);
				}
				catch (IdUnusedException e)
				{
					return 1;
				}
				catch (PermissionException e)
				{
					return 1;
				}
			}
			else if (m_criteria.equals (SORTED_BY_GRADE))
			{
				try
				{
					AssignmentSubmission submission1 = AssignmentService.getSubmission (((Assignment) o1).getId (), m_user);
					String grade1 = " ";
					if (submission1!= null && submission1.getGraded () && submission1.getGradeReleased ())
					{
						grade1 = submission1.getGrade ();
					}
					
					AssignmentSubmission submission2 = AssignmentService.getSubmission (((Assignment) o2).getId (), m_user);
					String grade2 = " ";
					if ( submission2 !=null && submission2.getGraded () && submission2.getGradeReleased ())
					{
						grade2 = submission2.getGrade ();
					}
					
					result = grade1.compareTo (grade2);
				}
				catch (IdUnusedException e)
				{
					return 1;
				}
				catch (PermissionException e)
				{
					return 1;
				}
			}
			else if (m_criteria.equals (SORTED_BY_MAX_GRADE))
			{
				String maxGrade1 = maxGrade(((Assignment) o1).getContent().getTypeOfGrade(), (Assignment) o1);
				String maxGrade2 = maxGrade(((Assignment) o2).getContent().getTypeOfGrade(), (Assignment) o2);

				try 
				{
					// do integer comparation inside point grade type
					int max1 = Integer.parseInt(maxGrade1);
					int max2 = Integer.parseInt(maxGrade2);
					result = (max1<max2)?-1:1;	
				}
				catch (NumberFormatException e)
				{
					// otherwise do an alpha-compare
					result = maxGrade1.compareTo (maxGrade2);
				}
			}
			/******************* for sorting submissions **************/
			else if (m_criteria.equals (SORTED_SUBMISSION_BY_LASTNAME))
			{
				// sorted by the submitters last name
				User[] u1 = ((AssignmentSubmission) o1).getSubmitters();
				User[] u2 = ((AssignmentSubmission) o2).getSubmitters();
				
				if ( u1 == null || u2 == null)
				{
					return 1;
				}
				else
				{
					String submitters1 = "";
					String submitters2 = "";

					for (int j=0; j < u1.length; j++ )
					{
						if (u1[j] != null && u1[j].getLastName() != null)
						{
							if (j>0)
							{
								submitters1 = submitters1.concat("; ");
							}
							submitters1 = submitters1.concat(""+ u1[j].getLastName());
						}
					}

					for (int j=0; j < u2.length; j++)
					{
						if (u2[j] != null && u2[j].getLastName() != null)
						{
							if (j>0)
							{
								submitters2 = submitters2.concat("; ");
							}
							submitters2 = submitters2.concat(u2[j].getLastName());
						}
					}
					result = submitters1.compareTo (submitters2);
				}
			}
			else if (m_criteria.equals (SORTED_SUBMISSION_BY_SUBMIT_TIME))
			{
				// sorted by submission time
				Time t1 = ((AssignmentSubmission) o1).getTimeSubmitted ();
				Time t2 = ((AssignmentSubmission) o2).getTimeSubmitted ();
				
				if (t1==null)
				{
					result = -1;
				}
				else if (t2==null)
				{
					result = 1;
				}
				if (t1.before (t2))
				{
					result = -1;
				}
				else
				{
					result = 1;
				}
			}
			else if (m_criteria.equals (SORTED_SUBMISSION_BY_STATUS))
			{
				// sort by submission status
				String status1 = getSubmissionStatus((AssignmentSubmission) o1);
				String status2 = getSubmissionStatus((AssignmentSubmission) o2);
				
				result = status1.compareTo(status2);				
			}
			else if (m_criteria.equals (SORTED_SUBMISSION_BY_GRADE))
			{
				// sort by submission grade
				String grade1 = ((AssignmentSubmission) o1).getGrade();
				String grade2 = ((AssignmentSubmission) o2).getGrade();
				if (grade1 == null)
				{
					grade1 = "";
				}
				if (grade2 == null)
				{
					grade2 = "";
				}
				
				// if scale is points
				if ((((AssignmentSubmission) o1).getAssignment().getContent().getTypeOfGrade() == 3) && ((((AssignmentSubmission) o2).getAssignment().getContent().getTypeOfGrade() == 3)))
				{
					if (grade1.equals(""))
					{
						result = -1;
					}
					else if (grade2.equals(""))
					{
						result = 1;
					}
					else
					{
						result = (new Integer(grade1)).intValue()>(new Integer(grade2)).intValue()?1:-1;
						
					}
				}
				else
				{
					result = grade1.compareTo(grade2);				
				}			
			}	
			else if (m_criteria.equals (SORTED_SUBMISSION_BY_MAX_GRADE))
			{
				Assignment a1 = ((AssignmentSubmission) o1).getAssignment();
				Assignment a2 = ((AssignmentSubmission) o2).getAssignment();
				String maxGrade1 = maxGrade(a1.getContent().getTypeOfGrade(), a1);
				String maxGrade2 = maxGrade(a2.getContent().getTypeOfGrade(), a2);

				try 
				{
					// do integer comparation inside point grade type
					int max1 = Integer.parseInt(maxGrade1);
					int max2 = Integer.parseInt(maxGrade2);
					result = (max1<max2)?-1:1;	
				}
				catch (NumberFormatException e)
				{
					// otherwise do an alpha-compare
					result = maxGrade1.compareTo (maxGrade2);
				}
			}
			else if (m_criteria.equals (SORTED_SUBMISSION_BY_RELEASED))
			{
				// sort by submission released
				String released1 = (new Boolean(((AssignmentSubmission) o1).getGradeReleased())).toString();
				String released2 = (new Boolean(((AssignmentSubmission) o2).getGradeReleased())).toString();
				
				result = released1.compareTo(released2);					
			}	
			else if (m_criteria.equals (SORTED_SUBMISSION_BY_ASSIGNMENT))
			{
				// sort by submission's assignment
				String title1 = ((AssignmentSubmission) o1).getAssignment().getContent().getTitle();
				String title2 = ((AssignmentSubmission) o2).getAssignment().getContent().getTitle();
				
				result = title1.compareTo(title2);					
			}	
			else if (m_criteria.equals (SORTED_SUBMISSION_BY_MAX_GRADE))
			{
				// sort by submission max grade
				int maxGrade1 = ((AssignmentSubmission) o1).getAssignment().getContent().getMaxGradePoint();
				int maxGrade2 = ((AssignmentSubmission) o2).getAssignment().getContent().getMaxGradePoint();
				
				result = (maxGrade1<maxGrade2)?1:-1;					
			}	
			
			// sort ascending or descending
			if (m_asc.equals (Boolean.FALSE.toString ()))
			{
				result = -result;
			}
			return result;
		}	// compare
		
		/**
		 * get the submissin status
		 */
		private String getSubmissionStatus(AssignmentSubmission s)
		{
			String status = "";
			if (s.getGraded())
			{
				if (s.getGradeReleased())
				{
					status = "Released";
				}
				else
				{
					status = "Graded";
				}
			}
			else
			{
				status = "Ungraded";
			}
			return status;
			
		}	// getSubmissionStatus
		
		/**
		 * get the status string of assignment
		 */
		private String getAssignmentStatus (Assignment a)
		{
			String status = "";
			Time currentTime = TimeService.newTime();
			
			if (a.getDraft ())
				status = "Draft";
			else if (a.getOpenTime ().after (currentTime))
				status = "Not Open";
			else if (a.getDueTime ().after (currentTime))
				status = "Open";
			else if ((a.getCloseTime ()!=null)&&(a.getCloseTime ().before (currentTime)))
				status = "Closed";
			else
				status = "Due";
			return status;
		}	// getAssignmentStatus
		
		/**
		 * get submission status
		 */
		private String getSubmissionStatus (AssignmentSubmission submission, Assignment assignment)
		{
			String status = "";
			
			if (submission !=null)
				if (submission.getSubmitted ())
					if (submission.getGraded () && submission.getGradeReleased ())
						status = "Graded";
					else if (submission.getReturned ())
						status = "Returned " + submission.getTimeReturned ().toStringLocalFull ();
					else
					{
						status = "Submitted" + submission.getTimeSubmitted ().toStringLocalFull ();
						if (submission.getTimeSubmitted ().after (assignment.getDueTime ()))
							status = status + "- late";
					}
				else
					status = "In progress";
			else
				status = "Not Started";
			
			return status;
			
		}	// getSubmissionStatus
		
		/**
		 * get assignment maximun grade available based on the assignment grade type
		 * @param gradeType The int value of grade type
		 * @param a The assignment object
		 * @return The max grade String
		 */
		private String maxGrade(int gradeType, Assignment a)
		{
			String maxGrade = "";
			
			if (gradeType == -1)
			{
				// Grade type not set 
				maxGrade = "Grade Type Not Set";
			}
			else if (gradeType == 1)
			{
				// Ungraded grade type
				maxGrade = "No grade";
			}
			else if (gradeType == 2)
			{
				// Letter grade type 
				maxGrade = "A";
			}
			else if (gradeType == 3)
			{
				// Score based grade type
				maxGrade = Integer.toString(a.getContent().getMaxGradePoint());
			}
			else if (gradeType == 4)
			{
				// Pass/fail grade type 
				maxGrade = "Pass";
			}
			else if (gradeType == 5)
			{
				//Grade type that only requires a check
				maxGrade = "checkmark";
			}
			
			return maxGrade;
			
		}	// maxGrade
		
	}	// DiscussionComparator
	
	/**
	* Fire up the permissions editor
	*/
	public void doPermissions(RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState(((JetspeedRunData)data).getJs_peid());

		String contextString = (String) state.getAttribute (STATE_CONTEXT_STRING);
		String siteRef = SiteService.siteReference(contextString);

		// setup for editing the permissions of the site for this tool, using the roles of this site, too
		state.setAttribute(PermissionsAction.STATE_REALM_ID, siteRef);
		state.setAttribute(PermissionsAction.STATE_REALM_ROLES_ID, siteRef);

		// ... with this description
		state.setAttribute(PermissionsAction.STATE_DESCRIPTION, "Set permissions for Assignments in worksite "
				+ SiteService.getSiteDisplay(contextString));

		// ... showing only locks that are prpefixed with this
		state.setAttribute(PermissionsAction.STATE_PREFIX, "asn.");
		
		// disable auto-updates while leaving the list view
		((MultipleEventsObservingCourier) state.getAttribute(STATE_OBSERVER)).disable();
				
		// start the helper
		state.setAttribute(PermissionsAction.STATE_MODE, PermissionsAction.MODE_MAIN);

	}	// doPermissions

	/**
	 * transforms the Iterator to Vector
	 */
	private Vector iterator_to_vector(Iterator l)
	{
		Vector v = new Vector();
		while (l.hasNext ())
		{
			v.add (l.next());
		}
		return v;
	}	// iterator_to_vector
	
	/**
	* Implement this to return alist of all the resources that there are to page.
	* Sort them as appropriate.
	*/
	protected List readAllResources(SessionState state)
	{
		String mode = (String) state.getAttribute(STATE_MODE);
		String search = (String) state.getAttribute(STATE_SEARCH);

		// all the resources for paging 
		List returnResources = new Vector();
		
		if (mode.equalsIgnoreCase(MODE_INSTRUCTOR_LIST_ASSIGNMENTS))
		{
			// read all Assignments
			Iterator allAssignmentsIterator = null;

			// get those opened and posted assignments
			allAssignmentsIterator = AssignmentService.getAssignmentsForContext ((String) state.getAttribute (STATE_CONTEXT_STRING));


			// deal with no assignments
			if (allAssignmentsIterator == null) return new Vector();
			
			Vector assignments = new Vector();

			while (allAssignmentsIterator.hasNext())
			{
				Assignment a = (Assignment) allAssignmentsIterator.next();

				String deleted = a.getProperties().getProperty(ResourceProperties.PROP_ASSIGNMENT_DELETED);
				if (deleted == null || deleted.equals(""))
				{
					// not deleted, show it
					if (search != null && !search.equals(""))
					{
						// filtering if searching
						if (	StringUtil.containsIgnoreCase(a.getTitle(),search)
							||	StringUtil.containsIgnoreCase(a.getContent().getInstructions(),search))
						{
							assignments.add(a);
						}
					}
					else
					{
						assignments.add(a);
					}
				}

			}
			returnResources = assignments;
		}
		else if (mode.equalsIgnoreCase(MODE_INSTRUCTOR_VIEW_STUDENTS_ASSIGNMENT))
		{
			Iterator assignments = AssignmentService.getAssignmentsForContext ((String) state.getAttribute (STATE_CONTEXT_STRING));
			boolean found = false;
			while (assignments.hasNext () && !found)
			{
				Assignment a = (Assignment) assignments.next();
				String deleted = a.getProperties().getProperty(ResourceProperties.PROP_ASSIGNMENT_DELETED);
				if ((deleted == null || deleted.equals("")) && (!a.getDraft()))
				{
					// not deleted, show it
					List users = AssignmentService.allowAddSubmissionUsers (a.getReference ());
					
					// not deleted, show it
					if (search != null && !search.equals(""))
					{
						for (int i=0; i < users.size(); i++)
						{
							User u = (User) users.get(i);
							if (StringUtil.containsIgnoreCase(u.getDisplayName(),search))
							{
								returnResources.add(u);
							}
						}
					}
					else
					{
						returnResources = users;
					}
					found = true;
				}
			}		
		}
		else if (mode.equalsIgnoreCase(MODE_INSTRUCTOR_REPORT_SUBMISSIONS))
		{
			List users = new Vector();
			
			Vector submissions = new Vector();
			
			Vector assignments = iterator_to_vector(AssignmentService.getAssignmentsForContext ((String) state.getAttribute (STATE_CONTEXT_STRING)));
			if (assignments.size()>0)
			{
				//users = AssignmentService.allowAddSubmissionUsers (((Assignment)assignments.get(0)).getReference ());
			}	
			
			for (int j=0; j < assignments.size(); j++)
			{
				Assignment a = (Assignment) assignments.get(j);
				String deleted = a.getProperties().getProperty(ResourceProperties.PROP_ASSIGNMENT_DELETED);
				if ((deleted == null || deleted.equals("")) && (!a.getDraft()))
				{
					try
					{
						Vector assignmentSubmissions = iterator_to_vector(AssignmentService.getSubmissions(a));
						for (int k=0; k<assignmentSubmissions.size(); k++)
						{
							AssignmentSubmission s = (AssignmentSubmission) assignmentSubmissions.get(k);
							if (search != null && !search.equals(""))
							{
								// search result
								User[] submitters = s.getSubmitters();
								String names = "";
								for (int kk = 0; kk < submitters.length; kk++)
								{
									names = names.concat(((User)submitters[kk]).getDisplayName());
								}
								if (StringUtil.containsIgnoreCase(names,search))
								{
									if (s!=null && (s.getSubmitted() || (s.getReturned() && (s.getTimeLastModified().before(s.getTimeReturned())))))
									{
										// has been subitted or has been returned and not work on it yet
										submissions.add(s);
									}
								}
							}
							else
							{
								if (s!=null && (s.getSubmitted() || (s.getReturned() && (s.getTimeLastModified().before(s.getTimeReturned())))))
								{
									// has been subitted or has been returned and not work on it yet
									submissions.add(s);
								}
							}	// if-else
						}
					}
					catch (Exception e){}
				}
			}
			
			returnResources = submissions;
		}
		else if (mode.equalsIgnoreCase(MODE_STUDENT_LIST_ASSIGNMENTS))
		{
			// in the student list view of assignments
			Iterator assignments = AssignmentService.getAssignmentsForContext ((String) state.getAttribute (STATE_CONTEXT_STRING));
			Vector v = new Vector ();
			Time currentTime= TimeService.newTime();
			while (assignments.hasNext ())
			{
				Assignment a = (Assignment) assignments.next ();
				try
				{
					AssignmentSubmission submission = AssignmentService.getSubmission(a.getReference(), (User) state.getAttribute(STATE_USER));
					String deleted = a.getProperties().getProperty(ResourceProperties.PROP_ASSIGNMENT_DELETED);
					if (deleted == null || deleted.equals("") || (deleted.equalsIgnoreCase(Boolean.TRUE.toString()) && submission!=null))
					{
						// show not deleted assignments and those deleted assignments but the user has made submissions to them
						Time openTime = a.getOpenTime();
						if (openTime!=null && currentTime.after (openTime) && !a.getDraft())
						{
							if (search != null && !search.equals(""))
							{
								// search based on assignment title and instructions
								if (	StringUtil.containsIgnoreCase(a.getTitle(),search)
									||	StringUtil.containsIgnoreCase(a.getContent().getInstructions(),search))
								{
									returnResources.add (a);
								}
							}
							else
							{
								returnResources.add(a);
							}
						}
					}
				}
				catch (IdUnusedException e)
				{
					addAlert(state, "Can not find assignment. ");
				}
				catch (PermissionException e)
				{
					addAlert(state, "You are not allowed to view the assignment. ");
				}
			}
		}
		else if (mode.equalsIgnoreCase(MODE_INSTRUCTOR_GRADE_ASSIGNMENT))
		{
			try
			{
				Assignment a = AssignmentService.getAssignment ((String) state.getAttribute (EXPORT_ASSIGNMENT_REF));
				Vector submissions = iterator_to_vector(AssignmentService.getSubmissions(a));
			
				if (search != null && !search.equals(""))
				{
					// filtering if searching based on submitter's last name and first name
					for (int i=0; i<submissions.size(); i++)
					{
						AssignmentSubmission submission = (AssignmentSubmission) submissions.get(i);
						User[] users = submission.getSubmitters();
						for (int j = 0; j < users.length; j++)
						{
							User user = users[j];
						
							if (	StringUtil.containsIgnoreCase(user.getLastName(), search)
								||	StringUtil.containsIgnoreCase(user.getFirstName(), search))
							{
								returnResources.add(submission);
							}
						}
					}
				}
				else
				{
					returnResources = submissions;
				}
				
			}
			catch (IdUnusedException e )
			{
				addAlert(state, "Can not find the assignment. ");
			}
			catch (PermissionException e)
			{
				addAlert(state, "You are not allowed to view the assignment. ");
			}
		}
			
		// sort them all
		String ascending = "true";
		String sort = "";
		if (mode.equalsIgnoreCase(MODE_INSTRUCTOR_GRADE_ASSIGNMENT) || mode.equalsIgnoreCase(MODE_INSTRUCTOR_REPORT_SUBMISSIONS))
		{
			ascending = (String) state.getAttribute (SORTED_SUBMISSION_ASC);
			sort = (String) state.getAttribute (SORTED_SUBMISSION_BY);
		}
		else
		{
			ascending = (String) state.getAttribute(SORTED_ASC);
			sort = (String) state.getAttribute(SORTED_BY);
		}
		
		if ((returnResources.size() > 1) && !mode.equalsIgnoreCase(MODE_INSTRUCTOR_VIEW_STUDENTS_ASSIGNMENT))
		{
			Collections.sort(returnResources, new AssignmentComparator(sort, ascending));
		}

		return returnResources;

	}	// readAllResources

	/**
	 *
	 */
	public void doView (RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		// we are changing the view, so start with first page again. 
		resetPaging(state);
	
		// clear search form
		doSearch_clear(data, context);
		
		String viewMode = data.getParameters ().getString("view");
		state.setAttribute(STATE_SELECTED_VIEW, viewMode);
		
		if (viewMode.equalsIgnoreCase("List of Assignments"))
		{
			doList_assignments(data, context);
		}
		else if (viewMode.equalsIgnoreCase("List of Assignments by student"))
		{
			doView_students_assignment(data, context);
		}
		else if (viewMode.equalsIgnoreCase("Grade Report"))
		{
			doReport_submissions(data, context);
		}
		else if (viewMode.equalsIgnoreCase("Student View"))
		{
			doView_student(data, context);
		}		
		
	}	// doView
	

	/**
	 * put those variables related to 2ndToolbar into context 
	 */
	private void add2ndToolbarFields(RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState (((JetspeedRunData)data).getJs_peid ());
		
		context.put("totalPageNumber", new Integer(totalPageNumber(state)));
		context.put("searchString", state.getAttribute(STATE_SEARCH));
		context.put("form_search", FORM_SEARCH);
		context.put("formPageNumber", FORM_PAGE_NUMBER);
		context.put("prev_page_exists", state.getAttribute(STATE_PREV_PAGE_EXISTS));
		context.put("next_page_exists", state.getAttribute(STATE_NEXT_PAGE_EXISTS));
		context.put("paged", state.getAttribute(STATE_PAGING));
		context.put("current_page", state.getAttribute(STATE_CURRENT_PAGE));
		context.put("selectedView", state.getAttribute(STATE_MODE));

	}	//add2ndToolbarFields
	
	/**
	 * valid grade?
	 */ 
	private void validPointGrade(SessionState state, String grade)
	{
		if (grade != null && !grade.equals(""))
		{
			if (grade.startsWith("-"))
			{
				// check for negative sign
				addAlert(state, "Please use zero or positive number for grade. ");
			}
			else
			{
				int index = grade.indexOf(".");
				if (index != -1)
				{
					// when there is decimal points inside the grade, scale the number by 10
					// but only one decimal place is supported
					// for example, change 100.0 to 1000
					if (!grade.equals("."))
					{
						if (grade.length() > index + 2)
						{
							// if there are more than one decimal point
							addAlert(state, "Please use only one decimal place in grade. ");
						}
						else
						{
							// decimal points is the only allowed character inside grade
							// replace it with '1', and try to parse the new String into int
							String parseString = grade.replace('.', '1');
							try
							{
								Integer.parseInt(parseString);
							}
							catch (NumberFormatException e)
							{
								addAlert(state, "Please use number for the grade field. ");
							}
						}
					}	
					else
					{
						//grade is "."
						addAlert(state, "Please use number for the grade field. ");
					}
				}
				else
				{
					// There is no decimal point; should be int number
					try 
					{
						Integer.parseInt(grade);
					}
					catch (NumberFormatException e)
					{
						addAlert(state, "Please use number for the grade field. ");
					}
				}
			}
		}
		
	}	// validPointGrade
	
	/**
	 * display grade properly
	 */
	private String displayGrade(SessionState state, String grade)
	{
		if (state.getAttribute(STATE_MESSAGE) == null)
		{
			if (grade != null && (grade.length() >= 1))
			{
				if (grade.indexOf(".") != -1 )	
				{
					if (grade.startsWith("."))
					{
						grade = "0".concat(grade);
					}
					else if (grade.endsWith("."))
					{
						grade = grade.concat("0");
					}
				}
				else
				{
					try 
					{
						Integer.parseInt(grade);
						grade = grade.substring(0, grade.length()-1) + "." + grade.substring(grade.length()-1);
					}
					catch (NumberFormatException e){}
				}
			}
			else
			{
				grade = "";
			}
		}
		return grade;
		
	}	// displayGrade
	
	/**
	 * scale the point value by 10 if there is a valid point grade
	 */
	private String scalePointGrade(SessionState state, String point) 
	{
		validPointGrade(state, point);
		if (state.getAttribute(STATE_MESSAGE) == null) 
		{
			if (point != null && (point.length() >=1))
			{
				// when there is decimal points inside the grade, scale the number by 10
				// but only one decimal place is supported
				// for example, change 100.0 to 1000
				int index = point.indexOf(".");
				if (index != -1) {
					if (index == 0) {
						// if the point is the first char, add a 0 for the integer part
						point = "0".concat(point.substring(1));
					}
					else if (index < point.length()-1) {
						// use scale integer for gradePoint
						point = point.substring(0, index) + point.substring(index + 1);
					}
					else {
						// decimal point is the last char
						point = point.substring(0, index) + "0";
					}
				}
				else {
					// if there is no decimal place, scale up the integer by 10
					point = point + "0";
				}
			}
		}
		return point;
		
	}	// scalePointGrade
	
	/**
	 * Processes formatted text that is coming back from the browser 
	 * (from the formatted text editing widget).
	 * @param state Used to pass in any user-visible alerts or errors when processing the text
	 * @param strFromBrowser The string from the browser
	 * @return The formatted text
	 */
	private String processFormattedTextFromBrowser(SessionState state, String strFromBrowser)
	{
		StringBuffer alertMsg = new StringBuffer();
		String text = FormattedText.processFormattedText(strFromBrowser, alertMsg);
		if (alertMsg.length() > 0) addAlert(state, alertMsg.toString());
		return text;
	}
	
}	// AssignmentAction

/**********************************************************************************
*
* $Header: /cvs/sakai/chef-tool/src/java/org/sakaiproject/tool/assignment/AssignmentAction.java,v 1.13 2004/06/22 03:04:56 ggolden Exp $
*
**********************************************************************************/
