/**********************************************************************************
*
* $Header: /cvs/sakai/legacy-component/src/java/org/sakaiproject/component/legacy/dissertation/BaseDissertationService.java,v 1.14 2004/06/22 03:13:58 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.component.legacy.dissertation;

// import
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.ListIterator;
import java.util.Stack;
import java.util.Vector;

import org.sakaiproject.exception.IdInvalidException;
import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.exception.IdUsedException;
import org.sakaiproject.exception.InUseException;
import org.sakaiproject.exception.PermissionException;
import org.sakaiproject.service.framework.config.cover.ServerConfigurationService;
import org.sakaiproject.service.framework.log.Logger;
import org.sakaiproject.service.framework.memory.Cache;
import org.sakaiproject.service.framework.memory.CacheRefresher;
import org.sakaiproject.service.framework.memory.MemoryService;
import org.sakaiproject.service.framework.session.cover.UsageSessionService;
import org.sakaiproject.service.framework.state.SessionStateBindingListener;
import org.sakaiproject.service.legacy.dissertation.BlockGrantGroup;
import org.sakaiproject.service.legacy.dissertation.BlockGrantGroupEdit;
import org.sakaiproject.service.legacy.dissertation.CandidateInfo;
import org.sakaiproject.service.legacy.dissertation.CandidateInfoEdit;
import org.sakaiproject.service.legacy.dissertation.CandidatePath;
import org.sakaiproject.service.legacy.dissertation.CandidatePathEdit;
import org.sakaiproject.service.legacy.dissertation.Dissertation;
import org.sakaiproject.service.legacy.dissertation.DissertationEdit;
import org.sakaiproject.service.legacy.dissertation.DissertationService;
import org.sakaiproject.service.legacy.dissertation.DissertationStep;
import org.sakaiproject.service.legacy.dissertation.DissertationStepEdit;
import org.sakaiproject.service.legacy.dissertation.StepStatus;
import org.sakaiproject.service.legacy.dissertation.StepStatusEdit;
import org.sakaiproject.service.legacy.event.Event;
import org.sakaiproject.service.legacy.event.cover.EventTrackingService;
import org.sakaiproject.service.legacy.id.cover.IdService;
import org.sakaiproject.service.legacy.resource.Edit;
import org.sakaiproject.service.legacy.resource.Resource;
import org.sakaiproject.service.legacy.resource.ResourceProperties;
import org.sakaiproject.service.legacy.resource.ResourcePropertiesEdit;
import org.sakaiproject.service.legacy.security.cover.SecurityService;
import org.sakaiproject.service.legacy.time.Time;
import org.sakaiproject.service.legacy.time.TimeBreakdown;
import org.sakaiproject.service.legacy.time.cover.TimeService;
import org.sakaiproject.service.legacy.user.User;
import org.sakaiproject.service.legacy.user.cover.UserDirectoryService;
import org.sakaiproject.util.StringUtil;
import org.sakaiproject.util.Validator;
import org.sakaiproject.util.Xml;
import org.sakaiproject.util.resource.BaseResourcePropertiesEdit;
import org.sakaiproject.util.storage.StorageUser;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
* <p>BaseDissertationService is the abstract service class for Dissertations.</p>
* <p>The Concrete Service classes extending this are the XmlFile and DbCached storage classes.</p>
*
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.14 $
* @see org.chefproject.core.Dissertation
*/
public abstract class BaseDissertationService
	implements DissertationService
{	
	/** regular expressions in data validation **/
	
		//general patterns
	public static final String  PATTERN_DATE = "(^([0-9]|[0-9][0-9])/([0-9]|[0-9][0-9])/[0-9]{4}$|)";
	public static final String  PATTERN_NAME = "(^\"[[A-Za-z][-][.][ ][,][']]*\"$|^\"\"$)";
	 
		//required (not null) data common to both database extracts
	public static final String	PATTERN_UMID = "^\"[0-9]{8}\"$";
	public static final String PATTERN_CAMPUS_ID = "(^\"[A-Za-z0-9]{1,8}\"\r?$)|(^\".+@.+\"\r?$)";
	 
		//MPathways fields
	public static final String PATTERN_ACAD_PROG = "(^\"[0-9]{5}\"$|^\"\"$)";
	public static final String PATTERN_ANTICIPATE = "(^\"[A-Z]{2}[- ][0-9]{4}\"$|^\"[A-Za-z]*( |,)[0-9]{4}\"$|^\"\"$)";
	public static final String PATTERN_DATE_COMPL = "(^([0-9]|[0-9][0-9])/([0-9]|[0-9][0-9])/[0-9]{4}$|)";
	public static final String PATTERN_MILESTONE = "(^\"[A-Za-z]*\"$|^\"\"$)";
	public static final String PATTERN_ACADEMIC_PLAN = "(^\"[0-9]{4}[A-Z0-9]*\"|^\"[0-9]{4}[A-Z0-9]*\"\r?$|^\"\"$|^\"\"\r?$)";
		
		//Rackham OARD database fields
	public static final String 	PATTERN_FOS = "(^\"[0-9]{4}\"$|^\"\"$)";
	public static final String 	PATTERN_LNAME = PATTERN_NAME;
	public static final String 	PATTERN_FNAME = PATTERN_NAME;
	public static final String 	PATTERN_DEGREETERM_TRANS = "(^\"[A-Za-z]{2}( |-)[0-9]{4}\"$|^\"\"$)";
	public static final String 	PATTERN_ORAL_EXAM_DATE = PATTERN_DATE;
	public static final String 	PATTERN_ORAL_EXAM_TIME = "(^\".*\"$|^\"\"$)"; //not restrictive
	public static final String	PATTERN_ORAL_EXAM_PLACE = "(^\".*\"$|^\"\"$)"; //not restrictive
	public static final String 	PATTERN_COMMITTEE_APPROVED_DATE = PATTERN_DATE;
	public static final String 	PATTERN_FIRST_FORMAT_DATE = PATTERN_DATE;
	public static final String 	PATTERN_BINDER_RECEIPT_DATE = PATTERN_DATE;
	public static final String 	PATTERN_FEE_REQUIREMENT_MET = "(^\"[A-Za-z]\"$|^\"\"$)";
	public static final String 	PATTERN_FEE_DATE_RECEIPT = PATTERN_DATE;
	public static final String 	PATTERN_FEE_DATE_RECEIVED = PATTERN_DATE;
	public static final String 	PATTERN_ORAL_REPORT_RETURN = PATTERN_DATE;
	public static final String 	PATTERN_UNBOUND_DATE = PATTERN_DATE;	
	public static final String 	PATTERN_ABSTRACT_DATE = PATTERN_DATE;
	public static final String 	PATTERN_BOUND_COPY_RECEIVED_DATE = PATTERN_DATE;
	public static final String 	PATTERN_DIPLOMA_APPLICATION_DATE = PATTERN_DATE;
	public static final String 	PATTERN_CONTRACT_RECEIVED_DATE = PATTERN_DATE;
	public static final String 	PATTERN_NSF_SURVEY_DATE = PATTERN_DATE;
	public static final String 	PATTERN_DEGREE_CONFERRED_DATE = PATTERN_DATE;
	public static final String 	PATTERN_FINAL_FORMAT_DATE = "(^\"([A-Za-z]{3}|[A-Za-z]{2})\"$|^\"\"$)";
	public static final String 	PATTERN_UPDATE_DATE = PATTERN_DATE;
	public static final String 	PATTERN_COMM_CERT_DATE = PATTERN_DATE;
	public static final String 	PATTERN_ROLE = "(^\"([A-Za-z]|[A-Za-z]{2})\"$|^\"\"$)";
	public static final String  PATTERN_MEMBER = "(^\".*\"$|^\"\"$)"; //not restrictive
	public static final String 	PATTERN_EVAL_DATE = "(^([0-9]|[0-9][0-9])/([0-9]|[0-9][0-9])/[0-9]{4}$|^([0-9]|[0-9][0-9])/([0-9]|[0-9][0-9])/[0-9]{4}\r$|\r$|)";
	
	/** A Storage object for persistent storage of Dissertations. */
	protected DissertationStorage m_dissertationStorage = null;

	/** A Storage object for persistent storage of DissertationSteps. */
	protected DissertationStepStorage m_stepStorage = null;
	
	/** A Storage object for persistent storage of CandidatePaths. */
	protected CandidatePathStorage m_pathStorage = null;

	/** A Storage object for persistent storage of StepStatus. */
	protected StepStatusStorage m_statusStorage = null;
	
	/** A Storage object for persistent storage of BlockGrantGroup. */
	protected BlockGrantGroupStorage m_groupStorage = null;

	/** A Storage object for persistent storage of CandidateInfos. */
	protected CandidateInfoStorage m_infoStorage = null;

	/** A Cache for this service -  Dissertations keyed by reference. */
	protected Cache m_dissertationCache = null;

	/** A Cache for this service -  DissertationSteps keyed by reference. */
	protected Cache m_stepCache = null;
	
	/** A Cache for this service -  CandidatePaths keyed by reference. */
	protected Cache m_pathCache = null;

	/** A Cache for this service -  StepStatus keyed by reference. */
	protected Cache m_statusCache = null;
	
	/** A Cache for this service -  BlockGrantGroup keyed by reference. */
	protected Cache m_groupCache = null;

	/** A Cache for this service -  CandidateInfos keyed by reference. */
	protected Cache m_infoCache = null;
	
	/** The access point URL. */
	protected String m_relativeAccessPoint = null;

	/** Is the system ready for student use ? */
	protected boolean m_initialized = false;
	
	/** Is is a Rackham OARD/MP data load in progress? */
	protected boolean m_loading = false;
	
	/** The group id for the parent graduate school. */
	protected String m_schoolSite = null;
	
	/** The group id for the parent music performance. */
	protected String m_musicPerformanceSite = null;
	
	/** Holds the time as a long that the db was last queried. */
	protected long m_lastQuery = -1;

	/** Holds the School administrative group numbers. */
	protected Hashtable m_schoolGroups = null;
	
	
	/*******************************************************************************
	* Abstractions, etc.
	*******************************************************************************/
	
	/**
	* Construct a Storage object for Dissertations.
	* @return The new storage object.
	*/
	protected abstract DissertationStorage newDissertationStorage();
	
	/**
	* Construct a Storage object for DissertationSteps.
	* @return The new storage object.
	*/
	protected abstract DissertationStepStorage newDissertationStepStorage();
	
	/**
	* Construct a Storage object for CandidatePaths.
	* @return The new storage object.
	*/
	protected abstract CandidatePathStorage newCandidatePathStorage();
	
	/**
	* Construct a Storage object for StepStatus.
	* @return The new storage object.
	*/
	protected abstract StepStatusStorage newStepStatusStorage();

	/**
	* Construct a Storage object for CandidateInfos.
	* @return The new storage object.
	*/
	protected abstract CandidateInfoStorage newCandidateInfoStorage();
	
	/**
	* Construct a Storage object for BlockGrantGroups.
	* @return The new storage object.
	*/
	protected abstract BlockGrantGroupStorage newBlockGrantGroupStorage();


	/**
	* Access the partial URL that forms the root of resource URLs.
	* @param relative - if true, form within the access path only (i.e. starting with /msg)
	* @return the partial URL that forms the root of resource URLs.
	*/
	protected String getAccessPoint(boolean relative)
	{
		return (relative ? "" : ServerConfigurationService.getAccessUrl()) + m_relativeAccessPoint;

	}	// getAccessPoint

	/**
	* Access the internal reference which can be used to access the resource from within the system.
	* @param id The dissertation id string.
	* @return The the internal reference which can be used to access the resource from within the system.
	*/
	protected String dissertationReference(String site, String id)
	{
		String retVal = null;
		if(site == null)
			retVal = getAccessPoint(true) + Resource.SEPARATOR + "d" + Resource.SEPARATOR + id;
		else
			retVal = getAccessPoint(true) + Resource.SEPARATOR + "d" + Resource.SEPARATOR + getSchoolSite() + Resource.SEPARATOR + site + Resource.SEPARATOR + id;
		return retVal;

	}   // dissertationReference
	
	/**
	* Access the internal reference which can be used to access the resource from within the system.
	* @param id The step id string.
	* @return The the internal reference which can be used to access the resource from within the system.
	*/
	protected String stepReference(String site, String id)
	{
		String retVal = null;
		if(site == null)
			retVal = getAccessPoint(true) + Resource.SEPARATOR + "s" + Resource.SEPARATOR + id;
		else
			retVal = getAccessPoint(true) + Resource.SEPARATOR + "s" + Resource.SEPARATOR + getSchoolSite() + Resource.SEPARATOR + site + Resource.SEPARATOR + id;
		return retVal;

	}   // stepReference

	/**
	* Access the internal reference which can be used to access the resource from within the system.
	* @param id The status id string.
	* @return The the internal reference which can be used to access the resource from within the system.
	*/
	protected String statusReference(String site, String id)
	{
		String retVal = null;
		if(site == null)
			retVal = getAccessPoint(true) + Resource.SEPARATOR + "ss" + Resource.SEPARATOR + id;
		else
			retVal = getAccessPoint(true) + Resource.SEPARATOR + "ss" + Resource.SEPARATOR + getSchoolSite() + Resource.SEPARATOR + site + Resource.SEPARATOR + id;
		return retVal;

	}   // statusReference

	/**
	* Access the internal reference which can be used to access the resource from within the system.
	* @param id The path id string.
	* @return The the internal reference which can be used to access the resource from within the system.
	*/
	protected String pathReference(String site, String id)
	{
		String retVal = null;
		if(site == null)
			retVal = getAccessPoint(true) + Resource.SEPARATOR + "p" + Resource.SEPARATOR + id;
		else
			retVal = getAccessPoint(true) + Resource.SEPARATOR + "p" + Resource.SEPARATOR + getSchoolSite() + Resource.SEPARATOR + site + Resource.SEPARATOR + id;
		return retVal;

	}   // pathReference

	/**
	* Access the internal reference which can be used to access the resource from within the system.
	* @param id The info id string.
	* @return The the internal reference which can be used to access the resource from within the system.
	*/
	protected String infoReference(String site, String id)
	{
		String retVal = null;
		if(site == null)
			retVal = getAccessPoint(true) + Resource.SEPARATOR + "i" + Resource.SEPARATOR + id;
		else
			retVal = getAccessPoint(true) + Resource.SEPARATOR + "i" + Resource.SEPARATOR + getSchoolSite() + Resource.SEPARATOR + site + Resource.SEPARATOR + id;
		return retVal;

	}   // infoReference
	
	/**
	* Access the internal reference which can be used to access the resource from within the system.
	* @param id The Block Grant Group id string.
	* @return The internal reference which can be used to access the resource from within the system.
	*/
	protected String blockGrantGroupReference(String site, String id)
	{
		String retVal = null;
		if(site == null)
			retVal = getAccessPoint(true) + Resource.SEPARATOR + "g" + Resource.SEPARATOR + id;
		else
			retVal = getAccessPoint(true) + Resource.SEPARATOR + "g" + Resource.SEPARATOR + getSchoolSite() + Resource.SEPARATOR + site + Resource.SEPARATOR + id;
		
		return retVal;

	}   // blockGrantGroupReference
	
	/**
	* Access the dissertation id extracted from an dissertation reference.
	* @param ref The dissertation reference string.
	* @return The the dissertation id extracted from an dissertation reference.
	*/
	protected String dissertationId(String ref)
	{
		int i = ref.lastIndexOf(Resource.SEPARATOR);
		if (i == -1) return ref;
		String id = ref.substring(i + 1);
		return id;

	}   // dissertationId
	
	/**
	* Access the BlockGrantGroup id extracted from a BlockGrantGroup reference.
	* @param ref The BlockGrantGroup reference string.
	* @return The BlockGrantGroup id extracted from a BlockGrantGroup reference.
	*/
	protected String blockGrantGroupId(String ref)
	{
		int i = ref.lastIndexOf(Resource.SEPARATOR);
		if (i == -1) return ref;
		String id = ref.substring(i + 1);
		return id;

	}   // blockGrantGroupId

	/**
	* Access the step id extracted from an step reference.
	* @param ref The step reference string.
	* @return The the step id extracted from an step reference.
	*/
	protected String stepId(String ref)
	{
		int i = ref.lastIndexOf(Resource.SEPARATOR);
		if (i == -1) return ref;
		String id = ref.substring(i + 1);
		return id;

	}   // stepId

	/**
	* Access the path id extracted from an path reference.
	* @param ref The path reference string.
	* @return The the path id extracted from an path reference.
	*/
	protected String pathId(String ref)
	{
		int i = ref.lastIndexOf(Resource.SEPARATOR);
		if (i == -1) return ref;
		String id = ref.substring(i + 1);
		return id;

	}   // pathId

	/**
	* Access the status id extracted from an status reference.
	* @param ref The status reference string.
	* @return The the status id extracted from an status reference.
	*/
	protected String statusId(String ref)
	{
		int i = ref.lastIndexOf(Resource.SEPARATOR);
		if (i == -1) return ref;
		String id = ref.substring(i + 1);
		return id;

	}   // statusId

	/**
	* Access the info id extracted from an info reference.
	* @param ref The info reference string.
	* @return The the info id extracted from an info reference.
	*/
	protected String infoId(String ref)
	{
		int i = ref.lastIndexOf(Resource.SEPARATOR);
		if (i == -1) return ref;
		String id = ref.substring(i + 1);
		return id;

	}   // infoId	


	/**
	* Check security permission.
	* @param lock - The lock id string.
	* @param resource - The resource reference string, or null if no resource is involved.
	* @return true if allowed, false if not
	*/
	protected boolean unlockCheck(String lock, String resource)
	{
		//m_logger.debug("DISSERTATION BASE SERVICE : UNLOCK CHECK");
		if (!SecurityService.unlock(lock, resource))
		{
			return false;
		}

		return true;

	}// unlockCheck

	/**
	* Check security permission.
	* @param lock - The lock id string.
	* @param resource - The resource reference string, or null if no resource is involved.
	* @exception PermissionException Thrown if the user does not have access
	*/
	protected void unlock(String lock, String resource)
		throws PermissionException
	{
		if (!unlockCheck(lock, resource))
		{
			throw new PermissionException(UsageSessionService.getSessionUser().getId(), lock, resource);
		}

	}	// unlock



	/*******************************************************************************
	* Dependencies and their setter methods
	*******************************************************************************/

	/** Dependency: logging service */
	protected Logger m_logger = null;

	/**
	 * Dependency: logging service.
	 * @param service The logging service.
	 */
	public void setLogger(Logger service)
	{
		m_logger = service;
	}

	/** Dependency: MemoryService. */
	protected MemoryService m_memoryService = null;

	/**
	 * Dependency: MemoryService.
	 * @param service The MemoryService.
	 */
	public void setMemoryService(MemoryService service)
	{
		m_memoryService = service;
	}

	/**
	 * Configuration: Set the school group id.
	 * @param id the schoold group id.
	 */
	public void setSchoolSite(String id)
	{
		m_schoolSite = id;
	}
	
	/**
	 * Configuration: Set the music performance site id.
	 * @param id the music performanced site id.
	 */
	public void setMusicPerformanceSite(String id)
	{
		m_musicPerformanceSite = id;
	}

	/**
	 * Final initialization, once all dependencies are set.
	 */
	public void init()
	{
		try
		{
			m_relativeAccessPoint = REFERENCE_ROOT;
			
			setSchoolSite("rackham");
			setMusicPerformanceSite("diss089");

			// construct storage helpers and read
			m_groupStorage = newBlockGrantGroupStorage();
			m_groupStorage.open();
			m_dissertationStorage = newDissertationStorage();
			m_dissertationStorage.open();
			m_stepStorage = newDissertationStepStorage();
			m_stepStorage.open();
			m_pathStorage = newCandidatePathStorage();
			m_pathStorage.open();
			m_statusStorage = newStepStatusStorage();
			m_statusStorage.open();
			m_infoStorage = newCandidateInfoStorage();
			m_infoStorage.open();
	
			m_groupCache = m_memoryService.newCache(new BlockGrantGroupCacheRefresher(), blockGrantGroupReference(null, ""));
			m_dissertationCache = m_memoryService.newCache(new DissertationCacheRefresher(), dissertationReference(null, ""));
			m_stepCache = m_memoryService.newCache(new DissertationStepCacheRefresher(), stepReference(null, ""));
			m_pathCache = m_memoryService.newCache(new CandidatePathCacheRefresher(), pathReference(null, ""));
			m_statusCache = m_memoryService.newCache(new StepStatusCacheRefresher(), statusReference(null, ""));
			m_infoCache = m_memoryService.newCache(new CandidateInfoCacheRefresher(), infoReference(null, ""));
		
			// lookup table of field/program to block grant group used in loading student data
			m_schoolGroups = new Hashtable();
			
			// test identities
			
			//check that group code storage has been initialized
			List storedCodes = m_groupStorage.getAll();
			boolean initialized = storedCodes.isEmpty() ? false : true;
			if(initialized)
			{
				List schoolGroups = (List)getBlockGrantGroups();
				for(int i=0; i<schoolGroups.size(); i++)
				{
					BlockGrantGroup group = (BlockGrantGroup)schoolGroups.get(i);
					Hashtable fields = (Hashtable)group.getFieldsOfStudy();
					Enumeration codes = fields.keys();
					while(codes.hasMoreElements())
					{
						String field = (String)codes.nextElement();
						m_schoolGroups.put(field, group.getCode());
					}
				}
			}
			else
			{
				//initialize the group code storage
				m_schoolGroups = initCodes();
			}
			
		}
		catch (Throwable t)
		{
			m_logger.warn(this +".init(): ", t);
		}
		

	} // init
	
	/**
	* Returns to uninitialized state.
	*/
	protected Hashtable initCodes()
	{
		//%%% groups with only program codes are skipped in creating groups
		Hashtable m_schoolGroups = new Hashtable();
		BlockGrantGroupEdit edit = null;
		String fosString = null;
		String programString = null;
		String bggString = null;
		try
		{	
			fosString = "0030";
			programString = "00043";
			bggString = "001";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
			
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("AEROSPACE ENGINEERING");
			edit.addFieldOfStudy(fosString, "Aerospace Engineering");
			commitEdit(edit);
		
			fosString = "0100";
			programString = "00045";
			bggString = "001";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
							
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("AEROSPACE ENGINEERING");
			edit.addFieldOfStudy(fosString, "Aerospace Science");
			commitEdit(edit);
					
			fosString = "0130";
			programString = "00047";
			bggString = "002";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("AMERICAN CULTURE");
			edit.addFieldOfStudy(fosString, "American Culture");
			commitEdit(edit);
					
			fosString = "0190";
			programString = "00049";
			bggString = "022";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("CELL & DEVELOPMENTAL BIOLOGY");
			edit.addFieldOfStudy(fosString, "Anatomy & Cell Biology");
			commitEdit(edit);
					
			fosString = "0300";
			programString = "00051";
			bggString = "003";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ANTHROPOLOGY");
			edit.addFieldOfStudy(fosString, "Anthropology");
			commitEdit(edit);
					
			fosString = "0310";
			programString = "00054";
			bggString = "004";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ANTHROPOLOGY & HISTORY");
			edit.addFieldOfStudy(fosString, "Anthropology and History");
			commitEdit(edit);
					
			fosString = "0400";
			programString = "01733";
			bggString = "009";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ASIAN LANGUAGES & CULTURES");
			edit.addFieldOfStudy(fosString, "Asian Languages & Cultures");
			commitEdit(edit);
					
			fosString = "0420";
			programString = "01568";
			bggString = "154";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("APPL & INTERDISCIPL MATHEMATICS");
			edit.addFieldOfStudy(fosString, "Applied and Interdisciplinary Mathematics");
			commitEdit(edit);
					
			fosString = "0460";
			programString = "01573";
			bggString = "014";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ASTRONOMY");
			edit.addFieldOfStudy(fosString, "Astronomy and Astrophysics");
			commitEdit(edit);
					
			fosString = "0480";
			programString = "00060";
			bggString = "006";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("APPLIED PHYSICS");
			edit.addFieldOfStudy(fosString, "Applied Physics");
			commitEdit(edit);
					
			fosString = "0570";
			programString = "00066";
			bggString = "081";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("MECHANICAL ENG. & APPLIED MECH.");
			edit.addFieldOfStudy(fosString, "Applied Mechanics");
			commitEdit(edit);
					
			fosString = "0590";
			programString = "00069";
			bggString = "007";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ARCHITECTURE");
			edit.addFieldOfStudy(fosString, "Architecture");
			commitEdit(edit);
					
			fosString = "0670";
			programString = "00075";
			bggString = "009";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ASIAN LANGUAGES & CULTURES");
			edit.addFieldOfStudy(fosString, "Asian Languages and Cultures:  Buddhist Studies");
			commitEdit(edit);
					
			fosString = "0680";
			programString = "00077";
			bggString = "009";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ASIAN LANGUAGES & CULTURES");
			edit.addFieldOfStudy(fosString, "Asian Languages and Cultures:  Chinese");
			commitEdit(edit);
					
			fosString = "0690";
			programString = "00079";
			bggString = "009";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ASIAN LANGUAGES & CULTURES");
			edit.addFieldOfStudy(fosString, "Asian Languages and Cultures:  Japanese");
			commitEdit(edit);
					
			fosString = "0700";
			programString = "00081";
			bggString = "014";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ASTRONOMY");
			edit.addFieldOfStudy(fosString, "Astronomy");
			commitEdit(edit);
					
			fosString = "0710";
			programString = "00084";
			bggString = "015";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ATMOS., OCEAN., & SPACE SCI.");
			edit.addFieldOfStudy(fosString, "Atmospheric and Space Sciences");
			commitEdit(edit);
					
			fosString = "0790";
			programString = "00086";
			bggString = "018";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("BIOMEDICAL ENGINEERING");
			edit.addFieldOfStudy(fosString, "Bioengineering");
			commitEdit(edit);
					
			fosString = "0820";
			programString = "00088";
			bggString = "016";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("BIOLOGICAL CHEMISTRY");
			edit.addFieldOfStudy(fosString, "Biological Chemistry");
			commitEdit(edit);
					
			fosString = "0840";
			programString = "00090";
			bggString = "017";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("BIOLOGY");
			edit.addFieldOfStudy(fosString, "Biological Sciences");
			commitEdit(edit);
					
			fosString = "0870";
			programString = "00092";
			bggString = "017";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("BIOLOGY");
			edit.addFieldOfStudy(fosString, "Biology");
			commitEdit(edit);
					
			fosString = "0880";
			programString = "00094";
			bggString = "018";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("BIOMEDICAL ENGINEERING");
			edit.addFieldOfStudy(fosString, "Biomedical Engineering");
			commitEdit(edit);
					
			fosString = "0920";
			programString = "00097";
			bggString = "041";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("DENTISTRY");
			edit.addFieldOfStudy(fosString, "Biomaterials");
			commitEdit(edit);
					
			fosString = "0930";
			programString = "00100";
			bggString = "019";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("BIOPHYSICS");
			edit.addFieldOfStudy(fosString, "Biophysics");
			commitEdit(edit);
					
			fosString = "0960";
			programString = "00104";
			bggString = "020";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("BIOSTATISTICS");
			edit.addFieldOfStudy(fosString, "Biostatistics");
			commitEdit(edit);
					
			fosString = "0990";
			programString = "00107";
			bggString = "017";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("BIOLOGY");
			edit.addFieldOfStudy(fosString, "Botany");
			commitEdit(edit);
					
			fosString = "1160";
			programString = "00109";
			bggString = "021";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("BUSINESS ADMINISTRATION");
			edit.addFieldOfStudy(fosString, "Business Administration");
			commitEdit(edit);
					
			fosString = "1220";
			programString = "00111";
			bggString = "023";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("CELLULAR & MOLECULAR BIOLOGY");
			edit.addFieldOfStudy(fosString, "Cellular & Molecular Biology");
			commitEdit(edit);
					
			//Medical Science Training Program (MSTP) %%% no FOS
			programString = "00732";
			bggString = "023";
			m_schoolGroups.put(programString, bggString);
			
			fosString = "1250";
			programString = "00114";
			bggString = "024";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("CHEMICAL ENGINEERING");
			edit.addFieldOfStudy(fosString, "Chemical Engineering");
			commitEdit(edit);
					
			fosString = "1260";
			programString = "00117";
			bggString = "022";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("CELL & DEVELOPMENTAL BIOLOGY");
			edit.addFieldOfStudy(fosString, "Cell, Developmental and Neural Biology");
			commitEdit(edit);
					
			fosString = "1290";
			programString = "01611";
			bggString = "022";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("CELL & DEVELOPMENTAL BIOLOGY");
			edit.addFieldOfStudy(fosString, "Cell and Developmental Biology");
			commitEdit(edit);
					
			fosString = "1380";
			programString = "00118";
			bggString = "025";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("CHEMISTRY");
			edit.addFieldOfStudy(fosString, "Chemistry");
			commitEdit(edit);
					
			fosString = "1550";
			programString = "00121";
			bggString = "026";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("CIVIL & ENVIRON. ENGINEERING");
			edit.addFieldOfStudy(fosString, "Civil Engineering");
			commitEdit(edit);
					
			fosString = "1580";
			programString = "00124";
			bggString = "028";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("CLASSICAL STUDIES");
			edit.addFieldOfStudy(fosString, "Classical Studies");
			commitEdit(edit);
					
			fosString = "1600";
			programString = "00126";
			bggString = "027";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("CLASSICAL ART & ARCHAEOLOGY");
			edit.addFieldOfStudy(fosString, "Classical Art & Archaeology");
			commitEdit(edit);
					
			fosString = "1720";
			programString = "00136";
			bggString = "031";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("COMMUNICATION STUDIES (Ph.D.)");
			edit.addFieldOfStudy(fosString, "Communication");
			commitEdit(edit);
					
			fosString = "1840";
			programString = "00140";
			bggString = "033";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("COMPARATIVE LITERATURE");
			edit.addFieldOfStudy(fosString, "Comparative Literature");
			commitEdit(edit);
					
			fosString = "1890";
			programString = "00147";
			bggString = "036";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("COMPUTER SCIENCE & ENGINEERING");
			edit.addFieldOfStudy(fosString, "Computer Science & Engineering");
			commitEdit(edit);
					
			fosString = "1900";
			programString = "00150";
			bggString = "036";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("COMPUTER SCIENCE & ENGINEERING");
			edit.addFieldOfStudy(fosString, "Computer & Communication Sciences");
			commitEdit(edit);
					
			fosString = "2570";
			programString = "01723";
			bggString = "171";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ECOLOGY & EVOLUTIONARY BIOLOGY");
			edit.addFieldOfStudy(fosString, "Ecology and Evolutionary Biology");
			commitEdit(edit);
					
			fosString = "2580";
			programString = "00165";
			bggString = "043";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ECONOMICS");
			edit.addFieldOfStudy(fosString, "Economics");
			commitEdit(edit);
					
			fosString = "2630";
			programString = "00167";
			bggString = "044";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("EDUCATION");
			edit.addFieldOfStudy(fosString, "Education");
			commitEdit(edit);
					
			programString = "00168";
			bggString = "044";
			m_schoolGroups.put(programString, bggString);
			
			programString = "01624";
			bggString = "044";
			m_schoolGroups.put(programString, bggString);
			
			programString = "01631";
			bggString = "044";
			m_schoolGroups.put(programString, bggString);
					
			fosString = "2750";
			programString = "00173";
			bggString = "045";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("EDUCATION & PSYCHOLOGY");
			edit.addFieldOfStudy(fosString, "Education & Psychology");
			commitEdit(edit);
					
			fosString = "2780";
			programString = "00175";
			bggString = "046";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ELEC. ENG. & AOS SCIENCES");
			edit.addFieldOfStudy(fosString, "Electrical Engineering & Atmospheric, Oceanic & Space Sci");
			commitEdit(edit);
					
			fosString = "2800";
			programString = "00177";
			bggString = "047";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ELECTRICAL ENG. & COMP. SCI.");
			edit.addFieldOfStudy(fosString, "Electrical Engineering");
			commitEdit(edit);
					
			fosString = "2810";
			programString = "00181";
			bggString = "048";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ELECTRICAL ENG.: SYSTEMS");
			edit.addFieldOfStudy(fosString, "Electrical Engineering:  Systems");
			commitEdit(edit);
					
			fosString = "3090";
			programString = "00189";
			bggString = "049";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ENGLISH & EDUCATION");
			edit.addFieldOfStudy(fosString, "English & Education");
			commitEdit(edit);
					
			fosString = "3140";
			programString = "00190";
			bggString = "051";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ENGLISH LANGUAGE & LITERATURE");
			edit.addFieldOfStudy(fosString, "English Language & Literature");
			commitEdit(edit);
					
			fosString = "3150";
			programString = "00193";
			bggString = "050";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ENGLISH & WOMEN'S STUDIES");
			edit.addFieldOfStudy(fosString, "English and Women's Studies");
			commitEdit(edit);
					
			fosString = "3170";
			programString = "00194";
			bggString = "026";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("CIVIL & ENVIRON. ENGINEERING");
			edit.addFieldOfStudy(fosString, "Environmental Engineering");
			commitEdit(edit);
					
			fosString = "3200";
			programString = "00196";
			bggString = "052";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ENVIRON. & INDUS. HEALTH");
			edit.addFieldOfStudy(fosString, "Environmental Health Sciences");
			commitEdit(edit);
					
			fosString = "3260";
			programString = "00198";
			bggString = "053";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("EPIDEMIOLOGY");
			edit.addFieldOfStudy(fosString, "Epidemiologic Science");
			commitEdit(edit);
					
			fosString = "3310";
			programString = "00200";
			bggString = "009";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ASIAN LANGUAGES & CULTURES");
			edit.addFieldOfStudy(fosString, "Far Eastern Languages & Literatures:  Buddhist Studies");
			commitEdit(edit);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("");
			edit.addFieldOfStudy(fosString, "");
			commitEdit(edit);
					
			fosString = "3330";
			programString = "00202";
			bggString = "009";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ASIAN LANGUAGES & CULTURES");
			edit.addFieldOfStudy(fosString, "Far Eastern Languages & Literatures:  Chinese");
			commitEdit(edit);
					
			fosString = "3350";
			programString = "00204";
			bggString = "009";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ASIAN LANGUAGES & CULTURES");
			edit.addFieldOfStudy(fosString, "Far Eastern Languages & Literatures:  Japanese");
			commitEdit(edit);
					
			fosString = "3830";
			programString = "00216";
			bggString = "060";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("GEOLOGICAL SCIENCES");
			edit.addFieldOfStudy(fosString, "Geology");
			commitEdit(edit);
					
			fosString = "3940";
			programString = "00220";
			bggString = "061";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("GERMANIC LANG. & LITERATURES");
			edit.addFieldOfStudy(fosString, "Germanic Languages & Literatures");
			commitEdit(edit);
					
			fosString = "4030";
			programString = "00222";
			bggString = "063";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("HEALTH BEHAVIOR & HEALTH EDUC.");
			edit.addFieldOfStudy(fosString, "Health Behavior & Health Education");
			commitEdit(edit);
					
			fosString = "4050";
			programString = "00224";
			bggString = "066";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("HISTORY");
			edit.addFieldOfStudy(fosString, "History");
			commitEdit(edit);
					
			fosString = "4110";
			programString = "00228";
			bggString = "067";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("HISTORY OF ART");
			edit.addFieldOfStudy(fosString, "History of Art");
			commitEdit(edit);
					
			fosString = "4190";
			programString = "00230";
			bggString = "065";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("HEALTH SERVICES ORG. & POLICY");
			edit.addFieldOfStudy(fosString, "Health Services Organization & Policy");
			commitEdit(edit);
					
			fosString = "4240";
			programString = "01571";
			bggString = "066";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("HISTORY");
			edit.addFieldOfStudy(fosString, "History & Women's Studies");
			commitEdit(edit);
					
			fosString = "4340";
			programString = "00236";
			bggString = "069";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("HUMAN GENETICS");
			edit.addFieldOfStudy(fosString, "Human Genetics");
			commitEdit(edit);
					
			fosString = "4390";
			programString = "00239";
			bggString = "072";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("INDUS. & OPERATIONS ENG.");
			edit.addFieldOfStudy(fosString, "Industrial & Operations Engineering");
			commitEdit(edit);
					
			fosString = "4420";
			programString = "00245";
			bggString = "052";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ENVIRON. & INDUS. HEALTH");
			edit.addFieldOfStudy(fosString, "Industrial Health");
			commitEdit(edit);
					
			fosString = "4440";
			programString = "00247";
			bggString = "074";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("");
			edit.addFieldOfStudy(fosString, "");
			commitEdit(edit);
					
			fosString = "4470";
			programString = "01100";
			bggString = "074";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("INFORMATION");
			edit.addFieldOfStudy(fosString, "Information and Library Studies");
			commitEdit(edit);
					
			fosString = "4490";
			programString = "01563";
			bggString = "085";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("MICROBIOLOGY & IMMUNOLOGY");
			edit.addFieldOfStudy(fosString, "Immunology");
			commitEdit(edit);
					
			fosString = "4510";
			programString = "00251";
			bggString = "105";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("POPULATION PLN. & INTL. HEALTH");
			edit.addFieldOfStudy(fosString, "International Health");
			commitEdit(edit);
					
			fosString = "4670";
			programString = "00255";
			bggString = "076";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("KINESIOLOGY");
			edit.addFieldOfStudy(fosString, "Kinesiology");
			commitEdit(edit);
					
			fosString = "4680";
			programString = "00258";
			bggString = "090";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("NATURAL RESOURCES & ENVIRON.");
			edit.addFieldOfStudy(fosString, "Landscape Architecture");
			commitEdit(edit);
					
			programString = "00260";
			bggString = "090";
			m_schoolGroups.put(programString, bggString);
					
			fosString = "4740";
			programString = "01637";
			bggString = "150";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("BIOINFORMATICS");
			edit.addFieldOfStudy(fosString, "Bioinformatics");
			commitEdit(edit);
					
			fosString = "4870";
			programString = "01596";
			bggString = "077";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("LINGUISTICS");
			edit.addFieldOfStudy(fosString, "Linguistics & Germanic Languages & Literatures");
			commitEdit(edit);
					
			fosString = "4880";
			programString = "01597";
			bggString = "077";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("LINGUISTICS");
			edit.addFieldOfStudy(fosString, "Linguistics & Romanic Languages & Literatures");
			commitEdit(edit);
					
			fosString = "4890";
			programString = "01598";
			bggString = "077";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("LINGUISTICS");
			edit.addFieldOfStudy(fosString, "Linguistics & Slavic Languages & Literatures");
			commitEdit(edit);
					
			fosString = "4900";
			programString = "00271";
			bggString = "077";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("LINGUISTICS");
			edit.addFieldOfStudy(fosString, "Linguistics");
			commitEdit(edit);
					
			fosString = "4940";
			programString = "00273";
			bggString = "078";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("MACROMOLECULAR SCI. & ENG.");
			edit.addFieldOfStudy(fosString, "Macromolecular Science & Engineering");
			commitEdit(edit);
					
			fosString = "5000";
			programString = "00275";
			bggString = "032";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("COMMUNICATION STUDIES (MA)");
			edit.addFieldOfStudy(fosString, "Mass Communications");
			commitEdit(edit);
					
			fosString = "5040";
			programString = "00276";
			bggString = "079";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
			
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("MATERIALS SCI. & ENGINEERING");
			edit.addFieldOfStudy(fosString, "Materials Science and Engineering");
			commitEdit(edit);
					
			//Molecular and Integrative Physiology
			fosString = "5070";
			bggString = "103";
			m_schoolGroups.put(fosString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("");
			edit.addFieldOfStudy(fosString, "Molecular and Integrative Physiology");
			commitEdit(edit);
					
			fosString = "5080";
			programString = "00278";
			bggString = "080";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("MATHEMATICS");
			edit.addFieldOfStudy(fosString, "Mathematics");
			commitEdit(edit);
					
			fosString = "5130";
			programString = "00281";
			bggString = "081";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("MECHANICAL ENG. & APPLIED MECH.");
			edit.addFieldOfStudy(fosString, "Mechanical Engineering");
			commitEdit(edit);
					
			fosString = "5140";
			programString = "00285";
			bggString = "065";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("HEALTH SERVICES ORG. & POLICY");
			edit.addFieldOfStudy(fosString, "Medical Care Organization");
			commitEdit(edit);
					
			fosString = "5200";
			programString = "00290";
			bggString = "084";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("MEDICINAL CHEMISTRY");
			edit.addFieldOfStudy(fosString, "Medicinal Chemistry");
			commitEdit(edit);
					
			fosString = "5230";
			programString = "00293";
			bggString = "079";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("MATERIALS SCI. & ENGINEERING");
			edit.addFieldOfStudy(fosString, "Metallurgical Engineering");
			commitEdit(edit);
					
			fosString = "5270";
			programString = "01726";
			bggString = "175";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("MOLECULAR, CELL & DEV BIOLOGY");
			edit.addFieldOfStudy(fosString, "Molecular, Cellular, and Developmental Biology");
			commitEdit(edit);
					
			fosString = "5330";
			programString = "00297";
			bggString = "085";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("MICROBIOLOGY & IMMUNOLOGY");
			edit.addFieldOfStudy(fosString, "Microbiology");
			commitEdit(edit);
					
			fosString = "5340";
			programString = "00299";
			bggString = "085";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("MICROBIOLOGY & IMMUNOLOGY");
			edit.addFieldOfStudy(fosString, "Microbiology & Immunology");
			commitEdit(edit);
					
			fosString = "5350";
			programString = "00300";
			bggString = "060";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("GEOLOGICAL SCIENCES");
			edit.addFieldOfStudy(fosString, "Mineralogy");
			commitEdit(edit);
					
			fosString = "5400";
			programString = "01067";
			bggString = "089";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("MUSIC: PERFORMANCE");
			edit.addFieldOfStudy(fosString, "Music: Conducting");
			commitEdit(edit);
					
			programString = "01068";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			fosString = "5430";
			programString = "00309";
			bggString = "088";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("MUSIC: ACADEMIC");
			edit.addFieldOfStudy(fosString, "Music: Music Education");
			commitEdit(edit);
					
			fosString = "5440";
			programString = "01065";
			bggString = "088";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
			
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("MUSIC: ACADEMIC");
			edit.addFieldOfStudy(fosString, "Music: Musicology");
			commitEdit(edit);
			
			programString = "01066";
			bggString = "088";
			m_schoolGroups.put(programString, bggString);
					
			fosString = "5460";
			programString = "00312";
			bggString = "088";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("MUSIC: ACADEMIC");
			edit.addFieldOfStudy(fosString, "Music: Theory");
			commitEdit(edit);
					
			fosString = "5470";
			programString = "00313";
			bggString = "089";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("MUSIC: ACADEMIC");
			edit.addFieldOfStudy(fosString, "Music: Composition");
			commitEdit(edit);
					
			fosString = "5640";
			programString = "01070";
			bggString = "089";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("MUSIC: PERFORMANCE");
			edit.addFieldOfStudy(fosString, "Music: Performance");
			commitEdit(edit);
					
			programString = "01071";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01072";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01073";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01074";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01075";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01076";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01077";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01078";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01079";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
			
			programString = "01080";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01081";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01082";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01083";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01084";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01085";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01086";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01087";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01088";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01089";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01090";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01091";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01092";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			programString = "01595";
			bggString = "089";
			m_schoolGroups.put(programString, bggString);
					
			fosString = "5670";
			programString = "00317";
			bggString = "088";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("MUSIC: ACADEMIC");
			edit.addFieldOfStudy(fosString, "Composition and Music Theory");
			commitEdit(edit);
					
			fosString = "5730";
			programString = "00318";
			bggString = "090";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("NATURAL RESOURCES & ENVIRON.");
			edit.addFieldOfStudy(fosString, "Natural Resources");
			commitEdit(edit);
					
			fosString = "5740";
			programString = "00320";
			bggString = "090";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("NATURAL RESOURCES & ENVIRON.");
			edit.addFieldOfStudy(fosString, "Natural Resources and Environment");
			commitEdit(edit);
					
			fosString = "5790";
			programString = "00323";
			bggString = "090";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("NATURAL RESOURCES & ENVIRON.");
			edit.addFieldOfStudy(fosString, "Natural Resources Economics");
			commitEdit(edit);
					
			fosString = "5810";
			programString = "00324";
			bggString = "091";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("NAVAL ARCH. & MARINE ENG.");
			edit.addFieldOfStudy(fosString, "Naval Architecture & Marine Engineering");
			commitEdit(edit);
					
			fosString = "5820";
			programString = "00329";
			bggString = "092";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("NEAR EASTERN STUDIES");
			edit.addFieldOfStudy(fosString, "Near Eastern Studies");
			commitEdit(edit);
					
			fosString = "5830";
			programString = "00331";
			bggString = "092";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("NEAR EASTERN STUDIES");
			edit.addFieldOfStudy(fosString, "Near Eastern Studies: Languages & Literatures");
			commitEdit(edit);
					
			fosString = "5950";
			programString = "00336";
			bggString = "093";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("NEUROSCIENCE");
			edit.addFieldOfStudy(fosString, "Neuroscience");
			commitEdit(edit);
					
			fosString = "5980";
			programString = "00338";
			bggString = "094";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("NUCLEAR ENG. & RADIOL. SCI.");
			edit.addFieldOfStudy(fosString, "Nuclear Engineering");
			commitEdit(edit);
					
			fosString = "6000";
			programString = "01565";
			bggString = "094";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("NUCLEAR ENG. & RADIOL. SCI.");
			edit.addFieldOfStudy(fosString, "Nuclear Engineering & Radiological Sciences");
			commitEdit(edit);
					
			fosString = "6100";
			programString = "00344";
			bggString = "094";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("NUCLEAR ENG. & RADIOL. SCI.");
			edit.addFieldOfStudy(fosString, "Nuclear Science");
			commitEdit(edit);
					
			fosString = "6130";
			programString = "00347";
			bggString = "095";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("NURSING");
			edit.addFieldOfStudy(fosString, "Nursing");
			commitEdit(edit);
					
			fosString = "6160";
			programString = "00351";
			bggString = "015";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ATMOS., OCEAN., & SPACE SCI.");
			edit.addFieldOfStudy(fosString, "Oceanic Science (no new students)");
			commitEdit(edit);
					
			fosString = "6170";
			programString = "00353";
			bggString = "060";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("GEOLOGICAL SCIENCES");
			edit.addFieldOfStudy(fosString, "Oceanography:  Marine Geology and Geochemistry");
			commitEdit(edit);
					
			fosString = "6180";
			programString = "00355";
			bggString = "015";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ATMOS., OCEAN., & SPACE SCI.");
			edit.addFieldOfStudy(fosString, "Oceanography: Physical");
			commitEdit(edit);
					
			fosString = "6300";
			programString = "00358";
			bggString = "041";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("DENTISTRY");
			edit.addFieldOfStudy(fosString, "Oral Biology");
			commitEdit(edit);
					
			fosString = "6340";
			programString = "00360";
			bggString = "041";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("DENTISTRY");
			edit.addFieldOfStudy(fosString, "Oral Health Sciences");
			commitEdit(edit);
					
			fosString = "6680";
			programString = "00367";
			bggString = "097";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("PATHOLOGY");
			edit.addFieldOfStudy(fosString, "Pathology");
			commitEdit(edit);
					
			fosString = "6810";
			programString = "00372";
			bggString = "099";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("PHARMACY");
			edit.addFieldOfStudy(fosString, "Pharmaceutics");
			commitEdit(edit);
					
			fosString = "6840";
			programString = "00374";
			bggString = "099";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("PHARMACY");
			edit.addFieldOfStudy(fosString, "Pharmaceutical Chemistry");
			commitEdit(edit);
					
			//Pharmaceutical Sciences
			fosString = "6860";
			bggString = "099";
			m_schoolGroups.put(fosString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("");
			edit.addFieldOfStudy(fosString, "Pharmaceutical Sciences");
			commitEdit(edit);
					
			fosString = "6890";
			programString = "00376";
			bggString = "099";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("PHARMACY");
			edit.addFieldOfStudy(fosString, "Pharmacognosy");
			commitEdit(edit);
					
			fosString = "6950";
			programString = "00378";
			bggString = "098";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("PHARMACOLOGY");
			edit.addFieldOfStudy(fosString, "Pharmacology");
			commitEdit(edit);
					
			fosString = "7010";
			programString = "00380";
			bggString = "099";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("PHARMACY");
			edit.addFieldOfStudy(fosString, "Pharmacy");
			commitEdit(edit);
					
			fosString = "7040";
			programString = "00384";
			bggString = "076";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("KINESIOLOGY");
			edit.addFieldOfStudy(fosString, "Physical Education: Kinesiology");
			commitEdit(edit);
					
			fosString = "7060";
			programString = "00387";
			bggString = "100";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("PHILOSOPHY");
			edit.addFieldOfStudy(fosString, "Philosophy");
			commitEdit(edit);
					
			fosString = "7180";
			programString = "00389";
			bggString = "101";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("PHYSICS");
			edit.addFieldOfStudy(fosString, "Physics");
			commitEdit(edit);
					
			fosString = "7350";
			programString = "00394";
			bggString = "103";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("PHYSIOLOGY");
			edit.addFieldOfStudy(fosString, "Physiology");
			commitEdit(edit);
					
			fosString = "7460";
			programString = "00397";
			bggString = "104";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("POLITICAL SCIENCE");
			edit.addFieldOfStudy(fosString, "Political Science");
			commitEdit(edit);
					
			fosString = "7490";
			programString = "00403";
			bggString = "105";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("POPULATION PLN. & INTL. HEALTH");
			edit.addFieldOfStudy(fosString, "Population Planning & International Health");
			commitEdit(edit);
					
			fosString = "7500";
			programString = "00404";
			bggString = "105";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("POPULATION PLN. & INTL. HEALTH");
			edit.addFieldOfStudy(fosString, "Population Planning");
			commitEdit(edit);
					
			fosString = "7720";
			programString = "01677";
			bggString = "151";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("PUBLIC POLICY JOINT DOCTORAL PGMS");
			edit.addFieldOfStudy(fosString, "Public Policy & Economics");
			commitEdit(edit);
					
			fosString = "7730";
			programString = "01679";
			bggString = "151";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("PUBLIC POLICY JOINT DOCTORAL PGMS");
			edit.addFieldOfStudy(fosString, "Public Policy & Political Science");
			commitEdit(edit);
					
			fosString = "7740";
			programString = "00409";
			bggString = "106";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("PSYCHOLOGY");
			edit.addFieldOfStudy(fosString, "Psychology");
			commitEdit(edit);
					
			fosString = "7750";
			programString = "01681";
			bggString = "151";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("PUBLIC POLICY JOINT DOCTORAL PGMS");
			edit.addFieldOfStudy(fosString, "Public Policy & Sociology");
			commitEdit(edit);
					
			fosString = "7770";
			programString = "00412";
			bggString = "107";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("PSYCHOLOGY & WOMEN'S STUDIES");
			edit.addFieldOfStudy(fosString, "Psychology and Women's Studies");
			commitEdit(edit);
					
			fosString = "7880";
			programString = "00415";
			bggString = "109";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("PUBLIC POLICY STUDIES");
			edit.addFieldOfStudy(fosString, "Public Policy Studies");
			commitEdit(edit);
					
			fosString = "8030";
			programString = "00426";
			bggString = "111";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ROMANCE LANGUAGES & LIT.");
			edit.addFieldOfStudy(fosString, "Romance Languages & Literatures:  French");
			commitEdit(edit);
					
			fosString = "8090";
			programString = "00456";
			bggString = "111";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ROMANCE LANGUAGES & LIT.");
			edit.addFieldOfStudy(fosString, "Romance Languages & Literatures:  Italian");
			commitEdit(edit);
					
			fosString = "8170";
			programString = "00458";
			bggString = "111";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ROMANCE LANGUAGES & LIT.");
			edit.addFieldOfStudy(fosString, "Romance Languages & Literatures:  Romance Linguistics");
			commitEdit(edit);
					
			fosString = "8200";
			programString = "00460";
			bggString = "111";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ROMANCE LANGUAGES & LIT.");
			edit.addFieldOfStudy(fosString, "Romance Languages & Literatures:  Spanish");
			commitEdit(edit);
					
			fosString = "8350";
			programString = "00463";
			bggString = "026";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("CIVIL & ENVIRON. ENGINEERING");
			edit.addFieldOfStudy(fosString, "Sanitary Engineering");
			commitEdit(edit);
					
			fosString = "8480";
			programString = "00467";
			bggString = "114";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("SLAVIC LANGUAGES & LITERATURES");
			edit.addFieldOfStudy(fosString, "Slavic Languages & Literatures");
			commitEdit(edit);
					
			fosString = "8890";
			programString = "00475";
			bggString = "116";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("SOCIAL WORK & SOCIAL SCIENCE");
			edit.addFieldOfStudy(fosString, "Social Work & Social Science");
			commitEdit(edit);
					
			fosString = "8940";
			programString = "00476";
			bggString = "117";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("SOCIOLOGY");
			edit.addFieldOfStudy(fosString, "Sociology");
			commitEdit(edit);
					
			fosString = "8950";
			programString = "00478";
			bggString = "118";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("SPACE & PLANETARY PHYSICS");
			edit.addFieldOfStudy(fosString, "Space and Planetary Physics");
			commitEdit(edit);
					
			fosString = "9090";
			programString = "00483";
			bggString = "119";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("STATISTICS");
			edit.addFieldOfStudy(fosString, "Statistics");
			commitEdit(edit);
					
			fosString = "9280";
			programString = "00490";
			bggString = "120";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("THEATRE");
			edit.addFieldOfStudy(fosString, "Theatre");
			commitEdit(edit);
					
			fosString = "9360";
			programString = "00498";
			bggString = "052";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("ENVIRON. & INDUS. HEALTH");
			edit.addFieldOfStudy(fosString, "Toxicology");
			commitEdit(edit);
					
			fosString = "9380";
			programString = "00501";
			bggString = "123";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("URBAN, TECH., & ENVIRON. PLN.");
			edit.addFieldOfStudy(fosString, "Urban, Technological, & Environmental Planning");
			commitEdit(edit);
					
			fosString = "9990";
			programString = "00512";
			bggString = "071";
			m_schoolGroups.put(fosString, bggString);
			m_schoolGroups.put(programString, bggString);
					
			// add a group
			edit = addBlockGrantGroup(m_schoolSite);
			edit.setCode(bggString);
			edit.setDescription("INDEPENDENT IDP");
			edit.addFieldOfStudy(fosString, "Independent Interdepartmental Degree Program");
			commitEdit(edit);
		}
		catch(Exception e)
		{
			if(m_logger.isWarnEnabled())
			{
				m_logger.warn("BaseDissertationService.initializeCodes() Exception " + e);
			}
		}
		
		return m_schoolGroups;
		
	}//initCodes

	/**
	* Returns to uninitialized state.
	*/
	public void destroy()
	{
		m_groupCache.clear();
		m_dissertationCache.clear();
		m_dissertationCache = null;
		m_stepCache.destroy();
		m_stepCache = null;
		m_pathCache.destroy();
		m_pathCache = null;
		m_statusCache.destroy();
		m_statusCache = null;
		m_infoCache.destroy();
		m_infoCache = null;

		m_groupStorage.close();
		m_groupStorage = null;
		m_dissertationStorage.close();
		m_dissertationStorage = null;
		m_stepStorage.close();
		m_stepStorage = null;
		m_pathStorage.close();
		m_pathStorage = null;
		m_statusStorage.close();
		m_statusStorage = null;
		m_infoStorage.close();
		m_infoStorage = null;

		//m_logger.debug(this + ".destroy()");
	}

	/*******************************************************************************
	* DissertationServiceimplementation
	*******************************************************************************/
	
	/**
	* Get properties of a Rackham data extract file.
	* @param content - The content for which properties are requested.
	* @return Hastable of property name(s) and value(s).
	*/
	public Hashtable getDataFileProperties(byte[] content)
	{
		Hashtable retVal = new Hashtable();
		try
		{
			//get number of lines of content property
			int lns = 0;
			char[] chars = ((String)new String(content)).toCharArray();
			for(int i = 0; i < chars.length; i++)
			{
				if(chars[i]=='\n')
				{
					lns = lns + 1;
				}
			}
			Integer lines = new Integer(lns);
			retVal.put(DissertationService.DATAFILE_LINES, lines);
		}
		catch (Exception e)
		{
			m_logger.warn(this + " .getDataFileProperties: " + e.getMessage());
		}
		return retVal;
		
	} //getDataFileProperties
	
	/**
	* See if Rackham extract data loading is in progress.
	* @return boolean - true if in progress, false otherwise.
	*/
	public boolean isLoading()
	{
		return m_loading;
	}
	
	/**
	* Load the Rackham data extract files.
	* @param oard - The content of the OARD data file.
	* @param mp - The content of the MPathways data file.
	* @return Vector of String errors or if no errors null.
	*/
	public Vector loadData(byte[] oard, byte[] mp)
	{
		//isLoading()
		m_loading = true;
		
		Vector rv = new Vector();
		Hashtable ids = new Hashtable();
		String msg = "";
		List OARDRecords = new Vector();
		List MPRecords = new Vector();
		List OARDerrors = new Vector();
		List MPerrors = new Vector();

		try
		{
			//parse and check for data validation errors
			if(oard.length > 1) 
			{
				OARDerrors = (Vector) validateOARDData(oard);
			}
			if(mp.length > 1)
			{
				MPerrors = (Vector) validateMPData(mp);
			}
			
			//if data was validated, load it, otherwise return validation errors
			if((OARDerrors==null || OARDerrors.size() == 0) && (MPerrors==null || MPerrors.size() == 0))
			{
				//no errors reported, so create records to use
				if(oard.length > 1)
				{
					OARDRecords = (Vector) createOARDRecords(oard, ids);
				}
				if(mp.length > 1)
				{
					MPRecords = (Vector) createMPRecords(mp, ids);
				}
				
				//if we have no Chef ids at this point, raise an error
				if(!ids.elements().hasMoreElements())
				{
					m_logger.warn("DISSERTATION : SERVICE: UMIAC QUERY FOR CHEF IDS : NO UNIQNAMES FOUND");
					msg = "Unable to get Chef ids for these U-M ids.";
					rv.add(msg);
				}
				
				//based on DissertationDataListenerService.queryDatabase
				List errorsLoading = (Vector)queryLists(OARDRecords, MPRecords, ids);
				
				if((errorsLoading != null) && (errorsLoading.size() > 0))
				{
					//return loading errors to the tool
					msg = "During loading of data:";
					rv.add(msg);
					for (ListIterator i = errorsLoading.listIterator(); i.hasNext(); )
					{
						msg = (String) i.next();
						rv.add(msg);
					}
				}
			}
			else
			{	
				//return validation errors to the tool
				msg = "During validation of data:";
				rv.add(msg);
				for (ListIterator i = OARDerrors.listIterator(); i.hasNext(); )
				{
					msg = (String) i.next();
					rv.add(msg);
				}
				for (ListIterator i = MPerrors.listIterator(); i.hasNext(); )
				{
					msg = (String) i.next();
					rv.add(msg);
				}
			}
		}
		catch(Exception e)
		{
			msg = this + ".loadData exception: " + e.getMessage();
			rv.add(msg);
		}
		//isLoading()
		m_loading = false;
		return rv;
		
	} //loadData
	
	/**
	* Create a Time object from the raw time string.
	* Based on DissertationDataListenerService.parseOracleTimeString()
	* @param timeString The raw oracle time string.
	* @return The CHEF Time object.
	*/
	protected Time parseTimeString(String timeString)
	{
		Time retVal = null;
		//m_logger.debug("DISSERTATION : SERVICE : ENTERING PARSE TIME STRING : " + timeString);
		if(timeString != null && !timeString.equals(""))
		{
			int year = 0;
			int month = 0;
			int day = 0;
			
			int index = 0;
			String tempString = null;
			String dayString = null;
			String[] parts = timeString.split("/");
			for(int x = 0; x < parts.length; x++)
				//m_logger.debug("DISSERTATION : SERVICE : PARSE TIME STRING : SLASH PART : " + parts[x]);
			try
			{
				year = Integer.parseInt(parts[2]);
				//m_logger.debug("DISSERTATION : SERVICE: PARSE TIME STRING : YEAR : " + year);
			}
			catch(NumberFormatException nfe)
			{
				m_logger.warn("DISSERTATION : SERVICE: PARSE TIME STRING : YEAR : NumberFormatException  " + timeString);
			}
			
			try
			{
				month = Integer.parseInt(parts[0]);
				//m_logger.debug("DISSERTATION : SERVICE : PARSE TIME STRING : MONTH : " + month);
			}
			catch(NumberFormatException nfe)
			{
				m_logger.warn("DISSERTATION : SERVICE: PARSE TIME STRING : MONTH : NumberFormatException  " + timeString);
			}
			
			try
			{
				day = Integer.parseInt(parts[1]);
				//m_logger.debug("DISSERTATION : SERVICE : PARSE TIME STRING : STRING TO CONVERT TO INT : " + timeString);
				//m_logger.debug("DISSERTATION : SERVICE : PARSE TIME STRING : DAY : " + day);
			}
			catch(NumberFormatException nfe)
			{
				m_logger.warn("DISSERTATION : SERVICE: PARSE TIME STRING : DAY : NumberFormatException  " + timeString);
			}
			TimeBreakdown tb = TimeService.newTimeBreakdown(year, month, day, 0, 0, 0, 0);
			retVal = TimeService.newTimeLocal(tb);
		}
		return retVal;

	}//parseTimeString
	
	/**
	*
	* Use Rackham data to initialize and update CandidateInfo.
	* Based on DissertationDataListenerService.queryDatabase().
	* @param OARDRecs - Vector of OARDRecords
	* @param  MPRecs - Vector of MPRecords
	* @param  ids - Hashtable of umid keys and chefid elements
	* @return rv - a Vector of error messages
	*/
	private Vector queryLists(List OARDRecords, List MPRecords, Hashtable ids)
	{
		Vector rv = new Vector();
		boolean reqMet = false;
		Vector data = new Vector();
		List OARDrecs = new Vector();
		List MPrecs = new Vector();
		String oneEmplid = null;
		String chefId = null;
		String program = null;
		String dataString = null;
		String msg = null;
		CandidateInfoEdit infoEdit = null;
		boolean commitEdit = false;
		boolean tryMPrecs = false;
		boolean existsOARDrecs = false;
		boolean existsMPrecs = false;
		
		Enumeration emplids = ids.keys();

		if(emplids != null)
		{
			while(emplids.hasMoreElements())
			{
				try
				{
					commitEdit = false;
					
					oneEmplid = (String)emplids.nextElement();
					
					OARDrecs.clear();
					MPrecs.clear();
					
					//m_logger.debug("DISSERTATION : SERVICE : PROCESSING EMPLID : " + oneEmplid);
					
					//get the MP records for this oneEmplid
					if((MPRecords!=null) && (MPRecords.size() > 0))
					{
						for(ListIterator i = MPRecords.listIterator(); i.hasNext(); )
						{
							MPRecord rec = (MPRecord) i.next();
							if(rec.m_umid.equals(oneEmplid)) MPrecs.add(rec);
						}
					}
					
					if((MPrecs != null) && (MPrecs.size() > 0))
					{
						existsMPrecs = true;
					}
					else
					{
						existsMPrecs = false;
					}
					
					//get the OARD records for this oneEmplid
					if((OARDRecords!=null) && (OARDRecords.size() > 0))
					{
						for(ListIterator i = OARDRecords.listIterator(); i.hasNext(); )
						{
							OARDRecord rec = (OARDRecord) i.next();
							if(rec.m_umid.equals(oneEmplid)) OARDrecs.add(rec);
						}
					}
					if((OARDrecs != null) && (OARDrecs.size() > 0)) 
					{
						existsOARDrecs = true;
					}
					else
					{
						existsOARDrecs = false;
					}

					infoEdit = getCandidateInfoEditForEmplid(oneEmplid);
					if(infoEdit == null)
					{
						//m_logger.debug("DISSERTATION : SERVICE : NO CANDIDATE INFO EDIT EXISTS FOR EMPLID : " + oneEmplid + " : WILL CREATE ONE");

						// NO CANDIDATE INFO EXISTS FOR THIS CANDIDATE - CREATE ONE
						infoEdit = addCandidateInfoFromListener("datalistener");
						infoEdit.setEmplid(oneEmplid);
						commitEdit = true;
					}
					else
					{
						//m_logger.debug("DISSERTATION : SERVICE : GOT CANDIDATE INFO EDIT FOR EMPLID : " + oneEmplid);
					}

					// CHECK FOR CHEF ID
					try
					{
						chefId = infoEdit.getChefId();
						//m_logger.debug("DISSERTATION : SERVICE : CHEF ID IN CANDIDATE INFO EDIT : " + chefId);
						if(chefId.equals(""))
						{
							//m_logger.debug("DISSERTATION : SERVICE : CHEF ID IS EMPTY - WILL QUERY ID HASHTABLE FOR CHEF ID");
							chefId = (String)ids.get(oneEmplid);
							///m_logger.debug("DISSERTATION : SERVICE : GOT CHEF ID FROM ID HASHTABLE : " + chefId);
							if(chefId != null)
									infoEdit.setChefId(chefId.toLowerCase());
							commitEdit = true;
						}
					}
					catch(Exception e)
					{
						msg = "// CHECK FOR CHEF ID " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : CHECK FOR CHEF ID: EXCEPTION : " + e);
					}
			
					// CHECK FOR PROGRAM
					try
					{
						program = infoEdit.getProgram();
						//m_logger.debug("DISSERTATION : SERVICE : PROGRAM IN CANDIDATE INFO EDIT : " + program);
						if(program == null || program.equals(""))
						{
							if(existsOARDrecs && existsMPrecs)
							{
								//CHECK OARD AND IF PROGRAM NULL THEN CHECK MP
								
								// FIRST TRY GETTING THE FOS FROM OARDEXT
								for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
								{
									OARDRecord rec = (OARDRecord) i.next();
									if(rec.m_fos != null) program = rec.m_fos;
								}
					 
								if(program != null)
								{
									//m_logger.debug("DISSERTATION : SERVICE : GOT FOS FOR EMPLID " + program);
									//m_logger.debug("DISSERTATION : SERVICE : TRANSLATED INTO GROUP ROLL-UP : " + getProgram(program));
									infoEdit.setProgram(getProgram(program));
									//infoEdit.setParentSiteId("diss" +  program);
									infoEdit.setParentSiteId("diss" +  getProgram(program));
								}
								else
								{
									tryMPrecs = true;
								}
								
								if(tryMPrecs)
								{
									// THEN TRY GETTING THE FOS PART OF THE ACADEMIC PLAN IN MPEXT
									for(ListIterator i = MPrecs.listIterator(); i.hasNext(); )
									{
										MPRecord rec = (MPRecord) i.next();
										if(rec.m_academic_plan != null) program = rec.m_academic_plan.substring(0,4);
									}
									if(program != null)
									{
										//m_logger.debug("DISSERTATION : SERVICE : GOT ACAD_PROG FOR EMPLID " + program);
										//m_logger.debug("DISSERTATION : SERVICE : TRANSLATED INTO GROUP ROLL-UP : " + getProgram(program));
										infoEdit.setProgram(getProgram(program));
										//infoEdit.setParentSiteId("diss" +  program);
										infoEdit.setParentSiteId("diss" +  getProgram(program));
									}
								}
							}
							else if(existsOARDrecs)
							{
								//CHECK OARD FOR THE FOS
								for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
								{
									OARDRecord rec = (OARDRecord) i.next();
									if(rec.m_fos != null) program = rec.m_fos;
								}
					 
								if(program != null)
								{
									//m_logger.debug("DISSERTATION : SERVICE : GOT FOS FOR EMPLID " + program);
									//m_logger.debug("DISSERTATION : SERVICE : TRANSLATED INTO GROUP ROLL-UP : " + getProgram(program));
									infoEdit.setProgram(getProgram(program));
									//infoEdit.setParentSiteId("diss" +  program);
									infoEdit.setParentSiteId("diss" +  getProgram(program));
								}
							}
							else if (existsMPrecs)
							{
								// THEN TRY GETTING THE FOS PART OF THE ACADEMIC PLAN
								for(ListIterator i = MPrecs.listIterator(); i.hasNext(); )
								{
									MPRecord rec = (MPRecord) i.next();
									if(rec.m_academic_plan != null) program = rec.m_academic_plan.substring(0,4);
								}
								if(program != null)
								{
									//m_logger.debug("DISSERTATION : SERVICE : GOT ACAD_PROG FOR EMPLID " + program);
									//m_logger.debug("DISSERTATION : SERVICE : TRANSLATED INTO GROUP ROLL-UP : " + getProgram(program));
									infoEdit.setProgram(getProgram(program));
									//infoEdit.setParentSiteId("diss" +  program);
									infoEdit.setParentSiteId("diss" +  getProgram(program));
								}
							}
							else
							{
								//WE SHOULDN"T GET HERE
								m_logger.warn("DISSERTATION : SERVICE : no OARD or MP records for " + oneEmplid);
							}
							commitEdit = true;
						}
					}
					catch(Exception e)
					{
						msg = "// CHECK FOR PROGRAM " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : LISTENER : SELECT FOS/ACAD_PROG FROM RECORDS WHERE UMID = ? : EXCEPTION : " + e);
					}

					// GET STATUS DATA
					try
					{
						// DEGREETERM_TRANS
						
						dataString = null;
	
						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_degreeterm_trans != null) dataString = rec.m_degreeterm_trans;
							}
							//m_logger.debug("DISSERTATION : SERVICE  : DEGTERM_TRANS : " + dataString);
							infoEdit.setDegreeTermTrans(dataString);
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // DEGREETERM_TRANS " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}
					
					try
					{
						// ORAL_EXAM_DATE
						
						dataString = null;

						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_oral_exam_date != null) dataString = rec.m_oral_exam_date;
							}
							//m_logger.debug("DISSERTATION : SERVICE : ORAL_EXAM_DATE : " + dataString);
							infoEdit.setTimeOralExam(parseTimeString(dataString));
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // ORAL_EXAM_DATE_TIME " + oneEmplid + " " + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}
					
					try
					{
						// ORAL_EXAM_PLACE
						
						dataString = null;

						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_oral_exam_place != null) dataString = rec.m_oral_exam_place;
							}

							//m_logger.debug("DISSERTATION : SERVICE : ORAL_EXAM_PLACE : " + dataString);
							infoEdit.setOralExamPlace(dataString);
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // ORAL_EXAM_PLACE " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}

					try
					{
						// COMMITTE_APPROVED_DATE
						
						dataString = null;

						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_committee_approved_date != null) dataString = rec.m_committee_approved_date;
							}
							
							//m_logger.debug("DISSERTATION : SERVICE : COMMITTE_APPROVED_DATE : " + dataString);
							infoEdit.setTimeCommitteeApproval(parseTimeString(dataString));
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // COMMITTE_APPROVED_DATE " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}
					
					try
					{
						// FIRST_FORMAT_DATE
						
						dataString = null;

						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_first_format_date != null) dataString = rec.m_first_format_date;
							}
							//m_logger.debug("DISSERTATION : SERVICE : FIRST_FORMAT_DATE : " + dataString);
							infoEdit.setTimeFirstFormat(parseTimeString(dataString));
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // FIRST_FORMAT_DATE " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}

					try
					{						
						// BINDER_RECEPT_DATE
						
						dataString = null;

						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_binder_receipt_date != null) dataString = rec.m_binder_receipt_date;
							}
							
							//m_logger.debug("DISSERTATION : SERVICE : BINDER_RECEPT_DATE : " + dataString);
							infoEdit.setTimeBinderReceipt(parseTimeString(dataString));
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // BINDER_RECEPT_DATE " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}
					
					try
					{
						// FEE_REQUIREMENT_MET
						
						dataString = null;

						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_fee_requirement_met != null) dataString = rec.m_fee_requirement_met;
							}
							reqMet = false;
							//m_logger.debug("DISSERTATION : SERVICE : FEE_REQUIREMENT_MET : " + dataString);
							if(dataString != null)
							{
								if((dataString.indexOf("y") != -1) || (dataString.indexOf("Y") != -1))
									reqMet = true;
							}
							infoEdit.setFeeRequirementMet(reqMet);
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // FEE_REQUIREMENT_MET " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}	
					
					try
					{
						// FEE_RECEIPT_DATE_SEEN
						
						dataString = null;

						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_fee_date_receipt_seen != null) dataString = rec.m_fee_date_receipt_seen;
							}
							//m_logger.debug("DISSERTATION : SERVICE : FEE_RECEIPT_DATE_SEEN : " + dataString);
							infoEdit.setTimeReceiptSeen(parseTimeString(dataString));
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // FEE_RECEIPT_DATE_SEEN " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}
					
					try
					{
						// PUB_FEE_DATE_RECEIVED
						
						dataString = null;

						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_pub_fee_date_received != null) dataString = rec.m_pub_fee_date_received;
							}
							//m_logger.debug("DISSERTATION : SERVICE : PUB_FEE_DATE_RECEIVED : " + dataString);
							infoEdit.setTimePubFee(parseTimeString(dataString));
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // PUB_FEE_DATE_RECEIVED " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}
					
					try
					{	
						// ORAL_REPORT_RETURN_DATE
						
						dataString = null;

						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_oral_report_return_date != null) dataString = rec.m_oral_report_return_date;
							}
							//m_logger.debug("DISSERTATION : SERVICE : ORAL_REPORT_RETURN_DATE : " + dataString);
							infoEdit.setTimeOralReportReturned(parseTimeString(dataString));
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // ORAL_REPORT_RETURN_DATE " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}
					
					try
					{	
						// UNBOUND_DATE
						
						dataString = null;

						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_unbound_date != null) dataString = rec.m_unbound_date;
							}
							//m_logger.debug("DISSERTATION : SERVICE : UNBOUND_DATE : " + dataString);
							infoEdit.setTimeUnbound(parseTimeString(dataString));
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // UNBOUND_DATE " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}
					
					try
					{	
						// ABSTRACT_DATE
						
						dataString = null;

						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_abstract_date != null) dataString = rec.m_abstract_date;
							}
							//m_logger.debug("DISSERTATION : SERVICE : ABSTRACT_DATE : " + dataString);
							infoEdit.setTimeAbstract(parseTimeString(dataString));
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // ABSTRACT_DATE " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}
					
					try
					{	
						// BOUND_COPY_RECEIVED_DATE
						
						dataString = null;

						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_bound_copy_received_date != null) dataString = rec.m_bound_copy_received_date;
							}
							//m_logger.debug("DISSERTATION : SERVICE : BOUND_COPY_RECEIVED_DATE : " + dataString);
							infoEdit.setTimeBound(parseTimeString(dataString));
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // BOUND_COPY_RECEIVED_DATE " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}
					
					try
					{	
						// DIPLOMA_APPLICATION_DATE
						
						dataString = null;

						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_diploma_application_date != null) dataString = rec.m_diploma_application_date;
							}
							//m_logger.debug("DISSERTATION : SERVICE : DIPLOMA_APPLICATION_DATE : " + dataString);
							infoEdit.setTimeDiplomaApp(parseTimeString(dataString));
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // DIPLOMA_APPLICATION_DATE " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}
					
					try
					{	
						// CONTRACT_RECEIVED_DATE
						
						dataString = null;

						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_contract_received_date != null) dataString = rec.m_contract_received_date;
							}
							//m_logger.debug("DISSERTATION : SERVICE : CONTRACT_RECEIVED_DATE : " + dataString);
							infoEdit.setTimeContract(parseTimeString(dataString));
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // CONTRACT_RECEIVED_DATE " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}
					
					try
					{	
						// NSF_survey_date
						
						dataString = null;

						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_nsf_survey_date != null) dataString = rec.m_nsf_survey_date;
							}
							//m_logger.debug("DISSERTATION : SERVICE : NSF_survey_date : " + dataString);
							infoEdit.setTimeSurvey(parseTimeString(dataString));
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // NSF_survey_date " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}
					
					try
					{	
						// DEGREE_CONFERRED_DATE
						
						dataString = null;

						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_degree_conferred_date != null) dataString = rec.m_degree_conferred_date;
							}
							//m_logger.debug("DISSERTATION : SERVICE : DEGREE_CONFERRED_DATE : " + dataString);
							infoEdit.setTimeDegreeConferred(parseTimeString(dataString));
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // DEGREE_CONFERRED_DATE " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}
					
					try
					{	
						// FINAL_FORMAT_RECORDER
						
						dataString = "";

						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_final_format_recorder != null) dataString = rec.m_final_format_recorder;
							}
							//m_logger.debug("DISSERTATION : SERVICE : FINAL_FORMAT_RECORDER : " + dataString);
							infoEdit.setFinalFormatRecorder(dataString);
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // FINAL_FORMAT_RECORDER " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}
					
					try
					{	
						// COMM_CERT_DATE
						
						dataString = null;

						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_comm_cert_date != null) dataString = rec.m_comm_cert_date;
							}
							//m_logger.debug("DISSERTATION : SERVICE : COMM_CERT_DATE : " + dataString);
							infoEdit.setTimeCommitteeCert(parseTimeString(dataString));
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // COMM_CERT_DATE " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : LISTENER : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}
					
					try
					{	
						// EVAL_DATE
						
						dataString = null;

						if(existsOARDrecs)
						{
							for(ListIterator i = OARDrecs.listIterator(); i.hasNext(); )
							{
								OARDRecord rec = (OARDRecord) i.next();
								if(rec.m_eval_date != null) dataString = rec.m_eval_date;
								//m_logger.debug("DISSERTATION : SERVICE : EVAL_DATE : " + dataString);
								infoEdit.addTimeCommitteeEval(parseTimeString(dataString));
							}
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // EVAL_DATE " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}
					
					try
					{	
						// ANTICIPATE
						
						dataString = null;

						if(existsMPrecs)
						{
							for(ListIterator i = MPrecs.listIterator(); i.hasNext(); )
							{
								MPRecord rec = (MPRecord) i.next();
								if(rec.m_anticipate != null) dataString = rec.m_anticipate;
							}
							//m_logger.debug("DISSERTATION : SERVICE : ANTICIPATE : " + dataString);
							infoEdit.setAdvCandDesc(dataString);
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA // ANTICIPATE " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}
					
					try
					{
						// DATE_COMPL
						
						dataString = null;

						if(existsMPrecs)
						{
							for(ListIterator i = MPrecs.listIterator(); i.hasNext(); )
							{
								MPRecord rec = (MPRecord) i.next();
								//if(rec.m_date_compl != null) dataString = rec.m_date_compl;
								if(rec.m_milestone.equalsIgnoreCase("prelim"))
								{
									if(rec.m_date_compl != null) 
									{
										dataString = rec.m_date_compl;
									}
									//m_logger.debug("DISSERTATION : SERVICE : DATE_COMPL : " + dataString);
									infoEdit.setPrelimTimeMilestoneCompleted(parseTimeString(dataString));
									infoEdit.setPrelimMilestone(rec.m_milestone);
								}
								else if(rec.m_milestone.equalsIgnoreCase("advcand"))
								{
									if(rec.m_date_compl != null) 
									{
										dataString = rec.m_date_compl;
									}
									//m_logger.debug("DISSERTATION : SERVICE : DATE_COMPL : " + dataString);
									infoEdit.setAdvcandTimeMilestoneCompleted(parseTimeString(dataString));
									infoEdit.setAdvcandMilestone(rec.m_milestone);
								}
							}
	
						}
					}
					catch(Exception e)
					{
						msg = "// GET STATUS DATA  // DATE_COMPL " + oneEmplid + e.getMessage();
						rv.add(msg);
						m_logger.warn("DISSERTATION : SERVICE : GETTING STATUS DATA FROM NAVIGO : EXCEPTION : " + e);
					}
					
					if(commitEdit)
					{
						commitEdit(infoEdit);
						//m_logger.debug("DISSERTATION : SERVICE : CANDIDATE INFO EDIT COMMITTED");
					}
					else
					{
						cancelEdit(infoEdit);
						//m_logger.debug("DISSERTATION : SERVICE : CANDIDATE INFO NOT MODIFIED - EDIT CANCELED");
					}
					
					data.add(infoEdit);
					
				} //try to add a candidateinfo to data
				catch(Exception e)
				{
					msg = "//TRYING TO ADD A CANDIDATEINFO " + oneEmplid + e.getMessage();
					rv.add(msg);
					m_logger.warn("DISSERTATION : SERVICE : EXCEPTION WHILE PROCESSING EMPLID  :  " + oneEmplid + "  :  " + e);
				}
				
			} //while emplid.hasMoreElements
			
			dumpData(data);
			
			//m_logger.debug("DISSERTATION SERVICE : LOAD : queryLists: complete");
		} //if emplid!=null

		return rv;
			
	} //queryLists
	
	/**
	*
	* Parse and validate a Rackham OARD data extract file
	* @param fi - the OARD FileItem to be parsed
	* @return rv - a Vector of String error messages
	*/
	private Vector validateOARDData(byte[] content)
	{
		Vector rv = new Vector();
		
		boolean replace = true;
	
		String message = "";
		String prefix = "";
		int lineNumber = 0;
		
		try
		{
			// get lines
			String[] lns = ((String)new String(content)).split("\n");
				
			// replace commas used as field separators, but leave commas within quoted fields
			for (int i = 0; i < lns.length; i++)
			{
				//skip last line which contains a single hex value 0x1A
				if(lns[i].length() > 1)
				{
					StringBuffer buf = new StringBuffer(lns[i]);
					for (int j = 0; j < ((String) lns[i]).length(); j++)
					{
						char ch = lns[i].charAt(j);
						if(ch == '"') 
						{
							replace = !replace;
						}
						if(ch == ',')
						{
							if(replace) 
							{
								buf.setCharAt(j, '%');
							}
							else
							{
								buf.setCharAt(j, ch);
							}
						}
						else
						{
							buf.setCharAt(j, ch);
						}
					}
						
					String line = buf.toString();
					buf.setLength(0);
						
					//get the fields
					String[] flds = line.split("[%]");
					
					lineNumber = i + 1;
					prefix = "Source: OARD File: Location: line " + lineNumber + ", field ";
					message = "";
					
					//check that we have the right number of fields
					if((flds.length==29))
					{
						//check that content of field matches field pattern, umid not null
						if(!(flds[0].matches(PATTERN_UMID))) 
						{
							message = prefix + "1  Explanation: umid = " + flds[0];
							rv.add(message);
						}
						if(!(flds[1] == null || flds[1].matches(PATTERN_FOS)))
						{
							message = prefix + "2  Explanation:  fos = " + flds[1];
							rv.add(message);
						}
						if(!(flds[2] == null || flds[2].matches(PATTERN_LNAME)) )
						{
							message = prefix + "3  Explanation: lname = " + flds[2];
							rv.add(message);
							message = "";
						}
						if(!(flds[3] == null || flds[3].matches(PATTERN_FNAME)))
						{
							message = prefix + "4  Explanation:  fname = " + flds[3];
							rv.add(message);
						}
						if(!(flds[4] == null || flds[4].matches(PATTERN_DEGREETERM_TRANS)))
						{
							message = prefix + "5  Explanation: degreeterm_trans = " + flds[4];
							rv.add(message);
						}
						if(!(flds[5] == null || flds[5].matches(PATTERN_ORAL_EXAM_DATE)))
						{
							message = prefix + "6  Explanation: oral_exam_date_time = " + flds[5];
							rv.add(message);
						}
						if(!(flds[6] == null || flds[6].matches(PATTERN_ORAL_EXAM_TIME)))
						{
							message = prefix + "7  Explanation: oral_exam_time = " + flds[6];
							rv.add(message);
						}
						if(!(flds[7] == null || flds[7].matches(PATTERN_ORAL_EXAM_PLACE)))
						{
							message = prefix + "8  Explanation: oral_exam_place = " + flds[7];
							rv.add(message);
						}
						if(!(flds[8] == null || flds[8].matches(PATTERN_COMMITTEE_APPROVED_DATE)))
						{
							message = prefix + "9  Explanation: committee_approved_date = " + flds[8];
							rv.add(message);
						}
						if(!(flds[9] == null || flds[9].matches(PATTERN_FIRST_FORMAT_DATE)))
						{
							message = prefix + "10  Explanation: first_format_date = " + flds[9];
							rv.add(message);
						}
						if(!(flds[10] == null || flds[10].matches(PATTERN_BINDER_RECEIPT_DATE)))
						{
						message = prefix + "11  Explanation: binder_receipt_date = " + flds[10];
						rv.add(message);
						}
						if(!(flds[11] == null || flds[11].matches(PATTERN_FEE_REQUIREMENT_MET)))
						{
							message = prefix + "12  Explanation: fee_requirement_met  = " + flds[11];
							rv.add(message);
						}
						if(!(flds[12] == null || flds[12].matches(PATTERN_FEE_DATE_RECEIPT)))
						{
							message = prefix + "13  Explanation: fee_receipt_date_seen = " + flds[12];
							rv.add(message);
						}
						if(!(flds[13] == null || flds[13].matches(PATTERN_FEE_DATE_RECEIVED)))
						{
							message = prefix + "14  Explanation: pub_fee_date_received = " + flds[13];
							rv.add(message);
						}
						if(!(flds[14] == null || flds[14].matches(PATTERN_ORAL_REPORT_RETURN)))
						{
							message = prefix + "15  Explanation: oral_report_return_date = " + flds[14];
							rv.add(message);
						}
						if(!(flds[15] == null || flds[15].matches(PATTERN_UNBOUND_DATE)))
						{
							message = prefix + "16  Explanation: unbound_date = " + flds[15];
							rv.add(message);
						}
						if(!(flds[16] == null || flds[16].matches(PATTERN_ABSTRACT_DATE)))
						{
							message = prefix + "17  Explanation: abstract_date = " + flds[16];
							rv.add(message);
						}
						if(!(flds[17] == null || flds[17].matches(PATTERN_BOUND_COPY_RECEIVED_DATE)))
						{
							message = prefix + "18  Explanation: bound_copy_received_date = " + flds[17];
							rv.add(message);
						}
						if(!(flds[18] == null || flds[18].matches(PATTERN_DIPLOMA_APPLICATION_DATE)))
						{
							message = prefix + "19  Explanation: diploma_application_date = " + flds[18];
							rv.add(message);
						}
						if(!(flds[19] == null || flds[19].matches(PATTERN_CONTRACT_RECEIVED_DATE)))
						{
							message = prefix + "20  Explanation: contract_received_date  = " + flds[19];
							rv.add(message);
						}
						if(!(flds[20] == null || flds[20].matches(PATTERN_NSF_SURVEY_DATE)))
						{
							message = prefix + "21  Explanation: nsf_survey_date = " + flds[20];
							rv.add(message);
						}
						if(!(flds[21] == null || flds[21].matches(PATTERN_DEGREE_CONFERRED_DATE)))
						{
							message = prefix + "22  Explanation: degree_conferred_date = " + flds[21];
							rv.add(message);
						}
						if(!(flds[22] == null || flds[22].matches(PATTERN_FINAL_FORMAT_DATE))) 
						{
							message = prefix + "23  Explanation: final_format_recorder = " + flds[22];
							rv.add(message);
						}
						if(!(flds[23] == null || flds[23].matches(PATTERN_UPDATE_DATE)))
						{
							message = prefix + "24  Explanation: update_date = " + flds[23];
							rv.add(message);
						}
						if(!(flds[24] == null || flds[24].matches(PATTERN_COMM_CERT_DATE)))
						{
							message = prefix + "25  Explanation: comm_cert_date = " + flds[24];
							rv.add(message);
						}
						if(!(flds[25] == null || flds[25].matches(PATTERN_ROLE))) 
						{
							message = prefix + "26  Explanation: role = " + flds[25];
							rv.add(message);
							}
						if(!(flds[26] == null || flds[26].matches(PATTERN_MEMBER)))
						{
							message = prefix + "27  Explanation: member = " + flds[26];
							rv.add(message);
						}
						if(!(flds[27] == null || flds[27].matches(PATTERN_EVAL_DATE))) 
						{
							message = prefix + "28  Explanation: eval_date = " + flds[27];
							rv.add(message);
						}
						if(!(flds[28] == null || flds[28].matches(PATTERN_CAMPUS_ID)))
						{
							message = prefix + "29  Explanation: campus_id = " + flds[28];
							rv.add(message);
						}
							
						//check that there is a group roll-up code that matches this field of study value
						String field_of_study = flds[1].substring(1,flds[1].length()-1);
						String rollup = getProgram(field_of_study);
						if(rollup==null)
						{
							//no matching group roll-up code
							message = prefix + "1  Explanation: fos " + field_of_study + " does not match a getProgram() group roll-up code.";
							rv.add(message);
						}
					}
					else
					{
						//number of fields exception
						lineNumber = i + 1;
						message = "Source: OARD File: Location: line " + lineNumber + " Explanation: has " + flds.length + " fields: 29 expected.";
						m_logger.warn(this + ".createRecordsMP: " + message);
						rv.add(message);
					}
				}
			}
		}
		catch (Exception e)
		{
			message = "Source: OARD File: Explanation: unexpected problem: "  + e.getMessage();
			m_logger.warn(this + ".validateOARDData: " + message);
			rv.add(message);
		}
		
		return rv;
		
	}//validateOARDData
	
	/**
	*
	* Parse a Rackham OARD data extract file
	* @param content - the content to be parsed
	* @param ids - Hashtable with key = emplid value = uniqname
	* @return rv - a Vector of OARDRecord
	*/
	private Vector createOARDRecords(byte[] content, Hashtable ids)
	{
		Vector rv = new Vector();
		boolean replace = true;
		char quote = '"';

		try
		{
			// get individual lines
			String[] lns = ((String) new String(content)).split("\n");
				
			// replace commas used as field separators, but leave commas within quoted fields (i.e., committee member's name)
			for (int i = 0; i < lns.length; i++)
			{
				//skip last line, which contains a single hidden character
				if(lns[i].length() > 1)
				{
					StringBuffer buf = new StringBuffer(lns[i]);
					for (int j = 0; j < ((String) lns[i]).length(); j++)
					{
						char ch = lns[i].charAt(j);
						
						//replace commas separating fields with %'s
						if(ch == '"') 
						{
							replace = !replace;
						}
						if(ch == ',')
						{
							if(replace) 
							{
								buf.setCharAt(j, '%');
							}
							else
							{
								buf.setCharAt(j, ch);
							}
						}
						else
						{
							buf.setCharAt(j, ch);
						}
					}
						
					//if last character is \r, remove it
					if(buf.indexOf("\r") != -1)
					{
						buf.setLength(buf.length() -1);
					}
					String line = buf.toString();
					buf.setLength(0);
					
					//get the individual fields
					String[] flds = line.split("%");
					
					//check that we have the right number of fields
					if(flds.length == 29)
					{
						//create a record and add it to the Vector
						OARDRecord oard = new OARDRecord();
						
						//* 1   Emplid     A8 - Student's emplid
						oard.m_umid = flds[0].substring(1,flds[0].length()-1);
						
						//* 2   Fos        A4    - Students field of study code
						oard.m_fos = flds[1].substring(1,flds[1].length()-1);
						
						//* 3   Lname      A25 - Students last name
						oard.m_lname = flds[2].substring(1,flds[2].length()-1);
						
						//* 4   Fname      A30 - Students first name
						oard.m_fname = flds[3].substring(1,flds[3].length()-1);
						
						//* 5   Degterm trans     A7 - Students degree term as TT-CCYY (e.g. FA-2003)
						oard.m_degreeterm_trans = flds[4].substring(1,flds[4].length()-1);
						
						//* 6   Oral exam date    D - Date of oral defense
						oard.m_oral_exam_date = flds[5];
						
						//* 7   Oral exam time    A7 - Time of oral defense
						oard.m_oral_exam_time = flds[6].substring(1,flds[6].length()-1);
						
						//* 8   Oral exam place   A25 - Place of oral defense
						oard.m_oral_exam_place = flds[7].substring(1,flds[7].length()-1);
						
						//* 9   Committee approved date   D - date committee was approved
						oard.m_committee_approved_date = flds[8];
						
						//*10   First format date    D - date of pre defense meeting in Rackham
						oard.m_first_format_date = flds[9];
						
						//*11   Binder receipt date    D 
						oard.m_binder_receipt_date = flds[10];
						
						//*12   Fee requirement met    A1 - Y or N 
						oard.m_fee_requirement_met = flds[11].substring(1,flds[11].length()-1);
						
						//*13   Fee date receipt seen   D 
						oard.m_fee_date_receipt_seen = flds[12];
						
						//*14   Pub fee date received   D
						oard.m_pub_fee_date_received = flds[13];
						
						//*15   Oral report return date   D
						oard.m_oral_report_return_date = flds[14];
						
						//*16   Unbound date     D
						oard.m_unbound_date = flds[15];
						
						//*17   Abstract date     D
						oard.m_abstract_date = flds[16];
						
						//*18   Bound copy received date   D
						oard.m_bound_copy_received_date = flds[17];
						
						//*19   Diploma application date   D
						oard.m_diploma_application_date = flds[18];
						
						//*20   Contract received date   D
						oard.m_contract_received_date = flds[19];
						
						//*21   NSF Survey date    D
						oard.m_nsf_survey_date = flds[20];
						
						//*22   Degree conferred date   D - date the degree was conferred in OARD system
						oard.m_degree_conferred_date = flds[21];
						
						//*23   Final format recorder   A3 - initials
						oard.m_final_format_recorder = flds[22].substring(1,flds[22].length()-1);
						
						//*24   Update date    D - date record was last modified
						oard.m_update_date = flds[23];
						
						//*25   Comm cert date    D -
						oard.m_comm_cert_date = flds[24];
						
						//*26   Role    A2 - role code
						oard.m_role = flds[25].substring(1,flds[25].length()-1);
						
						//*27   Member   A40 - faculty member name
						oard.m_member = flds[26].substring(1,flds[26].length()-1);
						
						//*28   Eval date    D - evaluation received date
						oard.m_eval_date = flds[27];
						
						//*29   Campus id    A1-A8 - uniqname
						if(flds[28].indexOf("\r") != -1)
						{
							oard.m_campus_id = flds[28].substring(1,flds[28].length()-2);
						}
						else
						{
							oard.m_campus_id = flds[28].substring(1,flds[28].length()-1);
						}
						
						//add a record
						rv.add(oard);
						
						//add a uniqname
						ids.put(oard.m_umid, oard.m_campus_id);
					}
					else
					{
						m_logger.warn(this + ".createOARDRecords: file has " + flds.length + " fields.");
					}
				}
			}
		}
		catch (Exception e)
		{
			m_logger.warn(this + " .createOARDRecords: " + e.getMessage());
		}

		return rv;
		
	}//createOARDRecords
	
	/**
	*
	* Parse and validate a Rackham MP data extract file
	* @param fi - the MP FileItem to be parsed
	* @return rv - a Vector of MPRecord
	*/
	private Vector validateMPData(byte[] content)
	{
		Vector rv = new Vector();
		String field_of_study = "";
		String message = "";
		String prefix = "";
		int lineNumber = 0;
		
		try
		{
			// get lines
			String[] lns = ((String) new String(content)).split("\n");
			for (int i = 0; i < lns.length; i++)
			{
				//skip last line which contains a single hex 0x1A
				if(lns[i].length() > 1)
				{
					String[] flds = lns[i].split(",");
					message = "";
					lineNumber = i + 1;
					prefix = "Source: MP File: Location: line " + lineNumber + ", field ";

					//check that we have the right number of fields
					if(flds.length==7)
					{
						//check that content of field matches field pattern, umid not null
						if(!flds[0].matches(PATTERN_UMID)) 
						{
							message = prefix + "1  Explanation: umid = " + flds[0];
							rv.add(message);
						}
						if(!(flds[1] == null || flds[1].matches(PATTERN_ACAD_PROG)))
						{
							message = prefix + "2  Explanation: acad_prog = " + flds[1];
							rv.add(message);
						}
						if(!(flds[2] == null || flds[2].matches(PATTERN_ANTICIPATE)))
						{
							message = prefix + "3  Explanation: anticipate = " + flds[2];
							rv.add(message);
						} 
						if(!(flds[3] == null || flds[3].matches(PATTERN_DATE_COMPL)))
						{
							message = prefix + "4  Explanation:  date_compl = " + flds[3];
							rv.add(message);
						}
						if(!(flds[4] == null || flds[4].matches(PATTERN_MILESTONE)))
						{
							message = prefix + "5  Explanation: milestone = " + flds[4];
							rv.add(message);
						}
						if(!(flds[5] == null || flds[5].matches(PATTERN_ACADEMIC_PLAN)))
						{
							message = prefix + "6  Explanation: academic_plan = " + flds[5];
							rv.add(message);
						}
						else
						{
							//get the field of study part of the academic plan for roll-up
							field_of_study = flds[5].substring(1,5);
						}
						if(!(flds[6].matches(PATTERN_CAMPUS_ID)))
						{
							message = prefix + "7  Explanation: campus_id = " + flds[6];
							rv.add(message);
						}
							
						//check that there is a group roll-up code that matches this field of study value
						String rollup = getProgram(field_of_study);
						if(rollup==null)
						{
							//no matching group roll-up code
							message = prefix + "1  Explanation: field of study " + field_of_study + " does not match a group roll-up code.";
							rv.add(message);
						}
					}
					else
					{
						lineNumber = i + 1;
						message = "Source: MP File: Location: line " + lineNumber + " Explanation: has " + flds.length + " fields: 7 expected.";
						m_logger.warn(this + ".validateMPData: " + message);
						rv.add(message);
					}
				}
			}
		}
		catch (Exception e)
		{
			message = "Source: MP File: Explanation: unexpected problem with data: "  + e.getMessage();
			m_logger.warn(this + ".validateMPData: " + message);
			rv.add(message);
		}
		
		return rv;
		
	}//validateMPData
	
	/**
	*
	* Parse a Rackham MP data extract file
	* @param content - the MPathways data content to be parsed
	* @param ids - Hashtable with key = emplid value = uniqname
	* @return rv - a Vector of MPRecord
	*/
	private Vector createMPRecords(byte[] content, Hashtable ids)
	{
		Vector rv = new Vector();
		
		try
		{
			// get lines
			String[] lns = ((String) new String(content)).split("\n");
			for (int i = 0; i < lns.length; i++)
			{
				//skip last line which contains a single hex value 0x1A
				if(lns[i].length() > 1)
				{
					String[] flds = lns[i].split(",");
						
					//check that we have the right number of fields
					if(flds.length == 7)
					{
						//strip quotation marks from quoted fields
						MPRecord mp = new MPRecord();
						
						//* 1   Emplid   A9	| Student's emplid
						mp.m_umid = flds[0].substring(1,flds[0].length()-1);
						
						//* 2   Acad_prog   A9	| Academic Program Code
						mp.m_acad_prog = flds[1].substring(1,flds[1].length()-1);
						
						//* 3   Anticipate_Anticipate_1   A15	| Adv to cand term code
						mp.m_anticipate = flds[2].substring(1,flds[2].length()-1);
						
						//* 4   Date_compl   D	| Date milestone was completed 
						mp.m_date_compl = StringUtil.trimToNull(flds[3]);
						
						//* 5   Milestone   A10	| name of milestone PRELIM or ADVCAND
						mp.m_milestone = flds[4].substring(1,flds[4].length()-1);
						
						//* 6 | Academic plan |  A4	| Field of study and degree (e.g. 1220PHD1)
						mp.m_academic_plan = flds[5].substring(1,flds[5].length()-1);
						
						//*7  | Campus id  | A1-A8   | uniqname
						if(flds[6].indexOf("\r") != -1)
						{
							mp.m_campus_id = flds[6].substring(1,flds[6].length()-2);
						}
						else
						{
							mp.m_campus_id = flds[6].substring(1,flds[6].length()-1);
						}
						
						//add a record
						rv.add(mp);
						
						//add a uniqname
						ids.put(mp.m_umid, mp.m_campus_id);
					}
					else
					{
						m_logger.warn(this + ".createRecordsMP: file has " + flds.length + " fields.");
					}
				}
			}
		}
		catch (Exception e)
		{
			m_logger.warn(this + ".createMPRecords: " + e.getMessage());
		}

		return rv;
		
	}//createMPRecords
	
	/** 
	* Access the Rackham program id for Rackham field of study id.
	* @return The program id.
	*/
	public String getProgram(String fos)
	{
		String retVal = "";
		if(fos != null)
			retVal = (String)m_schoolGroups.get(fos);
		return retVal;
	}
	
	/** 
	* Access whether the system is initialized - i.e. whether any CandidateInfo objects exist.
	* @return True if any CandidateInfo objects exist, false if not.
	*/
	public boolean hasCandidateInfos()
	{	
		boolean retVal = false;
		List keys = m_infoCache.getKeys();
		if(keys.size() > 0)
			retVal = true;
		else
		{
			// CHECK THE DB
			List allInfos = m_infoStorage.getAll();
			if(allInfos.size() > 0)
				retVal = true;
		}
		return retVal;
		
	}

	/** 
	* Access whether the db-integration system is initialized.
	* @return True if it's initialized, false if not.
	*/
	public boolean getIsInitialized()
	{
		return m_initialized;
	}

	/** 
	* Send in a load of data from the Rackham database.
	* @param data Vector of CandidateInfo objects.
	*/
	public void dumpData(Vector data)
	{
		boolean changed = false;
		CandidateInfoEdit info = null;
		CandidatePath path = null;
		Hashtable orderedStatus = null;
		String statusId = null;
		String statusRef = null;
		String autoValidationId = null;
		int autoValidNumber = 0;
		StepStatus status = null;
		StepStatusEdit statusEdit = null;
		Time completionTime = null;
		
		/** If we don't know the Candidate's site, set CandidatePath.site to candidate uniqname until it can be set
		 correctly in DissertationAction.initState() */
		
		for(int x = 0; x < data.size(); x++)
		{
			try
			{
				info = (CandidateInfoEdit)data.get(x);
					
				// UPDATE CANDIDATE PATH DATA
				path = getCandidatePathForCandidate(info.getChefId());
				if(path == null)
				{
					//NO PATH EXISTS FOR USER
					
					CandidatePathEdit pathEdit = null;
					// USER HAS NO CANDIDATE PATH - CREATING ONE
					
					//there is no site, so site is set to uniqname
					String currentSite = info.getChefId();
					
					try
					{
						//String parentSite = getParentSiteForUser(currentUser.getId());
						String parentSite = info.getParentSiteId();
						Dissertation dissertation = getDissertationForSite(parentSite);
						if(dissertation == null)
						{
							//DISSERTATION FOR USERS PARENT DEPT IS NULL - USING SCHOOL
							String schoolSite = getSchoolSite();
							if(parentSite.equals(m_musicPerformanceSite))
							{
								dissertation = getDissertationForSite(schoolSite, DissertationService.DISSERTATION_TYPE_MUSIC_PERFORMANCE);
							}
							else
							{
								dissertation = getDissertationForSite(schoolSite, DissertationService.DISSERTATION_TYPE_DISSERTATION_STEPS);
							}
							if(dissertation == null)
							{
								m_logger.warn(this + ". dumpData DISSERTATION FOR SCHOOL IS NULL");
								return;
							}
							//Candidiate site is not known. Site is set to uniqname.
							pathEdit = addCandidatePathFromListener(dissertation, currentSite);
							
							//CANDIDATE PATH CREATED FOR USER
							pathEdit.setCandidate(info.getChefId());
							commitEdit(pathEdit);
						}
						else
						{
							//Candidiate site is not known. Site is set to uniqname.
							pathEdit = addCandidatePathFromListener(dissertation, currentSite);
							
							//CANDIDATE PATH CREATED FOR USER
							pathEdit.setCandidate(info.getChefId());
							commitEdit(pathEdit);
						}
						path = getCandidatePathForCandidate(info.getChefId());
					}
					catch(Exception e)
					{
						m_logger.warn(this + ".dumpData Exception " + e);
					}
				}
				
				orderedStatus = path.getOrderedStatus();
				
				//ITERATE THROUGH ORDERED STEPS
				for(int y = 1; y < (orderedStatus.size()+1); y++)
				{
					//GET A STATUS REF
					statusRef = (String)orderedStatus.get("" + y);
					statusId = statusId(statusRef);
					status = m_statusStorage.get(statusId);
					autoValidationId = status.getAutoValidationId();
					
					//GET THE AUTO VALID ID IF ANY
					if((!"".equals(autoValidationId)) && (!"None".equals(autoValidationId)))
					{
						try
						{
							//GET THE AUTO VALID NUMBER
							autoValidNumber = Integer.parseInt(autoValidationId);
							completionTime = info.getExternalValidation(autoValidNumber);
							if(completionTime != null)
							{
								//We only want autovalidation #9 to set TimeCompleted to new Time() once
								if(autoValidNumber != 9)
								{
									statusEdit = m_statusStorage.edit(statusId);	
									statusEdit.setCompleted(true);
									statusEdit.setTimeCompleted(completionTime);
									commitEdit(statusEdit);
								}
								else
								{
									
									if(status.getTimeCompleted()==null || status.getTimeCompleted().equals(""))
									{
										statusEdit = m_statusStorage.edit(statusId);	
										statusEdit.setCompleted(true);
										statusEdit.setTimeCompleted(completionTime);
										commitEdit(statusEdit);
									}
								}
								
								/** If we want to set prerequisite step status when step status is set, do here
								// Are there any prerequisites to this step that haven't been marked completed?
								Iterator prerequisites = path.getThread(status);
								if(prerequisites.hasNext())
								{
									//If status of prerequisites hasn't been set, set it
									{
										(StepStatus) prereq = (StepStatus)prerequisites.next();
										if(!prereq.getCompleted)
										{
											statusEdit = m_statusStorage.edit(prereq.getId());
											statusEdit.setCompleted(true);
											statusEdit.setTimeCompleted(completionTime);
											commitEdit(statusEdit);
										}
									}
								}
								*/
							}
							else
							{
								//COMPLETION TIME IS NULL
								statusEdit = m_statusStorage.edit(statusId);	
								statusEdit.setCompleted(false);
								statusEdit.setTimeCompleted(completionTime);
								commitEdit(statusEdit);
							}
						}
						catch(Exception nfe)
						{
							m_logger.warn("DISSERTATION : BASE SERVICE : DUMP DATA : EXCEPTION PROCESSING AUTOVALID NUMBER : " + autoValidationId);
							m_logger.warn("DISSERTATION : BASE SERVICE : DUMP DATA : EXCEPTION MESSAGE : " + nfe);
						}
					}
				}
			}
			catch(Exception e)
			{
				m_logger.warn("DISSERTATION : BASE SERVICE : DUMP DATA : EXCEPTION STORING DATA FOR : " + info.getChefId());
				m_logger.warn("DISSERTATION : BASE SERVICE : DUMP DATA : EXCEPTION " + e);
			}
		}
		m_initialized = true;
		
	}//dumpData
	
	/** 
	* Access the site for a Rackham program.
	* @param program The Rackham program id.
	* @return The CHEF site id for that program.
	*/
	protected String getSiteForProgram(String program)
	{
		String retVal = "diss" + program;
		return retVal;
		
	}//getParentSiteIdForProgram
	
	/** 
	* Access the list of candidates with the parent department site.
	* @param site The site id.
	* @return The list of users with the parent site.
	*/
	public User[] getAllUsersForSite(String site)
	{
		User[] retVal = null;
		
		CandidatePath aPath = null;
		List userIds = new Vector();
		List paths = getCandidatePaths();
		User aUser = null;

		for(int x = 0; x < paths.size(); x++)
		{
			aPath = (CandidatePath)paths.get(x);
			if(site.equals(getSchoolSite()))
			{
				userIds.add(aPath.getCandidate());
			}
			else
			{
				//GET ALL USERS FOR SITE : PATH PARENT SITE
				if(site.equals(getParentSiteForUser(aPath.getCandidate())))
				{
					//GET ALL USERS FOR SITE : SITE MATCH
					userIds.add(aPath.getCandidate());
					//GET ALL USERS FOR SITE : A USER ID
				}
			}
		}
			
		retVal = new User[userIds.size()];
		for(int x = 0; x < userIds.size(); x++)
		{
			try
			{
				retVal[x] = UserDirectoryService.getUser((String) userIds.get(x));
			}
			catch(IdUnusedException e)
			{
				m_logger.warn(this + ".getAllUsersForSite " + site + " User not found " + e);
			}
		}
		return retVal;
		
	} //getAllUsersForSite
	
	/** 
	* Access whether this user is a candidate.
	* @param userId The user's CHEF id.
	* @return True if a CandidateInfo exists for this user, false if not.
	*/
	public boolean getIsCandidate(String userId)
	{
		boolean retVal = false;
		if(userId != null)
		{
			try
			{
				CandidateInfo info = null;
				List allInfos = getCandidateInfos();
				for(int x = 0; x < allInfos.size(); x++)
				{
					info = (CandidateInfo)allInfos.get(x);
					//m_logger.debug("DISSERTATION : BASE SERVICE : GET IS CANDIDATE : COMPARING : USERID : " + userId);
					//m_logger.debug("DISSERTATION : BASE SERVICE : GET IS CANDIDATE : COMPARING : INFO USERID : " + info.getChefId());
					if(userId.equals(info.getChefId()))
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : GET IS CANDIDATE : FOUND A MATCH - WILL RETURN TRUE");
						retVal = true;
					}
				}
			}
			catch(Exception e){}
		}
		return retVal;
	}
	
	/** 
	* Access whether this user is a Music Performance candidate.
	* @param userId The user's CHEF id.
	* @return True if CandidateInfo getProgram() is Music Performance for this user, false if not.
	*/
	public boolean getIsMusicPerformanceCandidate(String userId)
	{
		boolean retVal = false;
		if(userId != null)
		{
			try
			{
				CandidateInfo info = null;
				List allInfos = getCandidateInfos();
				for(int x = 0; x < allInfos.size(); x++)
				{
					info = (CandidateInfo)allInfos.get(x);
					//m_logger.debug("DISSERTATION : BASE SERVICE : GET IS CANDIDATE : COMPARING : USERID : " + userId);
					//m_logger.debug("DISSERTATION : BASE SERVICE : GET IS CANDIDATE : COMPARING : INFO USERID : " + info.getChefId());
					if(userId.equals(info.getChefId()))
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : GET IS CANDIDATE : FOUND A MATCH - WILL RETURN TRUE");
						if(getMusicPerformanceSite().equals(info.getProgram()))
						{
							retVal = true;
						}
					}
				}
			}
			catch(Exception e){}
		}
		return retVal;
	}
	/**
	* Access the site for the user's Rackham department.
	* @param userId The user's CHEF id.
	* @return The site id of the parent site.
	*/
	public String getParentSiteForUser(String userId)
	{
		String retVal = "";
		List allInfos = null;
		CandidateInfo info = null;

		if(userId != null)
		{
			try
			{
				allInfos = getCandidateInfos();
	
				for(int x = 0; x < allInfos.size(); x++)
				{
					info = (CandidateInfo)allInfos.get(x);
					if(userId.equals(info.getChefId()))
					{
						retVal = getSiteForProgram(info.getProgram());
					}
				}
			}
			catch(Exception e)
			{
				m_logger.warn(this + ".getParentSiteForUser " + userId + " Exception " + e);
			}
		}
		return retVal;
	}

	/**
	* Access the University id for the user.
	* @param userId The user's CHEF id.
	* @return The university id for the user.
	*/
	public String getEmplidForUser(String userId)
	{
		//m_logger.debug("DISSERTATION : BASE SERVICE : GET EMPLID FOR USER : USERID : " + userId);
		String retVal = "";
		List allInfos = null;
		CandidateInfo info = null;
		if(userId != null)
		{
			try
			{
				allInfos = getCandidateInfos();
				for(int x = 0; x < allInfos.size(); x++)
				{
					info = (CandidateInfo)allInfos.get(x);
					//m_logger.debug("DISSERTATION : BASE SERVICE : GET EMPLID FOR USER : COMPARING : USERID : " + userId);
					//m_logger.debug("DISSERTATION : BASE SERVICE : GET EMPLID FOR USER : COMPARING : INFO USERID : " + info.getChefId());
					if(userId.equals(info.getChefId()))
					{
						retVal = info.getEmplid();
						//m_logger.debug("DISSERTATION : BASE SERVICE : GET EMPLID FOR USER : FOUND EMPLID : " + retVal);
					}
				}
			}
			catch(Exception e){}
		}
		return retVal;

	}
	
	/** 
	* Creates and adds a new BlockGrantGroup to the service.
	* @param id - The BlockGrantGroup id.
	* @return BlockGrantGroupEdit The new BlockGrantGroup object.
	* @throws PermissionException if current User does not have permission to do this.
	*/
	public BlockGrantGroupEdit addBlockGrantGroup(String site)
		throws PermissionException
	{
		String blockGrantGroupId = null;
		boolean badId = false;
		
		do
		{
			badId = false;
			blockGrantGroupId = IdService.getUniqueId();
			try
			{
				Validator.checkResourceId(blockGrantGroupId);
			}
			catch(IdInvalidException iie)
			{
				badId = true;
			}

			if(m_stepStorage.check(blockGrantGroupId))
				badId = true;
			
		}while(badId);
		
		String key = blockGrantGroupReference(site, blockGrantGroupId);
			
		unlock(SECURE_ADD_DISSERTATION_GROUP, key);
		
		BlockGrantGroupEdit blockGrantGroupEdit = m_groupStorage.put(blockGrantGroupId, site);
		
		((BaseBlockGrantGroupEdit) blockGrantGroupEdit).setEvent(SECURE_ADD_DISSERTATION_GROUP);
		
		return blockGrantGroupEdit;
	}
	
	/**
	* Get a locked BlockGrantGroup object for editing.  Must commitEdit() to make official, or cancelEdit() when done!
	* @param blockGrantGroupReference The Block Grant Group reference string.
	* @return A BlockGrantGroupEdit object for editing.
	* @exception IdUnusedException if not found, or if not a BlockGrantGroupEdit object
	* @exception PermissionException if the current user does not have permission to edit this BlockGrantGroup.
	* @exception InUseException if the BlockGrantGroup is being edited by another user.
	*/
	public BlockGrantGroupEdit editBlockGrantGroup(String blockGrantGroupReference)
		throws IdUnusedException, PermissionException, InUseException
	{
		unlock(SECURE_UPDATE_DISSERTATION_GROUP, blockGrantGroupReference);
		
		String blockGrantGroupId = blockGrantGroupId(blockGrantGroupReference);

		// check for existance
		if ((m_groupCache.get(blockGrantGroupReference) == null) && (!m_groupStorage.check(blockGrantGroupId)))
		{
			throw new IdUnusedException(blockGrantGroupId);
		}

		// ignore the cache - get the Block Grant Group with a lock from the info store
		BlockGrantGroupEdit blockGrantGroupEdit = m_groupStorage.edit(blockGrantGroupId);
		
		if (blockGrantGroupEdit == null) throw new InUseException(blockGrantGroupId);

		((BaseBlockGrantGroupEdit) blockGrantGroupEdit).setEvent(SECURE_UPDATE_DISSERTATION_GROUP);

		return blockGrantGroupEdit;

	}   // editBlockGrantGroup
	
	/**
	* Cancel the changes made to a BlockGrantGroupEdit object, and release the lock.
	* @param blockGrantGroup The BlockGrantGroupEdit object to commit.
	*/
	public void cancelEdit(BlockGrantGroupEdit blockGrantGroup)
	{
		// check for closed edit
		if (!blockGrantGroup.isActiveEdit())
		{
			try { throw new Exception(); }
			catch (Exception e) { m_logger.warn(this + ".cancelEdit(): closed BlockGrantGroupEdit", e); }
			return;
		}

		// release the edit lock
		m_groupStorage.cancel(blockGrantGroup);

		// close the edit object
		((BaseBlockGrantGroupEdit) blockGrantGroup).closeEdit();

	}	// cancelEdit(BlockGrantGroupEdit)
	
	/**
	* Commit the changes made to a BlockGrantGroupEdit object, and release the lock.
	* @param blockGrantGroup The BlockGrantGroupEdit object to commit.
	*/
	public void commitEdit(BlockGrantGroupEdit blockGrantGroup)
	{
		// check for closed edit
		if (!blockGrantGroup.isActiveEdit())
		{
			try { throw new Exception(); }
			catch (Exception e) { m_logger.warn(this + ".commitEdit(): closed BlockGrantGroupEdit", e); }
			return;
		}

		// update the properties
		addLiveUpdateProperties(blockGrantGroup.getPropertiesEdit());

		// complete the edit
		m_groupStorage.commit(blockGrantGroup);

		// track it
		EventTrackingService.post(
			EventTrackingService.newEvent(((BaseBlockGrantGroupEdit) blockGrantGroup).getEvent(), blockGrantGroup.getReference(), true));

		// close the edit object
		((BaseBlockGrantGroupEdit) blockGrantGroup).closeEdit();

	}	// commitEdit(BlockGrantGroupEdit)
	
	
	/**
	* Access all BlockGrantGroup objects.
	* @return A list of BlockGrantGroup objects.
	*/
	public List getBlockGrantGroups()
	{
		List blockGrantGroups = new Vector();

		// if we have disabled the cache, don't use if
		if (m_groupCache.disabled())
		{
			blockGrantGroups = m_groupStorage.getAll();
		}

		else
		{
			
			// if the cache is complete, use it
			if (m_groupCache.isComplete())
			{
				blockGrantGroups = m_groupCache.getAll();
			}
	
			// otherwise get all the Block Grant Groups from storage
			else
			{
				// Note: while we are getting from storage, storage might change.  These can be processed
				// after we get the storage entries, and put them in the cache, and mark the cache complete.
				// -ggolden
				synchronized (m_groupCache)
				{
					// if we were waiting and it's now complete...
					if (m_groupCache.isComplete())
					{
						blockGrantGroups = m_groupCache.getAll();
						return blockGrantGroups;
					}
	
					// save up any events to the cache until we get past this load
					m_groupCache.holdEvents();
					blockGrantGroups = m_groupStorage.getAll();
	
					// update the cache, and mark it complete
					for (int i = 0; i < blockGrantGroups.size(); i++)
					{
						BlockGrantGroup blockGrantGroup = (BlockGrantGroup) blockGrantGroups.get(i);
						m_groupCache.put(blockGrantGroup.getReference(), blockGrantGroup);
					}
	
					m_groupCache.setComplete();
	
					// now we are complete, process any cached events
					m_groupCache.processEvents();
				}
			}
		}
		return blockGrantGroups;

	}   // getBlockGrantGroups
	
	/** 
	* Removes this Dissertation and all references to it.
	* @param dissertation - The Dissertation to remove.
	* @throws PermissionException if current User does not have permission to do this.
	*/
	public void removeBlockGrantGroup(BlockGrantGroupEdit blockGrantGroupEdit)
		throws PermissionException
	{
		if(blockGrantGroupEdit != null)
		{
			if(!blockGrantGroupEdit.isActiveEdit())
			{
				try { throw new Exception(); }
				catch (Exception e) { m_logger.warn(this + ".removeBlockGrantGroup(): closed BlockGrantGroupEdit", e); }
				return;
			}
			
			unlock(SECURE_REMOVE_DISSERTATION_GROUP, blockGrantGroupEdit.getReference());
			
			m_groupStorage.remove(blockGrantGroupEdit);
			
			EventTrackingService.post(EventTrackingService.newEvent(SECURE_REMOVE_DISSERTATION_GROUP, blockGrantGroupEdit.getReference(), true));
			
			((BaseBlockGrantGroupEdit)blockGrantGroupEdit).closeEdit();
		}
		
	}//removeBlockGrantGroup
	
	/**
	* Access the BlockGrantGroup for the specified Field of Study.
	* @param field - The field for which BlockGrantGroup is to be returned.
	* @return The BlockGrantGroup corresponding to the field, or null if it does not exist.
	* @throws PermissionException if the current user is not allowed to access this.
	*/
	public BlockGrantGroup getBlockGrantGroupForFieldOfStudy(String field)
		throws PermissionException
	{
		BlockGrantGroup retVal = null;
		if(field != null)
		{
			BlockGrantGroup tempGroup = null;
			List allGroups = getBlockGrantGroups();
			Hashtable groupFields = null;
			for(int x = 0; x < allGroups.size(); x++)
			{
				try
				{
					tempGroup = (BlockGrantGroup)allGroups.get(x);					
					groupFields = tempGroup.getFieldsOfStudy();
					if(groupFields.containsKey(field))
					{
						retVal = (BlockGrantGroup) m_groupStorage.get(tempGroup.getId());
						return retVal;
					}
				}
				catch(Exception e)
				{
					if(m_logger.isWarnEnabled())
					{
						m_logger.warn("DISSERTATION : BASE SERVICE : GET BLOCK GRANT GROUP EDIT FOR FIELD OF STUDY : EXCEPTION " + e);
					}
				}
			}
		}
		return retVal;

	}//getBlockGrantGroupForFieldOfStudy	

	
	/** 
	* Creates and adds a new Dissertation to the service.
	* @param site - The site for which permissions are being checked.
	* @return The new Dissertation object.
	* @throws PermissionException if current User does not have permission to do this.
	*/
	public DissertationEdit addDissertation(String site)
		throws PermissionException
	{
		String dissertationId = null;
		boolean badId = false;
		
		do
		{
			badId = false;
			dissertationId = IdService.getUniqueId();
			try
			{
				Validator.checkResourceId(dissertationId);
			}
			catch(IdInvalidException iie)
			{
				badId = true;
			}

			if(m_dissertationStorage.check(dissertationId))
				badId = true;
			
		}while(badId);
		
		String key = dissertationReference(site, dissertationId);
		
		unlock(SECURE_ADD_DISSERTATION, key);
		
		//storage
		DissertationEdit dissertation = m_dissertationStorage.put(dissertationId, site);
		
		dissertation.setSite(site);
		
		((BaseDissertationEdit) dissertation).setEvent(SECURE_ADD_DISSERTATION);
		
		return dissertation;
	}

	/**
	* Add a new dissertation to the directory, from a definition in XML.
	* Must commitEdit() to make official, or cancelEdit() when done!
	* @param el The XML DOM Element defining the dissertation.
	* @return A locked DissertationEdit object (reserving the id).
	* @exception IdInvalidException if the dissertation id is invalid.
	* @exception IdUsedException if the dissertation id is already used.
	* @exception PermissionException if the current user does not have permission to add a dissertation.
	*/
	public DissertationEdit mergeDissertation(Element el)
		throws IdInvalidException, IdUsedException, PermissionException
	{
		// construct from the XML
		Dissertation dissertationFromXml = new BaseDissertation(el);

		// check for a valid dissertation name
		Validator.checkResourceId(dissertationFromXml.getId());

		// check security (throws if not permitted)
		unlock(SECURE_ADD_DISSERTATION, dissertationFromXml.getReference());

		// reserve a dissertation with this id from the info store - if it's in use, this will return null
		DissertationEdit dissertation = m_dissertationStorage.put(dissertationFromXml.getId(), dissertationFromXml.getSite());
		if (dissertation == null)
		{
			throw new IdUsedException(dissertationFromXml.getId());
		}

		// transfer from the XML read dissertation object to the DissertationEdit
		((BaseDissertationEdit) dissertation).set(dissertationFromXml);

		((BaseDissertationEdit) dissertation).setEvent(SECURE_ADD_DISSERTATION);

		return dissertation;

	}//mergeDissertation
	
	/**
	* Access the Dissertation with the specified reference.
	* @param dissertationReference - The reference of the Dissertation.
	* @return The Dissertation corresponding to the reference, or null if it does not exist.
	* @throws IdUnusedException if there is no object with this id.
	* @throws PermissionException if the current user is not allowed to access this.
	*/
	public Dissertation getDissertation(String dissertationReference)
		throws IdUnusedException, PermissionException
	{
		Dissertation dissertation = null;
		
		String dissertationId = dissertationId(dissertationReference);
		
		if(m_dissertationCache.containsKey(dissertationReference))
			dissertation = (Dissertation)m_dissertationCache.get(dissertationReference);
		else
		{
			dissertation = m_dissertationStorage.get(dissertationId);
			m_dissertationCache.put(dissertationReference, dissertation);
		}
		
		if(dissertation == null) throw new IdUnusedException(dissertationId);
		
		unlock(SECURE_ACCESS_DISSERTATION, dissertationReference);
		
		// EventTrackingService.post(EventTrackingService.newEvent(SECURE_ACCESS_DISSERTATION, dissertation.getReference(), false));
		
		return dissertation;
	}
	
	
	/**
	* Access all dissertation objects - known to us (not from external providers).
	* @return A list of dissertation objects.
	*/
	public List getDissertations()
	{
		List dissertations = new Vector();

		// if we have disabled the cache, don't use if
		if (m_dissertationCache.disabled())
		{
			dissertations = m_dissertationStorage.getAll();
		}

		else
		{
			// if the cache is complete, use it
			if (m_dissertationCache.isComplete())
			{
				dissertations = m_dissertationCache.getAll();
			}
	
			// otherwise get all the dissertations from storage
			else
			{
				// Note: while we are getting from storage, storage might change.  These can be processed
				// after we get the storage entries, and put them in the cache, and mark the cache complete.
				// -ggolden
				synchronized (m_dissertationCache)
				{
					// if we were waiting and it's now complete...
					if (m_dissertationCache.isComplete())
					{
						dissertations = m_dissertationCache.getAll();
						return dissertations;
					}
	
					// save up any events to the cache until we get past this load
					m_dissertationCache.holdEvents();
	
					dissertations = m_dissertationStorage.getAll();
	
					// update the cache, and mark it complete
					for (int i = 0; i < dissertations.size(); i++)
					{
						Dissertation dissertation = (Dissertation) dissertations.get(i);
						m_dissertationCache.put(dissertation.getReference(), dissertation);
					}
	
					m_dissertationCache.setComplete();
	
					// now we are complete, process any cached events
					m_dissertationCache.processEvents();
				}
			}
		}

		return dissertations;

	}   // getDissertations
	
	
	/**
	* Get a locked dissertation object for editing.  Must commitEdit() to make official, or cancelEdit() when done!
	* @param dissertationReference The dissertation reference string.
	* @return An DissertationEdit object for editing.
	* @exception IdUnusedException if not found, or if not an DissertationEdit object
	* @exception PermissionException if the current user does not have permission to edit this dissertation.
	* @exception InUseException if the dissertation is being edited by another user.
	*/
	public DissertationEdit editDissertation(String dissertationReference)
		throws IdUnusedException, PermissionException, InUseException
	{
		unlock(SECURE_UPDATE_DISSERTATION, dissertationReference);
		
		String dissertationId = dissertationId(dissertationReference);

		// check for existance
		if ((m_dissertationCache.get(dissertationReference) == null) && (!m_dissertationStorage.check(dissertationId)))
		{
			throw new IdUnusedException(dissertationId);
		}

		// ignore the cache - get the dissertation with a lock from the info store
		DissertationEdit dissertation = m_dissertationStorage.edit(dissertationId);
		
		if (dissertation == null) throw new InUseException(dissertationId);

		((BaseDissertationEdit) dissertation).setEvent(SECURE_UPDATE_DISSERTATION);

		return dissertation;

	}   // editDissertation
	
	
	/**
	* Commit the changes made to an DissertationEdit object, and release the lock.
	* @param dissertation The DissertationEdit object to commit.
	*/
	public void commitEdit(DissertationEdit dissertation)
	{
		// check for closed edit
		if (!dissertation.isActiveEdit())
		{
			try { throw new Exception(); }
			catch (Exception e) { m_logger.warn(this + ".commitEdit(): closed DissertationEdit", e); }
			return;
		}

		// update the properties
		addLiveUpdateProperties(dissertation.getPropertiesEdit());

		// complete the edit
		m_dissertationStorage.commit(dissertation);

		// track it
		EventTrackingService.post(
			EventTrackingService.newEvent(((BaseDissertationEdit) dissertation).getEvent(), dissertation.getReference(), true));

		// close the edit object
		((BaseDissertationEdit) dissertation).closeEdit();

	}	// commitEdit

	
	/**
	* Cancel the changes made to a DissertationEdit object, and release the lock.
	* @param dissertation The DissertationEdit object to commit.
	*/
	public void cancelEdit(DissertationEdit dissertation)
	{
		// check for closed edit
		if (!dissertation.isActiveEdit())
		{
			try { throw new Exception(); }
			catch (Exception e) { m_logger.warn(this + ".cancelEdit(): closed DissertationEdit", e); }
			return;
		}

		// release the edit lock
		m_dissertationStorage.cancel(dissertation);

		// close the edit object
		((BaseDissertationEdit) dissertation).closeEdit();

	}	// cancelEdit(Dissertation)
	
	/** 
	* Removes this Dissertation and all references to it.
	* @param dissertation - The Dissertation to remove.
	* @throws PermissionException if current User does not have permission to do this.
	*/
	public void removeDissertation(DissertationEdit dissertation)
		throws PermissionException
	{
		if(dissertation != null)
		{
			if(!dissertation.isActiveEdit())
			{
				try { throw new Exception(); }
				catch (Exception e) { m_logger.warn(this + ".removeDissertation(): closed DissertationEdit", e); }
				return;
			}
			
			unlock(SECURE_REMOVE_DISSERTATION, dissertation.getReference());
			
			m_dissertationStorage.remove(dissertation);
			
			EventTrackingService.post(EventTrackingService.newEvent(SECURE_REMOVE_DISSERTATION, dissertation.getReference(), true));
			
			((BaseDissertationEdit)dissertation).closeEdit();
		}
	}//removeDissertaion

	/** 
	* Creates and adds a new DissertationStep to the service.
	* @param site - The site for which permissions are being checked.
	* @return DissertationStepEdit The new DissertationStep object.
	* @throws PermissionException if current User does not have permission to do this.
	*/
	public DissertationStepEdit addDissertationStep(String site)
		throws PermissionException
	{
		String stepId = null;
		boolean badId = false;
		
		do
		{
			badId = false;
			stepId = IdService.getUniqueId();
			try
			{
				Validator.checkResourceId(stepId);
			}
			catch(IdInvalidException iie)
			{
				badId = true;
			}

			if(m_stepStorage.check(stepId))
				badId = true;
			
		}while(badId);
		
		String key = stepReference(site, stepId);
		
		unlock(SECURE_ADD_DISSERTATION_STEP, key);
		
		DissertationStepEdit step = m_stepStorage.put(stepId, site);

		step.setSite(site);
		
		((BaseDissertationStepEdit) step).setEvent(SECURE_ADD_DISSERTATION_STEP);
		
		return step;
	}
	
	/** 
	* Creates and adds a new DissertationStep to the service.
	* @param site - The site for which permissions are being checked.
	* @param type - The pedagogical type inhent in the steps.
	* @return DissertationStepEdit The new DissertationStep object.
	* @throws PermissionException if current User does not have permission to do this.
	*/
	public DissertationStepEdit addDissertationStep(String site, String type)
		throws PermissionException
	{
		String stepId = null;
		boolean badId = false;
		
		do
		{
			badId = false;
			stepId = IdService.getUniqueId();
			try
			{
				Validator.checkResourceId(stepId);
			}
			catch(IdInvalidException iie)
			{
				badId = true;
			}

			if(m_stepStorage.check(stepId))
				badId = true;
			
		}while(badId);
		
		String key = stepReference(site, stepId);
		
		unlock(SECURE_ADD_DISSERTATION_STEP, key);
		
		DissertationStepEdit step = m_stepStorage.put(stepId, site);

		step.setSite(site);
		
		((BaseDissertationStepEdit) step).setEvent(SECURE_ADD_DISSERTATION_STEP);
		
		return step;
	}
	

	/**
	* Add a new step to the directory, from a definition in XML.
	* Must commitEdit() to make official, or cancelEdit() when done!
	* @param el The XML DOM Element defining the step.
	* @return A locked DissertationStepEdit object (reserving the id).
	* @exception IdInvalidException if the step id is invalid.
	* @exception IdUsedException if the step id is already used.
	* @exception PermissionException if the current user does not have permission to add a DissertationStep.
	*/
	public DissertationStepEdit mergeDissertationStep(Element el)
		throws IdInvalidException, IdUsedException, PermissionException
	{
		// construct from the XML
		DissertationStep stepFromXml = new BaseDissertationStep(el);

		// check for a valid step name
		Validator.checkResourceId(stepFromXml.getId());

		// check security (throws if not permitted)
		unlock(SECURE_ADD_DISSERTATION_STEP, stepFromXml.getReference());

		// reserve a step with this id from the info store - if it's in use, this will return null
		DissertationStepEdit step = m_stepStorage.put(stepFromXml.getId(), stepFromXml.getSite());
		if (step == null)
		{
			throw new IdUsedException(stepFromXml.getId());
		}

		// transfer from the XML read step object to the StepEdit
		((BaseDissertationStepEdit) step).set(stepFromXml);

		((BaseDissertationStepEdit) step).setEvent(SECURE_ADD_DISSERTATION_STEP);

		return step;
		
	}//mergeStep
	
	/**
	* Access the DissertationStep with the specified id.
	* @param stepReference - The reference of the DissertationStep.
	* @return The DissertationStep corresponding to the reference, or null if it does not exist.
	* @throws IdUnusedException if there is no object with this reference.
	* @throws PermissionException if the current user is not allowed to access this.
	*/
	public DissertationStep getDissertationStep(String stepReference)
		throws IdUnusedException, PermissionException
	{
		DissertationStep step = null;

		String stepId = stepId(stepReference);
		
		if(m_stepCache.containsKey(stepReference))
			step = (DissertationStep)m_stepCache.get(stepReference);
		else
		{
			step = m_stepStorage.get(stepId);
			m_stepCache.put(stepReference, step);
		}
		
		if(step == null) throw new IdUnusedException(stepId);

		unlock(SECURE_ACCESS_DISSERTATION_STEP, stepReference);
		
		return step;

	}//getDissertationStep


	/**
	* Access all DissertationStep objects - known to us (not from external providers).
	* @return A list of DissertationStep objects.
	*/
	public List getDissertationSteps()
	{
		List steps = new Vector();

		// if we have disabled the cache, don't use if
		if (m_stepCache.disabled())
		{
			steps = m_stepStorage.getAll();
		}

		else
		{
			// if the cache is complete, use it
			if (m_stepCache.isComplete())
			{
				steps = m_stepCache.getAll();
			}
	
			// otherwise get all the steps from storage
			else
			{
				// Note: while we are getting from storage, storage might change.  These can be processed
				// after we get the storage entries, and put them in the cache, and mark the cache complete.
				// -ggolden
				synchronized (m_stepCache)
				{
					// if we were waiting and it's now complete...
					if (m_stepCache.isComplete())
					{
						steps = m_stepCache.getAll();
						return steps;
					}
	
					// save up any events to the cache until we get past this load
					m_stepCache.holdEvents();
					steps = m_stepStorage.getAll();
	
					// update the cache, and mark it complete
					for (int i = 0; i < steps.size(); i++)
					{
						DissertationStep step = (DissertationStep) steps.get(i);
						m_stepCache.put(step.getReference(), step);
					}
					m_stepCache.setComplete();
					
					// now we are complete, process any cached events
					m_stepCache.processEvents();
				}
			}
		}

		return steps;

	}   // getDissertationSteps
	
	
	/**
	* Get a locked step object for editing.  Must commitEdit() to make official, or cancelEdit() when done!
	* @param stepReference The step reference string.
	* @return A DissertationStepEdit object for editing.
	* @exception IdUnusedException if not found, or if not a DissertationStepEdit object
	* @exception PermissionException if the current user does not have permission to edit this step.
	* @exception InUseException if the step is being edited by another user.
	*/
	public DissertationStepEdit editDissertationStep(String stepReference)
		throws IdUnusedException, PermissionException, InUseException
	{
		// check security (throws if not permitted)
		unlock(SECURE_UPDATE_DISSERTATION_STEP, stepReference);

		String stepId = stepId(stepReference);
		
		// check for existance
		if ((m_stepCache.get(stepReference) == null) && (!m_stepStorage.check(stepId)))
		{
			throw new IdUnusedException(stepId);
		}

		// ignore the cache - get the step with a lock from the info store
		DissertationStepEdit step = m_stepStorage.edit(stepId);
		
		if (step == null) throw new InUseException(stepId);

		((BaseDissertationStepEdit) step).setEvent(SECURE_UPDATE_DISSERTATION_STEP);

		return step;

	}   // editDissertationStep

	
	/**
	* Commit the changes made to a DissertationStepEdit object, and release the lock.
	* @param step The DissertationStepEdit object to commit.
	*/
	public void commitEdit(DissertationStepEdit step)
	{
		// check for closed edit
		if (!step.isActiveEdit())
		{
			try { throw new Exception(); }
			catch (Exception e) { m_logger.warn(this + ".commitEdit(): closed DissertationStepEdit", e); }
			return;
		}

		//m_logger.debug("DISSERTATION : BASE SERVICE : COMMIT EDIT - STEP : 1");
		// update the properties
		addLiveUpdateProperties(step.getPropertiesEdit());
		//m_logger.debug("DISSERTATION : BASE SERVICE : COMMIT EDIT - STEP : 2");
		// complete the edit
		m_stepStorage.commit(step);
		//m_logger.debug("DISSERTATION : BASE SERVICE : COMMIT EDIT - STEP : 3");
		// track it
		EventTrackingService.post(
			EventTrackingService.newEvent(((BaseDissertationStepEdit) step).getEvent(), step.getReference(), true));
		//m_logger.debug("DISSERTATION : BASE SERVICE : COMMIT EDIT - STEP : 4");
		// close the edit object
		((BaseDissertationStepEdit) step).closeEdit();

		//m_logger.debug("DISSERTATION : BASE SERVICE : COMMIT EDIT - STEP : 5");
	}	// commitEdit

	
	/**
	* Cancel the changes made to a DissertationStepEdit object, and release the lock.
	* @param step The DissertationStepEdit object to commit.
	*/
	public void cancelEdit(DissertationStepEdit step)
	{
		// check for closed edit
		if (!step.isActiveEdit())
		{
			try { throw new Exception(); }
			catch (Exception e) { m_logger.warn(this + ".cancelEdit(): closed DissertationStepEdit", e); }
			return;
		}

		// release the edit lock
		m_stepStorage.cancel(step);

		// close the edit object
		((BaseDissertationStepEdit) step).closeEdit();

	}	// cancelEdit(DissertationStep)
	
	/** 
	* Removes a DissertationStep
	* @param step - the DissertationStep to remove.
	* @throws PermissionException if current User does not have permission to do this.
	*/
	public void removeDissertationStep(DissertationStepEdit step)
		throws PermissionException
	{
		if(step != null)
		{
			if(!step.isActiveEdit())
			{
				try { throw new Exception(); }
				catch (Exception e) { m_logger.warn(this + ".removeDissertationStep(): closed DissertationStepEdit", e); }
				return;
			}

			unlock(SECURE_REMOVE_DISSERTATION_STEP, step.getReference());

			m_stepStorage.remove(step);

			EventTrackingService.post(EventTrackingService.newEvent(SECURE_REMOVE_DISSERTATION_STEP, step.getReference(), true));
			
			((BaseDissertationStepEdit)step).closeEdit();
		}
	}

	/** 
	* Adds an CandidatePath to the service.
	* @param dissertation - The parent Dissertation.
	* @param site - The site for which permissions are being checked.
	* @return The new CandidatePath.
	* @throws PermissionException if the current User does not have permission to do this.
	*/
	public CandidatePathEdit addCandidatePath(Dissertation dissertation, String site)
		throws PermissionException
	{
		String pathId = null;
		boolean badId = false;
		
		do
		{
			badId = false;
			pathId = IdService.getUniqueId();
			try
			{
				Validator.checkResourceId(pathId);
			}
			catch(IdInvalidException iie)
			{
				badId = true;
			}

			if(m_pathStorage.check(pathId))
				badId = true;

		}while(badId);
		
		String key = pathReference(site, pathId);
		
		unlock(SECURE_ADD_DISSERTATION_CANDIDATEPATH, key);
		
		CandidatePathEdit path = m_pathStorage.put(pathId, site);
		
		dissertation.initializeCandidatePath(path, site);
		
		((BaseCandidatePathEdit) path).setEvent(SECURE_ADD_DISSERTATION_CANDIDATEPATH);
		
		return path;
	}
	
	/** 
	* Adds an CandidateInfo to the service.
	* @param dissertation - The parent Dissertation.
	* @param site - There is no current site - request comes from DissertationDataListenerService.
	* @return The new CandidateInfo.
	*/
	protected CandidatePathEdit addCandidatePathFromListener(Dissertation dissertation, String site)
	{
		String pathId = null;
		boolean badId = false;
		
		do
		{
			badId = false;
			pathId = IdService.getUniqueId();
			try
			{
				Validator.checkResourceId(pathId);
			}
			catch(IdInvalidException iie)
			{
				badId = true;
			}

			if(m_pathStorage.check(pathId))
				badId = true;

		}while(badId);
		
		String key = pathReference(site, pathId);
		
		CandidatePathEdit path = m_pathStorage.put(pathId, site);
		
		dissertation.initializeCandidatePath(path, site);
		
		((BaseCandidatePathEdit) path).setEvent(SECURE_ADD_DISSERTATION_CANDIDATEPATH);
		
		return path;
	}

	
	/**
	* Add a new CandidatePath to the directory, from a definition in XML.
	* Must commitEdit() to make official, or cancelEdit() when done!
	* @param el The XML DOM Element defining the CandidatePath.
	* @return A locked CandidatePathEdit object (reserving the id).
	* @exception IdInvalidException if the CandidatePath id is invalid.
	* @exception IdUsedException if the CandidatePath id is already used.
	* @exception PermissionException if the current user does not have permission to add a CandidatePath.
	*/
	public CandidatePathEdit mergeCandidatePath(Element el)
		throws IdInvalidException, IdUsedException, PermissionException
	{
		// construct from the XML
		CandidatePath pathFromXml = new BaseCandidatePath(el);

		// check for a valid path name
		Validator.checkResourceId(pathFromXml.getId());

		// check security (throws if not permitted)
		unlock(SECURE_ADD_DISSERTATION_CANDIDATEPATH, pathFromXml.getReference());

		// reserve a path with this id from the info store - if it's in use, this will return null
		CandidatePathEdit path = m_pathStorage.put(pathFromXml.getId(), pathFromXml.getSite());
		if (path == null)
		{
			throw new IdUsedException(pathFromXml.getId());
		}

		// transfer from the XML read path object to the CandidatePathEdit
		((BaseCandidatePathEdit) path).set(pathFromXml);


		((BaseCandidatePathEdit) path).setEvent(SECURE_ADD_DISSERTATION_CANDIDATEPATH);

		return path;

	}//mergeCandidatePath

	/**
	* Access the CandidatePath with the specified reference.
	* @param pathReference - The reference of the CandidatePath.
	* @return The CandidatePath corresponding to the id, or null if it does not exist.
	* @throws IdUnusedException if there is no object with this id.
	* @throws PermissionException if the current user is not allowed to access this.
	*/
	public CandidatePath getCandidatePath(String pathReference)
		throws IdUnusedException, PermissionException
	{
		CandidatePath path = null;
		
		String pathId = pathId(pathReference);
		
		if(m_pathCache.containsKey(pathReference))
			path = (CandidatePath)m_pathCache.get(pathReference);
		else
		{
			path = m_pathStorage.get(pathId);
			m_pathCache.put(pathReference, path);
		}
		
		if(path == null) throw new IdUnusedException(pathId);

		unlock(SECURE_ACCESS_DISSERTATION_CANDIDATEPATH, pathReference);
		
		// EventTrackingService.post(EventTrackingService.newEvent(SECURE_ACCESS_DISSERTATION_CANDIDATEPATH, path.getReference(), false));
		
		return path;
	}
	
	
	/**
	* Access all CandidatePath objects - known to us (not from external providers).
	* @return A list of CandidatePath objects.
	*/
	public List getCandidatePaths()
	{
		List paths = new Vector();

		// if we have disabled the cache, don't use if
		if (m_pathCache.disabled())
		{
			paths = m_pathStorage.getAll();
		}

		else
		{
			// if the cache is complete, use it
			if (m_pathCache.isComplete())
			{
				paths = m_pathCache.getAll();
			}
	
			// otherwise get all the paths from storage
			else
			{
				// Note: while we are getting from storage, storage might change.  These can be processed
				// after we get the storage entries, and put them in the cache, and mark the cache complete.
				// -ggolden
				synchronized (m_pathCache)
				{
					// if we were waiting and it's now complete...
					if (m_pathCache.isComplete())
					{
						paths = m_pathCache.getAll();
						return paths;
					}
	
					// save up any events to the cache until we get past this load
					m_pathCache.holdEvents();
	
					paths = m_pathStorage.getAll();
	
					// update the cache, and mark it complete
					for (int i = 0; i < paths.size(); i++)
					{
						CandidatePath path = (CandidatePath) paths.get(i);
						m_pathCache.put(path.getReference(), path);
					}
	
					m_pathCache.setComplete();
	
					// now we are complete, process any cached events
					m_pathCache.processEvents();
				}
			}
		}

		return paths;

	}   // getCandidatePaths
	
	
	/**
	* Get a locked CandidatePath object for editing.  Must commitEdit() to make official, or cancelEdit() when done!
	* @param pathReference The path reference string.
	* @return A CandidatePathEdit object for editing.
	* @exception IdUnusedException if not found, or if not an CandidatePathEdit object
	* @exception PermissionException if the current user does not have permission to edit this path.
	* @exception InUseException if the CandidatePath is being edited by another user.
	*/
	public CandidatePathEdit editCandidatePath(String pathReference)
		throws IdUnusedException, PermissionException, InUseException
	{
		unlock(SECURE_UPDATE_DISSERTATION_CANDIDATEPATH, pathReference);
		
		String pathId = pathId(pathReference);

		// check for existance
		if ((m_pathCache.get(pathReference) == null) && (!m_pathStorage.check(pathId)))
		{
			throw new IdUnusedException(pathId);
		}

		// ignore the cache - get the CandidatePath with a lock from the info store
		CandidatePathEdit path = m_pathStorage.edit(pathId);
		
		if (path == null) throw new InUseException(pathId);

		((BaseCandidatePathEdit) path).setEvent(SECURE_UPDATE_DISSERTATION_CANDIDATEPATH);

		return path;

	}   // editCandidatePath

	
	/**
	* Commit the changes made to a CandidatePathEdit object, and release the lock.
	* @param path The CandidatePathEdit object to commit.
	*/
	public void commitEdit(CandidatePathEdit path)
	{
		// check for closed edit
		if (!path.isActiveEdit())
		{
			try { throw new Exception(); }
			catch (Exception e) { m_logger.warn(this + ".commitEdit(): closed CandidatePathEdit", e); }
			return;
		}

		// update the properties
		addLiveUpdateProperties(path.getPropertiesEdit());

		// complete the edit
		m_pathStorage.commit(path);

		// track it
		EventTrackingService.post(
			EventTrackingService.newEvent(((BaseCandidatePathEdit) path).getEvent(), path.getReference(), true));

		// close the edit object
		((BaseCandidatePathEdit) path).closeEdit();

	}	// commitEdit(CandidatePath)

	
	/**
	* Cancel the changes made to a CandidatePathEdit object, and release the lock.
	* @param path The CandidatePathEdit object to commit.
	*/
	public void cancelEdit(CandidatePathEdit path)
	{
		// check for closed edit
		if (!path.isActiveEdit())
		{
			try { throw new Exception(); }
			catch (Exception e) { m_logger.warn(this + ".cancelEdit(): closed CandidatePathEdit", e); }
			return;
		}

		// release the edit lock
		m_pathStorage.cancel(path);

		// close the edit object
		((BaseCandidatePathEdit) path).closeEdit();

	}	// cancelEdit(Path)
	
	/** 
	* Removes an CandidatePath and all references to it
	* @param path - the CandidatePath to remove.
	* @throws PermissionException if current User does not have permission to do this.
	*/
	public void removeCandidatePath(CandidatePathEdit path)
		throws PermissionException
	{
		if(path != null)
		{
			if(!path.isActiveEdit())
			{
				try { throw new Exception(); }
				catch (Exception e) { m_logger.warn(this + ".removeCandidatePath(): closed CandidatePathEdit", e); }
				return;
			}

			unlock(SECURE_REMOVE_DISSERTATION_CANDIDATEPATH, path.getReference());

			m_pathStorage.remove(path);
			
			EventTrackingService.post(EventTrackingService.newEvent(SECURE_REMOVE_DISSERTATION_CANDIDATEPATH, path.getReference(), true));
			
			((BaseCandidatePathEdit)path).closeEdit();
		}
	}

	/** 
	* Adds an StepStatus to the service.
	* @param site - The site for which permissions are being checked.
	* @param step - The parent DissertationStep.
	* @param oardStep - Is this step auto-validated from Rackham's database?
	* @return The new StepStatus.
	* @throws PermissionException if the current User does not have permission to do this.
	*/
	public StepStatusEdit addStepStatus(String site, DissertationStep step, boolean oardStep)
		throws PermissionException
	{
		String statusId = null;
		boolean badId = false;
		
		do
		{
			badId = false;
			statusId = IdService.getUniqueId();
		
			try
			{
				Validator.checkResourceId(statusId);
			}
			catch(IdInvalidException iie)
			{
				badId = true;
			}

			if(m_statusStorage.check(statusId))
				badId = true;
			
		}while(badId);
		
		String key = statusReference(site, statusId);
		
		unlock(SECURE_ADD_DISSERTATION_STEPSTATUS, key);
		
		StepStatusEdit status = m_statusStorage.put(statusId, site);
		
		//m_logger.debug("DISSERTATION : BASE SERVICE : ADD STEP STATUS : STATUS OBJECT CREATED");
		status.initialize(site, step, oardStep);
		//m_logger.debug("DISSERTATION : BASE SERVICE : ADD STEP STATUS : STATUS OBJECT INITIALIZED");

		((BaseStepStatusEdit) status).setEvent(SECURE_ADD_DISSERTATION_STEPSTATUS);
		
		return status;
	}

	/** 
	* Adds an StepStatus to the service.
	* @param site - There is no current site - request comes from DissertationDataListenerService.
	* @param step - The parent DissertationStep.
	* @param oardStep - Is this step auto-validated from Rackham's database?
	* @return The new StepStatus.
	*/
	protected StepStatusEdit addStepStatusFromListener(String site, DissertationStep step, boolean oardStep)
	{
		String statusId = null;
		boolean badId = false;
		
		do
		{
			badId = false;
			statusId = IdService.getUniqueId();
		
			try
			{
				Validator.checkResourceId(statusId);
			}
			catch(IdInvalidException iie)
			{
				badId = true;
			}

			if(m_statusStorage.check(statusId))
				badId = true;
			
		}while(badId);
		
		String key = statusReference(site, statusId);
		
		StepStatusEdit status = m_statusStorage.put(statusId, site);
		
		//m_logger.debug("DISSERTATION : BASE SERVICE : ADD STEP STATUS : STATUS OBJECT CREATED");
		status.initialize(site, step, oardStep);
		//m_logger.debug("DISSERTATION : BASE SERVICE : ADD STEP STATUS : STATUS OBJECT INITIALIZED");

		((BaseStepStatusEdit) status).setEvent(SECURE_ADD_DISSERTATION_STEPSTATUS);
		
		return status;
	}
	
	/** 
	* Adds an StepStatus to the service.
	* @param site - The site for which permissions are being checked.
	* @return The new StepStatus.
	* @throws PermissionException if the current User does not have permission to do this.
	*/
	public StepStatusEdit addStepStatus(String site)
		throws PermissionException
	{
		String statusId = null;
		boolean badId = false;
		
		do
		{
			badId = false;
			statusId = IdService.getUniqueId();
		
			try
			{
				Validator.checkResourceId(statusId);
			}
			catch(IdInvalidException iie)
			{
				badId = true;
			}

			if(m_statusStorage.check(statusId))
				badId = true;
			
		}while(badId);
		
		String key = statusReference(site, statusId);
		
		unlock(SECURE_ADD_DISSERTATION_STEPSTATUS, key);

		StepStatusEdit status = m_statusStorage.put(statusId, site);
		
		status.setSite(site);
		
		((BaseStepStatusEdit) status).setEvent(SECURE_ADD_DISSERTATION_STEPSTATUS);
		
		return status;
	}


	/**
	* Add a new StepStatus to the directory, from a definition in XML.
	* Must commitEdit() to make official, or cancelEdit() when done!
	* @param el The XML DOM Element defining the StepStatus.
	* @return A locked StepStatusEdit object (reserving the id).
	* @exception IdInvalidException if the StepStatus id is invalid.
	* @exception IdUsedException if the StepStatus id is already used.
	* @exception PermissionException if the current user does not have permission to add a CandidatePath.
	*/
	public StepStatusEdit mergeStepStatus(Element el)
		throws IdInvalidException, IdUsedException, PermissionException
	{
		// construct from the XML
		StepStatus statusFromXml = new BaseStepStatus(el);

		// check for a valid StepStatus name
		Validator.checkResourceId(statusFromXml.getId());

		// check security (throws if not permitted)
		unlock(SECURE_ADD_DISSERTATION_STEPSTATUS, statusFromXml.getReference());

		// reserve a StepStatus with this id from the info store - if it's in use, this will return null
		StepStatusEdit status = m_statusStorage.put(statusFromXml.getId(), statusFromXml.getSite());
		if (status == null)
		{
			throw new IdUsedException(statusFromXml.getId());
		}

		// transfer from the XML read StepStatus object to the StepStatusEdit
		((BaseStepStatusEdit) status).set(statusFromXml);

		((BaseStepStatusEdit) status).setEvent(SECURE_ADD_DISSERTATION_STEPSTATUS);

		return status;

	}//mergeStepStatus

	/**
	* Access the StepStatus with the specified reference.
	* @param pathReference - The reference of the StepStatus.
	* @return The StepStatus corresponding to the reference, or null if it does not exist.
	* @throws IdUnusedException if there is no object with this id.
	* @throws PermissionException if the current user is not allowed to access this.
	*/
	public StepStatus getStepStatus(String statusReference)
		throws IdUnusedException, PermissionException
	{
		StepStatus status = null;
		
		String statusId = statusId(statusReference);
		
		if(m_statusCache.containsKey(statusReference))
			status = (StepStatus)m_statusCache.get(statusReference);
		else
		{
			status = m_statusStorage.get(statusId);
			m_statusCache.put(statusReference, status);
		}
		
		if(status == null) throw new IdUnusedException(statusId);

		unlock(SECURE_ACCESS_DISSERTATION_STEPSTATUS, statusReference);
		
		// EventTrackingService.post(EventTrackingService.newEvent(SECURE_ACCESS_DISSERTATION_STEPSTATUS, status.getReference(), false));
		
		return status;

	}//getStepStatus
	
	
	/**
	* Access all StepStatus objects - known to us (not from external providers).
	* @return A list of StepStatus objects.
	*/
	public List getStepStatus()
	{
		List statusi = new Vector();

		// if we have disabled the cache, don't use if
		if (m_statusCache.disabled())
		{
			statusi = m_statusStorage.getAll();
		}

		else
		{
			// if the cache is complete, use it
			if (m_statusCache.isComplete())
			{
				statusi = m_statusCache.getAll();
			}
	
			// otherwise get all the status from storage
			else
			{
				// Note: while we are getting from storage, storage might change.  These can be processed
				// after we get the storage entries, and put them in the cache, and mark the cache complete.
				// -ggolden
				synchronized (m_statusCache)
				{
					// if we were waiting and it's now complete...
					if (m_statusCache.isComplete())
					{
						statusi = m_statusCache.getAll();
						return statusi;
					}
	
					// save up any events to the cache until we get past this load
					m_statusCache.holdEvents();
	
					statusi = m_statusStorage.getAll();
	
					// update the cache, and mark it complete
					for (int i = 0; i < statusi.size(); i++)
					{
						StepStatus status = (StepStatus) statusi.get(i);
						m_statusCache.put(status.getReference(), status);
					}
	
					m_statusCache.setComplete();
	
					// now we are complete, process any cached events
					m_statusCache.processEvents();
				}
			}
		}

		return statusi;

	}   // getStepStatus
	
	
	/**
	* Get a locked StepStatus object for editing.  Must commitEdit() to make official, or cancelEdit() when done!
	* @param statusReference The StepStatus reference string.
	* @return An StepStatusEdit object for editing.
	* @exception IdUnusedException if not found, or if not an StepStatusEdit object
	* @exception PermissionException if the current user does not have permission to edit this StepStatus.
	* @exception InUseException if the StepStatus is being edited by another user.
	*/
	public StepStatusEdit editStepStatus(String statusReference)
		throws IdUnusedException, PermissionException, InUseException
	{
		// check security (throws if not permitted)
		unlock(SECURE_UPDATE_DISSERTATION_STEPSTATUS, statusReference);
		
		String statusId = statusId(statusReference);

		// check for existance
		if ((m_statusCache.get(statusReference) == null) && (!m_statusStorage.check(statusId)))
		{
			throw new IdUnusedException(statusId);
		}

		// ignore the cache - get the StepStatus with a lock from the info store
		StepStatusEdit status = m_statusStorage.edit(statusId);
		
		if (status == null) throw new InUseException(statusId);

		((BaseStepStatusEdit) status).setEvent(SECURE_UPDATE_DISSERTATION_STEPSTATUS);

		return status;

	}   // editStepStatus


	/**
	* Get a locked StepStatus object for editing.  Must commitEdit() to make official, or cancelEdit() when done!
	* @param statusReference The StepStatus reference string.
	* @return An StepStatusEdit object for editing.
	* @exception IdUnusedException if not found, or if not an StepStatusEdit object
	* @exception PermissionException if the current user does not have permission to edit this StepStatus.
	* @exception InUseException if the StepStatus is being edited by another user.
	*/
	protected StepStatusEdit editStepStatusFromListener(String statusReference)
		throws IdUnusedException, InUseException
	{
		String statusId = statusId(statusReference);

		// check for existance
		if ((m_statusCache.get(statusReference) == null) && (!m_statusStorage.check(statusId)))
		{
			throw new IdUnusedException(statusId);
		}

		// ignore the cache - get the StepStatus with a lock from the info store
		StepStatusEdit status = m_statusStorage.edit(statusId);
		
		if (status == null) throw new InUseException(statusId);

		((BaseStepStatusEdit) status).setEvent(SECURE_UPDATE_DISSERTATION_STEPSTATUS);

		return status;

	}   // editStepStatusFromListener
	
	
	/**
	* Commit the changes made to an StepStatusEdit object, and release the lock.
	* @param status The StepStatusEdit object to commit.
	*/
	public void commitEdit(StepStatusEdit status)
	{
		// check for closed edit
		if (!status.isActiveEdit())
		{
			try { throw new Exception(); }
			catch (Exception e) { m_logger.warn(this + ".commitEdit(): closed StepStatusEdit", e); }
			return;
		}

		// update the properties
		addLiveUpdateProperties(status.getPropertiesEdit());

		// complete the edit
		m_statusStorage.commit(status);

		// track it
		EventTrackingService.post(
			EventTrackingService.newEvent(((BaseStepStatusEdit) status).getEvent(), status.getReference(), true));

		// close the edit object
		((BaseStepStatusEdit) status).closeEdit();

	}	// commitEdit

	
	/**
	* Cancel the changes made to a StepStatusEdit object, and release the lock.
	* @param status The StepStatusEdit object to commit.
	*/
	public void cancelEdit(StepStatusEdit status)
	{
		// check for closed edit
		if (!status.isActiveEdit())
		{
			try { throw new Exception(); }
			catch (Exception e) { m_logger.warn(this + ".cancelEdit(): closed StepStatusEdit", e); }
			return;
		}

		// release the edit lock
		m_statusStorage.cancel(status);

		// close the edit object
		((BaseStepStatusEdit) status).closeEdit();

	}	// cancelEdit(StepStatus)

	/** 
	* Removes a StepStatus and all references to it
	* @param status - the StepStatus to remove.
	* @throws PermissionException if current User does not have permission to do this.
	*/
	public void removeStepStatus(StepStatusEdit status)
		throws PermissionException
	{
		if(status != null)
		{
			if(!status.isActiveEdit())
			{
				try { throw new Exception(); }
				catch (Exception e) { m_logger.warn(this + ".removeStepStatus(): closed StepStatusEdit", e); }
				return;
			}

			unlock(SECURE_REMOVE_DISSERTATION_STEPSTATUS, status.getReference());

			m_statusStorage.remove(status);
			
			EventTrackingService.post(EventTrackingService.newEvent(SECURE_REMOVE_DISSERTATION_STEPSTATUS, status.getReference(), true));

			((BaseStepStatusEdit)status).closeEdit();		
		}

	}//removeStepStatus

	/** 
	* Adds an CandidateInfo to the service.
	* @param site - The site for which permissions are being checked.
	* @return The new CandidateInfo.
	* @throws PermissionException if the current User does not have permission to do this.
	*/
	public CandidateInfoEdit addCandidateInfo(String site)
		throws PermissionException
	{
		//m_logger.debug("DISSERTATION : BASE SERVICE : ENTERING ADD CANDIDATE INFO");
		String infoId = null;
		boolean badId = false;
		
		do
		{
			badId = false;
			infoId = IdService.getUniqueId();
			try
			{
				Validator.checkResourceId(infoId);
			}
			catch(IdInvalidException iie)
			{
				badId = true;
			}

			if(m_infoStorage.check(infoId))
				badId = true;

		}while(badId);
		
		//m_logger.debug("DISSERTATION : BASE SERVICE : ADD CANDIDATE INFO : GOT AN ID : " + infoId);
		
		String key = infoReference(site, infoId);

		//m_logger.debug("DISSERTATION : BASE SERVICE : ADD CANDIDATE INFO : REFERENCE : " + key);

		unlock(SECURE_ADD_DISSERTATION_CANDIDATEINFO, key);

		//m_logger.debug("DISSERTATION : BASE SERVICE : ADD CANDIDATE INFO : UNLOCKED ADD");
		
		CandidateInfoEdit info = m_infoStorage.put(infoId, site);
		
		//m_logger.debug("DISSERTATION : BASE SERVICE : ADD CANDIDATE INFO : CREATED OBJECT");
	
		info.setSite(site);
		
		//m_logger.debug("DISSERTATION : BASE SERVICE : ADD CANDIDATE INFO : SET SITE");
		
		try
		{
			((BaseCandidateInfoEdit) info).setEvent(SECURE_ADD_DISSERTATION_CANDIDATEINFO);
		}
		catch(Exception e)
		{
			m_logger.warn("DISSERTATION : BASE SERVICE : ADD CANDIDATE INFO : EXCEPTION TRACKING EVENT : " + e);
		}
		
		//m_logger.debug("DISSERTATION : BASE SERVICE : ADD CANDIDATE INFO : TRACKED EVENT");
		
		return info;
	}

	/** 
	* Adds an CandidateInfo to the service.
	* @param site - There is no current site - request comes from DissertationDataListenerService.
	* @return The new CandidateInfo.
	*/
	public CandidateInfoEdit addCandidateInfoFromListener(String site)
	{
		String infoId = null;
		boolean badId = false;
		
		do
		{
			badId = false;
			infoId = IdService.getUniqueId();
			try
			{
				Validator.checkResourceId(infoId);
			}
			catch(IdInvalidException iie)
			{
				badId = true;
			}

			if(m_infoStorage.check(infoId))
				badId = true;

		}while(badId);
		
		String key = infoReference(site, infoId);
		
		CandidateInfoEdit info = m_infoStorage.put(infoId, site);
		
		info.setSite(site);
		
		((BaseCandidateInfoEdit) info).setEvent(SECURE_ADD_DISSERTATION_CANDIDATEINFO);
		
		return info;
	}
	
	
	/**
	* Add a new CandidateInfo to the directory, from a definition in XML.
	* Must commitEdit() to make official, or cancelEdit() when done!
	* @param el The XML DOM Element defining the CandidateInfo.
	* @return A locked CandidateInfoEdit object (reserving the id).
	* @exception IdInvalidException if the CandidateInfo id is invalid.
	* @exception IdUsedException if the CandidateInfo id is already used.
	* @exception PermissionException if the current user does not have permission to add a CandidateInfo.
	*/
	public CandidateInfoEdit mergeCandidateInfo(Element el)
		throws IdInvalidException, IdUsedException, PermissionException
	{
		// construct from the XML
		CandidateInfo infoFromXml = new BaseCandidateInfo(el);

		// check for a valid CandidateInfo name
		Validator.checkResourceId(infoFromXml.getId());

		// check security (throws if not permitted)
		unlock(SECURE_ADD_DISSERTATION_CANDIDATEINFO, infoFromXml.getReference());

		// reserve a CandidateInfo with this id from the info store - if it's in use, this will return null
		CandidateInfoEdit info = m_infoStorage.put(infoFromXml.getId(), infoFromXml.getSite());
		if (info == null)
		{
			throw new IdUsedException(infoFromXml.getId());
		}

		// transfer from the XML read CandidateInfo object to the CandidateInfoEdit
		((BaseCandidateInfoEdit) info).set(infoFromXml);

		((BaseCandidateInfoEdit) info).setEvent(SECURE_ADD_DISSERTATION_CANDIDATEINFO);

		return info;

	}//mergeCandidateInfo

	/**
	* Access the CandidateInfo with the specified reference.
	* @param infoReference - The reference of the CandidateInfo.
	* @return The CandidateInfo corresponding to the reference, or null if it does not exist.
	* @throws IdUnusedException if there is no object with this reference.
	* @throws PermissionException if the current user is not allowed to access this.
	*/
	public CandidateInfo getCandidateInfo(String infoReference)
		throws IdUnusedException, PermissionException
	{
		CandidateInfo info = null;
		
		String infoId = infoId(infoReference);
		
		if(m_infoCache.containsKey(infoReference))
			info = (CandidateInfo)m_infoCache.get(infoReference);
		else
		{
			info = m_infoStorage.get(infoId);
			m_infoCache.put(infoReference, info);
		}
		
		if(info == null) throw new IdUnusedException(infoId);

		unlock(SECURE_ACCESS_DISSERTATION_CANDIDATEINFO, infoReference);

		// EventTrackingService.post(EventTrackingService.newEvent(SECURE_ACCESS_DISSERTATION_CANDIDATEINFO, info.getReference(), false));
		
		return info;

	}//getCandidateInfo

	/**
	* Access the CandidateInfo for the specified user.
	* @param emplid - The emplid of the user.
	* @return The CandidateInfo corresponding to the emplid, or null if it does not exist.
	* @throws PermissionException if the current user is not allowed to access this.
	*/
	public CandidateInfoEdit getCandidateInfoEditForEmplid(String emplid)
		throws PermissionException
	{
		//m_logger.debug("DISSERTATION : BASE SERVICE : GET INFO EDIT FOR EMPLID : " + emplid);
		CandidateInfoEdit retVal = null;
		
		if(emplid != null)
		{
			CandidateInfo tempInfo = null;
			List allInfos = getCandidateInfos();
			//m_logger.debug("DISSERTATION : BASE SERVICE : GET INFO EDIT FOR EMPLID : GOT ALL CANDIDATE INFOS :  #  : " + allInfos.size());
			for(int x = 0; x < allInfos.size(); x++)
			{
				try
				{
					tempInfo = (CandidateInfo)allInfos.get(x);

					if(tempInfo.getEmplid().equals(emplid))
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : GET INFO EDIT FOR EMPLID : FOUND A MATCH : " + tempInfo.getEmplid());
						//m_logger.debug("DISSERTATION : BASE SERVICE : GET INFO EDIT FOR EMPLID : GETTING INFO EDIT WITH REFERENCE : " + tempInfo.getReference());
						retVal = m_infoStorage.edit(tempInfo.getId());
						//m_logger.debug("DISSERTATION : BASE SERVICE : GET INFO EDIT FOR EMPLID : GOT INFO EDIT");
					}
				}
				catch(Exception e)
				{
					m_logger.warn("DISSERTATION : BASE SERVICE : GET INFO EDIT FOR EMPLID : EXCEPTION " + e);
				}
			}

			if(retVal != null)
			{
				// EventTrackingService.post(EventTrackingService.newEvent(SECURE_UPDATE_DISSERTATION_CANDIDATEINFO, retVal.getReference(), false));
			}
		}
		
		return retVal;

	}//getCandidateInfoEditForEmplid	
	
	/**
	* Access all CandidateInfo objects - known to us (not from external providers).
	* @return A list of CandidateInfo objects.
	*/
	protected List getCandidateInfos()
	{
		List infos = new Vector();

		// if we have disabled the cache, don't use if
		if (m_infoCache.disabled())
		{
			infos = m_infoStorage.getAll();
		}

		else
		{
			// if the cache is complete, use it
			if (m_infoCache.isComplete())
			{
				infos = m_infoCache.getAll();
			}
	
			// otherwise get all the infos from storage
			else
			{
				// Note: while we are getting from storage, storage might change.  These can be processed
				// after we get the storage entries, and put them in the cache, and mark the cache complete.
				// -ggolden
				synchronized (m_infoCache)
				{
					// if we were waiting and it's now complete...
					if (m_infoCache.isComplete())
					{
						infos = m_infoCache.getAll();
						return infos;
					}
	
					// save up any events to the cache until we get past this load
					m_infoCache.holdEvents();
	
					infos = m_infoStorage.getAll();
	
					// update the cache, and mark it complete
					for (int i = 0; i < infos.size(); i++)
					{
						CandidateInfo info = (CandidateInfo) infos.get(i);
						m_infoCache.put(info.getReference(), info);
					}
	
					m_infoCache.setComplete();
	
					// now we are complete, process any cached events
					m_infoCache.processEvents();
				}
			}
		}

		return infos;

	}   // getCandidateInfos
	
	
	/**
	* Get a locked CandidateInfo object for editing.  Must commitEdit() to make official, or cancelEdit() when done!
	* @param infoReference The CandidateInfo reference string.
	* @return An CandidateInfoEdit object for editing.
	* @exception IdUnusedException if not found, or if not an CandidateInfoEdit object
	* @exception PermissionException if the current user does not have permission to edit this CandidateInfo.
	* @exception InUseException if the CandidateInfo is being edited by another user.
	*/
	public CandidateInfoEdit editCandidateInfo(String infoReference)
		throws IdUnusedException, PermissionException, InUseException
	{
		// check security (throws if not permitted)
		unlock(SECURE_UPDATE_DISSERTATION_CANDIDATEINFO, infoReference);
		
		String infoId = infoId(infoReference);

		// check for existance
		if ((m_infoCache.get(infoReference) == null) && (!m_infoStorage.check(infoId)))
		{
			throw new IdUnusedException(infoId);
		}

		// ignore the cache - get the CandidateInfo with a lock from the info store
		CandidateInfoEdit info = m_infoStorage.edit(infoId);
		
		if (info == null) throw new InUseException(infoId);

		((BaseCandidateInfoEdit) info).setEvent(SECURE_UPDATE_DISSERTATION_CANDIDATEINFO);

		return info;

	}   // editCandidateInfo


	/**
	* Commit the changes made to an CandidateInfoEdit object, and release the lock.
	* @param info The CandidateInfoEdit object to commit.
	*/
	public void commitEdit(CandidateInfoEdit info)
	{
		// check for closed edit
		if (!info.isActiveEdit())
		{
			try { throw new Exception(); }
			catch (Exception e) { m_logger.warn(this + ".commitEdit(): closed CandidateInfoEdit", e); }
			return;
		}

		// update the properties
		addLiveUpdateProperties(info.getPropertiesEdit());

		//m_logger.debug("DISSERTATION : BASE SERVICE : COMMIT EDIT(INFO) : ABOUT TO COMMIT EDIT");
		// complete the edit
		m_infoStorage.commit(info);
		//m_logger.debug("DISSERTATION : BASE SERVICE : COMMIT EDIT(INFO) : COMMITED EDIT");
		// track it
		EventTrackingService.post(
			EventTrackingService.newEvent(((BaseCandidateInfoEdit) info).getEvent(), info.getReference(), true));

		// close the edit object
		((BaseCandidateInfoEdit) info).closeEdit();

	}	// commitEdit

	
	/**
	* Cancel the changes made to a CandidateInfoEdit object, and release the lock.
	* @param info The CandidateInfoEdit object to commit.
	*/
	public void cancelEdit(CandidateInfoEdit info)
	{
		// check for closed edit
		if (!info.isActiveEdit())
		{
			try { throw new Exception(); }
			catch (Exception e) { m_logger.warn(this + ".cancelEdit(): closed CandidateInfoEdit", e); }
			return;
		}

		// release the edit lock
		m_infoStorage.cancel(info);

		// close the edit object
		((BaseCandidateInfoEdit) info).closeEdit();

	}	// cancelEdit(CandidateInfo)

	/** 
	* Removes an CandidateInfo and all references to it
	* @param info - the CandidateInfo to remove.
	* @throws PermissionException if current User does not have permission to do this.
	*/
	public void removeCandidateInfo(CandidateInfoEdit info)
		throws PermissionException
	{
		if(info != null)
		{
			if(!info.isActiveEdit())
			{
				try { throw new Exception(); }
				catch (Exception e) { m_logger.warn(this + ".removeCandidateInfo(): closed CandidateInfoEdit", e); }
				return;
			}

			unlock(SECURE_REMOVE_DISSERTATION_CANDIDATEINFO, info.getReference());

			m_infoStorage.remove(info);
			
			EventTrackingService.post(EventTrackingService.newEvent(SECURE_REMOVE_DISSERTATION_CANDIDATEINFO, info.getReference(), true));

			((BaseCandidateInfoEdit)info).closeEdit();		
		}

	}//removeCandidateInfo


	public List getDissertationStepsForSite(String site)
		throws IdUnusedException, PermissionException
	{
		//m_logger.debug("DISSERTATION : BASE SERVICE : GET STEPS FOR SITE : SITE : " + site);
		List retVal = new ArrayList();
		DissertationStep tempStep = null;
		if(site != null)
		{
			List allSteps = getDissertationSteps();
			for(int x = 0; x < allSteps.size(); x++)
			{
				tempStep = (DissertationStep)allSteps.get(x);
				if(tempStep.getSite().equals(site))
				{
					retVal.add(tempStep);
					//m_logger.debug("DISSERTATION : BASE SERVICE : GET STEPS FOR SITE : ADDING STEP : " + tempStep.getId());
					//m_logger.debug("DISSERTATION : BASE SERVICE : GET STEPS FOR SITE : STEP'S DEPT : " + tempStep.getSite());
				}
			}
		}

		return retVal;
	}

	/** 
	* Get the List of CandidatePaths assiciated with a parent (department) site.
	* @param site - The site id.
	* @return The List of CandidatePath objects.
	*/
	public List getCandidatePathsForParentSite(String site)
	{
		List retVal = new ArrayList();
		//m_logger.debug("DISSERTATION : BASE SERVICE : GET CANDIDATE PATHS FOR PARENT SITE : " + site);
		if(site != null)
		{
			CandidatePath cp = null;
			List allPaths = getCandidatePaths();
			for(int x = 0; x < allPaths.size(); x++)
			{
				cp = (CandidatePath)allPaths.get(x);
				//m_logger.debug("DISSERTATION : BASE SERVICE : FOUND A CANDIDATE PATH WITH SITE : " + cp.getSite());
				if(site.equals(getParentSiteForUser(cp.getCandidate())))
				{
					//m_logger.debug("DISSERTATION : BASE SERVICE : GET CANDIDATE PATHS FOR DEPT : FOUND AN ACTIVE PATH FOR THIS SITE");
					retVal.add(cp);
				}
			}
		}
		return retVal;
	}
	
	/**
	* Access the CandidatePath for the specified candidate.
	* @param candidateId The CHEF user id of the candidate.
	* @return The CandidatePath corresponding to the candidate, or null if it does not exist.
	* @throws PermissionException if the current user is not allowed to access this.
	*/
	public CandidatePath getCandidatePathForCandidate(String candidateId)
		throws PermissionException
	{
		CandidatePath retVal = null;
		CandidatePath tempPath = null;
		if(candidateId != null)
		{
			List allPaths = getCandidatePaths();
			for(int x = 0; x < allPaths.size(); x++)
			{
				tempPath = (CandidatePath)allPaths.get(x);
				if(candidateId.equals(tempPath.getCandidate()))
					retVal = tempPath;
			}
		}

		return retVal;
	}

	/**
	* Access the CandidatePath for the specified site.
	* @param siteId The CHEF site id.
	* @return The CandidatePath corresponding to the site, or null if it does not exist.
	* @throws PermissionException if the current user is not allowed to access this.
	*/
	public CandidatePath getCandidatePathForSite(String siteId)
		throws PermissionException
	{
		CandidatePath retVal = null;
		CandidatePath tempPath = null;
		if(siteId != null)
		{
			List allPaths = getCandidatePaths();
			for(int x = 0; x < allPaths.size(); x++)
			{
				tempPath = (CandidatePath)allPaths.get(x);
				if(siteId.equals(tempPath.getSite()))
					retVal = tempPath;
			}
		}

		return retVal;
	}
	
	/**
	* Access the Dissertation associated with pedagogical approach of Music Performance or otherwise.
	* @param site - The site id.
	* @param type - The pedagogical approach inherent in the steps.
	* @return Dissertation associated with a site and type, or null if one does not exist.
	*/
	public Dissertation getDissertationForSite(String site, String type)
		throws IdUnusedException, PermissionException
	{
		Dissertation retVal = null;
		Dissertation tempDiss = null;
		if(site != null)
		{
			try
			{
				List allDissertations = getDissertations();
				for(int x = 0; x < allDissertations.size(); x++)
				{
					tempDiss = (Dissertation)allDissertations.get(x);
					if(site.equals(tempDiss.getSite()))
					{
						String tempType = tempDiss.getType();
						if(tempType!=null && !tempType.equals(""))
						{
							if(type.equals(DissertationService.DISSERTATION_TYPE_MUSIC_PERFORMANCE) && (tempType.equals(DissertationService.DISSERTATION_TYPE_MUSIC_PERFORMANCE)))
							{
								retVal = tempDiss;
							}
							else if (type.equals(DissertationService.DISSERTATION_TYPE_DISSERTATION_STEPS) && (tempType.equals(DissertationService.DISSERTATION_TYPE_DISSERTATION_STEPS)))
							{
								retVal = tempDiss;
							}
						}
					}
						
				}
			}
			catch(Exception e)
			{
				m_logger.warn("DISSERTATION : BASE SERVICE : GET DISSERTATION FOR SITE : EXCEPTION : " + e);
			}
		}
		return retVal;
		
	}//getDissertationForSite
	
	/**
	* Access the Dissertation associated with a site.
	* @param site - The site id.
	* @return Dissertation associated with a site, or null if one does not exist.
	*/
	public Dissertation getDissertationForSite(String site)
		throws IdUnusedException, PermissionException
	{
		Dissertation retVal = null;
		Dissertation tempDiss = null;
		if(site != null)
		{
			try
			{
				List allDissertations = getDissertations();
				for(int x = 0; x < allDissertations.size(); x++)
				{
					tempDiss = (Dissertation)allDissertations.get(x);
					if(site.equals(tempDiss.getSite()))
						retVal = tempDiss;
				}
			}
			catch(Exception e)
			{
				m_logger.warn("DISSERTATION : BASE SERVICE : GET DISSERTATION FOR SITE : EXCEPTION : " + e);
			}
		}

		return retVal;
	}
	
	/** 
	* Access whether any active CandidatePaths exist.
	* @return True if a CandidateInfo exists, false if not.
	*/
	public boolean getActivePaths()
	{
		boolean retVal = false;
		List keys = m_pathCache.getKeys();
		if(keys.size() > 0)
			retVal = true;
		else
		{
			List paths = m_pathStorage.getAll();
			if(paths.size() > 0)
				retVal = true;
		}
		return retVal;
	}
	
	/** 
	* Access CandidateInfo for a candidate.
	* @param candidate The candidate's Chef id.
	* @return CandidateInfo of the specified candidate or null if not found.
	*/
	public CandidateInfo getCandidateInfoForCandidate(String candidate)
	{
		CandidateInfo retVal = null;
		List infos = m_infoStorage.getAll();
		if(infos.size() > 0)
		{
			for (int i=0; i<infos.size(); i++)
			{
				CandidateInfo tempInfo = (CandidateInfo)infos.get(i);
				if(((String)tempInfo.getChefId()).equals(candidate))
				{
					retVal = tempInfo;
					break;
				}
			}
		}
		return retVal;
	}
	
	/** 
	* Access whether a department site has any active CandidatePaths associated with it.
	* @param site The site id.
	* @return True if a CandidateInfo exists for this parent site, false if not.
	*/
	public boolean getActivePathsForSite(String site)
	{
		if(site != null)
		{
			//m_logger.debug("DISSERTATION : BASE SERVICE : GET ACTIVE PATHS FOR SITE : " + site);
			CandidateInfo info = null;
			CandidatePath path = null;
			String candidateId = null;
			List allPaths = getCandidatePaths();
			for(int x = 0; x < allPaths.size(); x++)
			{
				path = (CandidatePath)allPaths.get(x);
				candidateId = path.getCandidate();
				//m_logger.debug("DISSERTATION : BASE SERVICE : GET ACTIVE PATHS FOR SITE : A CANDIDATE ID : " + candidateId);
				info = getCandidateInfoForCandidate(candidateId);
				//m_logger.debug("DISSERTATION : BASE SERVICE : GET ACTIVE PATHS FOR SITE : CANDIDATE'S PARENT GROUP FROM INFO : " + info.getParentSiteId());
				if(site.equals(info.getParentSiteId()))
				{
					//m_logger.debug("DISSERTATION : BASE SERVICE : GET ACTIVE PATHS FOR SITE : FOUND AN ACTIVE PATH FOR THIS SITE");
					return true;
				}
			}
		}

		return false;
	}

	/**
	* Access the CandidateInfo for the candidate.
	* @param id - The CHEF user id of the candidate.
	* @return The CandidateInfo for the candidate, or null if it does not exist.
	*/
	public CandidateInfo getInfoForCandidate(String id)
	{
		CandidateInfo retVal = null;
		if(id != null)
		{
			String candidateId = null;
			CandidateInfo info = null;
			List allInfos = getCandidateInfos();
			for(int x = 0; x < allInfos.size(); x++)
			{
				info = (CandidateInfo)allInfos.get(x);
				if(id.equals(info.getChefId()))
					retVal = info;
			}
		}
		return retVal;
	}
	
	/** 
	* Access the site id of the Rackham site.
	* @return The school site id.
	*/
	public String getSchoolSite()
	{
		return m_schoolSite;
	}
	
	/** 
	* Access the site id of the Music Performance site.
	* @return The Music Performance site id.
	*/
	public String getMusicPerformanceSite()
	{
		return m_musicPerformanceSite;
	}

	/**
	* Return the reference root for use in resource references and urls.
	* @return The reference root for use in resource references and urls.
	*/
	protected String getReferenceRoot()
	{
		return REFERENCE_ROOT;
	}

	/**
	* Update the live properties for an object when modified.
	*/
	protected void addLiveUpdateProperties(ResourcePropertiesEdit props)
	{
		props.addProperty(ResourceProperties.PROP_MODIFIED_BY,
				UsageSessionService.getSessionUser().getId());

		props.addProperty(ResourceProperties.PROP_MODIFIED_DATE,
			TimeService.newTime().toString());

	}	//  addLiveUpdateProperties

	/**
	* Create the live properties for the object.
	*/
	protected void addLiveProperties(ResourcePropertiesEdit props)
	{
		String current = UsageSessionService.getSessionUser().getId();
		props.addProperty(ResourceProperties.PROP_CREATOR, current);
		props.addProperty(ResourceProperties.PROP_MODIFIED_BY, current);
		
		String now = TimeService.newTime().toString();
		props.addProperty(ResourceProperties.PROP_CREATION_DATE, now);
		props.addProperty(ResourceProperties.PROP_MODIFIED_DATE, now);

	}	//  addLiveProperties
	
	/**
	* Check permissions for adding a BlockGrantGroup
	* @param site - the site id for which permissions are to be checked.
	* @return True if the current User is allowed to add a BlockGrantGroup to site, false if not.
	*/
	public boolean allowAddBlockGrantGroup(String site)
	{
		if(m_logger.isDebugEnabled())
		{
			m_logger.debug("Entering allow add BlockGrantGroup with resource string : " + site);
		}
		return unlockCheck(SECURE_ADD_DISSERTATION_GROUP, site);
	}
	
	/**
	* Check permissions for accessing the BlockGrantGroup
	* @param bggReference - The BlockGrantGroup's reference.
	* @return True if the current User is allowed to remove the BlockGrantGroup, false if not.
	*/
	public boolean allowGetBlockGrantGroup(String bggReference)
	{
		if(m_logger.isDebugEnabled())
		{
			m_logger.debug("Entering allow access BlockGrantGroup with resource string : " + bggReference);
		}
		return unlockCheck(SECURE_ACCESS_DISSERTATION_GROUP, bggReference);
	}
	
	/**
	* Check permissions for removing the BlockGrantGroup.
	* @param bggReference - The BlockGrantGroup's reference.
	* @return True if the current User is allowed to remove the BlockGrantGroup, false if not.
	*/
	public boolean allowRemoveBlockGrantGroup(String bggReference)
	{
		if(m_logger.isDebugEnabled())
		{
			m_logger.debug("Entering allow remove BlockGrantGroup with resource string : " + bggReference);
		}
		return unlockCheck(SECURE_REMOVE_DISSERTATION_GROUP, bggReference);
	}
	
	/**
	* Check permissions for removing the CandidatePath
	* @param pathReference - The CandidatePath's reference.
	* @return True if the current User is allowed to remove the CandidatePath, false if not.
	*/
	public boolean allowUpdateBlockGrantGroup(String bggReference)
	{
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow remove CandidatePath with resource string : " + pathReference);
		}
		return unlockCheck(SECURE_UPDATE_DISSERTATION_GROUP, bggReference);
	}
	
	/**
	* Check permissions for adding an Dissertation.
	* @param site - the site id.
	* @return True if the current User is allowed to add an Dissertation, false if not.
	*/
	public boolean allowAddDissertation(String site)
	{
			// check security (throws if not permitted)
		String resourceString = dissertationReference(site, "");
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow add Dissertation with resource string : " + resourceString);
		}
		return unlockCheck(SECURE_ADD_DISSERTATION, resourceString);
	}

	/**
	* Check permissions for accessing a Dissertation.
	* @param dissertationReference - The Dissertation's reference.
	* @return True if the current User is allowed to access the Dissertation, false if not.
	*/
	public boolean allowGetDissertation(String dissertationReference)
	{
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow get Dissertation with resource string : " + dissertationReference);
		}
		return unlockCheck(SECURE_ACCESS_DISSERTATION, dissertationReference);
	}

	/**
	* Check permissions for updating a Dissertation.
	* @param dissertationReference - The Dissertation's reference.
	* @return True if the current User is allowed to update the Dissertation, false if not.
	*/
	public boolean allowUpdateDissertation(String dissertationReference)
	{
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow update Dissertation with resource string : " + dissertationReference);
		}
		return unlockCheck(SECURE_UPDATE_DISSERTATION, dissertationReference);
	}

	/**
	* Check permissions for removing a Dissertation.
	* @param dissertationReference - The Dissertation's reference.
	* @return True if the current User is allowed to remove the Dissertation, false if not.
	*/
	public boolean allowRemoveDissertation(String dissertationReference)
	{
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow remove Dissertation with resource string : " + dissertationReference);
		}
		return unlockCheck(SECURE_REMOVE_DISSERTATION, dissertationReference);
	}

	/**
	* Check permissions for add CandidatePath
	* @param site - The site for which permissions are being checked.
	* @return True if the current User is allowed to add an CandidatePath, false if not.
	*/
	public boolean allowAddCandidatePath(String site)
	{
		String resourceString = pathReference(site, "");
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow add CandidatePath with resource string : " + resourceString);
		}
		return unlockCheck(SECURE_ADD_DISSERTATION_CANDIDATEPATH, resourceString);
	}

	/**
	* Check permissions for accessing a CandidatePath.
	* @param pathReference - The CandidatePath's reference.
	* @return True if the current User is allowed to get the CandidatePath, false if not.
	*/
	public boolean allowGetCandidatePath(String pathReference)
	{
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow get CandidatePath with resource string : " + pathReference);
		}
		return unlockCheck(SECURE_ACCESS_DISSERTATION_CANDIDATEPATH, pathReference);
	}

	/**
	* Check permissions for updating CandidatePath.
	* @param pathReference - The CandidatePath's reference.
	* @return True if the current User is allowed to update the CandidatePath, false if not.
	*/
	public boolean allowUpdateCandidatePath(String pathReference)
	{
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow update CandidatePath with resource string : " + pathReference);
		}
		return unlockCheck(SECURE_UPDATE_DISSERTATION_CANDIDATEPATH, pathReference);
	}

	/**
	* Check permissions for updating CandidatePath for Committee Members.
	* @param pathReference - The CandidatePath's reference.
	* @return True if the current User is allowed to update the CandidatePath, false if not.
	*/
	public boolean allowUpdateCandidatePathComm(String pathReference)
	{
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow update CandidatePathComm with resource string : " + pathReference);
		}
		return unlockCheck(SECURE_UPDATE_DISSERTATION_CANDIDATEPATH_COMM, pathReference);
	}
	
	/**
	* Check permissions for removing the CandidatePath
	* @param pathReference - The CandidatePath's reference.
	* @return True if the current User is allowed to remove the CandidatePath, false if not.
	*/
	public boolean allowRemoveCandidatePath(String pathReference)
	{
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow remove CandidatePath with resource string : " + pathReference);
		}
		return unlockCheck(SECURE_REMOVE_DISSERTATION_CANDIDATEPATH, pathReference);
	}

	/**
	* Check permissions for get DissertationStep
	* @param stepReference - The DissertationStep reference.
	* @return True if the current User is allowed to get the DissertationStep, false if not.
	*/
	public boolean allowGetDissertationStep(String stepReference)
	{
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow get DissertationStep with resource string : " + stepReference);
		}
		return unlockCheck(SECURE_ACCESS_DISSERTATION_STEP, stepReference);
	}

	/**
	* Check permissions for updating DissertationStep
	* @param stepReference - The DissertationStep reference.
	* @return True if the current User is allowed to update the DissertationStep, false if not.
	*/
	public boolean allowUpdateDissertationStep(String stepReference)
	{
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow update DissertationStep with resource string : " + stepReference);
		}
		return unlockCheck(SECURE_UPDATE_DISSERTATION_STEP, stepReference);
	}

	/**
	* Check permissions for adding a DissertationStep.
	* @param site - The site for which permissions are being checked.
	* @return True if the current User is allowed to add a DissertationStep, false if not.
	*/
	public boolean allowAddDissertationStep(String site)
	{
		String resourceString = stepReference(site, "");
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow add DissertationStep with resource string : " + resourceString);
			//m_logger.debug("                                           site string : " + site);
		}
		return unlockCheck(SECURE_ADD_DISSERTATION_STEP, resourceString);
	}
	
	/**
	* Check permissions for removing Dissertation step
	* @param stepReference - The DissertationStep reference.
	* @return True if the current User is allowed to remove the DissertationStep, false if not.
	*/
	public boolean allowRemoveDissertationStep(String stepReference)
	{
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow remove dissertation step with resource string : " + stepReference);
		}
		return unlockCheck(SECURE_REMOVE_DISSERTATION_STEP, stepReference);
	}

	/**
	* Check permissions for accessing a StepStatus.
	* @param statusReference - The StepStatus's reference.
	* @return True if the current User is allowed to get the StepStatus, false if not.
	*/
	public boolean allowGetStepStatus(String statusReference)
	{
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow get StepStatus with resource string : " + statusReference);
		}
		return unlockCheck(SECURE_ACCESS_DISSERTATION_STEP, statusReference);
	}

	/**
	* Check permissions for updating StepStatus.
	* @param statusReference - The StepStatus's reference.
	* @return True if the current User is allowed to update the StepStatus, false if not.
	*/
	public boolean allowUpdateStepStatus(String statusReference)
	{
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow update StepStatus with resource string : " + statusReference);
		}
		return unlockCheck(SECURE_UPDATE_DISSERTATION_STEP, statusReference);
	}

	/**
	* Check permissions for add StepStatus
	* @param site - The site for which permissions are being checked.
	* @return True if the current User is allowed to add an StepStatus, false if not.
	*/
	public boolean allowAddStepStatus(String site)
	{
		String resourceString = statusReference(site, "");
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow add StepStatus with resource string : " + resourceString);
		}
		return unlockCheck(SECURE_ADD_DISSERTATION_STEP, resourceString);
	}
	
	/**
	* Check permissions for removing the StepStatus
	* @param statusReference - The StepStatus's reference.
	* @return True if the current User is allowed to remove the StepStatus, false if not.
	*/
	public boolean allowRemoveStepStatus(String statusReference)
	{
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow remove dissertation step with resource string : " + statusReference);
		}
		return unlockCheck(SECURE_REMOVE_DISSERTATION_STEP, statusReference);
	}

	/**
	* Check permissions for add CandidateInfo
	* @param site - The site for which permissions are being checked.
	* @return True if the current User is allowed to add an CandidateInfo, false if not.
	*/
	public boolean allowAddCandidateInfo(String site)
	{
		String resourceString = infoReference(site, "");
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow add CandidateInfo with resource string : " + resourceString);
		}
		return unlockCheck(SECURE_ADD_DISSERTATION_CANDIDATEINFO, resourceString);
	}

	/**
	* Check permissions for accessing a CandidateInfo.
	* @param infoReference - The CandidateInfo's reference.
	* @return True if the current User is allowed to get the CandidateInfo, false if not.
	*/
	public boolean allowGetCandidateInfo(String infoReference)
	{
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow get CandidateInfo with resource string : " + infoReference);
		}
		return unlockCheck(SECURE_ACCESS_DISSERTATION_CANDIDATEINFO, infoReference);
	}

	/**
	* Check permissions for updating CandidateInfo.
	* @param infoReference - The CandidateInfo's reference.
	* @return True if the current User is allowed to update the CandidateInfo, false if not.
	*/
	public boolean allowUpdateCandidateInfo(String infoReference)
	{
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow update CandidateInfo with resource string : " + infoReference);
		}
		return unlockCheck(SECURE_UPDATE_DISSERTATION_CANDIDATEINFO, infoReference);
	}

	/**
	* Check permissions for removing the CandidateInfo
	* @param infoReference - The CandidateInfo's reference.
	* @return True if the current User is allowed to remove the CandidateInfo, false if not.
	*/
	public boolean allowRemoveCandidateInfo(String infoReference)
	{
		if(m_logger.isDebugEnabled())
		{
			//m_logger.debug("Entering allow remove CandidateInfo with resource string : " + infoReference);
		}
		return unlockCheck(SECURE_REMOVE_DISSERTATION_CANDIDATEINFO, infoReference);
	}


	/*******************************************************************************
	* BlockGrantGroup implementation
	*******************************************************************************/

	public class BaseBlockGrantGroup
		implements BlockGrantGroup
	{
		/** The BlockGrantGroup id. */
		protected String m_id;
		
		/** The BlockGrantGroup code. */
		protected String m_code;
		
		/** The BlockGrantGroup description. */
		protected String m_description;
		
		/** The BlockGrantGroup site. */
		protected String m_site;
		
		/** The properties. */
		protected ResourcePropertiesEdit m_properties;
		
		/** The DissertationStep references, keyed by display order, starting with 1. */
		protected Hashtable m_fieldsOfStudy;
		
		/**
		* Constructor.
		* @param code The BlockGrantGroup code (BGGC).
		* @param description The BlockGrantGroup description (BGGD).
		*/
		public BaseBlockGrantGroup(String id, String site)
		{
			m_id = id;
			m_site = site;
			m_code = null;
			m_description = null;
			m_fieldsOfStudy = new Hashtable();
			m_properties = new BaseResourcePropertiesEdit();
			addLiveProperties(m_properties);
			
		}//BaseBlockGrantGroup
		
		/**
		* Copy constructor.
		* @param blockGrantGroup The BlockGrantGroup to copy.
		*/
		public BaseBlockGrantGroup(BlockGrantGroup blockGrantGroup)
		{
			setAll(blockGrantGroup);

		}	// BaseBlockGrantGroup
		
		/**
		* Construct from an existing definition, in xml.
		* @param el The BlockGrantGroup in XML in a DOM element.
		*/
		//public BaseDissertation(Element el)
		public BaseBlockGrantGroup(Element el)
		{
			this("", "");
			
			String keyString = null;
			String valueString = null;
			List schoolPrereqs = null;
			
			m_id = el.getAttribute("id");
			m_code = el.getAttribute("code");
			m_description = el.getAttribute("description");
			m_site = el.getAttribute("site");
			
				// READ THE CHILD ELEMENTS
			NodeList children = el.getChildNodes();
			final int length = children.getLength();
			for(int i = 0; i < length; i++)
			{
				Node child = children.item(i);
				if (child.getNodeType() != Node.ELEMENT_NODE) continue;
				Element element = (Element)child;

				// look for properties
				if (element.getTagName().equals("properties"))
				{
					m_properties = new BaseResourcePropertiesEdit(element);
				}
				else if(element.getTagName().equals("fields"))
				{
					NodeList osChildren = element.getChildNodes();
					final int osLength = osChildren.getLength();
					if(m_logger.isDebugEnabled())
					{
						m_logger.debug("BaseBlockGrantGroup(Element el) Number of fields " + osLength);
					}
					
					Node osChild = null;
					Element osElement = null;
					for(int x = 0; x < osLength; x++)
					{
						if(m_logger.isDebugEnabled())
						{
							m_logger.debug("field " + x);
						}
						
						osChild = osChildren.item(x);
						if (osChild.getNodeType() != Node.ELEMENT_NODE) continue;
						osElement = (Element)osChild;
						if(m_logger.isDebugEnabled())
						{
							m_logger.debug("osElement.getTagName() " + osElement.getTagName());
						}
						
						if(osElement.getTagName().equals("field"))
						{
							keyString = osElement.getAttribute("code");
							valueString = osElement.getAttribute("description");
							if((keyString != null) && (valueString != null))
							{
								if(m_logger.isDebugEnabled())
								{
									m_logger.debug("add to fields of study field code " + keyString + "field name " + valueString);
				
								}
								m_fieldsOfStudy.put(keyString, valueString);
								if(m_logger.isDebugEnabled())
								{
									m_logger.debug("added " + keyString + " " + valueString + " to Hashtable m_fieldsOfStudy" );
								}
							}
						}
					}
				}
			}

		}// storage constructor

		/**
		* Deep copy of this object.
		* @param b - The BlockGrantGroup object to be copied.
		*/
		protected void setAll(BlockGrantGroup b)
		{
			m_id = b.getId();
			m_code = b.getCode();
			m_description = b.getDescription();
			m_site = b.getSite();
			m_fieldsOfStudy = b.getFieldsOfStudy();
			m_properties = new BaseResourcePropertiesEdit();
			m_properties.addAll(b.getProperties());

		}   // setAll
				
		/**
		* Serialize the resource into XML, adding an element to the doc under the top of the stack element.
		* @param doc The DOM doc to contain the XML (or null for a string return).
		* @param stack The DOM elements, the top of which is the containing element of the new "resource" element.
		* @return The newly added element.
		*/
		public Element toXml(Document doc, Stack stack)
		{
			Element group = doc.createElement("group");
			
			if(stack.isEmpty())
			{
				doc.appendChild(group);
			}
			else
			{
				((Element)stack.peek()).appendChild(group);
			}
			
			stack.push(group);
			
			Enumeration keys = null;
			String fieldCode = null;
			String fieldName = null;

			// SAVE THE BLOCK GRANT GROUP ATTRIBUTES
			group.setAttribute("id", m_id);
			group.setAttribute("code", m_code);
			group.setAttribute("description", m_description);
			group.setAttribute("site", m_site);
			
			// SAVE THE FIELDS OF STUDY
			if(m_logger.isDebugEnabled())
			{
				m_logger.debug("BaseBlockGrantGroup.toXml() Number of fields of study " + m_fieldsOfStudy.size());
			
			}
			Element field = null;
			Element fields = doc.createElement("fields");
			keys = m_fieldsOfStudy.keys();
			while(keys.hasMoreElements())
			{
				field = doc.createElement("field");
				fieldCode = (String)keys.nextElement();
				fieldName = (String)m_fieldsOfStudy.get((String)fieldCode);
				if(fieldCode!=null && fieldName!=null)
				{
					field.setAttribute("code", fieldCode);
					field.setAttribute("description", fieldName);
					fields.appendChild(field);
					
					if(m_logger.isDebugEnabled())
					{
						m_logger.debug("BaseBlockGrantGroup.toXml() Add field code " + fieldCode);
						m_logger.debug("BaseBlockGrantGroup.toXml() Add field description " + fieldName);
					}
				}

				fields.appendChild(field);
			}
			group.appendChild(fields);

				// SAVE THE PROPERTIES
			m_properties.toXml(doc, stack);
			stack.pop();

			return group;

		}//toXML
		
		/*******************************************************************************
		* Reference Implementation
		*******************************************************************************/
		
		public String getId()
		{
			return m_id;
		}
		
		/**
		* Access the BlockGrantGroup code.
		* @return The BlockGrantGroup code string.
		*/
		public String getCode()
		{
			return m_code;
		}
			
		/**
		* Access the BlockGrantGroup description.
		* @return The BlockGrantGroup description string.
		*/
		public String getDescription()
		{
			return m_description;
		}
		
		/**
		* Access the BlockGrantGroup FieldsOfSTudy.
		* @return The BlockGrantGroup FieldsOfStudy hash table.
		*/
		public Hashtable getFieldsOfStudy()
		{
			return m_fieldsOfStudy;
		}
		
		/**
		* Access the URL which can be used to access the resource.
		* @return The URL which can be used to access the resource.
		*/
		public String getUrl()
		{
			return getAccessPoint(false) + m_id;
		}
		
		/**
		* Access the internal reference which can be used to access the resource from within the system.
		* @return The internal reference which can be used to access the resource from within the system.
		*/
		public String getReference()
		{
			return blockGrantGroupReference(m_site, m_id);

		}   // getReference
		
		/**
		* Access the internal reference which can be used to access the resource from within the system.
		* @return The internal reference which can be used to access the resource from within the system.
		*/
		public String getSite()
		{
			return m_site;
			
		}//getSite
		
		/**
		* Access the resources's properties.
		* @return The resources's properties.
		*/
		public ResourceProperties getProperties()
		{
			return m_properties;

		}   // getProperties

		
	}//BaseBlockGrantGroup


	/*******************************************************************************
	* BlockGrantGroupEdit implementation
	*******************************************************************************/

	public class BaseBlockGrantGroupEdit
		extends BaseBlockGrantGroup
		implements BlockGrantGroupEdit, SessionStateBindingListener
	{
		/** The event code for the edit. */
		protected String m_event = null;
		
		/** Active flag. */
		protected boolean m_active = false;
		
		/**
		* Constructor.
		* @param id The BlockGrantGroup id.
		* @param site The BlockGrantGroup site.
		*/
		public BaseBlockGrantGroupEdit(String id, String site)
		{
			super(id, site);

		}// BaseBlockGrantGroupEdit

		/**
		* Construct from an existing definition, in xml.
		* @param el The BlockGrantGroupEdit in XML in a DOM element.
		*/
		public BaseBlockGrantGroupEdit(Element el)
		{
			super(el);

		}// BaseBlockGrantGroupEdit
		
		/**
		* Copy constructor.
		* @param blockGrantGroup The BlockGrantGroup to be copied.
		*/
		public BaseBlockGrantGroupEdit(BlockGrantGroup blockGrantGroup)
		{
			super(blockGrantGroup);

		}// BaseBlockGrantGroupEdit

		

		/**
		* Clean up.
		*/
		protected void finalize()
		{
			// catch the case where an edit was made but never resolved
			if (m_active)
			{
				cancelEdit(this);
			}

		}	// finalize
		
		/**
		* Access the event code for this edit.
		* @return The event code for this edit.
		*/
		protected String getEvent() { return m_event; }
		
		/**
		* Set the event code for this edit.
		* @param event The event code for this edit.
		*/
		protected void setEvent(String event) { m_event = event; }

		/**
		* Access the resource's properties for modification
		* @return The resource's properties.
		*/
		public ResourcePropertiesEdit getPropertiesEdit()
		{
			return m_properties;

		}	// getPropertiesEdit

		/**
		* Enable editing.
		*/
		protected void activate()
		{
			m_active = true;

		}	// activate
		
		/**
		* Check to see if the edit is still active, or has already been closed.
		* @return true if the edit is active, false if it's been closed.
		*/
		public boolean isActiveEdit()
		{
			return m_active;

		}	// isActiveEdit

		/**
		* Close the edit object - it cannot be used after this.
		*/
		protected void closeEdit()
		{
			m_active = false;

		}	// closeEdit

		/**
		* Set the code for this BlockGrantGroup.
		* @param code The BlockGrantGroup code.
		*/
		public void setCode(String code)
		{
			m_code = code;
		}
		
		/**
		* Set the description for this BlockGrantGroup.
		* @param description The BlockGrantGroup description.
		* */
		public void setDescription(String description)
		{
			m_description = description;
		}
		
		/**
		* Set the description for this BlockGrantGroup.
		* * @param code The BlockGrantGroup code.
		* @param description The BlockGrantGroup description.
		* */
		public void addFieldOfStudy(String code, String description)
		{
			m_fieldsOfStudy.put(code, description);
		}
		
		/**
		* Set the description for this BlockGrantGroup.
		* @param code The code of the BlockGrantGroup to remove.
		* */
		public void removeFieldOfStudy(String code)
		{
			m_fieldsOfStudy.remove(code);
		}
		
		/*******************************************************************************
		* SessionStateBindingListener implementation
		*******************************************************************************/
	
		/**
		* Accept notification that this object has been bound as a SessionState attribute.
		* @param sessionStateKey The id of the session state which holds the attribute.
		* @param attributeName The id of the attribute to which this object is now the value.
		*/
		public void valueBound(String sessionStateKey, String attributeName) {}
	
		/**
		* Accept notification that this object has been removed from a SessionState attribute.
		* @param sessionStateKey The id of the session state which held the attribute.
		* @param attributeName The id of the attribute to which this object was the value.
		*/
		public void valueUnbound(String sessionStateKey, String attributeName)
		{
			if (m_logger.isDebugEnabled())
			{
				//m_logger.debug(this + ".valueUnbound()");
			}
	
			// catch the case where an edit was made but never resolved
			if (m_active)
			{
				cancelEdit(this);
			}
	
		}	// valueUnbound

		
	}//BaseBlockGrantGroupEdit
		
		
	/*******************************************************************************
	* Dissertation implementation
	*******************************************************************************/

	public class BaseDissertation
		implements Dissertation
	{
		/** The Dissertation id. */
		protected String m_id;
		
		/** The properties. */
		protected ResourcePropertiesEdit m_properties;
		
		/** The DissertationStep references, keyed by display order, starting with 1. */
		protected Hashtable m_orderedSteps;
		
		/** The List objects containing DissertationStep references, which are
			the prerequisites added to School steps by department administrators for
			their department only.  Keyed by DissertationStep reference. */
		protected Hashtable m_schoolStepPrereqs;
		
		/** The site id. */
		protected String m_site;
		
		/** The time this object was last updated. */
		protected Time m_timeLastModified;
		
		/** The type of dissertation. */
		protected String m_type;

		/**
		* Constructor.
		* @param dissertationId The Dissertation id.
		* @param site The site id.
		*/
		public BaseDissertation(String dissertationId, String site)
		{
			m_id = dissertationId;
			m_site = site;
			m_orderedSteps = new Hashtable();
			m_schoolStepPrereqs = new Hashtable();
			m_properties = new BaseResourcePropertiesEdit();
			addLiveProperties(m_properties);
			
		}//BaseDissertation
		
		/**
		* Copy constructor.
		* @param dissertation The Dissertation to copy.
		*/
		public BaseDissertation(Dissertation dissertation)
		{
			setAll(dissertation);
		}

		/**
		* Construct from an existing definition, in xml.
		* @param el The Dissertation in XML in a DOM element.
		*/
		public BaseDissertation(Element el)
		{
			this("", "");
			
			String keyString = null;
			String valueString = null;
			List schoolPrereqs = null;

			m_id  = el.getAttribute("id");
			m_site = el.getAttribute("site");
			m_type = el.getAttribute("type");
			m_timeLastModified = getTimeObject(el.getAttribute("lastmod"));

			
				// READ THE CHILD ELEMENTS
			NodeList children = el.getChildNodes();
			final int length = children.getLength();
			for(int i = 0; i < length; i++)
			{
				Node child = children.item(i);
				if (child.getNodeType() != Node.ELEMENT_NODE) continue;
				Element element = (Element)child;

				// look for properties
				if (element.getTagName().equals("properties"))
				{
					m_properties = new BaseResourcePropertiesEdit(element);
				}
				else if(element.getTagName().equals("orderedsteps"))
				{
					NodeList osChildren = element.getChildNodes();
					final int osLength = osChildren.getLength();
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : STORAGE CONSTRUCTOR : OS LENGTH : " + osLength);
					Node osChild = null;
					Element osElement = null;
					for(int x = 0; x < osLength; x++)
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : STORAGE CONSTRUCTOR : ITERATION : X : " + x);
						osChild = osChildren.item(x);
						if (osChild.getNodeType() != Node.ELEMENT_NODE) continue;
						osElement = (Element)osChild;
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : STORAGE CONSTRUCTOR : ELEMENT TAG NAME : " + osElement.getTagName());
						if(osElement.getTagName().equals("order"))
						{
							keyString = osElement.getAttribute("ordernum");
							valueString = osElement.getAttribute("stepreference");
							if((keyString != null) && (valueString != null))
							{
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : STORAGE CONSTRUCTOR : ORDERED STEPS KEY STRING   : " + keyString);
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : STORAGE CONSTRUCTOR : ORDERED STEPS VALUE STRING : " + valueString);
								m_orderedSteps.put(keyString, valueString);
							}
						}
					}
				}
				else if(element.getTagName().equals("schoolstepprereqs"))
				{
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : STORAGE CONSTRUCTOR : ELEMENT SCHOOLSTEPPREREQS");
					keyString = element.getAttribute("stepreference");
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : STORAGE CONSTRUCTOR : KEY STRING : " + keyString);
					schoolPrereqs = new Vector();
					NodeList spChildren = element.getChildNodes();
					final int spLength = spChildren.getLength();
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : STORAGE CONSTRUCTOR : SP LENGTH : " + spLength);
					Node spChild = null;
					Element spElement = null;
					for(int x = 0; x < spLength; x++)
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : STORAGE CONSTRUCTOR : ITERATION : X : " + x);
						spChild = spChildren.item(x);
						if (spChild.getNodeType() != Node.ELEMENT_NODE) continue;
						spElement = (Element)spChild;
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : STORAGE CONSTRUCTOR : ELEMENT TAG NAME : " + spElement.getTagName());
						if(spElement.getTagName().equals("prereq"))
						{
							valueString = spElement.getAttribute("prereqstepreference");
							if(valueString != null)
							{
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : STORAGE CONSTRUCTOR : SCHOOL PREREQ TO ADD : " + valueString);
								schoolPrereqs.add(valueString);
							}
						}
					}
					if(keyString != null)
						m_schoolStepPrereqs.put(keyString, schoolPrereqs);
				}
			}

		}// storage constructor
		
		/**
		* Deep copy of this object.
		* @param d - The Disseration object to be copied.
		*/
		protected void setAll(Dissertation d)
		{
			m_id = d.getId();
			m_orderedSteps = d.getOrderedSteps();
			m_schoolStepPrereqs = d.getSchoolStepPrereqs();
			m_site = d.getSite();
			m_timeLastModified = d.getTimeLastModified();
			m_type = d.getType();
			m_properties = new BaseResourcePropertiesEdit();
			m_properties.addAll(d.getProperties());

		}   // setAll

		/**
		* Serialize the resource into XML, adding an element to the doc under the top of the stack element.
		* @param doc The DOM doc to contain the XML (or null for a string return).
		* @param stack The DOM elements, the top of which is the containing element of the new "resource" element.
		* @return The newly added element.
		*/
		public Element toXml(Document doc, Stack stack)
		{
			Element dissertation = doc.createElement("dissertation");
			
			if(stack.isEmpty())
			{
				doc.appendChild(dissertation);
			}
			else
			{
				((Element)stack.peek()).appendChild(dissertation);
			}
			
			stack.push(dissertation);
			
			
			List schoolPrereqs = null;
			String itemString = null;
			String keyString = null;
			Enumeration keys = null;
			
			dissertation.setAttribute("id", m_id);
			dissertation.setAttribute("site", m_site);
			dissertation.setAttribute("lastmod", getTimeString(m_timeLastModified));
			dissertation.setAttribute("type", m_type);

				// SAVE THE ORDERED STEPS
			if(m_logger.isDebugEnabled())
			{
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : TOXML : NUMBER OF ORDERED STEPS : " + m_orderedSteps.size());
			}
			Element order = null;
			Element orderedSteps = doc.createElement("orderedsteps");
			for(int x = 1; x < (m_orderedSteps.size()+1); x++)
			{
				order = doc.createElement("order");
				keyString = "" + x;
				order.setAttribute("ordernum", keyString);
				itemString = (String)m_orderedSteps.get(keyString);
				if(itemString == null)
					itemString = "";
				order.setAttribute("stepreference", itemString);
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : TOXML : ORDER ATTRIBUTE ORDERNUM : " + keyString);
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : TOXML : ORDER ATTRIBUTE STEP ID  : " + itemString);
				orderedSteps.appendChild(order);
			}
			dissertation.appendChild(orderedSteps);
			
			
				// SAVE THE SCHOOL STEP PREREQS
			if(m_logger.isDebugEnabled())
			{
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : TOXML : NUMBER OF SCHOOL PREREQS : " + m_schoolStepPrereqs.size());
			}
			Element schoolStepPrereqs = null;
			Element prereq = null;
			keys = m_schoolStepPrereqs.keys();
			while(keys.hasMoreElements())
			{
				schoolStepPrereqs = doc.createElement("schoolstepprereqs");
				keyString = (String)keys.nextElement();
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : TOXML : A KEY : " + keyString);
				schoolStepPrereqs.setAttribute("stepreference", keyString);
				schoolPrereqs = (List)m_schoolStepPrereqs.get(keyString);
				for(int x = 0; x < schoolPrereqs.size(); x++)
				{
					prereq = doc.createElement("prereq");
					prereq.setAttribute("prereqstepreference", (String) schoolPrereqs.get(x));
					schoolStepPrereqs.appendChild(prereq);
				}
				dissertation.appendChild(schoolStepPrereqs);
			}


				// SAVE THE PROPERTIES
			m_properties.toXml(doc, stack);
			stack.pop();

			return dissertation;

		}//toXML

		
		/*******************************************************************************
		* Reference Implementation
		*******************************************************************************/
		
		/**
		* Access the Dissertation id.
		* @return The Dissertation id string.
		*/
		public String getId()
		{
			return m_id;
		}
		
		/**
		* Access the URL which can be used to access the resource.
		* @return The URL which can be used to access the resource.
		*/
		public String getUrl()
		{
			return getAccessPoint(false) + m_id;
		}

		/**
		* Access the internal reference which can be used to access the resource from within the system.
		* @return The the internal reference which can be used to access the resource from within the system.
		*/
		public String getReference()
		{
			return dissertationReference(m_site, m_id);
		}

		/**
		* Access the resources's properties.
		* @return The resources's properties.
		*/
		public ResourceProperties getProperties()
		{
			return m_properties;

		}   // getProperties
		
		
		/**
		* Access the reference of the DissertationStep by display order.
		* @param order The order number as a String.
		* @return The reference of the DissertationStep, or null if there is no match.
		*/
		public String getStepReferenceByOrder(String order)
		{
			if(order == null)
				return null;
			else
				return (String)m_orderedSteps.get(order);
		}

		/**
		* Access the Hashtable containing the DissertationSteps.
		* @return The DissertationSteps keyed by display order.
		*/
		public Hashtable getOrderedSteps()
		{
			return m_orderedSteps;
		}
		
		/**
		* Access the display order for a DissertationStep.
		* @param stepReference The reference of the DissertationStep.
		* @return The display order number as a String.
		*/
		public String getOrderForStep(String stepReference)
		{
			String retVal = null;
			if(stepReference != null)
			{
				String key = null;
				String value = null;
				Enumeration keys = m_orderedSteps.keys();
				while(keys.hasMoreElements())
				{
					key = (String)keys.nextElement();
					value = (String)m_orderedSteps.get(key);
					if(stepReference.equals(value))
						retVal = key;
				}
			}
			return retVal;
		}
		
		/**
		* Access the site id for the Dissertation's site.
		* @return the site id.
		*/
		public String getSite()
		{
			return m_site;
		}
		
		/**
		* Access the references of the prerequisite steps for a School step.
		* @param schoolStepReference The reference of the step.
		* @return List containing the prerequisite step references.
		*/
		public List getSchoolPrereqs(String schoolStepReference)
		{
			List retVal = null;
			retVal = (List) m_schoolStepPrereqs.get(schoolStepReference);
			if(retVal == null)
				retVal = new Vector();
			return retVal;
		}

		/**
		* Access the order numbers of the prerequisites in a comma-delimited string for display.
		* @param step The DissertationStep.
		* @return The prerequisite steps display string.
		*/
		public String getPrerequisiteStepsDisplayString(DissertationStep step)
		{
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : ENTERING GET PREREQUISITE STEPS DISPLAY STRING");
			StringBuffer retVal = new StringBuffer();
			
			if(step != null)
			{
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : GET PREREQUISITE STEPS DISPLAY STRING : STEP ID : " + step.getId());
				
				String keyString = "";
				String stepReference = "";
				boolean firstAdded = true;
				boolean addToString = false;
				List deptAddedPrereqs = (List) m_schoolStepPrereqs.get(step.getReference());
				for(int x = 1; x < (m_orderedSteps.size()+1); x++)
				{
					addToString = false;
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : GET PREREQUISITE STEPS DISPLAY STRING : ORDERED STEPS ITERATION : " + x);
					keyString = "" + x;
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : GET PREREQUISITE STEPS DISPLAY STRING : KEY STRING : " + keyString);
					stepReference = (String)m_orderedSteps.get(keyString);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : GET PREREQUISITE STEPS DISPLAY STRING : STEP ID    : " + stepReference);
					
					if(step.hasPrerequisiteStep(stepReference))
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : GET PREREQUISITE STEPS DISPLAY STRING : STEP HAS PREREQ : ADDING : " + keyString);
						addToString = true;
					}
					else
					{
						if(deptAddedPrereqs != null)
						{
							if(deptAddedPrereqs.contains(stepReference))
							{
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : GET PREREQUISITE STEPS DISPLAY STRING : DEPT ADDED PREREQS CONTAINS THIS STEP ID - ADDING : " + keyString);
								addToString = true;
							}
						}
					}
					
					if(addToString)
					{
						if(firstAdded)
						{
							retVal.append(keyString);
							firstAdded = false;
						}
						else
							retVal.append(", " + keyString);
					}
				}
			}
			
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : GET PREREQUISITE STEPS DISPLAY STRING : RETURNING : " + retVal.toString());
			return retVal.toString();
		}

		/**
		* A snapshot of this Dissertation's structure is used to create the steps in the CandidatePath.
		* This method is used in the creation of new CandidatePath objects.  This Dissertation becomes
		* the CandidatePath's parent Dissertation.
		* @param candidatePath The CandidatePath to initialize.
		* @param siteString The site id of the CandidatePath.
		*/
		public void initializeCandidatePath(CandidatePathEdit candidatePath, String siteString)
		{
			Hashtable stepToStatus = new Hashtable();
			List stepPrereqs = null;
			List statusPrereqs = null;
			List schoolPrereqStatus = null;
			List schoolPrereqSteps = null;
			String keyString = null;
			String stepId = null;
			String stepReference = null;
			String statusId = null;
			String statusReference = null;
			String tempStatusReference = null;
			DissertationStep aStep = null;
			StepStatusEdit aStatus = null;
			Hashtable orderedStatus = new Hashtable();
			Hashtable schoolPrereqs = new Hashtable();
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : ENTERING initializeCandidatePath");
			StepStatusEdit stepStatus = null;
			Enumeration keys = null;
	
				// TRANSFER THE ORDERED STEPS DATA
			boolean oardStep = false;
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : got keys - first pass");
			for(int x = 1; x < (m_orderedSteps.size()+1); x++)
			{
				oardStep = false;
				keyString = "" + x;
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : a key : " + keyString);
				try
				{
					stepReference = (String)m_orderedSteps.get(keyString);
					stepId = stepId(stepReference);
					aStep = m_stepStorage.get(stepId);
					stepStatus = addStepStatusFromListener(siteString, aStep, oardStep);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : CREATED NEW STEP STATUS OBJECT : " );
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : STEPSTATUS INSTRUCTIONS : " + stepStatus.getInstructions());
					orderedStatus.put(keyString, stepStatus.getReference());
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : orderedStatus.put : " + keyString + " : " + stepStatus.getId());
					stepToStatus.put(stepReference, stepStatus.getReference());
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : STEP-TO-STATUS : " + stepReference + " : " + stepStatus.getId());
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : added step status to service");
					commitEdit(stepStatus);
				}
				catch(Exception e)
				{
					m_logger.warn("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : EXCEPTION : " + e);
				}
			}
			candidatePath.setOrderedStatus(orderedStatus);


				// TRANSFER THE PREREQUISITE STEPS DATA FOR INDIVIDUAL STEPS
			keys = m_orderedSteps.keys();
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : got keys - second pass");
			while(keys.hasMoreElements())
			{
				statusPrereqs = new Vector();
				keyString = (String)keys.nextElement();
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : a key : " + keyString);
				try
				{
					schoolPrereqStatus = new Vector();
					stepReference = (String)m_orderedSteps.get(keyString);
					stepId = stepId(stepReference);
					aStep = m_stepStorage.get(stepId);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : step ref   : " + stepReference);
					statusReference = candidatePath.getStatusReferenceByOrder(keyString);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : status ref : " + statusReference);
					schoolPrereqSteps = getSchoolPrereqs(aStep.getReference());
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : searching for school step prereqs : number found :  " + schoolPrereqSteps.size());
					for(int y = 0; y < schoolPrereqSteps.size(); y++)
					{
						tempStatusReference = (String)stepToStatus.get(schoolPrereqSteps.get(y));
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : adding school prereq status : " + tempStatusReference);
						schoolPrereqStatus.add(tempStatusReference);
					}
					if(schoolPrereqStatus.size() > 0)
						schoolPrereqs.put(statusReference, schoolPrereqStatus);

					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : finding prereqs for step");
					statusId = statusId(statusReference);
					aStatus = m_statusStorage.edit(statusId);
					//if(aStatus == null)
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : GET STATUS EDIT CLONE RETURNS NULL !!!!!!!!!!!!!!!!!!!!");
					//else
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : got status edit clone : " + aStatus.getId());

					stepPrereqs = aStep.getPrerequisiteStepReferences();
					for(int x = 0; x < stepPrereqs.size(); x++)
					{
						statusReference = (String)stepToStatus.get(stepPrereqs.get(x));
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : adding step prereq : " + statusReference);
						statusPrereqs.add(statusReference);
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : ADDED STEP PREREQ : "  + statusReference);
					}
					aStatus.setPrereqs(statusPrereqs);
					commitEdit(aStatus);

					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : added step status to service and path");
				}
				catch(Exception e)
				{
					m_logger.warn("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : EXCEPTION : " + e);
				}
			}
			
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : initializeCandidatePath : adding school prereqs to candidate path : number : " + schoolPrereqs.size());
			candidatePath.setSchoolPrereqs(schoolPrereqs);
			candidatePath.setSite(siteString);
		}

		/**
		* Access the creator of this Dissertation.
		* @return The user id of the creator.
		*/
		public String getCreator()
		{
			return m_properties.getProperty(ResourceProperties.PROP_CREATOR);
		}
		
		/**
		* Access the user who last modified this Disseration.
		* @return the user id the the author who last modified the Dissertation.
		*/
		public String getAuthorLastModified()
		{
			return m_properties.getProperty(ResourceProperties.PROP_MODIFIED_BY);
		}
		
		/**
		* Access the Time that this Dissertation was last modified.
		* @return the Time last modified.
		*/
		public Time getTimeLastModified()
		{
			return m_timeLastModified;
		}
		
		/**
		* Access the type of this Dissertation.
		* @return the String type.
		*/
		public String getType()
		{
			return m_type;
		}
		
		/**
		* Access the prerequisites added by department administrators to School steps for their department.
		* @return the Hashtable of Lists containing the references of prerequisite steps, keyed by DissertationStep reference.
		*/
		public Hashtable getSchoolStepPrereqs()
		{
			return m_schoolStepPrereqs;
		}

		/**
		* Are these objects equal?  If they are both Dissertation objects, and they have
		* matching id's, they are.
		* @return true if they are equal, false if not.
		*/
		public boolean equals(Object obj)
		{
			if (!(obj instanceof Dissertation)) return false;
			return ((Dissertation)obj).getId().equals(getId());

		}   // equals

		/**
		* Make a hash code that reflects the equals() logic as well.
		* We want two objects, even if different instances, if they have the same id to hash the same.
		*/
		public int hashCode()
		{
			return getId().hashCode();

		}	// hashCode

		/**
		* Compare this object with the specified object for order.
		* @return A negative integer, zero, or a positive integer as this object is
		* less than, equal to, or greater than the specified object.
		*/
		public int compareTo(Object obj)
		{
			if (!(obj instanceof Dissertation)) throw new ClassCastException();

			// if the object are the same, say so
			if (obj == this) return 0;

			// start the compare by comparing their sort names
			int compare = getSite().compareTo(((Dissertation)obj).getSite());

			// if these are the same
			if (compare == 0)
			{
				// sort based on (unique) id
				compare = getId().compareTo(((Dissertation)obj).getId());
			}

			return compare;

		}	// compareTo
		
	}//BaseDissertation



	public class BaseDissertationEdit
		extends BaseDissertation
		implements DissertationEdit, SessionStateBindingListener
	{
		/** The event code for the edit. */
		protected String m_event = null;
		
		/** Active flag. */
		protected boolean m_active;

		/**
		* Constructor.
		* @param dissertationId The Dissertation id.
		* @param site The site id.
		*/
		public BaseDissertationEdit(String dissertationId, String site)
		{
			super(dissertationId, site);
		}
		
		/**
		* Copy constructor.
		* @param d The Dissertation to be copied.
		*/
		public BaseDissertationEdit(Dissertation d)
		{
			super(d);
		}

		/**
		* Construct from an existing definition, in xml.
		* @param el The DissertationEdit in XML in a DOM element.
		*/
		public BaseDissertationEdit(Element el)
		{
			super(el);
		}// storage constructor

		/**
		* Clean up.
		*/
		protected void finalize()
		{
			// catch the case where an edit was made but never resolved
			if (m_active)
			{
				cancelEdit(this);
			}

		}	// finalize
		
		/**
		* Set the DissertationSteps for this Dissertation.
		* @param table The Hashtable of DissertationStep references, keyed by display order as a String.
		*/
		public void setOrderedSteps(Hashtable table)
		{
			if(table != null)
				m_orderedSteps = table;
		}

		/**
		* Set the site for this Dissertation.
		* @param site The site id.
		*/
		public void setSite(String site)
		{
			m_site = site;
		}
		
		/**
		* Set the pedagogical type for this Dissertation.
		* @param type The dissertation steps type.
		*/
		public void setType(String type)
		{
			m_type = type;
		}

		/**
		* Set the prerequisites for a School step.
		* @param schoolStepReference The reference of the school step.
		* @param prereqs The List of DissertationStep references.
		*/
		public void setSchoolPrereqs(String schoolStepReference, List prereqs)
		{
			if((schoolStepReference != null) && (prereqs != null))
				m_schoolStepPrereqs.put(schoolStepReference, prereqs);
		}

		/**
		* Add a prerequisite to a School step.
		* @param schoolStepReference The reference of the School step to which an prerequisite is to be added.
		* @param prereqStepReference The reference of the prerequisite step to be added.
		*/
		public void addSchoolPrereq(String schoolStepReference, String prereqStepReference)
		{
			//m_logger.debug("DISSERTATION : BASE SERVICE : ENTERING ADD SCHOOL PREREQ : SCHOOL STEP ID : " + schoolStepReference);
			//m_logger.debug("DISSERTATION : BASE SERVICE : ENTERING ADD SCHOOL PREREQ : PREREQ STEP ID : " + prereqStepReference);
			List prereqs = null;
			if((schoolStepReference != null) && (prereqStepReference != null))
			{
				prereqs = (List) m_schoolStepPrereqs.get(schoolStepReference);
				if(prereqs == null)
					prereqs = new Vector();
				//m_logger.debug("DISSERTATION : BASE SERVICE : ADD SCHOOL PREREQ : NUMBER OF EXISTING SCHOOL PREREQS : " + prereqs.size());
				//for(int x = 0; x < prereqs.size(); x++)
				//{
				//	//m_logger.debug("DISSERTATION : BASE SERVICE : ADD SCHOOL PREREQ : EXISTING SCHOOL PREREQ : " + prereqs.get(x));
				//}
				
				if(!prereqs.contains(prereqStepReference))
				{
					prereqs.add(prereqStepReference);
					//m_logger.debug("DISSERTATION : BASE SERVICE : ADD SCHOOL PREREQ : PREREQ ADDED");
				}
				//else
				//	//m_logger.debug("DISSERTATION : BASE SERVICE : ADD SCHOOL PREREQ : PREREQ ALREADY EXISTS - WILL NOT ADD");
				
				//m_logger.debug("DISSERTATION : BASE SERVICE : ADD SCHOOL PREREQ : AFTER : NUMBER OF EXISTING SCHOOL PREREQS : " + prereqs.size());
				//for(int x = 0; x < prereqs.size(); x++)
				//{
				//	//m_logger.debug("DISSERTATION : BASE SERVICE : ADD SCHOOL PREREQ : AFTER : EXISTING SCHOOL PREREQ : " + prereqs.get(x));
				//}
				m_schoolStepPrereqs.put(schoolStepReference, prereqs);
			}
			//m_logger.debug("DISSERTATION : BASE SERVICE : ABOUT TO LEAVE ADD SCHOOL PREREQ : HASHTABLE KEYS : ");
			//Enumeration keys = m_schoolStepPrereqs.keys();
			//while(keys.hasMoreElements())
			//	//m_logger.debug("DISSERTATION : BASE SERVICE : A KEY : " + (String)keys.nextElement());

			//m_logger.debug("DISSERTATION : BASE SERVICE : LEAVING ADD SCHOOL PREREQ : ");
		}

		
		/**
		* Remove a prerequisite from a School step.
		* @param schoolStepReference The reference of the School step from which an prerequisite is to be removed.
		* @param prereqStepReference The reference of the prerequisite step to be removed.
		*/
		public void removeSchoolPrereq(String schoolStepReference, String prereqStepReference)
		{
			//m_logger.debug("DISSERTATION : BASE SERVICE : ENTERING REMOVE SCHOOL PREREQ : SCHOOL STEP ID : " + schoolStepReference);
			//m_logger.debug("DISSERTATION : BASE SERVICE : ENTERING REMOVE SCHOOL PREREQ : PREREQ STEP ID : " + prereqStepReference);
			List prereqs = null;
			if((schoolStepReference != null) && (prereqStepReference != null))
			{
				prereqs = (List) m_schoolStepPrereqs.get(schoolStepReference);
				if(prereqs == null)
					prereqs = new Vector();
				//for(int x = 0; x < prereqs.size(); x++)
				//{
				//	//m_logger.debug("DISSERTATION : BASE SERVICE : REMOVE SCHOOL PREREQ : EXISTING SCHOOL PREREQ : " + prereqs.get(x));
				//}

				prereqs.remove(prereqStepReference);
			}
			//m_logger.debug("DISSERTATION : BASE SERVICE : LEAVING REMOVE SCHOOL PREREQ");
		}

		/**
		* Remove a DissertationStep from this Dissertation.
		* @param stepToRemoveReference The reference of the DissertationStep to be removed.
		*/
		public void removeStep(String stepToRemoveReference)
		{
			if(stepToRemoveReference != null)
			{
				Hashtable newOrder = new Hashtable();
				List schoolStepPrereqs = null;
				String keyString = null;
				String stepReference = null;
				boolean foundIt = false;
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : REMOVE STEP : REMOVING STEP FROM DISSERTATION : " + stepToRemoveReference);
				for(int y = 1; y < (m_orderedSteps.size()+1); y++)
				{
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : REMOVE STEP : LOOP PASS # " + y);
					keyString = "" + y;
					stepReference = (String)m_orderedSteps.get(keyString);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : REMOVE STEP : key      : " + keyString);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : REMOVE STEP : step ref : " + stepReference);
					
					// CHECK THE SCHOOL-STEP PREREQUISITES
					schoolStepPrereqs = (List)m_schoolStepPrereqs.get(stepReference);
					if(schoolStepPrereqs != null)
					{
						schoolStepPrereqs.remove(stepToRemoveReference);
					}
					
					// NOW REMOVE THE STEP FROM ORDERED STEPS
					if(stepReference.equals(stepToRemoveReference))
					{
						foundIt = true;
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : REMOVE STEP : FOUND A MATCH !!!!!!!");
					}
					else
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : REMOVE STEP : NO MATCH THIS PASS");
						if(foundIt)
						{
							keyString = "" + (y-1);
							newOrder.put(keyString, stepReference);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : REMOVE STEP : ALREADY FOUND : PUTTING : " + keyString + " : " + stepReference);
						}
						else
						{
							newOrder.put(keyString, stepReference);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : REMOVE STEP : NOT YET FOUND : PUTTING : " + keyString + " : " + stepReference);
						}
					}
				}
				m_orderedSteps = newOrder;
			}
		}

		
		/**
		* Add a DissertationStep to this Dissertation.
		* @param step The DissertationStep to be added.
		* @param previousstepReference The reference of the DissertationStep after which the step will be added.
		*/
		public void addStep(DissertationStep step, String previousstepReference)
		{
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : ENTERING ADD STEP ");
			if(step != null)
			{
				int position = 0;

				// NOW LOOK FOR THE PREVIOUS STEP ID
				if(previousstepReference != null)
				{
					try
					{
							// FIRST CHECK TO SEE IF ALL OF THIS STEPS PREREQUSISTES EXIST IN THIS DISSERTATION
						List prereqs = step.getPrerequisiteStepReferences();
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : STEP TO ADD HAS " + prereqs.length + " PREREQUISITES");
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : CHECKING TO SEE IF ALL EXIST IN THIS DISSERTATION");
						for(int x = 0; x < prereqs.size(); x++)
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : CHECKING : " + prereqs[x]);
							if(getOrderForStep((String) prereqs.get(x)) == null)
							{
								DissertationStepEdit stepEdit = editDissertationStep(step.getReference());
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : DOES NOT EXIST - REMOVING ");
								stepEdit.removePrerequisiteStep((String) prereqs.get(x));
								commitEdit(stepEdit);
							}
							//else
							//	//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : PREREQ EXISTS ");
						}

						if(previousstepReference.equals("start"))
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : ADD STEP : PREVIOUS STEP IS START - ADDING TO POSITION 1");
							addToOrderedSteps(step, 1);
						}
						else if(m_orderedSteps.contains(previousstepReference))
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : ADD STEP : FOUND PREVIOUS STEP IN THIS DISSERTATION");
								// FIND THE KEY FOR THE PREVIOUS STEP
							String aKey = null;
							String aValue = null;
							String previousStepKey = null;
							Enumeration keys = m_orderedSteps.keys();
							while(keys.hasMoreElements())
							{
								aKey = (String)keys.nextElement();
								aValue = (String)m_orderedSteps.get(aKey);
								if(aValue.equals(previousstepReference))
								{
									previousStepKey = aKey;
									//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : ADD STEP : FOUND KEY FOR PREVIOUS STEP : " + aKey);
								}
							}

							position = Integer.parseInt(previousStepKey);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : ADD STEP : GOT POSITION INTEGER FOR KEY : " + position);
							addToOrderedSteps(step, (position+1));
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : ADD STEP : ADDED STEP TO ORDERED STEPS");
						}
						else
						{
								// IF THE PREVIOUS STEP DOES NOT EXIST - PUT THE STEP AT THE END
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : ADD STEP : PREVIOUS STEP NOT IN THIS DISSERTATION - WILL ADD TO END");
							int stepPos = (m_orderedSteps.size() + 1);
							addToOrderedSteps(step, stepPos);
						}
					}
					catch(Exception e)
					{
						m_logger.warn("DISSERTATION : BASE SERVICE : BASE DISSERTATION : ADD STEP : EXCEPTION : "+ e);
					}
				}
			}
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : LEAVING ADD STEP ");
		}
		
		
		/**
		* Move a DissertationStep within this Dissertation.
		* @param stepToMoveRef The reference of the DissertationStep to be moved.
		* @param location The reference of the step after which the DissertationStep is to be moved.
		*/
		public void moveStep(String stepToMoveRef, String location)
		{
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : step to move ref     : " + stepToMoveRef);
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : location step ref    : " + location);
			Hashtable newOrder = new Hashtable();
			String keyString = null;
			String stepReference = null;
			
				// FIND THE ORDER NUMBERS FOR THE STEP TO MOVE AND LOCATION
			int oldPosition = -1;
			int newPosition = -1;
			String aKey = null;
			String aValue = null;
			Enumeration keys = m_orderedSteps.keys();
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : ORDERED STEPS CONTENTS : ");
			if(location.equals("start"))
				newPosition = 1;

			while(keys.hasMoreElements())
			{
				aKey = (String)keys.nextElement();
				aValue = (String)m_orderedSteps.get(aKey);
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : KEY : VALUE : " + aKey + " : " + aValue);
				if(aValue.equals(stepToMoveRef))
				{
					try
					{
						oldPosition = Integer.parseInt(aKey);
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : parsed oldPosition int : " + oldPosition);
					}
					catch(Exception e){}
				}
				if(aValue.equals(location))
				{
					try
					{
						newPosition = Integer.parseInt(aKey);
						newPosition++;
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : parsed newPosition int : " + newPosition);
					}
					catch(Exception e){}
				}
			}

			if((oldPosition != -1) && (newPosition != -1))
			{
				if(newPosition > oldPosition)     // MOVING UP
				{
					newPosition--;
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : MOVING UP");
					for(int x = 1; x < (m_orderedSteps.size()+1); x++)
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : LOOP PASS # " + x);
						if((x < oldPosition) || (x > newPosition))
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : (x < old) || (x > new)");
							keyString = "" + x;
							stepReference = (String)m_orderedSteps.get(keyString);
							newOrder.put(keyString, stepReference);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : put : " + keyString + " : " + stepReference);
						}
						else if(x == newPosition)
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : x = new");
							keyString = "" + x;
							newOrder.put(keyString, stepToMoveRef);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : put : " + keyString + " : " + stepToMoveRef);
						}
						else
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : (x >= old) && (x < new)");
							keyString = "" + (x+1);
							stepReference = (String)m_orderedSteps.get(keyString);
							keyString = "" + x;
							newOrder.put(keyString, stepReference);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : put : " + keyString + " : " + stepReference);
						}
					}
					m_orderedSteps = newOrder;
				}
				else if(oldPosition > newPosition)		// MOVING DOWN
				{
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : MOVING DOWN");
					for(int x = 1; x < (m_orderedSteps.size()+1); x++)
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : LOOP PASS # " + x);
						if((x < newPosition) || (x > oldPosition))
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : (x < new) || (x > old)");
							keyString = "" + x;
							stepReference = (String)m_orderedSteps.get(keyString);
							newOrder.put(keyString, stepReference);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : put : " + keyString + " : " + stepReference);
						}
						else if(x == newPosition)
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : x = new ");
							keyString = "" + x;
							newOrder.put(keyString, stepToMoveRef);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : put : " + keyString + " : " + stepToMoveRef);
						}
						else
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : (x >= old) && (x < new)");
							keyString = "" + (x-1);
							stepReference = (String)m_orderedSteps.get(keyString);
							keyString = "" + x;
							newOrder.put(keyString, stepReference);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : put : " + keyString + " : " + stepReference);
						}
					}
					m_orderedSteps = newOrder;
				}
				
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : NEW ORDERED STEPS HASHTABLE CONTNENTS : ");
				
				for(int y = 1; y < (newOrder.size()+1); y++)
				{
					keyString = "" + y;
					stepReference = (String)newOrder.get(keyString);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : moveStep : key : value : " + keyString + " : " + stepReference);
				}
			}
		}
		
		
		/**
		* Add a DissertationStep to this Dissertation.
		* @param newStep The DissertationStep to be added.
		* @param locationInt The display order number of this step.
		*/
		protected void addToOrderedSteps(DissertationStep newStep, int locationInt)
		{
			Hashtable newOrder = new Hashtable();
			String keyString = null;
			String stepReference = null;
			for(int x = 1; x < (m_orderedSteps.size()+2); x++)
			{
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : addToOrderedSteps : loop pass # : " + x);
				if(locationInt == x)
				{
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : addToOrderedSteps : locationInt = x");
					keyString = "" + x;
					newOrder.put(keyString, newStep.getReference());
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : addToOrderedSteps : keyString : " + keyString + " : newstepReference : " + newStep.getReference());
				}
				else if(locationInt < x)
				{
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : addToOrderedSteps : locationInt < x");
					keyString = "" + (x-1);
					stepReference = (String)m_orderedSteps.get(keyString);
					keyString = "" + (x);
					newOrder.put(keyString, stepReference);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : addToOrderedSteps : keyString : " + keyString + " : stepReference : " + stepReference);
				}
				else
				{
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : addToOrderedSteps : locationInt > x");
					keyString = "" + x;
					stepReference = (String)m_orderedSteps.get(keyString);
					newOrder.put(keyString, stepReference);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : addToOrderedSteps : keyString : " + keyString + " : stepReference : " + stepReference);
				}
			}
			m_orderedSteps = newOrder;
		}

		/**
		* Deep copy.
		* @param dissertation The dissertation object to be copied.
		*/
		protected void set(Dissertation dissertation)
		{
			setAll(dissertation);

		}   // set

		/**
		* Access the event code for this edit.
		* @return The event code for this edit.
		*/
		protected String getEvent() { return m_event; }
	
		/**
		* Set the event code for this edit.
		* @param event The event code for this edit.
		*/
		protected void setEvent(String event) { m_event = event; }

		/**
		* Access the resource's properties for modification.
		* @return The resource's properties.
		*/
		public ResourcePropertiesEdit getPropertiesEdit()
		{
			return m_properties;
		}
		
		/**
		* Enable editing.
		*/
		protected void activate()
		{
			m_active = true;

		}	// activate

		/**
		* Check to see if the edit is still active, or has already been closed.
		* @return true if the edit is active, false if it's been closed.
		*/
		public boolean isActiveEdit()
		{
			return m_active;

		}	// isActiveEdit

		/**
		* Close the edit object - it cannot be used after this.
		*/
		protected void closeEdit()
		{
			m_active = false;

		}	// closeEdit


		/*******************************************************************************
		* SessionStateBindingListener implementation
		*******************************************************************************/
	
		/**
		* Accept notification that this object has been bound as a SessionState attribute.
		* @param sessionStateKey The id of the session state which holds the attribute.
		* @param attributeName The id of the attribute to which this object is now the value.
		*/
		public void valueBound(String sessionStateKey, String attributeName) {}
	
		/**
		* Accept notification that this object has been removed from a SessionState attribute.
		* @param sessionStateKey The id of the session state which held the attribute.
		* @param attributeName The id of the attribute to which this object was the value.
		*/
		public void valueUnbound(String sessionStateKey, String attributeName)
		{
			if (m_logger.isDebugEnabled())
			{
				//m_logger.debug(this + ".valueUnbound()");
			}
	
			// catch the case where an edit was made but never resolved
			if (m_active)
			{
				cancelEdit(this);
			}
	
		}	// valueUnbound
		
	}//BaseDissertationEdit

	
	
	/*******************************************************************************
	* DissertationStep implementation
	*******************************************************************************/

	public class BaseDissertationStep
		implements DissertationStep
	{
		/** The DissertationStep id. */
		protected String m_id;
		
		/** The site id. */
		protected String m_site;
		
		/** The instructions for the step. */
		protected String m_instructionsText;
		
		/** The id for auto-validation from the Rackham database. */
		protected String m_autoValidationId;
		
		/** The time the DissertationStep was last modified. */
		protected Time m_timeLastModified;
		
		/** Type of permissions for marking the step as completed.
			See the DissertationStep interface for types. */
		protected String m_validationType;
		
		/** The id for the step's checklist section. 
			See the DissertationService for section heads.
		 */
		protected String m_section;
		
		/** The references of the prerequisites for this step. */
		protected List m_prerequisiteSteps;
		
		/** The properties. */
		protected BaseResourcePropertiesEdit m_properties;
		


		/**
		* Constructor.
		* @param stepId The DissertationStep id.
		* @param site The site id.
		*/
		public BaseDissertationStep(String stepId, String site)
		{
			m_id = stepId;
			m_site = site;
			m_instructionsText = "";
			m_autoValidationId = "";
			m_section = "0";
			m_timeLastModified = TimeService.newTime();
			m_validationType = "0";
			m_prerequisiteSteps = new Vector();
			m_properties = new BaseResourcePropertiesEdit();
			addLiveProperties(m_properties);
		}
		
		/**
		* Copy constructor.
		* @param step The DissertationStep to copy.
		*/
		public BaseDissertationStep(DissertationStep step)
		{
			setAll(step);

		}	// BaseDissertationStep
		
		/**
		* Construct from an existing definition, in xml.
		* @param el The DissertationStep in XML in a DOM element.
		*/
		public BaseDissertationStep(Element el)
		{
			String valueString = null;
			m_properties = new BaseResourcePropertiesEdit();
			m_prerequisiteSteps = new Vector();

			m_id = el.getAttribute("id");
			m_site = el.getAttribute("site");
			m_autoValidationId = el.getAttribute("autovalid");
			if(el.getAttribute("section")!=null)
			{
				m_section = el.getAttribute("section");
			}
			else
			{
				m_section = "1";
			}
			m_timeLastModified = getTimeObject(el.getAttribute("timelastmod"));
			m_validationType = el.getAttribute("validationtype");

			m_instructionsText = Xml.decodeAttribute(el, "instructionstext");

				// READ THE CHILD ELEMENTS
			NodeList children = el.getChildNodes();
			final int length = children.getLength();
			for(int i = 0; i < length; i++)
			{
				Node child = children.item(i);
				if (child.getNodeType() != Node.ELEMENT_NODE) continue;
				Element element = (Element)child;

				// look for properties
				if (element.getTagName().equals("properties"))
				{
					// re-create properties
					m_properties = new BaseResourcePropertiesEdit(element);
				}
				else if(element.getTagName().equals("prereqsteps"))
				{
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION STEP : STORAGE CONSTRUCTOR : ELEMENT PREREQUISITE STEPS");
					NodeList psChildren = element.getChildNodes();
					final int psLength = psChildren.getLength();
					Node psChild = null;
					Element psElement = null;
					for(int x = 0; x < psLength; x++)
					{
						psChild = psChildren.item(x);
						if (psChild.getNodeType() != Node.ELEMENT_NODE) continue;
						psElement = (Element)psChild;
						if(psElement.getTagName().equals("prereq"))
						{
							valueString = psElement.getAttribute("prereqstepreference");
							if(valueString != null)
							{
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : READ : REFERENCE ATTRIBUTE : " + valueString);
								m_prerequisiteSteps.add(valueString);
							}
						}
					}
				}
				// old style of encoding
				else if(element.getTagName().equals("instructionstext"))
				{
					if ((element.getChildNodes() != null) && (element.getChildNodes().item(0) != null))
					{
						m_instructionsText = element.getChildNodes().item(0).getNodeValue();
					}
					if (m_instructionsText == null)
					{
						m_instructionsText = "";
					}
				}
			}

		}// storage constructor

		/**
		* Deep copy of this object.
		* @param step - The DisserationStep object to be copied.
		*/
		protected void setAll(DissertationStep step)
		{
			m_id = step.getId();
			m_site = step.getSite();
			m_instructionsText = step.getInstructionsText();
			m_autoValidationId = step.getAutoValidationId();
			m_timeLastModified = step.getTimeLastModified();
			m_validationType = step.getValidationType();
			m_section = step.getSection();
			m_prerequisiteSteps = step.getPrerequisiteStepReferences();
			m_properties = new BaseResourcePropertiesEdit();
			m_properties.addAll(step.getProperties());

		}   // setAll

		
		/**
		* Serialize the resource into XML, adding an element to the doc under the top of the stack element.
		* @param doc The DOM doc to contain the XML (or null for a string return).
		* @param stack The DOM elements, the top of which is the containing element of the new "resource" element.
		* @return The newly added element.
		*/
		public Element toXml(Document doc, Stack stack)
		{
			Element step = doc.createElement("step");
			
			if (stack.isEmpty())
			{
				doc.appendChild(step);
			}
			else
			{
				((Element)stack.peek()).appendChild(step);
			}
			
			stack.push(step);

			String attributeString = null;
			String itemString = null;
			String intString = null;

			step.setAttribute("id", m_id);
			step.setAttribute("site", m_site);
			step.setAttribute("validationtype", m_validationType);
			step.setAttribute("section", m_section);
			step.setAttribute("autovalid", m_autoValidationId);
			
			//m_logger.debug("DISSERTATION : BASE SERVICE : DISSERTATION STEP : TOXML : BASIC ATTRIBUTES WRITTEN");
			
			
				// SAVE THE PREREQUISISTE STEPS
			Element prereq = null;
			Element prereqsteps = doc.createElement("prereqsteps");
			for(int x = 0; x < m_prerequisiteSteps.size(); x++)
			{
				prereq = doc.createElement("prereq");
				prereq.setAttribute("prereqstepreference", (String) m_prerequisiteSteps.get(x));
				prereqsteps.appendChild(prereq);
			}
			step.appendChild(prereqsteps);
			//m_logger.debug("DISSERTATION : BASE SERVICE : DISSERTATION STEP : TOXML : PREREQUISITES WRITTEN");
			
			
				// SAVE THE INSTRUCTIONS TEXT
			Xml.encodeAttribute(step, "instructionstext", m_instructionsText);
			
				// SAVE THE PROPERTIES
			m_properties.toXml(doc, stack);

			stack.pop();
			
			return step;

		}//toXml

		
		/*******************************************************************************
		* Reference Implementation
		*******************************************************************************/
		
		/**
		* Access the DissertationStep id.
		* @return The DissertationStep id string.
		*/
		public String getId()
		{
			return m_id;
		}

		/**
		* Access the URL which can be used to access the resource.
		* @return The URL which can be used to access the resource.
		*/
		public String getUrl()
		{
			return getAccessPoint(false) + m_id;
		}
		
		/**
		* Access the internal reference which can be used to access the resource from within the system.
		* @return The the internal reference which can be used to access the resource from within the system.
		*/
		public String getReference()
		{
			return stepReference(m_site, m_id);
		}

		/**
		* Access the resource's properties.
		* @return The resource's properties.
		*/
		public ResourceProperties getProperties()
		{
			return m_properties;
		}

		/**
		* Access the creator of this DissertationStep.
		* @return The user id of the creator.
		*/
		public String getCreator()
		{
			return m_properties.getProperty(ResourceProperties.PROP_CREATOR);
		}

		/**
		* Access the user who last modified this DisserationStep.
		* @return the user id the the author who last modified the DissertationStep.
		*/
		public String getAuthorLastModified()
		{
			return m_properties.getProperty(ResourceProperties.PROP_MODIFIED_BY);
		}
		
		/**
		* Access the Time that this DissertationStep was last modified.
		* @return the Time last modified.
		*/
		public Time getTimeLastModified()
		{
			return m_timeLastModified;
		}
		
		/**
		* Access the site id for the DissertationStep's site.
		* @return the site id.
		*/
		public String getSite()
		{
			return m_site;
		}
		
		/**
		* Access the section id for the DissertationStep's checklist section.
		* @return the section id.
		*/
		public String getSection()
		{
			return m_section;
		}

		/**
		* Access the instructions for the DissertationStep with html links for user-entered links.
		* @return the step's instructions with html links for user-entered links.
		*/
		public String getInstructions()
		{
			StringBuffer retVal = new StringBuffer();
			boolean goodSyntax = true;
			String tempString = null;
			String linkString = null;
			String linkTextString = null;
			String fullLinkString = null;
			int midIndex = -1;
			int endIndex = -1;
			int beginIndex = m_instructionsText.indexOf("http://");
		
			if(beginIndex == -1)
			{
				//m_logger.debug("NO LINKS - RETURNING ORIGINAL TEXT");
				retVal.append(m_instructionsText);
			}
			else
			{
				do
				{
					// FIRST ADD THE TEXT UP TO THE LINK TO THE BUFFER
					//m_logger.debug("LINK FOUND : INDEX : " + beginIndex);
					retVal.append(m_instructionsText.substring((endIndex+1), beginIndex));
					midIndex = m_instructionsText.indexOf("{", beginIndex);
					//m_logger.debug("MID INDEX : " + midIndex);
					if(midIndex == -1)
					{
						// LINK IS NOT IN PROPER FORMAT - RETURN ORIGINAL STRING
						goodSyntax = false;
						beginIndex = -1;
						//m_logger.debug("MISSING {");
					}
					else
					{
						// FIND THE END TAG
						endIndex = m_instructionsText.indexOf("}", midIndex);
						//m_logger.debug("END INDEX : " + endIndex);
						if(endIndex == -1)
						{
							goodSyntax = false;
							//m_logger.debug("MISSING }");
							beginIndex = -1;
						}
						else
						{
							//m_logger.debug("SYNTAX IS GOOD THIS PASS - CREATING LINK\n");
							linkString = m_instructionsText.substring(beginIndex, midIndex);
							//m_logger.debug("LINK STRING : " + linkString);
							linkTextString = m_instructionsText.substring((midIndex+1), endIndex);
							//m_logger.debug("LINK TEXT STRING : " + linkTextString);
							fullLinkString = "<a href='" + linkString + "' target='_blank' " + ">" + linkTextString + "</a>";
							//m_logger.debug("FULL LINK STRING : " + fullLinkString);
							retVal.append(fullLinkString);
							beginIndex = m_instructionsText.indexOf("http://", endIndex);
							//m_logger.debug("\n\nLINK CREATED : NEW BEGIN INDEX : " + beginIndex);
						}
					}
					
					if(beginIndex == -1)
					{
						//m_logger.debug("NO MORE LINKS - ADDING REMAINING TEXT");
						tempString = m_instructionsText.substring((endIndex+1), m_instructionsText.length());
						//m_logger.debug("ADDING REMAINDER OF TEXT : " + tempString);
						retVal.append(tempString);
					}

				}while(beginIndex != -1);
				
			}//else
		
			if(!goodSyntax)
			{
				//m_logger.debug("BAD SYNTAX : RETURNING ORIGINAL STRING\n");
				retVal = new StringBuffer();
				retVal.append(m_instructionsText);
			}

			//m_logger.debug("RESULT STRING : " + retVal.toString());
			return retVal.toString();

		}//getInstructions


		/**
		* Access the instructions as entered by the user.
		* @return the instructions as entered by the user.
		*/
		public String getInstructionsText()
		{
			return m_instructionsText;
		}

		/**
		* Access the instructions as entered by the user, with a maximum length of 80 characters for display in selects.
		* Remove links to save space in the select drop down list and prevent truncation from breaking an active link
		* @return the instructions as entered by the user, with a maximum length of 80 characters for display in selects.
		*/
		public String getShortInstructionsText()
		{
			StringBuffer buffer = new StringBuffer();
			String retVal = null;
			String tempString = null;
			String linkString = null;
			String linkTextString = null;
			String fullLinkString = null;
			boolean goodSyntax = true;
			int midIndex = -1;
			int endIndex = -1;
			int tagIndex = -1;
			
			int beginIndex = m_instructionsText.indexOf("http://");
			if(beginIndex == -1)
			{
				//NO LINKS - RETURNING ORIGINAL TEXT
				buffer.append(m_instructionsText);
			}
			else
			{
				do
				{
					// FIRST ADD THE TEXT UP TO THE LINK TO THE BUFFER
					buffer.append(m_instructionsText.substring((endIndex+1), beginIndex));
					
					//FIND THE BEGIN TAG
					midIndex = m_instructionsText.indexOf("{", beginIndex);
					if(midIndex == -1)
					{
						// LINK IS NOT IN PROPER FORMAT - RETURN ORIGINAL STRING
						goodSyntax = false;
						beginIndex = -1;
						// MISSING {
					}
					else
					{
						// FIND THE END TAG
						endIndex = m_instructionsText.indexOf("}", midIndex);
						if(endIndex == -1)
						{
							goodSyntax = false;
							// MISSING }
							beginIndex = -1;
						}
						else
						{
							// SYNTAX IS GOOD THIS PASS - KEEP LINK TEXT STRING
							linkTextString = m_instructionsText.substring((midIndex+1), endIndex);
							buffer.append(linkTextString);
							beginIndex = m_instructionsText.indexOf("http://", endIndex);
						}
					}
					if(beginIndex == -1)
					{
						// NO MORE LINKS - ADDING REMAINING TEXT
						tempString = m_instructionsText.substring((endIndex+1), m_instructionsText.length());
						
						// ADDING REMAINDER OF TEXT
						buffer.append(tempString);
					}

				}while(beginIndex != -1);
				
			}//else
		
			if(!goodSyntax)
			{
				//BAD SYNTAX : RETURNING ORIGINAL STRING
				buffer = new StringBuffer();
				buffer.append(m_instructionsText);
			}

			//RESULT STRING
			if(buffer.length() > 80)
			{
				retVal = ((String)buffer.toString()).substring(0, 79) + " . . . ";
			}
			else
			{
				retVal = buffer.toString();
			}
			return retVal;
			
		}//getShortInstructionsText
		
		/**
		* Access the id used for Rackham auto-validation.
		* @return the auto-validation id.
		*/
		public String getAutoValidationId()
		{
			return m_autoValidationId;
		}
		
		/**
		* Access the prerequisites for this step.
		* @return A List containing the references of the prerequisites for this step.
		*/
		public List getPrerequisiteStepReferences()
		{
			return m_prerequisiteSteps;
		}
		
		/**
		* Access the prerequisites for this step.
		* @return A List containing the prerequisite DissertationStep objects.
		*/
		public List getPrerequisiteSteps()
		{
			List retVal = new Vector();
			DissertationStep tempStep = null;
			
			for(int x = 0; x < m_prerequisiteSteps.size(); x++)
			{
				try
				{
					tempStep = getDissertationStep((String) m_prerequisiteSteps.get(x));
					if(tempStep != null)
						retVal.add(tempStep);
				}
				catch(Exception e){}
			}
			
			return retVal;
		}

		/**
		* See whether this step contains this prerequisite.
		* @return true if this step has this prerequisite, false otherwise.
		*/
		public boolean hasPrerequisiteStep(String stepReference)
		{
			return m_prerequisiteSteps.contains(stepReference);
		}
		
		/**
		* See whether this step has prerequisites.
		* @return true if the step has prerequisites, false otherwise.
		*/
		public boolean hasPrerequisites()
		{
			if(m_prerequisiteSteps.size() > 0)
				return true;
			else
				return false;
		}
		
		/**
		* Access the validation type of this step.
		* @return the type of permissions required to mark a step as completed.  See the DissertationStep interface.
		*/
		public String getValidationType()
		{
			return m_validationType;
		}
		
		/**
		* Access the validation type as a human-readable string.
		* @return The human-readable string corresonding to this step's validation type.
		*/
		public String getValidationTypeString()
		{
			String retVal = null;

			if(m_validationType.equals(DissertationStep.STEP_VALIDATION_TYPE_STUDENT))
				retVal = DissertationStep.STUDENT_VALIDATION_STRING;
			else if(m_validationType.equals(DissertationStep.STEP_VALIDATION_TYPE_CHAIR))
				retVal = DissertationStep.CHAIR_VALIDATION_STRING;
			else if(m_validationType.equals(DissertationStep.STEP_VALIDATION_TYPE_COMMITTEE))
				retVal = DissertationStep.COMMITTEE_VALIDATION_STRING;
			else if(m_validationType.equals(DissertationStep.STEP_VALIDATION_TYPE_DEPARTMENT))
				retVal = DissertationStep.DEPARTMENT_VALIDATION_STRING;
			else if(m_validationType.equals(DissertationStep.STEP_VALIDATION_TYPE_SCHOOL))
				retVal = DissertationStep.SCHOOL_VALIDATION_STRING;
			else if(m_validationType.equals(DissertationStep.STEP_VALIDATION_TYPE_STUDENT_CHAIR))
				retVal = DissertationStep.STUDENT_CHAIR_VALIDATION_STRING;
			else if(m_validationType.equals(DissertationStep.STEP_VALIDATION_TYPE_STUDENT_COMMITTEE))
				retVal = DissertationStep.STUDENT_COMMITTEE_VALIDATION_STRING;
			else if(m_validationType.equals(DissertationStep.STEP_VALIDATION_TYPE_STUDENT_DEPARTMENT))
				retVal = DissertationStep.STUDENT_DEPARTMENT_VALIDATION_STRING;
			else if(m_validationType.equals(DissertationStep.STEP_VALIDATION_TYPE_STUDENT_SCHOOL))
				retVal = DissertationStep.STUDENT_SCHOOL_VALIDATION_STRING;
			else
				retVal = "Unknown Validation Type";


			return retVal;
		}

		/**
		* Are these objects equal?  If they are both DissertationStep objects, and they have
		* matching id's, they are.
		* @return true if they are equal, false if not.
		*/
		public boolean equals(Object obj)
		{
			if (!(obj instanceof DissertationStep)) return false;
			return ((DissertationStep)obj).getId().equals(getId());

		}   // equals

		/**
		* Make a hash code that reflects the equals() logic as well.
		* We want two objects, even if different instances, if they have the same id to hash the same.
		*/
		public int hashCode()
		{
			return getId().hashCode();

		}	// hashCode

		/**
		* Compare this object with the specified object for order.
		* @return A negative integer, zero, or a positive integer as this object is
		* less than, equal to, or greater than the specified object.
		*/
		public int compareTo(Object obj)
		{
			if (!(obj instanceof DissertationStep)) throw new ClassCastException();

			// if the object are the same, say so
			if (obj == this) return 0;

			// start the compare by comparing their sort names
			int compare = getInstructionsText().compareTo(((DissertationStep)obj).getInstructionsText());

			// if these are the same
			if (compare == 0)
			{
				// sort based on (unique) id
				compare = getId().compareTo(((DissertationStep)obj).getId());
			}

			return compare;

		}	// compareTo

	}//BaseDissertationStep

	

	/*******************************************************************************
	* DissertationStepEdit implementation
	*******************************************************************************/	
	
	public class BaseDissertationStepEdit
		extends BaseDissertationStep
		implements DissertationStepEdit, SessionStateBindingListener
	{
		/** The event code for the edit. */
		protected String m_event = null;
		
		/** Active flag. */
		protected boolean m_active = false;


		/**
		* Constructor.
		* @param id The DissertationStepEdit id.
		* @param site The site id.
		*/
		public BaseDissertationStepEdit(String id, String site)
		{
			super(id, site);

		}   // BaseDissertationStepEdit

		
		/**
		* Copy constructor.
		* @param step The DissertationStepEdit to be copied.
		*/
		public BaseDissertationStepEdit(DissertationStep step)
		{
			super(step);

		}	// BaseDissertationStepEdit
		
		
		/**
		* Construct from an existing definition, in xml.
		* @param el The DissertationStepEdit in XML in a DOM element.
		*/
		public BaseDissertationStepEdit(Element el)
		{
			super(el);

		}	// BaseDissertationStepEdit


		/**
		* Clean up.
		*/
		protected void finalize()
		{
			// catch the case where an edit was made but never resolved
			if (m_active)
			{
				cancelEdit(this);
			}

		}	// finalize
		
		/**
		* Set the site for this DissertationStepEdit.
		* @param site The site id.
		*/
		public void setSite(String site)
		{
			m_site = site;
		}
		
		/**
		* Set the checklist section for this DissertationStepEdit.
		* @param section The section id.
		*/
		public void setSection(String section)
		{
			m_section = section;
		}
		
		/**
		* Set the instructions for this DissertationStepEdit.
		* @param instructionsText The instructions.
		*/
		public void setInstructionsText(String instructionsText)
		{
			if(instructionsText != null)
				m_instructionsText = Validator.escapeHtml(instructionsText);
		}

		/**
		* Add a prerequisite to this step.
		* @param stepReference The reference for the prerequisite DissertationStep to be added.
		*/
		public void addPrerequisiteStep(String stepReference)
		{
			m_prerequisiteSteps.add(stepReference);
		}

		/**
		* Remove a prerequisite to this step.
		* @param stepReference The reference for the prerequisite DissertationStep to be removed.
		*/
		public void removePrerequisiteStep(String stepReference)
		{
			m_prerequisiteSteps.remove(stepReference);
		}
		
		/**
		* Set the validation type for this step.
		* @param type The type of permissions required to mark this step as completed.  See the DissertationStep interface for values.
		*/
		public void setValidationType(String type)
		{
			m_validationType = type;
		}

		/**
		* Set the auto-validation id for this step.
		* @param validId The id for auto-validation from the Rackham database.
		*/
		public void setAutoValidationId(String validId)
		{
			if(validId != null)
				m_autoValidationId = validId;
		}

		/**
		* Take all values from this object.
		* @param user The user object to take values from.
		*/
		protected void set(DissertationStep step)
		{
			setAll(step);

		}   // set

		/**
		* Access the event code for this edit.
		* @return The event code for this edit.
		*/
		protected String getEvent() { return m_event; }
	
		/**
		* Set the event code for this edit.
		* @param event The event code for this edit.
		*/
		protected void setEvent(String event) { m_event = event; }

		/**
		* Access the resource's properties for modification
		* @return The resource's properties.
		*/
		public ResourcePropertiesEdit getPropertiesEdit()
		{
			return m_properties;

		}	// getPropertiesEdit

		/**
		* Enable editing.
		*/
		protected void activate()
		{
			m_active = true;

		}	// activate

		/**
		* Check to see if the edit is still active, or has already been closed.
		* @return true if the edit is active, false if it's been closed.
		*/
		public boolean isActiveEdit()
		{
			return m_active;

		}	// isActiveEdit

		/**
		* Close the edit object - it cannot be used after this.
		*/
		protected void closeEdit()
		{
			m_active = false;

		}	// closeEdit

		/*******************************************************************************
		* SessionStateBindingListener implementation
		*******************************************************************************/
	
		/**
		* Accept notification that this object has been bound as a SessionState attribute.
		* @param sessionStateKey The id of the session state which holds the attribute.
		* @param attributeName The id of the attribute to which this object is now the value.
		*/
		public void valueBound(String sessionStateKey, String attributeName) {}
	
		/**
		* Accept notification that this object has been removed from a SessionState attribute.
		* @param sessionStateKey The id of the session state which held the attribute.
		* @param attributeName The id of the attribute to which this object was the value.
		*/
		public void valueUnbound(String sessionStateKey, String attributeName)
		{
			if (m_logger.isDebugEnabled())
			{
				//m_logger.debug(this + ".valueUnbound()");
			}
	
			// catch the case where an edit was made but never resolved
			if (m_active)
			{
				cancelEdit(this);
			}
	
		}	// valueUnbound


	}//BaseDissertationStepEdit


	
	/*******************************************************************************
	* CandidatePath implementation
	*******************************************************************************/	
	
	public class BaseCandidatePath
		implements CandidatePath
	{
		/** The CandidatePath id. */
		protected String m_id;
		
		/** The candidate user id. */
		protected String m_candidate;
		
		/** The candidate's advisor's user id. */
		protected String m_advisor;
		
		/** The site id. */
		protected String m_site;
		
		/** The dissertation type set when created. */
		protected String m_type;
		
		/** The StepStatus references, keyed by display order, starting with 1. */
		protected Hashtable m_orderedStatus;
		
		/** The List objects containing StepStatus references, which are
			the prerequisites added to School steps by department administrators for
			their department only.  Keyed by StepStatus reference. */
		protected Hashtable m_schoolPrereqs;
		
		/** The properties. */
		protected ResourcePropertiesEdit m_properties;


		/**
		* Constructor.
		* @param pathId The CandidatePath id.
		* @param site The site id.
		*/
		public BaseCandidatePath(String pathId, String site)
		{
			m_id = pathId;
			m_site = site;
			m_advisor = "";
			m_properties = new BaseResourcePropertiesEdit();
			m_orderedStatus = new Hashtable();
			m_schoolPrereqs = new Hashtable();
			User currentUser = UsageSessionService.getSessionUser();
			if(currentUser != null)
			{
				m_candidate = currentUser.getId();
			}
			else
			{
				m_candidate = "";
				m_logger.warn("In BaseDissertationService, BaseCandidatePath constructor : call to UserDirectoryService.getSessionUser() returns null.");
			}
		}
		
		/**
		* Copy constructor.
		* @param path The CandidatePath to copy.
		*/
		public BaseCandidatePath(CandidatePath path)
		{
			setAll(path);

		}	// BaseCandidatePath

		/**
		* Construct from an existing definition, in xml.
		* @param el The CandidatePath in XML in a DOM element.
		*/
		public BaseCandidatePath(Element el)
		{
			m_properties = new BaseResourcePropertiesEdit();
			String keyString = null;
			String valueString = null;
			List schoolPrereqs = null;
			m_orderedStatus = new Hashtable();
			m_schoolPrereqs = new Hashtable();

			m_id = el.getAttribute("id");
			m_candidate = el.getAttribute("candidate");
			m_advisor = el.getAttribute("advisor");
			m_site = el.getAttribute("site");

				// READ THE PROPERTIES
			NodeList children = el.getChildNodes();
			final int length = children.getLength();
			for(int i = 0; i < length; i++)
			{
				Node child = children.item(i);
				if (child.getNodeType() != Node.ELEMENT_NODE) continue;
				Element element = (Element)child;

				// look for properties
				if (element.getTagName().equals("properties"))
				{
					// re-create properties
					m_properties = new BaseResourcePropertiesEdit(element);
				}
				else if(element.getTagName().equals("orderedstatus"))
				{
					NodeList osChildren = element.getChildNodes();
					final int osLength = osChildren.getLength();
					Node osChild = null;
					Element osElement = null;
					for(int x = 0; x < osLength; x++)
					{
						osChild = osChildren.item(x);
						if (osChild.getNodeType() != Node.ELEMENT_NODE) continue;
						osElement = (Element)osChild;
						if(osElement.getTagName().equals("order"))
						{
							keyString = osElement.getAttribute("ordernum");
							valueString = osElement.getAttribute("statusid");
							if((keyString != null) && (valueString != null))
							{
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE STATUS : STORAGE CONSTRUCTOR : ORDERED STATUS KEY STRING   : " + keyString);
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE STATUS : STORAGE CONSTRUCTOR : ORDERED STATUS VALUE STRING : " + valueString);
								m_orderedStatus.put(keyString, valueString);
							}
						}
					}
				}
				else if(element.getTagName().equals("schoolstatusprereqs"))
				{
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATEPATH : STORAGE CONSTRUCTOR : ELEMENT SCHOOLSTATUSPREREQS");
					keyString = element.getAttribute("statusid");
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATEPATH : STORAGE CONSTRUCTOR : KEY STRING : " + keyString);
					schoolPrereqs = new Vector();
					NodeList spChildren = element.getChildNodes();
					final int spLength = spChildren.getLength();
					Node spChild = null;
					Element spElement = null;
					for(int x = 0; x < spLength; x++)
					{
						spChild = spChildren.item(x);
						if (spChild.getNodeType() != Node.ELEMENT_NODE) continue;
						spElement = (Element)spChild;
						if(spElement.getTagName().equals("prereq"))
						{
							valueString = spElement.getAttribute("prereqstatusreference");
							if(valueString != null)
							{
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATEPATH : STORAGE CONSTRUCTOR : SCHOOL PREREQ TO ADD : " + valueString);
								schoolPrereqs.add(valueString);
							}
						}
					}
					if(keyString != null)
						m_schoolPrereqs.put(keyString, schoolPrereqs);

				}
			}
			
		}// storage constructor

		/**
		* Deep copy of this object.
		* @param path - The CandidatePath object to be copied.
		*/
		protected void setAll(CandidatePath path)
		{
			m_id = path.getId();
			m_candidate = path.getCandidate();
			m_site = path.getSite();
			m_type = path.getType();
			m_advisor = path.getAdvisor();
			m_orderedStatus = path.getOrderedStatus();
			m_schoolPrereqs = path.getSchoolPrereqs();
			m_properties = new BaseResourcePropertiesEdit();
			m_properties.addAll(path.getProperties());

		}   // setAll
		
		
		/**
		* Serialize the resource into XML, adding an element to the doc under the top of the stack element.
		* @param doc The DOM doc to contain the XML (or null for a string return).
		* @param stack The DOM elements, the top of which is the containing element of the new "resource" element.
		* @return The newly added element.
		*/
		public Element toXml(Document doc, Stack stack)
		{
			Element path = doc.createElement("path");
			
			if (stack.isEmpty())
			{
				doc.appendChild(path);
			}
			else
			{
				((Element)stack.peek()).appendChild(path);
			}

			stack.push(path);
			
			String numItemsString = null;
			String attributeString = null;
			String itemString = null;
			String keyString = null;
			String intString = null;

			path.setAttribute("id", m_id);
			path.setAttribute("candidate", m_candidate);
			path.setAttribute("advisor", m_advisor);
			path.setAttribute("site", m_site);
			path.setAttribute("type", m_type);


				// SAVE THE ORDERED STATUS
			Element order = null;
			Element orderedStatus = doc.createElement("orderedstatus");
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : TOXML : CREATED ORDERED STATUS ELEMENT");
			for(int x = 1; x < (m_orderedStatus.size()+1); x++)
			{
				order = doc.createElement("order");
				keyString = "" + x;
				order.setAttribute("ordernum", keyString);
				itemString = (String)m_orderedStatus.get(keyString);
				if(itemString == null)
					itemString = "";
				order.setAttribute("statusid", itemString);
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION CP: TOXML : ORDER ATTRIBUTE ORDERNUM  : " + keyString);
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION CP: TOXML : ORDER ATTRIBUTE STATUS ID : " + itemString);
				orderedStatus.appendChild(order);
			}
			path.appendChild(orderedStatus);


				// SAVE THE SCHOOL STATUS PREREQS
			if(m_logger.isDebugEnabled())
			{
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISSERTATION : TOXML : NUMBER OF SCHOOL PREREQS : " + m_schoolPrereqs.size());
			}
			Element schoolStatusPrereqs = null;
			Element prereq = null;
			Enumeration keys = m_schoolPrereqs.keys();
			List schoolPrereqs = null;
			while(keys.hasMoreElements())
			{
				schoolStatusPrereqs = doc.createElement("schoolstatusprereqs");
				keyString = (String)keys.nextElement();
				schoolStatusPrereqs.setAttribute("statusid", keyString);
				schoolPrereqs = (List)m_schoolPrereqs.get(keyString);
				for(int x = 0; x < schoolPrereqs.size(); x++)
				{
					prereq = doc.createElement("prereq");
					prereq.setAttribute("prereqstatusreference", (String) schoolPrereqs.get(x));
					schoolStatusPrereqs.appendChild(prereq);
				}
				path.appendChild(schoolStatusPrereqs);
			}

			
			m_properties.toXml(doc, stack);
			
			stack.pop();
			
			return path;
		}
		
		
		/*******************************************************************************
		* Reference Implementation
		*******************************************************************************/
		
		/**
		* Access the CandidatePath id.
		* @return The CandidatePath id string.
		*/
		public String getId()
		{
			return m_id;
		}

		/**
		* Access the URL which can be used to access the resource.
		* @return The URL which can be used to access the resource.
		*/
		public String getUrl()
		{
			return getAccessPoint(false) + m_id;
		}

		/**
		* Access the internal reference which can be used to access the resource from within the system.
		* @return The the internal reference which can be used to access the resource from within the system.
		*/
		public String getReference()
		{
			return pathReference(m_site, m_id);
		}

		/**
		* Access the resource's properties.
		* @return The resource's properties.
		*/
		public ResourceProperties getProperties()
		{
			return m_properties;
		}

		/**
		* Access the candidate's advisor.
		* @return The candidate's advisor's user id.
		*/
		public String getAdvisor()
		{
			return m_advisor;
		}
		
		/**
		* Access the site id for the CandidatePath's site.
		* @return the site id.
		*/
		public String getSite()
		{
			return m_site;
		}
		
		/**
		* Access the dissertation type set at CandidatePath creation.
		* @return the type String.
		*/
		public String getType()
		{
			return m_type;
		}

		/**
		* Access the candidate who owns this CandidatePath.
		* @return the candidate's user id.
		*/
		public String getCandidate()
		{
			return m_candidate;
		}
		
		/**
		* Access the prerequisites added by department administrators to School steps for their department.
		* @return the Hashtable of Lists containing the references of prerequisite steps, keyed by DissertationStep reference.
		*/
		public Hashtable getSchoolPrereqs()
		{
			return m_schoolPrereqs;
		}

		
		/**
		* Access The completion status for a step.
		* @param status The StepStatus.
		* @return The completion status of the StepStatus, as a String.  See the StepStatus interface for the status Strings.
		*/
		public String getStatusStatus(StepStatus status)
		{
			//m_logger.debug("DISSERTATION : BASE SERVICE : ENTERING GET STATUS STATUS : " + status.getReference());
			String retVal = "";
			List deptAddedPrereqs = null;
			StepStatus aStatus = null;
			boolean prereqsCompleted;

			if(status != null)
			{
				if(status.getCompleted())
				{
					retVal = STEP_STATUS_COMPLETED;
					//m_logger.debug("DISSERTATION : BASE SERVICE : GET STATUS STATUS : STATUS OBJECT IS MARKED COMPLETED - RETURNING STATUS=COMPLETED");
				}
				else
				{
					//m_logger.debug("DISSERTATION : BASE SERVICE : GET STATUS STATUS : STATUS OBJECT IS MARKED NOT COMPLETED - CHECKING PREREQS");
					prereqsCompleted = true;
					//m_logger.debug("DISSERTATION : BASE SERVICE : GET STATUS STATUS : about to get prereqs");
					List prereqs = status.getPrereqs();
					//m_logger.debug("DISSERTATION : BASE SERVICE : GET STATUS STATUS : got prereqs from status object");
					//if(prereqs == null)
					//	//m_logger.debug("DISSERTATION : BASE SERVICE : GET STATUS STATUS : status.getPrereqs() RETURNS NULL !!!!!!!!!!!!!!!");
					//else
					//	//m_logger.debug("DISSERTATION : BASE SERVICE : GET STATUS STATUS : NUMBER OF PREREQS : " + prereqs.size());

					for(int x = 0; x < prereqs.size(); x++)
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : GET STATUS STATUS : ENTERING CHECK STATUS PREREQS LOOP : ITERATION : " + x);
						//m_logger.debug("DISSERTATION : BASE SERVICE : GET STATUS STATUS : ENTERING CHECK STATUS PREREQS LOOP : PREREQ ID : " + prereqs.get(x));
						try
						{
							aStatus = getStepStatus((String) prereqs.get(x));
						}
						catch(Exception e){}
						//m_logger.debug("DISSERTATION : BASE SERVICE : GET STATUS STATUS : ENTERING CHECK STATUS PREREQS LOOP : STATUS OBJ : " + aStatus);
						//m_logger.debug("DISSERTATION : BASE SERVICE : GET STATUS STATUS : GOT STATUS OBJECT : " + aStatus.getId());
						if(!aStatus.getCompleted())
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : GET STATUS STATUS : STATUS OBJECT MARKED NOT COMPLETED - RETURNING STATUS=PREREQS NOT COMPLETED");
							prereqsCompleted = false;
						}
						//else
						//	//m_logger.debug("DISSERTATION : BASE SERVICE : GET STATUS STATUS : STATUS OBJECT MARKED COMPLETED - CONTINUING");
					}
					
					//m_logger.debug("DISSERTATION : BASE SERVICE : GET STATUS STATUS : ***********************  LOOP 1 COMPLETED ************************");
					deptAddedPrereqs = getSchoolPrereqs(status.getReference());
					//m_logger.debug("DISSERTATION : BASE SERVICE : GET STATUS STATUS : GOT DEPARTMENT-ADDED PREREQS : " + deptAddedPrereqs);
					if(deptAddedPrereqs != null)
					{
						for(int x = 0; x < deptAddedPrereqs.size(); x++)
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : GET STATUS STATUS : GOT DEPT-ADDED STEP PREREQ : " + deptAddedPrereqs.get(x));
							try
							{
								aStatus = getStepStatus((String) deptAddedPrereqs.get(x));
							}
							catch(Exception e){}
							if(!aStatus.getCompleted())
							{
								//m_logger.debug("DISSERTATION : BASE SERVICE : GET STATUS STATUS : STATUS OBJECT MARKED NOT COMPLETED - RETURNING STATUS=PREREQS NOT COMPLETED");
								prereqsCompleted = false;
							}
							//else
							//	//m_logger.debug("DISSERTATION : BASE SERVICE : GET STATUS STATUS : STATUS OBJECT MARKED COMPLETED - CONTINUING");
						}
					}
					
					if(prereqsCompleted)
					{
						retVal = STEP_STATUS_PREREQS_COMPLETED;
						//m_logger.debug("DISSERTATION : BASE SERVICE : GET STATUS STATUS : METHOD COMPLETE - RETURNING : " + retVal);
					}
					else
					{
						retVal = STEP_STATUS_PREREQS_NOT_COMPLETED;
						//m_logger.debug("DISSERTATION : BASE SERVICE : GET STATUS STATUS : METHOD COMPLETE - RETURNING : " + retVal);
					}
				}
			}

			return retVal;
		}

		/**
		* Access the Hashtable containing the StepStatus.
		* @return The StepStatus keyed by display order.
		*/
		public Hashtable getOrderedStatus()
		{
			return m_orderedStatus;
		}
		
		/**
		* Access the reference of the StepStatus by display order.
		* @param order The order number as a String.
		* @return The reference of the StepStatus, or null if there is no match.
		*/
		public String getStatusReferenceByOrder(String order)
		{
			if(order == null)
				return null;
			else
				return (String)m_orderedStatus.get(order);
		}
		
		/**
		* Access the display order for a StepStatus.
		* @param statusReference The reference of the StepStatus.
		* @return The display order number as a String.
		*/
		public int getOrderForStatus(String statusReference)
		{
			int retVal = 0;
			try
			{
				String keyString = null;
				String valueString = null;
				Enumeration keys = m_orderedStatus.keys();
				while(keys.hasMoreElements())
				{
					keyString = (String)keys.nextElement();
					valueString = (String)m_orderedStatus.get(keyString);
					if(statusReference.equals(valueString))
					{
						retVal = Integer.parseInt(keyString);
					}
				}
			}
			catch(Exception e){}
			return retVal;
		}

		/**
		* Access the references of the prerequisite steps for a School step.
		* @return List containing the prerequisite step references.
		*/
		public List getSchoolPrereqs(String schoolStepRef)
		{
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : ENTERING GET SCHOOL PREREQS : REF : " + schoolStepRef);
			List retVal = null;
			if(m_schoolPrereqs == null)
			{
				;
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : M_SCHOOLPREREQS IS NULL !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
			}
			else
				retVal = (List)m_schoolPrereqs.get(schoolStepRef);

			if(retVal == null)
				retVal = new Vector();
			return retVal;
		}		


		/**
		* Access the StepStatus that corresponds with the DissertationStep.
		* @param step The DissertationStep.
		* @return The reference of the StepStatus object that corresponds with the DissertationStep.
		*/
		public String matchStepWithStatus(DissertationStep step)
		{
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : ENTERING MATCH-STEP-WITH-STATUS : STEP REF : " + step.getReference());
			String retVal = null;
			String key = null;
			String value = null;
			StepStatus tempStatus = null;
			Enumeration keys = m_orderedStatus.keys();
			while(keys.hasMoreElements())
			{
				try
				{
					key = (String)keys.nextElement();
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : MATCH-STEP-WITH-STATUS : STATUS KEY : " + key);
					value = (String)m_orderedStatus.get(key);
					tempStatus = getStepStatus(value);
					if(matches(step, tempStatus))
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : MATCH-STEP-WITH-STATUS : FOUND A MATCH : STATUS ID : " + tempStatus.getReference());
						retVal = value;
					}
				}
				catch(Exception e){}
			}
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : MATCH-STEP-WITH-STATUS : RETURNING : " + retVal);
			return retVal;
		}

		
		/**
		* Determine whether the DissertationStep is the parent of the StepStatus.
		* Used when making retroactive changes.
		* @return True if the DissertationStep matches the StepStatus, false otherwise.
		*/
		protected boolean matches(DissertationStep step, StepStatus status)
		{
			boolean retVal = false;
				// FIRST SEE IF THE STATUS' PARENT IS THE STEP
			if(step.getReference().equals(status.getParentStepReference()))
			{
				retVal = true;
			}
			else
			{
				if((step != null) && (status != null))
				{
					int numMatches = 0;
					if(step.getInstructionsText().equals(status.getInstructions()))
						numMatches++;
					if(step.getValidationType() == status.getValidationType())
						numMatches++;
					if(step.getSite().equals(status.getSite()))
						numMatches++;
					if(numMatches == 3)
						retVal = true;
				}
			}
			return retVal;
		}

		/**
		* Access the order numbers of the prerequisites in a comma-delimited string for display.
		* @param status The StepStatus.
		* @return The prerequisite steps display string.
		*/
		public String getPrerequisiteStepsDisplayString(StepStatus status)
		{
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : ENTERING GET PREREQUISITE STEPS DISPLAY STRING");
			StringBuffer retVal = new StringBuffer();
			
			if(status != null)
			{
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : GET PREREQUISITE STEPS DISPLAY STRING : STATUS ID : " + status.getId());
				
				String keyString = "";
				String statusReference = "";
				boolean firstAdded = true;
				boolean addToString = false;
				List deptAddedPrereqs = (List)m_schoolPrereqs.get(status.getReference());
				for(int x = 1; x < (m_orderedStatus.size()+1); x++)
				{
					addToString = false;
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : GET PREREQUISITE STEPS DISPLAY STRING : ORDERED STEPS ITERATION : " + x);
					keyString = "" + x;
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : GET PREREQUISITE STEPS DISPLAY STRING : KEY STRING : " + keyString);
					statusReference = (String)m_orderedStatus.get(keyString);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : GET PREREQUISITE STEPS DISPLAY STRING : STEP ID    : " + statusReference);
					
					if(status.hasPrerequisite(statusReference))
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : GET PREREQUISITE STEPS DISPLAY STRING : STEP HAS PREREQ : ADDING : " + keyString);
						addToString = true;
					}
					else
					{
						if(deptAddedPrereqs != null)
						{
							if(deptAddedPrereqs.contains(statusReference))
							{
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : GET PREREQUISITE STEPS DISPLAY STRING : DEPT ADDED PREREQS CONTAINS THIS STEP ID - ADDING : " + keyString);
								addToString = true;
							}
						}
					}
					
					if(addToString)
					{
						if(firstAdded)
						{
							retVal.append(keyString);
							firstAdded = false;
						}
						else
						{
							retVal.append(", " + keyString);
						}
					}
				}
			}
			
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : GET PREREQUISITE STEPS DISPLAY STRING : RETURNING : " + retVal.toString());
			return retVal.toString();		
		}

		/**
		* Are these objects equal?  If they are both User objects, and they have
		* matching id's, they are.
		* @return true if they are equal, false if not.
		*/
		public boolean equals(Object obj)
		{
			if (!(obj instanceof CandidatePath)) return false;
			return ((CandidatePath)obj).getId().equals(getId());

		}   // equals

		/**
		* Make a hash code that reflects the equals() logic as well.
		* We want two objects, even if different instances, if they have the same id to hash the same.
		*/
		public int hashCode()
		{
			return getId().hashCode();

		}	// hashCode

		/**
		* Compare this object with the specified object for order.
		* @return A negative integer, zero, or a positive integer as this object is
		* less than, equal to, or greater than the specified object.
		*/
		public int compareTo(Object obj)
		{
			if (!(obj instanceof CandidatePath)) throw new ClassCastException();

			// if the object are the same, say so
			if (obj == this) return 0;

			// start the compare by comparing their sort names
			int compare = getCandidate().compareTo(((CandidatePath)obj).getCandidate());

			// if these are the same
			if (compare == 0)
			{
				// sort based on (unique) id
				compare = getId().compareTo(((CandidatePath)obj).getId());
			}

			return compare;

		}	// compareTo

	}//BaseCandidatePath



	/*******************************************************************************
	* CandidatePathEdit implementation
	*******************************************************************************/	
	
	public class BaseCandidatePathEdit
		extends BaseCandidatePath
		implements CandidatePathEdit, SessionStateBindingListener
	{
		/** The event code for the edit. */
		protected String m_event = null;
		
		/** Active flag. */
		protected boolean m_active = false;

		/**
		* Constructor.
		* @param pathId The CandidatePath id.
		* @param site The site id.
		*/
		public BaseCandidatePathEdit(String pathId, String site)
		{
			super(pathId, site);

		}   // BaseCandidatePathEdit

		/**
		* Copy constructor.
		* @param path The CandidatePath to be copied.
		*/
		public BaseCandidatePathEdit(CandidatePath path)
		{
			super(path);

		}	// BaseCandidatePathEdit
		
		/**
		* Construct from an existing definition, in xml.
		* @param el The CandidatePathEdit in XML in a DOM element.
		*/
		public BaseCandidatePathEdit(Element el)
		{
			super(el);

		}	// BaseCandidatePathEdit
		

		/**
		* Clean up.
		*/
		protected void finalize()
		{
			// catch the case where an edit was made but never resolved
			if (m_active)
			{
				cancelEdit(this);
			}

		}	// finalize


		/**
		* Set the candidate for this CandidatePath.
		* @param candidateId The candidate's user id.
		*/
		public void setCandidate(String candidateId)
		{
			m_candidate = candidateId;
		}
		
		/**
		* Set the candidate for this CandidatePath.
		* @param candidateId The candidate's user id.
		*/
		public void setAdvisor(String advisor)
		{
			m_advisor = advisor;
		}

		/**
		* Set the site for this CandidatePath.
		* @param siteId The site id.
		*/
		public void setSite(String siteId)
		{
			m_site = siteId;
		}
		
		/**
		* Set the dissertation type for this CandidatePath.
		* @param type The type String.
		*/
		public void setType(String type)
		{
			m_type = type;
		}
		
		/**
		* Set the StepStatus for this CandidatePath.
		* @param hash The Hashtable of StepStatus references, keyed by display order as a String.
		*/
		public void setOrderedStatus(Hashtable hash)
		{
			if(hash != null)
				m_orderedStatus = hash;
		}

		/**
		* Set the prerequisites for School steps.
		* @param The Hashtable of Lists containing the references
		* to the prerequisite steps, keyed by School step reference.
		*/
		public void setSchoolPrereqs(Hashtable schoolPrereqs)
		{
			if(schoolPrereqs != null)
				m_schoolPrereqs = schoolPrereqs;
		}

		
		/**
		* Add a StepStatus object to the ordered status HashTable.
		* @param newStatus - the StepStatus to be added.
		* @param location - the order number.
		*/
		public void addToOrderedStatus(StepStatus newStatus, int locationInt)
		{
			Hashtable newOrder = new Hashtable();
			String keyString = null;
			String statusReference = null;
			for(int x = 1; x < (m_orderedStatus.size()+2); x++)
			{
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : addToOrderedStatus : loop pass # : " + x);
				if(locationInt == x)
				{
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : addToOrderedStatus : locationInt = x");
					keyString = "" + x;
					newOrder.put(keyString, newStatus.getReference());
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : addToOrderedStatus : keyString : " + keyString + " : newstatus : " + newStatus.getReference());
				}
				else if(locationInt < x)
				{
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : addToOrderedStatus : locationInt < x");
					keyString = "" + (x-1);
					statusReference = (String)m_orderedStatus.get(keyString);
					keyString = "" + (x);
					newOrder.put(keyString, statusReference);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : addToOrderedStatus : keyString : " + keyString + " : statusRef : " + statusReference);
				}
				else
				{
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : addToOrderedStatus : locationInt > x");
					keyString = "" + x;
					statusReference = (String)m_orderedStatus.get(keyString);
					newOrder.put(keyString, statusReference);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : addToOrderedStatus : keyString : " + keyString + " : statusRef : " + statusRef);
				}
			}
			m_orderedStatus = newOrder;
		}

		
		/**
		* Remove a StepStatus object from the ordered status HashTable.
		* @param statusToRemoveReference - the reference of the StepStatus to be removed.
		*/
		public void removeFromOrderedStatus(String statusToRemoveReference)
		{
			if(statusToRemoveReference != null)
			{
				Hashtable newOrder = new Hashtable();
				List schoolStepPrereqs = null;
				String keyString = null;
				String statusId = null;
				boolean foundIt = false;
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : REMOVE FROM ORDERED STATUS : REMOVING STATUS FROM CANDIDATE PATH : " + statusToRemoveReference);
				for(int y = 1; y < (m_orderedStatus.size()+1); y++)
				{
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : REMOVE FROM ORDERED STATUS : LOOP PASS # " + y);
					keyString = "" + y;
					statusId = (String)m_orderedStatus.get(keyString);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : REMOVE FROM ORDERED STATUS : key       : " + keyString);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : REMOVE FROM ORDERED STATUS : status id : " + statusId);
					
						// REMOVE THE STEP FROM ORDERED STEPS
					if(statusId.equals(statusToRemoveReference))
					{
						foundIt = true;
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : REMOVE FROM ORDERED STATUS : FOUND A MATCH !!!!!!!");
					}
					else
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : REMOVE FROM ORDERED STATUS : NO MATCH THIS PASS");
						if(foundIt)
						{
							keyString = "" + (y-1);
							newOrder.put(keyString, statusId);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : REMOVE FROM ORDERED STATUS : ALREADY FOUND : PUTTING : " + keyString + " : " + statusId);
						}
						else
						{
							newOrder.put(keyString, statusId);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : REMOVE FROM ORDERED STATUS : NOT YET FOUND : PUTTING : " + keyString + " : " + statusId);
						}
					}
				}
				m_orderedStatus = newOrder;
			}
		}

		
		/**
		* Move a StepStatus object within the ordered status HashTable.
		* @param statusToMoveReference - the reference of the StepStatus to be moved.
		* @param location - the new order number.
		*/
		public void moveStatus(String statusToMoveReference, String location)
		{
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : status to move id     : " + statusToMoveReference);
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : location status id    : " + location);
			Hashtable newOrder = new Hashtable();
			String keyString = null;
			String statusReference = null;
			
				// FIND THE ORDER NUMBERS FOR THE STATUS TO MOVE AND LOCATION
			int oldPosition = -1;
			int newPosition = -1;
			String aKey = null;
			String aValue = null;
			Enumeration keys = m_orderedStatus.keys();
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : ORDERED STATUS CONTENTS : ");
			if(location.equals("start"))
				newPosition = 1;

			while(keys.hasMoreElements())
			{
				aKey = (String)keys.nextElement();
				aValue = (String)m_orderedStatus.get(aKey);
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : KEY : VALUE : " + aKey + " : " + aValue);
				if(aValue.equals(statusToMoveReference))
				{
					try
					{
						oldPosition = Integer.parseInt(aKey);
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : parsed oldPosition int : " + oldPosition);
					}
					catch(Exception e){}
				}
				if(aValue.equals(location))
				{
					try
					{
						newPosition = Integer.parseInt(aKey);
						newPosition++;
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : parsed newPosition int : " + newPosition);
					}
					catch(Exception e){}
				}
			}

			if((oldPosition != -1) && (newPosition != -1))
			{
				if(newPosition > oldPosition)     // MOVING UP
				{
					newPosition--;
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : MOVING UP");
					for(int x = 1; x < (m_orderedStatus.size()+1); x++)
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : LOOP PASS # " + x);
						if((x < oldPosition) || (x > newPosition))
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : (x < old) || (x > new)");
							keyString = "" + x;
							statusReference = (String)m_orderedStatus.get(keyString);
							newOrder.put(keyString, statusReference);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : put : " + keyString + " : " + statusReference);
						}
						else if(x == newPosition)
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : x = new");
							keyString = "" + x;
							newOrder.put(keyString, statusToMoveReference);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : put : " + keyString + " : " + statusToMoveReference);
						}
						else
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : (x >= old) && (x < new)");
							keyString = "" + (x+1);
							statusReference = (String)m_orderedStatus.get(keyString);
							keyString = "" + x;
							newOrder.put(keyString, statusReference);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : put : " + keyString + " : " + statusReference);
						}
					}
					m_orderedStatus = newOrder;
				}
				else if(oldPosition > newPosition)		// MOVING DOWN
				{
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : MOVING DOWN");
					for(int x = 1; x < (m_orderedStatus.size()+1); x++)
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : LOOP PASS # " + x);
						if((x < newPosition) || (x > oldPosition))
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : (x < new) || (x > old)");
							keyString = "" + x;
							statusReference = (String)m_orderedStatus.get(keyString);
							newOrder.put(keyString, statusReference);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : put : " + keyString + " : " + statusReference);
						}
						else if(x == newPosition)
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : x = new ");
							keyString = "" + x;
							newOrder.put(keyString, statusToMoveReference);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : put : " + keyString + " : " + statusToMoveReference);
						}
						else
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : (x >= old) && (x < new)");
							keyString = "" + (x-1);
							statusReference = (String)m_orderedStatus.get(keyString);
							keyString = "" + x;
							newOrder.put(keyString, statusReference);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : put : " + keyString + " : " + statusReference);
						}
					}
					m_orderedStatus = newOrder;
				}
				
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : NEW ORDERED STATUS HASHTABLE CONTNENTS : ");
				
				for(int y = 1; y < (newOrder.size()+1); y++)
				{
					keyString = "" + y;
					statusReference = (String)newOrder.get(keyString);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : moveStatus : key : value : " + keyString + " : " + statusReference);
				}
			}
		}//moveStatus
		
		/**
		* Add a step to a CandidatePath in use.
		* @param step - The DissertationStep to be added.
		* @param previousStepReferences - The references of the steps preceeding the step to be added in the parent Dissertation.
		* @param siteId - the site id at the time of creation.
		*/
		public void liveAddStep(DissertationStep step, String[] previousStepReferences, String siteId)
		{
			try
			{
				//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : ENTERING LIVE ADD STEP ");
				//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : STEP ID       : " + step.getId());
				int numPrevSteps = 0;
				if(previousStepReferences != null)
				{
					numPrevSteps = previousStepReferences.length;
					//for(int a = 0; a < numPrevSteps; a++)
					//{
					//	Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : PREV STEP ID " + a + " : " + previousStepReferences[a]);
					//}
				}
				//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : SITE       : " + siteId);
				if(step != null)
				{
					boolean oardValidated = false;
					if(!(step.getAutoValidationId().equals("")))
						oardValidated = true;
					
					StepStatusEdit statusEdit = addStepStatus(siteId, step, oardValidated);
					//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : NEW STATUS OBJECT CREATED - CHECKING FOR PREREQ STEPS");
					List stepPrereqs = step.getPrerequisiteSteps();
					String statusPrereq = null;
					DissertationStep tempStep = null;
					if(stepPrereqs.size() > 0)
					{
						//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : PREREQS EXIST !!");
						//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : TRANSLATING PREREQ STEPS TO STATUS : ");
						for(int x = 0; x < stepPrereqs.size(); x++)
						{
							tempStep = (DissertationStep)stepPrereqs.get(x);
							//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : STEP PREREQ   : " + tempStep.getId());
							statusPrereq = matchStepWithStatus(tempStep);
							//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : STATUS PREREQ : " + statusPrereq);
							if(statusPrereq != null)
								statusEdit.addPrerequisiteStatus(statusPrereq);
						}
					}
					commitEdit(statusEdit);
					
					if(statusEdit.getSite().equals(getSchoolSite()))
					{
						//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CP : LIVE ADD STEP : SCHOOL STEP : CREATING SCHOOL-PREREQS FLEXSTRINGARRAY");
						// FIRST CREATE A SPOT IN SCHOOL-STEP-PREREQS
						m_schoolPrereqs.put(statusEdit.getReference(), new Vector());
					}

					// NOW LOOK FOR THE PREVIOUS STATUS IDS IN THE CANDIDATE PATH
					String previousStatusReference = null;
					boolean notFound = true;
					DissertationStep previousStep = null;
					for(int x = (numPrevSteps-1); x > -1; x--)
					{
						if(notFound)
						{
							//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : MATCHING STEP WITH STATUS");
							if(previousStepReferences[x].equals("start"))
							{
								//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : PREVIOUS STEP ID IS START");
								previousStatusReference = "start";
								notFound = false;
							}
							else
							{
								previousStep = getDissertationStep(previousStepReferences[x]);
								//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : GOT PREVIOUS STEP OBJECT : ID : " + previousStepReferences[x]);
								previousStatusReference = matchStepWithStatus(previousStep);
								//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : PREVIOUS STATUS ID : " + previousStatusReference);
								if(previousStatusReference != null)
									notFound = false;
							}
						}
					}
					//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : FOUND PREVIOUS STATUS ID : " + previousStatusReference);
				
					// PLACE THE NEW STATUS OBJECT IN THE CANDIDATE PATH
					int position = 0;
					if(previousStatusReference == null)
					{
						//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : NO MATCH FOR PREVIOUS STEP - WILL PUT AT END");
						position = m_orderedStatus.size();
						addToOrderedStatus(statusEdit, (position+1));
					}
					else
					{
						if(previousStatusReference.equals("start"))
						{
							//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : PREVIOUS STEP IS START - ADDING TO POSITION 1");
							addToOrderedStatus(statusEdit, 1);
						}
						else
						{
							if(m_orderedStatus.contains(previousStatusReference))
							{
								//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : FOUND PREVIOUS STEP IN THIS CANDIDATE PATH");
									// FIND THE KEY FOR THE PREVIOUS STEP
								String aKey = null;
								String aValue = null;
								String previousStepKey = null;
								Enumeration keys = m_orderedStatus.keys();
								while(keys.hasMoreElements())
								{
									aKey = (String)keys.nextElement();
									aValue = (String)m_orderedStatus.get(aKey);
									if(aValue.equals(previousStatusReference))
									{
										previousStepKey = aKey;
										//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : FOUND KEY FOR PREVIOUS STEP : " + aKey);
									}
								}

								position = Integer.parseInt(previousStepKey);
								//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : GOT POSITION INTEGER FOR KEY : " + position);
								addToOrderedStatus(statusEdit, (position+1));
								//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : ADDED STEP TO ORDERED STEPS");
							}
						}
					}
				}
				//Log.info("chef", "DISSERTATION : BASE SERVICE : BASE DISS : LEAVING LIVE ADD STEP ");
			}
			catch(Exception e)
			{
				m_logger.warn(this + ".BaseDissertationService.liveAddStep " + e);
			}
		}


		/**
		* Add a step to a CandidatePath in use.
		* @param step - The DissertationStep to be added.
		* @param previousStepReferences - The references of the steps preceeding the step to be added in the parent Dissertation.
		* @param siteId - the site id at the time of creation.
		*/
		public void liveAddStep(DissertationStep step, List previousStepReferences, String siteId)
		{
			try
			{
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : ENTERING LIVE ADD STEP ");
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : STEP ID       : " + step.getId());
				int numPrevSteps = 0;
				if(previousStepReferences != null)
				{
					numPrevSteps = previousStepReferences.size();
					//numPrevSteps = previousStepReferences.length;
					for(int a = 0; a < numPrevSteps; a++)
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : PREV STEP ID " + a + " : " + previousStepReferences.get(a));
					}
				}
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : SITE       : " + siteId);
				if(step != null)
				{
					boolean oardValidated = false;
					if(!(step.getAutoValidationId().equals("")))
						oardValidated = true;
					
					StepStatusEdit statusEdit = addStepStatus(siteId, step, oardValidated);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : NEW STATUS OBJECT CREATED - CHECKING FOR PREREQ STEPS");
					List stepPrereqs = step.getPrerequisiteSteps();
					String statusPrereq = null;
					DissertationStep tempStep = null;
					if(stepPrereqs.size() > 0)
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : PREREQS EXIST !!");
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : TRANSLATING PREREQ STEPS TO STATUS : ");
						for(int x = 0; x < stepPrereqs.size(); x++)
						{
							tempStep = (DissertationStep)stepPrereqs.get(x);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : STEP PREREQ   : " + tempStep.getId());
							statusPrereq = matchStepWithStatus(tempStep);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : STATUS PREREQ : " + statusPrereq);
							if(statusPrereq != null)
								statusEdit.addPrerequisiteStatus(statusPrereq);
						}
					}
					commitEdit(statusEdit);
					
					if(statusEdit.getSite().equals(getSchoolSite()))
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE ADD STEP : SCHOOL STEP : CREATING SCHOOL-PREREQS FLEXSTRINGARRAY");
						// FIRST CREATE A SPOT IN SCHOOL-STEP-PREREQS
						m_schoolPrereqs.put(statusEdit.getReference(), new Vector());
					}

					// NOW LOOK FOR THE PREVIOUS STATUS IDS IN THE CANDIDATE PATH
					String previousStatusReference = null;
					boolean notFound = true;
					DissertationStep previousStep = null;
					for(int x = (numPrevSteps-1); x > -1; x--)
					{
						if(notFound)
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : MATCHING STEP WITH STATUS");
							//if(previousStepReferences[x].equals("start"))
							if(previousStepReferences.get(x).equals("start"))
							{
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : PREVIOUS STEP ID IS START");
								previousStatusReference = "start";
								notFound = false;
							}
							else
							{
								previousStep = getDissertationStep((String)previousStepReferences.get(x));
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : GOT PREVIOUS STEP OBJECT : ID : " + (String)previousStepReferences.get(x));
								previousStatusReference = matchStepWithStatus(previousStep);
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : PREVIOUS STATUS ID : " + previousStatusReference);
								if(previousStatusReference != null)
									notFound = false;
							}
						}
					}
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : FOUND PREVIOUS STATUS ID : " + previousStatusReference);
				
					// PLACE THE NEW STATUS OBJECT IN THE CANDIDATE PATH
					int position = 0;
					if(previousStatusReference == null)
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : NO MATCH FOR PREVIOUS STEP - WILL PUT AT END");
						position = m_orderedStatus.size();
						addToOrderedStatus(statusEdit, (position+1));
					}
					else
					{
						if(previousStatusReference.equals("start"))
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : PREVIOUS STEP IS START - ADDING TO POSITION 1");
							addToOrderedStatus(statusEdit, 1);
						}
						else
						{
							if(m_orderedStatus.contains(previousStatusReference))
							{
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : FOUND PREVIOUS STEP IN THIS CANDIDATE PATH");
									// FIND THE KEY FOR THE PREVIOUS STEP
								String aKey = null;
								String aValue = null;
								String previousStepKey = null;
								Enumeration keys = m_orderedStatus.keys();
								while(keys.hasMoreElements())
								{
									aKey = (String)keys.nextElement();
									aValue = (String)m_orderedStatus.get(aKey);
									if(aValue.equals(previousStatusReference))
									{
										previousStepKey = aKey;
										//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : FOUND KEY FOR PREVIOUS STEP : " + aKey);
									}
								}

								position = Integer.parseInt(previousStepKey);
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : GOT POSITION INTEGER FOR KEY : " + position);
								addToOrderedStatus(statusEdit, (position+1));
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : ADDED STEP TO ORDERED STEPS");
							}
						}
					}
				}
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE DISS : LEAVING LIVE ADD STEP ");
			}
			catch(Exception e)
			{
				m_logger.warn("DISSERTATION : BASE SERVICE : BASE CANDIDATE PATH : LIVE ADD STEP : EXCEPTION : " + e);
			}
		}
		
		/**
		* Add a StepStatus to a CandidatePath in use, based on the DissertationStep.
		* @param step - The DissertationStep to be added.
		*/
		public void liveRemoveStep(DissertationStep step)
		{
			try
			{
					// FIND THE STATUS FOR THIS STEP
				String statusReference = matchStepWithStatus(step);
				
					// REMOVE ANY PREREQUISITE REFERENCES TO THIS STATUS
				StepStatusEdit statusToRemoveEdit = editStepStatus(statusReference);
				StepStatus status = null;
				StepStatusEdit statusEdit = null;
				List prereqs = null;
				Enumeration keys = m_orderedStatus.keys();
				String key = null;
				String value = null;
				while(keys.hasMoreElements())
				{
					key = (String)keys.nextElement();
					value = (String)m_orderedStatus.get(key);
					status = getStepStatus(value);
					prereqs = status.getPrereqs();
					if(prereqs.contains(statusReference))
					{
						statusEdit = editStepStatus(status.getReference());
						prereqs.remove(statusReference);
						commitEdit(statusEdit);
					}
				}
				
				keys = m_schoolPrereqs.keys();
				while(keys.hasMoreElements())
				{
					key = (String)keys.nextElement();
					prereqs = (List)m_schoolPrereqs.get(key);
					prereqs.remove(statusReference);
				}
				
					// REMOVE THE STATUS FROM ALL DATA STRUCTURES AND STORAGE
				removeFromOrderedStatus(statusReference);
				m_schoolPrereqs.remove(statusReference);
				removeStepStatus(statusToRemoveEdit);
			}
			catch(Exception e)
			{
				m_logger.warn("DISSERTATION : BASE SERVICE : BASE CP : LIVE REMOVE FROM ORDERED STATUS : EXCEPTION :" + e);
			}
		}
		
		/**
		* Update a StepStatus in this CandidatePath, while the CandidatePath is in use.
		* @param before - The DissertationStep to be updated, before changes were made.  Used to find the matching StepStatus object.
		* @param after - The same DissertationStep, after changes.
		*/
		public void liveUpdateStep(DissertationStep before, DissertationStepEdit after)
		{
			try
			{
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : ENTERING LIVE-UPDATE-STEP : BEFORE ID : " + before.getId());
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : ENTERING LIVE-UPDATE-STEP : BEFORE ID : " + after.getId());
					// FIND THE STATUS FOR THIS STEP
				String statusReference = matchStepWithStatus(before);
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE-UPDATE-STEP : MATCHED STATUS ID : " + statusReference);
				if(statusReference != null)
				{
					StepStatusEdit statusEdit = editStepStatus(statusReference);
					statusEdit.setInstructions(after.getInstructionsText());
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE-UPDATE-STEP : UPDATED INSTR : " + statusEdit.getInstructions());
					statusEdit.setValidationType(after.getValidationType());
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE-UPDATE-STEP : UDDATED VTYPE : " + statusEdit.getValidationType());
					List prereqStatus = new Vector();
					List prereqSteps = after.getPrerequisiteStepReferences();
					DissertationStep step = null;
					String prereqStatusReference = null;
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE-UPDATE-STEP : TRANSLATING STEP PREREQS INTO STATUS PREREQS : ");
					for(int x = 0; x < prereqSteps.size(); x++)
					{
						step = getDissertationStep((String) prereqSteps.get(x));
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE-UPDATE-STEP : STEP PREREQ   : " + prereqSteps[x]);
						prereqStatusReference = matchStepWithStatus(step);
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE-UPDATE-STEP : STATUS PREREQ : " + prereqStatusReference);
						if(prereqStatusReference != null)
							prereqStatus.add(prereqStatusReference);
					}

					statusEdit.setPrereqs(prereqStatus);
					commitEdit(statusEdit);
				}
			}
			catch(Exception e)
			{
				m_logger.warn("DISSERTATION : BASE SERVICE : BASE CP : LIVE UPDATE STEP : EXCEPTION :" + e);
			}
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE-UPDATE-STEP : LEAVING ");
			
		}//liveUpdateStep
		
		
		/**
		* Move a StepStatus within a CandidatePath in use.
		* @param step - The DissertationStep which is the parent of the StepStatus to be moved.
		* @param location - The reference of the DissertationStep location previous to the new position.
		* @param previousStepPosition - The reference of the step preceeding the step to be moved in the parent Dissertation.
		*/
		public void liveMoveStep(DissertationStep step, String location, String previousStepPosition)
		{
			try
			{
					// FIND THE STATUS FOR THIS STEP
				String statusRef = matchStepWithStatus(step);
				
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : step to move id       : " + step.getId());
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : location step ref      : " + location);
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : prev step position    : " + previousStepPosition);
				
				DissertationStep locationStep = null;
				Hashtable newOrder = new Hashtable();
				String keyString = null;
				String stepRef = null;
				
				
				int oldPosition = -1;
				int newPosition = -1;
				String locationStatusRef = "";				
				if(location.equals("start"))
				{
					newPosition = 1;
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : ");
				}
				else
				{
					locationStep = getDissertationStep(location);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : GOT LOCATION STEP OBJECT : ID : " + locationStep.getId());
					locationStatusRef = matchStepWithStatus(locationStep);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : GOT LOCATION STATUS REF : " + locationStatusRef);
				}

					// FIND THE ORDER NUMBERS FOR THE STEP TO MOVE AND LOCATION
				String aKey = null;
				String aValue = null;
				Enumeration keys = m_orderedStatus.keys();
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : ORDERED STEPS CONTENTS : ");
				while(keys.hasMoreElements())
				{
					aKey = (String)keys.nextElement();
					aValue = (String)m_orderedStatus.get(aKey);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : KEY : VALUE : " + aKey + " : " + aValue);
					if(aValue.equals(statusRef))
					{
						try
						{
							oldPosition = Integer.parseInt(aKey);
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : parsed oldPosition int : " + oldPosition);
						}
						catch(Exception e){}
					}
					if(aValue.equals(locationStatusRef))
					{
						try
						{
							newPosition = Integer.parseInt(aKey);
							newPosition++;
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : parsed newPosition int : " + newPosition);
						}
						catch(Exception e){}
					}
				}

				if((oldPosition != -1) && (newPosition != -1))
				{
					if(newPosition > oldPosition)     // MOVING UP
					{
						newPosition--;
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : MOVING UP");
						for(int x = 1; x < (m_orderedStatus.size()+1); x++)
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : LOOP PASS # " + x);
							if((x < oldPosition) || (x > newPosition))
							{
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : (x < old) || (x > new)");
								keyString = "" + x;
								stepRef = (String)m_orderedStatus.get(keyString);
								newOrder.put(keyString, stepRef);
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : put : " + keyString + " : " + stepRef);
							}
							else if(x == newPosition)
							{
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : x = new");
								keyString = "" + x;
								newOrder.put(keyString, statusRef);
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : put : " + keyString + " : " + statusRef);
							}
							else
							{
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : (x >= old) && (x < new)");
								keyString = "" + (x+1);
								stepRef = (String)m_orderedStatus.get(keyString);
								keyString = "" + x;
								newOrder.put(keyString, stepRef);
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : put : " + keyString + " : " + stepRef);
							}
						}
						m_orderedStatus = newOrder;
					}
					else if(oldPosition > newPosition)		// MOVING DOWN
					{
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : MOVING DOWN");
						for(int x = 1; x < (m_orderedStatus.size()+1); x++)
						{
							//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : LOOP PASS # " + x);
							if((x < newPosition) || (x > oldPosition))
							{
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : (x < new) || (x > old)");
								keyString = "" + x;
								stepRef = (String)m_orderedStatus.get(keyString);
								newOrder.put(keyString, stepRef);
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : put : " + keyString + " : " + stepRef);
							}
							else if(x == newPosition)
							{
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : x = new ");
								keyString = "" + x;
								newOrder.put(keyString, statusRef);
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : put : " + keyString + " : " + statusRef);
							}
							else
							{
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : (x >= old) && (x < new)");
								keyString = "" + (x-1);
								stepRef = (String)m_orderedStatus.get(keyString);
								keyString = "" + x;
								newOrder.put(keyString, stepRef);
								//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : put : " + keyString + " : " + stepRef);
							}
						}
						m_orderedStatus = newOrder;
					}
				
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : NEW ORDERED STEPS HASHTABLE CONTNENTS : ");
				
					for(int y = 1; y < (newOrder.size()+1); y++)
					{
						keyString = "" + y;
						stepRef = (String)newOrder.get(keyString);
						//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : key : value : " + keyString + " : " + stepRef);
					}
				}		
			}
			catch(Exception e)
			{
				m_logger.warn("DISSERTATION : BASE SERVICE : BASE CP : LIVE MOVE STEP : EXCEPTION :" + e);
			}
			
		}//liveMoveStep

		
		/**
		* Add a prerequisite to a StepStatus in a CandidatePath in use.
		* @param stepRef - the reference of the step to which the prerequisite is to be added.
		* @param prereqRef - the reference of the prerequisite to be added.
		*/
		public void liveAddSchoolPrereq(String stepRef, String prereqRef)
		{
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : ENTERING LIVE-ADD-SCHOOL-PREREQ : STEP REF   : " + stepRef);
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : ENTERING LIVE-ADD-SCHOOL-PREREQ : PREREQ REF : " + prereqRef);
			// FIRST MAKE SURE THESE STEPS EXIST
			try
			{
				DissertationStep step = getDissertationStep(stepRef);
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE-ADD-SCHOOL-PREREQ : STEP   : " + step.getId());
				DissertationStep prereq = getDissertationStep(prereqRef);
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE-ADD-SCHOOL-PREREQ : PREREQ : " + prereq.getId());

				List schoolPrereqs = null;
				
				String statusRef = matchStepWithStatus(step);
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE-ADD-SCHOOL-PREREQ : MATCHED STATUS ID : " + statusRef);
				String prereqStatusReference = matchStepWithStatus(prereq);
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE-ADD-SCHOOL-PREREQ : MATCHED PREREQ ID : " + prereqStatusReference);
				
				if((statusRef != null) && (prereqStatusReference != null))
				{
					schoolPrereqs = (List)m_schoolPrereqs.get(statusRef);
					if(schoolPrereqs == null)
						schoolPrereqs = new Vector();
					
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE-ADD-SCHOOL-PREREQ : GOT SCHOOL PREREQS FOR STATUS : NUMBER EXISTING : " + schoolPrereqs.size());
					if(schoolPrereqs != null)
					{
						schoolPrereqs.add(prereqStatusReference);
						m_schoolPrereqs.put(statusRef, schoolPrereqs);
					}
				}
			}
			catch(Exception e){}
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE-ADD-SCHOOL-PREREQ : LEAVING");

		}//liveAddSchoolPrereq


		/**
		* Remove a prerequisite from a StepStatus in a CandidatePath in use.
		* @param stepRef - the reference of the step from which the prerequisite is to be removed.
		* @param prereqRef - the reference of the prerequisite to be removed.
		*/
		public void liveRemoveSchoolPrereq(String stepRef, String prereqRef)
		{
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : ENTERING LIVE-REMOVE-SCHOOL-PREREQ : STEP REF   : " + stepRef);
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : ENTERING LIVE-REMOVE-SCHOOL-PREREQ : PREREQ REF : " + prereqRef);
			// FIRST MAKE SURE THESE STEPS EXIST
			try
			{
				DissertationStep step = getDissertationStep(stepRef);
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE-REMOVE-SCHOOL-PREREQ : STEP   : " + step.getId());
				DissertationStep prereq = getDissertationStep(prereqRef);
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE-REMOVE-SCHOOL-PREREQ : PREREQ : " + prereq.getId());
				List schoolPrereqs = null;

				String statusRef = matchStepWithStatus(step);
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE-REMOVE-SCHOOL-PREREQ : MATCHED STATUS REF : " + statusRef);
				String prereqStatusReference = matchStepWithStatus(prereq);
				//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE-REMOVE-SCHOOL-PREREQ : MATCHED PREREQ REF : " + prereqStatusReference);
				
				if((statusRef != null) && (prereqStatusReference != null))
				{
					schoolPrereqs = (List)m_schoolPrereqs.get(statusRef);
					//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE-REMOVE-SCHOOL-PREREQ : GOT SCHOOL PREREQS FOR STATUS : NUMBER EXISTING : " + schoolPrereqs.size());
					if(schoolPrereqs != null)
					{
						schoolPrereqs.remove(prereqStatusReference);
					}
				}
			}
			catch(Exception e){}
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CP : LIVE-REMOVE-SCHOOL-PREREQ : LEAVING");

		}//liveRemoveSchoolPrereq


		/**
		* Take all values from this object.
		* @param path The CandidatePath object to take values from.
		*/
		protected void set(CandidatePath path)
		{
			setAll(path);

		}   // set

		/**
		* Access the event code for this edit.
		* @return The event code for this edit.
		*/
		protected String getEvent() { return m_event; }
	
		/**
		* Set the event code for this edit.
		* @param event The event code for this edit.
		*/
		protected void setEvent(String event) { m_event = event; }

		/**
		* Access the resource's properties for modification
		* @return The resource's properties.
		*/
		public ResourcePropertiesEdit getPropertiesEdit()
		{
			return m_properties;

		}	// getPropertiesEdit

		/**
		* Enable editing.
		*/
		protected void activate()
		{
			m_active = true;

		}	// activate

		/**
		* Check to see if the edit is still active, or has already been closed.
		* @return true if the edit is active, false if it's been closed.
		*/
		public boolean isActiveEdit()
		{
			return m_active;

		}	// isActiveEdit

		/**
		* Close the edit object - it cannot be used after this.
		*/
		protected void closeEdit()
		{
			m_active = false;

		}	// closeEdit

		/*******************************************************************************
		* SessionStateBindingListener implementation
		*******************************************************************************/
	
		/**
		* Accept notification that this object has been bound as a SessionState attribute.
		* @param sessionStateKey The id of the session state which holds the attribute.
		* @param attributeName The id of the attribute to which this object is now the value.
		*/
		public void valueBound(String sessionStateKey, String attributeName) {}
	
		/**
		* Accept notification that this object has been removed from a SessionState attribute.
		* @param sessionStateKey The id of the session state which held the attribute.
		* @param attributeName The id of the attribute to which this object was the value.
		*/
		public void valueUnbound(String sessionStateKey, String attributeName)
		{
			if (m_logger.isDebugEnabled())
			{
				//m_logger.debug(this + ".valueUnbound()");
			}
	
			// catch the case where an edit was made but never resolved
			if (m_active)
			{
				cancelEdit(this);
			}
	
		}	// valueUnbound


	}//BaseCandidatePathEdit


	
	/*******************************************************************************
	* StepStatus implementation
	*******************************************************************************/
	
	public class BaseStepStatus
		implements StepStatus
	{
		/** The StepStatus id. */
		protected String m_id;
		
		/** The user id of the user who marked the step as completed. */
		protected String m_validator;
		
		/** The instructions for the step. */
		protected String m_instructions;
		
		/** The site id. */
		protected String m_site;
		
		/** The reference of the DissertationStep on which this StepStatus is created from. */
		protected String m_parentStepReference;
		
		/** The id for auto-validation from the Rackham database. */
		protected String m_autoValidationId;
		
		/** Time the step is marked as completed. */
		protected Time m_timeCompleted;
		
		/** Time of completion recommended by the creator of the step. (Not yet used.) */
		protected Time m_recommendedDeadline;
		
		/** Absolute deadline for completion set by the creator of the step. (Not yet used.) */
		protected Time m_hardDeadline;
		
		/** Type of permissions for marking the step as completed.
			See the DissertationStep interface for types. */
		protected String m_validationType;
		
		/** The references of the prerequisites for this step. */
		protected List m_prereqStatus;
		
		/** Signifies whether the step has been marked as completed. */
		protected boolean m_completed;
		
		/** Signifies whether this step is auto-validated from the Rackham database. */
		protected boolean m_oardValidated;
		
		/** The properties. */
		protected ResourcePropertiesEdit m_properties;
		

		/**
		* Constructor.
		* @param statusId The StepStatus id.
		* @param site The site id.
		*/
		public BaseStepStatus(String statusId, String site)
		{
			m_id = statusId;
			m_site = site;
			m_validator = "";
			m_parentStepReference = "-1";
			m_validationType = "0";
			m_instructions = "";
			m_autoValidationId = "";
			m_prereqStatus = new Vector();
			m_completed = false;
			m_oardValidated = false;
			m_properties = new BaseResourcePropertiesEdit();
			addLiveProperties(m_properties);
		}
		
		/**
		* Copy constructor.
		* @param status The StepStatus to copy.
		*/
		public BaseStepStatus(StepStatus status)
		{
			setAll(status);

		}	// BaseStepStatus
		
		/**
		* Construct from an existing definition, in xml.
		* @param el The StepStatus in XML in a DOM element.
		*/
		public BaseStepStatus(Element el)
		{
			m_properties = new BaseResourcePropertiesEdit();
			
			String valueString = null;
			m_prereqStatus = new Vector();

			m_id = el.getAttribute("id");
			m_site = el.getAttribute("site");
			m_validator = el.getAttribute("validator");
			m_parentStepReference = el.getAttribute("parentstepreference");
			m_autoValidationId = el.getAttribute("autovalid");
			m_timeCompleted = getTimeObject(el.getAttribute("timecompleted"));
			m_recommendedDeadline = getTimeObject(el.getAttribute("recommendeddeadline"));
			m_hardDeadline = getTimeObject(el.getAttribute("harddeadline"));
			m_completed = getBool(el.getAttribute("completed"));
			m_oardValidated = getBool(el.getAttribute("oardvalidated"));
			m_validationType = el.getAttribute("validtype");

			m_instructions = Xml.decodeAttribute(el, "instructions");

				// READ THE CHILD ELEMENTS
			NodeList children = el.getChildNodes();
			final int length = children.getLength();
			for(int i = 0; i < length; i++)
			{
				Node child = children.item(i);
				if (child.getNodeType() != Node.ELEMENT_NODE) continue;
				Element element = (Element)child;

				// look for properties
				if (element.getTagName().equals("properties"))
				{
					// re-create properties
					m_properties = new BaseResourcePropertiesEdit(element);
				}
				else if(element.getTagName().equals("prereqstatus"))
				{
					NodeList ssChildren = element.getChildNodes();
					int ssLength = ssChildren.getLength();
					Node ssChild = null;
					Element ssElement = null;
					for(int x = 0; x < ssLength; x++)
					{
						ssChild = ssChildren.item(x);
						if (ssChild.getNodeType() != Node.ELEMENT_NODE) continue;
						ssElement = (Element)ssChild;
						if(ssElement.getTagName().equals("prereq"))
						{
							valueString = ssElement.getAttribute("prereqstatusreference");
							m_prereqStatus.add(valueString);
						}
					}
				}
				// old way of encoding
				else if(element.getTagName().equals("instructions"))
				{
					if ((element.getChildNodes() != null) && (element.getChildNodes().item(0) != null))
					{
						m_instructions = element.getChildNodes().item(0).getNodeValue();
					}
					if (m_instructions == null)
					{
						m_instructions = "";
					}
				}
					
			}
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE STATUS : LEAVING STORAGE CONSTRUCTOR");
		}
		
		/**
		* Deep copy of this object.
		* @param status - The StepStatus object to be copied.
		*/
		protected void setAll(StepStatus status)
		{
			m_id = status.getId();
			m_validator = status.getValidator();
			m_instructions = status.getInstructions();
			m_site = status.getSite();
			m_parentStepReference = status.getParentStepReference();
			m_autoValidationId = status.getAutoValidationId();
			m_validationType = status.getValidationType();
			m_prereqStatus = status.getPrereqs();
			m_completed = status.getCompleted();
			m_timeCompleted = status.getTimeCompleted();
			m_recommendedDeadline = status.getRecommendedDeadline();
			m_hardDeadline = status.getHardDeadline();
			m_properties = new BaseResourcePropertiesEdit();
			m_properties.addAll(status.getProperties());

		}   // setAll

		
		/**
		* Serialize the resource into XML, adding an element to the doc under the top of the stack element.
		* @param doc The DOM doc to contain the XML (or null for a string return).
		* @param stack The DOM elements, the top of which is the containing element of the new "resource" element.
		* @return The newly added element.
		*/
		public Element toXml(Document doc, Stack stack)
		{
			Element status = doc.createElement("status");

			if (stack.isEmpty())
			{
				doc.appendChild(status);
			}
			else
			{
				((Element)stack.peek()).appendChild(status);
			}

			stack.push(status);

			status.setAttribute("id", m_id);
			status.setAttribute("site", m_site);
			status.setAttribute("validator", m_validator);
			status.setAttribute("parentstepreference", m_parentStepReference);
			status.setAttribute("validtype", m_validationType);
			status.setAttribute("autovalid", m_autoValidationId);
			status.setAttribute("timecompleted", getTimeString(m_timeCompleted));
			status.setAttribute("recommendeddeadline", getTimeString(m_recommendedDeadline));
			status.setAttribute("harddeadline", getTimeString(m_hardDeadline));
			status.setAttribute("completed", getBoolString(m_completed));
			status.setAttribute("oardvalidated", getBoolString(m_oardValidated));
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE STATUS : TOXML : BASIC ATTRIBUTES WRITTEN");

				// SAVE THE PREREQUISISTE STEPS
			Element prereq = null;
			Element prereqStatus = doc.createElement("prereqstatus");
			for(int x = 0; x < m_prereqStatus.size(); x++)
			{
				prereq = doc.createElement("prereq");
				prereq.setAttribute("prereqstatusreference", (String) m_prereqStatus.get(x));
				prereqStatus.appendChild(prereq);
			}
			status.appendChild(prereqStatus);
			//m_logger.debug("DISSERTATION : BASE SERVICE : STEP STATUS : TOXML : PREREQUISITES WRITTEN");
			
				// SAVE THE INSTRUCTIONS
			Xml.encodeAttribute(status, "instructions", m_instructions);
/*
			Element instructions = doc.createElement("instructions");
			status.appendChild(instructions);
			instructions.appendChild(doc.createCDATASection(m_instructions));
*/
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE STATUS : TOXML : instructions element set");
			
				// SAVE THE PROPERTIES
			m_properties.toXml(doc, stack);
			
			stack.pop();

			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE STATUS : TOXML : LEAVING");
			return status;

		}//toXml


		/*******************************************************************************
		* Reference Implementation
		*******************************************************************************/
		
		/**
		* Access the StepStatus id.
		* @return The StepStatus id string.
		*/
		public String getId()
		{
			return m_id;
		}

		/**
		* Access the URL which can be used to access the resource.
		* @return The URL which can be used to access the resource.
		*/
		public String getUrl()
		{
			return getAccessPoint(false) + m_id;
		}

		/**
		* Access the internal reference which can be used to access the resource from within the system.
		* @return The the internal reference which can be used to access the resource from within the system.
		*/
		public String getReference()
		{
			return statusReference(m_site, m_id);
		}

		/**
		* Access the resource's properties.
		* @return The resource's properties.
		*/
		public ResourceProperties getProperties()
		{
			return m_properties;
		}
		
		/**
		* Access the reference of the parent DissertationStep.
		* @return The reference of the parent DissertationStep, if it exists, -1 otherwise.
		*/
		public String getParentStepReference()
		{
			return m_parentStepReference;
		}

		/**
		* Access the creator of this StepStatus.
		* @return The user id of the creator.
		*/
		public String getCreator()
		{
			return m_properties.getProperty(ResourceProperties.PROP_CREATOR);
		}
		
		/**
		* Access the recommended deadline for completion of this step.
		* @return The Time recommended by the creator for completion.
		*/
		public Time getRecommendedDeadline()
		{
			return m_recommendedDeadline;
		}

		/**
		* Access the absolute deadline for completion of the step.
		* @return The absolute deadline set by the creator for completion.
		*/
		public Time getHardDeadline()
		{
			return m_hardDeadline;
		}

		/**
		* Access the time the step was marked as completed.
		* @return The time the step was marked as completed, or null if it is not completed.
		*/
		public Time getTimeCompleted()
		{
			return m_timeCompleted;
		}
		
		/**
		* Access the user who marked the step as completed.
		* @return The user id of the user who marked the step as completed.
		*/
		public String getValidator()
		{
			return m_validator;
		}

		/**
		* Access the id for auto-validation by the Rackham database.
		* @return the id used for auto-validation by the Rackham database,
		* or -1 if the step is not auto-validated.
		*/
		public String getAutoValidationId()
		{
			return m_autoValidationId;
		}
		
		/**
		* Has this step been marked completed by the candidate yet ?
		* @return True if the step has been marked as completed, false otherwise.
		*/
		public boolean getCompleted()
		{
			return m_completed;
		}

		/**
		* Access whether this step is auto-validated by the Rackham database.
		* @return True if the step is auto-validated by the Rackham database, false otherwise.
		*/
		public boolean getOardValidated()
		{
			return m_oardValidated;
		}
		
		/**
		* Access the prerequisites for this step.
		* @return A List containing the references of the prerequisites for this step.
		*/
		public List getPrereqs()
		{
			return m_prereqStatus;
		}
		
		/**
		* See whether this step contains this prerequisite.
		* @param statusRef The reference of the prerequisite.
		* @return true if this step has this prerequisite, false otherwise.
		*/
		public boolean hasPrerequisite(String statusRef)
		{
			boolean retVal = false;
			if(statusRef != null)
			{
				for(int x = 0; x < m_prereqStatus.size(); x++)
				{
					if(statusRef.equals(m_prereqStatus.get(x)))
						retVal = true;
				}
			}

			return retVal;
		}

		/**
		* Access the validation type of this step.
		* @return the type of permissions required to mark a step as completed.  See the DissertationStep interface.
		*/
		public String getValidationType()
		{
			return m_validationType;
		}

		/**
		* Access the instructions for the StepStatus with html links for user-entered links.
		* @return the step's instructions with html links for user-entered links.
		*/
		public String getInstructions()
		{
			StringBuffer retVal = new StringBuffer();
			boolean goodSyntax = true;
			String tempString = null;
			String linkString = null;
			String linkTextString = null;
			String fullLinkString = null;
			int midIndex = -1;
			int endIndex = -1;
			int beginIndex = m_instructions.indexOf("http://");
		
			if(beginIndex == -1)
			{
				//m_logger.debug("NO LINKS - RETURNING ORIGINAL TEXT");
				retVal.append(m_instructions);
			}
			else
			{
				do
				{
					// FIRST ADD THE TEXT UP TO THE LINK TO THE BUFFER
					//m_logger.debug("LINK FOUND : INDEX : " + beginIndex);
					retVal.append(m_instructions.substring((endIndex+1), beginIndex));
					midIndex = m_instructions.indexOf("{", beginIndex);
					//m_logger.debug("MID INDEX : " + midIndex);
					if(midIndex == -1)
					{
						// LINK IS NOT IN PROPER FORMAT - RETURN ORIGINAL STRING
						goodSyntax = false;
						beginIndex = -1;
						//m_logger.debug("MISSING {");
					}
					else
					{
						// FIND THE END TAG
						endIndex = m_instructions.indexOf("}", midIndex);
						//m_logger.debug("END INDEX : " + endIndex);
						if(endIndex == -1)
						{
							goodSyntax = false;
							//m_logger.debug("MISSING }");
							beginIndex = -1;
						}
						else
						{
							//m_logger.debug("SYNTAX IS GOOD THIS PASS - CREATING LINK\n");
							linkString = m_instructions.substring(beginIndex, midIndex);
							//m_logger.debug("LINK STRING : " + linkString);
							linkTextString = m_instructions.substring((midIndex+1), endIndex);
							//m_logger.debug("LINK TEXT STRING : " + linkTextString);
							fullLinkString = "<a href='" + linkString + "' target='_blank' " + ">" + linkTextString + "</a>";
							//m_logger.debug("FULL LINK STRING : " + fullLinkString);
							retVal.append(fullLinkString);
							beginIndex = m_instructions.indexOf("http://", endIndex);
							//m_logger.debug("\n\nLINK CREATED : NEW BEGIN INDEX : " + beginIndex);
						}
					}
					
					if(beginIndex == -1)
					{
						//m_logger.debug("NO MORE LINKS - ADDING REMAINING TEXT");
						tempString = m_instructions.substring((endIndex+1), m_instructions.length());
						//m_logger.debug("ADDING REMAINDER OF TEXT : " + tempString);
						retVal.append(tempString);
					}

				}while(beginIndex != -1);
				
			}//else
		
			if(!goodSyntax)
			{
				//m_logger.debug("BAD SYNTAX : RETURNING ORIGINAL STRING\n");
				retVal = new StringBuffer();
				retVal.append(m_instructions);
			}

			//m_logger.debug("RESULT STRING : " + retVal.toString());
			return retVal.toString();
		}

		/**
		* Access the instructions as entered by the user.
		* @return the instructions as entered by the user.
		*/
		public String getInstructionsText()
		{
			return m_instructions;
		}
		
		/**
		* Access the instructions as entered by the user, with a maximum length of 80 characters for display in selects.
		* Remove links to save space in the select drop down list and prevent truncation from breaking an active link
		* @return the instructions as entered by the user, with a maximum length of 80 characters for display in selects.
		*/
		public String getShortInstructionsText()
		{
			StringBuffer buffer = new StringBuffer();
			String retVal = null;
			String tempString = null;
			String linkString = null;
			String linkTextString = null;
			String fullLinkString = null;
			boolean goodSyntax = true;
			int midIndex = -1;
			int endIndex = -1;
			int tagIndex = -1;
			
			int beginIndex = m_instructions.indexOf("http://");
			if(beginIndex == -1)
			{
				//No links, so return the original text
				buffer.append(m_instructions);
			}
			else
			{
				try
				{
					do
					{
						//First, add the text up to the link to the buffer
						tagIndex = m_instructions.lastIndexOf("<", beginIndex);
						buffer.append(m_instructions.substring((endIndex+1), tagIndex));
						
						//Find the begin tag
						midIndex = m_instructions.indexOf(">", beginIndex);
						if(midIndex == -1)
						{
							//Link is not in the proper format, so return the original string
							goodSyntax = false;
							beginIndex = -1;
							//Missing >
						}
						else
						{
							//Find the end tag
							endIndex = m_instructions.indexOf("<", midIndex);
							if(endIndex == -1)
							{
								goodSyntax = false;
								//Missing <
								beginIndex = -1;
							}
							else
							{
								//Syntax is good this pass, so keep the link text string
								linkTextString = m_instructions.substring((midIndex+1), endIndex);
								buffer.append(linkTextString);
								endIndex = m_instructions.indexOf(">", endIndex);
								beginIndex = m_instructions.indexOf("http://", endIndex);
							}
						}
						if(beginIndex == -1)
						{
							//No more links, so add remaining text
							tempString = m_instructions.substring((endIndex+1), m_instructions.length());
						
							//Add remainder of text
							buffer.append(tempString);
						}

					}while(beginIndex != -1);
				}
				catch(Exception e)
				{
					m_logger.warn(this + ".getShortInstructionsText Exception " + e);
				}
				
			}//else
		
			if(!goodSyntax)
			{
				//Bad syntax, so return original string
				buffer = new StringBuffer();
				buffer.append(m_instructions);
			}

			//Result string
			if(buffer.length() > 80)
			{
				retVal = ((String)buffer.toString()).substring(0, 79) + " . . . ";
			}
			else
			{
				retVal = buffer.toString();
			}
			return retVal;
			
		}//getShortInstructionsText


		/**
		* Access the site id for the StepStatus's site.
		* @return the site id.
		*/
		public String getSite()
		{
			return m_site;
		}

		/**
		* Are these objects equal?  If they are both StepStatus objects, and they have
		* matching id's, they are.
		* @return true if they are equal, false if not.
		*/
		public boolean equals(Object obj)
		{
			if (!(obj instanceof StepStatus)) return false;
			return ((StepStatus)obj).getId().equals(getId());

		}   // equals

		/**
		* Make a hash code that reflects the equals() logic as well.
		* We want two objects, even if different instances, if they have the same id to hash the same.
		*/
		public int hashCode()
		{
			return getId().hashCode();

		}	// hashCode

		/**
		* Compare this object with the specified object for order.
		* @return A negative integer, zero, or a positive integer as this object is
		* less than, equal to, or greater than the specified object.
		*/
		public int compareTo(Object obj)
		{
			if (!(obj instanceof StepStatus)) throw new ClassCastException();

			// if the object are the same, say so
			if (obj == this) return 0;

			// start the compare by comparing their sort names
			int compare = getInstructions().compareTo(((StepStatus)obj).getInstructions());

			// if these are the same
			if (compare == 0)
			{
				// sort based on (unique) id
				compare = getId().compareTo(((StepStatus)obj).getId());
			}

			return compare;

		}	// compareTo

	}


	
	/*******************************************************************************
	* StepStatusEdit implementation
	*******************************************************************************/
	
	public class BaseStepStatusEdit
		extends BaseStepStatus
		implements StepStatusEdit, SessionStateBindingListener
	{
		/** The event code for the edit. */
		protected String m_event = null;
		
		/** Active flag. */
		protected boolean m_active = false;
		
		
		/**
		* Constructor.
		* @param statusId The StepStatusEdit id.
		* @param site The site id.
		*/
		public BaseStepStatusEdit(String statusId, String site)
		{
			super(statusId, site);

		}   // BaseStepStatusEdit

		/**
		* Copy constructor.
		* @param status The StepStatusEdit to be copied.
		*/
		public BaseStepStatusEdit(StepStatus status)
		{
			super(status);

		}	// BaseStepStatusEdit
		
		/**
		* Construct from an existing definition, in xml.
		* @param el The StepStatusEdit in XML in a DOM element.
		*/
		public BaseStepStatusEdit(Element el)
		{
			super(el);

		}	// BaseStepStatusEdit


		/**
		* Fill the properties of the StepStatus based on the parent DissertationStep.
		* @param site The site id of the current site at the time of creation.
		* @param step The parent DissertationStep from which this StepStatus is spawned.
		* @param oardstep Signifies whether this step is auto-validated by the Rackham database.
		*/
		public void initialize(String site, DissertationStep step, boolean oardstep)
		{
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE STEP STATUS EDIT : INITIALIZE : STEP OBJECT : " + step);
			m_validator = "";
			m_parentStepReference = step.getReference();
			m_validationType = step.getValidationType();
			m_instructions = step.getInstructions();
			m_site = site;
			m_autoValidationId = step.getAutoValidationId();
			//m_logger.debug("****************************************************************************************************");
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE STEP STATUS : INITIALIZE : AUTO VALIDATION ID : " + m_autoValidationId);
			//m_logger.debug("****************************************************************************************************");
			m_prereqStatus = new Vector();
			m_properties = new BaseResourcePropertiesEdit();
			m_completed = false;
			m_oardValidated = oardstep;
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE STEP STATUS EDIT : INITIALIZE : STATUS OBJECT INITIALIZED");
		}
		
		/**
		* Clean up.
		*/
		protected void finalize()
		{
			// catch the case where an edit was made but never resolved
			if (m_active)
			{
				cancelEdit(this);
			}

		}	// finalize

		/**
		* Set the recommended deadline for completion of this step.
		* @param deadline The Time recommended by the creator for completion.
		*/
		public void setRecommendedDeadline(Time deadline)
		{
			m_recommendedDeadline = deadline;
		}

		/**
		* Set the absolute deadline for completion of the step.
		* @param hardDeadline The absolute deadline set by the creator for completion.
		*/
		public void setHardDeadline(Time hardDeadline)
		{
			m_hardDeadline = hardDeadline;
		}

		/**
		* Set the time the step was marked as completed.
		* @param time The time the step was marked as completed, or null if it is not completed.
		*/
		public void setTimeCompleted(Time time)
		{
			m_timeCompleted = time;
		}

		/**
		* Set the user who marked the step as completed.
		* @param validator The user id of the user who marked the step as completed.
		*/
		public void setValidator(String validator)
		{
			m_validator = validator;
		}

		/**
		* Set the id for auto-validation by the Rackham database.
		* @param validId the id used for auto-validation by the Rackham database,
		* or -1 if the step is not auto-validated.
		*/
		public void setAutoValidationId(String validId)
		{
			if(validId != null)
				m_autoValidationId = validId;
		}

		/**
		* Has this step been marked completed by the candidate yet ?
		* @param completed True if the step has been marked as completed, false otherwise.
		*/
		public void setCompleted(boolean completed)
		{
			m_timeCompleted = TimeService.newTime();
			m_completed = completed;
		}

		/**
		* Add a prerequisite to this step.
		* @param statusReference The reference for the prerequisite StepStatus to be added.
		*/
		public void addPrerequisiteStatus(String statusReference)
		{
			if(statusReference != null)
				m_prereqStatus.add(statusReference);
		}

		/**
		* Remove a prerequisite to this step.
		* @param statusReference The reference for the prerequisite StepStatus to be removed.
		*/
		public void removePrerequisiteStatus(String statusReference)
		{
			if(statusReference != null)
				m_prereqStatus.remove(statusReference);
		}
		
		/**
		* Set the collection of prerequisites to this step.
		* @param prereqs List containing the references to the prerequisite StepStatus objects.
		*/
		public void setPrereqs(List prereqs)
		{
			if(prereqs != null)
				m_prereqStatus = prereqs;
		}

		/**
		* Set the validation type for this step.
		* @param type The type of permissions required to mark this step as completed.  See the DissertationStep interface for values.
		*/
		public void setValidationType(String type)
		{
			m_validationType = type;
		}

		/**
		* Set the instructions for this StepStatusEdit.
		* @param instructions The instructions.
		*/
		public void setInstructions(String instructions)
		{
			if(instructions != null)
				m_instructions = Validator.escapeHtml(instructions);
		}
		
		/**
		* Set the instructions for this StepStatusEdit.
		* @param instructions The instructions.
		*/
		public void setInstructionsHtml(String instructions)
		{
			if(instructions != null)
				m_instructions = instructions;
		}

		/**
		* Set the site for this StepStatusEdit.
		* @param site The site id.
		*/
		public void setSite(String site)
		{
			m_site = site;
		}
		
		/**
		* Take all values from this object.
		* @param status The StepStatus object to take values from.
		*/
		protected void set(StepStatus status)
		{
			setAll(status);

		}   // set

		/**
		* Access the event code for this edit.
		* @return The event code for this edit.
		*/
		protected String getEvent() { return m_event; }
	
		/**
		* Set the event code for this edit.
		* @param event The event code for this edit.
		*/
		protected void setEvent(String event) { m_event = event; }

		/**
		* Access the resource's properties for modification
		* @return The resource's properties.
		*/
		public ResourcePropertiesEdit getPropertiesEdit()
		{
			return m_properties;

		}	// getPropertiesEdit

		/**
		* Enable editing.
		*/
		protected void activate()
		{
			m_active = true;

		}	// activate

		/**
		* Check to see if the edit is still active, or has already been closed.
		* @return true if the edit is active, false if it's been closed.
		*/
		public boolean isActiveEdit()
		{
			return m_active;

		}	// isActiveEdit

		/**
		* Close the edit object - it cannot be used after this.
		*/
		protected void closeEdit()
		{
			m_active = false;

		}	// closeEdit

		/*******************************************************************************
		* SessionStateBindingListener implementation
		*******************************************************************************/
	
		/**
		* Accept notification that this object has been bound as a SessionState attribute.
		* @param sessionStateKey The id of the session state which holds the attribute.
		* @param attributeName The id of the attribute to which this object is now the value.
		*/
		public void valueBound(String sessionStateKey, String attributeName) {}
	
		/**
		* Accept notification that this object has been removed from a SessionState attribute.
		* @param sessionStateKey The id of the session state which held the attribute.
		* @param attributeName The id of the attribute to which this object was the value.
		*/
		public void valueUnbound(String sessionStateKey, String attributeName)
		{
			if (m_logger.isDebugEnabled())
			{
				//m_logger.debug(this + ".valueUnbound()");
			}
	
			// catch the case where an edit was made but never resolved
			if (m_active)
			{
				cancelEdit(this);
			}
	
		}	// valueUnbound

		
	}//BaseStepStatusEdit

	
	/*******************************************************************************
	* MPRecord implementation
	*******************************************************************************/	
	/**
	*
	* Contains a Rackham MPathways data record
	* 
	*/
	private class MPRecord
	{
		/**
		* All dates are formatted as mm/dd/ccyy.  
		* MPEXT.txt Data Structure:
		* 1   Emplid					  A9	| Student's emplid
		* 2   Acad_prog				  A9	| Academic Program Code
		* 3   Anticipate_Anticipate_1	  A15	| Adv to cand term code
		* 4	  Date_compl				  D	| Date milestone was completed 
		* 5   Milestone				  A10	| name of milestone PRELIM or ADVCAND
		* 6 | Academic plan				|  A4	| Field of study and degree (e.g. 1220PHD1)
		* 7 | Campus id					|  A1-A8| Student's uniqname (Chef id)
		*/
		public String m_umid = null;
		public String m_acad_prog = null;
		public String m_anticipate = null;
		public String m_date_compl = null;
		public String m_milestone = null;
		public String m_academic_plan = null;
		public String m_campus_id = null;
		
		//methods
		public String getUmid(){ return m_umid; }
		public String getAcad_prog(){ return m_acad_prog; }
		public String getAnticipate(){ return m_anticipate; }
		public String getDate_compl(){ return m_date_compl; }
		public String getMilestone(){ return m_milestone; }
		public String getAcademicPlan() { return m_academic_plan; }
		public String getCampusId() { return m_campus_id; }
		
	}//MPRecord

	/*******************************************************************************
	* OARDRecord implementation
	*******************************************************************************/	
	
	/**
	*
	* Contains a Rackham OARD data record
	* 
	*/
	private class OARDRecord
	{
		/**
		* corresponding to OARD db output fields
		* All dates are formatted as mm/dd/ccyy.  
		* 
		* OARDEXT.txt Data Structure:
		* STRUCT  Field Name            Field Type
		* 1   Emplid                      A8 - Student's emplid
		* 2   Fos                         A4    - Students field of study code
		* 3   Lname                       A25 - Students last name
		* 4   Fname                       A30 - Students first name
		* 5   Degterm trans               A7 - Students degree term as TT-CCYY (e.g. FA-2003)
		* 6   Oral exam date              D - Date of oral defense
		* 7   Oral exam time              A7 - Time of oral defense
		* 8   Oral exam place             A25 - Place of oral defense
		* 9   Committee approved date     D - date committee was approved
		*10   First format date           D - date of pre defense meeting in Rackham
		*11   Binder receipt date         D 
		*12   Fee requirement met         A1 - Y or N 
		*13   Fee date receipt seen       D 
		*14   Pub fee date received       D
		*15   Oral report return date     D
		*16   Unbound date                D
		*17   Abstract date               D
		*18   Bound copy received date    D
		*19   Diploma application date    D
		*20   Contract received date      D
		*21   NSF Survey date             D
		*22   Degree conferred date       D - date the degree was conferred in OARD system
		*23   Final format recorder       A3 - initials
		*24   Update date                 D - date record was last modified
		*25   Comm cert date              D -
		*26   Role                        A2 - role code
		*27   Member                      A40 - faculty member name
		*28   Eval date                   D - evaluation received date
		*29 |  Campus id                 |  A1-A8 - student's uniqname (Chef id)
		*
		*/
		public String m_umid = null;
		public String m_fos = null;
		public String m_lname = null;
		public String m_fname = null;
		public String m_degreeterm_trans = null;
		public String m_oral_exam_date = null;
		public String m_oral_exam_time = null;
		public String m_oral_exam_place = null;
		public String m_committee_approved_date = null;
		public String m_first_format_date = null;
		public String m_binder_receipt_date = null;
		public String m_fee_requirement_met = null;
		public String m_fee_date_receipt_seen = null;
		public String m_pub_fee_date_received = null;
		public String m_oral_report_return_date = null;
		public String m_unbound_date = null;
		public String m_abstract_date = null;
		public String m_bound_copy_received_date = null;
		public String m_diploma_application_date = null;
		public String m_contract_received_date = null;
		public String m_nsf_survey_date = null;
		public String m_degree_conferred_date = null;
		public String m_final_format_recorder = null;
		public String m_update_date = null;
		public String m_comm_cert_date = null;
		public String m_role = null;
		public String m_member = null;
		public String m_eval_date = null;
		public String m_campus_id = null;
		
		//methods
		public String getUmid(){ return m_umid; }
		public String getFos(){ return m_fos; }
		public String getLname(){ return m_lname; }
		public String getFname(){ return m_fname; }
		public String getDegreeterm_trans(){ return m_degreeterm_trans; }
		public String getOral_exam_date(){ return m_oral_exam_date; }
		public String getOral_exam_time(){ return m_oral_exam_time; }
		public String getOral_exam_place(){ return m_oral_exam_place; }
		public String getCommittee_approved_date(){ return m_committee_approved_date; }
		public String getFirst_format_date(){ return m_first_format_date; }
		public String getBinder_receipt_date(){ return m_binder_receipt_date; }
		public String getFee_requirement_met(){ return m_fee_requirement_met; }
		public String getFee_date_receipt_seen(){ return m_fee_date_receipt_seen; }
		public String getPub_fee_date_received(){ return m_pub_fee_date_received; }
		public String getOral_report_return_date(){ return m_oral_report_return_date; }
		public String getUnbound_date(){ return m_unbound_date; }
		public String getAbstract_date(){ return m_abstract_date; }
		public String getBound_copy_received_date(){ return m_bound_copy_received_date; }
		public String getDiploma_application_date(){ return m_diploma_application_date; }
		public String getContract_received_date(){ return m_contract_received_date; }
		public String getNsf_survey_date(){ return m_nsf_survey_date; }
		public String getDegree_conferred_date(){ return m_degree_conferred_date; }
		public String getFinal_format_recorder(){ return m_final_format_recorder; }
		public String getUpdate_date(){ return m_update_date; }
		public String getComm_cert_date(){ return m_comm_cert_date; }
		public String getRole(){ return m_role; }
		public String getMember(){ return m_member; }
		public String getEval_date() { return m_eval_date; }
		public String getCampusId() { return m_campus_id; }
		
	} // OARDRecord

	/*******************************************************************************
	* CandidateInfo implementation
	*******************************************************************************/	
	protected class BaseCandidateInfo
		implements CandidateInfo
	{
		/** The CandidateInfo id. */
		public String m_id;
		
		/** The user's University id - from extracts. */
		public String m_emplid;
		
		/** The users' CHEF user id. */
		public String m_chefId;
		
		/** The id for the user's Rackham program - from the Rackham database. */
		public String m_program;
		
		/** The id for the user's parent(department) site. */
		public String m_parentSiteId;
		
		/** The id of the user's site. */
		public String m_site;
		
		/** The properties. */
		public ResourcePropertiesEdit m_properties;

		
		/** RACKHAM DATA */
		public String m_advCandDesc;
		public String m_oralExamPlace;
		public String m_finalFormatRecorder;
		public String m_degreeTermTrans;
		public String m_milestone;
		public Time m_milestoneCompleted;
		public String m_prelimMilestone;
		public Time m_prelimTimeMilestoneCompleted;
		public String m_advcandMilestone;
		public Time m_advcandTimeMilestoneCompleted;
		public Time m_committeeApproval;
		public Time m_oralExam;
		public Time m_firstFormat;
		public Time m_oralReportReturned;
		public Time m_committeeCert;
		public Time m_binderReceipt;
		public Time m_receiptSeen;
		public Time m_pubFee;
		public Time m_unbound;
		public Time m_abstract;
		public Time m_bound;
		public Time m_diplomaApp;
		public Time m_contract;
		public Time m_survey;
		public Time m_degreeConferred;
		public boolean m_feeRequirementMet;
		protected Vector m_timeCommitteeEval;
		
		/**
		* Constructor.
		* @param infoId The CandidateInfo id.
		* @param site The site id.
		*/
		public BaseCandidateInfo(String infoId, String site)
		{
			m_id = infoId;
			m_site = site;
			m_chefId = "";
			m_program = "";
			m_parentSiteId = "";
			m_properties = new BaseResourcePropertiesEdit();
			addLiveProperties(m_properties);
			m_feeRequirementMet = false;
			m_timeCommitteeEval = new Vector();
		}

		/**
		* Copy constructor.
		* @param info The CandidateInfo to copy.
		*/
		public BaseCandidateInfo(CandidateInfo info)
		{
			setAll(info);

		}	// BaseCandidateInfo
		
		/**
		* Construct from an existing definition, in xml.
		* @param el The CandidateInfo in XML in a DOM element.
		*/
		public BaseCandidateInfo(Element el)
		{
			m_properties = new BaseResourcePropertiesEdit();

			m_id = el.getAttribute("id");
			m_emplid = el.getAttribute("emplid");
			m_chefId = el.getAttribute("chefid");
			m_program = el.getAttribute("program");
			m_parentSiteId = el.getAttribute("siteid");
			m_site = el.getAttribute("site");
			m_timeCommitteeEval = new Vector();
			// the children (roles, properties)
			NodeList children = el.getChildNodes();
			final int length = children.getLength();
			for(int i = 0; i < length; i++)
			{
				Node child = children.item(i);
				if (child.getNodeType() != Node.ELEMENT_NODE) continue;
				Element element = (Element)child;

				// look for properties
				if (element.getTagName().equals("properties"))
				{
					// re-create properties
					m_properties = new BaseResourcePropertiesEdit(element);
				}
			}
			//m_logger.debug("DISSERTATION : BASE SERVICE : BASE CANDIDATE INFO : LEAVING STORAGE CONSTRUCTOR");
		}

		/**
		* Deep copy of this object.
		* @param info - The CandidateInfo object to be copied.
		*/
		protected void setAll(CandidateInfo info)
		{
			m_id = info.getId();
			m_emplid = info.getEmplid();
			m_chefId = info.getChefId();
			m_program = info.getProgram();
			m_parentSiteId = info.getParentSiteId();
			m_site = info.getSite();
			m_timeCommitteeEval = new Vector();
			m_properties = new BaseResourcePropertiesEdit();
			m_properties.addAll(info.getProperties());

		}   // setAll
		
		/**
		* Serialize the resource into XML, adding an element to the doc under the top of the stack element.
		* @param doc The DOM doc to contain the XML (or null for a string return).
		* @param stack The DOM elements, the top of which is the containing element of the new "resource" element.
		* @return The newly added element.
		*/
		public Element toXml(Document doc, Stack stack)
		{
			Element info = doc.createElement("info");
			
			if (stack.isEmpty())
			{
				doc.appendChild(info);
			}
			else
			{
				((Element)stack.peek()).appendChild(info);
			}

			stack.push(info);

			info.setAttribute("id", m_id);
			info.setAttribute("chefid", m_chefId);
			info.setAttribute("emplid", m_emplid);
			info.setAttribute("siteid", m_parentSiteId);
			info.setAttribute("program", m_program);
			info.setAttribute("site", m_site);
			
			m_properties.toXml(doc, stack);
			
			stack.pop();
			
			return info;
		}
	
	
		public String getId()
		{
			return m_id;
		}

		/**
		* Access the URL which can be used to access the resource.
		* @return The URL which can be used to access the resource.
		*/
		public String getUrl()
		{
			return getAccessPoint(false) + m_id;

		}   // getUrl

		/**
		* Access the internal reference which can be used to access the resource from within the system.
		* @return The the internal reference which can be used to access the resource from within the system.
		*/
		public String getReference()
		{
			return infoReference(m_site, m_id);

		}   // getReference

		/**
		* Access the resources's properties.
		* @return The resources's properties.
		*/
		public ResourceProperties getProperties()
		{
			return m_properties;

		}   // getProperties

		/**
		* Access the user's CHEF user id.
		* @return The user id string.
		*/
		public String getChefId()
		{
			return m_chefId;
		}

		/**
		* Access the user's university id.
		* @return The user's university id string.
		*/
		public String getEmplid()
		{
			return m_emplid;
		}
		
		/**
		* Access the user's Rackham program id.
		* @return The user's program id string.
		*/
		public String getProgram()
		{
			return m_program;
		}

		/**
		* Access the user's CHEF user id.
		* @return The user id string.
		*/
		public String getParentSiteId()
		{
			return m_parentSiteId;
		}

		/**
		* Access the site id for the CandidateInfo.
		* @return the site id.
		*/
		public String getSite()
		{
			return m_site;
		}
		
		/**
		* Access the time the committee evaluations were completed.
		* @return The latest time held in the m_timeCommitteeEval vector.
		*/
		public Time getTimeCommitteeEvalCompleted()
		{
			//m_logger.debug("DISSERTATION : BASE SERVICE : CANDIDATEINFO : ENTERING GET TIME COMM EVAL COMPLETED");
			boolean completed = true;
			Time latestTime = null;
			Time aTime = null;
			//m_logger.debug("DISSERTATION : BASE SERVICE : CANDIDATEINFO : GET TIME COMM EVAL COMPLETED : TIMES IN VECTOR : " + m_timeCommitteeEval.size());
			for(int x = 0; x < m_timeCommitteeEval.size(); x++)
			{
				aTime = (Time)m_timeCommitteeEval.get(x);
				if(aTime == null)
				{
					//m_logger.debug("DISSERTATION : BASE SERVICE : CANDIDATEINFO : GET TIME COMM EVAL COMPLETED : A TIME IS NULL - WILL RETURN NULL");
					completed = false;
				}
				else
				{
					if(latestTime == null)
					{
						latestTime = aTime;
					}

					if(latestTime.before(aTime))
					{
						latestTime = aTime;
					}
				}
			}

			if(!completed)
			{
				latestTime = null;
			}

			return latestTime;
		}

		/**
		* Access whether the committee evaluations are completed.
		* @return False if the m_timeCommitteeEval vector is empty, true otherwise.
		*/
		public boolean getCommitteeEvalCompleted()
		{
			boolean completed = true;
			Time aTime = null;
			for(int x = 0; x < m_timeCommitteeEval.size(); x++)
			{
				aTime = (Time)m_timeCommitteeEval.get(x);
				if(aTime == null)
					completed = false;
			}
			return completed;
		}
	
		/**
		* Access the time a Rackham auto-validated step was completed.
		* @param The validation id for the Rackham auto-validated step.
		* @return Time - the Time of completion, or null if the step is not completed.
		*/
		public Time getExternalValidation(int stepNum)
		{
			Time retVal = null;
			
			switch(stepNum)
			{
				case 1:
					if((("prelim".equalsIgnoreCase(m_prelimMilestone)) && (m_prelimTimeMilestoneCompleted != null)) || ("advcand".equalsIgnoreCase(m_advcandMilestone)))
					{
						retVal = m_prelimTimeMilestoneCompleted;
					}
					break;
				
				case 2:
					if(("advcand".equalsIgnoreCase(m_advcandMilestone)) && (m_advcandTimeMilestoneCompleted != null) && (m_advCandDesc != null))
					{
						retVal = m_advcandTimeMilestoneCompleted;
					}
					break;
				
				case 3:
					retVal = m_committeeApproval;
					break;
				
				case 4:
					if((m_oralExam != null) && (m_oralExamPlace != null))
					{
						retVal = m_oralExam;
					}
					break;
				
				case 5:
					retVal = m_firstFormat;
					break;
				
				case 6:
					retVal = getTimeCommitteeEvalCompleted();
					break;


				case 7:
					retVal = m_oralReportReturned;
					break;


				case 8:
					retVal = m_committeeCert;
					break;


				case 9:
					if(m_finalFormatRecorder != null && !m_finalFormatRecorder.equals(""))
					{
						retVal = TimeService.newTime();
					}
					break;


				case 10:
					if((m_committeeApproval != null) && 
					   (m_binderReceipt != null) &&
					   ((m_feeRequirementMet && m_receiptSeen==null) || 
					   (!m_feeRequirementMet && m_receiptSeen!=null)) &&
					   (m_pubFee != null) && 
					   (m_oralReportReturned != null) &&
					   (m_abstract != null) &&
					   (m_diplomaApp != null) && 
					   (m_contract != null) && 
					   (m_survey != null) && 
					   (m_committeeCert != null))
					{
						retVal = m_committeeCert;
					}
					break;


				case 11:
					if((m_degreeTermTrans != null) && (m_degreeConferred != null))
					{
						retVal = m_degreeConferred;
					}
					break;

				
				case 12:
					retVal = m_bound;
					break;
			}

			//m_logger.debug("DISSERTATION : BASE SERVICE : GET EXTERNAL VALIDATION : RETURNING : " + retVal);
			
			return retVal;
		}
		
		/**
		* Are these objects equal?  If they are both CandidateInfo objects, and they have
		* matching id's, they are.
		* @return true if they are equal, false if not.
		*/
		public boolean equals(Object obj)
		{
			if (!(obj instanceof CandidateInfo)) return false;
			return ((CandidateInfo)obj).getId().equals(getId());

		}   // equals

		/**
		* Make a hash code that reflects the equals() logic as well.
		* We want two objects, even if different instances, if they have the same id to hash the same.
		*/
		public int hashCode()
		{
			return getId().hashCode();

		}	// hashCode

		/**
		* Compare this object with the specified object for order.
		* @return A negative integer, zero, or a positive integer as this object is
		* less than, equal to, or greater than the specified object.
		*/
		public int compareTo(Object obj)
		{
			if (!(obj instanceof CandidateInfo)) throw new ClassCastException();

			// if the object are the same, say so
			if (obj == this) return 0;

			// start the compare by comparing their sort names
			int compare = getChefId().compareTo(((CandidateInfo)obj).getChefId());

			// if these are the same
			if (compare == 0)
			{
				// sort based on (unique) id
				compare = getId().compareTo(((CandidateInfo)obj).getId());
			}

			return compare;

		}	// compareTo


	}//BaseCandidateInfo

	
	
	/*******************************************************************************
	* CandidateInfoEdit implementation
	*******************************************************************************/
	protected class BaseCandidateInfoEdit
		extends BaseCandidateInfo
		implements CandidateInfoEdit, SessionStateBindingListener
	{
		/** The event code for the edit. */
		protected String m_event = null;
		
		/** Active flag. */
		protected boolean m_active = false;
		
		/**
		* Constructor.
		* @param infoId The CandidateInfo id.
		* @param site The site id.
		*/
		public BaseCandidateInfoEdit(String infoId, String site)
		{
			super(infoId, site);

		}   // BaseCandidateInfoEdit

		/**
		* Construct from an existing definition, in xml.
		* @param el The CandidateInfoEdit in XML in a DOM element.
		*/
		public BaseCandidateInfoEdit(Element el)
		{
			super(el);

		}	// BaseCandidateInfoEdit

		/**
		* Copy constructor.
		* @param info The CandidateInfo to be copied.
		*/
		public BaseCandidateInfoEdit(CandidateInfo info)
		{
			super(info);

		}	// BaseCandidateInfoEdit

		/**
		* Clean up.
		*/
		protected void finalize()
		{
			// catch the case where an edit was made but never resolved
			if (m_active)
			{
				cancelEdit(this);
			}

		}	// finalize


		/**
		* Set the user's university id.
		* @param emplid The user's university id string.
		*/
		public void setEmplid(String emplid)
		{
			m_emplid = emplid;
		}
		
		/**
		* Set the user's CHEF user id.
		* @param chefId The user id string.
		*/
		public void setChefId(String chefid)
		{
			m_chefId = chefid;
		}
		
		/**
		* Set the user's Rackham program id.
		* @param program The user's program id string.
		*/
		public void setProgram(String program)
		{
			m_program = program;
		}
		
		/**
		* Set the user's parent (department) site.
		* @param parentSiteId The site id of the user's parent (department) site.
		*/
		public void setParentSiteId(String parentSiteId)
		{
			m_parentSiteId = parentSiteId;
		}
		
		/**
		* Set the site for the CandidateInfo.
		* @param site The site id of the CandidateInfo's site.
		*/
		public void setSite(String site)
		{
			m_site = site;
		}
		
		/**
		* Set the Rackham data value.
		* @param desc The Advance to Candidacy Description.
		*/
		public void setAdvCandDesc(String desc)
		{
			m_advCandDesc = desc;
		}

		/**
		* Set the Rackham data value.
		* @param milestone The milestone.
		*/
		public void setMilestone(String milestone)
		{
			m_milestone = milestone;
		}
		
		/**
		* Set the Rackham data value.
		* @param milestone The prelim milestone.
		*/
		public void setPrelimMilestone(String milestone)
		{
			m_prelimMilestone = milestone;
		}
		
		/**
		* Set the Rackham data value.
		* @param completed The prelim milestone completion time.
		*/
		public void setPrelimTimeMilestoneCompleted(Time completed)
		{
			m_prelimTimeMilestoneCompleted = completed;
		}
		
		/**
		* Set the Rackham data value.
		* @param milestone The advcand milestone.
		*/
		public void setAdvcandMilestone(String milestone)
		{
			m_advcandMilestone = milestone;
		}
		
		/**
		* Set the Rackham data value.
		* @param completed The advcand milestone completion time.
		*/
		public void setAdvcandTimeMilestoneCompleted(Time completed)
		{
			m_advcandTimeMilestoneCompleted = completed;
		}
		
		/**
		* Set the Rackham data value.
		* @param completed The milestone completion time.
		*/
		public void setTimeMilestoneCompleted(Time completed)
		{
			m_milestoneCompleted = completed;
		}

		/**
		* Set the time of completion of the Rackham milestone.
		* @param approval The committee approval time.
		*/
		public void setTimeCommitteeApproval(Time approval)
		{
			m_committeeApproval = approval;
		}

		/**
		* Set the time of completion of the Rackham milestone.
		* @param exam The oral exam time.
		*/
		public void setTimeOralExam(Time exam)
		{
			m_oralExam = exam;
		}

		/**
		* Set the Rackham data value.
		* @param place The oral exam place.
		*/
		public void setOralExamPlace(String place)
		{
			if(place != null)
				m_oralExamPlace = place;
		}

		/**
		* Set the time of completion of the Rackham milestone.
		* @param format The first format time.
		*/
		public void setTimeFirstFormat(Time format)
		{
			m_firstFormat = format;
		}

		/**
		* Add a Rackham data value.
		* @param timeCompleted The time of committee evaluation.
		*/
		public void addTimeCommitteeEval(Time timeCompleted)
		{
			m_timeCommitteeEval.add(timeCompleted);
		}

		/**
		* Set the time of completion of the Rackham milestone.
		* @param timeReturned The time oral report returned.
		*/
		public void setTimeOralReportReturned(Time timeReturned)
		{
			m_oralReportReturned = timeReturned;
		}

		/**
		* Set the time of completion of the Rackham milestone.
		* @param timeOfCert The time of committee certification.
		*/
		public void setTimeCommitteeCert(Time timeOfCert)
		{
			m_committeeCert = timeOfCert;
		}

		/**
		* Set the Rackham data value.
		* @param recorder The final format recorder.
		*/
		public void setFinalFormatRecorder(String recorder)
		{
			if(recorder != null)
				m_finalFormatRecorder = recorder;
		}

		/**
		* Set the time of completion of the Rackham milestone.
		* @param receipt The binder receipt time.
		*/
		public void setTimeBinderReceipt(Time receipt)
		{
			m_binderReceipt = receipt;
		}

		/**
		* Set the Rackham data value.
		* @param met The fee requirement.
		*/
		public void setFeeRequirementMet(boolean met)
		{
			m_feeRequirementMet = met;
		}

		/**
		* Set the time of completion of the Rackham milestone.
		* @param seen The time receipt seen.
		*/
		public void setTimeReceiptSeen(Time seen)
		{
			m_receiptSeen = seen;
		}

		/**
		* Set the time of completion of the Rackham milestone.
		* @param fee The time the publication fee paid.
		*/
		public void setTimePubFee(Time fee)
		{
			m_pubFee = fee;
		}

		/**
		* Set the time of completion of the Rackham milestone.
		* @param unbound The unbound time.
		*/
		public void setTimeUnbound(Time unbound)
		{
			m_unbound = unbound;
		}

		/**
		* Set the time of completion of the Rackham milestone.
		* @param abst The abstract time.
		*/
		public void setTimeAbstract(Time abst)
		{
			m_abstract = abst;
		}

		/**
		* Set the time of completion of the Rackham milestone.
		* @param bound The bound time.
		*/
		public void setTimeBound(Time bound)
		{
			m_bound = bound;
		}

		/**
		* Set the time of completion of the Rackham milestone.
		* @param app The diploma app time.
		*/
		public void setTimeDiplomaApp(Time app)
		{
			m_diplomaApp = app;
		}

		/**
		* Set the time of completion of the Rackham milestone.
		* @param contract The contract time.
		*/
		public void setTimeContract(Time contract)
		{
			m_contract = contract;
		}

		/**
		* Set the time of completion of the Rackham milestone.
		* @param survey The survey time.
		*/
		public void setTimeSurvey(Time survey)
		{
			m_survey = survey;
		}

		/**
		* Set the time of completion of the Rackham milestone.
		* @param conferred The degree conferred time.
		*/
		public void setTimeDegreeConferred(Time conferred)
		{
			m_degreeConferred = conferred;
		}

		/**
		* Set the Rackham data value.
		* @param trans The degree term trans.
		*/
		public void setDegreeTermTrans(String trans)
		{
			if(trans != null)
				m_degreeTermTrans = trans;
		}

		/**
		* Take all values from this object.
		* @param info The CandidateInfo object to take values from.
		*/
		protected void set(CandidateInfo info)
		{
			setAll(info);

		}   // set

		/**
		* Access the event code for this edit.
		* @return The event code for this edit.
		*/
		protected String getEvent() { return m_event; }
	
		/**
		* Set the event code for this edit.
		* @param event The event code for this edit.
		*/
		protected void setEvent(String event) { m_event = event; }

		/**
		* Access the resource's properties for modification
		* @return The resource's properties.
		*/
		public ResourcePropertiesEdit getPropertiesEdit()
		{
			return m_properties;

		}	// getPropertiesEdit

		/**
		* Enable editing.
		*/
		protected void activate()
		{
			m_active = true;

		}	// activate

		/**
		* Check to see if the edit is still active, or has already been closed.
		* @return true if the edit is active, false if it's been closed.
		*/
		public boolean isActiveEdit()
		{
			return m_active;

		}	// isActiveEdit

		/**
		* Close the edit object - it cannot be used after this.
		*/
		protected void closeEdit()
		{
			m_active = false;

		}	// closeEdit

		/*******************************************************************************
		* SessionStateBindingListener implementation
		*******************************************************************************/
	
		/**
		* Accept notification that this object has been bound as a SessionState attribute.
		* @param sessionStateKey The id of the session state which holds the attribute.
		* @param attributeName The id of the attribute to which this object is now the value.
		*/
		public void valueBound(String sessionStateKey, String attributeName) {}
	
		/**
		* Accept notification that this object has been removed from a SessionState attribute.
		* @param sessionStateKey The id of the session state which held the attribute.
		* @param attributeName The id of the attribute to which this object was the value.
		*/
		public void valueUnbound(String sessionStateKey, String attributeName)
		{
			if (m_logger.isDebugEnabled())
			{
				//m_logger.debug(this + ".valueUnbound()");
			}
	
			// catch the case where an edit was made but never resolved
			if (m_active)
			{
				cancelEdit(this);
			}
	
		}	// valueUnbound

	}//BaseCandidateInfoEdit
	
	

	/*******************************************************************************
	* Storage implementations
	*******************************************************************************/
	
	
	/*******************************************************************************
	* BlockGrantGroupStorage
	*******************************************************************************/

	protected interface BlockGrantGroupStorage
	{
		/**
		* Open.
		*/
		public void open();

		/**
		* Close.
		*/
		public void close();

		/**
		* Check if a Block Grant Group by this id exists.
		* @param id The Block Grant Group id.
		* @return true if a Block Grant Group by this id exists, false if not.
		*/
		public boolean check(String id);

		/**
		* Get the Block Grant Group with this id, or null if not found.
		* @param id The Block Grant Group id.
		* @param id The Block Grant Group site.
		* @return The Block Grant Group with this id, or null if not found.
		*/
		public BlockGrantGroup get(String id);

		/**
		* Get all Block Grant Groups.
		* @return The list of all Block Grant Groups.
		*/
		public List getAll();

		/**
		* Add a new Block Grant Group with this id.
		* @param id The Block Grant Group id.
		* @param site The Block Grant Group site.
		* @return The locked BlockGrantGroup object with this id, or null if the id is in use.
		*/
		public BlockGrantGroupEdit put(String id, String site);

		/**
		* Get a lock on the Block Grant Group with this id, or null if a lock cannot be gotten.
		* @param id The Block Grant Group id.
		* @return The locked BlockGrantGroup with this id, or null if this records cannot be locked.
		*/
		public BlockGrantGroupEdit edit(String id);

		/**
		* Commit the changes and release the lock.
		* @param blockGrantGroupEdit The blockGrantGroupEdit to commit.
		*/
		public void commit(BlockGrantGroupEdit blockGrantGroupEdit);

		/**
		* Cancel the changes and release the lock.
		* @param blockGrantGroupEdit The BlockGrantGroupEdit to commit.
		*/
		public void cancel(BlockGrantGroupEdit blockGrantGroupEdit);

		/**
		* Remove this BlockGrantGroup.
		* @param blockGrantGroup The BlockGrantGroup to remove.
		*/
		public void remove(BlockGrantGroupEdit blockGrantGroupEdit);

	}   // BlockGrantGroupStorage


	/*******************************************************************************
	* DissertationStorage
	*******************************************************************************/

	protected interface DissertationStorage
	{
		/**
		* Open.
		*/
		public void open();

		/**
		* Close.
		*/
		public void close();

		/**
		* Check if a dissertation by this id exists.
		* @param id The dissertation id.
		* @return true if a dissertation by this id exists, false if not.
		*/
		public boolean check(String id);

		/**
		* Get the dissertation with this id, or null if not found.
		* @param id The dissertation id.
		* @return The dissertation with this id, or null if not found.
		*/
		public Dissertation get(String id);

		/**
		* Get all dissertations.
		* @return The list of all dissertations.
		*/
		public List getAll();

		/**
		* Add a new dissertation with this id.
		* @param id The dissertation id.
		* @return The locked Dissertation object with this id, or null if the id is in use.
		*/
		public DissertationEdit put(String id, String site);

		/**
		* Get a lock on the dissertation with this id, or null if a lock cannot be gotten.
		* @param id The dissertation id.
		* @return The locked Dissertation with this id, or null if this records cannot be locked.
		*/
		public DissertationEdit edit(String id);

		/**
		* Commit the changes and release the lock.
		* @param dissertation The dissertation to commit.
		*/
		public void commit(DissertationEdit dissertation);

		/**
		* Cancel the changes and release the lock.
		* @param dissertation The dissertation to commit.
		*/
		public void cancel(DissertationEdit dissertation);

		/**
		* Remove this dissertation.
		* @param dissertation The dissertation to remove.
		*/
		public void remove(DissertationEdit dissertation);

	}   // DissertationStorage

	
	/*******************************************************************************
	* DissertationStepStorage
	*******************************************************************************/

	protected interface DissertationStepStorage
	{
		/**
		* Open.
		*/
		public void open();

		/**
		* Close.
		*/
		public void close();

		/**
		* Check if a DissertationStep by this id exists.
		* @param id The DissertationStep id.
		* @return true if a DissertationStep by this id exists, false if not.
		*/
		public boolean check(String id);

		/**
		* Get the DissertationStep with this id, or null if not found.
		* @param id The DissertationStep id.
		* @return The DissertationStep with this id, or null if not found.
		*/
		public DissertationStep get(String id);

		/**
		* Get all DissertationSteps.
		* @return The list of all DissertationSteps.
		*/
		public List getAll();

		/**
		* Add a new DissertationStep with this id.
		* @param id The DissertationStep id.
		* @return The locked DissertationStep object with this id, or null if the id is in use.
		*/
		public DissertationStepEdit put(String id, String site);

		/**
		* Get a lock on the DissertationStep with this id, or null if a lock cannot be gotten.
		* @param id The DissertationStep id.
		* @return The locked DissertationStep with this id, or null if this records cannot be locked.
		*/
		public DissertationStepEdit edit(String id);

		/**
		* Commit the changes and release the lock.
		* @param DissertationStep The DissertationStep to commit.
		*/
		public void commit(DissertationStepEdit step);

		/**
		* Cancel the changes and release the lock.
		* @param DissertationStep The DissertationStep to commit.
		*/
		public void cancel(DissertationStepEdit step);

		/**
		* Remove this DissertationStep.
		* @param DissertationStep The DissertationStep to remove.
		*/
		public void remove(DissertationStepEdit step);

	}   // DissertationStepStorage


	/*******************************************************************************
	* CandidatePathStorage
	*******************************************************************************/

	protected interface CandidatePathStorage
	{
		/**
		* Open.
		*/
		public void open();

		/**
		* Close.
		*/
		public void close();

		/**
		* Check if a CandidatePath by this id exists.
		* @param id The CandidatePath id.
		* @return true if a CandidatePath by this id exists, false if not.
		*/
		public boolean check(String id);

		/**
		* Get the CandidatePath with this id, or null if not found.
		* @param id The CandidatePath id.
		* @return The CandidatePath with this id, or null if not found.
		*/
		public CandidatePath get(String id);

		/**
		* Get all CandidatePaths.
		* @return The list of all CandidatePaths.
		*/
		public List getAll();

		/**
		* Add a new CandidatePath with this id.
		* @param id The CandidatePath id.
		* @return The locked CandidatePath object with this id, or null if the id is in use.
		*/
		public CandidatePathEdit put(String id, String site);

		/**
		* Get a lock on the CandidatePath with this id, or null if a lock cannot be gotten.
		* @param id The CandidatePath id.
		* @return The locked CandidatePath with this id, or null if this records cannot be locked.
		*/
		public CandidatePathEdit edit(String id);

		/**
		* Commit the changes and release the lock.
		* @param path The CandidatePath to commit.
		*/
		public void commit(CandidatePathEdit path);

		/**
		* Cancel the changes and release the lock.
		* @param path The CandidatePath to commit.
		*/
		public void cancel(CandidatePathEdit path);

		/**
		* Remove this CandidatePath.
		* @param path The CandidatePath to remove.
		*/
		public void remove(CandidatePathEdit path);

	}   // CandidatePathStorage


	/*******************************************************************************
	* StepStatusStorage
	*******************************************************************************/

	protected interface StepStatusStorage
	{
		/**
		* Open.
		*/
		public void open();

		/**
		* Close.
		*/
		public void close();

		/**
		* Check if a StepStatus by this id exists.
		* @param id The StepStatus id.
		* @return true if a StepStatus by this id exists, false if not.
		*/
		public boolean check(String id);

		/**
		* Get the StepStatus with this id, or null if not found.
		* @param id The StepStatus id.
		* @return The StepStatus with this id, or null if not found.
		*/
		public StepStatus get(String id);

		/**
		* Get all StepStatuss.
		* @return The list of all StepStatuss.
		*/
		public List getAll();

		/**
		* Add a new StepStatus with this id.
		* @param id The StepStatus id.
		* @return The locked StepStatus object with this id, or null if the id is in use.
		*/
		public StepStatusEdit put(String id, String site);

		/**
		* Get a lock on the StepStatus with this id, or null if a lock cannot be gotten.
		* @param id The StepStatus id.
		* @return The locked StepStatus with this id, or null if this records cannot be locked.
		*/
		public StepStatusEdit edit(String id);

		/**
		* Commit the changes and release the lock.
		* @param status The StepStatus to commit.
		*/
		public void commit(StepStatusEdit status);

		/**
		* Cancel the changes and release the lock.
		* @param status The StepStatus to commit.
		*/
		public void cancel(StepStatusEdit status);

		/**
		* Remove this StepStatus.
		* @param status The StepStatus to remove.
		*/
		public void remove(StepStatusEdit status);

	}   // StepStatusStorage

	
	/*******************************************************************************
	* CandidateInfoStorage
	*******************************************************************************/

	protected interface CandidateInfoStorage
	{
		/**
		* Open.
		*/
		public void open();

		/**
		* Close.
		*/
		public void close();

		/**
		* Check if a CandidateInfo by this id exists.
		* @param id The CandidateInfo id.
		* @return true if a CandidateInfo by this id exists, false if not.
		*/
		public boolean check(String id);

		/**
		* Get the CandidateInfo with this id, or null if not found.
		* @param id The CandidateInfo id.
		* @return The CandidateInfo with this id, or null if not found.
		*/
		public CandidateInfo get(String id);

		/**
		* Get all CandidateInfos.
		* @return The list of all CandidateInfos.
		*/
		public List getAll();

		/**
		* Add a new CandidateInfo with this id.
		* @param id The CandidateInfo id.
		* @return The locked CandidateInfo object with this id, or null if the id is in use.
		*/
		public CandidateInfoEdit put(String id, String site);

		/**
		* Get a lock on the CandidateInfo with this id, or null if a lock cannot be gotten.
		* @param id The CandidateInfo id.
		* @return The locked CandidateInfo with this id, or null if this records cannot be locked.
		*/
		public CandidateInfoEdit edit(String id);

		/**
		* Commit the changes and release the lock.
		* @param info The CandidateInfo to commit.
		*/
		public void commit(CandidateInfoEdit info);

		/**
		* Cancel the changes and release the lock.
		* @param info The CandidateInfo to commit.
		*/
		public void cancel(CandidateInfoEdit info);

		/**
		* Remove this CandidateInfo.
		* @param info The CandidateInfo to remove.
		*/
		public void remove(CandidateInfoEdit info);

	}   // CandidateInfoStorage


	/*******************************************************************************
	* StorageUser implementations
	*******************************************************************************/	
	
	/*******************************************************************************
	* BlockGrantGroupStorageUser implementation
	*******************************************************************************/

	protected class BlockGrantGroupStorageUser 
		implements StorageUser
	{
		/**
		* Construct a new container given just an id.
		* @param id The id for the new object.
		* @return The new container Resource.
		*/
		public Resource newContainer(String ref) { return null; }

		/**
		* Construct a new container resource, from an XML element.
		* @param element The XML.
		* @return The new container resource.
		*/
		public Resource newContainer(Element element) { return null; }

		/**
		* Construct a new container resource, as a copy of another
		* @param other The other contianer to copy.
		* @return The new container resource.
		*/
		public Resource newContainer(Resource other) { return null; }

		/**
		* Construct a new resource given just an id.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param id The id for the new object.
		* @return The new resource.
		*/
		public Resource newResource(Resource container, String id, Object[] others)
		{ return new BaseBlockGrantGroup(id, (String) others[0]); }

		/**
		* Construct a new resource, from an XML element.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param element The XML.
		* @return The new resource from the XML.
		*/
		public Resource newResource(Resource container, Element element)
		{ return new BaseBlockGrantGroup(element); }

		/**
		* Construct a new resource from another resource of the same type.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param other The other resource.
		* @return The new resource as a copy of the other.
		*/
		public Resource newResource(Resource container, Resource other)
		{ return new BaseBlockGrantGroup((BlockGrantGroup) other); }

		/**
		* Construct a new continer given just an id.
		* @param id The id for the new object.
		* @return The new containe Resource.
		*/
		public Edit newContainerEdit(String ref) { return null; }

		/**
		* Construct a new container resource, from an XML element.
		* @param element The XML.
		* @return The new container resource.
		*/
		public Edit newContainerEdit(Element element) { return null; }

		/**
		* Construct a new container resource, as a copy of another
		* @param other The other contianer to copy.
		* @return The new container resource.
		*/
		public Edit newContainerEdit(Resource other) { return null; }

		/**
		* Construct a new rsource given just an id.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param id The id for the new object.
		* @return The new resource.
		*/
		public Edit newResourceEdit(Resource container, String id, Object[] others)
		{
			BaseBlockGrantGroupEdit e = new BaseBlockGrantGroupEdit(id, (String)others[0]);
			e.activate();
			return e;
		}

		/**
		* Construct a new resource, from an XML element.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param element The XML.
		* @return The new resource from the XML.
		*/
		public Edit newResourceEdit(Resource container, Element element)
		{
			BaseBlockGrantGroupEdit e =  new BaseBlockGrantGroupEdit(element);
			e.activate();
			return e;
		}

		/**
		* Construct a new resource from another resource of the same type.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param other The other resource.
		* @return The new resource as a copy of the other.
		*/
		public Edit newResourceEdit(Resource container, Resource other)
		{
			BaseBlockGrantGroupEdit e = new BaseBlockGrantGroupEdit((BlockGrantGroup) other);
			e.activate();
			return e;
		}

		/**
		* Collect the fields that need to be stored outside the XML (for the resource).
		* @return An array of field values to store in the record outside the XML (for the resource).
		*/
		public Object[] storageFields(Resource r) { return null; }

	}//BlockGrantGroupStorageUser
	
	
	/*******************************************************************************
	* DissertationStorageUser implementation
	*******************************************************************************/

	protected class DissertationStorageUser 
		implements StorageUser
	{
		/**
		* Construct a new continer given just an id.
		* @param id The id for the new object.
		* @return The new container Resource.
		*/
		public Resource newContainer(String ref) { return null; }

		/**
		* Construct a new container resource, from an XML element.
		* @param element The XML.
		* @return The new container resource.
		*/
		public Resource newContainer(Element element) { return null; }

		/**
		* Construct a new container resource, as a copy of another
		* @param other The other contianer to copy.
		* @return The new container resource.
		*/
		public Resource newContainer(Resource other) { return null; }

		/**
		* Construct a new resource given just an id.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param id The id for the new object.
		* @return The new resource.
		*/
		public Resource newResource(Resource container, String id, Object[] others)
		{ return new BaseDissertation(id, (String) others[0]); }

		/**
		* Construct a new resource, from an XML element.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param element The XML.
		* @return The new resource from the XML.
		*/
		public Resource newResource(Resource container, Element element)
		{ return new BaseDissertation(element); }

		/**
		* Construct a new resource from another resource of the same type.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param other The other resource.
		* @return The new resource as a copy of the other.
		*/
		public Resource newResource(Resource container, Resource other)
		{ return new BaseDissertation((Dissertation) other); }

		/**
		* Construct a new continer given just an id.
		* @param id The id for the new object.
		* @return The new containe Resource.
		*/
		public Edit newContainerEdit(String ref) { return null; }

		/**
		* Construct a new container resource, from an XML element.
		* @param element The XML.
		* @return The new container resource.
		*/
		public Edit newContainerEdit(Element element) { return null; }

		/**
		* Construct a new container resource, as a copy of another
		* @param other The other contianer to copy.
		* @return The new container resource.
		*/
		public Edit newContainerEdit(Resource other) { return null; }

		/**
		* Construct a new rsource given just an id.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param id The id for the new object.
		* @return The new resource.
		*/
		public Edit newResourceEdit(Resource container, String id, Object[] others)
		{
			BaseDissertationEdit e = new BaseDissertationEdit(id, (String)others[0]);
			e.activate();
			return e;
		}

		/**
		* Construct a new resource, from an XML element.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param element The XML.
		* @return The new resource from the XML.
		*/
		public Edit newResourceEdit(Resource container, Element element)
		{
			BaseDissertationEdit e =  new BaseDissertationEdit(element);
			e.activate();
			return e;
		}

		/**
		* Construct a new resource from another resource of the same type.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param other The other resource.
		* @return The new resource as a copy of the other.
		*/
		public Edit newResourceEdit(Resource container, Resource other)
		{
			BaseDissertationEdit e = new BaseDissertationEdit((Dissertation) other);
			e.activate();
			return e;
		}

		/**
		* Collect the fields that need to be stored outside the XML (for the resource).
		* @return An array of field values to store in the record outside the XML (for the resource).
		*/
		public Object[] storageFields(Resource r) { return null; }

	}//DissertationStorageUser
	
	
	/*******************************************************************************
	* DissertationStepStorageUser implementation
	*******************************************************************************/

	protected class DissertationStepStorageUser 
		implements StorageUser
	{
		/**
		* Construct a new continer given just an id.
		* @param id The id for the new object.
		* @return The new container Resource.
		*/
		public Resource newContainer(String ref) { return null; }

		/**
		* Construct a new container resource, from an XML element.
		* @param element The XML.
		* @return The new container resource.
		*/
		public Resource newContainer(Element element) { return null; }

		/**
		* Construct a new container resource, as a copy of another
		* @param other The other contianer to copy.
		* @return The new container resource.
		*/
		public Resource newContainer(Resource other) { return null; }

		/**
		* Construct a new resource given just an id.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param id The id for the new object.
		* @return The new resource.
		*/
		public Resource newResource(Resource container, String id, Object[] others)
		{ return new BaseDissertationStep(id, (String)others[0]); }

		/**
		* Construct a new resource, from an XML element.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param element The XML.
		* @return The new resource from the XML.
		*/
		public Resource newResource(Resource container, Element element)
		{ return new BaseDissertationStep(element); }

		/**
		* Construct a new resource from another resource of the same type.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param other The other resource.
		* @return The new resource as a copy of the other.
		*/
		public Resource newResource(Resource container, Resource other)
		{ return new BaseDissertationStep((DissertationStep) other); }

		/**
		* Construct a new continer given just an id.
		* @param id The id for the new object.
		* @return The new containe Resource.
		*/
		public Edit newContainerEdit(String ref) { return null; }

		/**
		* Construct a new container resource, from an XML element.
		* @param element The XML.
		* @return The new container resource.
		*/
		public Edit newContainerEdit(Element element) { return null; }

		/**
		* Construct a new container resource, as a copy of another
		* @param other The other contianer to copy.
		* @return The new container resource.
		*/
		public Edit newContainerEdit(Resource other) { return null; }

		/**
		* Construct a new rsource given just an id.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param id The id for the new object.
		* @return The new resource.
		*/
		public Edit newResourceEdit(Resource container, String id, Object[] others)
		{
			BaseDissertationStepEdit e = new BaseDissertationStepEdit(id, (String)others[0]);
			e.activate();
			return e;
		}

		/**
		* Construct a new resource, from an XML element.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param element The XML.
		* @return The new resource from the XML.
		*/
		public Edit newResourceEdit(Resource container, Element element)
		{
			BaseDissertationStepEdit e =  new BaseDissertationStepEdit(element);
			e.activate();
			return e;
		}

		/**
		* Construct a new resource from another resource of the same type.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param other The other resource.
		* @return The new resource as a copy of the other.
		*/
		public Edit newResourceEdit(Resource container, Resource other)
		{
			BaseDissertationStepEdit e = new BaseDissertationStepEdit((DissertationStep) other);
			e.activate();
			return e;
		}

		/**
		* Collect the fields that need to be stored outside the XML (for the resource).
		* @return An array of field values to store in the record outside the XML (for the resource).
		*/
		public Object[] storageFields(Resource r) { return null; }

	}//DissertationStepStorageUser

	
	/*******************************************************************************
	* CandidatePathStorageUser implementation
	*******************************************************************************/

	protected class CandidatePathStorageUser 
		implements StorageUser
	{
		/**
		* Construct a new continer given just an id.
		* @param id The id for the new object.
		* @return The new container Resource.
		*/
		public Resource newContainer(String ref) { return null; }

		/**
		* Construct a new container resource, from an XML element.
		* @param element The XML.
		* @return The new container resource.
		*/
		public Resource newContainer(Element element) { return null; }

		/**
		* Construct a new container resource, as a copy of another
		* @param other The other contianer to copy.
		* @return The new container resource.
		*/
		public Resource newContainer(Resource other) { return null; }

		/**
		* Construct a new resource given just an id.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param id The id for the new object.
		* @return The new resource.
		*/
		public Resource newResource(Resource container, String id, Object[] others)
		{ return new BaseCandidatePath(id, (String)others[0]); }

		/**
		* Construct a new resource, from an XML element.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param element The XML.
		* @return The new resource from the XML.
		*/
		public Resource newResource(Resource container, Element element)
		{ return new BaseCandidatePath(element); }

		/**
		* Construct a new resource from another resource of the same type.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param other The other resource.
		* @return The new resource as a copy of the other.
		*/
		public Resource newResource(Resource container, Resource other)
		{ return new BaseCandidatePath((CandidatePath) other); }

		/**
		* Construct a new continer given just an id.
		* @param id The id for the new object.
		* @return The new containe Resource.
		*/
		public Edit newContainerEdit(String ref) { return null; }

		/**
		* Construct a new container resource, from an XML element.
		* @param element The XML.
		* @return The new container resource.
		*/
		public Edit newContainerEdit(Element element) { return null; }

		/**
		* Construct a new container resource, as a copy of another
		* @param other The other contianer to copy.
		* @return The new container resource.
		*/
		public Edit newContainerEdit(Resource other) { return null; }

		/**
		* Construct a new rsource given just an id.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param id The id for the new object.
		* @return The new resource.
		*/
		public Edit newResourceEdit(Resource container, String id, Object[] others)
		{
			BaseCandidatePathEdit e = new BaseCandidatePathEdit(id, (String)others[0]);
			e.activate();
			return e;
		}

		/**
		* Construct a new resource, from an XML element.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param element The XML.
		* @return The new resource from the XML.
		*/
		public Edit newResourceEdit(Resource container, Element element)
		{
			BaseCandidatePathEdit e =  new BaseCandidatePathEdit(element);
			e.activate();
			return e;
		}

		/**
		* Construct a new resource from another resource of the same type.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param other The other resource.
		* @return The new resource as a copy of the other.
		*/
		public Edit newResourceEdit(Resource container, Resource other)
		{
			BaseCandidatePathEdit e = new BaseCandidatePathEdit((CandidatePath) other);
			e.activate();
			return e;
		}

		/**
		* Collect the fields that need to be stored outside the XML (for the resource).
		* @return An array of field values to store in the record outside the XML (for the resource).
		*/
		public Object[] storageFields(Resource r) { return null; }

	}//CandidatePathStorageUser

	
	/*******************************************************************************
	* StepStatusStorageUser implementation
	*******************************************************************************/

	protected class StepStatusStorageUser 
		implements StorageUser
	{
		/**
		* Construct a new continer given just an id.
		* @param id The id for the new object.
		* @return The new container Resource.
		*/
		public Resource newContainer(String ref) { return null; }

		/**
		* Construct a new container resource, from an XML element.
		* @param element The XML.
		* @return The new container resource.
		*/
		public Resource newContainer(Element element) { return null; }

		/**
		* Construct a new container resource, as a copy of another
		* @param other The other contianer to copy.
		* @return The new container resource.
		*/
		public Resource newContainer(Resource other) { return null; }

		/**
		* Construct a new resource given just an id.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param id The id for the new object.
		* @return The new resource.
		*/
		public Resource newResource(Resource container, String id, Object[] others)
		{ return new BaseStepStatus(id, (String)others[0]); }

		/**
		* Construct a new resource, from an XML element.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param element The XML.
		* @return The new resource from the XML.
		*/
		public Resource newResource(Resource container, Element element)
		{ return new BaseStepStatus(element); }

		/**
		* Construct a new resource from another resource of the same type.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param other The other resource.
		* @return The new resource as a copy of the other.
		*/
		public Resource newResource(Resource container, Resource other)
		{ return new BaseStepStatus((StepStatus) other); }

		/**
		* Construct a new continer given just an id.
		* @param id The id for the new object.
		* @return The new containe Resource.
		*/
		public Edit newContainerEdit(String ref) { return null; }

		/**
		* Construct a new container resource, from an XML element.
		* @param element The XML.
		* @return The new container resource.
		*/
		public Edit newContainerEdit(Element element) { return null; }

		/**
		* Construct a new container resource, as a copy of another
		* @param other The other contianer to copy.
		* @return The new container resource.
		*/
		public Edit newContainerEdit(Resource other) { return null; }

		/**
		* Construct a new rsource given just an id.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param id The id for the new object.
		* @return The new resource.
		*/
		public Edit newResourceEdit(Resource container, String id, Object[] others)
		{
			BaseStepStatusEdit e = new BaseStepStatusEdit(id, (String)others[0]);
			e.activate();
			return e;
		}

		/**
		* Construct a new resource, from an XML element.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param element The XML.
		* @return The new resource from the XML.
		*/
		public Edit newResourceEdit(Resource container, Element element)
		{
			BaseStepStatusEdit e =  new BaseStepStatusEdit(element);
			e.activate();
			return e;
		}

		/**
		* Construct a new resource from another resource of the same type.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param other The other resource.
		* @return The new resource as a copy of the other.
		*/
		public Edit newResourceEdit(Resource container, Resource other)
		{
			BaseStepStatusEdit e = new BaseStepStatusEdit((StepStatus) other);
			e.activate();
			return e;
		}

		/**
		* Collect the fields that need to be stored outside the XML (for the resource).
		* @return An array of field values to store in the record outside the XML (for the resource).
		*/
		public Object[] storageFields(Resource r) { return null; }

	}//StepStatusStorageUser

	
	/*******************************************************************************
	* CandidateInfoStorageUser implementation
	*******************************************************************************/

	protected class CandidateInfoStorageUser 
		implements StorageUser
	{
		/**
		* Construct a new continer given just an id.
		* @param id The id for the new object.
		* @return The new container Resource.
		*/
		public Resource newContainer(String ref) { return null; }

		/**
		* Construct a new container resource, from an XML element.
		* @param element The XML.
		* @return The new container resource.
		*/
		public Resource newContainer(Element element) { return null; }

		/**
		* Construct a new container resource, as a copy of another
		* @param other The other contianer to copy.
		* @return The new container resource.
		*/
		public Resource newContainer(Resource other) { return null; }

		/**
		* Construct a new resource given just an id.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param id The id for the new object.
		* @return The new resource.
		*/
		public Resource newResource(Resource container, String id, Object[] others)
		{ return new BaseCandidateInfo(id, (String)others[0]); }

		/**
		* Construct a new resource, from an XML element.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param element The XML.
		* @return The new resource from the XML.
		*/
		public Resource newResource(Resource container, Element element)
		{ return new BaseCandidateInfo(element); }

		/**
		* Construct a new resource from another resource of the same type.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param other The other resource.
		* @return The new resource as a copy of the other.
		*/
		public Resource newResource(Resource container, Resource other)
		{ return new BaseCandidateInfo((CandidateInfo) other); }

		/**
		* Construct a new continer given just an id.
		* @param id The id for the new object.
		* @return The new containe Resource.
		*/
		public Edit newContainerEdit(String ref) { return null; }

		/**
		* Construct a new container resource, from an XML element.
		* @param element The XML.
		* @return The new container resource.
		*/
		public Edit newContainerEdit(Element element) { return null; }

		/**
		* Construct a new container resource, as a copy of another
		* @param other The other contianer to copy.
		* @return The new container resource.
		*/
		public Edit newContainerEdit(Resource other) { return null; }

		/**
		* Construct a new rsource given just an id.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param id The id for the new object.
		* @return The new resource.
		*/
		public Edit newResourceEdit(Resource container, String id, Object[] others)
		{
			BaseCandidateInfoEdit e = new BaseCandidateInfoEdit(id, (String)others[0]);
			e.activate();
			return e;
		}

		/**
		* Construct a new resource, from an XML element.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param element The XML.
		* @return The new resource from the XML.
		*/
		public Edit newResourceEdit(Resource container, Element element)
		{
			BaseCandidateInfoEdit e =  new BaseCandidateInfoEdit(element);
			e.activate();
			return e;
		}

		/**
		* Construct a new resource from another resource of the same type.
		* @param container The Resource that is the container for the new resource (may be null).
		* @param other The other resource.
		* @return The new resource as a copy of the other.
		*/
		public Edit newResourceEdit(Resource container, Resource other)
		{
			BaseCandidateInfoEdit e = new BaseCandidateInfoEdit((CandidateInfo) other);
			e.activate();
			return e;
		}

		/**
		* Collect the fields that need to be stored outside the XML (for the resource).
		* @return An array of field values to store in the record outside the XML (for the resource).
		*/
		public Object[] storageFields(Resource r) { return null; }

	}//CandidateInfoStorageUser

	
	/*******************************************************************************
	* CacheRefresher implementations (no container)
	*******************************************************************************/

	/*******************************************************************************
	* BlockGrantGroupCacheRefresher implementation
	*******************************************************************************/

	protected class BlockGrantGroupCacheRefresher
		implements CacheRefresher
	{
		/**
		* Get a new value for this key whose value has already expired in the cache.
		* @param key The key whose value has expired and needs to be refreshed.
		* @param oldValue The old expired value of the key.
		* @return a new value for use in the cache for this key; if null, the entry will be removed.
		*/	
		public Object refresh(Object key, Object oldValue, Event event)
		{
			// key is a reference, but our storage wants an id
			String id = blockGrantGroupId((String) key);

			// get whatever we have from storage for the cache for this value
			BlockGrantGroup blockGrantGroup = m_groupStorage.get(id);

			if (m_logger.isDebugEnabled())
			{
				//m_logger.debug(this + ".refresh(): " + key + " : " + id);
			}

			return blockGrantGroup;

		}	// refresh

	}//BlockGrantGroupCacheRefresher


	/*******************************************************************************
	* DissertationCacheRefresher implementation
	*******************************************************************************/

	protected class DissertationCacheRefresher
		implements CacheRefresher
	{
		/**
		* Get a new value for this key whose value has already expired in the cache.
		* @param key The key whose value has expired and needs to be refreshed.
		* @param oldValue The old expired value of the key.
		* @return a new value for use in the cache for this key; if null, the entry will be removed.
		*/	
		public Object refresh(Object key, Object oldValue, Event event)
		{
		
			// key is a reference, but our storage wants an id
			String id = dissertationId((String) key);

			// get whatever we have from storage for the cache for this value
			Dissertation dissertation = m_dissertationStorage.get(id);
			//Dissertation dissertation = getDissertation(id);

			if (m_logger.isDebugEnabled())
			{
				//m_logger.debug(this + ".refresh(): " + key + " : " + id);
			}

			return dissertation;

		}	// refresh

	}//DissertationCacheRefresher

	
	/*******************************************************************************
	* DissertationStepCacheRefresher implementation
	*******************************************************************************/

	protected class DissertationStepCacheRefresher
		implements CacheRefresher
	{
		/**
		* Get a new value for this key whose value has already expired in the cache.
		* @param key The key whose value has expired and needs to be refreshed.
		* @param oldValue The old expired value of the key.
		* @return a new value for use in the cache for this key; if null, the entry will be removed.
		*/	
		public Object refresh(Object key, Object oldValue, Event event)
		{
		
			// key is a reference, but our storage wants an id
			String id = stepId((String) key);

			// get whatever we have from storage for the cache for this vale
			DissertationStep step = m_stepStorage.get(id);

			if (m_logger.isDebugEnabled())
			{
				//m_logger.debug(this + ".refresh(): " + key + " : " + id);
			}

			return step;

		}	// refresh

	}//DissertationStepCacheRefresher


	/*******************************************************************************
	* CandidatePathCacheRefresher implementation
	*******************************************************************************/

	protected class CandidatePathCacheRefresher
		implements CacheRefresher
	{
		/**
		* Get a new value for this key whose value has already expired in the cache.
		* @param key The key whose value has expired and needs to be refreshed.
		* @param oldValue The old expired value of the key.
		* @return a new value for use in the cache for this key; if null, the entry will be removed.
		*/	
		public Object refresh(Object key, Object oldValue, Event event)
		{
		
			// key is a reference, but our storage wants an id
			String id = pathId((String) key);

			// get whatever we have from storage for the cache for this vale
			CandidatePath path = m_pathStorage.get(id);

			if (m_logger.isDebugEnabled())
			{
				//m_logger.debug(this + ".refresh(): " + key + " : " + id);
			}

			return path;

		}	// refresh

	}//CandidatePathCacheRefresher


	/*******************************************************************************
	* StepStatusCacheRefresher implementation
	*******************************************************************************/

	protected class StepStatusCacheRefresher
		implements CacheRefresher
	{
		/**
		* Get a new value for this key whose value has already expired in the cache.
		* @param key The key whose value has expired and needs to be refreshed.
		* @param oldValue The old expired value of the key.
		* @return a new value for use in the cache for this key; if null, the entry will be removed.
		*/	
		public Object refresh(Object key, Object oldValue, Event event)
		{
		
			// key is a reference, but our storage wants an id
			String id = statusId((String) key);

			// get whatever we have from storage for the cache for this vale
			StepStatus status = m_statusStorage.get(id);

			if (m_logger.isDebugEnabled())
			{
				//m_logger.debug(this + ".refresh(): " + key + " : " + id);
			}

			return status;

		}	// refresh

	}//StepStatusCacheRefresher

	
	/*******************************************************************************
	* CandidateInfoCacheRefresher implementation
	*******************************************************************************/

	protected class CandidateInfoCacheRefresher
		implements CacheRefresher
	{
		/**
		* Get a new value for this key whose value has already expired in the cache.
		* @param key The key whose value has expired and needs to be refreshed.
		* @param oldValue The old expired value of the key.
		* @return a new value for use in the cache for this key; if null, the entry will be removed.
		*/	
		public Object refresh(Object key, Object oldValue, Event event)
		{
		
			// key is a reference, but our storage wants an id
			String id = infoId((String) key);

			// get whatever we have from storage for the cache for this vale
			CandidateInfo info = m_infoStorage.get(id);

			if (m_logger.isDebugEnabled())
			{
				//m_logger.debug(this + ".refresh(): " + key + " : " + id);
			}

			return info;

		}	// refresh

	}//CandidateInfoCacheRefresher


	/**
	* Utility function which returns the string representation of the
	* long value of the time object.
	* @param t - the Time object.
	* @return A String representation of the long value of the time object.
	*/
	protected String getTimeString(Time t)
	{
		String retVal = "";
		if(t != null)
			retVal = String.valueOf(t.getTime());
		return retVal;
	}

	/**
	* Utility function which returns a string from a boolean value.
	* @param b - the boolean value.
	* @return - "True" if the input value is true, "false" otherwise.
	*/
	protected String getBoolString(boolean b)
	{
		if(b)
			return "true";
		else
			return "false";
	}

	/**
	* Utility function which returns the user's id.
	* @param User object.
	* @return The id for the user.
	*/
	protected String getUserId(User u)
	{
		String retVal = "";
		if(u != null)
			retVal = u.getId();
		return retVal;
	}

	/**
	* Utility function which returns a boolean value from a string.
	* @param s - The input string.
	* @return the boolean true if the input string is "true", false otherwise.
	*/
	protected boolean getBool(String s)
	{
		boolean retVal = false;
		if(s != null)
		{
			if(s.equalsIgnoreCase("true"))
				retVal = true;
		}
		return retVal;
	}

	/**
	* Utility function which find a User from an id.
	* @param id - The User's id.
	* @return The User, or null if one does not correspond with the id.
	*/		
	protected User getUserObject(String id)
	{
		User retVal = null;
		if(id != "")
		{

			try
			{
				retVal = UserDirectoryService.getUser(id);
			}
			catch(IdUnusedException iue)
			{
				m_logger.warn(this + " Exception getting user from user dir service with id : " + id + " : " + iue);
			}
		}
		return retVal;
	}

	/**
	* Utility function which converts a string into a chef time object.
	* @param timeString - String version of a time in long format, representing the standard ms since the epoch, Jan 1, 1970 00:00:00.
	* @return A chef Time object.
	*/
	protected Time getTimeObject(String timeString)
	{
		Time aTime = null;
		if(timeString != "")
		{
			try
			{
				long longTime = Long.parseLong(timeString);
				aTime = TimeService.newTime(longTime);
			}
			catch(Exception e)
			{
				m_logger.warn(this + " Exception creating time object from xml file : " + e);
			}
		}
		return aTime;
	}
}

/**********************************************************************************
*
* $Header: /cvs/sakai/legacy-component/src/java/org/sakaiproject/component/legacy/dissertation/BaseDissertationService.java,v 1.14 2004/06/22 03:13:58 ggolden Exp $
*
**********************************************************************************/
